/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	OcpLTool.c	***
 **************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"


#include	"GpType/GpType.h"


#include	"VimageType/OcpType.h"




/***** ocpL_kf_add *****
Description:The method adds a keyframe (scp) after pcp at frame fno
Params:		ocp	- the list
			pcp	- the frame to add after 
			scp - the keyframe to add
			fno	- frame number of the new keyframe.
Returns:the new added keyfraem
*****************************/

ocp_type *ocpL_kf_add( ocp_list *ocp, ocp_type *pcp, ocp_type *scp, int fno)
{// method doesn't have NULL checkings!!!
ocp_type	*cp;
 
 
	cp = ocp_make_copy( scp );
 
	cp->next = pcp->next;
	pcp->next = cp;
 
 
	cp->fno = fno;

	pcp->dfno = fno - pcp->fno;
	if( cp->next != NULL )
		cp->dfno = cp->next->fno - cp->fno;
	else
		cp->dfno = 1;

	cp->Fshot = SHOT_MID;
	cp->Fgc = 0;
 

	if( pcp->Fshot == SHOT_END ){
		pcp->Fshot = SHOT_MID;
		pcp->Fgc = 0;
 
		cp->Fshot = SHOT_END;
		cp->Fgc = 1;
	}

 


	ocp->no++;

	return( cp );
}






/***** ocpL_kf_removable *****
Description:The method checks whether it is leagal to remove the keyframe number "frame"
			The leagal keyframes are :	1.standalone keyframe
										2.keyframe in a segment with more than 2 keyframes in it.

Params:	ocp		- the list
		frame	- frame number to delete
Returns:removable(1) or not(0)
*****************************/
int ocpL_kf_removable(ocp_list *ocp, int frame)
{
	ocp_type *cp0, *cp1, *cp2;
	
	if ( ocp->cp == NULL )
		return 0;
	cp0 = NULL;
	cp1 = NULL;
	for( cp2 = ocp->cp ; cp2->fno + cp2->dfno < frame; cp2 = cp2->next)
	{
		cp0 = cp1;
		cp1 = cp2;	
	}
	if ( !cp2)
		return 0;
	if ( cp2->fno != frame )
		return 0;
	// cp0 is two keyframes before frame
	// cp1 is the keyframe before frame .
	if ( !cp1 || cp1->Fshot == SHOT_END)// start of a segment
	{
		if ( cp2->next && cp2->next->Fshot == SHOT_END)// it is a start of a segment with 1 more keyframes
			return 0;
		// else - it is a single keyframe, or one of at least 3 in a segment
	}
	// it's not the start of a segment - 
	// check that if it is the end - there are two more frames in the segment!!!
	if (!cp0 || cp0->Fshot == SHOT_END)// it is in a segment with only 2 keyframes
	{
		if ( !cp1->next )
			return 0;
	}
	return 1;
}


/***** ocpL_kf_delete_segment *****
Description:The method removes all keyframes between f0 and f1
Params:ocp - the list
		f0, f1 - the range
Returns:the previous keyframe to f0.
		if f0 is the first cp, then next one is returned.
		NULL is returned if the operation wasn't performed.	
*****************************/
ocp_type * ocpL_kf_delete_segment( ocp_list *ocp, int f0, int f1)
{
	ocp_type *pcp0,*cp0, *pcp1, *cp1,	 *cp;
	int Fshot;

	
	if ( ocp->cp == NULL )
		return NULL;

	pcp0 = NULL;
	for( cp0 = ocp->cp ; cp0 && (cp0->fno + cp0->dfno < f0); cp0 = cp0->next)
		;	
	// cp0 is the keyframe before f0.
	for( cp1 = cp0 ; cp1 && (cp1->fno  <= f1); cp1 = cp1->next)
		pcp1 = cp1;
	// pcp1 is the keyframe before f1 or on f1.
	// cp1 is the keyframe after f1 ( might be null ).

	if ( pcp1 == NULL )
		return NULL;// no need to delete anything.
	
	Fshot = pcp1->Fshot;
	if ( ocp->cp->fno >= f0 )
	{
		//start deletion befor first keyframe (ocp->cp)
		while ( ocp->cp != cp1 ) 
		{
			cp = ocp->cp;
			ocp->cp = cp->next;
			ocp->no --;
			free ( cp );
		}	
		if ( cp1 )
		{
			cp1->gc = cp1->c0;
			cp1->Fgc = 1;
		}
		return cp1;
	}

	// delete all keyframes between cp0 and cp1
	cp = cp0->next;	
	{
		while ( cp != cp1 )
		{
			cp0->next = cp->next;
			cp0->dfno += cp->dfno;
			ocp->no --;
			free ( cp );			
			cp = cp0->next;			
		}
		if ( cp0->next == NULL )
			cp0->dfno = 1;
	}



	switch( Fshot )
	{
	case SHOT_MID:
	    break;
/*
	case SHOT_BEGIN:
		
		
		if ( cp1 )
		{
			if( cp1->Fgc != 1 )	cp1->gc = cp1->c0;
			cp1->Fgc = 1;
		}
	    break;
*/
	case SHOT_END:
		if ( cp0 )
		{
			cp0->Fshot = SHOT_END;
			cp0->Fgc = 1;
		}
	    break;
	}
 

	if ( cp0 )
		return cp0;
	return cp1;
}


/***** ocpL_kf_delete *****
Description:The method removes a keyframe.
Params:	ocp - the list
		ccp - the frame to remove
Returns:the previous keyframe.
		if ccp is the first cp, then next one is returned.
		NULL is returned if the operation wasn't performed.
*****************************/
ocp_type * ocpL_kf_delete( ocp_list *ocp, ocp_type *ccp )
{
ocp_type *pcp,	*Lcp;
// addition
	
	if ( ocp->cp == NULL || ccp == NULL )
		return NULL;

	
	if( ocp->cp != ccp )
	{
		for( pcp = ocp->cp ; pcp->next != ccp ; pcp = pcp->next);

		pcp->next = ccp->next;

		if( pcp->next != NULL )
			pcp->dfno = pcp->next->fno - pcp->fno;
		else 
			pcp->dfno = 1;
	}
	else    
	{
		ocp->cp = ccp->next;
		pcp = ccp->next;
	}

	ocp->no--;




	switch( ccp->Fshot ) 
	{
	case SHOT_MID:
	    break;
/*
	case SHOT_BEGIN:
		Fcp = ccp->next;		
		if ( Fcp )
		{
			if( Fcp->Fgc != 1 )	Fcp->gc = Fcp->c0;
			Fcp->Fgc = 1;
		}
	    break;
*/
	case SHOT_END:
		Lcp = pcp;
		if ( Lcp )
		{
			Lcp->Fshot = SHOT_END;
			Lcp->Fgc = 1;
		}
	    break;
	}
 

	free( ccp );

	return( pcp );
}

/*the method sets an ocp to have a new fno*/
void
ocpL_kf_set_fno( ocp_list *ocp, ocp_type *acp, int fno )
{
ocp_type *pcp,	*cp;	
	 pcp = NULL;
	 /*find the kframe predecesor to acp in the list*/
	 if( ocp->cp != acp ){
		for( pcp = ocp->cp ; pcp->next != acp ; pcp = pcp->next );
		if( fno <= pcp->fno )	fno = pcp->fno+1;/*cannot swap locations in the list with another kframe*/
	 }

	/*refresh the fdno and fno of the rest ocps from that pcp an on*/		
	 for( cp = acp ; cp != NULL ; cp = cp->next ){
		cp->fno = fno;
		if( pcp != NULL )	pcp->dfno = cp->fno - pcp->fno;

		fno = cp->fno + cp->dfno;
		pcp = cp;

		if(	cp->Fshot == SHOT_END && cp->next != NULL ){
			if( cp->fno < cp->next->fno ){
				cp->dfno = cp->next->fno - cp->fno;
				return;
			}
			fno = cp->fno+1;
		}
	 }


/*
	for( pcp = ocp->cp ; pcp->next != acp ; pcp = pcp->next );

	if( fno - pcp->fno <= 0 )      fno = pcp->fno+1;
 
	pcp->dfno = fno - pcp->fno;
 
	acp->fno = fno;
*/
}



void	ocpL_kf_set_Fgc(ocp_list *ocpL, int kfno, int Fgc)
{
	ocp_type *cp1, *cp2;
	
	if ( ocpL->cp == NULL )
		return ;
	cp1 = NULL;	
	for( cp2 = ocpL->cp ; cp2->fno + cp2->dfno <= kfno; cp2 = cp2->next)
	{		
		cp1 = cp2;	
	}
	if ( cp2 == NULL )
		return;
	if ( Fgc == 0 )
	{
		if ( cp1 && cp1->Fshot != SHOT_END && cp2->Fshot != SHOT_END)
			cp2->Fgc = Fgc;		
	}
	else
	{
		cp2->Fgc = Fgc; 
	}
}


/***** ocpL_kf_index *****
Description:The method returns the location of the shape in the list.
	The cp doesn't have the same fno, and so no using ocp_is_equal itself!
Params:	kf		- the keyframe to check
		list	- the list of keyframes
Returns:-1 if no kf found, or the index which exceeds 0.
*****************************/
int ocpL_kf_index(ocp_type *kf, ocp_list *list)
{
	int i  ;
	ocp_type *cp;
	
	for ( i = 0, cp = list->cp; cp != NULL ; cp = cp->next, i ++)
	{
		if( cp->c.x == kf->c.x && cp->c.y == kf->c.y )
			return i;
	}
	return -1;
}







/***** ocpL_reset_fno *****
Description:The method resets the fno and dfno parameters in the ocp list.
Params:ocp - the list to reset
Returns:fno of the last ocp_type node in the list.
*****************************/
int ocpL_reset_fno( ocp_list *ocp )
{
ocp_type *pcp,	*cp;
int	fno;

	fno = ocp->cp->fno;

	pcp = NULL;
	for( cp = ocp->cp ; cp != NULL ; cp = cp->next ){
		cp->fno = fno;
		if( pcp != NULL )	pcp->dfno = cp->fno - pcp->fno;

		fno = cp->fno + cp->dfno;
		pcp = cp;



		if(	cp->Fshot == SHOT_END && cp->next != NULL ){
			if( cp->fno < cp->next->fno ){
				cp->dfno = cp->next->fno - cp->fno;
				fno = cp->fno + cp->dfno;
			}
			else	{
				fno = cp->fno+1;
				cp->dfno = 1;
			}
		}
	 }

	return( fno );
}




void
ocpL_rescale_c( ocp_list *ocp, float x0, float x, float y0, float y )
{
ocp_type	*cp;

	for( cp = ocp->cp ; cp != NULL ; cp = cp->next ){
		cp->c.x = ( cp->c.x + x0 ) * x;
		cp->c.y = ( cp->c.y + y0 ) * y;
	}
}



void
ocpL_shift2D( ocp_list *ocp, ocp_type *acp, int flag, vec2d *dp )
{
ocp_type	*Fcp,	*Lcp, *cp;
float	t;


	acp->c.x += dp->x;
	acp->c.y += dp->y;

	if( flag == 0 )	return;

	ocpLF_shot( ocp, acp->fno, &Fcp, &Lcp );

	if( flag & 0x01 ){
		if( acp != Fcp )
			for( cp = Fcp->next ; cp != acp ; cp = cp->next ){
				t = (cp->fno - Fcp->fno )/ (float)(acp->fno - Fcp->fno );
				cp->c.x += dp->x * t;
				cp->c.y += dp->y * t;
			}
	}

	if( flag & 0x02 ){
		if( acp != Lcp )
			for( cp = acp->next ; cp != Lcp ; cp = cp->next ){
				t = 1.0 - (cp->fno - acp->fno )/ (float)(Lcp->fno - acp->fno );
				cp->c.x += dp->x * t;
				cp->c.y += dp->y * t;
			}
	}
}





void
ocp_rotate( ocp_type *acp, vec2d *ctr, vec2d *p0, vec2d *p1 )
{
vec2d   u0,     u;
float   t,      cosa,   sina;
 

	u0.x = p0->x - ctr->x;
	u0.y = p0->y - ctr->y;
 
	if( u0.x == 0 && u0.y == 0 )    return;
 
	t = (float)(1.0 / hypot( u0.x, u0.y ));
	u0.x *= t;
	u0.y *= t;
 
 
	u.x = p1->x - ctr->x;
	u.y = p1->y - ctr->y;
 
	if( u.x == 0 && u.y == 0 )      return;
 
	t = (float)(1.0 / hypot( u.x, u.y ));
	u.x *= t;
	u.y *= t;
 
 
 
	cosa = u.x * u0.x + u.y * u0.y;
	sina = -u.y * u0.x + u.x * u0.y;
 
	skl_rotate( acp->skl, ctr, cosa, sina );
} 


//eyal

/***** ocpL_setAnimAt *****
Description:The method sets an
Params:	ocpL - the ocp 
		startAt - new startFrame for the animation
Returns:0 - the operation failed.
		n - the last fno of the animation
*****************************/
int ocpL_setAnimAt(ocp_list *ocpL, int startAt)
{
	ocp_type *cp;
	int diff;	
	int fno = 0;
	if ( ocpL == NULL || ocpL->cp == NULL )
		return fno;		
	// set new one
	diff = startAt - ocpL->cp->fno;	
	for ( cp = ocpL->cp; cp != NULL ; cp = cp->next)
	{
		cp->fno += diff;
		fno = cp->fno;
	}
	return fno;		
}


void 
ocpL_fno_add(ocp_list *ocpL, int dframe )
{
ocp_type *cp;

	if ( ocpL == NULL  )	return;

	for ( cp = ocpL->cp; cp != NULL ; cp = cp->next)
		cp->fno += dframe;
	
}
