/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	OcpLTool1.c	***
 **************************/
#include	<math.h>

#include	"Uigp/igp.h"


#include	"VimageType/OcpType.h"



ocp_type *
ocpL_add_shot( ocp_list *ocp,
		ocp_type *ccp, int frame, int Fcontinuance,
		ocp_list *ocp_n )
{
ocp_type	*Lcp;
ocp_type	*cp,	*lcp;
config3d	c;

	
	c = ocp_n->cp->c;
	for( cp = ocp_n->cp ; cp != NULL ; cp = cp->next ){
		cp->Fshot = SHOT_MID;
		cp->Fgc = 0;

		cp->c.x -= c.x;
		cp->c.y -= c.y;

		lcp = cp;
	}

	ocp_n->cp->Fshot = SHOT_MID;
	ocp_n->cp->Fgc = 1;
	lcp->Fshot = SHOT_END;
	lcp->Fgc = 1;

	ocp_n->cp->fno = frame;



	Lcp = ocpLF_get_kf( ocp, frame );



	if( Fcontinuance == 1 && Lcp != NULL ){
		c = ccp->gc;
		c.x += Lcp->c.x;
		c.y += Lcp->c.y;
	}
	else    config3d_init( &c );



	if( Lcp == NULL ){
		lcp->next = ocp->cp;
		ocp->cp = ocp_n->cp;
	}
	else	{
		lcp->next = Lcp->next;
		Lcp->next =  ocp_n->cp;

		Lcp->dfno = Lcp->next->fno - Lcp->fno;
		if( Lcp->dfno < 1 )	Lcp->dfno = 1;
	}


	ocp->no += ocp_n->no;

	cp = ocp_n->cp;
	ocp_n->cp = NULL;
	ocp_n->no = 0;

	return(cp );
}




ocp_type *
ocpL_add_segment( ocp_list *ocp, ocp_type *cp0, int frame, ocp_list *ocp_n )
{
ocp_type	*cp,	*cp1,	*lcp;
config3d	c;
int	fno;

	cp1 = cp0->next;
	
	c = ocp_n->cp->c;
	for( cp = ocp_n->cp ; cp != NULL ; cp = cp->next ){
		cp->Fshot = SHOT_MID;
		cp->Fgc = 0;

		cp->c.x += cp0->c.x - c.x;
		cp->c.y += cp0->c.y - c.y;

		if( cp->dfno < 1 )	cp->dfno = 1;

		lcp = cp;
	}

	lcp->dfno = 1;


	lcp->next = cp0->next;
	cp0->next = ocp_n->cp;
	ocp->no += ocp_n->no;


	ocp_n->cp->fno = frame;
	cp0->dfno = ocp_n->cp->fno - cp0->fno;
	if( cp0->dfno < 1 )	cp0->dfno = 1;

	if( cp1 != NULL ){
		fno =  cp0->fno;
		for( cp = ocp_n->cp ; cp != cp1 ; cp = cp->next ){
			fno += cp->dfno;
			cp->fno = fno;
		}
		lcp->dfno = cp1->fno - lcp->fno;
		if( lcp->dfno < 1 )	lcp->dfno = 1;
	}

	

	if( cp0->Fshot == SHOT_END ){
		cp0->dfno = 1;

		cp0->Fshot = SHOT_MID;
		lcp->Fshot = SHOT_END;

		cp = ocp_n->cp;
		ocp_n->cp = NULL;
		ocp_n->no = 0;

		return( cp );
	}


	c.x = lcp->c.x - cp0->c.x + cp0->c.x - cp0->c.x;
	c.y = lcp->c.y - cp0->c.y + cp0->c.y - cp0->c.y;

	for( cp = cp1 ; cp != NULL ; cp = cp->next ){
		cp->c.x += c.x;
		cp->c.y += c.y;

		if( cp->Fshot == SHOT_END )	break;
	}


	cp = ocp_n->cp;
	ocp_n->cp = NULL;
	ocp_n->no = 0;

	return( cp );
}




/***** ocpL_get_shot *****
Description:The method puts in Fcp and Lcp the keyframes that enclose cp0 in a shot.
Params:	ocp	-	the list
		cp0	-	the keyframe to look for
		Fcp -	First keyframe in the shot 
		Lcp -	Last keyframe in the shot
Returns:
*****************************/
void ocpL_get_shot( ocp_list *ocp, ocp_type *cp0, ocp_type **Fcp, ocp_type **Lcp )
{
	ocp_type	*cp;
	if ( Fcp != NULL )
		*Fcp =  NULL;
	if ( Lcp != NULL )
		*Lcp = NULL;

	if( (cp0 == NULL) || (ocp == NULL) || (ocp->cp == NULL))	return;
	
	cp = ocp->cp;
	for( *Fcp = cp ; cp != cp0  ; cp = cp->next )
		if( cp->Fshot == SHOT_END ) 
			*Fcp = cp->next;
	 
	for( cp = cp0 ;  cp->Fshot != SHOT_END && cp->next != NULL ;cp = cp->next ) 
		; 
	*Lcp = cp;
	
}


void
ocpL_copy_shot( ocp_type *Fcp, ocp_type *Lcp, ocp_list *ocp )
{
ocp_type	**cp,	*scp;
int	fno;

	fno = Fcp->fno;

        ocp->no = 0;
        cp = &ocp->cp;
        for( scp = Fcp ; scp != NULL ; scp = scp->next ){
 
		*cp = ocp_make_copy( scp );
		(*cp)->fno -= fno;

                ocp->no++;

		cp = &(*cp)->next;
 
 
                if( scp == Lcp )     break;
        }

	*cp = NULL;
}


/*** DEPRECATED ***
USE IN GPVEWER ONLY
******************/
void
ocpL_inverse( ocp_list *ocp )
{
ocp_type	*cp,	*cp0,	*cp1;
int	fno;

	fno = 0;
	for( cp = ocp->cp; cp != NULL ; cp = cp->next ){
		cp->fno = fno;
		if( cp->next != NULL )
			fno += cp->dfno;
	}

	cp0 = NULL;
	for( cp = ocp->cp; cp != NULL ;  ){
		cp->fno = fno - cp->fno;
		if( cp0 != NULL )
			cp->dfno = cp0->fno -  cp->fno;
		else	cp->dfno = 1;

		cp1 = cp;
		cp = cp->next;

		cp1->next = cp0;
		cp0 = cp1;
	}

	ocp->cp = cp0;
}




/***** ocpL_anim_init *****
Description:The method sets the first cp at 0,0 and moves all the rest by the same Delta.
Params:ocp - the list
		fInit3D - flag - erase the 3d information? (1 - yes, 0 - no)
Returns:
*****************************/
int ocpL_anim_init( ocp_list *ocp ,int fInit3D )
{

ocp_type	*cp,	*lcp;
config3d c;
int	fno;

	c = ocp->cp->c;

	for( cp = ocp->cp, fno = 0 ; cp != NULL ; cp = cp->next ){

		if( cp->next == NULL )	cp->dfno = 1;

		cp->fno = fno;
		fno += cp->dfno;

		cp->Fshot = SHOT_MID;

		if ( fInit3D)	cp->Fgc = 0;

		cp->c.x -= c.x;
		cp->c.y -= c.y;

		lcp = cp;
	}

	ocp->cp->Fgc = 1;
	

	lcp->Fshot = SHOT_END;
	lcp->Fgc = 1;


	if ( fInit3D)
	{
		config3d_init( &ocp->cp->gc );
		config3d_init( &lcp->gc );
	}


	return( fno );
}







/***** ocpL_size *****
Description:The method finds the number of keyframes in the ocp_list
Params:ocp	- the list to check
Returns:the number of keyframes in the list
*****************************/
int			ocpL_size(ocp_list *ocpL)
{
	int no = 0;
	ocp_type * cp;
	for ( cp = ocpL->cp; cp != NULL ; cp = cp->next)
		no++;
	return no;
}

/***** ocpL_length *****
Description:The method finds out the length (in frames) of the animation
Params:ocpL - the list to check
Returns:the length of the animation ( 0 if NULL or bad parameters )
*****************************/
int			ocpL_length(ocp_list *ocpL)
{
	int start,end;
	ocp_type * cp = ocpL->cp;
	if ( cp == NULL)
		return 0;
	start = cp->fno;
	for (; cp->next != NULL ; cp = cp->next);
	end = cp->fno;
	if ( end < start ) 
		return 0;
	return (end - start + 1);
}
