/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***********************
 ***    OcpTool.c    ***
 ***********************/ 
#include	<math.h>

#include	"Uigp/igp.h"


#include	"VimageType/OcpType.h"



ocp_list *
ocpL_alloc()
{ 
ocp_list *ocp;

	ocp = (ocp_list *)malloc( sizeof(ocp_list) );

	ocp->cp = ocp->acp = NULL; 
	ocp->no = 0;    

	ocp->fi = -1;

	return( ocp );
}



ocp_list *
ocpL_alloc2(  skl_type *skl, int f0, int f1 )
{ 
ocp_list *ocp;
ocp_type	*cp;

	ocp = (ocp_list *)malloc( sizeof(ocp_list) );

	if( f0 == f1 ){
		cp = ocp_alloc( skl );
		ocp->cp = cp;

		cp->Fshot = SHOT_END;
		cp->fno = f0;
		cp->dfno = 1;
		cp->Fgc = 1;

		cp->next = NULL;
		ocp->no = 2;
		ocp->acp = ocp->cp;
		return( ocp );
	}

 
 
	cp = ocp_alloc( skl );
	ocp->cp = cp;

	cp->Fshot = SHOT_MID;
	cp->fno = f0;
	cp->dfno = f1 - f0;
	cp->Fgc = 1;


	cp = ocp_alloc( skl );
	ocp->cp->next = cp;

	cp->Fshot = SHOT_END;
	cp->fno = f1;
	cp->dfno = 1;
	cp->Fgc = 1;


	cp->next = NULL;
	ocp->no = 2;
	ocp->acp = ocp->cp;

	return( ocp );
}





void
ocpL_destroy( ocp_list *ocp )
{ 
	ocpL_free( ocp );

	free( ocp );
}



int
ocpL_is_init( ocp_list *ocp )
{
ocp_type        *cp;
		if ( ocp->cp == NULL )//eyal
			return 1;//eyal
        for( cp = ocp->cp->next ; cp != NULL ; cp = cp->next )
                if( ocp_is_init( cp ) == 0 )    return( 0 );
 
        return( 1 );
}



/***** ocpL_is_clear *****
Description:The method checks whether 
Params:	ocp		- the list
		skl		- the skl to compare to ( movement of skl )
		frame	- frames in the scene?
Returns:0 - there is animation
		1 - the ocp is clear (or - the cast is static, and the frames appear at start and end of the scene)
		2 - the ocp is static, but won't appear at all frames and thus there is animation, 
			or has more than 2 keyframes 
*****************************/
int ocpL_is_clear( ocp_list *ocp, skl_type *skl, int frame )
{
ocp_type        *cp,	*pcp;
	if ( ocp->cp == NULL )
		return 1;
	for( cp = ocp->cp ; cp != NULL ; cp = cp->next ){
		if( cp->c.x != 0 || cp->c.y != 0 )
			return( 0 ); 

		if( cp->Fgc == 1 && !CONFIG3D_IS_INIT( cp->gc ) )
			return( 0 );

		if( !skl_is_equal( cp->skl, skl ) )
			return( 0 );

		pcp = cp;
	}

	if( ocp->cp->fno != 0 || pcp->fno != frame-1 )	return( 2 );
	if ( ocp->cp->next && ocp->cp->next->fno != frame-1) return 2;
	return( 1 );
}



void
ocp_list_init( ocp_list *ocp )
{ 
        ocp->cp = ocp->acp = NULL; 
        ocp->no = 0;    
}

 
void
ocpL_free( ocp_list *ocp )
{
ocp_type       *cp;
 
        while( (cp = ocp->cp ) != NULL ){
                ocp->cp = ocp->cp->next;
 
                skl_free( cp->skl );
                free( cp );
        }

	ocp->no = 0;
}



void
ocp_list_clear( ocp_list *ocp, skl_type *skl, int cp_no )
{
ocp_type	*cp;

	ocpL_free( ocp );

        for( ocp->no = 0; ocp->no < cp_no ; ocp->no ++ ){
                cp = ocp_alloc( skl );

                cp->next = ocp->cp;
                ocp->cp = cp;
        }
}






void
ocp_list_rescale_c( ocp_list *ocp, float rescale )
{
ocp_type	*cp;

	for( cp = ocp->cp ; cp != NULL ; cp = cp->next ){
		cp->c.x *= rescale;
		cp->c.y *= rescale;
	}
}






ocp_type *
ocp_alloc( skl_type *skl )
{
ocp_type	*cp;

	cp = (ocp_type *)malloc( sizeof(ocp_type) );

	config3d_init( &cp->gc );
	cp->Fgc = 0;

	config3d_init( &cp->c );
	cp->dp.x = cp->dp.y = 0;

	cp->dfno = 1;
	cp->fno = 0;
	cp->Fshot = 0;



	cp->skl = ( skl == NULL )? NULL : skl_make_copy( skl );



	return( cp );
}


void
ocp_clear( ocp_type *cp )
{
	config3d_init( &cp->c );
	cp->dp.x = cp->dp.y = 0;

	cp->dfno = 1;
	cp->Fshot = 0;
}


void
ocp_free( ocp_type *cp )
{
	if( cp == NULL )	return;

	if( cp->skl != NULL )
		skl_free( cp->skl );

	free( cp );
}


ocp_type *
ocp_make_copy( ocp_type *cp )
{
ocp_type * ocp;

        ocp = (ocp_type *)malloc( sizeof(ocp_type) );

	*ocp = *cp;

	ocp->next = NULL;//*****
	ocp->skl = skl_make_copy( cp->skl );

	return( ocp );
}


void
ocp_copy( ocp_type *scp, ocp_type *tcp )
{
	tcp->c = scp->c;
	tcp->dp = scp->dp;
	tcp->dfno = scp->dfno;
	tcp->Fshot = scp->Fshot;

	tcp->Fgc = scp->Fgc;
	tcp->gc = scp->gc;

	skl_copy( scp->skl, tcp->skl );
}


int
ocp_is_init( ocp_type *cp )
{
	if( cp->c.x != 0 || cp->c.y != 0 )	return( 0 );
                                                
	if( cp->dfno != 1 )	return( 0 );
	if( cp->Fshot != 0 )	return( 0 );

	return( 1 );
}


int
ocp_is_equal( ocp_type *cp, ocp_type *rcp )
{
	if( cp->c.x != rcp->c.x || cp->c.y != rcp->c.y )
		return( 0 );
                                                

	if( CONFIG3D_IS_EQUAL( cp->c0, rcp->c0 ) != 0 )
		return( 0 );


	if( skl_is_equal( cp->skl, rcp->skl ) != 0 )
		return( 0 );

	return( 1 );
}



// return true if this kframe add nothing to the animation
int 
ocp_is_clear( ocp_type *cp, skl_type *bskl )
{
	if( cp->c.x != 0 || cp->c.y != 0 )
		return( 0 ); 

	if( cp->Fgc == 1 && !CONFIG3D_IS_INIT( cp->gc ) )
		return( 0 );

	if( skl_is_equal( cp->skl, bskl )!= 0 )
		return( 0 );

	return( 1 );
}



void
ocp_list_flipV( ocp_list *ocp, int column )
{
ocp_type	*cp;

	for( cp = ocp->cp  ; cp != NULL ; cp = cp->next ){

		cp->c.x = -cp->c.x;
		cp->gc.x = -cp->gc.x;
		skl_flipV( cp->skl, column );
	}
}

