/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*************************
 ***    TcpLFrame.c    ***
 *************************/
#include	"Uigp/igp.h"



#include	"VimageType/TcpType.h"


tcp_type *
tcpL_kf_add( tcp_list *tcp, tcp_type *pcp, config3d *c, int fno )
{
tcp_type	*cp;

	cp = tcp_alloc();

	cp->fno = fno;
	cp->c = *c;

	cp->next = pcp->next;
	pcp->next = cp;
	tcp->no++;



	pcp->dfno = cp->fno - pcp->fno;

	if( cp->next != NULL )
		cp->dfno = cp->next->fno - cp->fno;

	return( cp );
}


tcp_type *
tcpL_kf_delete( tcp_list *tcp, tcp_type *acp )
{
tcp_type	*cp;

	for( cp = tcp->cp ; cp->next != acp ; cp = cp->next );

	cp->next = acp->next;

	cp->dfno = cp->next->fno - cp->fno;

	free( acp );

	if( tcp->a_cp == acp )	tcp->a_cp = cp;

	tcp->no--;

	return( cp );
}


tcp_type *
tcpLF_copy( tcp_list *tcp, int frame )
{
tcp_type	*acp,	*cp;
float	t;


	for( acp = tcp->cp ; acp->fno + acp->dfno <= frame && acp->next != NULL ; acp = acp->next );


	if( acp->fno == frame ){
		cp = tcp_make_copy( acp );

		return( cp );
	}


	t = ( frame - acp->fno ) / (float)acp->dfno;
	cp = tcp_affine( acp, acp->next, t, NULL );

	cp->fno = frame;
	cp->dfno = cp->next->fno - cp->fno;

	return( cp );
}


int tcpLF_paste( tcp_list *tcp, int frame, tcp_type *cp, int Fadd )
{
tcp_type	*acp;


	for( acp = tcp->cp ; acp->fno + acp->dfno <= frame ; acp = acp->next );



	if( acp->fno != frame ){
		if( Fadd == 0 )	return( 0 );
		acp = tcpL_kf_add( tcp, acp, &cp->c, frame );
		return( 2 );
	}



	acp->c = cp->c;

	return( 1 );
}



void
tcpL_kf_set_fno( tcp_list *tcp, tcp_type *acp, int fno )
{
tcp_type *pcp;

	if( acp->fno == 0 )	return;
 
	for( pcp = tcp->cp ; pcp->next != acp ; pcp = pcp->next );
 
	if( fno - pcp->fno <= 0 )      fno = pcp->fno+1;
 
	pcp->dfno = fno - pcp->fno;
 
	acp->fno = fno;
} 


int
tcpLF_is_kfG( tcp_list *tcp, int frame )
{
tcp_type        *cp;
 
        if( tcp->cp == NULL )   return( 0 );
 
        for( cp = tcp->cp ; cp != NULL ; cp = cp->next ){
                if( frame == cp->fno )	return( 1 );
 
                if( frame < cp->fno ){
 
                        return( 0 );
                }
        }
 
        return( 0 );
}



/***** tcpLF_is_kf *****
Description: The method determines if the frame is keyframe or not.
Params:	tcp - the list to check
		frame - the number of frame to query
Returns: 1 - a keyframe, 0 - not a key frame
*****************************/
int tcpLF_is_kf( tcp_list *tcp, int frame )
{
	tcp_type * cp = tcp->cp;
	if (( cp == NULL) || ( cp->fno > frame ) )
		return 0;
	while ( cp && (cp->fno < frame ))
		cp = cp->next;			
	if ( !cp || (cp->fno > frame))
		return 0;
	return 1;	
}




/***** tcpLF_status *****
Description: The method determines the status of frame number "frame"
	there might be 4 states:
		regular frame		-----	110	
		
		end keyframe		--*		101
		start keyframe		  *--	011
		middle keyframe		--*--	111

		
Params:	ocp - the list to check
		frame - the number of frame to query
Returns: a mask according to the state
*****************************/
int tcpLF_status( tcp_list *tcp, int frame )
{
	tcp_type * acp ,*prevCp = NULL;
	
	for( acp = tcp->cp ; acp != NULL && acp->fno + acp->dfno <= frame ; prevCp = acp, acp = acp->next )
		;
	if ( acp == NULL || acp->fno > frame )
		return FRAME_BLANK;// the only possible way to get a blank frame from tcp - the frame exceeds the scene
		
	
	if ( acp->fno == frame )// a keyframe
	{
		if ( prevCp )
		{
			if (acp->next == NULL)
				return FRAME_K_END;
			else 
				return FRAME_K_MIDDLE;
		}// has a previous
		else 
			return FRAME_K_BEGIN;				
	}
	return FRAME_MIDDLE;		
	

}





tcp_type *
tcpLF_force_kf( tcp_list *tcp, int frame )
{
tcp_type	*acp,	*cp;
float	t;


	for( acp = tcp->cp ; acp != NULL && acp->fno + acp->dfno <= frame ; acp = acp->next );

	if( acp == NULL )	return(NULL);


	if( acp->fno == frame )	return( acp );



	t = ( frame - acp->fno ) / (float)acp->dfno;
	cp = tcp_affine( acp, acp->next, t, NULL );


	cp->next = acp->next;
	acp->next = cp;


	cp->fno = frame;
	cp->dfno = cp->next->fno - cp->fno;

	acp->dfno = cp->fno - acp->fno;

	tcp->no++;


	return( cp );
}
