/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***********************
 ***    TcpTool.c    ***
 ***********************/
#include	"Uigp/igp.h"



#include	"VimageType/TcpType.h"



tcp_type * tcp_alloc()
{
tcp_type	*cp;

    cp = (tcp_type *)malloc( sizeof(tcp_type) );
	memset(cp,0,sizeof(tcp_type));
	

	config3d_init( &cp->c );

	cp->Fshot = 0;


	return( cp );
}



void tcp_free( tcp_type *cp )
{
	if( cp == NULL )	
		return;

	free( cp );
}



tcp_type *tcp_make_copy( tcp_type *scp)
{
tcp_type	*cp;

	cp = (tcp_type *)malloc( sizeof(tcp_type) );

	*cp = *scp;


	return( cp );
}


/*********  tcp_equals  **********
* @Description	:  method to check whether two camera nodes have the same plane
* 
* @Param  tcp_type *cp0 :  first camera node to check
* @Param  tcp_type *cp1 :  second camera node to check
* @Return   :  1 - equals, 0 - different
*****************************************/
int	tcp_equals(tcp_type *cp0, tcp_type *cp1)
{
	if (	cp0->c.u1 == cp1->c.u1 &&
		cp0->c.u2 == cp1->c.u2 &&
		cp0->c.u3 == cp1->c.u3 &&
		cp0->c.x == cp1->c.x &&
		cp0->c.y == cp1->c.y &&
		cp0->c.z == cp1->c.z )
		return  1;
	return 0;
}


/*********  *tcp_affine  **********
* @Description	:  method to affine an interpolation of camera node out of two others, 

* 
* @Param   tcp_type *cp0 :  first node
* @Param  tcp_type *cp1 :  second node
* @Param  float t :  ratio for interpolation
* @Param  tcp_type *cp :  NULL for new allocation, or node to put in the data.
* @Return  tcp_type  :  the resulting node
*****************************************/
tcp_type *tcp_affine( tcp_type *cp0, tcp_type *cp1, float t, tcp_type *cp )
{
	if( cp == NULL )
		cp = tcp_make_copy( cp0 );


	cp->c.x = t*( cp1->c.x - cp0->c.x ) + cp0->c.x;
	cp->c.y = t*( cp1->c.y - cp0->c.y ) + cp0->c.y;
	cp->c.z = t*( cp1->c.z - cp0->c.z ) + cp0->c.z;

	cp->c.u1 = t*( cp1->c.u1 - cp0->c.u1 ) + cp0->c.u1;
	cp->c.u2 = t*( cp1->c.u2 - cp0->c.u2 ) + cp0->c.u2;
	cp->c.u3 = t*( cp1->c.u3 - cp0->c.u3 ) + cp0->c.u3;

	return( cp );
}









/*********  *tcpL_alloc  **********
* @Description	:  allocates an empty list of camera nodes
* 
* @Return  tcp_list  :  empty list
*****************************************/
tcp_list *tcpL_alloc()
{
tcp_list	*tcp;

	tcp = (tcp_list *)malloc( sizeof(tcp_list) );

	tcp->cp = NULL;
	tcp->no = 0;

	return( tcp );
}





/*********  tcpL_clear  **********
* @Description	:  clears the list of all it's contents, and sets new no of nodes.
* 
* @Param   tcp_list *tcp :  the list
* @Param  int no :  number of nodes to create
* @Param  int frame :  length of the scene
* @Return  void  :  
*****************************************/
void tcpL_clear( tcp_list *tcp, int no, int frame )
{
tcp_type	*cp;
int	i;

	while( (cp = tcp->cp) != NULL ){
		tcp->cp = tcp->cp->next;
		free( cp );
	}


	for( i = no-1 ; i >= 0 ; i-- ){
		cp = tcp_alloc();

		cp->fno = (i * (frame-1) ) / (no-1 );
		cp->dfno = 1;

		if( tcp->cp != NULL )	cp->dfno = tcp->cp->fno - cp->fno;

		cp->next = tcp->cp;
		tcp->cp = cp;
	}

	tcp->a_cp = tcp->cp;
	tcp->no = no;
}


void
tcpL_destroy( tcp_list *tcp )
{
tcp_type       *cp;

	while( (cp = tcp->cp ) != NULL ){
		tcp->cp = tcp->cp->next;
		free( cp );
	}

	free( tcp );
}






/*********  tcpL_is_clear  **********
* @Description	:  returns true only if all of the nodes has the initial configuration.
* 
* @Param  tcp_list *tcp :  list
* @Return  int  :  1 - clear, 0 - not clear.
*****************************************/
int tcpL_is_clear( tcp_list *tcp )
{
tcp_type	*cp;

	for( cp = tcp->cp ; cp != NULL ; cp = cp->next )
		if( !CONFIG3D_IS_INIT( cp->c ) )	return( 0 );

	return( 1 );
}



/*********  tcpL_rescale_frame  **********
* @Description	:  rescales the scene
* 
* @Param   tcp_list *tcp :  list to work on
* @Param  int cframe_no :  current frame number in the scene
* @Param  int frame_no :  new frame number in the scene
* @Return  void  :  
*****************************************/
void tcpL_rescale_frame( tcp_list *tcp, int cframe_no, int frame_no )
{
tcp_type	*cp,	*pcp;
 
	pcp = NULL;
	for( cp = tcp->cp ; cp != NULL ; cp = cp->next ){
		cp->fno = ( cp->fno * (frame_no-1) ) / (cframe_no-1 );
		cp->dfno = 1;
 
		if( pcp != NULL )	pcp->dfno = cp->fno - pcp->fno;
 
		pcp = cp;
	}
} 