/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	TcpEncodingp.c	***
 *************************/
#include	<math.h>

#include	"Ubst/BstType.h"


#include	"BframeType.h"


void
bframeL_encoding( bframeL_type *bfL, bst_type *bst )
{
bframe_type		*bf;
int	no,	head;
int	frame_no,	bitFno;
int	maxFlashing,	maxLedFlashing,	maxVibration,	maxDelay;
int	bitFlashing,	bitLedFlashing,	bitVibration,	bitDelay;


	head = 0;

	maxFlashing = maxLedFlashing = maxVibration = maxDelay = 0;
	for( bf = bfL->bf ; bf != NULL ; bf = bf->next ){
		if( bf->priority != 0 ){
			head |= BFRAME_PRIORITY;

			if( bf->delay > maxDelay )	maxDelay = bf->delay;
		}

		if( bf->flashing > maxFlashing )	maxFlashing = bf->flashing;

		if( bf->ledFlashing > maxLedFlashing )	maxLedFlashing = bf->ledFlashing;


		if( bf->vibration > maxVibration )	maxVibration = bf->vibration;

		frame_no = bf->fno;
	}

	if( maxDelay > 0 ){
		head |= BFRAME_DELAY;
		bitDelay = coding_ilog2( maxDelay );
	}

	if( maxFlashing > 0 ){
		head |= BFRAME_FALSHING;
		bitFlashing = coding_ilog2( maxFlashing );
	}

	if( maxLedFlashing > 0 ){
		head |= BFRAME_LED_FALSHING;
		bitLedFlashing = coding_ilog2( maxLedFlashing );
	}

	if( maxVibration > 0 ){
		head |= BFRAME_VIBRATION;
		bitVibration = coding_ilog2( maxVibration );
	}


	bst_put_int( bst, head, 8 );	// head

	bitFno = coding_ilog2( frame_no );
	bst_put_int( bst, bitFno, 4 );

	if( head & BFRAME_DELAY )
		bst_put_int( bst, bitDelay, 4 );

	if( head & BFRAME_FALSHING )
		bst_put_int( bst, bitFlashing, 4 );

	if( head & BFRAME_LED_FALSHING )
		bst_put_int( bst, bitLedFlashing, 4 );

	if( head & BFRAME_VIBRATION )
		bst_put_int( bst, bitVibration, 4 );




	for( bf = bfL->bf, no = 0 ; bf != NULL ; bf = bf->next, no++ );
	bst_put_int( bst, no, bitFno );

	for( bf = bfL->bf ; bf != NULL ; bf = bf->next ){
        bst_put_int( bst, bf->fno, bitFno );

		if( head & BFRAME_PRIORITY ){
			bst_put_int( bst, bf->priority, 2 );

			if( head & BFRAME_DELAY )
				bst_put_int( bst, bf->delay, 8 );
		}

		if( head & BFRAME_FALSHING )
			bst_put_int( bst, bf->flashing, bitFlashing );

		if( head & BFRAME_LED_FALSHING )
			bst_put_int( bst, bf->ledFlashing, bitLedFlashing );

		if( head & BFRAME_VIBRATION )
			bst_put_int( bst, bf->vibration, bitVibration );
	}
}
