/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***    BframeTool.c    ***
 **************************/
#include	"Uigp/igp.h"



#include	"BframeType.h"



bframe_type * bframe_alloc()
{
bframe_type	*bf;

    bf = (bframe_type *)malloc( sizeof(bframe_type) );

	bf->fno = -1;
	bf->next = NULL;

	bf->delay = 0;
	bf->priority = BFRAME_PRIORITY_NONE;
	

	return( bf );
}



void bframe_destroy( bframe_type *bf )
{
	if( bf == NULL )	
		return;

	free( bf );
}





bframeL_type * bframeL_alloc()
{
bframeL_type	*bfL;

    bfL = (bframeL_type *)malloc( sizeof(bframeL_type) );

	bfL->bf = NULL;
	

	return( bfL );
}



void
bframeL_destroy( bframeL_type *bfL )
{
bframe_type	*bf;

	while( (bf = bfL->bf) != NULL ){
		bfL->bf = bfL->bf->next;
		free( bf );
	}

	free( bfL );
}


int	
bframeL_set_bframe( bframeL_type *bfL, int frame, bframe_type *bf )
{
	return bframeL_set( bfL, frame, bf->priority, bf->delay,
				bf->flashing, bf->ledFlashing, bf->vibration );
}

int	
bframeL_set( bframeL_type *bfL, int frame, int priority, int delay,
				int flashing, int ledFlashing, int vibration )
{
bframe_type	*bf,	*pbf;


	pbf = bframeL_get_previous( bfL, frame );

	if( pbf != NULL && pbf->fno == frame ){
		pbf->priority = priority;
		pbf->delay = delay;
		pbf->flashing = flashing;
		pbf->ledFlashing = ledFlashing;
		pbf->vibration = vibration;
		return( 1 );
	}


	bf = bframe_alloc();
	bf->fno = frame;
	bf->priority = priority;
	bf->delay = delay;



	if( pbf == NULL ){
		bf->next = bfL->bf;
		bfL->bf = bf;
		return(1);
	}



	bf->next = pbf->next;
	pbf->next = bf;

	return( 1 );
}



bframe_type *	
bframeL_get( bframeL_type *bfL, int frame )
{
bframe_type	*bf;

	bf = bframeL_get_previous( bfL, frame );

	if( bf == NULL || bf->fno != frame )	return( NULL );

	return( bf );
}


void	
bframeL_delete( bframeL_type *bfL, int frame )
{
bframe_type	*bf,	*pbf;

	if( bfL == NULL )	return;

	if( bfL->bf->fno == frame ){
		bf = bfL->bf;
		bfL->bf = bf->next;
		free( bf );
		return;
	}

	pbf = bframeL_get_previous( bfL, frame-1 );

	if( (bf = pbf->next) == NULL )	return;
	
	if( bf->fno != frame )	return;

	pbf->next = bf->next;

	free( bf );
}



void	
bframeL_clip( bframeL_type *bfL, int maxFrame )
{
bframe_type	*bf,	*pbf,	*bf0;

	if( bfL == NULL )	return;

	pbf = bframeL_get_previous( bfL, maxFrame-1 );

	if( pbf == NULL ){
		bf0 = bfL->bf;
		bfL->bf = NULL;
	}
	else	{
		bf0 = pbf->next;
		pbf->next = NULL;
	}


	while( (bf = bf0) != NULL ){
		bf0 = bf0->next;
		free( bf );
	}
}


bframe_type *
bframeL_get_previous( bframeL_type *bfL, int frame )
{
bframe_type * bf;

	
	if( bfL == NULL || bfL->bf == NULL || bfL->bf->fno > frame )
		return( NULL );

	for( bf = bfL->bf ; bf->next != NULL && bf->next->fno <= frame ;
										bf = bf->next );
	return( bf );
}


bframeL_type *	
bframeL_copy_segment( bframeL_type *bfL, int minFrame, int maxFrame, int Frame0 )
{
bframeL_type	*tbfL;
bframe_type	*bf,	*pbf,	*bf0;

	if( bfL == NULL || bfL->bf == NULL )	return( NULL );

	bf0 = bframeL_get_previous( bfL, minFrame );


	if( bf0 == NULL ){
		if( bfL->bf->fno > maxFrame )	return( NULL ); 
		bf0 = bfL->bf;
	}


	if( bf0->fno < minFrame ){
		bf0 = bf0->next;
		if( bf0 == NULL || bf0->fno > maxFrame )	return( NULL );
	}


	tbfL = bframeL_alloc();
	pbf = NULL;
	for(  ; bf0 != NULL && bf0->fno < maxFrame ; bf0 = bf0->next ){
		bf = bframe_alloc();
		
		*bf = *bf0;

		bf->fno += ( Frame0 - minFrame );
		bf->next = NULL;
		if( tbfL->bf == NULL )
			tbfL->bf = bf;
		else	pbf->next = bf;

		pbf = bf;
	}

	return( tbfL );
}



void	
bframeL_fno_add( bframeL_type *bfL, int dfno )
{
bframe_type	*bf;

	if( bfL->bf->fno + dfno < 0 )
		dfno = -bfL->bf->fno;

	for(  bf = bfL->bf ; bf != NULL ; bf = bf->next )
		bf->fno += dfno;
}


int
bframeL_append( bframeL_type *bfL, bframeL_type *abfL, int startFrame )
{
bframe_type	*bf;

	for( bf = bfL->bf ; bf->next != NULL ; bf = bf->next );


	if( startFrame == -1 && startFrame < bf->fno )
		startFrame = bf->fno + 1;


	bframeL_fno_add( abfL, startFrame );


	bf->next = abfL->bf;

	abfL->bf = NULL;
	free( abfL );

	return( 1 );
}



void 
bframeL_rescale_frame( bframeL_type *bfL, int cframe_no, int frame_no )
{
bframe_type	*bf;

	if( bfL == NULL )	return;
 
	for( bf = bfL->bf ; bf != NULL ; bf = bf->next ){
		bf->fno = ( bf->fno * (frame_no-1) ) / (cframe_no-1 );
 
	}
} 