/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/******************************
 ***	InteractionTool.c   ***
 ******************************/
#include	<string.h>
#include	"Uigp/igp.h"

#include	"InteractionType.h"



interaction_type *
interaction_alloc()
{
	interaction_type *in = (interaction_type *)malloc( sizeof(interaction_type) );

	in->trigger = -1;
	in->fno = -1;

//	in->soundId = -1;
	in->sound = NULL;

	in->ocpL = NULL;

	in->next = NULL;

	return in;
}



void
interaction_destroy(interaction_type *in )
{
	if( in->ocpL != NULL )
		ocpL_destroy( in->ocpL );

	free( in );
}




interaction_type *
interaction_create( int trigger, int frame, int type, int soundId )
{
interaction_type *in;

	in = interaction_alloc();
	in->trigger = trigger;
	in->fno = frame;
	in->type = type;
//	in->soundId = soundId;
	in->sound = NULL;

	return( in );
}



interaction_type *
interaction_copy( interaction_type *sin )
{
interaction_type *in;

	in = interaction_alloc();

	in->trigger = sin->trigger;
	in->fno = sin->fno;
	in->type = sin->type;

	strcpy( in->url, sin->url );

//	in->soundId = sin->soundId;

	return( in );
}


interactionL_type * 
interactionL_alloc()
{
interactionL_type	*inL;

    inL = (interactionL_type *)malloc( sizeof(interactionL_type) );

	inL->in = NULL;
	

	return( inL );
}



void
interactionL_destroy( interactionL_type *inL )
{
interaction_type	*in;

	while( (in = inL->in) != NULL ){
		inL->in = inL->in->next;
		free( in );
	}

	free( inL );
}


void 
interactionL_rescale_frame( interactionL_type *inL, int cframe_no, int frame_no )
{
interaction_type	*in;

	if( inL == NULL )	return;
 
	for( in = inL->in ; in != NULL ; in = in->next ){
		in->fno = ( in->fno * (frame_no-1) ) / (cframe_no-1 );
 
	}
} 


void 
interactionL_add_sound( interactionL_type *inL,
					   int trigger, int frame, sound_type *sound )
{

interaction_type *in;

	in = interaction_alloc();
	in->trigger = trigger;
	in->fno = frame;
	in->type = INTERACTION_SOUND;


	strcpy( in->url, sound->name );

	in->sound = sound;


	interactionL_add( inL, in );
}


void 
interactionL_add( interactionL_type *inL, interaction_type *in )
{

interaction_type *pin;


	if( in->trigger != TRIGGER_FRAME || inL->in == NULL || in->fno <= inL->in->fno ){
		in->next = inL->in;
		inL->in = in;
		return;
	}

	for( pin = inL->in ; pin->next != NULL && pin->next->fno < in->fno ; pin = pin->next );

	in->next = pin->next;
	pin->next = in;
}



interaction_type * 
interactionL_detach_frame_interaction( interactionL_type *inL, int frame )
{
interaction_type *in,	*pin;

	if( inL == NULL )	return( NULL );


	for( in = inL->in, pin = NULL ; in != NULL ; pin = in, in = in->next ){
		if( in->fno > frame )	return( NULL );

		if( in->fno < frame )	continue;

		if( pin == NULL )
			inL->in = in->next;
		else	pin->next = in->next;

		return( in );
	}

	return( NULL );
}


int
interactionL_detach( interactionL_type *inL, interaction_type *din )
{
interaction_type *in,	*pin;

	if( inL == NULL )	return( -1 );


	for( in = inL->in, pin = NULL ; in != NULL ; pin = in, in = in->next ){

		if( in != din)	continue;

		if( pin == NULL )
			inL->in = in->next;
		else	pin->next = in->next;

		return( 1 );
	}

	return( -1 );
}



interaction_type * 
interactionL_get( interactionL_type *inL, int trigger, int frame, int type )
{
interaction_type *in;

	if( inL == NULL )	return( NULL );

	for( in = inL->in ; in != NULL ; in = in->next ){
		if( in->trigger != trigger  )	continue;
		if( type != 0 && in->type != type )	continue;

		if( in->trigger == TRIGGER_FRAME ){
			if( in->fno >  frame )	return( NULL );
			if( in->fno < frame )	continue;
		}

		return( in );
	}

	return( NULL );
}



interaction_type * 
interactionL_get_TRIGGER_FRAME( interactionL_type *inL, int type, int minFrame, int maxFrame )
{
interaction_type *in,	*sin;

	if( inL == NULL )	return( NULL );

	sin = NULL;
	for( in = inL->in ; in != NULL ; in = in->next ){
		if( in->type != type )	continue;

		if( in->trigger != TRIGGER_FRAME )	continue;

		if( in->fno < minFrame )	continue;
		if( in->fno >  maxFrame )	return( sin );
		
		sin = in;
	}

	return( sin );
}



int
interactionL_delete( interactionL_type *inL, int trigger, int frame, int type )
{
interaction_type *in,	*pin;

	if( inL == NULL )	return( -1 );

	for( in = inL->in, pin = NULL ; in != NULL ; pin = in, in = in->next ){
		if( in->trigger != trigger || in->type != type )	continue;

		if( in->trigger == TRIGGER_FRAME )
			if( in->fno !=  frame )	continue;

		if( pin == NULL )
			inL->in = in->next;
		else	pin->next = in->next;

		interaction_destroy( in );
		return( 1 );
	}

	return( -1 );
}


int
interactionL_delete_in( interactionL_type *inL, interaction_type *sin )
{
interaction_type	*in;

	if( inL == NULL )	return( -1 );

	if( inL->in == sin ){
		inL->in = sin->next;
		interaction_destroy( sin );
		return( 1 );
	}

	for( in = inL->in ; in != NULL  ; in = in->next )
		if( in->next == sin ){
			in->next = sin->next;
			interaction_destroy( sin );
			return( 1 );
		}


	return( -1 );
}

void	
interactionL_clip( interactionL_type *inL, int maxFrame )
{
interaction_type	*in,	**pin;


	if( inL == NULL )	return;


	for( pin = &inL->in ; *pin != NULL ;  ){
		in = *pin;
		if( in->fno < maxFrame ){
			pin = &in->next;
			continue;
		}

		*pin = in->next;

		interaction_destroy( in );
	}
}


interactionL_type *	
interactionL_copy_segment( interactionL_type *sinL, int minFrame, int maxFrame,	int Frame0 )
{
interactionL_type	*inL;
interaction_type	*sin,	*in,	*pin;


	if( sinL == NULL )	return(NULL);

	inL = interactionL_alloc();
	pin = NULL;

	for( sin = sinL->in ; sin != NULL ; sin = sin->next ){

		if( sin->trigger == TRIGGER_FRAME &&
						sin->fno < minFrame || sin->fno > maxFrame )
				continue;


		in = interaction_copy( sin );

		if( in->trigger == TRIGGER_FRAME )
			in->fno += ( Frame0 - minFrame);

		in->next = NULL;

		if( pin == NULL )
			inL->in = in;
		else	pin->next = in;
	}


	if( inL->in == NULL ){
		interactionL_destroy(inL);
		inL = NULL;
	}

	return( inL );
}

void	
interactionL_fno_add( interactionL_type *inL, int dfno )
{
interaction_type	*in;

	if( inL->in->fno + dfno < 0 )
		dfno = -inL->in->fno;

	for(  in = inL->in ; in != NULL ; in = in->next ){
		if( in->trigger != TRIGGER_FRAME )	continue;

		in->fno += dfno;
	}
}



interactionL_type *	
interactionL_append( interactionL_type *inL, interactionL_type *sinL, int frame0 )
{
interaction_type	*sin,	*in;


	if( sinL == NULL )	return(NULL);

	if( inL == NULL )
		inL = interactionL_alloc();


	for( sin = sinL->in ; sin != NULL ; sin = sin->next ){

		in = interaction_copy( sin );

		if( in->trigger == TRIGGER_FRAME )
			in->fno += frame0;


		interactionL_add( inL, in );

	}



	return( inL );
}



void 
interactionL_add_URL( interactionL_type *inL,
					   int trigger, int type, char *url )
{

interaction_type *in;

	in = interaction_alloc();
	in->trigger = trigger;
	in->type = type; // INTERACTION_REPLACE


	strncpy( in->url, url, URL_LEN );

	interactionL_add( inL, in );
}


void 
interactionL_add_ANIMATION( interactionL_type *inL,
					   int trigger, ocpL_type *ocpL )
{

interaction_type *in;

	in = interaction_alloc();
	in->trigger = trigger;	// 
	in->type = INTERACTION_ANIMATATION;


	in->ocpL = ocpL;

	interactionL_add( inL, in );
}