/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*****************************
 ***	InteractionType.h  ***
 *****************************/
#ifndef         _INTERACTION_TYPE_
#define         _INTERACTION_TYPE_ 

#include	"Uigp/igp.h"
#include	"Ubst/BstType.h"

#include	"Sound/SoundType.h"
#include	"VimageType/OcpType.h"

#define		URL_LEN		512


// Interaction_type.trigger   4 bit
#define		TRIGGER_CLICK	0x01
#define		TRIGGER_FRAME	0x02
#define		TRIGGER_OVER	0x03


// Interaction_type.type				4 bit
#define		INTERACTION_SOUND			0x01
#define		INTERACTION_REPLACE			0x02
#define		INTERACTION_MERGE			0x03
#define		INTERACTION_ANIMATATION		0x04
#define		INTERACTION_GOTO			0x05
#define		INTERACTION_LINK			0x06



typedef struct interaction_type {
	int		trigger;

	int		fno;			// if TRIGGER_FRAME, frame no

	int		type;			// of interaction	sound, url-replace, url-merage, link

	char	url[URL_LEN];

//	int		soundId;		// if INTERACTION_SOUND,  the sound index
	sound_type	*sound;		// for INTERACTION_SOUND 

	ocpL_type	*ocpL;		// for roleover

	struct interaction_type	*next;

}	interaction_type;


typedef struct interactionL_type {

	interaction_type	*in;

}	interactionL_type;



interaction_type *	interaction_alloc();

void		interaction_destroy( interaction_type *ita );


interaction_type *	interaction_create( int trigger, int frame, int type, int soundId );

interaction_type *	interaction_copy( interaction_type *sin );



interactionL_type * interactionL_alloc();

void	interactionL_destroy( interactionL_type *itL );

void	interactionL_rescale_frame( interactionL_type *itL, int cframe_no, int frame_no );


void	interactionL_add_sound( interactionL_type *inL,
					   int trigger, int frame, sound_type *sound );

void	interactionL_add( interactionL_type *inL, interaction_type *in );


int	interactionL_detach( interactionL_type *inL, interaction_type *din );

interaction_type *	interactionL_get( interactionL_type *inL, int trigger, int frame, int type );

interaction_type *	interactionL_get_TRIGGER_FRAME( interactionL_type *inL, int type, int minFrame, int maxFrame );

int	interactionL_delete( interactionL_type *inL, int trigger, int frame, int type );

int	interactionL_delete_in( interactionL_type *inL, interaction_type *sin );



void		interactionL_clip( interactionL_type *inL, int maxFrame );

interactionL_type *		interactionL_copy_segment( interactionL_type *sinL,
										int minFrame, int maxFrame, int Frame0 );

void		interactionL_fno_add( interactionL_type *inL, int dfno );


interactionL_type *	interactionL_append( interactionL_type *inL,
								interactionL_type *sinL, int frame0 );


void	interactionL_add_URL( interactionL_type *inL,
					   int trigger, int type, char *url );

void	interactionL_add_ANIMATION( interactionL_type *inL,
					   int trigger, ocpL_type *ocp );



	/* InteractionEncoding.c */
void	interactionL_encoding( interactionL_type *inL, skl_type *skl, bst_type *bst );

	/* InteractionDecoding.c */
interactionL_type *	interactionL_decoding( skl_type *skl, bst_type *bst );
interactionL_type *	interactionL_decoding0005( bst_type *bst );

#endif
