/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************
 ***	RenderGeo.c   ***
 ************************/
#include	<stdio.h>   
#include	<math.h>

#include	"Render.h"


static void	render_hunch( hunch_type *h,
			view_type *base, view_type *vw, gp_array *agp, box2d *box );

static void	render_bgp( bgp_type *bgp,
			view_type *base, view_type *vw, gp_array *agp, box2d *box );

/*
static void	render_jnt( jnt_type *jnt,
			view_type *base, view_type *vw, gp_array *agp );
*/

static void	render_link( link_type *link,
			view_type *base, view_type *vw, gp_array *agp, box2d *box );



#define         DSIZE   15



void
render_agp( view_type *base, gp_array *agp, float depth0, float depth1,
                                view_type *vw, gp_array *nagp ) 
{
view_type	nbase,	nvw;
box2d	box;
gp_type	*gp;
bgp_type	*bgp;
hunch_type	*h;
link_type	*link;
int	i,	j;
int	i0,	j0,	i1,	j1;

int	Sbr;
float	dt;
box2d	Box;


	view_per_normal_base( base, &nbase );
	view_per_normal( base, vw, &nvw );
 
	view_actoal_depth_pixel_box( &nbase, depth0, depth1, &nvw, &box );

	if( (i0 = box.x0/agp->bgp - DSIZE) < 0 )	i0 = 0;
	if( (i1 = box.x1/agp->bgp + DSIZE + 1) >= agp->Ni )	i1 = agp->Ni-1;

	if( (j0 = box.y0/agp->bgp - DSIZE) < 0 )	j0 = 0;
	if( (j1 = box.y1/agp->bgp + DSIZE + 1) >= agp->Nj )	j1 = agp->Nj-1;




	Box.x0 = -nagp->bgp + 0.001;
	Box.x1 = nagp->rowR + nagp->bgp - 0.001;

	Box.y0 = -nagp->bgp + 0.001;
	Box.y1 = nagp->columnR + nagp->bgp - 0.001;




	dt = nbase.Ua_inv / nvw.Ua_inv;




	for( i = i0 ; i <= i1 ; i++ )
	    for( j = j0 ; j <= j1 ; j++ ){
		gp = &agp->gp[i][j];


		for( h = gp->hunch ; h != NULL ; h = h->rnh )
			render_hunch( h, &nbase, &nvw, nagp, &Box);


		if( ( bgp = gp->bgp) != NULL ){
		    render_bgp( bgp, &nbase, &nvw, nagp, &Box);
		    Sbr = (bgp->Sbr == 2 && dt*bgp->rdepth<= bgp->depth);

		    for( bgp = bgp->rp ;
				bgp != NULL && bgp->Sbr >= Sbr ; bgp = bgp->rp)
			render_bgp( bgp, &nbase, &nvw, nagp, &Box);
		}




		for( link = gp->link ; link != NULL ; link = link->rnl ){
			if( link->type == NF_N3 )	continue;
			render_link( link, &nbase, &nvw, nagp, &Box );
		}

	    }
}



void
render_point( view_type *base, vec2d *p, float depth,
				view_type *vw, vec2d *np, float *ndepth )
{
vec2d	v,	nv;
vec3d	v3;
float	t;

	v.x = base->Ua_inv*(p->y - base->Ub);
	v.y = base->Va_inv*(p->x - base->Vb);


	v3.x = depth * v.x;
	v3.y = depth * v.y;
	v3.z = 1 + depth * -base->dis;


	v3.x -= vw->cp.x;
	v3.y -= vw->cp.y;
	v3.z -= vw->cp.z;


	*ndepth = t = VEC3D_INNER( vw->N, v3 );
	t = 1.0 / t;

	nv.x = t* VEC3D_INNER( vw->U, v3 );
	nv.y = t* VEC3D_INNER( vw->V, v3 );

	np->x = vw->Va *nv.y + vw->Vb;
	np->y = vw->Ua *nv.x + vw->Ub;
}


void
render_point_inv( view_type *vw, vec2d *np, float depth,
						view_type *base, vec2d *p )
{
vec3d	v3;
vec2d	v,	nv;
float	a1,	a2,	a3;
float	b1,	b2,	b3;
float	det;


	nv.x = vw->Ua_inv*(np->y - vw->Ub);
	nv.y = vw->Va_inv*(np->x - vw->Vb);


	if( base->type == VIEW_PARALLEL )
		v3.z = -depth;
	else	v3.z = -base->dis;

	v3.z -= vw->cp.z;

	a1 = nv.x*vw->N.x - vw->dis*vw->U.x;
	a2 = nv.x*vw->N.y - vw->dis*vw->U.y;
	a3 = -(nv.x*vw->N.z - vw->dis*vw->U.z)* v3.z;

	b1 = nv.y*vw->N.x - vw->dis*vw->V.x;
	b2 = nv.y*vw->N.y - vw->dis*vw->V.y;
	b3 = -(nv.y*vw->N.z - vw->dis*vw->V.z)* v3.z;

	det = a1*b2 - b1*a2;
	v3.x = (a3*b2 - b3*a2)/det;
	v3.y = (a1*b3 - b1*a3)/det;


	v.x = v3.x + vw->cp.x;
	v.y = v3.y + vw->cp.y;


	p->x = base->Va *v.y + base->Vb;
	p->y = base->Ua *v.x + base->Ub;
}





static void
render_hunch( hunch_type *h, view_type *base, view_type *vw, gp_array *agp, box2d *box )
{
gp_type	*gp;
int	i,	j;

	render_point( base, &h->rctr, h->rdepth, vw, &h->ctr, &h->depth );

	if( !BOX2D_IS_IN_O( *box, h->ctr.x, h->ctr.y ) )    return;


	i = (h->ctr.x+agp->bgp) * agp->inv_bgp;
	j = (h->ctr.y+agp->bgp) * agp->inv_bgp;


	gp = &agp->gp[i][j];

	h->ctr.x -= gp->ctr.x;
	h->ctr.y -= gp->ctr.y;
	h->nh = gp->hunch;
	gp->hunch = h;
}


static void
render_bgp( bgp_type *bgp, view_type *base, view_type *vw, gp_array *agp, box2d *box )
{
gp_type	*gp;
int	i,	j;

	render_point( base, &bgp->rctr, bgp->rdepth,
					vw, &bgp->ctr, &bgp->depth );

	if( !BOX2D_IS_IN_O( *box, bgp->ctr.x, bgp->ctr.y ) )    return;


	i = (bgp->ctr.x+agp->bgp) * agp->inv_bgp;
	j = (bgp->ctr.y+agp->bgp) * agp->inv_bgp;


	gp = &agp->gp[i][j];

	bgp->ctr.x -= gp->ctr.x;
	bgp->ctr.y -= gp->ctr.y;

	bgp->p = gp->bgp;
	gp->bgp = bgp;
}


/*
static void
render_jnt( jnt_type *jnt, view_type *base, view_type *vw, gp_array *agp )
{
gp_type	*gp;
int	i,	j;

	render_point( base, &jnt->rctr, jnt->rdepth,
				vw, &jnt->gctr, &jnt->depth );




	if( !BOX2D_IS_IN_O( Box, jnt->gctr.x, jnt->gctr.y ) )    return;

	i = (jnt->gctr.x+agp->bgp) * agp->inv_bgp;
	j = (jnt->gctr.y+agp->bgp) * agp->inv_bgp;



	gp = &agp->gp[i][j];

	jnt->ctr.x = jnt->gctr.x - gp->ctr.x;
	jnt->ctr.y = jnt->gctr.y - gp->ctr.y;

	jnt->nj = gp->jnt;
	gp->jnt = jnt;
}
*/






static void
render_link( link_type *link, view_type *base, view_type *vw, gp_array *agp, box2d *box )
{
gp_type	*gp;
profile_type	pr;
vec2d	pv0,	pv1,	v0,	v1;
float	t;
int	i,	j;

	pv0 = link->rctr;

	pv1.x = pv0.x + link->rv.x;
	pv1.y = pv0.y + link->rv.y;


	pr = link->pr[0];
	render_point( base, &pv0, pr.p->rdepth, vw, &v0, &pr.p->depth );




	pr = link->pr[1];
	render_point( base, &pv1, pr.p->rdepth, vw, &v1, &pr.p->depth );



	link->v.x = v1.x - v0.x;
	link->v.y = v1.y - v0.y;
	link->ctr = v0;

	if( !BOX2D_IS_IN_O( *box, v0.x, v0.y ) &&
				!BOX2D_IS_IN_O( *box, v1.x, v1.y ) )
		return;





	/* link_set_aux( link ); */
 
#ifdef PDA
	link->len = hypotA( link->v.x, link->v.y );
#else
	link->len = hypot( link->v.x, link->v.y );
#endif
	t = 1.0 /link->len;
 
	link->u.x = link->v.y * t;
	link->u.y = -link->v.x * t;
 
 
	link->a = link->ra * link->len;
	link->c_prb = -4.0 *link->a *t *t; 




		
	i = (v0.x+agp->bgp) * agp->inv_bgp;
	j = (v0.y+agp->bgp) * agp->inv_bgp;

	i = PUSH_TO_RANGE( i, 0, agp->Ni-1 );
	j = PUSH_TO_RANGE( j, 0, agp->Nj-1 );

	gp = &agp->gp[i][j];


	link->ctr.x -= gp->ctr.x;
	link->ctr.y -= gp->ctr.y;

	link->nl = gp->link;
	gp->link = link;
}
