/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/

/*************************
 ***	RenderPlW.c    ***
 *************************/
#include	<stdio.h>   
#include	<math.h>

#include	"Render.h"

#include	"SklType/LnkType.h"
#include	"SklType/PlType.h"


static void	render_hunch( hunch_type *h,
			view_type *base, view_type *vw, gp_array *agp, box2d *box );

static void	render_bgp( bgp_type *bgp,
			view_type *base, view_type *vw, gp_array *agp, box2d *box );



static void	render_link( link_type *link,
			view_type *base, view_type *vw, gp_array *agp, box2d *box );


/*
static void	pl_point( pl_type *pl, vec2d *p );
*/
static void	plw_point( pl_type *pl, vec2d *p );




void
render_agp_plw( view_type *base, gp_array *agp, float depth0, float depth1,
				view_type *vw, gp_array *nagp )
{
view_type	nbase,	nvw; 
gp_type	*gp;
bgp_type	*bgp;
hunch_type	*h;
link_type	*link;
int	i,	j;
int	Sbr;
box2d	Box;

float	dt;


	view_per_normal_base( base, &nbase );
	view_per_normal( base, vw, &nvw );



	Box.x0 = -nagp->bgp +0.001;
	Box.x1 = nagp->rowR + nagp->bgp - 0.001;

	Box.y0 = -nagp->bgp +0.001;
	Box.y1 = nagp->columnR + nagp->bgp - 0.001;





	dt = nbase.Ua_inv / nvw.Ua_inv;



	for( i = 0 ; i < agp->Ni ; i++ ){
	    gp = &agp->gp[i][0];
	    for( j = 0 ; j <  agp->Nj ; j++, gp++ ){


		for( h = gp->hunch ; h != NULL ; h = h->rnh )
			render_hunch( h, &nbase, &nvw, nagp, &Box);


		if( ( bgp = gp->bgp) != NULL ){ 
		    render_bgp( bgp, &nbase, &nvw, nagp, &Box);
		    Sbr = (bgp->Sbr == 2 && dt*bgp->rdepth <= bgp->depth); 
		    for( bgp = bgp->rp ;
				bgp != NULL && bgp->Sbr >= Sbr ; bgp = bgp->rp)
			render_bgp( bgp, &nbase, &nvw, nagp, &Box);
		} 





		for( link = gp->link ; link != NULL ; link = link->rnl ){
			if( link->type == NF_N3 )	continue;
			render_link( link, &nbase, &nvw, nagp, &Box );
		}

	    }
	}
}



void
plw_point( pl_type *pl, vec2d *p )
{
vec2d	p0,	v;
float	w;

	pl_point( pl, &v );
	p0.x = pl->w * v.x;
	p0.y = pl->w * v.y;
	w = pl->w;

	if( pl->p != NULL ){
		pl_point( pl->p, &v );
		p0.x += pl->p->w * v.x;
		p0.y += pl->p->w * v.y;
		w += pl->p->w;
	}

	if( w >= 1 ){
		p->x = p0.x / w;
		p->y = p0.y / w;
		return;
	}

	p->x = p0.x + ( 1 - w )* p->x;
	p->y = p0.y + ( 1 - w )* p->y;
}

	
/*
static void
pl_point( pl_type *pl, vec2d *p )
{
float   t,	u,	h,	len;
lnk_type	*l;

	l = pl->l;

	len = l->len  - l->t0 * pl->u + l->t1 * pl->u;

	u = pl->u;
	t = pl->t * len + u * l->t0;


	if( t < 0 || t > l->len ){
		p->x = l->ctr.x  +  t * -l->u.y  +  u * l->u.x;
		p->y = l->ctr.y  +  t *  l->u.x  +  u * l->u.y;

	}


	h = l->c_prb * t * (t - l->len) + u;

	p->x = l->ctr.x  +  t * -l->u.y  +  h * l->u.x;
	p->y = l->ctr.y  +  t *  l->u.x  +  h * l->u.y;
}
*/


static void
render_hunch( hunch_type *h, view_type *base, view_type *vw, gp_array *agp, box2d *box )
{
vec2d	v;
gp_type	*gp;
int	i,	j;

	v = h->rctr;
	if( h->pl != NULL )	plw_point( h->pl, &v );

	render_point( base, &v, h->rdepth, vw, &h->ctr, &h->depth );

	if( !BOX2D_IS_IN_O( *box, h->ctr.x, h->ctr.y ) )    return;


	i = (h->ctr.x+agp->bgp) * agp->inv_bgp;
	j = (h->ctr.y+agp->bgp) * agp->inv_bgp;


	gp = &agp->gp[i][j];

	h->ctr.x -= gp->ctr.x;
	h->ctr.y -= gp->ctr.y;
	h->nh = gp->hunch;
	gp->hunch = h;
}


static void
render_bgp( bgp_type *bgp, view_type *base, view_type *vw, gp_array *agp, box2d *box )
{
vec2d	v;
gp_type	*gp;
int	i,	j;

	v = bgp->rctr;
	if( bgp->pl != NULL )	plw_point( bgp->pl, &v );

	render_point( base, &v, bgp->rdepth, vw, &bgp->ctr, &bgp->depth );

	if( !BOX2D_IS_IN_O( *box, bgp->ctr.x, bgp->ctr.y ) )    return;


	i = (bgp->ctr.x+agp->bgp) * agp->inv_bgp;
	j = (bgp->ctr.y+agp->bgp) * agp->inv_bgp;


	gp = &agp->gp[i][j];

	bgp->ctr.x -= gp->ctr.x;
	bgp->ctr.y -= gp->ctr.y;

	bgp->p = gp->bgp;
	gp->bgp = bgp;
}



static void
render_link( link_type *link,
			view_type *base, view_type *vw, gp_array *agp, box2d *box )
{
gp_type	*gp;
profile_type	pr;
vec2d	pv0,	pv1,	v0,	v1;
float	t;
int	i,	j;

	pv0 = link->rctr;

	pv1.x = pv0.x + link->rv.x;
	pv1.y = pv0.y + link->rv.y;


	if( link->pl[0] != NULL )
		plw_point( link->pl[0], &pv0 );
	pr = link->pr[0];
	render_point( base, &pv0, pr.p->rdepth, vw, &v0, &pr.p->depth );




	if( link->pl[1] != NULL )
		plw_point( link->pl[1], &pv1 );
	pr = link->pr[1];
	render_point( base, &pv1, pr.p->rdepth, vw, &v1, &pr.p->depth );



	if( !BOX2D_IS_IN_O( *box, v0.x, v0.y ) &&
				!BOX2D_IS_IN_O( *box, v1.x, v1.y ) )
		return;


	link->v.x = v1.x - v0.x;
	link->v.y = v1.y - v0.y;
	link->ctr = v0;




        /* link_set_aux( link ); */
 
#ifdef PDA
        link->len = hypotA( link->v.x, link->v.y );
#else
        link->len = hypot( link->v.x, link->v.y );
#endif
        t = 1.0 /link->len;
 
        link->u.x = link->v.y * t;
        link->u.y = -link->v.x * t;
 
 
        link->a = link->ra * link->len;
        link->c_prb = -4.0 *link->a *t *t; 



		
	i = (v0.x+agp->bgp) * agp->inv_bgp;
	j = (v0.y+agp->bgp) * agp->inv_bgp;

	i = PUSH_TO_RANGE( i, 0, agp->Ni-1 );
	j = PUSH_TO_RANGE( j, 0, agp->Nj-1 );

	gp = &agp->gp[i][j];


	link->ctr.x -= gp->ctr.x;
	link->ctr.y -= gp->ctr.y;

	link->nl = gp->link;
	gp->link = link;
}
