/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include        <stdio.h>

#include        "Uigp/igp.h"

#include        "SklType/JsklType.h"


jskl_type *
jskl_alloc()
{
jskl_type        *jnt;

	jnt = (jskl_type *)malloc( sizeof(jskl_type) );

	jnt->bno = 0;

	jnt->br = -1;
	jnt->pl = NULL;

	jnt->nj = NULL;

	return( jnt );
}




void
jskl_free( jskl_type *jnt )
{
int	i;

	for( i = 0 ; i < jnt->bno ; i++ )
		if( jnt->b[i].direct == F_BEGIN )
			lnk_free( jnt->b[i].link );

	jnt->bno = 0;

	if( jnt->pl != NULL ){
		free( jnt->pl );
		jnt->pl = NULL;
	}

	free( jnt );
}


void
jskl_untied_brn( jskl_type *jnt, int bno )
{
lnk_type	*link;
int	i,	direct;

	link = jnt->b[bno].link;
	direct = jnt->b[bno].direct;
	link->p[direct].bno = -1;
	link->p[direct].p.l = NULL;

	jnt->bno--;
	for( i = bno ; i < jnt->bno ; i++ ){
		jnt->b[i] = jnt->b[i+1];
		jnt->b[i].link->p[ jnt->b[i].direct ].bno = i;
	}
}


int
jskl_connet_link( jskl_type *jnt, int bno, lnk_type *link, int direct )
{
bskl_type	*b;

	lnk_update_point( &link->ctr, link, direct, &jnt->ctr );

	if( bno == -1 )
		bno = jnt->bno++;


	b = &jnt->b[bno];
	
	b->direct = direct;
	b->link = link; 

	link->p[direct].p.jnt = jnt;
	link->p[direct].bno = bno;

	return( bno );
}



void
jskl_update_ctr( jskl_type *jnt, vec2d *ctr )
{
lnk_type	*l;
int	i;

	for( i = 0 ; i < jnt->bno ; i++ ){
		l = jnt->b[i].link;

		if( jnt->b[i].direct == F_BEGIN ){

			l->v.x = l->ctr.x + l->v.x - ctr->x;
			l->v.y = l->ctr.y + l->v.y - ctr->y;
			l->ctr = *ctr;

			lnk_set_aux( l );

			continue;
		}

		l->v.x = ctr->x - l->ctr.x;
		l->v.y = ctr->y - l->ctr.y;
		lnk_set_aux( l );
	}

	jnt->ctr = *ctr;
}
