/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	LnkSelect.c	***
 **************************/

#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"SklType/LnkType.h"


float
lnk_select( vec2d *ctr, lnk_type *link, vec2d *p,
				lnk_type **sl, int *direct )
{
float	t,	u;
int	no;

	if( link == NULL )	return( -1 );

	no = lnk_get_status( &link->ctr, link, NULL, p, sl, &u, &t);

	if( ABS(u) > 2.0 )   return( -1 );

	if( no == -1 ){
		*direct = F_BEGIN;
		return( ABS(u) );
	}

	if( no == -2 ){
		*direct = F_END;
		return( ABS(u) );
	}
 
 
	t = PUSH_TO_RANGE( t, 0, (*sl)->len );
 
	*direct = ( t < ABS( t-(*sl)->len) ) ? F_BEGIN : F_END;
 
	return( ABS(u) );
}



float
lnk_select_point( vec2d *ctr, lnk_type *link, vec2d *p,
				lnk_type **sl, int *direct )
{
lnk_type	*l;
float	t,	u;
int	no;

	if( link == NULL )	return( -1 );

	no = lnk_get_status( &link->ctr, link, NULL, p, sl, &u, &t );

	if( ABS(u) > 4.0 )   return( -1 );

	if( no == -1 ){
		*direct = F_BEGIN;
		return( ABS(u) );
	}

	if( no == -2 ){
		*direct = F_END;
		return( ABS(u) );
	}
 
 
	t = PUSH_TO_RANGE( t, 0, (*sl)->len );
 
	*direct = ( t < ABS( t-(*sl)->len) ) ? F_BEGIN : F_END;

	if( *direct == F_BEGIN && t < 1.0 ){
		l = *sl;
		if( (l = LNK_PREV(l) ) != NULL ){
			*sl = l;
			*direct = F_END;
		}
	}
		
 
	return( ABS(u) );
}





int
lnk_get_status( vec2d *vm, lnk_type  *link, lnk_type *elink, vec2d  *v,
		lnk_type **sl, float *u, float *t )
{
lnk_type	*l;
int         i,	ret;
vec2d       cv;
float	cu,	ct,	dt;
float	dis,	tmp;


	*sl = NULL;
	*u = 1000;
	*t = 100;
	ret = -1;

	dis = 100000;

	cv.x = v->x - vm->x;
	cv.y = v->y - vm->y;

	for( l = link, i = 0 ; l != elink ; l = LNK_NEXT(l), i++ ){

		cu = l->u.x * cv.x + l->u.y * cv.y;

		ct = -l->u.y * cv.x + l->u.x * cv.y;

		cv.x -= l->v.x;
		cv.y -= l->v.y;



		if( ct < 0 ){
			tmp = cu*cu + 1.44*ct*ct;

			if( cu > 0 )	cu -= ct;
			else	cu += ct;

			ct = 0;
		}
		else	if( (dt = ct - l->len) > 0 ){

				tmp = cu*cu + 1.44*dt*dt;

				if( cu > 0 )	cu += dt;
				else	cu -= dt;
				ct = l->len;
			}
			else	{
				cu -= l->c_prb*ct *(ct - l->len );
				tmp = cu*cu;
			}


		if( tmp >= dis )	continue;

		dis = tmp;

		*u = cu;
		*t = ct;
		*sl = l;
		ret = i;
	}

	return( ret );
}



int
lnk_array_select( lnk_type  *link[], int no, vec2d  *v, mbr_type mbr,
				lnk_type **sl, float *u, float *t )
{
lnk_type	*l;
int         i,	ret;
vec2d       cv;
float	cu,	ct,	dt;
float	dis,	tmp;


	*sl = NULL;
	*u = 1000;
	*t = 100;
	ret = -1;

	dis = 100000;


	for( i = 0 ; i < no ; i++ ){
		l = link[i];

/*
		if( l->br != br )	continue;
*/
		if( !(l->mbr & mbr) )	continue;


		cv.x = v->x - l->ctr.x;
		cv.y = v->y - l->ctr.y;

		cu = l->u.x * cv.x + l->u.y * cv.y;

		ct = -l->u.y * cv.x + l->u.x * cv.y;



		if( ct < 0 ){
			tmp = cu*cu + 1.44*ct*ct;

			if( cu > 0 ) cu -= ct;
			else	cu += ct;

			ct = 0;
		}
		else	if( (dt = ct - l->len) > 0 ){
				tmp = cu*cu + 1.44*dt*dt;

				if( cu > 0 ) cu += dt;
				else	cu -= dt;

				ct = l->len;
			}
			else	{
				tmp = cu*cu;
				cu -= l->c_prb*ct *(ct - l->len );
			}

		if( l->width > 0  &&  ABS(cu) > l->width )	continue;


/*
                if( ABS( cu ) > ABS(*u) )	continue;
*/
		if( tmp > dis )	continue;
		dis = tmp;

		*u = cu;
		*t = ct;
		*sl = l;
		ret = i;
	}

	return( ret );
}
