/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	LnkTool.c	***
 **************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"SklType/LnkType.h"
#include	"SklType/JsklType.h"


lnk_type *
lnk_make_copy( lnk_type *link )
{
lnk_type       *nlink, *l,     *sl,    *tl;
 
	if( link == NULL )	return( NULL );

	nlink = l = LNK_ALLOC();
	*l = *link;

	while( ( sl = LNK_NEXT(l) ) != NULL ){
		
		tl = LNK_ALLOC();
		*tl = *sl;

		tl->p[F_BEGIN].p.l = l;
		l->p[F_END].p.l = tl;

		l = tl;
	}

	return( nlink );
}


void
lnk_free( lnk_type *link )
{
lnk_type       *l;
        while( (l = link) != NULL ){
                link = LNK_NEXT(link);
                free( l );
        }
}



void
lnk_set_aux( lnk_type *l )
{
	if( l->v.x == 0.0 && l->v.y == 0 ){
		l->len = l->u.x = l->u.y = l->c_prb = l->a = 0.0;
		return;
	}

	l->len = hypot( l->v.x, l->v.y );

	l->u.x = l->v.y/l->len;
	l->u.y = -l->v.x/l->len;

	l->c_prb = -4.0 *l->a/( l->len*l->len);
}


void
lnk_t2xy( vec2d *ctr, lnk_type *link, float t, vec2d *p )
{
float   h;

	h = link->c_prb * t * (t - link->len);

	p->x = ctr->x  +  t * -link->u.y  +  h * link->u.x;
	p->y = ctr->y  +  t *  link->u.x  +  h * link->u.y;
}






void
lnk_update_point( vec2d *ctr, lnk_type *link, int direct, vec2d *p )
{
vec2d   v;
	v = *p;  

	if( direct == F_BEGIN ){
		link->v.x = ctr->x + link->v.x - v.x;   
		link->v.y = ctr->y + link->v.y - v.y;
		*ctr = v; 
	} 
	else    { 
		link->v.x = v.x - ctr->x;
		link->v.y = v.y - ctr->y;
	} 
 
	lnk_set_aux( link );
}


void
lnks_update_point( lnk_type *link, int direct, vec2d *p )
{
lnk_type	*l;

	if( direct == F_END ){
		lnk_update_point( &link->ctr, link, F_END, p );
		return;
	}
	
 
	if( (l = LNK_PREV( link )) != NULL )
		lnk_update_point( &l->ctr, l, F_END, p );
 
	lnk_update_point( &link->ctr, link, F_BEGIN, p );
}




void
lnk_a_point( vec2d *ctr, lnk_type *l, vec2d *p )
{
vec2d   dv;
float   t,      u;

	dv.x = p->x - ctr->x;
	dv.y = p->y - ctr->y;

	u = dv.x * l->u.x + dv.y * l->u.y;
	t = -l->u.y * dv.x + l->u.x * dv.y;
	if( t <= 0 || t >= l->len )     return;

	l->c_prb = u /( t * (t - l->len) );

	l->a = -l->c_prb * ( l->len*l->len)/4.0;

	l->c_prb = -4.0 *l->a/( l->len*l->len);
}


void
lnk_a_add( vec2d *ctr, lnk_type *l, float d )
{
	l->a += d;

	l->c_prb = -4.0 *l->a/( l->len*l->len);
}




void
lnk_conent( lnk_type *link, lnk_type *l )
{
	link->p[F_END].p.l = l;

	l->p[F_BEGIN].p.l = link;

	link->p[F_END].bno = l->p[F_BEGIN].bno = -1;
}





void
lnk_set_gctr( vec2d *ctr, lnk_type *link )
{
lnk_type	*l;
vec2d	v;
 
	for( l = link, v = *ctr ; l != NULL; l = LNK_NEXT(l) ){
 
		l->ctr = v;
 
		v.x += l->v.x;
		v.y += l->v.y;
	}
}




void
lnk_set_bes( lnk_type *link )
{
lnk_type	*l,     *nl;
vec2d   tmp;
float   t;

	link->t0 = 0;


	for( l = link, nl = LNK_NEXT(l) ; nl != NULL ;
					l = nl, nl = LNK_NEXT(nl) ){
		tmp.y = ( l->u.y + nl->u.y );
		tmp.x =  ( l->u.x + nl->u.x );


		t = tmp.x*nl->u.x + tmp.y*nl->u.y;

		if( ABS(t) > 0.001 )	t = 1.0 / t;
		else	t = 1.0 / 0.001;
/*
		t = 1.0 / (tmp.x*nl->u.x + tmp.y*nl->u.y);
*/

		nl->t0 = t*( tmp.x * -nl->u.y + tmp.y * nl->u.x );
		l->t1 = -nl->t0;

	}

	l->t1 = 0;
}



lnk_type *
lnk_inverse( lnk_type *link )
{
lnk_type	*l,	*l1;

	while( (l = LNK_PREV(link)) != NULL )	link = l;

	l = link;

	while(  l != NULL ){
		lnk_1_inverse( l );

		l1 = l;

		l = LNK_PREV(l);
	}

	return( l1 );
}


void
lnk_1_inverse( lnk_type *l )
{

lnk_ptr	p;

	l->ctr.x += l->v.x;
	l->ctr.y += l->v.y;

	l->v.x = - l->v.x;
	l->v.y = - l->v.y;

	l->u.x = - l->u.x;
	l->u.y = - l->u.y;

	l->a = -l->a;
	l->c_prb = -l->c_prb;




	p = l->p[0];
	l->p[0] = l->p[1];
	l->p[1] = p;



	if( LNK_PREV_JNT(l) != NULL )
		l->p[0].p.jnt->b[l->p[0].bno ].direct = F_BEGIN;

	if( LNK_NEXT_JNT(l) != NULL )
		l->p[1].p.jnt->b[l->p[1].bno ].direct = F_END;
}

float
lnk_length(lnk_type *link, lnk_type *endLink)
{
lnk_type	*l;
float length = 0.0F;

	
	for ( l = link; l != NULL && l != endLink ; l = LNK_NEXT(l))
	{
		length += l->len;
	}	
	if( endLink != NULL )
		length += endLink->len;

	return length;
}