/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*********************
 ***	PlTool.c   ***
 *********************/

#include        <stdio.h>   
#include	<math.h>

#include        "SklType/LnkType.h"

#include        "SklType/PlType.h"



/*
creates a new pl_type which connects between point v, and polylink that starts at link, 
and ctr is it's location.
*/
pl_type *
pl_point_set( vec2d *ctr, lnk_type *link, vec2d *v )
{
lnk_type	*l;
pl_type	*pl;
vec2d	dv;
float	t,	len,	t0;

	pl = PL_ALLOC();

	lnk_get_status( ctr, link, NULL, v, &l/*closest link*/, &pl->u, &pl->t );

	pl->l = l;


	dv.x = v->x - l->ctr.x;
	dv.y = v->y - l->ctr.y;


	t = -l->u.y * dv.x + l->u.x * dv.y;
	if( t < 0 || t > l->len ){
		pl->t = t;
		pl->u = l->u.x * dv.x + l->u.y * dv.y; 
	}


	t0 = t - l->t0 * pl->u;
	len = l->len  - l->t0 * pl->u + l->t1 * pl->u;

	pl->t  = t0 / len;
/*
	pl->u /= len;
*/


	return( pl );
}





void
pl_point( pl_type *pl, vec2d *p )
{
float   t,      u,      h,      len;
lnk_type        *l;
 
        l = pl->l;
 
        len = l->len  - l->t0 * pl->u + l->t1 * pl->u;
/*
        u = pl->u * len;
*/
        u = pl->u;
 
        t = pl->t * len + u * l->t0;
 
 
        if( t < 0 || t > l->len ){
                p->x = l->ctr.x  +  t * -l->u.y  +  u * l->u.x;
                p->y = l->ctr.y  +  t *  l->u.x  +  u * l->u.y;
                return;
        }
 
 
        h = l->c_prb * t * (t - l->len) + u;
 
        p->x = l->ctr.x  +  t * -l->u.y  +  h * l->u.x;
        p->y = l->ctr.y  +  t *  l->u.x  +  h * l->u.y;
} 



void
plw_point( pl_type *pl, vec2d *p )
{
vec2d   p0,     v;
float   w;
 
        pl_point( pl, &v );
        p0.x = pl->w * v.x;
        p0.y = pl->w * v.y;
        w = pl->w;
 
        if( pl->p != NULL ){
                pl_point( pl->p, &v );
                p0.x += pl->p->w * v.x;
                p0.y += pl->p->w * v.y;
                w += pl->p->w;
        }
 
        if( w >= 1 ){
                p->x = p0.x / w;
                p->y = p0.y / w;
                return;
        }
 
        p->x = p0.x + ( 1 - w )* p->x;
        p->y = p0.y + ( 1 - w )* p->y;
}  
