/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>
#include	<math.h>

#include        "SklType/SklType.h"



static void	lnk_affine( lnk_type *link0, lnk_type *link1,
				float t, vec2d *ctr, lnk_type *link );




void
skl_affine( skl_type *skl0, skl_type *skl1, float t, skl_type *skl )
{
jskl_type	*jnt,	*j0,	*j1;
bskl_type	*b,	*b0;
int	i;

	for( jnt = skl->jnt, j0 = skl0->jnt, j1 = skl1->jnt ; jnt != NULL ;
				jnt= jnt->nj, j0 = j0->nj, j1 = j1->nj ){

		if( jnt->Ftopology == 0 )	continue;

		jnt->ctr.x = (1 - t ) *j0->ctr.x + t *j1->ctr.x;
		jnt->ctr.y = (1 - t ) *j0->ctr.y + t *j1->ctr.y;

		for( i = 0 ; i < jnt->bno ; i++ ){

			b = &jnt->b[i];
			if( b->direct == F_END )	continue;

			b0 = &j0->b[i];

			lnk_affine( j0->b[i].link, j1->b[i].link,
					t, &jnt->ctr, b->link);
		}
	}


	for( jnt = skl->jnt, j0 = skl0->jnt, j1 = skl1->jnt ; jnt != NULL ;
				jnt= jnt->nj, j0 = j0->nj, j1 = j1->nj ){

		if( jnt->Ftopology == 1 )	continue;

		if( jnt->pl != NULL )
			pl_point( jnt->pl, &jnt->ctr );

		for( i = 0 ; i < jnt->bno ; i++ ){

			b = &jnt->b[i];
			if( b->direct == F_END )	continue;

			b0 = &j0->b[i];

			lnk_affine( j0->b[i].link, j1->b[i].link,
					t, &jnt->ctr, b->link);
		}
	}
}



static void
lnk_affine( lnk_type *link0, lnk_type *link1,
			float t, vec2d *ctr, lnk_type *link )
{
lnk_type	*l,	*l0,	*l1,	*pl;
jskl_type	*jnt;
vec2d	v;
float	tmp;

	for( l = link, l0 = link0, l1 = link1, v = *ctr ; l != NULL ; 
			l = LNK_NEXT(l), l0 = LNK_NEXT(l0), l1 = LNK_NEXT(l1) ){

		l->ctr = v;

		l->u.x = ( 1-t)*l0->u.x + t * l1->u.x;
		l->u.y = ( 1-t)*l0->u.y + t * l1->u.y;

		tmp = 1.0 / hypot( l->u.x, l->u.y );
		l->u.x *= tmp;
		l->u.y *= tmp;

		l->len = ( 1-t)*l0->len + t * l1->len;
		l->v.x = - l->len * l->u.y;
		l->v.y =   l->len * l->u.x;

		l->a = ( 1-t)*l0->a + t * l1->a;

		l->c_prb = ( 1-t)*l0->c_prb + t * l1->c_prb;


		v.x += l->v.x;
		v.y += l->v.y;

		pl = l;
	}

	jnt = pl->p[F_END].p.jnt;
	if( jnt != NULL && jnt->Ftopology == 0 )
		jnt->ctr = v;

}
