/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	SklDecoding.c	***
 *************************/
#include        <stdio.h>
#include        "Uigp/igp.h"
#include        "Uprm/prm.h"
#include        "Ucode/code.h"
#include        "Ubst/BstType.h"

#include		"SklType/SklType.h"

#include		"SklCoding.h"


static void	decoding_jskl( bst_type *bst, codingSkl_type *cd, jskl_type *jnt );
static void	decoding_jskl_attach( bst_type *bst, codingSkl_type *cd,
				skl_type *skl, jskl_type *jnt );

static void	decoding_lnk( bst_type *bst, codingSkl_type *cd, 
				skl_type *skl, jskl_type *jskl, int bno );





skl_type *
skl_decoding_from_file( char *file )
{
bst_type	*bst;
skl_type	*skl;


	if( (bst = bst_read_from_file( file ) ) == NULL ){
		bst_free( bst );
		return( NULL );
	}


	skl = skl_decoding( bst );


	bst_free( bst );

	return( skl );
}



skl_type *
skl_decoding( bst_type *bst )
{
codingSkl_type	*cd;
skl_type	*skl;
jskl_type	*jnt;
int	no,	i,	head;

	skl = skl_alloc();



	head = bst_get_int( bst, 8 );

	if( head & SKL_NAME )
		bst_get_string( bst, skl->name, 15 );


	if(  head & SKL_ORIENTATION )
		skl->orientation = bst_get_int( bst, 2 );



	skl->row = bst_get_int( bst, 12 );
	skl->column = bst_get_int( bst, 12 );

	cd = skl_coding_prm_get( skl->row, skl->column );


	cd->Br_bit = bst_get_int( bst, 4 );
	cd->La_bit = bst_get_int( bst, 5 );
	cd->Llen_bit = bst_get_int( bst, 5 );


	no = bst_get_int( bst, 6 );
	cd->Jnt_bit = coding_ilog2( no );





	/* decoding  JNT */
	for( i = 0 ; i < no ; i++ ){
		jnt = skl_new_jnt( skl );
		decoding_jskl( bst, cd, jnt );
	}


	/* decoding  LINK */
	for( jnt = skl->jnt ; jnt != NULL ; jnt = jnt->nj )
		for( i = 0 ; i < jnt->bno ; i++ ){
			if( jnt->b[i].direct == F_END )	continue;

			decoding_lnk( bst, cd, skl, jnt, i );
		}


	/* decoding  ATTACH */
	for( jnt = skl->jnt ; jnt != NULL ; jnt = jnt->nj )
		decoding_jskl_attach( bst, cd, skl, jnt );


	skl_set_type( skl );

	skl_coding_prm_destroy( cd );

	return( skl );
}



static void
decoding_jskl( bst_type *bst, codingSkl_type *cd, jskl_type *jnt )
{
int	i,	cno;
bskl_type	*b;

	cno = bst_get_int( bst, cd->Sctr_x.rbit );
	jnt->ctr.x = UNCODE( cno, cd->Sctr_x );

	cno = bst_get_int( bst, cd->Sctr_y.rbit );
	jnt->ctr.y = UNCODE( cno, cd->Sctr_y );


	jnt->br = bst_get_int( bst, cd->Br_bit );

	jnt->bno = bst_get_int( bst, 3 ) + 1;


	for( i = 0 ; i < jnt->bno ; i++ ){
		b = &jnt->b[i];

		b->direct = bst_get( bst );
	}
}

static void
decoding_lnk( bst_type *bst, codingSkl_type *cd, skl_type *skl, jskl_type *jskl, int bno )
{
jskl_type	*jnt;
lnk_type	*l,	*pl;
vec2d	ctr;
int	no,	i;
int	br;
float	width;

	br = bst_get_int( bst, cd->Br_bit );


	if( bst_get( bst ) != 0 ){
		width = bst_get_int( bst, 6 );
	}
	else	width = 0;

	l = LNK_ALLOC();


	l->p[F_BEGIN].bno = bno;
	l->p[F_BEGIN].p.jnt = jskl;
	jskl->b[bno].link = l;

	ctr = jskl->ctr;
	while( 1 ){

		l->ctr = ctr;

		l->a = bst_get_float_code( bst, &cd->Sa, cd->La_bit );
	

/*
		if( LNK_PREV(l) != NULL )
			l->Fkey = bst_get( bst );
*/
	
		l->br = br;

		l->width = width;
	
		if( bst_get( bst ) == 0 )	break;


		l->v.x = bst_get_float_code( bst, &cd->Sctr_x, cd->Llen_bit );
		l->v.y = bst_get_float_code( bst, &cd->Sctr_y, cd->Llen_bit );

		ctr.x += l->v.x;
		ctr.y += l->v.y;


		lnk_set_aux( l );


		pl = l;
		l = LNK_ALLOC();

		pl->p[F_END].bno = l->p[F_BEGIN].bno = -1;
		pl->p[F_END].p.l = l;
		l->p[F_BEGIN].p.l = pl;
	}


	if( bst_get( bst ) == 1 ){
		l->v.x = bst_get_float_code( bst, &cd->Sctr_x, cd->Llen_bit );
		l->v.y = bst_get_float_code( bst, &cd->Sctr_x, cd->Llen_bit );




		l->p[F_END].bno = -1;
		l->p[F_END].p.l = NULL;

		lnk_set_aux( l );

		return;
	}



	no = bst_get_int( bst, cd->Jnt_bit );
	for( jnt = skl->jnt, i = 0 ; i < no ; i++ , jnt = jnt->nj );

	l->v.x = jnt->ctr.x - l->ctr.x;
	l->v.y = jnt->ctr.y - l->ctr.y;
	lnk_set_aux( l );

	no = bst_get_int( bst, 2 );

	l->p[F_END].bno = no;
	l->p[F_END].p.jnt = jnt;

	jnt->b[no].link = l;
}


static void
decoding_jskl_attach( bst_type *bst, codingSkl_type *cd, skl_type *skl, jskl_type *jnt )
{
jskl_type	*j;
lnk_type	*l;
int	no,	i;

	if( bst_get( bst ) == 0 ){
		jnt->pl = NULL;
		return;
	}

	no = bst_get_int( bst, cd->Jnt_bit );
	for( j = skl->jnt, i = 0 ; i < no ; i++, j = j->nj );

	no = bst_get_int( bst, 4 );

	l = j->b[no].link;

	lnk_set_bes( l );
	jnt->pl = pl_point_set( &l->ctr, l, &jnt->ctr );
}

