/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	SklFlip.c	***
 *************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"SklType/SklType.h"




void
skl_flipV( skl_type *skl, int column )
{
jskl_type	*jnt;
lnk_type	*l;
vec2d	v;
int	i;

	for( jnt = skl->jnt ; jnt != NULL ; jnt = jnt->nj ){

		jnt->ctr.y = column - jnt->ctr.y;


		for( i = 0 ; i < jnt->bno ; i++ ){
			if( jnt->b[i].direct != F_BEGIN )	continue;

			v = jnt->ctr;
			for( l = jnt->b[i].link ; l != NULL ; l = LNK_NEXT(l)){

				l->ctr = v;

				l->v.y = -l->v.y;
				l->a = -l->a;

				lnk_set_aux( l );

				v.x += l->v.x;
				v.y += l->v.y;
			}
		}
	}


	for( jnt = skl->jnt ; jnt != NULL ; jnt = jnt->nj ){
		if( jnt->pl == NULL )	continue;

		l = jnt->pl->l;
		while( LNK_PREV(l) != NULL )	l = LNK_PREV(l);
		free (jnt->pl );
		jnt->pl = pl_point_set( &l->ctr, l, &jnt->ctr );
	}
}




void
skl_copy_flipV( skl_type *skl, int column, skl_type *tskl )
{
jskl_type	*j,	*tj;
lnk_type	*l,	*tl;
int	i;

	tskl->type = skl->type;
	tskl->topology = skl->topology;

	for( j = skl->jnt, tj = tskl->jnt ; j != NULL ; j = j->nj,tj = tj->nj){

		tj->ctr.x = j->ctr.x;
		tj->ctr.y = column - j->ctr.y;
		tj->Ftopology = j->Ftopology;

		for( i = 0 ; i < j->bno ; i++ ){
			if( j->b[i].direct != F_BEGIN )	continue;

			for( l = j->b[i].link, tl = tj->b[i].link ; l != NULL ;
					l = LNK_NEXT(l), tl = LNK_NEXT(tl) ){

				tl->ctr.x = l->ctr.x;
				tl->ctr.y = column - l->ctr.y;

				tl->v.x =  l->v.x;
				tl->v.y = -l->v.y;
				tl->a = -l->a;


				tl->u.x = -l->u.x;
				tl->u.y =  l->u.y;

				tl->c_prb = -l->c_prb;
				tl->len = l->len;

			}
		}
	}
}


/*eyal
Description:
	The method accepts a link, and flipps the direction of the owner polylink
Params:
	link - part of a polylink to be flipped.
*/
void skl_PLFlip(lnk_type *link)
{
	jskl_type * jnt;
	lnk_type *l1,*l0 = link;
	int i;
	l1 = LNK_NEXT(l0);
	while ( l1 != NULL)
	{
		l0 = l1;
		l1 = LNK_NEXT(l0);
	}
	// l0 is the last link in the polylink
	jnt = l0->p[F_END].p.jnt;
	for (i = 0 ; i < jnt->bno ; i ++)
		if ( jnt->b[i].link == l0 )
			break;
	if ( i == jnt->bno )// didn't find the link!!!
		return;
	jnt->b[i].direct = F_BEGIN;// becomes the first in the pl
	// before the flip prev is the next one we want to go to,
	// after the flip, LNK_NEXT is what was PREV...
	for ( ; LNK_PREV(l0) != NULL ; l0 = LNK_NEXT(l0) )
	{
		flip_link(l0);
	}
	flip_link(l0);
	jnt = l0->p[F_END].p.jnt;
	for (i = 0 ; i < jnt->bno ; i ++)
		if ( jnt->b[i].link == l0 )
			break;
	if ( i == jnt->bno )// didn't find the link!!!
		return;
	jnt->b[i].direct = F_END;// now it's the last in the pl
}// skl_PLFlip


/*change the direction of the link in the polylink*/
void flip_link(lnk_type *link)
{
	vec2d ctr,v;
	lnk_ptr tmp;
	float		len;	
	if ( link == NULL )
		return;
	v.x = link->ctr.x + link->v.x;
	v.y = link->ctr.y + link->v.y;
	ctr = v;
	v = link->ctr;
	link->ctr = ctr;
	link->v.x = v.x - link->ctr.x;
	link->v.y = v.y - link->ctr.y;
	len = (float)hypot( link->v.x, link->v.y );
	link->a = link->a * len /link->len;
	tmp = link->p[F_BEGIN];
	link->p[F_BEGIN] = link->p[F_END];
	link->p[F_END] = tmp;
	lnk_set_aux( link );	
}// flip_link

void skl_JntSetFirst(jskl_type *jnt,skl_type *skl)
{
	jskl_type *jnt1, *jnt0,*tmp;
	if ( skl == NULL || jnt == NULL )
		return;
	jnt1 = skl->jnt;
	jnt0 = NULL;
	while ( jnt1 != jnt && jnt1 != NULL)
	{
		jnt0 = jnt1;
		jnt1 = jnt1->nj;
	}
	if ((jnt0 == NULL) || (jnt1 == NULL))// the desired jnt is the first already, or didn't find it
		return;
	tmp = skl->jnt;
	skl->jnt = jnt1;
	jnt0->nj = jnt1->nj;
	jnt1->nj = tmp;	
	// trigger the skl MP and Topology
	skl_mass_point(skl);
	skl_topology(skl);
}