/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"


#include	"GpType/GpType.h"


#include	"SklType/SklType.h"




float
skl_select_point( skl_type *skl, vec2d *p, float sensitive, lnk_type *eclude_lnk,
		lnk_type **slink, int *sdirect, jskl_type **jskl, vec2d *spoint )
{
jskl_type	*j;
lnk_type	*l;
vec2d	v,	ctr;
float	tmp,	dis;
int	i;

	dis = sensitive*sensitive;

	*jskl = NULL;
	*slink = NULL;

	for( j = skl->jnt ; j != NULL ; j = j->nj ){

		v.x = j->ctr.x - p->x;
		v.y = j->ctr.y - p->y;

		tmp = v.x *v.x + v.y *v.y;
		if( tmp >= dis )	continue;

		*jskl = j;
		*spoint = j->ctr;
		dis = tmp;
	}


	for( j = skl->jnt ; j != NULL ; j = j->nj ){

		for( i = 0 ; i < j->bno ; i++ ){
			if( j->b[i].direct != F_BEGIN )	continue;

			for( l = j->b[i].link ; l != NULL ; l = LNK_NEXT(l) ){

				if( l == eclude_lnk )	continue;

				v.x = l->ctr.x - p->x;
				v.y = l->ctr.y - p->y;
 
				tmp = v.x *v.x + v.y *v.y;
				if( tmp < dis ){
					*jskl = NULL;
					*slink = l; 
					*spoint = l->ctr;
					*sdirect = F_BEGIN;
					dis = tmp;
				}

				if( LNK_NEXT(l) != NULL )	continue;

				ctr.x = l->ctr.x + l->v.x;
				ctr.y = l->ctr.y + l->v.y;
		                v.x = ctr.x - p->x;
				v.y = ctr.y - p->y;
 
				tmp = v.x *v.x + v.y *v.y;
				if( tmp < dis ){
					*jskl = NULL;
					*slink = l; 
					*sdirect = F_END;
					*spoint = ctr;
					dis = tmp;
				}
			}
		}
	}


	if( *jskl == NULL && *slink == NULL )	return( -1 );

	if( *slink == NULL )	return( dis );

	l = *slink;
	if( *sdirect == F_BEGIN ){
		if( LNK_PREV(l) == NULL ){
			*jskl = l->p[F_BEGIN].p.jnt;
			*slink = NULL;
		}
		return( dis );
	}

	if( LNK_NEXT(l) == NULL && l->p[F_END].p.jnt != NULL ){
		*jskl = l->p[F_END].p.jnt; 
		*slink = NULL;
	}

	return( dis );
}




float
skl_select_link( skl_type *skl, vec2d *p, float sensitive, lnk_type **slink )
{
jskl_type	*j;
lnk_type	*l,	*sl;
float	dis,	u,	t;
int	i;

	dis = sensitive;
	*slink = NULL;

	for( j = skl->jnt ; j != NULL ; j = j->nj ){

		for( i = 0 ; i < j->bno ; i++ ){
			if( j->b[i].direct != F_BEGIN )	continue;

			for( l = j->b[i].link ; l != NULL ; l = LNK_NEXT(l) ){

				lnk_get_status( &l->ctr, l, NULL, p, &sl, &u, &t);

				if( sl == NULL )	continue;

				if( u < 0 )	u = -u;

				if( u > dis )	continue;

				*slink = sl;
				dis = u;
			}
		}
	}

	if( *slink == NULL )	return( -1 );

	return( dis );
}



void
skl_find_link( skl_type *skl, lnk_type *link,
			int *jnt_i, int *brn_i, int *lnk_i, int *direct  )
{
jskl_type	*jnt;
lnk_type	*l;
vec2d	ctr,	ctr1,	end,	end1;
int	i,	j,	k;

	ctr = link->ctr;
	end.x = link->ctr.x + link->v.x;
	end.y = link->ctr.y + link->v.y;

	*jnt_i = -1;

	for( jnt = skl->jnt, i = 0 ; jnt != NULL ; jnt = jnt->nj, i++ ){

		*jnt_i = i;

		for( j = 0 ; j < jnt->bno ; j++ ){
			if( jnt->b[j].direct != F_BEGIN )	continue;

			*brn_i = j;

			for( l = jnt->b[j].link, k = 0 ; l != NULL ;
							l = LNK_NEXT(l), k++){
				if( l->br != link->br )	continue;
				ctr1 = l->ctr;

				end1.x = l->ctr.x + l->v.x;
				end1.y = l->ctr.y + l->v.y;

				*lnk_i = k;

				if( ABS(ctr1.x - ctr.x ) < 0.3 &&
					ABS(ctr1.y - ctr.y ) < 0.3 &&
					ABS(end1.x - end.x ) < 0.3 &&
					ABS(end1.y - end.y ) < 0.3 ){

					*direct = 0;
					return;
				}
				if( ABS(end1.x - ctr.x ) < 0.3 &&
					ABS(end1.y - ctr.y ) < 0.3 &&
					ABS(ctr1.x - end.x ) < 0.3 &&
					ABS(ctr1.y - end.y ) < 0.3 ){

					*direct = 1;
					return;
				}
			}
		}
	}

	*jnt_i = -1;
}



void
skl_find_link_br( skl_type *skl, int br, int *jnt_i, int *brn_i, int *lnk_i )
{
jskl_type	*jnt;
lnk_type	*l;
int	i,	j,	k;


	for( jnt = skl->jnt, i = 0 ; jnt != NULL ; jnt = jnt->nj, i++ ){

		*jnt_i = i;

		for( j = 0 ; j < jnt->bno ; j++ ){
			if( jnt->b[j].direct != F_BEGIN )	continue;

			*brn_i = j;

			for( l = jnt->b[j].link, k = 0 ; l != NULL ;
							l = LNK_NEXT(l), k++){
				*lnk_i = k;
				if( l->br == br )	return;
			}
		}
	}

	*jnt_i = -1;
}



void
skl_find_jnt( skl_type *skl, jskl_type *jskl, int *jnt_i )
{
jskl_type	*jnt;
int	i;


	*jnt_i = -1;
	for( jnt = skl->jnt, i = 0 ; jnt != NULL ; jnt = jnt->nj, i++ ){

		
		if( jnt->ctr.x == jskl->ctr.x && jnt->ctr.y == jskl->ctr.y ){
			*jnt_i = i;
			return;
		}
	}
}
