/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	SklShift.c	***
 *************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"

#include	"SklType/SklType.h"


/* static functions*/
static void
skl_shift_sub_tree_move( skl_type *skl, jskl_type* src_jnt, float sx, float sy );
static void skl_shift_sub_tree_reorder_lnks(vec2d *ctr, lnk_type* l);


void
skl_shift_sub_tree_jnt( skl_type *skl, jskl_type* src_jnt, float sx, float sy )
{
jskl_type	*jnt;
lnk_type	*l;
int	i;

	skl_shift_sub_tree_move(skl, src_jnt, sx, sy);

	/*for( jnt = skl->jnt ; jnt != NULL ; jnt = jnt->nj ){
		for( i = 0 ; i < jnt->bno ; i++ ){
			if( jnt->b[i].direct != F_END )	continue;
			l = jnt->b[i].link;
			if ( (l->v.x + l->ctr.x == jnt->ctr.x) && (l->v.y + l->ctr.y == jnt->ctr.y) ) continue;
			l->v.x = jnt->ctr.x - l->ctr.x;
			l->v.y = jnt->ctr.y - l->ctr.y;
			lnk_set_aux(l);
		}
	}*/

	for( jnt = skl->jnt ; jnt != NULL ; jnt = jnt->nj ){
		for( i = 0 ; i < jnt->bno ; i++ ){
			if( jnt->b[i].direct != F_END )	continue;
			l = jnt->b[i].link;
			if ( (l->v.x + l->ctr.x == jnt->ctr.x) && (l->v.y + l->ctr.y == jnt->ctr.y) ) continue;
			
			skl_shift_sub_tree_reorder_lnks(&(jnt->ctr), l);
		}
	}

	skl_update_attach(skl);
	skl_mass_point( skl );
}


void
skl_shift_sub_tree_lnk( skl_type *skl, lnk_type* lnk, int direct, float sx, float sy )
{
jskl_type	*fjnt = NULL;
lnk_type	*l;
vec2d ctr;

	l = (direct == F_BEGIN) ? LNK_PREV(lnk) : lnk;
	if ( l != NULL) {
	/* change v of prev lnk	*/
/*		l->v.x += sx;
		l->v.y += sy;
		lnk_set_aux( l );*/
		ctr.x = l->ctr.x + l->v.x + sx;
		ctr.y = l->ctr.y + l->v.y + sy;
		skl_shift_sub_tree_reorder_lnks(&ctr, l);
	}
	else if (lnk->p[F_BEGIN].bno >= 0){
		/* if previous was jnt - change it and all its links */
		skl_shift_sub_tree_jnt( skl, lnk->p[F_BEGIN].p.jnt, sx, sy );
		return;
	}

	/* move all links till jnt*/
	l = (direct == F_BEGIN) ? lnk : LNK_NEXT(lnk);
	while (l != NULL) {
		l->ctr.x += sx;
		l->ctr.y += sy;

		if (LNK_NEXT(l) == NULL) {
			if (l->p[1].bno >= 0)
				fjnt = l->p[F_END].p.jnt;
			break;
		}

		l = LNK_NEXT(l);
	}

	if (fjnt == NULL) {
/*
		skl_mass_point( skl );
*/
		return;
	}

	if( fjnt->Ftopology == 1 ){
		l->v.x = fjnt->ctr.x - l->ctr.x;
		l->v.y = fjnt->ctr.y - l->ctr.y;

		lnk_set_aux(l);
		return;
	}


	/* do the rest with jnt*/
	skl_shift_sub_tree_jnt( skl, fjnt, sx, sy );
}



static void
skl_shift_sub_tree_move( skl_type *skl, jskl_type* src_jnt, float sx, float sy )
{
jskl_type	*jnt;
lnk_type	*l, *pl;
int	i;

		src_jnt->ctr.x += sx;
		src_jnt->ctr.y += sy;

		// traverse through the graph and move all reachables links and jnts.
		for( i = 0 ; i < src_jnt->bno ; i++ ){
			if( src_jnt->b[i].direct != F_BEGIN )	continue;

			for( pl = NULL, l = src_jnt->b[i].link ; l != NULL ; pl = l, l = LNK_NEXT(l)) {
				l->ctr.x += sx;
				l->ctr.y += sy;
			}

			/* if we have a joint and its center didn't moved , enter into recursion */
			if (pl == NULL || pl->p[F_END].bno < 0) continue;
			jnt = pl->p[F_END].p.jnt; 
			if ( (pl->v.x + pl->ctr.x == jnt->ctr.x) && (pl->v.y + pl->ctr.y == jnt->ctr.y)) continue;
			skl_shift_sub_tree_move( skl, jnt , sx, sy );
		}

		if (src_jnt->pl != NULL) 
			src_jnt->pl = pl_point_set( &(src_jnt->pl->l->ctr), src_jnt->pl->l, &(src_jnt->ctr) );
}


static void skl_shift_sub_tree_reorder_lnks(vec2d *ctr, lnk_type* lnk)
{
	lnk_type* l;
	jskl_type* jnt;
	vec2d old_v, angle_v, cur_ctr, new_v, cur_v, old_u, new_u;
	float new_len, old_len, rel;

	/* find previus jnt or first link*/
	l = lnk;
	while (l->p[F_BEGIN].bno < 0 && LNK_PREV(l) != NULL) 
		l = LNK_PREV(l);

	/* error . this cannot occur*/
	if (l->p[F_BEGIN].bno < 0) return;

	jnt = l->p[F_BEGIN].p.jnt;

	/*calculate old vector from jnt to old ctr*/
	old_v.x = lnk->ctr.x + lnk->v.x - jnt->ctr.x;
	old_v.y = lnk->ctr.y + lnk->v.y - jnt->ctr.y;
	old_len = (float)hypot(old_v.x, old_v.y);
	old_u.x = - old_v.y / old_len;
	old_u.y = old_v.x / old_len;

	new_v.x = ctr->x - jnt->ctr.x;
	new_v.y = ctr->y - jnt->ctr.y;
	new_len = (float)hypot(new_v.x, new_v.y);
	new_u.x = - new_v.y / new_len;
	new_u.y = new_v.x / new_len;

	rel = new_len / old_len;
	/* angle */
	angle_v.x = old_u.x * new_u.x + old_u.y * new_u.y;
	angle_v.y = old_u.y * new_u.x - old_u.x * new_u.y;

	cur_ctr = jnt->ctr;
	for (; l != NULL ; l = LNK_NEXT(l)) {

		/* rotation */
		l->ctr = cur_ctr;

		cur_v.x = (  l->v.x * angle_v.x + l->v.y * angle_v.y) * rel;
		cur_v.y = (- l->v.x * angle_v.y + l->v.y * angle_v.x) * rel;

		l->v = cur_v;
		lnk_set_aux(l);
		cur_ctr.x += cur_v.x;
		cur_ctr.y += cur_v.y;

		if (l == lnk) break;
	}
	

}