/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"



#include	"SklType/SklType.h"
static int
skl_delete_link1( skl_type *skl, lnk_type *link );



skl_type *
skl_alloc()
{
skl_type	*skl;

	skl = (skl_type *)malloc( sizeof(skl_type) );

	skl->jnt = NULL;

	skl->type = SKL_NULL;
	skl->orientation = 0;

	skl->name[0] = 0;

	return( skl );
}


void
skl_free( skl_type *skl )
{
	if( skl == NULL )	return;

	skl_clear( skl );

	free( skl );
}
	


void
skl_clear( skl_type *skl )
{
jskl_type	*j;


	while( (j = skl->jnt) != NULL ){
		skl->jnt = skl->jnt->nj;
		jskl_free( j );
	}
}



jskl_type *
skl_new_jnt( skl_type *skl )
{
jskl_type	*jnt,	*j;

	jnt = jskl_alloc();
	jnt->bno = 0;

	jnt->nj = NULL;

	if( skl->jnt == NULL ){
		skl->jnt = jnt;

		return( jnt );
	}

	for( j = skl->jnt ; j->nj != NULL ; j = j->nj );
	j->nj = jnt;

	return( jnt );
}



void
skl_untied_jnt( skl_type *skl, jskl_type *jnt )
{
jskl_type	**pj;

	for( pj = &skl->jnt ; *pj != jnt ; pj = &(*pj)->nj );
		if( *pj == NULL )	return;
 
	*pj = jnt->nj;
}




int
skl_is_equal( skl_type *skl, skl_type *skl0 )
{
jskl_type	*jnt,	*j0;
bskl_type	*b,	*b0;
lnk_type	*l,	*l0;
float	t;
int	i;


	for( jnt = skl->jnt, j0 = skl0->jnt ; jnt != NULL ;
						jnt= jnt->nj, j0 = j0->nj ){

		if( jnt->ctr.x != j0->ctr.x || jnt->ctr.y != j0->ctr.y )
			return( 0 );


		for( i = 0 ; i < jnt->bno ; i++ ){

			b = &jnt->b[i];
			if( b->direct == F_END )	continue;

			b0 = &j0->b[i];

			for( l = b->link, l0 = b0->link ; l != NULL ;
				l = LNK_NEXT(l), l0 = LNK_NEXT(l0) ){

				if( (t = l->ctr.x - l0->ctr.x) < 0 )	t = -t;
				if( t > 0.001 )	return( 0 );

				if( (t = l->ctr.y - l0->ctr.y) < 0 )	t = -t;
				if( t > 0.001 )	return( 0 );

				if( (t = l->v.x - l0->v.x) < 0 )	t = -t;
				if( t > 0.001 )	return( 0 );

				if( (t = l->v.y - l0->v.y) < 0 )	t = -t;
				if( t > 0.001 )	return( 0 );

				if( (t = l->a - l0->a) < 0 )	t = -t;
				if( t > 0.001 )	return( 0 );
/*
				if( l->ctr.x != l0->ctr.x ||
						l->ctr.y != l0->ctr.y ||
						l->v.x != l0->v.x ||
						l->v.y != l0->v.y ||
						l->a != l0->a )
					return( 0 );
*/
			}
		}
	}

	return( 1 );
}




int
skl_topology( skl_type *skl )
{
jskl_type	*jnt;
lnk_type	*l;
int	top,	i;

	top = 0;
	if ( !skl )
		return 0;
	// make all joints dependent
	for( jnt = skl->jnt, i = 0 ; jnt != NULL ; jnt = jnt->nj ){
		jnt->no = i++;
		jnt->Ftopology = 0;
	}

	for( jnt = skl->jnt ; jnt != NULL ; jnt = jnt->nj ){
		// if a joint has not been reached before and not connected with PL - it's independent
		if( jnt->no >= 0 && jnt->pl == NULL ){
			top |= 1L << jnt->no;
			jnt->Ftopology = 1;
		}

		jnt->no = -1;

		for( i = 0 ; i < jnt->bno ; i++ ){

			if( jnt->b[i].direct == F_END )	continue;

			for( l = jnt->b[i].link ; LNK_NEXT(l) != NULL ;
							l = LNK_NEXT(l) );

			if (l->p[F_END].p.jnt != NULL )
				l->p[F_END].p.jnt->no = -1;// reached the joint, therefore no = -1 and dependent.
		}
	}

	skl->topology = top;

	return( top );
}



int
skl_is_equal_topology( skl_type *skl, skl_type *skl0 )
{
jskl_type	*jnt,	*j0;
bskl_type	*b,	*b0;
lnk_type	*l,	*l0;
int	i;


	for( jnt = skl->jnt, j0 = skl0->jnt ; jnt != NULL && j0 != NULL;
						jnt= jnt->nj, j0 = j0->nj ){


		for( i = 0 ; i < jnt->bno ; i++ ){

			b = &jnt->b[i];

			b0 = &j0->b[i];


			if( b->direct != b0->direct )	return( -1 );

			if( b->direct == F_END )	continue;


			for( l = b->link, l0 = b0->link ; l != NULL && l0 != NULL;
				l = LNK_NEXT(l), l0 = LNK_NEXT(l0) ){
			}

			if( l != NULL || l0 != NULL )	return( -1 );
		}
	}

	if( jnt != NULL || j0 != NULL )	return( -1 );

	return( 1 );
}




void
skl_set_type( skl_type *skl )
{
jskl_type	*jnt;
int	i;

	if( skl->jnt == NULL ){
		skl->type = SKL_NULL;
		return;
	}

	for( jnt = skl->jnt ; jnt != NULL ; jnt = jnt->nj ){


		for( i = 0 ; i < jnt->bno ; i++ ){
			if( jnt->b[i].direct != F_BEGIN )	continue;

			if( jnt->b[i].link->width > 0 ){
				skl->type = SKL_WIDTH;
				return;
			}
		}
	}

	skl->type = SKL_GLOBAL;
}




/**** skl_validate ****/
/*!
 * The method drops standalone joints, so than when skl is read, 
 * it is read correctly.
 *
 * @param *skl : the skeleton to check
 *
 * @return skl_type *  : the same skeleton with changes, or NULL if no skeleton left
 */
skl_type * skl_validate(skl_type *skl)
{
jskl_type	*j,	*prevJ = NULL;	

	j = skl->jnt; 
	while ( j != NULL ) 
	{
		if ( j->bno == 0 )
		{
			// is it the first joint?
			if ( prevJ == NULL ) 
			{
				skl->jnt = j->nj;				
				j->nj = NULL;
				jskl_free(j);
				j = skl->jnt;
				if ( j == NULL ) // don't have more joints....
					break;
				continue;

			}
			else
			{
				prevJ->nj = j->nj;
				j->nj = NULL;
				jskl_free(j);
				j = prevJ;
			}
			
		}
		else
		{
			prevJ = j;
		}
		j = j->nj;
	}// while


	if ( skl->jnt == NULL ) 
	{
		skl->type = SKL_NULL;
		return skl;
	}
	skl_topology(skl);
	return skl;
}




void	skl_filter_short_links(skl_type *skl, float threshold)
{
jskl_type	*jnt, *nj;

bskl_type	*b;
lnk_type	*l,	*l0,*l1;

int	i,bno;

	jnt = skl->jnt ;
	while (  jnt != NULL )
	{
		nj = jnt->nj;
		bno = jnt->bno;
		for( i = 0 ; i < bno ; i++ )
		{
			b = &jnt->b[i];
			if( b->direct == F_END )	
				continue;
			
			//get to last link
			for( l = b->link ; LNK_NEXT(l) != NULL ; l = LNK_NEXT(l));
			// go back and drop each short link
			while(  l != NULL  )
			{
				if( l->len < threshold)
				{
					l0 = LNK_PREV(l);
					l1 = LNK_NEXT(l);
					if ( l1 == NULL ) 
					{
						if ( l->p[F_END].p.jnt && l->p[F_END].p.jnt->bno == 1 )
						{
							if ( nj == l->p[F_END].p.jnt)
								nj = jnt->nj->nj;
						}

					}
					if ( skl_delete_link1( skl, l ) == 0 )
					{				

						l1 = LNK_NEXT(l);						
						l0->p[1].p = l->p[1].p;
						l0->p[1].bno = l->p[1].bno;

						l0->v.x += l->v.x;
						l0->v.y += l->v.y;
						lnk_set_aux(l0);
						if ( l1 != NULL ) 
						{
							l1->p[0].p = l->p[0].p;
							l1->p[0].bno = l->p[0].bno;
						}
					}					
					l = l0;
					continue;
				}
				l = LNK_PREV(l);
			}		
		}
		jnt= nj;
	}
	skl_validate(skl);
	
}


// ************
// NOT THE SAME AS IN SklDelete !!!!!
// 1)HERE WE JOIN THE LINKS AND NOT JUST DROP A LINK
// 2)HERE THE CENTER OF JOINT ISN'T CHANGED, 
//	 BUT THE LENGTH AND CTR OF THE LINKS ARE
// ************
static int
skl_delete_link1( skl_type *skl, lnk_type *link )
{
jskl_type	*jnt;
lnk_type	*l;


	for( jnt = skl->jnt ; jnt != NULL ; jnt = jnt->nj ){
		if( jnt->pl == NULL )	continue;
		if( jnt->pl->l != link )	continue;

		free( jnt->pl );
		jnt->pl = NULL;
	}


	if( LNK_NEXT(link) != NULL )
	{
		if( LNK_PREV(link) != NULL )	// it's a middle link
			return(0);

		// ok, it's a first link...
		l = LNK_NEXT(link);
		l->p[F_BEGIN] = link->p[F_BEGIN];

		jnt = l->p[F_BEGIN].p.jnt;
		jnt->b[link->p[F_BEGIN].bno].link = l;

		l->ctr.x -= link->v.x;
		l->ctr.y -= link->v.y;
		l->v.x += link->v.x;
		l->v.y += link->v.y;
		lnk_set_aux(l);

		//jnt->ctr = l->ctr;

		free( link );
		return(1);
	}


	if( (jnt = link->p[F_END].p.jnt) != NULL ){
		jskl_untied_brn( jnt, link->p[F_END].bno );

		if( jnt->bno == 0 ){
			skl_untied_jnt( skl, jnt );
			jskl_free( jnt );
		}
	}


	if( ( l = LNK_PREV(link)) != NULL )
	{
		
		l->p[F_END].p.l = NULL;
		l->v.x += link->v.x;
		l->v.y += link->v.y;
		lnk_set_aux(l);
		lnk_free( link );
		return(1);
	}

	if( (jnt = link->p[F_BEGIN].p.jnt) != NULL ){

		jskl_untied_brn( jnt, link->p[F_BEGIN].bno );

		lnk_free( link );

		if( jnt->bno == 0 ){
			skl_untied_jnt( skl, jnt );
			jskl_free( jnt );
		}
	}

	return( 1 );
}















