/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#ifndef         _SKL_TYPE_
#define         _SKL_TYPE_

#include        <stdio.h>


#include        "JsklType.h"
#include        "Ubst/BstType.h"
#include		"Ubox/Box2d.h"


/* skl.type */
#define		SKL_NULL	0
#define		SKL_GLOBAL	1
#define		SKL_WIDTH	2

#define		OLD2NEW 0
#define		NEW2OLD 1
typedef struct skl_type {

	int	type;			// SKL_NULL or something else - for future use.
	int	orientation;	
	vec2d	mp;			/* mass point */
	jskl_type	*jnt;	/*	list of jnt */
	int	topology;		// how many independent joins are in the skl
	short	row,	column;	// size of bounding box
	char	name[16];
}       skl_type;





	/* SklTool.c */
skl_type *	skl_alloc();

void	skl_free( skl_type *skl );

void	skl_clear( skl_type *skl );

jskl_type * skl_new_jnt( skl_type *skl );

void 	skl_untied_jnt( skl_type *skl, jskl_type *jnt );

void	skl_set_bes( skl_type *skl );

int	skl_is_equal( skl_type *skl, skl_type *skl0 );

int	skl_is_equal_topology( skl_type *skl, skl_type *skl0 );

int	skl_topology( skl_type *skl );

void	skl_set_type( skl_type *skl );

skl_type * skl_validate(skl_type *skl);
	/* SklCopy.c */
void	skl_copy( skl_type *skl, skl_type *tskl );

skl_type *	skl_make_copy( skl_type *skl );


	/* SklMassPoint.c */
int	skl_mass_point( skl_type *skl );

int	skl_mass_joint_set( skl_type *skl, jskl_type *jnt );



	/* SklSelect.c */
float	skl_select_link( skl_type *skl, vec2d *p, float sensitive,
						lnk_type **slink );

float	skl_select_point( skl_type *skl, vec2d *p, float sensitive,
			lnk_type *eclude_lnk,
			lnk_type **slink, int *sdirect,
			jskl_type **jskl, vec2d *spoint );

void	skl_find_link( skl_type *skl, lnk_type *link,
                        int *jnt_i, int *brn_i, int *lnk_i, int *direct  );

void	skl_find_link_br( skl_type *skl, int br,
			int *jnt_i, int *brn_i, int *lnk_i );

void	skl_find_jnt( skl_type *skl, jskl_type *jskl, int *jnt_i );


	/* SklBr.c */
int	skl_br( skl_type *skl );

void	skl_br_add( skl_type *skl, int br );

void	skl_br_set( skl_type *skl, int br );


	/* SklEncoding.c */
void	skl_encoding_to_file( skl_type *skl,
			int row, int column, char *dir, char *name );

void	skl_encoding( skl_type *skl, int row, int column, bst_type *bst );



	/* SklDecoding.c */
skl_type *	skl_decoding_from_file( char *file );

skl_type *	skl_decoding( bst_type *bst );


	/* SklUpdateAttach */
void	skl_update_attach( skl_type *skl );



	/* SklLshift.c */
void	skl_Lshift_update_link( skl_type *skl,
			lnk_type *link, vec2d *p, int flag );

void	skl_Lshift_update_link_oposite( skl_type *skl,
			lnk_type *link, vec2d *p, int flag );

void	skl_Lshift_update_jnt_ctr( skl_type *skl,
				jskl_type *jnt, int bno, vec2d *ctr );


	/* SklRotate.c */
void	skl_rotate( skl_type *skl, vec2d *ctr, float cosa, float sina );


	/* SklRescale.c */
void	skl_rescale( skl_type *skl, float sx, float sy );


	/* SklShift.c */
void	skl_shift( skl_type *skl, float sx, float sy );

int		skl_box( skl_type *skl, box2d *box );


	/* SklShiftSubTree.c */
void    skl_shift_sub_tree_jnt( skl_type *skl, jskl_type* jnt, float sx, float sy );
void    skl_shift_sub_tree_lnk( skl_type *skl, lnk_type* lnk, int direct, float sx, float sy );



	/* SklAffine.c */
void	skl_affine( skl_type *skl0, skl_type *skl1, float t, skl_type *skl );


	/* SklFlip.c */
void	skl_flipV( skl_type *skl, int column );

void	skl_copy_flipV( skl_type *skl, int column, skl_type *tskl );


	/* SklComponnet.c */
int	skl_componnet( skl_type *skl );


	/* SklCompare.c */
int	skl_compare( skl_type *skl0, skl_type *skl );


	/* SklAppend.c */
void	skl_append ( skl_type *bskl, vec2d *vec, skl_type *skl );

	/*SklDecrease.c*/
int		skl_is_close_enough( skl_type *skl, skl_type *skl0, float delta );
float	skl_distance( skl_type *skl, skl_type *skl0);

void	skl_PLFlip(lnk_type *link);
void	flip_link(lnk_type *link);
void	skl_JntSetFirst(jskl_type *jnt,skl_type *skl);


	/* SklSimplifay.c */
void	skl_simplifay( skl_type *skl, float Tl );

	/* SklSimplifayKey.c */
void	skl_simplifay_key( skl_type *skl );

void	skl_filter_short_links(skl_type *skl, float threshold);

	/* SklDeleteLink.c */
int	skl_delete_polylink( skl_type *skl, lnk_type *link );

	/* SklFilter.c */
void	skl_filter( skl_type *skl );

	/* SklEdit.c */
jskl_type *	skl_lnk_to_jnt( skl_type *skl, lnk_type *l );
int	skl_jnt_to_point( skl_type *skl, jskl_type *jnt );

#endif

