/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	LnkType.h	***
 **************************/

#ifndef         LNK_TYPE
#define         LNK_TYPE

#include	"Uvec/Vec2d.h"

#include	"GpType/MbrType.h"


typedef struct lnk_ptr {
        int     bno;
        union   {
                struct lnk_type *l;
                struct jskl_type *jnt;
        } p;
}       lnk_ptr;



typedef struct lnk_type {
	vec2d	ctr;	// start point of the link
	vec2d	v;		// base vector for the parabola - correct direction
	float	a;		/* parabola  height */
	/*	AUX	*/
	float	len;	// length of v
	vec2d	u;		/* right normal to v	*/
	float   c_prb;	/* 2-nd order coefficient of parabola */
	float	t0,	t1; // for pl_type calculations
	char	Fkey;	// 

	char	br;		// layer number
	float	width;	// width of influence  - infinity (0) or something else ( != 0)
	mbr_type	mbr;		// maks for layers
	struct lnk_ptr	p[2];/* F_BEGIN and F_END previous and next links (or joint if last one in the pl)*/
} lnk_type;




#define		LNK_ALLOC()	\
			( (lnk_type *)malloc( sizeof( lnk_type ) ) )


/*
#define		LNK_NEXT( l )	( l->p[F_END] )

#define		LNK_PREV( l )	( l->p[F_BEGIN] )
*/


#define         LNK_NEXT( link )       \
                (( link->p[F_END].bno < 0  )? link->p[F_END].p.l : NULL )

#define         LNK_PREV( link )       \
                (( link->p[F_BEGIN].bno < 0  )? link->p[F_BEGIN].p.l : NULL )



#define		LNK_NEXT_JNT( link )	\
		(( link->p[F_END].bno < 0 )? NULL : link->p[F_END].p.jnt )

#define		LNK_PREV_JNT( link )	\
		(( link->p[F_BEGIN].bno < 0 )? NULL : link->p[F_BEGIN].p.jnt )



lnk_type * lnk_make_copy( lnk_type *link );

void	lnk_free( lnk_type *lnk );


	/* LnkTool.c */
void	lnk_set_aux( lnk_type *l );

void	lnk_t2xy( vec2d *ctr, lnk_type *link, float t, vec2d *p );

void	lnk_update_point( vec2d *ctr, lnk_type *link, int direct, vec2d *p );

void	lnks_update_point( lnk_type *link, int direct, vec2d *p );

void	lnk_a_point( vec2d *ctr, lnk_type *l, vec2d *p );

void	lnk_a_add( vec2d *ctr, lnk_type *l, float d );

void	lnk_conent( lnk_type *link, lnk_type *l );

void	lnk_set_gctr( vec2d *ctr, lnk_type *link );

void	lnk_set_bes( lnk_type *link );


lnk_type *	lnk_inverse( lnk_type *link );

void	lnk_1_inverse( lnk_type *l );

float	lnk_length(lnk_type *link, lnk_type *endLink);

	/* LnkSelect.c */

float	lnk_select( vec2d *ctr, lnk_type *link, vec2d *p,
			 	lnk_type **sl, int *direct );
float	lnk_select_point( vec2d *ctr, lnk_type *link, vec2d *p,
			 	lnk_type **sl, int *direct );

int	lnk_get_status( vec2d *vm, lnk_type  *link, lnk_type *elink, vec2d  *v,
			lnk_type **sl, float *u, float *t );

int	lnk_array_select( lnk_type  *link[], int no, vec2d  *v, mbr_type br,
                                lnk_type **sl, float *u, float *t );
#endif
