/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/******************************
 ***	VimageReadBlock.c   ***
 ******************************/
#include	<stdio.h>
#include	<math.h>
#include	<string.h>

#include	"Uigp/igp.h"

#include	"GpType/Configuration.h"

#include	"Uigp/igp.h"
#include	"Ubst/BstType.h"


#include	"VimageType/VimageType.h"
#include	"VimCoding.h"
#include	"VimCodingIn.h"

#include	"VimIo.h"




static void	vim_read_sound_block( vimage_type *vim,
								 bst_type *bst, vimIo_type *vio );

static void	vim_read_bframe_block( vimage_type *vim,
								  bst_type *bst, vimIo_type *vio );





int
vim_decoding_block( vimage_type *vim, int Ftexture,
					int head, bst_type *bst,
					vimIo_type *vio )
{
cast_type	*cast;
#ifndef _ST
int	ret;
#endif


	if( vio->version <= 0x0005 ){
#ifndef _ST
		ret = vim_decoding_block0005( vim, Ftexture, head, bst, vio );
		return( ret );
#else
		return( -1 );
#endif // _ST
	}


	switch( head ){

	case 'S':
		vim_decoding_scene_block( vim, bst, vio );
	    break;

	case 'C':
		if( (cast = vim_decoding_cast_block( bst, vio )) != NULL )
			vim->cast[vim->no++] = cast;

		 break;

	case 'A':
			vim_decoding_texture_block( vim, Ftexture, bst, vio );
			bst = NULL;
	
		break;


	case 'M':	// sound
			vim_read_sound_block( vim, bst, vio );
		break;

#ifndef __SYMBIAN32__
	case 'F':	// frame priority
			vim_read_bframe_block( vim, bst, vio );
		break;
#endif



#ifdef IMAGE_BACKGROUND
	case 'J':
		vim->bim->im = vim_read_image( bst, vio );

		vim->bim->row = IMAGE_ROW(vim->bim->im);
		vim->bim->column = IMAGE_COLUMN(vim->bim->im);

		vim->row = vim->bim->row;
		vim->column = vim->bim->column;
	    break;
#endif

	}


	vio->stat++;

	if( bst != NULL )	bst_free( bst );

	return( vio->stat );
}


void vim_decoding_flags( bst_type *bst, u_char head[], int len )
{
int	i;
u_char flag;
			
	head[0] = bst_get_align_num( bst, 1 );
	for( i = 1, flag = head[0] ; (flag & 0x80) ; i++ ){

		flag = bst_get_align_num( bst, 1 );
		if( i < len )	head[i] = flag;
	}

	for( ; i < len ; i++ )	head[i] = 0;
}




static void
vim_read_sound_block( vimage_type *vim, bst_type *bst, vimIo_type *vio )
{
#ifndef _VIM_NAUDIO_
sound_type	*sound;

	sound = sound_read( bst );

	if( sound->Fcharcter == 1 )
		vim->sound = sound;
	else	
		vim->asound[ vim->sound_no++] = sound;	
#endif //  _VIM_NAUDIO_
}


static void
vim_read_bframe_block( vimage_type *vim, bst_type *bst, vimIo_type *vio )
{
	vim->bframeL = bframeL_decoding( bst );
}

