/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/*****************************
 ***	VimageDecodingFile.c   ***
 *****************************/
#include	<string.h>


#include	"Uigp/igp.h"
#include	"Ubst/BstType.h"

#include	"AgpTool/AgpTool.h"
#include	"Coding/VimVersion.h"

#include	"VimageType/VimageType.h"
#include	"VimCoding.h"
#include	"VimCodingIn.h"


#include	"VimIo.h"
#include	"VimageType/VimHeader.h"




static void	vim_decoding_defualt_animation( vimage_type *vim );



int
vim_decoding_version_from_file( char *file, int *version )
{
vimIo_type	*vio;
int	ret;	

	if( (vio = vimIo_openR_file( file ) ) == NULL )	return( -1 );


	ret = vimIo_get_version( vio, version  );

	vimIo_destroy( vio );
 

	return( ret );
}




int 
vim_decoding_from_file( vimage_type *avim[], int vim_no, vimHeader_type **vh,
				int Ftexture, int Faux, char *file )
{
vimIo_type	*vio;
int	no;



	if( (vio = vimIo_openR_file( file ) ) == NULL )	return( -1 );



	no = vim_decoding( avim, vim_no, vh, Ftexture, Faux, vio );


	vimIo_destroy( vio );

	return( no );
}






int 
vim_decoding_from_buffer( vimage_type *avim[], int vim_no, vimHeader_type **vh,
				int Ftexture, int Faux, char *data, int bytes )
{
vimIo_type	*vio;
int	no;

	

	if( (vio = vimIo_openR_buffer( data, bytes, 0 ) ) == NULL )	return( -1 );



	no = vim_decoding( avim, vim_no, vh, Ftexture, Faux, vio );


	vimIo_destroy( vio );

	return( no );
}



int 
vim_decoding( vimage_type *avim[], int vim_no, vimHeader_type **vh,
				int Ftexture, int Faux, vimIo_type *vio )
{
vimage_type	*vim;
int	version,	head;
bst_type	*bst;

int	no,	block;
int	i;


	if( ( vimIo_get_version( vio, &version )) <= 0 )	return( -1 );


	if( vio->version <= 0x0005 ){
#ifdef _ST
		return( -1 );
#else
		return( vim_decoding005( avim, vim_no, vh, Ftexture, Faux, vio ) );		
#endif
	}



	if( vh != NULL )	*vh = NULL;

	no = block = 0;
	vim = NULL;

	while( (bst = vimIo_cut_data( vio, &head )) != NULL ){


		if( head == 'H' ){
			if( vh != NULL ){
				*vh = vim_decoding_header( bst );
				(*vh)->version = vio->version;
			}

			continue;
		}


		if( head == 'S' && block > 0 ){
			if( no >= vim_no )	break;
			vim = NULL;
		}


		if( vim == NULL ){
			vim = avim[no++] = vimage_alloc();
			block = 0;
		}


	
		vim_decoding_block( vim, Ftexture, head, bst, vio );

		block++;
	}


	// animation and faux
	for( i = 0 ; i < no ; i++ ){
		vim = avim[i];
/*
		vim_retrieve_texture( vim, Ftexture, vio );

		vim_decoding_defualt_animation( vim );
*/
		vim_decoding_final( vim, Ftexture, vio );

		if( Faux == 1 ){
			vimage_aux( vim );

			vimage_anim_set_aux( vim );
		}
	}


	return( no );
}


void
vim_decoding_final( vimage_type *vim, int Ftexture, vimIo_type *vio )
{
	vim_retrieve_texture( vim, Ftexture, vio );

	vim_decoding_defualt_animation( vim );
}



static void
vim_decoding_defualt_animation( vimage_type *vim )
{
cast_type	*cast;
int	i;
	if( vim->tcp == NULL ){
		vim->tcp = tcpL_alloc();
		tcpL_clear( vim->tcp, 2, vim->frame );
	}

	for( i = 0 ; i < vim->no ; i++ ){
		cast = vim->cast[i];

		if( cast->ocp == NULL )
			cast_anim_clear( cast, vim->frame );
	}
}
