/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/******************
 ***	VimIo.c	***
 ******************/
#include	<stdio.h>
#include	<math.h>
#include	<string.h>


#include	"Uigp/igp.h"
#include	"Ubst/BstType.h"

#include	"VimageType/VimageType.h"
#include        "Coding/VimVersion.h"

#include	"VimIo.h"


vimIo_type *
vimIo_openR_file( char *file )
{
FILE	*fp;
vimIo_type	*vio;

	if( (fp = fopen( file, "rb" )) == NULL )	return( NULL );


	vio = vimIo_alloc();

	vio->fp = fp;
	vimIo_attach_filename( vio, file );

	return( vio );
}


vimIo_type * 
vimIo_openR_buffer( char *data, int len, int Fdata )
{
vimIo_type	*vio;

	vio = vimIo_alloc();

	if( Fdata == 1 ){
		vio->data = (char *)malloc( len );
		memcpy(&vio->data[vio->available_data], data, len);
	}
	else	{
		vio->data = data;
		vio->available_data = len;
	}


	vio->available_data = len;
	vio->data_i = 0;

	vio->Fdata = Fdata;

	return( vio );
}


vimIo_type *
vimIo_openW_file( char *file )
{
FILE	*fp;
vimIo_type	*vio;

	if( (fp = fopen( file, "wb" )) == NULL )	return( NULL );


	vio = vimIo_alloc();

	vio->fp = fp;
	vimIo_attach_filename( vio, file );

	return( vio );
}



vimIo_type	*
vimIo_alloc( )
{
vimIo_type	*vio;

	vio = (vimIo_type *)malloc( sizeof(vimIo_type));


//	strcpy( vio->config_dir, "./" );

	vio->fp = NULL;
	vio->Ffile = 0;

	vio->version = 0;

	vio->available_data = -1;
	vio->data = NULL;
	vio->cast_no = -1;

	vio->ve = NULL;
	vio->stat = 0;

	return( vio );
}


void
vimIo_attach_filename( vimIo_type *vio, char *file )
{
	vio->Ffile = 1;
	strcpy( vio->file, file );
}


void
vimIo_free( vimIo_type	*vio )
{
vimE_type	*ve;

	while( ( ve = vio->ve) != NULL ){
		vio->ve = vio->ve->next;

		if( ve->name != NULL )	free( ve->name );

		if( ve->bst != NULL )	bst_free( ve->bst );
		free( ve );
	}

	if( vio->data != NULL )	free( vio->data );
	free( vio );
}


int
vimIo_is_in( vimIo_type *vio, char head, char *name, int type )
{
vimE_type      *ve;
 
        for( ve = vio->ve ; ve != NULL ; ve = ve->next )
                if( ve->head == head && ve->type == type &&
					strcmp( ve->name, name ) == 0 )
			return( 1 );
 
        return( 0 );
}

 
 

vimE_type *
vimIo_get( vimIo_type *vio, char head, char *name )
{
vimE_type	*ve;
 
	for( ve = vio->ve ; ve != NULL ; ve = ve->next )
		if( ve->head == head && strcmp( ve->name, name ) == 0 ){
			bst_rewind( ve->bst );
			return( ve );
		}
 
	return( NULL );
}
 
 
void
vimIo_add( vimIo_type *vio, char *name, int type,
			int byte0, char head, bst_type *bst )
{
vimE_type      *ve;
 
	ve = (vimE_type *)malloc( sizeof(vimE_type) );

	ve->name = (char *)malloc( strlen( name) + 1 );
	strcpy( ve->name, name );

	ve->type = type;

	ve->byte0 = byte0;
	ve->head = head;
	ve->bst = bst;
 

	ve->next = vio->ve;
	vio->ve = ve;
}



void 
vimIo_append_data( vimIo_type *vio, char *data, int len )
{
	if( vio->data == NULL ){
#ifndef PLAMOS
//		vio->data_N = MAX( 100000, len );
		vio->data_N = len+1000;
#else
		vio->data_N = len+1000;
#endif

		vio->data = (char *)malloc( vio->data_N );
		vio->available_data = 0;
		vio->data_i = 0;
	}


	if( vio->data_i > 0 ){
		memcpy((void*)&vio->data[0],
		(void*)&vio->data[vio->data_i], vio->available_data - vio->data_i );
		vio->available_data -= vio->data_i;
		vio->data_i = 0;
	}

	if( vio->available_data + len >= vio->data_N ){
		vio->data_N = vio->available_data + len;
		vio->data = (char *)realloc( vio->data, vio->data_N );
	}

	memcpy(&vio->data[vio->available_data], data, len);

	vio->available_data +=len;
}



void 
vimIo_set_data( vimIo_type *vio, char *buffer, int len, int Fdata )
{
	if( Fdata == 1 ){
		if( vio->data == NULL ){
			vio->data = (char *)malloc( len );
			vio->available_data = 0;
				memcpy(&vio->data[vio->available_data], buffer, len);
			vio->available_data +=len;
		}
	}
	else	{
		vio->data = buffer;
		vio->available_data =len;
	}

}


void
vimIo_destroy( vimIo_type *vio )
{
vimE_type	*ve;

	while( ( ve = vio->ve) != NULL ){
		vio->ve = vio->ve->next;

		if( ve->name != NULL )	free( ve->name );

		if( ve->bst != NULL )	bst_free( ve->bst );
		free( ve );
	}


	if( vio->fp != NULL )	fclose( vio->fp );

	if( vio->Fdata == 1 && vio->data != NULL )	free( vio->data );

	free( vio );
}




bst_type *
vimIo_cut_data( vimIo_type *vio, int *head )
{
bst_type	*bst;
u_char	*data;
int	bytes;

	if( vio->fp != NULL ){
		vio->byte0 = ftell( vio->fp );

		if( (*head = getc( vio->fp )) == 0 )	return( NULL );

		bytes = gp_fread_num( vio->fp, 3 );
		bst = bst_read( vio->fp, bytes );

		if( bst != NULL && bst->byte_no < bytes ){
			bst_destroy( bst );
			return( NULL );
		}


		return( bst );
	}


	*head = -1;
	if( vio->available_data - vio->data_i < 1 )	return( NULL );

	*head = vio->data[vio->data_i];
 
	if( *head == 0 )	return( NULL );

 
	if( vio->available_data - vio->data_i < 4 )
		return( NULL );
 
 
	data = (u_char *)vio->data;
	bytes = (data[vio->data_i+ 1]*256 + data[vio->data_i+ 2])*256 + data[vio->data_i+ 3];
 
	if(bytes +4 > vio->available_data - vio->data_i )
		return( NULL );
 


	bst = bst_alloc( bytes );
 
	bst->byte_no = bytes;
	memcpy( bst->data, &vio->data[vio->data_i+4], bytes );
 
	bst_rewind( bst ); 

	vio->data_i += bytes + 4;

	vio->byte0 = -1;
	return( bst );
}





int
vimIo_get_version( vimIo_type *vio, int *version )
{
u_char	*data;


	if( vio->fp != NULL ){
		if( getc(vio->fp) != 'V' )   return( -1 );
 
		vio->version = gp_fread_num( vio->fp, 2 );
		if( version != NULL )	*version = vio->version;


		if( vio->version < VIM_VERSION0 )    return( -3 );
 
		if( vio->version > VIM_VERSION )    return( -2 );

//		if( version != VIM_VERSION )    return( -1 );


		return( 1 );
	}


	if( vio->available_data < 3 )	return( 0 );

	if( vio->data[0] != 'V' )	return( -1 );

 
	data = (u_char *)vio->data;
	vio->version = data[1]*256 + data[2];
	if( version != NULL )	*version = vio->version;



	if( vio->version < VIM_VERSION0 )    return( -1 );
 
	if( vio->version > VIM_VERSION )    return( -2 );

//	if( version != VIM_VERSION )	return( -1 );


	vio->data_i = 3;

//	vio->version = *ersion;

	return( 1 );
}
 



/* write vim file */
int
vimIo_write_version( vimIo_type *vio )
{
	putc( 'V', vio->fp );
	gp_fwrite_num( vio->fp, VIM_VERSION, 2 );

	return( 1 );
}


int
vimIo_write_block( vimIo_type *vio, int head, bst_type *bst )
{
int	bytes;



	putc( head, vio->fp );

	if( head == 0 )	return( 1 );

	bytes = MAX( BST_BYTES( bst), bst->byte_no );
	gp_fwrite_num( vio->fp, bytes, 3 );
	fwrite( bst->data, 1, bytes, vio->fp );
 
 
	return( bytes + 4 );
}

int
vimIo_write_blockD( vimIo_type *vio, int head, char *data, int bytes )
{
	putc( head, vio->fp );
	gp_fwrite_num( vio->fp, bytes, 3 );
	fwrite( data, 1, bytes, vio->fp );

	return( bytes + 4 );
}
 

