/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/************************
 ***	CastzEdit.c   ***
 ************************/
#include	"VimageType/VimageType.h"

#include	"AgpTool/AgpTool.h"
#include	"AgpLayer/AgpLayer.h"

gp_array *	agp_move_layer( gp_array *sagp, int iLayer, vec2d *dp );


int
cast_edit_reposition_layer( cast_type *cast, int iLayer, vec2d *dp )
{
vec2d	vp,	dv;

	if( cast->oim->agp_bst != NULL ){
		free( cast->oim->agp_bst );
		cast->oim->agp_bst = NULL;
	}

	if( cast->skl->jnt != NULL )
		view_pixel2vp( cast->oim->agp->ad->vw, &cast->skl->jnt->ctr, &vp );

	cast->oim->agp = agp_reposition_layer( cast->oim->agp, iLayer, dp );


	// the some movment to the skelton
	if( cast->skl->jnt != NULL ){
		view_vp2pixel( cast->oim->agp->ad->vw, &vp, &dv );
		dv.x =  dv.x - cast->skl->jnt->ctr.x;
		dv.y =  dv.y - cast->skl->jnt->ctr.y;
		skl_shift( cast->skl, dv.x, dv.y );
		skl_shift( cast->oim->skl, dv.x, dv.y );
	}


	cast->scale = cast->oim->agp->row;

	return( 1 );
}



int
cast_edit_add_layer( cast_type *cast, gp_array *agp ) 
{
vec2d	vp,	dv;

	if( cast->oim->agp_bst != NULL ){
		free( cast->oim->agp_bst );
		cast->oim->agp_bst = NULL;
	}

	if( cast->skl->jnt != NULL )
		view_pixel2vp( cast->oim->agp->ad->vw, &cast->skl->jnt->ctr, &vp );

	cast->oim->agp = agp_make_union2( cast->oim->agp, agp );
//	cast->scale = cast->oim->agp->row;

	if( cast->skl->jnt != NULL ){
		view_vp2pixel( cast->oim->agp->ad->vw, &vp, &dv );
		dv.x =  dv.x - cast->skl->jnt->ctr.x;
		dv.y =  dv.y - cast->skl->jnt->ctr.y;
		skl_shift( cast->skl, dv.x, dv.y );
		skl_shift( cast->oim->skl, dv.x, dv.y );
	}

	return( 1 );
}



static void	agp_set_view_position( gp_array *oagp, int oiLayer, gp_array *agp, int iLayer );


void
cast_edit_replace_layer( cast_type *cast, int iLayer, gp_array *agp )
{
layer_type *layer;
int	i;

	if( cast->oim->agp_bst != NULL ){
		free( cast->oim->agp_bst );
		cast->oim->agp_bst = NULL;
	}

	agp_set_view_position( cast->oim->agp, iLayer, agp, 0 );

	if( cast->oim->agp->layer_no == 1 ){
		agp_destroy( cast->oim->agp );
		cast->oim->agp = agp;
		return;
	}


	agp_layer_cut( cast->oim->agp, cast->oim->agp->layer[iLayer]->Mbr );


	cast_edit_add_layer( cast, agp );

	layer = cast->oim->agp->layer[ cast->oim->agp->layer_no-1];
	for( i = cast->oim->agp->layer_no-1 ; i > iLayer; i-- )
		cast->oim->agp->layer[i] = cast->oim->agp->layer[i-1];

	cast->oim->agp->layer[iLayer] = layer;
}

static void
agp_set_view_position( gp_array *oagp, int oiLayer, gp_array *agp, int iLayer )
{
box2d	b,	ob;
vec2d	op,	ovp,	p,	vp;

	
	agp_layer_box( oagp, oagp->layer[oiLayer]->Mbr, &ob );
	op.x = 0.5*(ob.x1 + ob.x0);
	op.y = 0.5*(ob.y1 + ob.y0);
	view_pixel2vp( oagp->ad->vw, &op, &ovp );


	agp_layer_box( agp, agp->layer[iLayer]->Mbr, &b );
	p.x = 0.5*(b.x1 + b.x0);
	p.y = 0.5*(b.y1 + b.y0);
	view_pixel2vp( agp->ad->vw, &p, &vp );


	agp->ad->vw->uv.x0 += ( ovp.x - vp.x );
	agp->ad->vw->uv.x1 += ( ovp.x - vp.x );

	agp->ad->vw->uv.y0 += ( ovp.y - vp.y );
	agp->ad->vw->uv.y1 += ( ovp.y - vp.y );

	view_set_pixel( agp->ad->vw, agp->ad->vw->Vpixel, agp->ad->vw->Upixel );
}


void
cast_edit_replace_layerA( cast_type *cast, int iLayer, gp_array *agp )
{
layer_type *layer;
int	i;

	if( cast->oim->agp_bst != NULL ){
		free( cast->oim->agp_bst );
		cast->oim->agp_bst = NULL;
	}

//	agp_set_view_position( cast->oim->agp, iLayer, agp, 0 );

	if( cast->oim->agp->layer_no == 1 ){
		agp_destroy( cast->oim->agp );
		cast->oim->agp = agp;
		return;
	}


	agp_layer_cut( cast->oim->agp, cast->oim->agp->layer[iLayer]->Mbr );


	cast_edit_add_layer( cast, agp );

	layer = cast->oim->agp->layer[ cast->oim->agp->layer_no-1];
	for( i = cast->oim->agp->layer_no-1 ; i > iLayer; i-- )
		cast->oim->agp->layer[i] = cast->oim->agp->layer[i-1];

	cast->oim->agp->layer[iLayer] = layer;
}