/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	OpropertyTool.c   ***
 ****************************/
#include	<stdio.h>
#include	<string.h>
#include	<math.h>

#include	"Uigp/igp.h"


#include	"VimageType/CastType.h"
#include	"VimageType/VimageType.h"



cast_type *
cast_alloc()
{
cast_type	*cast;

	cast = (cast_type *)malloc( sizeof(cast_type) );

	cast->type = CAST_REGULAR;

	cast->oim = NULL;

	cast->skl = NULL;

	cast->ocp = NULL;

//	cast->Mproperty = 0;

//	cast->rcp = NULL;

//	cast->url = NULL;

	cast->Moveable = 0;

	cast->Flipable = 0;

	cast->text = NULL;

	cast->inL = NULL;

	cast->flip = 0;

	cast->Flock = 0;

	cast->Fshow = 1;
	
	memset(cast->name,0,64);


	// Aux
	cast->Rcp = NULL;
	
	return( cast );
}



cast_type * cast_copy(cast_type *origCast)
{
	cast_type *cast;
	cast = cast_alloc();

	cast->type = origCast->type;

	cast->oim = oimage_make_copy(origCast->oim);

	if ( origCast->skl != NULL )
	{
		cast->skl = skl_make_copy(origCast->skl);
		skl_topology(cast->skl);
	}
	
	if ( origCast->ocp != NULL )
		cast->ocp = ocpL_copy(origCast->ocp);

/*
	cast->Mproperty = origCast->Mproperty;
	
	if ( origCast->rcp != NULL )
		cast->rcp = ocpL_copy(origCast->rcp);

#ifdef CE
	if ( origCast->url )
		cast->url = _strdup(origCast->url) ;	
#else
	if ( origCast->url )
		cast->url = strdup(origCast->url) ;	
#endif // CE
*/

	cast->text = NULL;// need to be copied

	cast->Flock = origCast->Flock;
	cast->Fshow = origCast->Fshow;
	cast->scale = origCast->scale;

	strncpy(cast->name,origCast->name,64);
	return cast;
}



cast_type *
cast_create_from_agp( gp_array *agp, char *name )
{
cast_type	*cast;
oimage_type	*oim;

	cast = cast_alloc();
	oim = cast->oim = oimage_alloc();
 
	oim->agp  = agp;
 

	cast->skl = skl_alloc();
	oim->skl = skl_alloc();
	oim->skl->row = agp->row;
	oim->skl->column = agp->column;
 
	oim->Faux = 0;

	cast->scale = agp->row;

	strcpy( cast->name, name );

	return( cast );
}






void
cast_destroy( cast_type *cast )
{
	if( cast->skl != NULL )
		 skl_free( cast->skl );

	if( cast->oim != NULL )
		 oimage_free( cast->oim );


	if( cast->ocp != NULL )
		ocpL_destroy( cast->ocp );



	/* Property */
/*
	if( cast->rcp != NULL )
		ocpL_destroy( cast->rcp ); 


	if( cast->url != NULL )
		free( cast->url );
*/

	if( cast->text != NULL )
		free( cast->text );


	if( cast->inL != NULL )
		interactionL_destroy( cast->inL );


	free( cast );
}


void
cast_anim_free( cast_type *cast )
{
	if( cast->ocp != NULL ){
		ocpL_destroy( cast->ocp );
		cast->ocp = NULL;
	}


/*
	if( cast->rcp != NULL ){
		ocpL_destroy( cast->rcp ); 
		 cast->rcp = NULL;
	}


	if( cast->url != NULL ){
		free( cast->url );
		cast->url = NULL;
	}
*/
}



void
cast_anim_clear( cast_type *cast, int frame )
{
/* 
	cast->Mproperty = MC_NULL;

	if( cast->rcp != NULL )
	{
		ocpL_destroy( cast->rcp ); 
		cast->rcp = NULL;

	}

	if( cast->url != NULL )
	{
		free( cast->url );
		cast->url = NULL;
	}
*/
 

	if( cast->ocp != NULL )	
	{
		ocpL_destroy( cast->ocp );
		cast->ocp = NULL;
	}

	cast->ocp = ocpL_alloc2(  cast->skl, 0, frame-1 );
}



void
cast_set_agp( cast_type *cast, gp_array *agp )
{
oimage_type     *oim;
 
    if( cast->oim == NULL )
		cast->oim = oimage_alloc();

    oim = cast->oim;


    strncpy( oim->name, cast->name, 16 );


    if( oim->agp != NULL )  agp_destroy( oim->agp );
    oim->agp  = agp;
 
 
	if( oim->skl != NULL )	skl_free( oim->skl );
        oim->skl = skl_make_copy( cast->skl );
 
	oim->Faux = 0;
} 


cast_type *
cast_create( gp_array *agp, char *fname )
{
cast_type	*cast;
oimage_type	*oim;

	cast = cast_alloc();
	oim = cast->oim = oimage_alloc();
 
	oim->agp  = agp;
 
	oim->skl = skl_alloc();
	oim->skl->row = agp->row;
	oim->skl->column = agp->column;
 
	oim->Faux = 0;


	cast->scale = agp->row;


	if( fname != NULL && fname[0] != 0 )
		gp_filename_extract_name( fname, cast->name, CAST_NAME_LEN );
	else	strcpy( cast->name, "Untitled" );


	cast->skl = skl_make_copy( oim->skl );

	return( cast );
}

int
cast_rescale( cast_type *cast, int scale )
{
	if( cast->scale == scale )	return( 0 );

	oimage_rescale( cast->oim, (float)scale / (float)cast->scale );

	cast->scale = scale;

	return( 1 );
}
