/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	OimageCp.c	***
 **************************/
#include	<stdio.h>
#include	<math.h>

#include	"Uigp/igp.h"


#include	"GpType/GpType.h"


#include	"VimageType/VimageType.h"




void
oimage_cp_set_dp_all( oimage_type *oim, ocp_list *ocp )
{
ocp_type	*cp;

	for( cp = ocp->cp ; cp != NULL ; cp = cp->next )
		oimage_cp_set_dp( oim, cp );
}


void
oimage_cp_set_dp( oimage_type *oim, ocp_type *cp )
{
vec3d   p30,    p31;
vec2d   p0,     p1,	vp0,    vp1;

	vp0.x = vp0.y = 0;
	view_vp2pixel( oim->vw, &vp0, &p0 );
	view_pixel2plane_xyz( oim->vw, &p0, &oim->P, &oim->N, &p30 );


	p31.x = p30.x + cp->c.x *oim->U.x + cp->c.y* oim->V.x;
	p31.y = p30.y + cp->c.x *oim->U.y + cp->c.y* oim->V.y;
	p31.z = p30.z + cp->c.x* oim->U.z + cp->c.y* oim->V.z;

	view_project_point( oim->vw, &p31, &vp1 );
	view_vp2pixel( oim->vw, &vp1, &p1 );

	cp->dp.x = p1.x - p0.x;
	cp->dp.y = p1.y - p0.y;
}


void
oimage_cp_set_c( oimage_type *oim, vec2d *dp, config3d *c )
{
vec3d   p30,    p31;
vec2d   p0,	p1,	vp0;

	vp0.x = vp0.y = 0;
	view_vp2pixel( oim->vw, &vp0, &p0 );
	view_pixel2plane_xyz( oim->vw, &p0, &oim->P, &oim->N, &p30 );

	p1.x = dp->x + p0.x;
	p1.y = dp->y + p0.y;
	view_pixel2plane_xyz( oim->vw, &p1, &oim->P, &oim->N, &p31 );

	p31.x -= p30.x;
	p31.y -= p30.y;
	p31.z -= p30.z;

	c->x = VEC3D_INNER( oim->vw->U, p31 );
	c->y = VEC3D_INNER( oim->vw->V, p31 );
}



void
oimage_kf_shiftD( oimage_type *oim, vec2d *p0, vec2d *p1, vec2d *dp )
{
vec3d   p30,    p31,    dp3;
 
 
	view_pixel2plane_xyz( oim->vw, p0, &oim->P, &oim->N, &p30 );
	view_pixel2plane_xyz( oim->vw, p1, &oim->P, &oim->N, &p31 );
 
	dp3.x = p31.x - p30.x;
	dp3.y = p31.y - p30.y;
	dp3.z = p31.z - p30.z;
 
 
	dp->x = VEC3D_INNER( oim->U, dp3 );
	dp->y = VEC3D_INNER( oim->V, dp3 );
}



void
oimage_kf_shift( oimage_type *oim, ocp_type *acp,
			view_type *vw, vec2d *p0, vec2d *p1, int Fshot )
{
ocp_type	*cp;
vec3d   p30,    p31,    dp3;
vec2d   dp;
 
 
	view_pixel2plane_xyz( vw, p0, &oim->P, &oim->N, &p30 );
	view_pixel2plane_xyz( vw, p1, &oim->P, &oim->N, &p31 );
 
	dp3.x = p31.x - p30.x;
	dp3.y = p31.y - p30.y;
	dp3.z = p31.z - p30.z;
 
 
	dp.x = VEC3D_INNER( oim->U, dp3 );
	dp.y = VEC3D_INNER( oim->V, dp3 );
 
	acp->c.x += dp.x;
	acp->c.y += dp.y;
 
	acp->dp.x += p1->x - p0->x;
	acp->dp.y += p1->y - p0->y;
 

	if( Fshot == 0 || acp->Fshot == SHOT_END )	return;

	for( cp = acp->next ; cp != NULL ; cp = cp->next ){
		cp->c.x += dp.x;
		cp->c.y += dp.y;
 
		cp->dp.x += p1->x - p0->x;
		cp->dp.y += p1->y - p0->y;
 
		if( cp->Fshot == SHOT_END )     break;
	}
}



void
oimage_kf_shift_selection( oimage_type *oim, ocp_type *acp,
			view_type *vw, vec2d *p0, vec2d *p1, int endSelection )
{
ocp_type	*cp;
vec3d   p30,    p31,    dp3;
vec2d   dp;
 
 
	view_pixel2plane_xyz( vw, p0, &oim->P, &oim->N, &p30 );
	view_pixel2plane_xyz( vw, p1, &oim->P, &oim->N, &p31 );
 
	dp3.x = p31.x - p30.x;
	dp3.y = p31.y - p30.y;
	dp3.z = p31.z - p30.z;
 
 
	dp.x = VEC3D_INNER( oim->U, dp3 );
	dp.y = VEC3D_INNER( oim->V, dp3 );
 
	acp->c.x += dp.x;
	acp->c.y += dp.y;
 
	acp->dp.x += p1->x - p0->x;
	acp->dp.y += p1->y - p0->y;
 
	for( cp = acp->next ; cp != NULL && cp->fno <= endSelection ; cp = cp->next ){
		cp->c.x += dp.x;
		cp->c.y += dp.y;
 
		cp->dp.x += p1->x - p0->x;
		cp->dp.y += p1->y - p0->y;
 
		if( cp->Fshot == SHOT_END )     break;
	}
}


void
oimage_kf_normal_shift_ocpL( oimage_type *oim, ocp_list *ocp )
{
ocp_type	*cp;

	for( cp = ocp->cp ; cp != NULL ; cp = cp->next )
		oimage_kf_normal_shift( oim, cp );
}


void
oimage_kf_normal_shift( oimage_type *oim, ocp_type *acp )
{
	if( skl_mass_point( acp->skl ) == 0 )	return;

	if( oim->skl->mp.x == acp->skl->mp.x &&
			oim->skl->mp.y == acp->skl->mp.y )
		return;

	oimage_kf_shift( oim, acp, oim->vw,
				&oim->skl->mp, &acp->skl->mp, 0 );

	skl_shift( acp->skl, oim->skl->mp.x - acp->skl->mp.x,
			oim->skl->mp.y - acp->skl->mp.y );
}
