/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>
#include	<math.h>


#include	"Uigp/igp.h"

#include	"VimageType/OimageType.h"
#include	"Rendering/Render.h"
#include	"AgpTool/AgpTool.h"
#include	"N3Prime/N3Prime.h"
#include	"AgpLayer/AgpLayer.h"
#include	"Decoding/Decoding.h"


static gp_array *	oim_make_agp_copy( oimage_type *oim );


oimage_type *
oimage_alloc()
{
oimage_type *oim;

	oim = (oimage_type *)malloc( sizeof(oimage_type) );

	oim->agp = NULL;
	oim->agpF = NULL;

	oim->icp = oim->icpF = NULL;

	oim->skl = NULL;

	oim->Faux = 0;


	oim->Falive = 0;

	oim->vw = NULL;
	oim->br0 = 0;
//	config3d_init( &oim->cp0 );
	config3d_init( &oim->Gcp );
	config3d_init( &oim->Bcp );


	oim->agp_bst = NULL;

#ifndef _ST
	/* ****************** OimageR2 **********************/
	oim->oimR2 = 0;
#endif// _ST

	return( oim );
}



void
oimage_free( oimage_type *oim )
{

	if( oim->Faux == 1 && oim->agp != NULL ){
		oimage_skl_aux_clear( oim );
		render_agp_aux_free( oim->agp );
	}

	if( oim->skl != NULL ){
		skl_free( oim->skl );
		oim->skl = NULL;
	}


	if( oim->agp != NULL ){
		agp_destroy( oim->agp );
		oim->agp = NULL;
	}


	if( oim->icp != NULL ){
		ocp_free( oim->icp );
		oim->icp = NULL;
	}



	if( oim->agpF != NULL ){
		render_agp_aux_free( oim->agpF );
		agp_destroy( oim->agpF );
		oim->agpF = NULL;

		ocp_free( oim->icpF );
		oim->icpF = NULL;
	}


	if( oim->agp_bst != NULL ){
		bst_destroy( oim->agp_bst );
		oim->agp_bst = NULL;
	} 

#ifndef _ST
	/* ****************** OimageR2 **********************/
	oimage_R2_destroy( oim );
#endif

	free( oim );
}




void
oimage_flipV( oimage_type *oim )
{
gp_array	*agp;

	agp = agp_flipV( oim->agp );

	agp_destroy( oim->agp );

	oim->agp = agp;

	skl_flipV( oim->skl, oim->agp->columnR );
}



oimage_type *
oimage_make_copy( oimage_type *oim )
{
oimage_type *coim;

	coim = oimage_alloc();

	*coim = *oim;
	coim->Faux = 0;
	coim->agp = coim->agpF = NULL;


	coim->skl = skl_make_copy( oim->skl );
	if( oim->Faux == 1 )
		skl_br_add( coim->skl, -oim->br0 );

/*
	coim->agp = agp_alloc();

	if ( oim->agp )
		agp_copy( oim->agp, coim->agp,oim->agp->row, 
			oim->agp->column, oim->agp->bgp );
*/
	coim->scale = oim->scale;

	if( oim->agp_bst != NULL ){
		coim->agp_bst = bst_make_copy( oim->agp_bst );
		coim->agp = decoding_agp_from_bst( coim->agp_bst );
	}
	else
		coim->agp = oim_make_agp_copy( oim );

	return( coim );
}

static gp_array *
oim_make_agp_copy( oimage_type *oim )
{
gp_array	*agp;

	if( oim->Faux == 1 ){
		oimage_skl_aux_clear( oim );
		render_agp_aux_free( oim->agp );
		n3_prime_free( oim->agp );
	}

	agp = agp_make_copy( oim->agp );

	if( oim->Faux == 1 ){
		n3_prime_agp( oim->agp );
		render_agp_aux( oim->agp );
		oimage_skl_aux( oim );

		agp_layer_add_const( agp, -oim->br0 );
		adepth_rescale( agp->ad, (float)agp->row / (float)oim->scale );
	}


	if( oim->agp->vf != NULL ){
		agp->vf = VIM_FILE_ALLOC();
		*agp->vf = *oim->agp->vf;
	}

	return( agp );
}