/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	OimageView.c    ***
 **************************/
#include	<math.h>

#include	"Uigp/igp.h"

#include	"VimageType/VimageType.h"


void
oimage_view( oimage_type *oim, config3d *cm, view_type *vw )
{
float	cosu,	sinu;
vec3d	U1,	V1,	N1;

	*vw = *oim->vw;



	/* rotate around vw->V */
	cosu = (float)cos( cm->u2*M_PI/180.0 );
	sinu = (float)sin( cm->u2*M_PI/180.0 );
	N1.x = sinu*vw->U.x + cosu * vw->N.x;
	N1.y = sinu*vw->U.y + cosu * vw->N.y;
	N1.z = sinu*vw->U.z + cosu * vw->N.z;

	U1.x = cosu*vw->U.x + -sinu * vw->N.x;
	U1.y = cosu*vw->U.y + -sinu * vw->N.y;
	U1.z = cosu*vw->U.z + -sinu * vw->N.z;



	/* rotate around U1 */
	cosu = (float)cos( cm->u3*M_PI/180.0 ); 
	sinu = (float)sin( cm->u3*M_PI/180.0 );

	vw->N.x = sinu*vw->V.x + cosu * N1.x;
	vw->N.y = sinu*vw->V.y + cosu * N1.y;
	vw->N.z = sinu*vw->V.z + cosu * N1.z;

	V1.x = cosu*vw->V.x + -sinu * N1.x;
	V1.y = cosu*vw->V.y + -sinu * N1.y;
	V1.z = cosu*vw->V.z + -sinu * N1.z;



	/* rotate around N1 */
	cosu = (float)cos( cm->u1*M_PI/180.0 ); 
	sinu = (float)sin( cm->u1*M_PI/180.0 );

	vw->V.x = sinu*U1.x + cosu * V1.x;
	vw->V.y = sinu*U1.y + cosu * V1.y;
	vw->V.z = sinu*U1.z + cosu * V1.z;

	vw->U.x = cosu*U1.x + -sinu * V1.x;
	vw->U.y = cosu*U1.y + -sinu * V1.y;
	vw->U.z = cosu*U1.z + -sinu * V1.z;




	vw->cp.x = oim->P.x + cm->x + oim->Vp.x*vw->U.x +
				oim->Vp.y*vw->V.x + oim->Vp.z*vw->N.x;
	vw->cp.y = oim->P.y + cm->y + oim->Vp.x*vw->U.y +
				oim->Vp.y*vw->V.y + oim->Vp.z*vw->N.y;
	vw->cp.z = oim->P.z + cm->z + oim->Vp.x*vw->U.z +
				oim->Vp.y*vw->V.z + oim->Vp.z*vw->N.z;



	vw->rp.x = vw->cp.x + vw->dis*vw->N.x;
	vw->rp.y = vw->cp.y + vw->dis*vw->N.y;
	vw->rp.z = vw->cp.z + vw->dis*vw->N.z;
}






void
oimage_view_cp( oimage_type *oim, config3d *cp0,
			config3d *cm, float u, float v, config3d *cp )
{
float	cosu,	sinu,	a;
vec3d	dp1;
vec3d	U1,	V1,	N1;
vec3d	U,	V,	N;
view_type	*vw;


	vw = oim->vw;



	/* rotate around vw.V */
	a = cp0->u2 *M_PI/180.0;
	cosu = (float)cos( a );
	sinu = (float)sin( a );
	N1.x = sinu*vw->U.x + cosu * vw->N.x;
	N1.y = sinu*vw->U.y + cosu * vw->N.y;
	N1.z = sinu*vw->U.z + cosu * vw->N.z;

	U1.x = cosu*vw->U.x + -sinu * vw->N.x;
	U1.y = cosu*vw->U.y + -sinu * vw->N.y;
	U1.z = cosu*vw->U.z + -sinu * vw->N.z;



	/* rotate around U1 */
	a = cp0->u3 *M_PI/180.0;
	cosu = (float)cos( a );
	sinu = (float)sin( a );

	N.x = sinu*vw->V.x + cosu * N1.x;
	N.y = sinu*vw->V.y + cosu * N1.y;
	N.z = sinu*vw->V.z + cosu * N1.z;

	V1.x = cosu*vw->V.x + -sinu * N1.x;
	V1.y = cosu*vw->V.y + -sinu * N1.y;
	V1.z = cosu*vw->V.z + -sinu * N1.z;



	/* rotate around N1 */
	a = cp0->u1 *M_PI/180.0;
	cosu = (float)cos( a );
	sinu = (float)sin( a );

	V.x = sinu*U1.x + cosu * V1.x;
	V.y = sinu*U1.y + cosu * V1.y;
	V.z = sinu*U1.z + cosu * V1.z;

	U.x = cosu*U1.x + -sinu * V1.x;
	U.y = cosu*U1.y + -sinu * V1.y;
	U.z = cosu*U1.z + -sinu * V1.z;




	dp1.x = u * U.x + v * V.x;
	dp1.y = u * U.y + v * V.y;
	dp1.z = u * U.z + v * V.z;


	cp->x = cp0->x + cm->x + dp1.x;
	cp->y = cp0->y + cm->y + dp1.y;
	cp->z = cp0->z + cm->z + dp1.z;

	cp->u1 = cp0->u1 + cm->u1;
	cp->u2 = cp0->u2 + cm->u2;
	cp->u3 = cp0->u3 + cm->u3;
}



void
oimage_view_N( oimage_type *oim, config3d *cm, vec3d *N )
{
float	cosu,	sinu;
vec3d	N1;
view_type	*vw;

	vw = oim->vw;



	/* rotate around vw->V */
	cosu = (float)cos( cm->u2*M_PI/180.0 );
	sinu = (float)sin( cm->u2*M_PI/180.0 );
	N1.x = sinu*vw->U.x + cosu * vw->N.x;
	N1.y = sinu*vw->U.y + cosu * vw->N.y;
	N1.z = sinu*vw->U.z + cosu * vw->N.z;


	/* rotate around U1 */
	cosu = (float)cos( cm->u3*M_PI/180.0 ); 
	sinu = (float)sin( cm->u3*M_PI/180.0 );

	N->x = sinu*vw->V.x + cosu * N1.x;
	N->y = sinu*vw->V.y + cosu * N1.y;
	N->z = sinu*vw->V.z + cosu * N1.z;
}
