/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	VimageAux.c	***
 **************************/

#include	<stdio.h>
#include	<math.h>


#include	"Uigp/igp.h"
#include	"AgpTool/AgpTool.h"
#include	"AgpDepth/AgpDepth.h"



#include	"N3Prime/N3Prime.h"


#include	"Rendering/Render.h"
#include	"AgpPrime/PrimeIn.h"

#include	"AgpLayer/AgpLayer.h"



#include	"VimageType/VimageType.h"



static void	bimage_aux( bimage_type *bim, int *layer );
static void	oimage_aux( oimage_type *oim, int scale,
					int Fnflip, int *layer0 );


//static void	oimage_view_init( oimage_type *oim );


static void	agp_depth_br_range( gp_array *agp, int *br0, int *br1 );

static void	agp_dpeth_add_bgp( gp_array *agp );



void 
vimage_aux( vimage_type *vim )
{
cast_type	*cast;
int	i,	br0;


	if( vim->bim == NULL )
		vim->bim = bimage_create( NULL,
				vim->row, vim->column, vim->color );

	bimage_aux( vim->bim, &br0 );


	vim->Mbr = vim->bim->Mbr;

	for( i = 0 ; i < vim->no ; i++ ){
		cast = vim->cast[i];

		oimage_aux( cast->oim, cast->scale,
				cast->Flipable, &br0 );

		vim->Mbr |= vim->cast[i]->oim->Mbr;
	}


	vim->layer_no = br0;

	if( vim->ragp == NULL ){
		vim->ragp = agp_alloc();
		vim->ragp->Fn3_prime = 1;
		vim->ragp->type = AGP_RENDER;
		vim->ragp->color_type = AGP_RGB;
	}

	vim->rbgp = 8;
#ifndef PDA
	if( vim->bim->agp != NULL )	vim->rbgp = vim->bim->agp->bgp;
#endif


	if( vim->expA == NULL )
		vim->expA = expAux_alloc();


/*
	if( vim->Avw == NULL )	vim->Avw = VIEW_ALLOC();
	if( vim->bim->agp != NULL )
		*vim->Avw = *vim->bim->agp->ad->vw;
	else	 view_create_noraml( vim->Avw, 1.0/512.0,
					vim->bim->row, vim->bim->column ); 
*/

	vimage_view_init( vim );	/* set vim->Avw */
}




static void 
bimage_aux( bimage_type *bim, int *layer )
{
adepth_type	*ad;
int	br0,	br1;

	if( bim->agp == NULL ){
		bim->Faux = 1;
		*layer = 0;
		return;
	}


	if( bim->agp->ad == NULL )
		agp_depth_default( bim->agp, 100 );


	if( bim->Faux == 0 ){
		ad = bim->agp->ad;

		agp_depth_range( bim->agp, &bim->depth0, &bim->depth1 );
#ifndef	PDA
		agp_border_add( bim->agp );

		agp_depth_smooth0x4( bim->agp );

		agp2prime( bim->agp );
#endif

		n3_prime_agp( bim->agp );

#ifndef SINGLE_BGP
		agp_dpeth_add_bgp( bim->agp );
#endif

		render_agp_aux( bim->agp );
		agp_pl_init( bim->agp );


		bim->row = bim->agp->row;
		bim->column = bim->agp->column;

		bim->Faux = 1;
	}



	agp_depth_br_range( bim->agp, &br0, &br1 );
	*layer = br1+1;

	bim->Mbr = bim->agp->ad->Mbr;
}






static void
oimage_aux( oimage_type *oim, int scale, int Flipable, int *layer0 )
{
//adepth_type	*ad;
int	br0,	br1;

	if( oim->Faux == 1 ){
		agp_depth_br_range( oim->agp, &br0, &br1 );
		*layer0 = br1 + 1;
		return;
	}


	oim->row = oim->agp->row;
	oim->column = oim->agp->column;


#ifndef	PDA
	agp_depth_smooth0x4( oim->agp );

	agp2prime( oim->agp );
#endif

	agp_depth_range( oim->agp, &oim->depth0, &oim->depth1 );
 

	oim->scale = scale;
	adepth_rescale( oim->agp->ad, scale / (float)oim->agp->row );


/*
	if( oim->FflipV == 1 )
		oimage_flipV( oim );
*/



	agp_depth_br_range( oim->agp, &br0, &br1 );
	oim->br0 = 0;
	if( br0 != *layer0 ){
		oim->br0 = *layer0 - br0;
		agp_layer_add_const( oim->agp, oim->br0 );

		skl_br_add( oim->skl, oim->br0 );
	}
	*layer0 = oim->br0 + br1 + 1;






/*
	ad = oim->agp->ad;
	ad->vw->cp.x += oim->cp0.x;
	ad->vw->cp.y += oim->cp0.y;
	ad->vw->cp.z += oim->cp0.z;

	ad->vw->rp.x += oim->cp0.x;
	ad->vw->rp.y += oim->cp0.y;
	ad->vw->rp.z += oim->cp0.z;

	ad->P.x += oim->cp0.x;
	ad->P.y += oim->cp0.y;
	ad->P.z += oim->cp0.z;
*/


	oimage_view_init( oim );



#ifdef	FLIP
	if( Flipable == 1  )
		oim->agpF = agp_flipH( oim->agp );
#endif



	n3_prime_agp( oim->agp );

#ifndef SINGLE_BGP
	agp_dpeth_add_bgp( oim->agp );
#endif


	render_agp_aux( oim->agp );
	agp_pl_init( oim->agp );


	oim->icp = ocp_alloc( oim->skl );




	/* preper agpF for rendering */
	if( oim->agpF != NULL ){
		n3_prime_agp( oim->agpF );

#ifndef SINGLE_BGP
		agp_dpeth_add_bgp( oim->agpF );
#endif


		render_agp_aux( oim->agpF );
		agp_pl_init( oim->agpF );


		oim->icpF = ocp_alloc( oim->skl );
		skl_flipV( oim->icpF->skl, oim->agp->columnR );
	}


	oimage_skl_aux( oim );



	oim->Faux = 1;
}

/*
void
oimage_auxR2( oimage_type *oim, int scale, int *layer0 )
{
adepth_type	*ad;

	if( oim->Faux == 1 ){
//		*layer0 = br1 + 1;
		return;
	}


	oim->row = oim->agp->row;
	oim->column = oim->agp->column;


	oim->scale = scale;
	adepth_rescale( oim->agp->ad, scale / (float)oim->agp->row );


	ad = oim->agp->ad;
	ad->vw->cp.x += oim->cp0.x;
	ad->vw->cp.y += oim->cp0.y;
	ad->vw->cp.z += oim->cp0.z;

	ad->vw->rp.x += oim->cp0.x;
	ad->vw->rp.y += oim->cp0.y;
	ad->vw->rp.z += oim->cp0.z;

	ad->P.x += oim->cp0.x;
	ad->P.y += oim->cp0.y;
	ad->P.z += oim->cp0.z;



	oimage_view_init( oim );


	oim->icp = ocp_alloc( oim->skl );


	oim->Faux = 1;
}

*/

void
oimage_view_init( oimage_type *oim )
{
view_type	*vw;
vec3d	p;
float	t;

	oim->vw = oim->agp->ad->vw;


	oim->Mbr = oim->agp->ad->Mbr;



	oim->P = oim->agp->ad->P;
	oim->N = oim->agp->ad->N;


	t = 1/VEC3D_NORM( oim->N );
	oim->N.x *= t;
	oim->N.y *= t;
	oim->N.z *= t;


 


	/* U, V plane vector */
	vw = oim->vw;

	t = VEC3D_INNER( oim->N, vw->V );
	oim->V.x = vw->V.x - t*oim->N.x;
	oim->V.y = vw->V.y - t*oim->N.y;
	oim->V.z = vw->V.z - t*oim->N.z;
	t = 1/VEC3D_NORM( oim->V);
	oim->V.x *= t;
	oim->V.y *= t;
	oim->V.z *= t;


	t = VEC3D_INNER( oim->N, vw->U );
	oim->U.x = vw->U.x - t*oim->N.x;
	oim->U.y = vw->U.y - t*oim->N.y;
	oim->U.z = vw->U.z - t*oim->N.z;
	t = 1/VEC3D_NORM( oim->U );
	oim->U.x *= t;
	oim->U.y *= t;
	oim->U.z *= t;


	p.x = vw->cp.x - oim->P.x;
	p.y = vw->cp.y - oim->P.y;
	p.z = vw->cp.z - oim->P.z;
 
	oim->Vp.x = VEC3D_INNER( p, vw->U );
	oim->Vp.y = VEC3D_INNER( p, vw->V );
	oim->Vp.z = VEC3D_INNER( p, vw->N );
}



void
oimage_rescale( oimage_type *oim, float scale )
{
//	if( oim->rescale == scale )	return;

//	adepth_rescale( oim->agp->ad, scale / oim->rescale );

//	oim->rescale = scale;
	adepth_rescale( oim->agp->ad, scale );
	oimage_view_init( oim ); 
}




static void
agp_depth_br_range( gp_array *agp, int *br0, int *br1 )
{
int	i;
mbr_type	mbr;

	*br0 = *br1 = -1;
	if( agp->ad == NULL )	return;

	mbr = agp->ad->Mbr;

	for( i = 0 ; mbr != 0 ; i++, mbr >>= 1 ){
		if( !( mbr & 1 ) )	continue;

		if( *br0 == -1 ){
			*br0 = *br1 = i;
			continue;
		}

		*br1 = i;
	}
}



#ifndef	SINGLE_BGP
static void
agp_dpeth_add_bgp( gp_array *agp )
{
gp_type	*gp;
bgp_type	*bgp,	*bgp0,	*bgp1,	*nbp,	**lbp,	bp;
int	i,	j,	flag,	n;

	for( i = 0 ; i < agp->Ni ; i++ )
	    for( j = 0 ; j < agp->Nj ; j++ ){

		gp = &agp->gp[i][j];


		flag = 0;

		for( bgp0 = gp->bgp; bgp0 != NULL ; ){

			bgp0->Sbr = 1;
			for( bgp = bgp0->p, n = 1 ; bgp != NULL &&
				bgp->br == bgp0->br ; bgp = bgp->p, n++ ){
				bgp->Sbr = 1;
			}

			bgp1 = bgp;

			if( n < 4 ){
				bgp0 = bgp1;
				flag |= 0x2;
				continue;
			}


			
			
			bp = *bgp0;
			bgp0->Sbr = 0;
			for( bgp = bgp0->p ; bgp != bgp1 ; bgp = bgp->p ){
				bp.ctr.x += bgp->ctr.x;
				bp.ctr.y += bgp->ctr.y;
				bp.val[SP_Y] += bgp->val[SP_Y];
				bp.val[SP_Q] += bgp->val[SP_Q];
				bp.val[SP_A] += bgp->val[SP_A];

				bp.depth += bgp->depth;

				bgp->Sbr = 0;
			}


			nbp = BGP_ALLOC();
			nbp->ctr.x = bp.ctr.x / n;
			nbp->ctr.y = bp.ctr.y / n;
			nbp->val[SP_Y] = bp.val[SP_Y] / n;
			nbp->val[SP_Q] = bp.val[SP_Q] / n;
			nbp->val[SP_A] = bp.val[SP_A] / n;
			nbp->depth = bp.depth / n;

			nbp->br = bp.br;
			nbp->Sbr = 2;

			nbp->p = gp->bgp;
			gp->bgp = nbp;

			flag |= 0x1;

			bgp0 = bgp1;
		}

		if( flag != 0x3 )	continue;


		bgp1 = NULL;
		lbp = &bgp1;

		for( bgp = gp->bgp ; bgp->p != NULL ; ){

			if( bgp->p->Sbr != 0 ){
				bgp = bgp->p;
				continue;
			}

			*lbp = bgp->p;
			lbp = &(*lbp)->p;

			bgp->p = bgp->p->p;
		}

		bgp->p = bgp1;
		*lbp = NULL;
	}
}
#endif
