/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***********************************
 ***    VimageConfiguration.c    ***
 ***********************************/
#include	<stdio.h>
#include	<math.h>

#include	"VimageType/VimageType.h"




static mbr_type	skl_Fcopy( skl_type *skl0, skl_type *skl );

static int	lnk_Fcopy( lnk_type *link0, vec2d *ctr, lnk_type *link );




void
vimage_configuration_init( vimage_type *vim )
{
oimage_type	*oim;
int	i;
 
	if( !CONFIG3D_IS_INIT( vim->Ccp ) ){
		config3d_init( &vim->Ccp );
		vim->Sbr = 0;
	}
	else    vim->Sbr = vim->bim->Mbr;
 
	for( i = 0 ; i < vim->no ; i++ ){
		oim = vim->cast[i]->oim;
 
		oim->Falive = 1;
 
		config3d_init( &oim->Bcp );
 
		config3d_init( &oim->icp->c );
 
		skl_copy( oim->skl, oim->icp->skl );
 
		skl_set_bes( oim->icp->skl );
	}
}




int
oimage_configuration_cp2d( oimage_type *oim, ocp_type *cp )
{
 
	oim->icp->c = cp->c;
 
	config3d_init( &oim->Bcp );
	oim->Bcp.x = cp->c.x;
	oim->Bcp.y = cp->c.y;
 
	skl_copy( cp->skl, oim->icp->skl );
 
	skl_set_bes( oim->icp->skl );

	return( 0 );
}






void
vimage_configuration( vimage_type *vim, int fno )
{
cast_type	*cast;
int	i;
mbr_type	sbr;

	if( fno <= vim->frame_i )	vimage_anim_rewind( vim );


	vim->Sbr = 0;

	for( i = 0 ; i < vim->no ; i++ ){
		cast = vim->cast[i];


		sbr = oimage_configuration( cast->oim, cast->ocp, fno, cast->Fshow );

		skl_set_bes( cast->oim->icp->skl );

		vim->Sbr |= sbr;
	}


//	if( tcp_trajectory_point( &vim->tcp->a_cp, fno,
//					&vim->Gcp, &vim->Ccp ) == 1 )
	if( tcp_configuration( vim->tcp, fno, &vim->Gcp, &vim->Ccp ) == 1 )
					
		vim->Sbr |= vim->bim->Mbr;
	else	vim->Sbr = 0;

	vim->frame_i = fno;
}



mbr_type
oimage_configuration( oimage_type *oim, ocp_list *ocp, int fno, int Fshow )
{
skl_type	*skl,	*skl0,	*skl1;
ocp_type	*cp0,	*cp1;
config3d	gc,	c0;
float	t;
mbr_type	sbr = 0,	tmp;

	if( ocp->cp == NULL ||Fshow == 0 ){
		oim->Falive = 0;
		return( 0 );
	}


	while( ocp->acp->fno + ocp->acp->dfno <= fno && ocp->acp->next != NULL )
		ocp->acp = ocp->acp->next;



	if( fno < ocp->acp->fno ||
			ocp->acp->Fshot == SHOT_END && fno > ocp->acp->fno ){
		oim->Falive = 0;
		return( 0 );
	}



	skl = oim->icp->skl;

	cp0 = ocp->acp;
	skl0 = cp0->skl;

	if( fno == cp0->fno ){

		tmp = skl_Fcopy( skl0, skl );
		sbr = oim->Mbr & ~tmp;


		if( !CONFIG3D_IS_EQUAL( cp0->c0, oim->Bcp ) ){

			oim->Bcp = cp0->c0;
			sbr = 0;
		}

		oim->icp->c.x = cp0->c.x;
		oim->icp->c.y = cp0->c.y;
		oim->icp->gc = cp0->gc;


		if( oim->Falive == 0 ){
			sbr = 0;
			oim->Falive = 1;
		}
		return( sbr );
	}


	t = ( fno - cp0->fno ) / (float)cp0->dfno;


	cp1 = ocp->acp->next;
	skl1 = cp1->skl;


	if( skl_is_equal( skl0, skl1 ) ){
		tmp = skl_Fcopy( skl0, skl );
		sbr = oim->Mbr & ~tmp;
	}
	else	{
		skl_affine( skl0, skl1, t, skl );
		sbr = 0;
	}



	gc.x = t *cp1->dgc.x;
	gc.y = t *cp1->dgc.y;
	gc.z = t *cp1->dgc.z;
	gc.u1 = 0;
	gc.u2 = 0;
	gc.u3 = 0;


	oim->icp->c.x = (1 - t ) *cp0->c.x + t *cp1->c.x;
	oim->icp->c.y = (1 - t ) *cp0->c.y + t *cp1->c.y;

	oim->icp->gc.x = cp0->gc.x + gc.x;
	oim->icp->gc.y = cp0->gc.y + gc.y;
	oim->icp->gc.z = cp0->gc.z + gc.z;
	oim->icp->gc.u1 = oim->icp->gc.u2 = oim->icp->gc.u3 = 0;


	oimage_view_cp( oim, &cp0->c0, &gc, t*cp1->u, t*cp1->v, &c0 );

	if( !CONFIG3D_IS_EQUAL(c0, oim->Bcp ) ){
		oim->Bcp = c0;
		sbr = 0;
	}



	if( oim->Falive == 0 ){
		sbr = 0;
		oim->Falive = 1;
	}
	return( sbr );
}




static mbr_type
skl_Fcopy( skl_type *skl0, skl_type *skl )
{
jskl_type	*jnt,	*j0;
bskl_type	*b,	*b0;
mbr_type	sbr;
int	i;

	sbr = 0;

	for( jnt = skl->jnt, j0 = skl0->jnt ; jnt != NULL ;
						jnt= jnt->nj, j0 = j0->nj ){

		jnt->ctr = j0->ctr;


		for( i = 0 ; i < jnt->bno ; i++ ){

			b = &jnt->b[i];
			if( b->direct == F_END )	continue;

			b0 = &j0->b[i];

//			sbr |= lnk_Fcopy( b0->link, &jnt->ctr, b->link );
			if( lnk_Fcopy( b0->link, &jnt->ctr, b->link ) > 0 )
				MBR_SET( sbr, b->link->br );
		}
	}

	return( sbr );
}



static int
lnk_Fcopy( lnk_type *link0, vec2d *ctr, lnk_type *link )
{
lnk_type	*l,	*l0;
vec2d	v;
int		flag;

	flag = 0;
	for( l = link, l0 = link0, v = *ctr ; l != NULL ; 
				l = LNK_NEXT(l), l0 = LNK_NEXT(l0) ){

		if( l->ctr.x == v.x && l->ctr.y == v.y &&
				l->v.x == l0->v.x && l->v.y == l0->v.y &&
				l->a == l0->a ){
			v.x += l->v.x;
			v.y += l->v.y;
			continue;
		}


		l->ctr = v;
		l->v = l0->v;
		l->u = l0->u;
		l->a = l0->a;
		l->len = l0->len;
		l->c_prb = l0->c_prb;

		v.x += l->v.x;
		v.y += l->v.y;

//		sbr |= 1L << l->br;
		flag = 1;

	}

	return( flag );
}
