/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/**************************
 ***	VImageFrame.c   ***
 **************************/
#include	<math.h>

#include	"Uigp/igp.h"

#include	"VimageType/VimageType.h"



/***** vimage_frame *****
Description:The method 'arranges' the numbers in the vim. It resets the fno and dfno of all casts, 
			and changes the vim->frame parameter if needs to. If the scene is now shorter, it would
			clip the tcpL to fit the new size only if Fclip not equals 0.
			If the new scene is longer, it would extend the tcp.
Params:	vim - 
		Fclip - if the "arranged" scene has shorter size than vim->frame, this param determines 
				whether to clip the rest of the scene from the camera or not.
Returns: 1 or 0 for action performed or not.
*****************************/
int vimage_frame( vimage_type *vim, int Fclip )
{
cast_type	*cast;
int	frame,	i,	tmp;

	frame = 0;
	for( i = 0 ; i < vim->no ; i++ ){
		cast = vim->cast[i];


		if ( cast->ocp->cp != NULL )
		{
			tmp = ocpL_reset_fno( cast->ocp );

			if( tmp > frame )	frame = tmp;
		}
	}
	
	if( frame == vim->frame || frame <= 0 )	return(0);

	if( frame < vim->frame ){
		if( Fclip == 0 )	return(0);

		tcpL_clip( vim->tcp, frame );

		bframeL_clip( vim->bframeL, frame );

		vim->frame = frame;
		return( 1 );
	}

	vim->frame = frame;
		
	tcpL_extend_frame( vim->tcp, vim->frame );

	return( 1 );
}


int
vimage_frame_clip( vimage_type *vim, int frame )
{
cast_type	*cast;
int	i,	ret=0;

	for( i = 0 ; i < vim->no ; i++ ){
		cast = vim->cast[i];

		ret += ocpL_clip( cast->ocp, frame );
	}

	tcpL_clip( vim->tcp, frame );

	bframeL_clip( vim->bframeL, frame );

	vim->frame = frame;

	return( ret );
}


/*eyal*/
/*
Description:
	the method inserts fInserted frames starting at fno.
Params:
	vim		- the vimage
	fno		- frame number to insert at. 
	fInserted - number of frames to insert.
*/
void vimage_frame_insert(vimage_type *vim,int fno, int fInserted)
{
	cast_type	* tmpCast;	
	int i, maxFno = 0, tmpFno ;
	
	/*insert frames to all casts*/
	if ( fno == 0 )
	{
		for( i = 0 ; i < vim->no ; i++ )
		{
			tmpCast = vim->cast[i];		
			tmpFno = ocpL_f_push(tmpCast->ocp, fInserted) ;
			maxFno = MAX(maxFno,tmpFno);			
		}	
	}//if 
	else 
	{
		for( i = 0 ; i < vim->no ; i++ )
		{
			tmpCast = vim->cast[i];		
			tmpFno = ocpL_f_insert(tmpCast->ocp,fno, fInserted) ;
			maxFno = MAX(maxFno,tmpFno);			
		}	
	}//else
	tmpFno = tcpL_insert(vim->tcp,fno,fInserted);
	maxFno = MAX(maxFno,tmpFno);	
	maxFno ++;
	if ( vim->frame < maxFno )
		vim->frame = maxFno ;	
}// vimage_frame_insert


/*eyal*/
/*
Description:
	the method adds fInserted frames starting at the end of the scene.
Params:
	vim		- the vimage
	cast	- the cast to which the frames are inserted.
				if NULL then insert to all casts. *** don't use this option yet! ***
	fInserted - number of frames to insert.
	inBetween - interpolate or not
*/
void vimage_frame_add(vimage_type *vim, int fInserted)
{
	int  maxFno = 0, tmpFno ;
	int fno = vim->frame;

//	add frames to all casts	 -
//					by current scenario no frames are added to the casts,
//					but only to camera
/*
	cast_type	*tmpCast;	
	int i;
	for( i = 0 ; i < vim->no ; i++ )
	{
		tmpCast = vim->cast[i];		
		tmpFno = ocpL_f_add(tmpCast->ocp, fInserted, fno) ;
		maxFno = MAX(maxFno,tmpFno);			
	}
*/


	tmpFno = tcpL_add(vim->tcp, fInserted);
	maxFno = MAX(maxFno, tmpFno);	
	maxFno ++;
	if ( vim->frame < maxFno )
		vim->frame = maxFno ;	
}// vimage_frame_add


/***** vimage_frame_rescale *****
Description:The method sets new length to the scene.
Params:		vim - the vimage
			frame_no - new length to the scene
*****************************/
void vimage_frame_rescale( vimage_type *vim, int frame_no ) 
{
cast_type	*cast;
int	i;

	if( vim->frame == frame_no )	return;

	if( vim->bframeL != NULL )
		bframeL_rescale_frame( vim->bframeL, vim->frame, frame_no );


	tcpL_rescale_frame( vim->tcp, vim->frame, frame_no );


	vim->tcp->a_cp = vim->tcp->cp;


	for( i = 0 ; i < vim->no ; i++ ){
		cast = vim->cast[i];

		ocpL_rescale_frame( cast->ocp, vim->frame, frame_no );
	}

	vim->frame = frame_no;


	vimage_anim_set_aux( vim );
}
