/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#include	<stdio.h>
#include	<math.h>


#include	"Uigp/igp.h"

#include	"VimageType/VimageType.h"
#include	"AgpLayer/AgpLayer.h"

#include	"Rendering/Render.h"



vimage_type *
vimage_alloc()
{
vimage_type	*vim;

	vim = ( vimage_type *)malloc( sizeof( vimage_type ));
	vimage_init( vim );

	return( vim );
}







void
vimage_init( vimage_type *vim )
{
	int i;
	vim->frame = 30;
	vim->row = vim->Srow = 480;
	vim->column  = vim->Scolumn = 720;
	vim->color = 0xffffff;

	vim->Srow  =  vim->row;
	vim->Scolumn  =  vim->column;

	vim->previewFrame = 0;
	vim->frameRate = 0;

	vim->name[0] = 0;
	
	for ( i = 0 ; i < CAST_NO ; i ++ )
		vim->cast[i] = NULL;

	vim->bim = NULL;

	vim->ragp = NULL;
	vim->expA = NULL;

	vim->tcp = NULL;

	vim->sound = NULL;
	vim->sound_no = 0;

#ifdef	VIM_BFRAME
	vim->bframeL = NULL;
#endif

	vim->inL = NULL;

	vim->no = 0;

	vim->Avw = NULL;

	config3d_init( &vim->Gcp );

	config3d_init( &vim->Ccp );
}



void
vimage_free( vimage_type *vim )
{
int	i;

	if( vim->bim != NULL ){
		bimage_free( vim->bim );
		vim->bim = NULL;
	}


	if( vim->tcp != NULL ){
		tcpL_destroy( vim->tcp );
		vim->tcp = NULL;
	}

#ifndef _ST
	if( vim->sound != NULL ){
		sound_destroy( vim->sound );
		vim->sound = NULL;
	}

	for( i = 0 ; i < vim->sound_no ; i++ )
		sound_destroy( vim->asound[i] );
#endif // _ST

#ifdef	VIM_BFRAME
	if( vim->bframeL != NULL ){
		bframeL_destroy( vim->bframeL );
		vim->bframeL = NULL;
	}
#endif

	if( vim->inL != NULL )
		interactionL_destroy( vim->inL );

	for( i = 0 ; i < vim->no ; i++ ){

		cast_destroy( vim->cast[i] );

		vim->cast[i] = NULL;
	}



	vim->no = 0;


	if( vim->ragp != NULL ){
		gp_array_free( vim->ragp );
		free( vim->ragp );
		vim->ragp = NULL;
	}


	if( vim->expA != NULL ){
		expAux_destroy( vim->expA );
		vim->expA = NULL;
	}


	if( vim->Avw != NULL ){
		free( vim->Avw );
		vim->Avw = NULL;
	}

}


void
vimage_destroy( vimage_type *vim )
{
	vimage_free( vim );

	free( vim );
}


int
vimage_layer_no( vimage_type *vim )
{
int	i,	no = 0;

	if( vim->bim != NULL )
		no = bimage_layer_no( vim->bim );

	for( i = 0 ; i < vim->no ; i++ )
		no += cast_layer_no( vim->cast[i] );

	return( no );
}


int
vimage_remove_unseen_cast( vimage_type *vim )
{
int	i,	j;
	for( i = 0, j = 0 ; i < vim->no ; i++ ){
		if( vim->cast[i]->ocp->cp == NULL ){
			cast_destroy( vim->cast[i] );
			vim->cast[i] = NULL;
			continue;
		}

		vim->cast[j++] = vim->cast[i];
	}

	vim->no = j;

	return( vim->no );
}