/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
/***********************
 ***	CastType.h   ***
 ***********************/
#ifndef         _CAST_TYPE_
#define         _CAST_TYPE_

#ifdef __cplusplus
extern "C" {
#endif

#include	"OcpType.h"


#include	"OimageType.h"

#include	"Sound/SoundType.h"

#include	"Interaction/InteractionType.h"

#include	"TextType.h"



/* for cast type */
#define		CAST_REGULAR	1
#define		CAST_TEXT		2
#define		CAST_SOUND		3
#define		CAST_TEXT_ASCII	6

#define		CAST_NORMAL	CAST_REGULAR




/* for cast property */
#define		MC_NULL     0
#define		MC_STATIC	0x01
#define		MC_LOCK		0x02
#define		MC_NFLIP	0x04

#define		MC_ROLE_OVER	0x10
#define		MC_URL			0x20
#define		MC_SOUND		0x40
#define		MC_MERGEVIM		0x80



#define		CAST_NAME_LEN		65
#define		CAST_TEXT_LEN		(CAST_NAME_LEN-1)



typedef struct cast_type {

	int	type;

	char	name[CAST_NAME_LEN];

	int	scale;			// pixels

	int	flip;			// 

	skl_type	*skl;	// base skeleton

	oimage_type	*oim;

	ocp_list	*ocp;	// list of keyframes of the anim

	text_type	*text;


	int	Moveable;		// Does the object can move by the player
	int Flipable;		// Does the object can rotate by more than 180 dgree

	/* Property */
//	int	Mproperty;

//	ocp_list	*rcp;	/* ROLL OVER */

//	char	*url;
	


	interactionL_type	*inL;


	/* Aux */
	ocpL_type	*Rcp;	// for INTERACTION_ANIMATION
	int	Flock;
	int	Fshow;

}       cast_type;





cast_type *	cast_alloc();

cast_type * cast_copy(cast_type *cast);

cast_type *	cast_create_from_agp( gp_array *agp, char *name );

void		cast_destroy( cast_type *cast );

void		cast_anim_free( cast_type *cast );

void		cast_anim_clear( cast_type *cast, int frame );


void		cast_set_agp( cast_type *cast, gp_array *agp );

cast_type *	cast_create( gp_array *agp, char *fname );


int		cast_rescale( cast_type *cast, int scale );



	/* CastTool1.c */
void	cast_simplifay_skl( cast_type *cast );


void	cast_simplifay_skl_key( cast_type *cast );

void	cast_replace_skl( cast_type *cast, skl_type *skl );

cast_type *	cast_copy_template( cast_type *scast, int Fanim );

int		cast_layer_no( cast_type *cast );

view_type *		cast_view( cast_type *cast );


void	cast_set_moveable( cast_type *cast, int value );

void	cast_set_flipable( cast_type *cast, int value );

int	cast_is_moveable( cast_type *cast );

int	cast_is_flipable( cast_type *cast );

interaction_type *	cast_interaction_get( cast_type *cast, int trigger, int frame, int type );

void	cast_interaction_add_URL( cast_type *cast, int trigger, int type, char *url );



	/* CastEdit.c */
int	cast_edit_add_layer( cast_type *cast, gp_array *agp );

int	cast_edit_reposition_layer( cast_type *cast, int iLayer, vec2d *dp );

void	cast_edit_replace_layer( cast_type *cast, int iLayer, gp_array *agp );

void	cast_edit_replace_layerA( cast_type *cast, int iLayer, gp_array *agp );

#ifdef __cplusplus
}
#endif
#endif
