/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#ifndef         _OCP_TYPE_
#define         _OCP_TYPE_

#include	"Uconfig3d/Config3d.h"

#include	"GpType/GpType.h"


#include	"SklType/SklType.h"



/* value for Fsoht */
#define		SHOT_MID	1
//#define		SHOT_BEGIN	2
#define		SHOT_END	4

#define		SHOT_FRAME	8


/*eyal*/
//#define OCP_SHOT_START	2
//#define OCP_SHOT_MID	1
//#define OCP_KEYFRAME	4


  // grid macros!!!
#define		FRAME_BLANK			0
#define		FRAME_MIDDLE		3
#define		FRAME_K_STANDALONE	4
#define		FRAME_K_END			5
#define		FRAME_K_BEGIN		6
#define		FRAME_K_MIDDLE		7

#define		FRAME_K				0x04



typedef struct ocp_type {

    config3d        gc;		/* Object's plane (3D)  oreintation and position*/
    config3d        dgc;	/* Object's plane (3D)  delta from previous keyframe*/
	int	Fgc;
				
    config3d        c;		/* Object's <x,y> position on the plane */

	float	u,	v;
    vec2d   dp;             /* pixel shift */
	config3d	c0;			/* absolute c */

	int	Fshot;				/* BEGIN END or MID (for inbetween) */

    int     fno,    dfno;	/* frame number, distance to next keyframe */

	skl_type	*skl;		/* pointer to current skeleton position */

    struct ocp_type *next;	/* next keyframe in the list */
}       ocp_type;




typedef struct ocp_list {
	int	no;

	ocp_type	*cp;	// first in the list
	ocp_type	*acp;	// active keyframe
	int	fi;

}       ocp_list;

typedef ocp_list ocpL_type;


	
	/* OcpTool.c */

ocp_list *	ocpL_alloc();

ocp_list *	ocpL_alloc2(  skl_type *skl, int f0, int f1 );


void		ocpL_destroy( ocp_list *ocp );


void		ocp_list_init( ocp_list *ocp );

void		ocpL_free( ocp_list *ocp );

void		ocp_list_clear( ocp_list *ocp, skl_type *skl, int cp_no );

void		ocp_list_rescale_c( ocp_list *ocp, float rescale );

int			ocp_is_clear( ocp_type *cp, skl_type *bskl );



ocp_type *	ocp_alloc( skl_type *skl );

void		ocp_free( ocp_type *cp );

void		ocp_clear( ocp_type *ocp );

int		ocpL_is_clear( ocp_list *ocp, skl_type *skl, int frame );

int		ocp_is_init( ocp_type *cp );

int		ocp_is_equal( ocp_type *cp, ocp_type *rcp );


ocp_type *	ocp_make_copy( ocp_type *cp );

void		ocp_copy( ocp_type *scp, ocp_type *tcp );


void		ocp_list_flipV( ocp_list *ocp, int column );

ocp_type *	ocp_affine( ocp_type *cp0, ocp_type *cp1,
						float t, ocp_type *cp );
//eyal
void OcpLF_exit(ocp_list *ocp, int frame);
void OcpLF_enter(ocp_list *ocp, skl_type* skl,int frame0, int frame1);



	/*OcpLFilter.c*/
/*eyal */
int			ocpL_filter( ocp_list *ocpL, float dx, float dy, float ds);
int			ocp_is_close_enough( ocp_type *cp, ocp_type *rcp ,float dx, float dy ,float ds);
int			ocp_may_be_dropped( ocp_type *cp0, ocp_type *cp1, ocp_type * cp2, 
							   float t,float dx, float dy, float ds);
int			ocpLF_filter(ocp_list *ocpL, float dx, float dy, float ds, int fno);


	/* OcpLInsert.c */

/*eyal */
int		ocpL_f_insert(ocp_list *ocp, int fno, int fInserted);

int		ocpL_f_add(ocp_list *ocpL, int fInserted,  int fno) ;

int		ocpL_f_push(ocp_list *ocpL, int fPushed);

	

	/* OcpLTool.c */
int			ocpL_kf_removable(ocp_list *ocp, int frame);

ocp_type *	ocpL_kf_add( ocp_list *ocp, ocp_type *pcp, ocp_type *scp, int fno );

ocp_type *	ocpL_kf_delete_segment( ocp_list *ocp, int f0, int f1);

ocp_type *	ocpL_kf_delete( ocp_list *ocp, ocp_type *ccp );

void	ocpL_kf_set_Fgc(ocp_list *ocpL, int kfno, int Fgc);

void	ocpL_kf_set_fno( ocp_list *ocp, ocp_type *acp, int fno );

int		ocpL_reset_fno( ocp_list *ocp );

int ocpL_kf_index(ocp_type *kf, ocp_list *list);

void	ocpL_rescale_c( ocp_list *ocp, float x0, float x, float y0, float y );

void	ocp_rotate( ocp_type *acp, vec2d *ctr, vec2d *p0, vec2d *p1 );


void	ocpL_shift2D( ocp_list *ocp, ocp_type *acp, int flag, vec2d *dp );

void	ocpL_fno_add(ocp_list *ocpL, int dframe );


//eyal
int ocpL_setAnimAt(ocp_list *ocpL, int startAt);






	/* OcpLTool1.c */

int ocpL_anim_init( ocp_list *ocp ,int fInit3D);

ocp_type *	ocpL_add_shot( ocp_list *ocp,
			ocp_type *ccp, int frame, int Fcontinuance,
			ocp_list *ocp_n );

ocp_type *	ocpL_add_segment( ocp_list *ocp,
			ocp_type *cp0, int frame, ocp_list *ocp_n );

void		ocpL_get_shot( ocp_list *ocp,
			ocp_type *cp0, ocp_type **Fcp, ocp_type **Lcp );

void		ocpL_copy_shot( ocp_type *Fcp, ocp_type *Lcp, ocp_list *ocp );

void		ocpL_inverse( ocp_list *ocp );

int			ocpL_size(ocp_list *ocpL);

int			ocpL_length(ocp_list *ocpL);

	/* OcpLFrame.c */
int		ocpLF_shot( ocp_list *ocp, int frame,
				ocp_type **Fcp, ocp_type **Lcp );


int		ocpLF_Fshot( ocp_list *ocp, int frame );

int		ocpLF_shot_nieg( ocp_list *ocp, int frame );

void	ocpLF_set_tween( ocp_type* cp, int enable);


int		ocpLF_status(ocp_list *ocp, int frame );




ocp_type *	ocpLF_force_kf( ocp_list *ocp, int frame );

int		ocpLF_is_kf( ocp_list *ocp, int frame );


int		ocpLF_is_3d_kf( ocp_list *ocp, int frame );

ocp_type *	ocpLF_get_kf( ocp_list *ocp, int frame );

ocp_type *	ocpLF_prev_kf( ocp_list *ocp, int frame );

ocp_type *	ocpLF_next_kf( ocp_list *ocp, int frame );

ocp_type *	ocpLF_copy( ocp_list *ocp, int frame );

int		ocpLF_paste( ocp_list *ocp, int frame,
			ocp_type *cp, int Fadd, int F3d );








	/* OcpLChange.c */
ocp_list * ocpL_Copy_Change ( skl_type* skl, skl_type* bskl, ocp_list* socp );



	/* OcpLCopy.c */
ocp_list *	ocpL_copy( ocp_list *socp );
ocp_list *	ocpL_copy_segment( ocp_list *socp, int frame0, int frame1);

int	ocpL_paste( ocp_list *tocp, ocp_list *socp, int frame0, int fDeleteKFs );

int	ocpL_delete( ocp_list *ocp, int frame0, int frame1 );

int ocpL_shrink(ocp_list *ocp, int f0, int f1);

int ocpL_delete_shrink(ocp_list *ocp, int frame0, int frame1);

int	ocpL_clip( ocp_list *ocp, int frame );

ocp_list *	ocpL_adjustment( skl_type *sskl, ocp_list *socp, skl_type *skl );

int ocpL_exist(ocp_list *ocp, int f0, int f1);



	/* OcpLRescale.c */
ocp_list *	ocpL_rescale( ocp_list *ocp, float Zf );

//eyal	
	/* OcpLImplicate .c */
ocp_list* ocpL_Implication ( skl_type* skl, skl_type* bskl, ocp_list* socp );



	/* OcpLRescaleFrame.c */
void	ocpL_rescale_frame( ocp_list *ocp, int cframe_no, int frame_no );




	/* OcpLClearLayar.c */
void	ocpL_clear_layer( skl_type* bskl, ocp_list* ocp, int Mlayer );

	/* OcpLClearLinkLen.c */
void	ocpL_clear_link_len( skl_type* bskl, ocp_list* ocp );




	/* OcpLAppend.c */
int	ocpL_extand( ocp_list *ocp, int frames );

int	ocpL_append2D( ocp_list *ocp, ocp_list *aocp, int startFrame );


int	ocpL_append3D( ocp_list *ocp, ocp_list *aocp, int startFrame );

	/* OcpLaddSegment.c */
ocp_list *	ocpL_copy_segmentN( ocp_list *socp, int *f0, int *f1 );

ocp_type *	ocpL_add_segmentN( ocp_list *ocp, int frame, ocp_list *ocp_n );






/*** DEPRECATED  ***
USE IN GPVIEWER ONLY
*******************/
void 
ocpL_shot_init( ocp_list *ocp);


#endif
