/***********************************************************************
This software module was originally developed by Yoram Elichai
(Vimatix Inc.) in the course of development of the MPEG-4
Systems (ISO/IEC 14496-1) standard. This software module is an implementation
of a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Vimatix Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003
************************************************************************

For more information or to receive an up to date version of this module,
you may contact Yoram Elichai at yorame@vimatix.com .
----------------------------------------------------------------------*/
#ifndef         _VIMAGE_TYPE_
#define         _VIMAGE_TYPE_

#ifdef __cplusplus
extern "C" {
#endif

#include	"Uconfig3d/Config3d.h"


#include	"GpType/GpType.h"


#include	"DepthExp/DepthExp.h"


#include	"TcpType.h"

#include	"Sound/SoundType.h"

#define	VIM_BFRAME
#ifdef	VIM_BFRAME
#include	"Bframe/BframeType.h"
#endif


#include	"BimageType.h"

#include	"CastType.h"





#define		CAST_NO		50
#define		SOUND_NO	24


#define		VIM_FRAME_MIN	2
#define		VIM_FRAME_MAX	960
#define		VIM_ROW_MIN		50
#define		VIM_ROW_MAX		800
#define		VIM_COLUMN_MIN	50
#define		VIM_COLUMN_MAX	1024


#define		MP_AUTOMAT_PLAY			0x01
#define		MP_DT_AUTOMAT_PLAY		0x02		// NOT ALLOW TO AUTOMAT_PLAY setting
#define		MP_LOOP					0x04
#define		MP_DT_LOOP				0x08		// NOT ALLOW TO CHNAGE LOOP setting


typedef struct vimage_type { 

	int	frame;			/*number of frames in the scene*/

	int	row,	column;	/*actual size*/

	int	Srow,	Scolumn;/*screen size*/

	int	frameRate;

	int	color;

	tcp_list	*tcp;	/*list of camera framekeys*/

	sound_type	*sound;

	int	sound_no;
	sound_type	*asound[SOUND_NO];

#ifdef	VIM_BFRAME
	bframeL_type	*bframeL;
#endif

	interactionL_type	*inL;


	bimage_type	*bim;	/*background image*/

	int	no;				/*number of casts*/

	cast_type	*cast[CAST_NO];/*array of all casts taking part at that vimage*/

	int	previewFrame;		/*frame number to preview*/

	int	playingRule;	// bit mask

	mbr_type	Mbr;			/*layers mask*/

	view_type	*Avw;	/*view*/

	config3d	Gcp;	/* Global camera Configurtion */

	/* Current Configurtion */

	int		frame_i;		/*number of frame currently presented*/

	config3d	Ccp;		/* Current camera Configurtion */

	mbr_type	Sbr;			/*mask for the static layers for next rendering*/

	/* AUX */

	gp_array	*ragp;		/*the whole picture (casts and background)*/

	int	rbgp;

	expAux_type	*expA;

	char	name[64];

	char	dir[64];

	int	layer_no;

}	vimage_type;








	/* VimageTool.c */
vimage_type *	vimage_alloc();

void	vimage_init( vimage_type *vim );

void	vimage_free( vimage_type *vim );

void	vimage_destroy( vimage_type *vim );


int		vimage_layer_no( vimage_type *vim );


int		vimage_remove_unseen_cast( vimage_type *vim );




	/* VimageAux.c */
void	vimage_aux( vimage_type *vim );

void	oimage_rescale( oimage_type *oim, float scale );

void	oimage_view_init( oimage_type *oim );



	/* VimageView.c */
void	vimage_view_init( vimage_type *vim );

void	vimage_view_init2( vimage_type *vim );

void	vimage_view_crop( vimage_type *vim, int row, int column );

void	vimage_view_rescale( vimage_type *vim, int row, int column );




	/* VimageMerge.c */
void	vimage_merge( vimage_type *vim, vimage_type *mvim,
					int Fbackground, int size );



	/* VimageRender.c */
void	vimage_render( vimage_type *vim, gp_array *agp );
void	oimage_render( oimage_type *oim, view_type *vw, gp_array *agp );


	/* OimageView.c */
void	oimage_view( oimage_type *oim, config3d *cm, view_type *vw );

void	oimage_view_cp( oimage_type *oim, config3d *cp0,
			config3d *cm, float u, float v, config3d *cp );

void	oimage_view_N( oimage_type *oim, config3d *cm, vec3d *N );




	/* VimageAnimAux.c */
void	vimage_anim_rewind( vimage_type *vim );

void	vimage_anim_set_aux( vimage_type *vim );
int		cast_anim_aux( cast_type *cast, ocp_list *ocp );

int		ocp_anim_aux( ocp_list *ocp );

void	oimage_anim_aux_c0( oimage_type *oim, ocp_list *ocp, config3d *gc0);



	/* VimageConfiguration.c */
void	vimage_configuration( vimage_type *vim, int fno );

mbr_type	oimage_configuration( oimage_type *oim, ocp_list *ocp, int fno, int Fshow );


void	vimage_configuration_init( vimage_type *vim );

int	oimage_configuration_cp2d( oimage_type *oim, ocp_type *ocp );




	/* VimageAnimInit.c */
void	vimage_anim_init( vimage_type *vim, int frame_no );

void	vimage_anim_init_global( vimage_type *vim );

int		vimage_anim_is_clear( vimage_type *vim );



	/* VimageFrame.c */
int	vimage_frame( vimage_type *vim, int Fclip );

int	vimage_frame_clip( vimage_type *vim, int frame );

void	vimage_frame_rescale( vimage_type *vim, int frame_no );



	/* VimageSoundAux.c */
void	vimage_sound_aux( vimage_type *vim, int Fplay );

void	vimage_sound_set_interaction( vimage_type *vim );


	/* VimageSound.c */
int				vimage_sound_add( vimage_type *vim, sound_type *sound );

int				vimage_sound_index( vimage_type *vim, char *sound_name );

sound_type *	vimage_sound_get( vimage_type *vim, char *soundName );

void			vimage_sound_delete_unuse( vimage_type *vim );



	/* VimageReplaceBimage.c */
int		vimage_replace_bimage( vimage_type *vim,
					bimage_type *bim, int Faux );

int	vimage_replace_cast( vimage_type *vim, int iCast, cast_type *cast, int Fdestroy );

int vimage_insert_cast (vimage_type *vim,cast_type *cast);

void	cast_replace_ocp( cast_type *cast, skl_type *skl, ocp_list *ocp );


cast_type *	vimage_remove_cast( vimage_type *vim, int iCast, int Fdestroy );

void	vimage_render_clear( vimage_type *vim, int Fagp, mbr_type Sbr );

void	vimage_upadte_layer_no( vimage_type *vim );

cast_type *	vimage_detuch_cast( vimage_type *vim, int id );



	/* VimageSelect.c */
int	vimage_select_object( vimage_type *vim, gp_array *agp, int Flock,
						vec2d *p, float *depth );



/*eyal*/
	/* vimageAnimDecrease.c */
int	vimage_anim_filter_kf( vimage_type *vim, float dx, float dy, float ds );

	/* vimageAnimInsertFrames.c */
void vimage_frame_insert(vimage_type *vim,int fno, int fInserted);
void vimage_frame_add(vimage_type *vim, int fInserted);

	/* vimageAnimDelete*/
void vimage_frame_delete(vimage_type *vim, int frame);
void vimage_frame_delete1_segment(vimage_type *vim, int f_begin, int f_end);
void vimage_frame_delete_segment(vimage_type *vim, int f_begin, int f_end);

#ifdef __cplusplus
}
#endif

#endif
