// This software module was originally developed by 
// Gabriel Abrantes (AT&T) in the course
// of development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1998.

#if !defined(_phonemefap_converter_h)	// Sentry, use file only if it's not already included.
#define _phonemefap_converter_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

FILE:       PhonemeFAP_Converter.h
AUTHOR:     Gabriel Antunes Abrantes (AT&T)
OVERVIEW:   Definition of Phonem&Bookmark to FAP Converter
-------------------------------------------------------------------------*/

#include "Include/ztl.h"
#include "Core/Im1defs.h"

#include "TTSFAPInterface.h"
// PORTING to CC 5.3
// SOLARIS PORTING - typo
#include "TTS_Festival_client.h"

#define NO_MPEG_Phonemes 25


static int  Phoneme_to_Visemes[NO_MPEG_Phonemes] = {   0,  1,  1,  1,  2,  2,  3,  
													   3,  4,  4,  5,  5,  6,  6,  
													   6,  7,  7,  8,  8,  9, 10, 
													  11, 12, 13, 14 };

// visemes defined by low level FAP obtained from visemes.fap from (Lucent)

static int ttsvisemes_mask[68] = { 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 1, 0, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0,};


static int ttsvisemes_fap[15][17] = {
//0
{          0,     0,     0,     0,0,	   0,0,		   0,0,          0,     0,     0,     0,     0,     0},
//1
{        200,  -200,     0,     0,0,	-100,-100,     0,0,       -200,  -100,     0,  -100,  -400,  -100},  
//2
{        200,  -100,  -200,     0,0,	-50,-50,	-100,-100,       0,     0,     0,     0,   200,     0},  
//3
{        200,  -100,  -200,     0,0,	-50,-50,	-100,-100,       0,     0,     0,   400,   -50,   100}, 
//4
{        400,  -100,  -200,     0,0,	-50,-50,	-100,-100,       0,     0,     0,  -200,  -600,   400},  
//5
{        400,  -200,  -300,     0,0,	-50,-50,	-100,-100,       0,   100,   100,   400,  -400,   200},  
//6
{        400,  -100,  -200,     0,0,	-50,-50,	-100,-100,       0,     0,     0,   300,  -100,   100},  
//7
{        400,  -100,  -400,     0,0,	-50,-50,	-200,-200,       0,     0,     0,   400,  -50,    50 },
//8
{        200,  -200,  -300,     0,0,	-50,-50,	-100,-100,       0,   100,   100,   100,  -400,   150},  
//9
{        600,  -200,  -400,     0,0,	-100,-100,  -200,-200,       0,     0,     0,  -200,  -400,  -100},  
//10
{        400,  -100,  -200,     0,0,	-50,-50,	-100,-100,       0,     0,     0,     0,  -600,   200},  
//11
{        300,  -200,  -200,   200,200,  -50,-50,	-100,-100,     100,   100,   100,     0,  -600,   400},  
//12
{        800,  -200,  -600,     0,0,	-100,-100,  -300,-300,		 0,     0,     0,  -200,  -400,  -100},  
//13
{        400,  -200,  -300,     0,0,	-50,-50,	-100,-100,       0,   150,   150,  -200,  -400,  -100}, 
//14
{        400,  -200,  -300,     0,0,	-50,-50,	-100,-100,		 0,   150,   150,  -200,  -400,  -100},  
};

static int ttsvisemes_max[17] = { 
		 600,  -200,  -600,   200,200, -100,-100,	-300,-300,     200,   150,   150,   400,   600,   400};



// expressions defined by low level FAP obtained from expressions.fap (IST)
static int ttsexpressions_mask[7][68] = { 
	{	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
		0, 0, 0, 0, 0, 0, 0, 0 },
// joy 
	{	0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 
		1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
		0, 0, 0, 0, 0, 0, 0, 0 },
// sadness 
	{	0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 0, 0, 0, 0, 0, 1, 1, 
		1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
		0, 0, 0, 0, 0, 0, 0, 0 },
// anger 
	{	0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 
		1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
		0, 0, 0, 0, 0, 0, 0, 0 },
// fear
	{	0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 
		1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
		0, 0, 0, 0, 0, 0, 0, 0 },
// disgust
	{	0, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 0, 1, 1, 
		1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 
		1, 1, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
		0, 0, 0, 0, 0, 0, 0, 0 }, 
// surprise 
	{	0, 0, 1, 0, 1, 1, 1, 0, 0, 1, 1, 1, 1, 0, 0, 0, 0, 0, 0, 0, 
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 1, 1, 1, 1, 1, 1, 0, 0, 0, 0, 
		0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 0, 
		0, 0, 0, 0, 0, 0, 0, 0 },
};


static int ttsexpressions_fap[7][68] = {
	{0},
	// joy 
	{75, -360, -105, 135, 135, -330, -330,-80, -80, 525, 525,   120, 120, 20, 20},
	// sadness 
	{-300, -300,  300, 300, 150, 150,     50 ,50 ,13 ,13 ,-37 ,-37 ,150, 150},
	// anger 
	{-150, -22, 12, 12 ,-150 ,-150 ,-22 ,-22, -135 ,-135  , 225 ,225, 135 ,135 , 
	 -175,-175, -140 ,-140 ,-63, -63,145, 145 },
	// fear
	{325, 0, -325 ,0 ,0 ,0, 0 ,-360, -360 , 0 ,0    ,-15 ,-15, 8 ,8   , 60 ,60, 15,15,
	 -45 ,-45, 180, 180 },
	// disgust
	{150 ,-100, -100 ,100,0 ,-200 ,-50 ,-25, -100 ,300 ,0 ,   220 ,150 ,160 ,100 , 
	 -100, -50,-100, -50 ,-130, -70  , 200, 140},
	// surprise 
	{775, -795 ,-90 ,-90, -735 ,-735 ,-135, -135, 210 ,210 ,210, 210, 195, 195},
};




struct TTS_viseme {
	int blend;
	int select1;
	int select2;
};


struct TTS_expression {
	int select1;
	int select2;
	int intensity1;
	int intensity2;
};


struct TTSBOOKMARK {
	int FAP;		// which FAP
	int FAP_value;	// low level FAP value
	TTS_expression expression;
	TTS_viseme viseme;
	int word;		// after which word in the sentence
	int T;			// transition time in ms
	int C;			// time curve 
	int start_time; // beginning of this bookmark
	int as[2];
	int gs[2];
	int gs_1[2];
};
typedef struct TTSBOOKMARK TTS_BookMark;



struct TTSFAFRAME {
	int expression_set;
	TTS_expression expression;
	int viseme_set;
	TTS_viseme viseme;
	int	time_stamp;
	int faps[68];
};
typedef struct TTSFAFRAME TTS_FA_frame;




class Face;

class PhonemeFAP_Converter
{
	
public:

	// Constructor
	PhonemeFAP_Converter();
	
	// Destructor
	~PhonemeFAP_Converter();

	// control by TTS_Interface
	void SetTTSFace(Face* ttsface);
	void SetTTSFAPInterface(ZArray<TtsFAPInterface*> *TttsFAPI);

	// for face node to get a FAP frame
	void Get1FA_Frame();

private:

	// convert Phonemes to FAP
	void Phonemes_to_FAP();
	void Phoneme_2_FA_Frames();

	void ParseBookmark(char *bm,int start_time);
	int  BookmarksToFrame(TTS_FA_frame* frame);
	void AddFAPFrame(TTS_FA_frame* );
	void ExtraFrames4Bookmarks();
	void AddLowLevelFAPs4VisExp(TTS_FA_frame* FAframe);

	int  GetInt(char *bm,int *bc);
	int  Interpolation(int function,float as,float a,float gs_1,float t);
	int  Tangent(float as,float a,float gs_1,float t);
	void ClearFrames();

	Face* TTS_Face;
	int frames_lock;
	int last_frame;
	ZArray<TTS_FA_frame*> TTS_FAframes;
	ZArray<TTS_BookMark*> TTS_BookMarks;
	ZArray<TtsFAPInterface*>  TTSFAPInterface;
};

#endif                                      // _phonemefap_converter_h sentry.
