// This software module was originally developed by 
// Gabriel Abrantes (AT&T) in the course
// of development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1998.


#include <assert.h>
#include <time.h> 

#include "TtsDecoder/TTS_Interface.h"
#include "TtsDecoder/PhonemeFAP_Converter.h"

#include "Nodes.h"

#include "OpenGl/Fba/FaceProxy.h"


//Constructor

TTS_Interface::TTS_Interface(): 
	thread_on(0),
	end_thread(0),
	server_fd(-1),
	TTS_Face(NULL),
	tts_packet(NULL),
	tts_source(NULL),
	PhonemeFAP_Conv(NULL)
{
}

// Destructor

TTS_Interface::~TTS_Interface()
{
	if(this) Close();
}


void TTS_Interface::SetTTSFace(Face* ttsface)
{
	if(this) TTS_Face = ttsface;
}



void TTS_Interface::SetTTSPacket(TTS_Packet* tts_p)
{
	if(this) tts_packet = tts_p;
}


void TTS_Interface::SetAudioSource(AudioSource* source)
{
	if(this) tts_source = source;
}


// start the tts_interface


void TTS_Interface::Start()
{
	thread_on = 1;
	m_smpActive.P();

	if(TTS_Face) {
		if(((Viseme *) TTS_Face->fap->viseme) == NULL)
			MessageBox(NULL,"No Viseme node, problems will ocurr!","TTS in IM1 3D-Player",MB_OK);
		
		if(((Expression *) TTS_Face->fap->expression) == NULL)
			MessageBox(NULL,"No Expression node, problems will ocurr!","TTS in IM1 3D-Player",MB_OK);
	}

	if (ZThread<TTS_Interface> (TTS_InterfaceThread,this).Begin () == -1) {
		m_smpActive.V();
		throw;
	}

}

// end tts-interface thread

void TTS_Interface :: Close ()
{
	end_thread = 1;
	while(thread_on);
	CloseConnection(server_fd);
	m_smpActive.P ();
	m_smpActive.V ();
	if(PhonemeFAP_Conv) delete PhonemeFAP_Conv;
}


// launch the tts_interface_thread

void TTS_Interface::TTS_InterfaceThread()
{
	Run();
	m_smpActive.V();
	if (TTS_Face) 
    ((FaceProxy *) TTS_Face->GetRenderingObject())->
    SetTTS_Mode(0);
	thread_on = 0;
}




void TTS_Interface::Run()
{
	int i = 0;
	char word[256],sentence[4096];

	if(!SetupTTSSystem()) {
		sprintf(word,"TTS System is not working!");	
		MessageBox(NULL,word,"TTS in IM1 3D-Player",MB_OK);
	}

	if(TTS_Face) { 
		PhonemeFAP_Conv = new(PhonemeFAP_Converter);
		PhonemeFAP_Conv->SetTTSFace(TTS_Face);
		((FaceProxy *) TTS_Face->GetRenderingObject())->
      SetPhonemeFAPConverter(PhonemeFAP_Conv);
		((FaceProxy *) TTS_Face->GetRenderingObject())->
      SetTTS_Mode(1);
	}

	while(tts_packet->current_frame<tts_packet->no_frames) {
		
		if(end_thread) break; // user interruption

		strcpy(sentence,tts_packet->Frames[tts_packet->current_frame]->Sentence->Text);

		GetBookmarks(sentence);

		// say one sentence with TTS

		if(TTS_Face) {
			// interface with facial animation:
			// only if a FaceNode is associated to the TTS:
			// get phonemes and timing info from TTS

			_flushall();
			SaySentence(server_fd,sentence,1,&TTSFAPInterface,&start_tts);
			FestivalPho_to_MPEGPho();
			// give phonemes to PhonemeFAPConverter
			PhonemeFAP_Conv->SetTTSFAPInterface(&TTSFAPInterface);
			BookMarks.Clear();

			// text-> tts -> phonemes & timinging_info -> FAP-frames-> put in structure
		}
		else SaySentence(server_fd,sentence,0,NULL,&start_tts);		
		
		tts_packet->current_frame++;
	}
	
	SaySentence(server_fd,"  ",0,&TTSFAPInterface,&start_tts);

	MessageBox(NULL,"THE END of TTSInterface!","TTS in IM1 3D-Player",MB_OK);
}


	
// get bookmarks in TTS-text

void TTS_Interface::GetBookmarks(char *sentence)
{
	int slength = strlen(sentence);
	int swords  = GetNoWordsinSentence(); 	

	int word_cnt = 1;

	int i=0;
	while(i<slength) {

		while(isspace(sentence[i]))i++;

		if(sentence[i]=='<') {
			char bm[256];int k=0;
			while((sentence[i]!='>') && (i<slength)) {
				sentence[i++] = ' ';
				bm[k++] = sentence[i];
			}
			sentence[i]=' ';
			bm[--k]='\0';

			BookMark* bkm = (BookMark*) calloc(1,sizeof(BookMark));

			strcpy(bkm->bookm,bm); 
			if(word_cnt<=swords)
				bkm->word  = word_cnt;
			else bkm->word  = word_cnt-1;
			BookMarks+=bkm;
		}
		else {
			while(!isspace(sentence[i++]) && (i<slength));
			word_cnt++;
		}
	}
}


// put TTS-system working & build up connection

int TTS_Interface::SetupTTSSystem()
{

	if(StartServer()) {
		server_fd = ConnectoServer();
		if(server_fd<0) {
			//close server
			return 0;
		}
		return 1;
	}
	else return 0;
}




// convert Festival phonemes to MPEG phonemes

void TTS_Interface::FestivalPho_to_MPEGPho()
{
	int word_cnt = 0;
	int bookmark = BookMarks.GetLength();
	int no_phon  = TTSFAPInterface.GetLength();


	for(int i=0;i<no_phon;i++) {
		
		TTSFAPInterface[i]->start_time*=CLOCKS_PER_SEC;
		TTSFAPInterface[i]->start_time+=start_tts;
		TTSFAPInterface[i]->phonemeDuration*=CLOCKS_PER_SEC;

		if(bookmark) {
			
			// Add bookmarks of sentence where necessary

			if(TTSFAPInterface[i]->wordBegin) { 
				// phoneme that corresponds to a word beginning
				word_cnt++;
				// now search for corresponding bookmark, if it exist
				int bcnt = 0;
				for(int k=0;k<bookmark;k++)
					if(BookMarks[k]->word==word_cnt) 
						strcpy(TTSFAPInterface[i]->bookmark[bcnt++],BookMarks[k]->bookm);
			}
		}
		
		int found = 0;
		for(int j=0;j<NO_Festival_Phonemes;j++) {
			if(!strcmp(Festival_Phonemes_Alphabet[j],TTSFAPInterface[i]->phonemeSymbol)) {
				strcpy(TTSFAPInterface[i]->phonemeSymbol,Festival_to_MPEG_Phonemes[j]);
				found = j+1;
			}
		}
		
		if(!found) {
			char word[256];
			sprintf(word,"Unknown Festival phoneme: - %s -",TTSFAPInterface[i]->phonemeSymbol);
			MessageBox(NULL,word,"TTS in IM1 3D-Player",MB_OK);
		}
	}
}




int TTS_Interface::GetNoWordsinSentence() 
{
	int i=0,no_words = 0;
	char _text[1024];
	
	strcpy(_text,tts_packet->Frames[tts_packet->current_frame]->Sentence->Text);
	int length = strlen(_text);
	
	while(i<length) {
		
		while(isspace(_text[i]))i++;
		
		if(_text[i]=='<') {
			while((_text[i]!='>') && (i<length)) _text[i++] = ' ';
			_text[i]=' ';
		}
		else if (_text[i]=='.') break;
		else {
			while(!isspace(_text[i++]) && (i<length));
			no_words++;
		}
	}
	
	return no_words;

}