// This software module was originally developed by 
// Gabriel Abrantes (AT&T) in the course
// of development of the MPEG-4 Systems standard (ISO/IEC 14496-1).  This
// software module is an implementation of a part of one or more MPEG-4
// Systems tools as specified by the MPEG-4 Systems standard.  ISO/IEC
// gives users of the MPEG-4 Systems standard free license to this software
// module or modifications thereof for use in hardware or software products
// claiming conformance to the MPEG-4 Systems standard.  Those intending to
// use this software module in hardware or software products are advised
// that its use may infringe existing patents.  The original developer of
// this software module and his/her company, the subsequent editors and
// their companies, and ISO/IEC have no liability for use of this software
// module or modifications thereof in an implementation.  Copyright is not
// released for non MPEG-4 Systems conforming products.  The original
// developer retains full right to use the code for his/her own purpose,
// assign or donate the code to a third party and to inhibit third parties
// from using the code for non MPEG-4 Systems conforming products.  This
// copyright notice must be included in all copies or derivative works.
//
// Copyright 1998.

#if !defined(_tts_interface_h)	// Sentry, use file only if it's not already included.
#define _tts_interface_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

FILE:       TTS_Interface.h
AUTHOR:     Gabriel Antunes Abrantes (AT&T)
OVERVIEW:   Definition of TTSI
-------------------------------------------------------------------------*/


extern "C" {
  #include "tts_decoder.h"
}

#include "ztl.h"
#include "Core/Im1defs.h"
#include "TTSFAPInterface.h"
#include "tts_festival_client.h"


#define NO_MPEG_Phonemes 25
#define NO_Festival_Phonemes 44

static char *Festival_Phonemes_Alphabet[] = {
   "brth",	
   "SIL",	
   "pau",	
   "aa",	
   "ao",	
   "ah",	
   "ax",	
   "ey",	
   "ch",	
   "dh",	
   "iy",	
   "ay",	
   "jh",	
   "ng",	
   "ow",	
   "er",	
   "sh",	
   "th",	
   "uw",	
   "aw",	
   "oy",	
   "zh",    
   "ae",	
   "b",		
   "d",		
   "eh",	
   "f",		
   "g",		
   "hh",	
   "ih",	
   "k",		
   "l",		
   "m",		
   "n",		
   "aa",	
   "p",		
   "r",		
   "s",		
   "t",		
   "uh",	
   "v",		
   "w",		
   "y",		
   "z",		
   "nx",	
   "q",	
   "dx"
};



static char *Festival_to_MPEG_Phonemes[] = {
   "na",
   "na",
   "na",
   "A:",
   "O",	
   "A:",
   "A:",
   "e",	
   "tS",
   "D",	
   "I",	
   "A:",
   "dZ",
   "Inter",
   "O",	
   "Inter",
   "S",	
   "T",	
   "Inter",
   "O",	
   "A:",
   "Inter",
   "e",	
   "b",	
   "d",	
   "e",	
   "f",	
   "g",	
   "Inter",
   "I",	
   "k",	
   "l",	
   "m",	
   "n",	
   "O",	
   "p",	
   "r",	
   "s",	
   "t",	
   "U",	
   "v",	
   "Inter",
   "Inter",
   "Z",	
   "Inter",
   "Inter",
   "Inter" 
};


static char *MPEG_Phonemes_Alphabet[] = { "na","p","b","m","f" ,"v" ,"T","D",
										  "t", "d","k","g","tS","dZ","S",
										  "s", "z","n","l","r" ,"A:","e",
										  "I", "O","U" };

struct BookMark {
	char bookm[256];
	int  word;
};



class AudioSource;
class Face;
class PhonemeFAP_Converter;

class TTS_Interface
{
	
public:
	/// Constructor
	
	TTS_Interface();
	
	/// Destructor
	
	~TTS_Interface();


	void SetTTSPacket(TTS_Packet* tts_p);
	void SetAudioSource(AudioSource* source);
	void SetTTSFace(Face* ttsface);

	// start the tts_interface
	void Start();

private:

	void Close ();

	// launch the tts_interface_thread
	void TTS_InterfaceThread();

	// run the whole module
	void Run();

	// get bookmarks in TTS-text
	void GetBookmarks(char *sentence);

	// put TTS-system working & build up connection
	int SetupTTSSystem();

	// convert Festival phonemes to MPEG phonemes
	void FestivalPho_to_MPEGPho();

	// get number of words in one sentence
	int  GetNoWordsinSentence();

	AudioSource* tts_source;
	Face* TTS_Face;
	TTS_Packet* tts_packet;
	PhonemeFAP_Converter* PhonemeFAP_Conv;
	int thread_on;
	int end_thread;
	int start_tts;
	ZArray<TtsFAPInterface*>  TTSFAPInterface;
	ZArray<BookMark*> BookMarks;

	int server_fd;

protected:
	BOOL m_bActive;
	ZSemaphore m_smpActive;
};

#endif                                      // _tts_interface_h sentry.
