/*
This software module was originally developed by 
        Hang-Seop Lee (hslee@etri.re.kr), ETRI
        Jung-Chul Lee (jclee@etri.re.kr), ETRI,

edited by Hang-Seop Lee, Jung-Chul Lee of ETRI, 

and modified by Gabriel Antunes Abrantes (AT&T) for integration in 
the IM1 3D player
  
in the course of development of the MPEG-4.
This software module is an implementation of a part of one or
more MPEG-4 tools as specified by the MPEG-4.
ISO/IEC gives users of the MPEG-4 free license to this                
software module or modifications thereof for use in hardware
or software products claiming conformance to the MPEG-4.
Those intending to use this software module in hardware or software
products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company,
the subsequent editors and their companies, and ISO/IEC have no
liability for use of this software module or modifications thereof
in an implementation.
Copyright is not released for non MPEG-4 conforming
products. ETRI and AT&T retains full right to use the code for his/her own
purpose, assign or donate the code to a third party and to
inhibit third parties from using the code for non
MPEG-4 conforming products.
This copyright notice must be included in all copies or
derivative works. Copyright (c) 1997/1998.
*/    


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "tts_decoder.h"


#define	TEXT_SYNC	(int)0
#define	VIDEO_SYNC	(int)1
#define FAPESCAPE "<FAP"

#define FALSE   0
#define TRUE    1



//extern union SYN { short iwave[1024*32]; unsigned char buf[1024*64]; } syn;

FILE *Open_File();
void Init_Data();
int  rd_txt(FILE *fp, TTS_Data *Tts, FRAME_Data *Frame);
void RemoveFAPBook(char *text);
char **cmatrix(int nrh, int nch);
void free_cmatrix(char **m, int nrh, int nch);

void Intdata_read(int *A, int n, int nbits, unsigned char *B);
void Shortdata_read(short *A, int n, int nbits, unsigned char *B);
void Chardata_read(char *A, int n, int nbits, unsigned char *B);
void CharMatrixdata_read(char **A, int n1, int n2, int nbits, unsigned char *B);
void free_Data(FRAME_Data *Frame);
int  Read_Text(TTS_Data *Tts, FRAME_Data *Frame);


int IsVideo;
int T_Sync[3];
// T_Sync[0] == current time
// T_Sync[1] == total time
// T_Sync[2] == start time of Sentence

TTS_Data *Tts;
FRAME_Data *Frame;


SENTENCE_Data *Snt;
Prosody_Data *Prosody;
Video_Data *Video;
Lip_Data *Lip;

unsigned char YY[2];



/******************** TTS Bit-Stream ********************/ 

int TTS_seq=0x000000011;
int TTS_sent=0x012121212;
char TTS_seq_id;
short L_code;
char G_en, A_en, R_en, P_en, V_en, L_en, T_en;
short TTS_id;
char Silence;
short Sdur;
char Gender, Age, Speech_Rate;
short Ntext;
char Text[500];
char Dur_en, F0_en, En_en;
short Nphone, Sphone;
char Phone[1000];
short Dur[1000];
char Ptch[1000][3], Energy[1000][3];
short SntDur, PinSnt, Offset, Nlip, LinSnt[1000];
char Lshape[1000];
char Niframe;

/* Bit assignment */

int BTTS_seq_id=5, BL_code=10, BG_en=1, BA_en=1, BR_en=1;
int BP_en=1, BV_en=1, BL_en=1, BT_en=1;
int BTTS_id=10;
int BSilence=1, BSdur=12;
int BGender=1, BAge=3, BSpeech_Rate=4;
int BNtext=12, BText=8;
int BDur_en=1, BF0_en=1, BEn_en=1;
int BNphone=10, BSphone=13, BPhone=8;
int BDur=12, BPtch=8, BEnergy=8;
int BSntDur=16, BPinSnt=16, BOffset=10;
int BNlip=10, BLinSnt=16, BLshape=8;




// structure for reading from ttsbuffer

typedef struct
{
	unsigned char *buffer_ptr;
	int byte_cnt;
	int header_byte_cnt;
	unsigned char header_YY[2];
	int buffer_size;
} StreamInfo;

StreamInfo tts_stream_info;


/******************** TTS Bit-Stream ********************/


int Read_Header(TTS_Packet *tts_packet)
{
    int i; char c;
	
    YY[0] = 0; 
	YY[1] = 0;
	
    Intdata_read(&i,1,32,YY);
	
    if(i!=TTS_seq) { 
		return 0; 
	}
	
    Chardata_read(&(tts_packet->TTS_Header->TTS_seq_id),1,BTTS_seq_id,YY);
    Shortdata_read(&(tts_packet->TTS_Header->L_code),1,BL_code,YY);
    Chardata_read(&(tts_packet->TTS_Header->G_en),1,BG_en,YY);
    Chardata_read(&(tts_packet->TTS_Header->A_en),1,BA_en,YY);
    Chardata_read(&(tts_packet->TTS_Header->R_en),1,BR_en,YY);
    Chardata_read(&(tts_packet->TTS_Header->P_en),1,BP_en,YY);
    Chardata_read(&(tts_packet->TTS_Header->V_en),1,BV_en,YY);
    Chardata_read(&(tts_packet->TTS_Header->L_en),1,BL_en,YY);
    Chardata_read(&(tts_packet->TTS_Header->T_en),1,BT_en,YY);
	
    if(YY[1]!=0) { 
		c=0; 
		i=YY[1]; 
		Chardata_read(&c,i,1,YY); 
		YY[1]=0; 
	}
	
    return(1);
}




void Init_Data()
{
    Frame = (FRAME_Data *) malloc(sizeof(FRAME_Data));
    Frame->Sentence = NULL;
}


void Init_TTS_Packet(TTS_Packet *tts_packet)
{
    tts_packet->TTS_Header = (TTS_Data *) calloc(1,sizeof(TTS_Data));
	tts_packet->no_frames  = 0;
	tts_packet->Frames = NULL;

}

void TTS_Data_Init(TTS_Packet *tts_packet)
{
	int i;

	tts_packet->Frames = (FRAME_Data **)calloc(tts_packet->no_frames,sizeof(FRAME_Data*));

	for(i=0;i<tts_packet->no_frames;i++) {
		tts_packet->Frames[i] = (FRAME_Data *)calloc(tts_packet->no_frames,sizeof(FRAME_Data*));
	    tts_packet->Frames[i]->Sentence = NULL;
	}
}




int tts_decoder(unsigned char *AUptr, int AUbitcount,TTS_Packet *tts_packet)
{
    int i,numer_of_sentences = 0;

	tts_stream_info.buffer_ptr = AUptr;
	tts_stream_info.byte_cnt = 0;
	tts_stream_info.buffer_size = AUbitcount;

	Init_TTS_Packet(tts_packet);
	Read_Header(tts_packet);


	// 1st determine how many sentences

	Init_Data(); // for figuring out no of sentences

	tts_stream_info.header_byte_cnt = tts_stream_info.byte_cnt;
	tts_stream_info.header_YY[0] = YY[0];
	tts_stream_info.header_YY[1] = YY[1];


    while( (i = Read_Text(tts_packet->TTS_Header,Frame)) != 0 ) {
		numer_of_sentences++; 
		free_Data(Frame); 
	}


	tts_packet->no_frames = numer_of_sentences;

	TTS_Data_Init(tts_packet);

	// reset
	tts_stream_info.byte_cnt = tts_stream_info.header_byte_cnt; 
	YY[0] = tts_stream_info.header_YY[0];
	YY[1] = tts_stream_info.header_YY[1];

	numer_of_sentences = 0;
    while(Read_Text(tts_packet->TTS_Header,tts_packet->Frames[numer_of_sentences]))
		numer_of_sentences++; 

	return 0;
}




int Read_Text(TTS_Data *Tts, FRAME_Data *Frame)
{
    int i, TTS_sent=0x012121212;
    short Ntext, Sdur, Nphone, Nlip;

    YY[0]=YY[1]=0;
    Intdata_read(&i,1,32,YY);

	if(i!=TTS_sent) return(0);

    Shortdata_read(&(Frame->TTS_id),1,BTTS_id,YY);
    Chardata_read(&Frame->Silence,1,BSilence,YY);

    if(Frame->Silence) {
		Shortdata_read(&Sdur,1,BSdur,YY);
		Frame->Silence=(char)Sdur; 
		Frame->Sentence=NULL;
    	YY[0]=YY[1]=0;
	}
    else {
        Frame->Silence=0; 
        Frame->Sentence=Snt=(SENTENCE_Data *)malloc(sizeof(SENTENCE_Data));
        Snt->Prosody=NULL; 
		Snt->Video=NULL; 
		Snt->Lip_shape=NULL;

        if(Tts->G_en) {
			Chardata_read(&Snt->Gender,1,BGender,YY);
	    }
		else Snt->Gender = 0;	// default gender is female
        if(Tts->A_en) {
			Chardata_read(&Snt->Age,1,BAge,YY);
	    }
		else Snt->Age = 4;
		if(!Tts->V_en && Tts->R_en) {
			Chardata_read(&Snt->Speech_Rate,1,BSpeech_Rate,YY);
		}
		else Snt->Speech_Rate = 3;

		Shortdata_read(&Ntext,1,BNtext,YY); 
		Snt->Ntext=Ntext;
		Snt->Text=(char *)malloc(Ntext+2); 
		Chardata_read(Snt->Text,Ntext,BText,YY); 

		Snt->Text[Ntext]=0;

        if(Tts->P_en) { 
			Snt->Prosody=Prosody=(Prosody_Data *)malloc(sizeof(Prosody_Data));
			Chardata_read(&(Prosody->Dur_en),1,BDur_en,YY);
			Chardata_read(&(Prosody->F0_en),1,BF0_en,YY);
			Chardata_read(&(Prosody->En_en),1,BEn_en,YY);
			Shortdata_read(&Nphone,1,BNphone,YY);
			Prosody->Nphone=Nphone;
			Shortdata_read(&(Prosody->Sphone),1,BSphone,YY); 
			i = Prosody->Sphone;
			Prosody->Phone=(char *)malloc((i+2));
			Chardata_read(Prosody->Phone,i,BPhone,YY);
			if(Prosody->Dur_en) {
	    		Prosody->Dur=(short *)malloc((Nphone+2)*2); 
	    		Shortdata_read(Prosody->Dur,Nphone,BDur,YY);
			}
			else Prosody->Dur=NULL;

			if(Prosody->F0_en) {
	    		Prosody->Ptch=cmatrix(Nphone+2,3);
	    		CharMatrixdata_read(Prosody->Ptch,Nphone,3,BPtch,YY);
			}
			else Prosody->Ptch=NULL;

			if(Prosody->En_en) {
	    		Prosody->Energy=cmatrix(Nphone+2,3);
	    		CharMatrixdata_read(Prosody->Energy,Nphone,3,BEnergy,YY);
			}
			else Prosody->Energy=NULL;
	    }
        if(Tts->V_en) { 
			Snt->Video=Video=(Video_Data *) malloc(sizeof(Video_Data));
			Shortdata_read(&Video->SntDur,1,BSntDur,YY);
			Shortdata_read(&Video->PinSnt,1,BPinSnt,YY);
			Shortdata_read(&Video->Offset,1,BOffset,YY);
	    }
        if(Tts->L_en) { 
			Snt->Lip_shape=Lip=(Lip_Data *)malloc(sizeof(Lip_Data));
			Shortdata_read(&Nlip,1,BNlip,YY); 
			Lip->Nlip=Nlip;
			Lip->LinSnt=(short *)malloc((Nlip+2)*2); 
			Shortdata_read(Lip->LinSnt,Nlip,BLinSnt,YY);
			Lip->Lshape=(char *)malloc(Nlip+2);
			Chardata_read(Lip->Lshape,Nlip,BLshape,YY);
	    }
		YY[0]=0; 
		YY[1]=0;
	}
    return(1);
}






char **cmatrix(int nrh, int nch)
{
    int i;
    char **m;
    m = (char **)malloc((unsigned) (nrh+1)*sizeof(char*));
    for( i=0;i <=nrh;i++) 
        m[i] = (char *) malloc((unsigned) (nch+1)*sizeof(char));

    return m;
}


void free_cmatrix(char **m, int nrh, int nch)
{
    int i;
    for( i=nrh;i >=0;i--) free((char*) (m[i]));
    free((char*) m);
}



void Intdata_read(int *A, int n, int nbits, unsigned char *B)
{
    int i, nx, ny;
    unsigned int X; unsigned char Y; 
    short W[9] = { 0,1,3,7,0x0F,0x01F,0x03F,0x07F,0x0FF }; 
	
    Y  = B[0]; 
	ny = B[1];
	
    for(i=0; i<n; i++)  {
		nx = nbits; 
		*(A+i)=0; 
		
		while(nx!=0) {
			if(nx>ny) { 
				X=(Y & W[ny]); 
				*(A+i)=*(A+i) | (X << (nx-ny)); 
				nx-=ny; 
				ny = 8; 
				Y = tts_stream_info.buffer_ptr[tts_stream_info.byte_cnt++];
			}
			else { 
				X=(Y >> (ny-nx)) & W[nx]; 
				*(A+i)=*(A+i) | X; 
				ny-=nx; 
				nx=0; 
			}
		}
	} 
	
	B[0]=Y; 
	B[1]=ny;
}

void Shortdata_read(short *A, int n, int nbits, unsigned char *B)
{
    int i, nx, ny;
    unsigned int X; unsigned char Y; 
    short W[9]={ 0,1,3,7,0x0F,0x01F,0x03F,0x07F,0x0FF }; 

    Y=B[0]; ny=B[1];
    for(i=0; i<n; i++) 
	{
		nx=nbits; *(A+i)=0; 
		while(nx!=0) 
		{
			if(nx>ny) 
			{ 
				X=(Y & W[ny]); 
				*(A+i)=*(A+i) | (X << (nx-ny)); 
	    		nx-=ny; 
				ny=8; 
				Y = tts_stream_info.buffer_ptr[tts_stream_info.byte_cnt++];
			}
			else 
			{ 
				X=(Y >> (ny-nx)) & W[nx]; 
				*(A+i)=*(A+i) | X; 
				ny-=nx; 
				nx=0; 
			}
	    }
	} 
	B[0]=Y; B[1]=ny;
}


void Chardata_read(char *A, int n, int nbits, unsigned char *B)
{
    int i, nx, ny;
    unsigned char X,Y;
    short W[9]={ 0,1,3,7,0x0F,0x01F,0x03F,0x07F,0x0FF }; 

    Y=B[0]; ny=B[1];
    for(i=0; i<n; i++) 
	{
		nx=nbits; *(A+i)=0; 
		while(nx!=0) 
		{
			if(nx>ny) 
			{ 
				X=(Y & W[ny]); 
				*(A+i)=*(A+i) | (X << (nx-ny)); 
	    		nx-=ny; ny=8; 
				Y = tts_stream_info.buffer_ptr[tts_stream_info.byte_cnt++];
			}
			else
			{ 
				X=(Y >> (ny-nx)) & W[nx]; 
				*(A+i)=*(A+i) | X;
				ny-=nx; nx=0; 
			}
	    }
	} 
	B[0]=Y; B[1]=ny;

}


void CharMatrixdata_read(char **A, int n1, int n2, int nbits, unsigned char *B)
{
    int i, j, nx, ny;
    unsigned char X,Y;
    short W[9]={ 0,1,3,7,0x0F,0x01F,0x03F,0x07F,0x0FF }; 

    Y=B[0]; ny=B[1];
    for(i=0; i<n1; i++) 
	{
        for(j=0; j<n2; j++) 
		{
		    nx=nbits; A[i][j]=0; 
		    while(nx!=0) 
			{
		        if(nx>ny) 
				{ 
				    X=(Y & W[ny]); 
					A[i][j]=A[i][j] | (X << (nx-ny)); 
		    	    nx-=ny; ny=8; 
					Y = tts_stream_info.buffer_ptr[tts_stream_info.byte_cnt++];
			    }
		        else 
				{
				    X=(Y >> (ny-nx)) & W[nx]; 
					A[i][j]=A[i][j] | X; 
				    ny-=nx; nx=0;
				}   
			}
	    } 
		B[0]=Y; B[1]=ny;
	}
}


void free_Data(FRAME_Data *Frame)
{
    SENTENCE_Data *Snt; int np;

    Snt = Frame->Sentence;

    if(Snt==NULL) return;
    if(Snt->Text!=NULL) free(Snt->Text);

    if(Snt->Prosody!=NULL) {
		np=Snt->Prosody->Nphone+2;
		if(Snt->Prosody->Phone!=NULL) free(Snt->Prosody->Phone); 
		if(Snt->Prosody->Dur!=NULL) free(Snt->Prosody->Dur);
		if(Snt->Prosody->Ptch!=NULL) free_cmatrix(Snt->Prosody->Ptch,np,3);
		if(Snt->Prosody->Energy!=NULL) free_cmatrix(Snt->Prosody->Energy,np,3);
		free(Snt->Prosody); 
	}

    if(Snt->Video!=NULL) free(Snt->Video);
  
	if(Snt->Lip_shape!=NULL) {	
		if(Snt->Lip_shape->LinSnt!=NULL) free(Snt->Lip_shape->LinSnt);
		if(Snt->Lip_shape->Lshape!=NULL) free(Snt->Lip_shape->Lshape);
		free(Snt->Lip_shape); 
	} 
    if(Snt!=NULL) { 
		free(Snt); 
		Frame->Sentence = NULL; 
	}

}


