#include "defs.h"
#include "UIDevice.h"
#include "SysCoder.h"

DeviceFilter::DeviceFilter() : m_nDeviceId(0) {
}

DeviceFilter::~DeviceFilter() 
{
}

struct deviceDesc* DeviceFilter :: LockDevice (const DeviceListener* pDeviceListener) 
{
	int i;
	struct deviceDesc* pDeviceDesc=NULL;

	m_smpDevices.P();
	for (i = m_loadedDevices.GetLength (); --i >= 0;)
	{
		if (m_loadedDevices[i]->m_pDeviceListener == pDeviceListener)
		{
			pDeviceDesc = m_loadedDevices[i];
			pDeviceDesc->m_mtxDevice.P();
			pDeviceDesc->m_nUse++;
			pDeviceDesc->m_mtxDevice.V();
			m_smpDevices.V();
			break;
		}
	}
	if (pDeviceDesc==NULL)
		m_smpDevices.V();
	return (pDeviceDesc);
}

struct deviceDesc* DeviceFilter :: LockDevice (const UINT nDeviceId) 
{
	int i;
	struct deviceDesc* pDeviceDesc=NULL;

	m_smpDevices.P();
	for (i = m_loadedDevices.GetLength (); --i >= 0;)
	{
		if (m_loadedDevices[i]->m_nDeviceId == nDeviceId)
		{
			pDeviceDesc = m_loadedDevices[i];
			pDeviceDesc->m_mtxDevice.P();
			pDeviceDesc->m_nUse++;
			pDeviceDesc->m_mtxDevice.V();
			m_smpDevices.V();
			break;
		}
	}
	if (pDeviceDesc==NULL)
		m_smpDevices.V();
	return (pDeviceDesc);
}

void DeviceFilter :: UnlockDevice (struct deviceDesc* pDeviceDesc) 
{
	m_smpDevices.P();
	pDeviceDesc->m_mtxDevice.P();
	pDeviceDesc->m_nUse--;
	pDeviceDesc->m_mtxDevice.V();
	m_smpDevices.V();
}

void DeviceFilter :: DeviceListener_Attach_Req(	DeviceListener*			pDeviceListener,
												const char*				szDeviceName,
												const BYTE*				devSpecInfo)
{
	int i;
	UINT attachedId = 0;
	DEVICE_STATUS eStatus = NULL_DEVICE; 
	
	BOOL bFound = false;

	m_smpDevices.P ();
	/*-------------------------------------------------------------
	Find which plugin can handle this deviceName. 
	First in the loaded plugins (usually keyboard, mouse)
	-------------------------------------------------------------*/
	for (i = m_loadedDevices.GetLength (); --i >= 0;)
	{
		DevicePlugin* pLoadedDevicePlugin = m_loadedDevices[i]->m_pDevicePlugin;
		if (pLoadedDevicePlugin -> IsYourName (szDeviceName))
		{	
			m_loadedDevices[i] -> m_pDeviceListener = pDeviceListener;
			m_loadedDevices[i] -> m_eStatus			= PLUGIN_AND_LISTENER_ATTACHED;
			m_loadedDevices[i] -> m_szDeviceName	= new char[strlen(szDeviceName) + 1]; strcpy(m_loadedDevices[i] -> m_szDeviceName	, szDeviceName);
			if (devSpecInfo) {
				m_loadedDevices[i] -> m_devSpecInfo		= new BYTE[sizeof(devSpecInfo)/sizeof(BYTE)]; memcpy(m_loadedDevices[i] -> m_devSpecInfo, devSpecInfo, sizeof(devSpecInfo));
			}

			attachedId	= m_loadedDevices[i] -> m_nDeviceId;
			eStatus		= m_loadedDevices[i] -> m_eStatus;
			bFound = true;
			break;
		}
	}

	/*-------------------------------------------------------------
	If none of the loaded plugins can handle this deviceName
	Keep track of this need
	-------------------------------------------------------------*/
	if (i < 0)
	{	
		struct deviceDesc* pDeviceDesc	= new(struct deviceDesc);
		pDeviceDesc->m_nUse				= 0;
		pDeviceDesc->m_pDeviceListener	= pDeviceListener;
		pDeviceDesc->m_pDevicePlugin	= NULL;
		pDeviceDesc->m_nDeviceId		= m_nDeviceId++;
		pDeviceDesc->m_szDeviceName		= new char[strlen(szDeviceName) + 1]; strcpy(pDeviceDesc->m_szDeviceName, szDeviceName);
		pDeviceDesc->m_devSpecInfo		= new BYTE[sizeof(devSpecInfo)/sizeof(BYTE)]; memcpy(m_loadedDevices[i] -> m_devSpecInfo, devSpecInfo, sizeof(devSpecInfo));
		pDeviceDesc->m_eStatus			= LISTENER_ATTACHED_ONLY;
		m_loadedDevices += pDeviceDesc;

		attachedId	= pDeviceDesc->m_nDeviceId;
		eStatus		= pDeviceDesc->m_eStatus;
		bFound = true;
	} 

	/*-------------------------------------------------------------
	Send the Id and the status back
	-------------------------------------------------------------*/
	if (!bFound) {
		eStatus    = DEVICE_UNKNOWN_ERROR;
	}

	pDeviceListener -> DeviceListener_Attach_Cnf(attachedId, eStatus);

	m_smpDevices.V ();

}

void DeviceFilter :: DeviceListener_Detach_Req(	const UINT	nDeviceId) {

	deviceDesc* pDeviceDesc=LockDevice(nDeviceId);

	if (pDeviceDesc)
	{
		pDeviceDesc->m_pDeviceListener = NULL;

		if (pDeviceDesc->m_eStatus == PLUGIN_AND_LISTENER_ATTACHED) {
			pDeviceDesc->m_eStatus	= PLUGIN_ATTACHED_ONLY;
		} else if (pDeviceDesc->m_eStatus == PLUGIN_ATTACHED_ONLY) {
			pDeviceDesc->m_eStatus	= NULL_DEVICE;
		}

		UnlockDevice(pDeviceDesc);
	}
}

void DeviceFilter :: DevicePlugin_Attach_Req(DevicePlugin *pDevicePlugin) {

	int i;
	UINT attachedId = 0;
	DEVICE_STATUS eStatus = NULL_DEVICE;
	BOOL bFound = false;

	m_smpDevices.P ();

	/*-------------------------------------------------------------
	Find if a Listener is waiting for this Plugin
	-------------------------------------------------------------*/
	for (i = m_loadedDevices.GetLength (); --i >= 0;)
	{
		if (m_loadedDevices[i] -> m_eStatus	== LISTENER_ATTACHED_ONLY) {
			if (pDevicePlugin->IsYourName(m_loadedDevices[i] ->m_szDeviceName)) {

				m_loadedDevices[i] -> m_pDevicePlugin = pDevicePlugin;
				m_loadedDevices[i] -> m_eStatus		  = PLUGIN_AND_LISTENER_ATTACHED;
				attachedId	= m_loadedDevices[i] -> m_nDeviceId;
				eStatus		= m_loadedDevices[i] -> m_eStatus;
				bFound = true;
				break;
			}
		}
	}

	/*-------------------------------------------------------------
	If none of the existing name can be handled by this Plugin
	-------------------------------------------------------------*/
	if (i < 0)
	{	
		struct deviceDesc* pDeviceDesc	= new(struct deviceDesc);
		pDeviceDesc->m_nUse				= 0;
		pDeviceDesc->m_pDeviceListener	= NULL;
		pDeviceDesc->m_pDevicePlugin	= pDevicePlugin;
		pDeviceDesc->m_nDeviceId		= m_nDeviceId++;
		pDeviceDesc->m_szDeviceName		= "";
		pDeviceDesc->m_eStatus			= PLUGIN_ATTACHED_ONLY;
		m_loadedDevices += pDeviceDesc;

		attachedId	= pDeviceDesc->m_nDeviceId;
		eStatus		= pDeviceDesc->m_eStatus;
		bFound = true;

	} 

	/*-------------------------------------------------------------
	Send the Id and the status back
	-------------------------------------------------------------*/	
	if (!bFound) {
		eStatus    = DEVICE_UNKNOWN_ERROR;
	}

	pDevicePlugin -> DevicePlugin_Attach_Cnf(attachedId, eStatus);

	
	m_smpDevices.V ();

}

void DeviceFilter :: DevicePlugin_Detach_Req(const UINT nDeviceId) 
{
	deviceDesc *pDeviceDesc = LockDevice(nDeviceId);

	if (pDeviceDesc) {
		pDeviceDesc->m_pDevicePlugin = NULL;

		if (pDeviceDesc->m_eStatus == PLUGIN_AND_LISTENER_ATTACHED) {
			pDeviceDesc->m_eStatus	= LISTENER_ATTACHED_ONLY;
		} else if (pDeviceDesc->m_eStatus == PLUGIN_ATTACHED_ONLY) {
			pDeviceDesc->m_eStatus	= NULL_DEVICE;
		}

		UnlockDevice(pDeviceDesc);
		delete pDeviceDesc;
	}
}

void DeviceFilter :: DataReceive_Ind(const UINT		nDeviceId,
									 const BYTE*	pStreamData,
									 const int		nStreamDataLength,
									 const int		nErrorFlag) {

	deviceDesc* pDeviceDesc=LockDevice(nDeviceId);

	if (pDeviceDesc)
	{
		if (pDeviceDesc -> m_pDeviceListener) {
			pDeviceDesc -> m_pDeviceListener -> DataReceive_Ind(pStreamData,nStreamDataLength,nErrorFlag);
		}
		
		UnlockDevice(pDeviceDesc);
	}

	return;
}

DeviceListener::DeviceListener() : m_pDeviceFilter(NULL) {
}

/* --------------------------------------------------------------
-					 KeySensor Plug-in				    -
---------------------------------------------------------------*/
LPCSTR KeySensorInstance::m_szDeviceName = "KeySensor";

void KeySensorInstance::DataReceive_Ind(const UINT nKeyPress, 
											 const UINT nKeyRelease, 
											 const UINT nActionKeyPress,
											 const UINT nActionKeyRelease,
											 const UINT bShiftKey_Changed,
											 const UINT bControlKey_Changed,
											 const UINT bAltKey_Changed
											 ) 
{
	
	SysCoder coder(NULL, NULL, NULL, 0, 0);
	coder.Reset();
	// KeyPress or KeyRelease and optionally (nActionKeyPress or nActionKeyRelease) and 3
	int nValues = (nKeyPress || nKeyRelease) + 
				  (nActionKeyPress || nActionKeyRelease) +
				  (bShiftKey_Changed != 2) + 
				  (bControlKey_Changed != 2) + 
				  (bAltKey_Changed != 2);
	coder.EncodeInt(nValues,8,"number of values");
	if (nKeyPress) 
	{
		coder.EncodeInt(0,8,"field1");
		coder.EncodeInt(3,6,"fieldType");
		coder.EncodeInt(nKeyPress,32,"keyPress");
	}
	if (nKeyRelease) 
	{
		coder.EncodeInt(1,8,"field2");
		coder.EncodeInt(3,6,"fieldType");
		coder.EncodeInt(nKeyRelease,32,"keyRelease");
	}
	if (nActionKeyPress) 
	{
		coder.EncodeInt(2,8,"field3");
		coder.EncodeInt(3,6,"fieldType");
		coder.EncodeInt(nActionKeyPress,32,"actionKeyPress");
	}
	if (nActionKeyRelease) 
	{
		coder.EncodeInt(3,8,"field4");
		coder.EncodeInt(3,6,"fieldType");
		coder.EncodeInt(nActionKeyRelease,32,"actionKeyRelease");
	}
	if (bShiftKey_Changed != 2) 
	{
		coder.EncodeInt(4,8,"field5");
		coder.EncodeInt(0,6,"fieldType");
		coder.EncodeInt(bShiftKey_Changed,1,"shiftKey_changed");
	}
	if (bControlKey_Changed != 2) 
	{
		coder.EncodeInt(5,8,"field6");
		coder.EncodeInt(0,6,"fieldType");
		coder.EncodeInt(bControlKey_Changed,1,"controlKey_changed");
	}
	if (bAltKey_Changed != 2) 
	{
		coder.EncodeInt(6,8,"field7");
		coder.EncodeInt(0,6,"fieldType");
		coder.EncodeInt(bAltKey_Changed,1,"altKey_changed");
	}
	coder.Align();

	m_pDeviceFilter->DataReceive_Ind(m_nDeviceId,coder.GetData(),coder.GetDataLength(),0);
}

BOOL KeySensorInstance::IsYourName(const char* szDeviceName) {
	if (_strnicmp( szDeviceName, m_szDeviceName, strlen(m_szDeviceName) ) == 0) {
		return TRUE;
	}
	return FALSE;
}
/* --------------------------------------------------------------
-					  StringSensor Plug-in				    -
---------------------------------------------------------------*/
LPCSTR StringSensorInstance::m_szDeviceName = "StringSensor";
ZString StringSensorInstance::enteredText = "";

void StringSensorInstance::DataReceive_Ind(const ZString &string) {
	
	SysCoder coder(NULL, NULL, NULL, 0, 0);
	coder.Reset();
	if (string == ZString("\b")) {
		if (enteredText.GetLength() > 0) enteredText.DeleteFrom(enteredText.GetLength()-1);
		coder.EncodeInt(1,8,"number of values");
		coder.EncodeInt(0,8,"field1");
		coder.EncodeInt(4,6,"fieldType");
		coder.EncodeSFString(enteredText);
	} else if (string == ZString("\r")) {
		coder.EncodeInt(2,8,"number of values");
		coder.EncodeInt(0,8,"field1");
		coder.EncodeInt(4,6,"fieldType");
		coder.EncodeSFString("");
		coder.EncodeInt(1,8,"field2");
		coder.EncodeInt(4,6,"fieldType");
		coder.EncodeSFString(enteredText);
		enteredText = "";
	} else {
		enteredText += string;
		coder.EncodeInt(1,8,"number of values");
		coder.EncodeInt(0,8,"field1");
		coder.EncodeInt(4,6,"fieldType");
		coder.EncodeSFString(enteredText);
	}
	coder.Align();
	m_pDeviceFilter->DataReceive_Ind(m_nDeviceId,coder.GetData(),coder.GetDataLength(),0);
}

BOOL StringSensorInstance::IsYourName(const char* szDeviceName) {
	if (_strnicmp( szDeviceName, m_szDeviceName, strlen(m_szDeviceName) ) == 0) {
		return TRUE;
	}
	return FALSE;
}

/* --------------------------------------------------------------
-					 Mouse  Plug-in								-
---------------------------------------------------------------*/
LPCSTR MouseSensorInstance::m_szDeviceName = "Mouse";

void MouseSensorInstance::DataReceive_Ind(float X, float Y, const float Wheel, BOOL bDown, const int ButtonType)
{
	static float oldX;
	static float oldY;
	int buttonPressed = 0;
	int buttonReleased = 0;
	// SOLARIS PORTING
#ifdef WIN32
	switch (ButtonType) {
	case MK_LBUTTON:
		if (bDown) 
			buttonPressed = 1;
		else
			buttonReleased = 1;
		break;
	case MK_RBUTTON:
		if (bDown) 
			buttonPressed = 2;
		else
			buttonReleased = 2;
		break;
	case MK_MBUTTON:
		if (bDown) 
			buttonPressed = 3;
		else
			buttonReleased = 3;
		break;
	}
#endif
	
	int nElseValues = (Wheel != 0) + (buttonPressed != 0) + (buttonReleased != 0);
	int nValues = ( (X != oldX) || (Y != oldY) ) + nElseValues;

	if (nValues) 
	{
		SysCoder coder(NULL, NULL, NULL, 0, 0);
		coder.Reset();
		coder.EncodeInt(nElseValues + 1,8,"number of values");

		coder.EncodeInt(0,8,"field1");
		coder.EncodeInt(6,6,"fieldType");
		coder.EncodeFloat(X,"x");
		coder.EncodeFloat(Y,"y");

		if (Wheel != 0) {
			coder.EncodeInt(1,8,"field2");
			coder.EncodeInt(1,6,"fieldType");
			coder.EncodeFloat(Wheel,"wheel");
		}

		if (buttonPressed != 0) {
			coder.EncodeInt(2,8,"field3");
			coder.EncodeInt(0,6,"fieldType");
			coder.EncodeInt(buttonPressed,2,"button_pressed");
		}

		if (buttonReleased != 0) {
			coder.EncodeInt(3,8,"field4");
			coder.EncodeInt(0,6,"fieldType");
			coder.EncodeInt(buttonReleased,2,"button_released");
		}
		coder.Align();

		m_pDeviceFilter ->DataReceive_Ind(m_nDeviceId,coder.GetData(),coder.GetDataLength(),0);
	}
	oldX = X;
	oldY = Y;
}


BOOL MouseSensorInstance::IsYourName(const char* szDeviceName)
{
	if (_strnicmp( szDeviceName, m_szDeviceName, strlen(m_szDeviceName) ) == 0) {
		return TRUE;
	}
	return FALSE;
}

