/**************************************************************************
 * Project Im1Player
 * INTERNATIONAL STANDARD ORGANIZATION
 * ISO/IEC JTC1/SC29/WG11
 * 
 * SUBSYSTEM: 
 * FILE:      Device.h
 * AUTHOR:    Cyril Concolato (ENST/COMELEC cyril.concolato@enst.fr )
 * OVERVIEW:  
 * 
 **************************************************************************/

#ifndef _Device_h_
#define _Device_h_

#include "defs.h"
#include "ztl.h"

typedef unsigned short	u_short;

typedef enum statusParameter
{
	NULL_DEVICE						= 0x0,
	PLUGIN_ATTACHED_ONLY			= 0x1,
	LISTENER_ATTACHED_ONLY			= 0x2,
	PLUGIN_AND_LISTENER_ATTACHED	= 0x3,
	DEVICE_UNKNOWN_ERROR			= 0xF
} DEVICE_STATUS;

class VisualRenderer;
class DeviceFilter;

class DeviceListener
{
public:

	DeviceListener();

	// Functions for data reception
	virtual void DataReceive_Ind(
		const BYTE*		pStreamData,
		const int		nStreamDataLength,
		const int		nErrorFlag
	) = 0;

	virtual void DeviceListener_Attach_Cnf(UINT attachedId, DEVICE_STATUS eStatus){
		m_nDeviceId = attachedId; 
		eStatus;
	};

	UINT m_nDeviceId;
	DeviceFilter *m_pDeviceFilter;
};

class DevicePlugin {
public:
	UINT m_nDeviceId;
	DeviceFilter *m_pDeviceFilter;
	virtual void DevicePlugin_Attach_Cnf(UINT attachedId, DEVICE_STATUS eStatus) {
		m_nDeviceId = attachedId; 
		eStatus;
	};
	void SetDeviceFilter( DeviceFilter *pDeviceFilter) { m_pDeviceFilter= pDeviceFilter; }
	virtual BOOL IsYourName(const char* szDeviceName) = 0;
};

class KeySensorInstance : public DevicePlugin
{
public:
	void DataReceive_Ind(const UINT nKeyPress, 
						 const UINT nKeyRelease, 
						 const UINT nActionKeyPress,
						 const UINT nActionKeyRelease,
						 const UINT bShiftKey_Changed,
						 const UINT bControlKey_Changed,
						 const UINT bAltKey_Changed
						 );
	virtual BOOL IsYourName(const char* szDeviceName);
private:
	static LPCSTR m_szDeviceName;
};

class StringSensorInstance : public DevicePlugin
{
public:
	void DataReceive_Ind(const ZString &string);
	virtual BOOL IsYourName(const char* szDeviceName);
private:
	static LPCSTR m_szDeviceName;
	static ZString enteredText;
};

class MouseSensorInstance : public DevicePlugin
{
public:
	//X and Y are in BIFS coordinates
	void DataReceive_Ind(float X, float Y, const float Wheel = 0.0f, const BOOL bDown = FALSE, const int ButtonType=0);
	virtual BOOL IsYourName(const char* szDeviceName);

private:
	static LPCSTR m_szDeviceName;
};

struct deviceDesc
{
	UINT				m_nDeviceId;
	UINT				m_nUse;
	char*				m_szDeviceName;
	BYTE*				m_devSpecInfo;
	DeviceListener*		m_pDeviceListener;
	DevicePlugin*		m_pDevicePlugin;
	ZMutex				m_mtxDevice;
	DEVICE_STATUS		m_eStatus;
};

class DeviceFilter
{
protected:
	ZSemaphore	m_smpDevices;
	ZArray<struct deviceDesc *> m_loadedDevices;
	struct deviceDesc* LockDevice(const DeviceListener* pDeviceListener);
	struct deviceDesc* LockDevice(const UINT nDeviceId);
	void UnlockDevice(struct deviceDesc* pDeviceDesc);
	UINT m_nDeviceId;
	VisualRenderer *m_pVisualRenderer;

public:
	DeviceFilter();
	~DeviceFilter();

	void DeviceListener_Attach_Req(	DeviceListener*	pDeviceListener,
									const char*		szDeviceName,
									const BYTE*		devSpecInfo);
	void DeviceListener_Detach_Req(	const UINT	nDeviceId);

	void DevicePlugin_Attach_Req( DevicePlugin *pDevicePlugin);
	void DevicePlugin_Detach_Req( const UINT nDeviceId);

	void DataReceive_Ind( const UINT	nDeviceId,
						  const BYTE*	pStreamData,
						  const int		nStreamDataLength,
						  const int		nErrorFlag);

	void SetVisualRenderer(VisualRenderer *pVisualRenderer) {
		m_pVisualRenderer = pVisualRenderer;
	}
	VisualRenderer *GetVisualRenderer() {
		return m_pVisualRenderer;
	}
};

#endif  
