/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

  SUBSYSTEM:  Decoder
  FILE:       UIStreamDec.h
  AUTHOR:     Cyril Concolato (ENST/COMELEC)
  OVERVIEW:   Definition of UIStreamDecoder  
-------------------------------------------------------------------------*/
#ifndef UISTREAMDEC_H 
#define UISTREAMDEC_H

#include "SysCoder.h"
#include "Nodes.h"
#include "NodeFld.h"
#include "UIDevice.h"
#include "bifs.h"
#include "MediaObj.h"


class InlineImp;
class InputSensorImp;
class Root;
class Executive;
class MemBIFSDecoder;

typedef NodeField* NodeFieldPtr;
typedef MediaObject* MediaObjectPtr;
typedef const BifsFieldTable* EntryPtr;

class UIStreamDecoder : public SysCoder, public DeviceListener
{
public:
	UIStreamDecoder(Executive *pExecutive, 
					InlineImp *pSource, 
					FILE *fList,
					LPCSTR szListTitle,
					int flags,
					int objectTypeIndication);
	~UIStreamDecoder();
	bool Configure (const BYTE *decspecificInfo, int nInfoLength);

	Executive *GetExecutive () const {return m_pExecutive;}
	InlineImp *GetSource () const {return m_pSource;}
	Root *GetScene () const {return m_pScene;}
	void UpdateBuffer(const BYTE *nodeBuffer, int nodeBufferLength);
	
	void SetNode(InputSensorImp *pNode) { m_pNode = pNode; }
	InputSensorImp* GetNode() { return m_pNode; }
	
	void SetFields(NodeField *pField, MediaObject *pMediaObject, const BifsFieldTable* pEntry, int nPos = 0);
	virtual void DataReceive_Ind(const BYTE* pStreamData, const int	nStreamDataLength, const int nErrorFlag);
	void Terminate() {
		if (m_pDeviceFilter) m_pDeviceFilter->DeviceListener_Detach_Req(m_nDeviceId);
	}

private:
	virtual void ParseFrame (LPCBYTE pData, int nLength, DWORD dwTS);
	void ExecuteUpdate();
	void UpdateDecoder();

	LPCSTR m_szDeviceName;
	LPCBYTE m_devSpecInfo;
	int m_nbOfFields;
	NodeFieldPtr *pFields;
	MediaObjectPtr *pMediaObjects;
	EntryPtr *pEntries;
	int *pNPos;

	int currentField;

	Root *m_pScene;
	Executive *m_pExecutive;
	InlineImp *m_pSource;
	MemBIFSDecoder *m_pBufferDecoder;
	InputSensorImp *m_pNode;
	int m_nNodeBufferLength;
	BYTE * m_pNodeBuffer;
	ZSemaphore semBuffer;
};

#endif                                      // UISTREAMDEC_H
