<?xml version="1.0"?>

<!-- This XSLT file is courtesy of Lyndon Nixon/GMD FOKUS and should not be changed without
the authors permission; it may be used as long as the author is credited. Lyndon J Nixon can be 
contacted at nixon@fokus.gmd.de -->

<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform">

<xsl:output
  method = "text" 
  omit-xml-declaration = "yes"
  indent = "yes" /> 

<xsl:key name="ODids" match="ObjectDescriptor | InitialObjectDescriptor"
	use="@objectDescriptorID" />
<xsl:key name="ESids" match="ES_Descriptor" use="@ES_ID" />
<xsl:key name="IPMPids" match="IPMP_Descriptor" use="@IPMP_DescriptorID"/>


<xsl:template match="/">
 <xsl:apply-templates select="XMT-A/Header/*"/>
</xsl:template>

<xsl:template match="meta">
<!-- metadata coding to be added -->
</xsl:template>

<xsl:template match="ObjectDescriptor">ObjectDescriptor {

<xsl:if test="@objectDescriptorID">objectDescriptorID  <xsl:value-of select="@binaryID"/><xsl:text>
</xsl:text></xsl:if>

<xsl:apply-templates select="URL | Descr | extDescr"/>

} 

</xsl:template>

<xsl:template match="URL">
<xsl:if test="@URLstring"> URLstring  <xsl:value-of select="@URLstring"/>
</xsl:if>
</xsl:template>

<xsl:template match="Descr">
<xsl:apply-templates select="esDescr | ociDescr | ipmpDescrPtr"/>
</xsl:template>
<xsl:template match="esDescr">esDescr [

<xsl:apply-templates select="ES_Descriptor"/>

]

</xsl:template>
<xsl:template match="ociDescr">ociDescr [

<xsl:apply-templates select="ContentClassificationDescriptor|KeyWordDescriptor
|RatingDescriptor|LanguageDescriptor|ShortTextualDescriptor|ExpandedTextualDescriptor
|ContentCreatorNameDescriptor|ContentCreationDateDescriptor|OCICreatorNameDescriptor
|OCICreationDateDescriptor|SMPTECameraPositionDescriptor"/>

]

</xsl:template>

<xsl:template match="InitialObjectDescriptor">InitialObjectDescriptor {

<xsl:if test="@objectDescriptorID">objectDescriptorID  <xsl:value-of select="@binaryID"/><xsl:text>
</xsl:text></xsl:if>
<xsl:apply-templates select="Profiles | URL"/>
<xsl:apply-templates select="Descr"/>
<xsl:apply-templates select="extDescr"/>
} 

</xsl:template>

<xsl:template match="Profiles">
<xsl:if test="@ODProfileLevelIndication"> ODProfileLevelIndication  <xsl:value-of select="@ODProfileLevelIndication"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@sceneProfileLevelIndication"> sceneProfileLevelIndication  <xsl:value-of select="@sceneProfileLevelIndication"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@audioProfileLevelIndication"> audioProfileLevelIndication  <xsl:value-of select="@audioProfileLevelIndication"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@visualProfileLevelIndication"> visualProfileLevelIndication  <xsl:value-of select="@visualProfileLevelIndication"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@graphicsProfileLevelIndication"> graphicsProfileLevelIndication  <xsl:value-of select="@graphicsProfileLevelIndication"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@includeInlineProfileLevelFlag"> includeInlineProfileLevelFlag  <xsl:value-of select="@includeInlineProfileLevelFlag"/><xsl:text>
</xsl:text>
</xsl:if>
</xsl:template>

<xsl:template match="ES_Descriptor"> ES_Descriptor {
<xsl:if test="@ES_ID">ES_ID   <xsl:value-of select="@binaryID"/><xsl:text>
</xsl:text></xsl:if>
<xsl:if test="@streamPriority"> streamPriority  <xsl:value-of select="@streamPriority"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@OCR_ES_ID"> OCR_ES_ID  <xsl:value-of select="key('ESids',@OCR_ES_ID)/@binaryID"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@dependsOn_ES_ID"> dependsOn_ES_ID  <xsl:value-of select="key('ESids',@dependsOn_ES_ID)/@binaryID"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:apply-templates select="URL"/>
<xsl:apply-templates select="StreamSource | decConfigDescr | slConfigDescr | ipiPtr | ipIDs | ipmpDescrPtr | langDescr | regDescr | qosDescr"/>
}
</xsl:template>
<xsl:template match="decConfigDescr">decConfigDescr {

<xsl:apply-templates select="DecoderConfigDescriptor"/>

}  

</xsl:template>
<xsl:template match="slConfigDescr">slConfigDescr {

<xsl:apply-templates select="SLConfigDescriptor"/>

} 

</xsl:template>
<xsl:template match="ipiPtr">ipiPtr 

<xsl:apply-templates select="IPI_DescrPointer"/>

</xsl:template>
<xsl:template match="ipIDs">ipIDs [

<xsl:apply-templates select="ContentIdentificationDescriptor | SupplementaryContentIdentificationDescriptor"/>

]

</xsl:template>

<xsl:template match="ipmpDescr">ipmpDescr [

<xsl:apply-templates select="IPMP_Descriptor"/>

]
</xsl:template>

<xsl:template match="ipmpDescrPtr">ipmpDescrPtr [

<xsl:apply-templates select="IPMP_Descriptor | IPMP_DescriptorPointer"/>

]

</xsl:template>
<xsl:template match="extDescr">extDescr [

<xsl:apply-templates select="ExtensionDescriptor"/>

]

</xsl:template>

<xsl:template match="langDescr">langDescr [

<xsl:apply-templates select="LanguageDescriptor"/>

]

</xsl:template>
<xsl:template match="regDescr">regDescr 

<xsl:apply-templates select="RegistrationDescriptor"/>

</xsl:template>
<xsl:template match="qosDescr">qosDescr 

<xsl:apply-templates select="QoS_Descriptor"/>

</xsl:template>

<xsl:template match="URL">
<xsl:if test="URLstring"> URLstring  <xsl:value-of select="URLstring"/>
</xsl:if>
</xsl:template>

<xsl:template match="DecoderConfigDescriptor">
<xsl:if test="@objectTypeIndication"> objectTypeIndication  <xsl:value-of select="@objectTypeIndication"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@streamType"> streamType  <xsl:value-of select="@streamType"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@upStream"> upStream  <xsl:value-of select="@upStream"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@bufferSizeDB"> bufferSizeDB  <xsl:value-of select="@bufferSizeDB"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@maxBitrate"> maxBitrate  <xsl:value-of select="@maxBitrate"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@avgBitrate"> avgBitrate  <xsl:value-of select="@avgBitrate"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:apply-templates select="decSpecificInfo | profileLevelIndicationIndexDescr"/>
</xsl:template>

<xsl:template match="decSpecificInfo">
<xsl:apply-templates select="BIFSConfig | BIFSv2Config | DecoderSpecificInfo"/>
</xsl:template>

<!-- these are currently not supported by BIFSEnc

<xsl:template match="profileLevelIndicationIndexDescr">profileLevelIndicationIndexDescr [

<xsl:apply-templates select="ExtensionProfileLevelDescriptor"/>

]

</xsl:template>

<xsl:template match="ExtensionProfileLevelDescriptor">ExtensionProfileLevelDescriptor {

<xsl:if test="@profileLevelIndicationIndex">profileLevelIndicationIndex   <xsl:value-of select="@binaryID"/><xsl:text>
</xsl:text></xsl:if>

<xsl:if test="@ODProfileLevelIndication"> ODProfileLevelIndication  <xsl:value-of select="@ODProfileLevelIndication"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@sceneProfileLevelIndication"> sceneProfileLevelIndication  <xsl:value-of select="@sceneProfileLevelIndication"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@audioProfileLevelIndication"> audioProfileLevelIndication  <xsl:value-of select="@audioProfileLevelIndication"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@visualProfileLevelIndication"> visualProfileLevelIndication  <xsl:value-of select="@visualProfileLevelIndication"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@graphicsProfileLevelIndication"> graphicsProfileLevelIndication  <xsl:value-of select="@graphicsProfileLevelIndication"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@MPEGJLevelIndication"> MPEGJProfileLevelIndication  <xsl:value-of select="@MPEGJLevelIndication"/><xsl:text>
</xsl:text>
</xsl:if>

} 

</xsl:template>

-->

<xsl:template match="DecoderSpecificInfo">
<!-- the attributes type and src can not be handled by XSLT. Rather than passing data
from a separate file, string details can be passed to the encoder with the param element.
This seems to differ from the intentions of XMT, which refers to passing XML data -->
decSpecificInfo DecoderSpecificInfoString {
<xsl:apply-templates select="param"/>
}
</xsl:template>

<xsl:template match="param">
 <xsl:value-of select="@name"/><xsl:text> </xsl:text>" <xsl:value-of select="@value"/>"<xsl:text>
</xsl:text>
</xsl:template>

<xsl:template name="BIFSConfig">

<xsl:if test="@nodeIDbits"> nodeIDbits  <xsl:value-of select="@nodeIDbits"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@routeIDbits"> routeIDbits  <xsl:value-of select="@routeIDbits"/><xsl:text>
</xsl:text>
</xsl:if>

<xsl:apply-templates select="commandStream | animMask"/>

</xsl:template>

<xsl:template match="BIFSConfig">decSpecificInfo BIFSConfig {
<xsl:call-template name="BIFSConfig" />
}
</xsl:template>

<xsl:template match="BIFSv2Config">decSpecificInfo BIFSv2Config {

<xsl:if test="@use3DMeshCoding"> use3DMeshCoding  <xsl:value-of select="@use3DMeshCoding"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@usePredictiveMFField"> usePredictiveMFField  <xsl:value-of select="@usePredictiveMFField"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@PROTOIDbits"> PROTOIDbits  <xsl:value-of select="@PROTOIDbits"/><xsl:text>
</xsl:text>
</xsl:if>

<xsl:call-template name="BIFSConfig" />

} 
</xsl:template>

<xsl:template match="animMask">
<xsl:if test="@randomAccess"> randomAccess  <xsl:value-of select="@randomAccess"/><xsl:text>
</xsl:text>
</xsl:if>
animMask() {
<!-- omitted -->
} 

</xsl:template>
<xsl:template match="commandStream">
<xsl:if test="@pixelMetric"> pixelMetric  <xsl:value-of select="@pixelMetric"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:apply-templates select="size"/>
</xsl:template>

<xsl:template match="size">
<xsl:if test="@pixelWidth"> pixelWidth  <xsl:value-of select="@pixelWidth"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@pixelHeight"> pixelHeight  <xsl:value-of select="@pixelHeight"/><xsl:text>
</xsl:text>
</xsl:if>
</xsl:template>

<xsl:template match="SLConfigDescriptor">

<xsl:apply-templates select="predefined | custom"/>

</xsl:template>
<xsl:template match="predefined">
<xsl:if test="@value"> predefined  <xsl:value-of select="@value"/><xsl:text>
</xsl:text>
</xsl:if>
</xsl:template>

<xsl:template match="custom">
<xsl:if test="@useAccessUnitStartFlag"> useAccessUnitStartFlag  <xsl:value-of select="@useAccessUnitStartFlag"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@useAccessUnitEndFlag"> useAccessUnitEndFlag  <xsl:value-of select="@useAccessUnitEndFlag"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@useRandomAccessPointFlag"> useRandomAccessPointFlag  <xsl:value-of select="@useRandomAccessPointFlag"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@useRandomAccessUnitsOnlyFlag"> useRandomAccessUnitsOnlyFlag  <xsl:value-of select="@useRandomAccessUnitsOnlyFlag"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@usePaddingFlag"> usePaddingFlag  <xsl:value-of select="@usePaddingFlag"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@useIdleFlag"> useIdleFlag  <xsl:value-of select="@useIdleFlag"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@timeStampResolution"> timeStampResolution  <xsl:value-of select="@timeStampResolution"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@OCRResolution"> OCRResolution  <xsl:value-of select="@OCRResolution"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@timeStampLength"> timeStampLength  <xsl:value-of select="@timeStampLength"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@OCRLength"> OCRLength  <xsl:value-of select="@OCRLength"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@AU_Length"> AU_Length  <xsl:value-of select="@AU_Length"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@instantBitrateLength"> instantBitrateLength  <xsl:value-of select="@instantBitrateLength"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@degradationPriorityLength"> degradationPriorityLength  <xsl:value-of select="@degradationPriorityLength"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@AU_seqNumLength"> AU_seqNumLength  <xsl:value-of select="@AU_seqNumLength"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@packetAU_seqNumLength"> packetAU_seqNumLength  <xsl:value-of select="@packetAU_seqNumLength"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:apply-templates select="duration | noUseTimeStamps"/>
</xsl:template>

<xsl:template match="duration">
<xsl:if test="@timeScale"> timeScale  <xsl:value-of select="@timeScale"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@accessUnitDuration"> accessUnitDuration  <xsl:value-of select="@accessUnitDuration"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@compositionUnitDuration"> compositionUnitDuration  <xsl:value-of select="@compositionUnitDuration"/><xsl:text>
</xsl:text>
</xsl:if>
</xsl:template>
<xsl:template match="noUseTimeStamps">
<xsl:if test="@startDecodingTimeStamp"> startDecodingTimeStamp  <xsl:value-of select="@startDecodingTimeStamp"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@startCompositionTimeStamp"> startCompositionTimeStamp  <xsl:value-of select="@startCompositionTimeStamp"/><xsl:text>
</xsl:text>
</xsl:if>
</xsl:template>
<xsl:template match="ContentIdentificationDescriptor"> ContentIdentificationDescriptor {
<xsl:if test="@protectedContent"> protectedContent  <xsl:value-of select="@protectedContent"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:apply-templates select="contentType | contentIdentifier"/>
}
</xsl:template>
<xsl:template match="contentType">
<xsl:if test="@contentType"> contentType  <xsl:value-of select="@contentType"/><xsl:text>
</xsl:text>
</xsl:if>
</xsl:template>
<xsl:template match="contentIdentifier">
<xsl:if test="@contentIdentifierType"> contentIdentifierType  <xsl:value-of select="@contentIdentifierType"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@contentIdentifier"> contentIdentifier  <xsl:value-of select="@contentIdentifier"/><xsl:text>
</xsl:text>
</xsl:if>
</xsl:template>
<xsl:template match="SupplementaryContentIdentificationDescriptor"> SupplementaryContentIdentificationDescriptor {
<xsl:if test="@languageCode"> languageCode  <xsl:value-of select="@languageCode"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@supplContentIdentifierTitle"> supplContentIdentifierTitle  <xsl:value-of select="@supplContentIdentifierTitle"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@supplContentIdentifierValue"> supplContentIdentifierValue  <xsl:value-of select="@supplContentIdentifierValue"/><xsl:text>
</xsl:text>
</xsl:if>
}
</xsl:template>
<xsl:template match="IPI_DescrPointer"> IPI_DescrPointer {
<xsl:if test="@IPI_ES_Id"> IPI_ES_Id  <xsl:value-of select="key('ESids',@IPI_ES_ID)/@binaryID"/><xsl:text>
</xsl:text>
</xsl:if>
}
</xsl:template>
<xsl:template match="IPMP_DescriptorPointer">IPMP_DescriptorPointer { 
<xsl:if test="@IPMP_DescriptorID"> IPMP_DescriptorID  <xsl:value-of select="key('IPMPids',@IPMP_DescriptorID)/@binaryID"/><xsl:text>
</xsl:text>
</xsl:if>
}
</xsl:template>
<xsl:template match="IPMP_Descriptor">IPMP_Descriptor {
<xsl:if test="@IPMP_DescriptorID">IPMP_DescriptorID   <xsl:value-of select="@binaryID"/><xsl:text>
</xsl:text></xsl:if>
<xsl:apply-templates select="IPMPS_URL | IPMPS_data"/>
}
</xsl:template>

<xsl:template match="IPMPS_URL">
IPMPS_Type  0 <xsl:text>
</xsl:text>
<xsl:if test="@URLString"> URLString  <xsl:value-of select="@URLString"/><xsl:text>
</xsl:text>
</xsl:if>
</xsl:template>
<xsl:template match="IPMPS_data">
<xsl:if test="@type"> IPMPS_Type  <xsl:value-of select="@type"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@IPMPS_data"> IPMP_data  <xsl:value-of select="@IPMPS_data"/><xsl:text>
</xsl:text>
</xsl:if>
</xsl:template>
<xsl:template match="QoS_Descriptor"> QoS_Descriptor {
<xsl:apply-templates select="predefined | qualifiers"/>
}
</xsl:template>

<xsl:template match="qualifiers">
qualifiers [
<xsl:apply-templates select="MaxDelay|PrefMaxDelay|LossProb|MaxGapLoss|MaxAUSize|
AvgAUSize|MaxAURate|RebufferingRatio|QoSCustom"/> ]
</xsl:template>
<xsl:template match="MaxDelay">QoS_Qualifier_MAX_DELAY {
<xsl:if test="@value"> MAX_DELAY <xsl:value-of select="@value"/><xsl:text>
</xsl:text>
</xsl:if>
}
</xsl:template>
<xsl:template match="PrefMaxDelay">QoS_Qualifier_PREF_MAX_DELAY {

<xsl:if test="@value"> PREF_MAX_DELAY <xsl:value-of select="@value"/><xsl:text>
</xsl:text>
</xsl:if>
} 

</xsl:template>
<xsl:template match="LossProb">QoS_Qualifier_LOSS_PROB {

<xsl:if test="@value"> LOSS_PROB  <xsl:value-of select="@value"/><xsl:text>
</xsl:text>
</xsl:if>

} 

</xsl:template>
<xsl:template match="MaxGapLoss">QoS_Qualifier_MAX_GAP_LOSS {

<xsl:if test="@value"> MAX_GAP_LOSS  <xsl:value-of select="@value"/><xsl:text>
</xsl:text>
</xsl:if>

} 

</xsl:template>
<xsl:template match="MaxAUSize">QoS_Qualifier_MAX_AU_SIZE {

<xsl:if test="@value"> MAX_AU_SIZE <xsl:value-of select="@value"/><xsl:text>
</xsl:text>
</xsl:if>

} 

</xsl:template>
<xsl:template match="AvgAUSize">QoS_Qualifier_AVG_AU_SIZE {

<xsl:if test="@value"> AVG_AU_SIZE  <xsl:value-of select="@value"/><xsl:text>
</xsl:text>
</xsl:if>

} 

</xsl:template>
<xsl:template match="MaxAURate">QoS_Qualifier_MAX_AU_RATE {

<xsl:if test="@value"> MAX_AU_RATE  <xsl:value-of select="@value"/><xsl:text>
</xsl:text>
</xsl:if>

} 

</xsl:template>

<!-- not supported by BIFSEnc
<xsl:template match="RebufferingRatio">QoS_Qualifier_REBUFFERING_RATIO {

<xsl:if test="@value"> REBUFFERING_RATIO <xsl:value-of select="@value"/><xsl:text>
</xsl:text>
</xsl:if>

}
</xsl:template>
-->

<!-- reference software does not need to handle provisional extensions

<xsl:template match="QoSCustom">
 <xsl:value-of select="@tag"/> {
 <xsl:value-of select="@customData"/>
} 

</xsl:template>
<xsl:template match="ExtensionDescriptor">
 <xsl:value-of select="@tag"/> {
 <xsl:value-of select="@customData"/>
} 
</xsl:template>
-->

<xsl:template match="RegistrationDescriptor">RegistrationDescriptor {

<xsl:if test="@formatIdentifier"> formatIdentifier  <xsl:value-of select="@formatIdentifier"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@additionalIdentificationInfo"> additionalIdentificationInfo  <xsl:value-of select="@additionalIdentificationInfo"/><xsl:text>
</xsl:text>
</xsl:if>

} 

</xsl:template>
<xsl:template match="ContentClassificationDescriptor">ContentClassificationDescriptor {

<xsl:if test="@classificationEntity"> classificationEntity  <xsl:value-of select="@classificationEntity"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@classificationTable"> classificationTable  <xsl:value-of select="@classificationTable"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@contentClassificationData"> contentClassificationData  <xsl:value-of select="@contentClassificationData"/><xsl:text>
</xsl:text>
</xsl:if>

} 

</xsl:template>
<xsl:template match="KeyWordDescriptor">KeyWordDescriptor {

<xsl:if test="@isUTF8"> isUTF8  <xsl:value-of select="@isUTF8"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@languageCode">languageCode <xsl:value-of select="@languageCode"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:apply-templates select="keyWord"/>

} 

</xsl:template>

<xsl:template match="keyWord">
<xsl:if test="@value">keyWord  <xsl:value-of select="@value"/><xsl:text>
</xsl:text>
</xsl:if>
</xsl:template>

<xsl:template match="RatingDescriptor">RatingDescriptor {

<xsl:if test="@ratingEntity"> ratingEntity  <xsl:value-of select="@ratingEntity"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@ratingCriteria"> ratingCriteria  <xsl:value-of select="@ratingCriteria"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@ratingInfo"> ratingInfo  <xsl:value-of select="@ratingInfo"/><xsl:text>
</xsl:text>
</xsl:if>

} 

</xsl:template>
<xsl:template match="ShortTextualDescriptor">ShortTextualDescriptor {

<xsl:if test="@isUTF8"> isUTF8  <xsl:value-of select="@isUTF8"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@languageCode"> languageCode  <xsl:value-of select="@languageCode"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:apply-templates select="event"/>
} 

</xsl:template>
<xsl:template match="event">
<xsl:if test="name"> eventName  <xsl:value-of select="name"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="text"> eventText  <xsl:value-of select="text"/><xsl:text>
</xsl:text>
</xsl:if>
</xsl:template>

<xsl:template match="ExpandedTextualDescriptor">ExpandedTextualDescriptor {

<xsl:if test="@isUTF8"> isUTF8  <xsl:value-of select="@isUTF8"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@languageCode"> languageCode  <xsl:value-of select="@languageCode"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@nonItemText"> nonItemText  <xsl:value-of select="@nonItemText"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:apply-templates select="item"/>

} 

</xsl:template>
<xsl:template match="item">
<xsl:if test="description"> itemDescription  <xsl:value-of select="description"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="text"> itemText  <xsl:value-of select="text"/><xsl:text>
</xsl:text>
</xsl:if>
</xsl:template>
<xsl:template match="LanguageDescriptor">LanguageDescriptor {

<xsl:if test="@languageCode"> languageCode  <xsl:value-of select="@languageCode"/><xsl:text>
</xsl:text>
</xsl:if>

} 

</xsl:template>
<xsl:template match="ContentCreatorNameDescriptor">ContentCreatorNameDescriptor {

<xsl:apply-templates select="creator"/>

} 

</xsl:template>
<xsl:template match="creator">

<xsl:if test="parent::ContentCreatorNameDescriptor">
<xsl:if test="@language"> languageCode  <xsl:value-of select="@language"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@isUTF8"> isUTF8_string  <xsl:value-of select="@isUTF8"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@name"> contentCreatorName  <xsl:value-of select="@name"/><xsl:text>
</xsl:text>
</xsl:if>
</xsl:if>

<xsl:if test="parent::OCICreatorNameDescriptor">
<xsl:if test="@language"> languageCode  <xsl:value-of select="@language"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@isUTF8"> isUTF8_string  <xsl:value-of select="@isUTF8"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@name"> OCICreatorName  <xsl:value-of select="@name"/><xsl:text>
</xsl:text>
</xsl:if>
</xsl:if>

</xsl:template>
<xsl:template match="ContentCreationDateDescriptor">ContentCreationDateDescriptor {

<xsl:if test="@contentCreationName"> contentCreationDate  <xsl:value-of select="@contentCreationName"/><xsl:text>
</xsl:text>
</xsl:if>

} 

</xsl:template>
<xsl:template match="OCICreatorNameDescriptor">OCICreatorNameDescriptor {

<xsl:apply-templates select="creator"/>

} 

</xsl:template>

<xsl:template match="OCICreationDateDescriptor">OCICreationDateDescriptor {

<xsl:if test="@contentCreationName"> OCICreationDate  <xsl:value-of select="@contentCreationName"/><xsl:text>
</xsl:text>
</xsl:if>

} 

</xsl:template>
<xsl:template match="SMPTECameraPositionDescriptor">SMPTECameraPositionDescriptor {

<xsl:if test="@cameraID"> cameraID  <xsl:value-of select="@cameraID"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:apply-templates select="parameter"/>

} 

</xsl:template>
<xsl:template match="parameter">
<xsl:if test="@id"> parameterID  <xsl:value-of select="@id"/><xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@value"> parameter  <xsl:value-of select="@value"/><xsl:text>
</xsl:text>
</xsl:if>
</xsl:template>

<!-- i have decided to map StreamSource to 'muxInfo', which is a provisional field used
by BIFSEnc to identify the (file) location of the stream and the stream type -->
<xsl:template match="StreamSource"> muxInfo {
fileName  <xsl:value-of select="@url"/><xsl:text>
</xsl:text>
<!-- if no encoding hints are given, it is assumed the stream is of type BIFS. otherwise 
the streamFormat should be declared in the parameter element of sourceFormat or targetFormat
 (name 'streamFormat' and value corresponding to a streamFormat recognised by BIFSEnc) -->
<xsl:if test="not(EncodingHints)">streamFormat BIFS<xsl:text>
</xsl:text></xsl:if>
<xsl:apply-templates select="EncodingHints|BIFSEncodingHints"/>
}
</xsl:template>

<!-- no textual format exists for BIFSEnc to represent EncodingHints -->
<xsl:template match="EncodingHints">
<xsl:apply-templates select="sourceFormat|targetFormat"/>

</xsl:template>
<xsl:template match="sourceFormat">

<xsl:apply-templates select="param"/>

</xsl:template>
<xsl:template match="targetFormat">

<xsl:apply-templates select="param"/> 

</xsl:template>

<!-- no textual format exists for BIFSEnc to represent BIFSEncodingHints 

<xsl:template match="BIFSEncodingHints">BIFSEncodingHints {

<xsl:if test="@SFNodeCoding"> SFNodeCoding " <xsl:value-of select="@SFNodeCoding"/>"<xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@MFListCoding"> MFListCoding " <xsl:value-of select="@MFListCoding"/>"<xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@SFScriptCoding"> SFScriptCoding " <xsl:value-of select="@SFScriptCoding"/>"<xsl:text>
</xsl:text>
</xsl:if>
<xsl:if test="@ROUTESCoding"> ROUTESCoding " <xsl:value-of select="@ROUTESCoding"/>"<xsl:text>
</xsl:text>
</xsl:if>

} 

</xsl:template>

-->


</xsl:stylesheet>
