/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) and extended by Michael Stepping (FernUni Hagen)
and Taher Nasched (FernUni Hagen) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  all
FILE:       ztl.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of the template library for data containers.
Currently includes:

ZArray<type>
ZString
-------------------------------------------------------------------------*/
#include "defs.h"
#include "ztl.h"
#ifdef WIN32
#include <io.h>
#elif defined (__UNIX)
// SOLARIS PORTING
#ifdef __LINUX
#include <sys/io.h>
#elif __SOLARIS
#include <stdio.h>
#endif // SOLARIS PORTING
#endif


//Michael Stepping
#ifdef WIN32
	// links against this lib for the Registry manipulation stuff
	// no longer need to take care of adding the lib to the project settings
	#pragma comment(lib, "advapi32.lib")

	// INLINE
	// implementation of some methods is now "inline". Which means, the body is
	// commented out here for easier history revisions and shifted to "ztl.inl".
	// 
	// some methods can be auto-inlined in _this_ file.
	#ifdef NDEBUG
	#pragma auto_inline(on)
	// warning C4711: function 'xxx' selected for automatic inline expansion
	#pragma warning( disable: 4711 ) 
	#endif
#endif

#ifdef __UNIX
#include "dirent.h"
#endif

/*-----------------------------------------------------------------
ZArrayBase :: Stress () - make sure we have room for nSize+1 elements
-----------------------------------------------------------------*/
void ZArrayBase :: Stress (int nSize)
{
	int nNewSize = (++nSize) * m_nElementSize;

	/*-----------------------------------------------------------------
	If necessary, extend the buffer by 25% or GROW_BY, whatever is larger,
	but not less than what we need.
	-----------------------------------------------------------------*/
	if (nNewSize > m_nSize)
	{	nNewSize = max (nNewSize, m_nSize + max (GROW_BY, m_nSize / 4));
		if (m_pBuffer == NULL)
			m_pBuffer = malloc (nNewSize);
		else
			m_pBuffer = realloc (m_pBuffer, nNewSize);
		if (m_pBuffer == NULL)
			throw;
		m_nSize = nNewSize;
	}
	if (m_nLength < nSize)
		m_nLength = nSize;
}

/*-----------------------------------------------------------------
ZArrayBase :: Shrink () - size down the buffer
-----------------------------------------------------------------*/
void ZArrayBase :: Shrink (int nSize)
{
	int nNewSize = nSize * m_nElementSize;

	if (nNewSize < m_nSize)
	{
    m_pBuffer = realloc (m_pBuffer, nNewSize);
		if (m_pBuffer == NULL)
			throw;
    m_nSize = nNewSize;
		m_nLength = nSize;
	}
}

/*-----------------------------------------------------------------
Make room for new element at position i
-----------------------------------------------------------------*/
/*INLINE void ZArrayBase :: MakeRoomAt (int i)
{
	Stress (m_nLength);
	if (m_nLength > i + 1)
		memmove (
			((char *) m_pBuffer) + (i + 1) * m_nElementSize,
			((char *) m_pBuffer) + i * m_nElementSize,
			(m_nLength - i - 1) * m_nElementSize
		);
	memset (((char *) m_pBuffer) + i * m_nElementSize, 0, m_nElementSize);
} */

/*-----------------------------------------------------------------
Delete element at position i
-----------------------------------------------------------------*/
/*void ZArrayBase :: DeleteFrom (int i)
{
	if (m_nLength > i + 1)
		memmove (
			((char *) m_pBuffer) + i * m_nElementSize,
			((char *) m_pBuffer) + (i + 1) * m_nElementSize,
			(m_nLength - i - 1) * m_nElementSize
		);
	--m_nLength;
} */

/*-----------------------------------------------------------------
Scans the array and returns a pointer to an element which meets some
criterion, or NULL if none does.
ZCriterion is a base class for objects which have a virtual function,
IsOK(), which receives a pointer to NULL, casts it to pointer to
some object, and checks whether the object meets some criterion
-----------------------------------------------------------------*/
void *ZArrayBase :: Search (ZCriterion *pCriterion)
{
	int i;
	void *p;

	for (i = 0; i < m_nLength; i++)
	{	p = ((char *) m_pBuffer) + i * m_nElementSize;
		if (pCriterion -> IsOK (p))
			return p;
	}
	return NULL;
}

/*-----------------------------------------------------------------
Copy the contents of one array to the other, leaving the source array
empty
-----------------------------------------------------------------*/
void ZArrayBase :: Pump (ZArrayBase &rArray)
{
	m_pBuffer = rArray.m_pBuffer;
	m_nLength = rArray.m_nLength;
	m_nSize = rArray.m_nSize;
	rArray.m_pBuffer = NULL;
	rArray.m_nLength = rArray.m_nSize = 0;
}

/*-------------------------------------------------------------
ZMapBase constructor. Set initial values
-------------------------------------------------------------*/
ZMapBase :: ZMapBase (int nElementSize) :
	m_pIndice (NULL),
	m_pBuffer (NULL),
	m_nElementSize (nElementSize),
	m_nSize (0),
	m_nLength (0),
	m_nMask (0),
	m_nShift (0)
{}

/*-------------------------------------------------------------
ZMapBase destructor deletes the buffers
-------------------------------------------------------------*/
ZMapBase :: ~ZMapBase ()
{
	delete [] m_pBuffer;
	delete [] m_pIndice;
}

/*-------------------------------------------------------------
Find an index in a ZMap table
-------------------------------------------------------------*/
/*INLINE int ZMapBase :: FindIndex (int index) const
{
	int i = (index ^ (index >> m_nShift)) & m_nMask;
	int n = m_nSize;
	for (n = m_nSize; n > 0; n--)
	{	if (m_pIndice [i] == index)
			return i;
		if (i == 0)
			i = m_nSize;
		--i;
	}
	return -1;
} */

/*-------------------------------------------------------------
Insert a new index into a ZMap table
-------------------------------------------------------------*/
int ZMapBase :: InsertIndex (int index)
{
	/*-------------------------------------------------------------
	If table is full allocate a new table double in size, copy old table to the new
	one, and delete the old table
	-------------------------------------------------------------*/
	int i, n;
	if (m_nLength == m_nSize)
	{	int nOldSize = m_nSize;
		int *pOldIndice = m_pIndice;
		char *pOldBuffer = m_pBuffer;
		m_nLength = 0;
		m_nSize <<= 1;
		m_nMask <<= 1;
		m_nMask |= 1;
		m_nShift++;
		if (m_nSize == 0)
		{	m_nSize = 64;
			m_nMask = 0x3f;
			m_nShift = 6;
		}
		m_pIndice = new int [m_nSize];
		m_pBuffer = new char [m_nSize * m_nElementSize];
		memset (m_pIndice, -1, m_nSize * sizeof (int));
		for (i = nOldSize; --i >= 0;)
		{	if (pOldIndice [i] >= 0)
			{	n = InsertIndex (pOldIndice [i]);
				memcpy (
					m_pBuffer + n * m_nElementSize,
					pOldBuffer + i * m_nElementSize,
					m_nElementSize
				);
			}
		}
		delete [] pOldIndice;
		delete [] pOldBuffer;
	}

	/*-------------------------------------------------------------
	Now find a slot for the new index
	-------------------------------------------------------------*/
	i = (index ^ (index >> m_nShift)) & m_nMask;
	n = m_nSize;
	while (--n >= 0)
	{	if (m_pIndice [i] == -1)
		{	m_pIndice [i] = index;
			++m_nLength;
			return i;
		}
		if (i == 0)
			i = m_nSize;
		--i;
	}
	return -1;
}

/*-------------------------------------------------------------
Delete an index (if exists) from the table
-------------------------------------------------------------*/
int ZMapBase :: DeleteIndex (int index)
{
	int i = FindIndex (index);
	if (i >= 0)
	{	m_pIndice [i] = -1;
		--m_nLength;
	}
	return i;
}

/*-----------------------------------------------------------------
ZString constructor. Allocate memory and assign value
-----------------------------------------------------------------*/
ZString :: ZString (LPCSTR szStr)
{
	if (szStr == NULL)
	{	m_nLength = m_nSize = 0;
		m_pBuffer = NULL;
	}
	else
	{	m_nLength = strlen (szStr);
		m_nSize = m_nLength + 1;
		if ((m_pBuffer = (unsigned char *) malloc (m_nSize)) == NULL)
			throw;
		memcpy (m_pBuffer, szStr, m_nSize);
	}
}

/*-----------------------------------------------------------------
ZString constructor with string and length
-----------------------------------------------------------------*/
ZString :: ZString (const unsigned char *szStr, int nLength)
{
	m_nLength = nLength;
	m_nSize = m_nLength + 1;
	if ((m_pBuffer = (unsigned char *) malloc (m_nSize)) == NULL)
		throw;
	memcpy (m_pBuffer, szStr, nLength);
	m_pBuffer [nLength] = 0;

}

/*-----------------------------------------------------------------
ZString copy constructor
-----------------------------------------------------------------*/
ZString :: ZString (const ZString &str)
{
	m_nLength = str.m_nLength;
	m_nSize = str.m_nSize;
	m_pBuffer = NULL;
	if (str.m_pBuffer)
	{
        	if ((m_pBuffer = (unsigned char *) malloc (m_nSize)) == NULL)
				throw;
		memcpy (m_pBuffer, str.m_pBuffer, m_nLength + 1);
	}
}

/*-----------------------------------------------------------------
Make sure that the string has enough space for the ith byte.
If bFill is TRUE, extend the string by padding with blanks and appending
a null byte.
-----------------------------------------------------------------*/
void ZString :: Stress (int i, BOOL bFill)
{
	int nNewSize;

	/*-----------------------------------------------------------------
	If necessary, extend the buffer by 25% or GROW_BY, whatever is larger,
	but not less than i + 2 (for the terminating null byte).
	-----------------------------------------------------------------*/
	if (i+1 >= m_nSize)
	{	nNewSize = max (i + 2, m_nSize + max (GROW_BY, m_nSize / 4));
		m_pBuffer = m_pBuffer == NULL ?
			(unsigned char *) malloc (nNewSize) :
			(unsigned char *) realloc (m_pBuffer, nNewSize);

		if (m_pBuffer == NULL)
			throw;
		m_nSize = nNewSize;
		if (i == 0)
			*m_pBuffer = 0;
	}

	if (!bFill)
		return;

	/*-----------------------------------------------------------------
	If necessary, fill with blanks and append a null byte
	-----------------------------------------------------------------*/
	if (i > m_nLength)
		memset (m_pBuffer + m_nLength, ' ', i - m_nLength);
	if (i >= m_nLength)
		m_pBuffer [m_nLength = i + 1] = 0;
}

/*-----------------------------------------------------------------
Return a reference to position i. Make sure string is large enough
-----------------------------------------------------------------*/
/*INLINE unsigned char & ZString :: operator [] (int i)
{
	Stress (i, TRUE);
	return m_pBuffer [i];
} */

/*-----------------------------------------------------------------
ZString assignment
-----------------------------------------------------------------*/
/*INLINE ZString &ZString :: operator = (const ZString &str)
{
	if (str.m_pBuffer == NULL)
	{	if (m_pBuffer)
			free (m_pBuffer);
		m_pBuffer = NULL;
		m_nLength = m_nSize = 0;
	}
	else
	{
		m_nLength = str.m_nLength ;
		Stress (m_nLength);
		memcpy (m_pBuffer, str.m_pBuffer, m_nLength + 1);
	}
	return *this;
} */

/*-----------------------------------------------------------------
ZString assignment
-----------------------------------------------------------------*/
/*INLINE ZString &ZString :: operator = (LPCSTR szStr)
{
	if (szStr == NULL)
	{	if (m_pBuffer)
			free (m_pBuffer);
		m_pBuffer = NULL;
		m_nLength = m_nSize = 0;
	}
	else
	{	Stress (m_nLength = strlen (szStr));
		strcpy ((LPSTR) m_pBuffer, szStr);
	}
	return *this;
} */

/*-----------------------------------------------------------------
Append one character to a string
-----------------------------------------------------------------*/
/*INLINE ZString &ZString :: operator += (char c)
{
	Stress (m_nLength + 1);
	m_pBuffer [m_nLength++] = c;
	m_pBuffer [m_nLength] = 0;
	return *this;
} */

/*-----------------------------------------------------------------
Concatenate two strings
-----------------------------------------------------------------*/
/*INLINE ZString &ZString :: operator += (char *szStr)
{
	int n = strlen (szStr);
	Stress (m_nLength + n);
	memcpy (m_pBuffer + m_nLength, szStr, n);
	m_pBuffer [m_nLength += n] = 0;
	return *this;
} */

/*-----------------------------------------------------------------
Concatenate two strings
-----------------------------------------------------------------*/
/*INLINE ZString &ZString :: operator += (const ZString &str)
{
	Stress (m_nLength + str.m_nLength);
	memcpy (m_pBuffer + m_nLength, str.m_pBuffer, str.m_nLength);
	m_pBuffer [m_nLength += str.m_nLength] = 0;
	return *this;
} */

/*-----------------------------------------------------------------
Concatenate two strings
-----------------------------------------------------------------*/
/*INLINE ZString ZString :: operator + (const ZString &str) const
{
	return ZString (*this) += str;
} */

/*-----------------------------------------------------------------
Insert a character into position i
-----------------------------------------------------------------*/
unsigned char ZString :: InsertAt (int i, char c, int n)
{
	m_nLength = max (m_nLength, i) + n;
	Stress (m_nLength-1 , true); //gianluca first cahnge, was m_nlentght
	memmove (m_pBuffer + i + n, m_pBuffer + i, m_nLength - i - n);
	memset (m_pBuffer + i, c, n);
	return c;
}

/*-------------------------------------------------------------
Insert one string into another
-------------------------------------------------------------*/
const ZString &ZString :: InsertAt (int i, const ZString &str)
{
	int n = str.GetLength ();
	if (n > 0)
	{	m_nLength = max (m_nLength, i) + n;
		Stress (m_nLength - 1, true);
		memmove (m_pBuffer + i + n, m_pBuffer + i, m_nLength - i);
		//MS str.m_pBuffer
		// WIN-SOL release
#ifdef WIN32
		memcpy (m_pBuffer + i, str.m_pBuffer, n);
#else
        // PORTING to CC 5.3
		memcpy ((BYTE *)m_pBuffer + i, (BYTE *)(str.m_pBuffer), n);
#endif
	}
	return str;
}


/*-----------------------------------------------------------------
Delete a character from position i. Returns the deleted character
-----------------------------------------------------------------*/
/*INLINE char ZString :: DeleteFrom (int i)
{
	char cResult = m_pBuffer [i];

	memmove (m_pBuffer + i, m_pBuffer + i + 1, m_nLength - i);
	--m_nLength;
	return cResult;
} */

/*-----------------------------------------------------------------
Compare two strings
-----------------------------------------------------------------*/
/*INLINE BOOL ZString :: operator == (const ZString &str) const
{
	return m_nLength == str.m_nLength && !memcmp ((BYTE *) ((*this).m_pBuffer), (BYTE *)(str.m_pBuffer), m_nLength);
} */

/*-----------------------------------------------------------------
Find a character c in a ZString, starting the search at offset iOffset.
Returns the distance of the index of the found character from nOffset,
or -1 if did not find
-----------------------------------------------------------------*/
int ZString :: Find (int c, int nOffset) const
{
	int i;

	for (i = nOffset; i < GetLength (); i++)
		if ((*this) [i] == c)
			return i - nOffset;
	return -1;
}

/*-----------------------------------------------------------------
Find a character c in a ZString, starting the search at offset iOffset.
Returns the distance of the index of the found character from nOffset,
or -1 if did not find
-----------------------------------------------------------------*/
int ZString :: ReverseFind (int c, int nOffset) const
{
	int i;

	for (i = GetLength ()- nOffset; i >=0; i--)
		if ((*this) [i] == c)
			return i + nOffset;
	return -1;
}

/*-----------------------------------------------------------------
Return a string made of a substring of this string.
If nLength is negative, substring is from nFrom till end of string
-----------------------------------------------------------------*/
ZString ZString :: Substr (int nFrom, int nLength) const
{
	if (nFrom >= m_nLength || nLength == 0)
		return ZString ();
	if (nLength < 0)
		nLength = m_nLength - nFrom;
	return ZString (
		m_pBuffer + nFrom,
		min (m_nLength - nFrom, nLength)
	);
}

/*-----------------------------------------------------------------
Cut nLength chars out of a string, starting at nFrom.
If nLength is negative, cut till end of string
-----------------------------------------------------------------*/
void ZString :: Cut (int nFrom, int nLength)
{
	if (nFrom >= m_nLength || nLength == 0)
		return;
	if (nLength < 0)
		nLength = m_nLength - nFrom;
	memmove (
		m_pBuffer + nFrom,
		m_pBuffer + nFrom + nLength,
		m_nLength - nFrom - nLength + 1
	);
	m_nLength -= nLength;
}

/*-----------------------------------------------------------------
Copy a string to the object. No more than nMax characters
will be copied
-----------------------------------------------------------------*/
ZString & ZString :: Copy (LPCSTR szStr, int nMax)
{
	m_nLength = min ((int) strlen (szStr), nMax);
	Stress (m_nLength);
	memcpy (m_pBuffer, szStr, m_nLength);
	m_pBuffer [m_nLength] = 0;
	return *this;
}

/*-----------------------------------------------------------------
Copy an array of bytes to the object.
-----------------------------------------------------------------*/
/*INLINE ZString & ZString :: Assign (LPCBYTE pData, int nLength)
{
	m_nLength = nLength;
	Stress (m_nLength);
	memcpy (m_pBuffer, pData, m_nLength);
	m_pBuffer [m_nLength] = 0;
	return *this;
} */

/*-------------------------------------------------------------
Read a string from a file
-------------------------------------------------------------*/
/*INLINE int ZString :: FRead (FILE *pFile, int nSize)
{
	Stress (nSize);
	m_nLength = fread (m_pBuffer, 1, nSize, pFile);
	m_pBuffer [m_nLength] = 0;
	return m_nLength;
} */

/*-------------------------------------------------------------
Write a string into a file
-------------------------------------------------------------*/
/*INLINE int ZString :: FWrite (FILE *pFile)
{
	return fwrite (m_pBuffer, 1, m_nLength, pFile);
} */

int ZString::Format(const char* szFormatString,...)
{
	va_list marker;
	va_start( marker, szFormatString );     /* Initialize variable arguments. */

#ifdef WIN32
	#if ( _MSC_VER >= 1300 )	// Visual Studio .NET
		// calculate the string length in advance.
		const int nLength = _vscprintf(szFormatString,marker);
	#else // Visual C++ 6.0
		// temporary buffer, hopefully big enough to hold the string
		char aBuffer[ 2048 ] ; 
		// get back the actual size of string
		const int nLength = vsprintf( aBuffer, szFormatString, marker);
	#endif
#elif defined __UNIX
	// calculate the string length in advance.
	const int nLength = vsnprintf(NULL,0,szFormatString,marker);
#endif

	// now expand the private member to hold the string
	Stress(nLength + 1);//terminating \0
	// "copy" content (move is faster, mostly)
	vsprintf((char*) m_pBuffer, szFormatString, marker);

	// store length
	m_nLength = nLength;
	va_end(marker);
	return nLength ;
}

void ZString::Split(char Splitter, ZString& sLeft, ZString& sRight) const
{
	int pos = this->Find(Splitter,0);

	if(pos != -1)
	{
		sLeft = this->Substr(0,pos);
		if(m_nLength-pos-1 != 0)
			sRight = this->Substr(pos+1,m_nLength-pos-1);
		else
			sRight = "";
	}
	else
	{
		sLeft = sRight = "";
	}
}

#ifdef WIN32

/*----------------------        Win32       -------------------------------
ZSemaphore constructor. Create unnamed non-inherited semaphore.
If lInitialCount is -1, it is set to lMaxCount.
------------------------        Win32       -----------------------------*/
/*INLINE ZSemaphore :: ZSemaphore (long lMaxCount, long lInitialCount)
{
	if (lInitialCount < 0)
		lInitialCount = lMaxCount;
	m_hSemaph = CreateSemaphore (NULL, lInitialCount, lMaxCount, NULL);
} */

/*-------------------------------------------------------------------------
Try to reserve the semphore. Return immediately if semaphore already
reserved. Return TRUE if succeeded.
-------------------------------------------------------------------------*/
/*INLINE BOOL ZSemaphore :: Try ()
{
	return WaitForSingleObject (m_hSemaph, 0) != WAIT_TIMEOUT;
} */

/*-------------------------------------------------------------------------
Try to reserve the mutex. Return immediately if mutex already
reserved. Return TRUE if succeeded.
-------------------------------------------------------------------------*/
/*INLINE BOOL ZMutex :: Try ()
{
	return WaitForSingleObject (m_hMutex, 0) != WAIT_TIMEOUT;
}	   */

#ifdef _DEBUG
void ZMutex :: Trace (LPCSTR szOp)
{
	char szBuffer [80];

	sprintf (
		szBuffer,
		"Mutex handle: %x, operation: %s\n",
		m_hMutex,
		szOp
	);
	OutputDebugString (szBuffer);
}
#endif		// _DEBUG


/*----------------------        UNIX        -----------------------------
ZSemaphore constructor. Create unnamed non-inherited semaphore.
If lInitialCount is -1, it is set to lMaxCount.
------------------------        UNIX        -----------------------------*/

//#elif defined(PTHREADS)
#elif defined(__UNIX)

#include <sys/time.h>

/*-------------------------------------------------------------------------
ZSemaphore constructor. Create unnamed non-inherited semaphore.
If lInitialCount is -1, it is set to lMaxCount.
-------------------------------------------------------------------------*/
ZSemaphore :: ZSemaphore (long lMaxCount , long lInitialCount)
{
	if (lInitialCount < 0)
		lInitialCount = lMaxCount;
	m_nCount = lInitialCount;

	::pthread_cond_init  (&m_Cond, NULL);
	::pthread_mutex_init (&m_Mutex, NULL);
}

/*-------------------------------------------------------------------------
ZSemaphore destructor.
-------------------------------------------------------------------------*/
ZSemaphore :: ~ZSemaphore ()
{
	//wakeup_all ->
	::pthread_cond_broadcast(&m_Cond);

	::pthread_cond_destroy  (&m_Cond);
	::pthread_mutex_destroy (&m_Mutex);
}

/*-------------------------------------------------------------------------
Reserve the semaphore. Return TRUE if succeeded.
WAIT_TIMEOUT or FALSE
-------------------------------------------------------------------------*/
DWORD ZSemaphore :: P (UINT uTimeout)
{
	::pthread_mutex_lock(&m_Mutex);

	m_nCount--;

	if (uTimeout == INFINITE)
	{
		while (m_nCount < 0)
		{
			if (::pthread_cond_wait(&m_Cond, &m_Mutex))
			{
				::perror("pthread_cond_wait");
				::pthread_mutex_unlock(&m_Mutex);
				return FALSE;
			}
		}
	}
	else
	{
		struct timeval currenttime;
		struct timespec timeout;

		// MS: NULL
		::gettimeofday(&currenttime,NULL);

		timeout.tv_sec = currenttime.tv_sec + uTimeout / 1000;
		uTimeout %= 1000;
		timeout.tv_nsec = long ((long) currenttime.tv_usec * (long) 1000);
		timeout.tv_nsec += (long) uTimeout * (long) 1000000;
		timeout.tv_sec += timeout.tv_nsec / (long) 1000000000;
		timeout.tv_nsec %= (long) 1000000000;

		while (m_nCount < 0)
		{
			switch (::pthread_cond_timedwait(&m_Cond,&m_Mutex,&timeout))
			{
				case ETIMEDOUT:
					m_nCount++;
					::pthread_mutex_unlock(&m_Mutex);
					return WAIT_TIMEOUT;

				case 0: //success
					;
					break;

				default:
					m_nCount++;
					::perror("pthread_cond_timedwait");
					break;
			}
		}
	}

	::pthread_mutex_unlock(&m_Mutex);
	return TRUE;
}

/*-------------------------------------------------------------------------
Release the semphore. Return TRUE if succeeded.
-------------------------------------------------------------------------*/
BOOL ZSemaphore :: V (long nSignalled)
{
	::pthread_mutex_lock(&m_Mutex);

	for ( int i = 0; i < nSignalled; i++ )
	{
		if (m_nCount < 0)
			::pthread_cond_signal(&m_Cond);
		m_nCount++;
	}
	// 0 : success, >0 : error number
	const int ret = ::pthread_mutex_unlock(&m_Mutex);
	return (ret == 0) ;
}

/*-------------------------------------------------------------------------
Try to reserve the semphore. Return immediately if semaphore already
reserved. Return TRUE if succeeded.
-------------------------------------------------------------------------*/
BOOL ZSemaphore :: Try ()
{
	BOOL ret;
	::pthread_mutex_lock(&m_Mutex);
	if (m_nCount < 0)
		ret = FALSE;
	else
	{
		m_nCount--;
		ret = TRUE;
	}
	::pthread_mutex_unlock(&m_Mutex);
	return ret;
}

/*----------------------        SGI        -----------------------------
ZSemaphore, ZMutex
------------------------        SGI        -----------------------------*/

#else //elifdef __UNIX

usptr_t* ZSGIArena::m_pArena = NULL;

/*-------------------------------------------------------------------------
ZSemaphore constructor. Create unnamed non-inherited semaphore.
If lInitialCount is -1, it is set to lMaxCount.
-------------------------------------------------------------------------*/
ZSemaphore :: ZSemaphore (long lMaxCount, long lInitialCount)
{
	if (lInitialCount < 0)
		lInitialCount = lMaxCount;

	m_pSema = usnewsema(ZSGIArena::arena(), lInitialCount);
	if (!m_pSema)
	  ::perror("usnewsema");
}

/*-------------------------------------------------------------------------
Try to reserve the mutex. Return immediately if mutex already
reserved. Return TRUE if succeeded.
-------------------------------------------------------------------------*/
BOOL ZMutex :: Try ()
{
	switch (uscsetlock(m_Mutex, 1)) {
	default:
		::perror("uscsetlock");
	case 0:
		return FALSE;
	case 1:
		return TRUE;
	}
}

/*-------------------------------------------------------------------------
Reserve the mutex.  Return TRUE if succeeded.
-------------------------------------------------------------------------*/
DWORD ZMutex :: P ()
{
	switch (ussetlock(m_Mutex)) {
	default:
		::perror("ussetlock");
	case 0:
		return FALSE;
	case 1:
		return TRUE;
	}
}

#endif //WIN32 -> __UNIX



/*-----------------------------------------------------------------
A function that calculates how many bits are required to store a number
which is less than nMax
-----------------------------------------------------------------*/
/*int BitFieldWidth (int nMax)
{
	int nResult = 0;
	--nMax;
	while (nMax > 0)
	{	++nResult;
		nMax >>= 1;
	}
	return nResult;
} */

#ifdef WIN32

/*-------------------------------------------------------------
ZRegistry encapsulates registry access. The constructor creates a
handle for the given section. If bCreate is TRUE, the section is created
if does not yet exist.
-------------------------------------------------------------*/
ZRegistry :: ZRegistry (LPCSTR szSection, BOOL bCreate, HKEY hKey, LPCSTR szRoot)
{
	DWORD dwDisposition;
	LPSTR szEntry = new char[strlen(szSection)+strlen(szRoot)+1];
	sprintf(szEntry,"%s%s",szRoot,szSection);
	m_hSection = NULL;
	if (bCreate)
		RegCreateKeyEx (
			hKey,
			szEntry,//szSection,
			0,
			NULL,
			REG_OPTION_NON_VOLATILE,
			KEY_WRITE,
			NULL,
			&m_hSection,
			&dwDisposition
		);
	else
		RegOpenKeyEx(
			hKey,		// handle of open key
			szEntry,//szSection,				// address of name of subkey to open
			0,						// reserved
			KEY_READ,				// security access mask
			&m_hSection				// address of handle of open key
		);
	delete szEntry;
}

/*-------------------------------------------------------------
ZRegistry destructor closes the key
-------------------------------------------------------------*/
ZRegistry :: ~ZRegistry ()
{
	if (m_hSection)
		RegCloseKey (m_hSection);
}

/*-------------------------------------------------------------
Look into a registry section, find the DLL name that matches the given
key, load the DLL, and calls its "CreateInstance" exported function, which
should instantiate an object and return its address.
-------------------------------------------------------------*/
void * ZRegistry :: CreateInstance (LPCSTR szKey, HINSTANCE &hDLL) const
{
	char szDLL [100];
	DWORD dwcbData = 100;
	typedef void * (*InstantiationFunction) ();

	hDLL = NULL;
	if (!m_hSection || RegQueryValueEx (
			m_hSection,
			szKey,
			NULL,
			NULL,
			(LPBYTE) szDLL,
			&dwcbData
		) != ERROR_SUCCESS
	)
		return NULL;
	HINSTANCE hInstance = LoadLibrary (szDLL);
	if (hInstance == NULL)
		return NULL;
	InstantiationFunction function =
		(InstantiationFunction) GetProcAddress (hInstance, "CreateInstance");
	if (function == NULL)
		return NULL;

	void *object = (*function) ();
	if (object == NULL)
	{	FreeLibrary (hInstance);
		hInstance = NULL;
	}
	hDLL = hInstance;
	return object;
}

/*-------------------------------------------------------------
Enumerate sub-keys of registry key. Return TRUE is sub-key no. index
exists.
-------------------------------------------------------------*/
BOOL ZRegistry :: Enumerate (int index, LPSTR szBuffer, int nBufferLen)
{
	DWORD dwBufferLen = nBufferLen;

	return
		m_hSection &&
		RegEnumValue (
			m_hSection,
			index,		          // index of value to query
			szBuffer,
			&dwBufferLen,
			NULL,
			NULL,
			NULL,
			0
		) == ERROR_SUCCESS;
}

/*-------------------------------------------------------------
Retrieve a value
-------------------------------------------------------------*/
BOOL ZRegistry :: QueryValue(LPCSTR szName,LPBYTE szData,DWORD& nDataLen) const
{
	DWORD nLen = nDataLen;
	if (m_hSection==NULL) return FALSE;
	LONG ret = RegQueryValueEx(
		m_hSection,		// handle to key
		szName,			// value name
		0,				// reserved
		0,				// value type&valueType
		szData,	// value data
		&nLen			// size of value data
	);
	nDataLen = nLen;

	if(ERROR_SUCCESS == ret)
		return TRUE;
	else
		return FALSE;
}
#endif // WIN32

// SOLARIS PORTING
#ifdef __UNIX
/*-------------------------------------------------------------
ZRegistry encapsulates registry access. The constructor creates a
handle for the given section. If bCreate is TRUE, the section is created
if does not yet exist.
-------------------------------------------------------------*/
ZRegistry :: ZRegistry (LPCSTR szSection, BOOL bCreate, HKEY hKey, LPCSTR szRoot, LPCSTR RegFile)
{
	Init(szSection, bCreate, hKey, szRoot , RegFile);
}

/*-------------------------------------------------------------
ZRegistry destructor closes the key
-------------------------------------------------------------*/
ZRegistry :: ~ZRegistry ()
{
	if (m_hFile)
	{
		fclose (m_hFile);
		m_hFile = NULL ;
	}

	free( m_pszSectionValue );
	free( m_pszRegPath );
	free( m_pRegFile );
}

/*-------------------------------------------------------------
Look into a registry section, find the DLL name that matches the given
key, load the DLL, and calls its "CreateInstance" exported function, which
should instantiate an object and return its address.
-------------------------------------------------------------*/
void * ZRegistry :: CreateInstance (LPCSTR szKey, HINSTANCE &hDLL)
{
	typedef void * (*InstantiationFunction) ();

	hDLL = NULL;
	void *hInstance = dlopen(szKey,RTLD_LAZY);
	if (hInstance == NULL){
//		cout<<dlerror()<<endl;
//		cout<<"dlopen() failed"<<endl;
		return NULL;
	}

	InstantiationFunction function =
		(InstantiationFunction) dlsym(hInstance, "CreateInstance");

	if (function == NULL)
		return NULL;
	void *object = (*function) ();
	if (object == NULL)
	{
		delete (hInstance);
	}

	return object;
}

/*-------------------------------------------------------------
Enumerate sub-keys of registry key. Return TRUE is sub-key no. index
exists.
-------------------------------------------------------------*/
BOOL ZRegistry :: Enumerate (int index, LPSTR szBuffer, int nBufferLen)
{
	if (!RootExists(m_pszRegPath))
		return FALSE;

	char Buffer[BUFFERSIZE];

	while(fgets(Buffer,BUFFERSIZE,m_hFile))
	{
		RemoveNewLine(Buffer);

		if (!strlen(Buffer) || Buffer[0]=='[')
			break;

		if (index--)
			continue;

		char* PChar = strchr(Buffer,'=');

		if (!PChar)
			break;

		*PChar = '\0';
		PChar++;

		if ((int) strlen(Buffer) < nBufferLen)
		{
			strcpy(szBuffer,Buffer);
			return TRUE;
		}
	}

	if (nBufferLen > 4)
		strcpy(szBuffer,"NULL");

	return FALSE;


}

/*-------------------------------------------------------------
Retrieve a value
-------------------------------------------------------------*/
BOOL ZRegistry :: QueryValue(LPCSTR szName,LPBYTE szData,DWORD& nDataLen)
{
	szData[0] = '\0';
	if (RootExists( m_pszRegPath))
	{
		if (SectionExists(szName))
		{
			unsigned int s = strlen( m_pszSectionValue);

			if( s && s < nDataLen)
			{
				strcpy((LPSTR) szData, m_pszSectionValue);
				nDataLen = s;
				return TRUE;
			}
		}
	}


	return FALSE;
}


LPCSTR ZRegistry::IdentifyKeyValue(HKEY hKey)
{
	switch(hKey)
	{
		case HKEY_CLASSES_ROOT:
			return "HKEY_CLASSES_ROOT";
		case HKEY_CURRENT_USER:
			return "HKEY_CURRENT_USER";
		case HKEY_LOCAL_MACHINE:
			return "HKEY_LOCAL_MACHINE";
		case HKEY_USERS:
			return "HKEY_USERS";
		case HKEY_PERFORMANCE_DATA:
			return "HKEY_PERFORMANCE_DATA";
		case HKEY_CURRENT_CONFIG:
			return "HKEY_CURRENT_CONFIG";
		case HKEY_DYN_DATA:
			return "HKEY_DYN_DATA";
	}

	return LPCSTR("");
}

BOOL ZRegistry::RootExists(LPCSTR szRoot)
{
	// file open?
	if (!m_hFile)
		return FALSE;

	char Buffer[BUFFERSIZE];

	fseek(m_hFile,0,SEEK_SET);
	while(fgets(Buffer,BUFFERSIZE,m_hFile))
	{
		if (Buffer[0] != '[')
			continue;

		RemoveNewLine(Buffer);
		if (!strcmp(szRoot,Buffer))
			return TRUE;
	}
	return FALSE;
}

BOOL ZRegistry::CreateRoot(LPCSTR szRoot)
{
	if (!m_hFile || strlen(szRoot) > (BUFFERSIZE - 1))
		return FALSE;

	fseek(m_hFile,0,SEEK_END);
	fputs(szRoot,m_hFile);
	fputs("\n\n",m_hFile);


	fflush(m_hFile);

	return TRUE;
}

BOOL ZRegistry::SectionExists(LPCSTR szSection)
{
	if (!m_hFile)
		return FALSE;

	char Buffer1[BUFFERSIZE];
	char Buffer2[BUFFERSIZE];

	sprintf(Buffer2,"\"%s\"",szSection);

	while(fgets(Buffer1,BUFFERSIZE,m_hFile))
	{
		RemoveNewLine(Buffer1);

		if (!strlen(Buffer1) || Buffer1[0]=='[')
			break;

		char* PChar = strchr(Buffer1,'=');

		if (!PChar)
			return FALSE;

		*PChar = '\0';
		PChar++;

		if (!strcmp(Buffer1,Buffer2))
		{
			delete m_pszSectionValue;
			int n = strlen(PChar)-1;
			m_pszSectionValue = new char[n];
			strncpy(m_pszSectionValue,PChar+1,n-1);
			m_pszSectionValue[n-1] = '\0';
			return TRUE;
		}
	}
	return FALSE;
}

BOOL ZRegistry::CreateSection(LPCSTR szRoot,LPCSTR szSection)
{
	FILE* m_hTempFile = fopen(TEMPFILE,"w");

	if (!m_hTempFile || !m_hFile)
		return false;

	char Buffer[BUFFERSIZE];

	fseek(m_hFile,0,SEEK_SET);
	while(fgets(Buffer,BUFFERSIZE,m_hFile))
	{
		RemoveNewLine(Buffer);
		if (!strcmp(szRoot,Buffer))
		{
			fprintf(m_hTempFile,"%s\n",Buffer);
			fprintf(m_hTempFile,"\"%s\"=\"\"\n",szSection);
		}
		else
		{
			fprintf(m_hTempFile,"%s\n",Buffer);
		}
	}

	fclose(m_hTempFile);

	delete m_pszSectionValue;
	m_pszSectionValue = new char[1];
	m_pszSectionValue[0] = '\0';

	fclose(m_hFile);
	m_hFile = NULL;

	if (!unlink(m_pRegFile))
	{
		rename(TEMPFILE,m_pRegFile);
		m_hFile = fopen(m_pRegFile,"a+");
		return TRUE;
	}

	return FALSE;
}

BOOL ZRegistry::SetValue(LPCSTR szSection,LPCSTR szValue,BOOL Create)
{
	if (!RootExists(m_pszRegPath))
		return FALSE;

	BOOL BSectionExists = SectionExists(szSection);


	FILE* m_hTempFile = fopen(TEMPFILE,"w");

  // both files should be open
	if (!m_hTempFile || !m_hFile)
		return false;

	char Buffer1[BUFFERSIZE];
	char Buffer2[BUFFERSIZE];
	char Buffer3[BUFFERSIZE];

	sprintf(Buffer2,"\"%s\"",szSection);

	delete m_pszSectionValue;
	m_pszSectionValue = NULL;

	fseek(m_hFile,0,SEEK_SET);
	while(fgets(Buffer1,BUFFERSIZE,m_hFile))
	{
		fputs(Buffer1,m_hTempFile);
		RemoveNewLine(Buffer1);
		if (!strcmp(m_pszRegPath,Buffer1))
		{
			while(fgets(Buffer1,BUFFERSIZE,m_hFile))
			{
				if (!BSectionExists)
				{
					if (Create)
					{
						delete m_pszSectionValue;
						int n = strlen(szValue)+1;
						m_pszSectionValue = new char[n];
						strcpy(m_pszSectionValue,szValue);
						fprintf(m_hTempFile,"\"%s\"=\"%s\"\n",szSection,szValue);
					}
					fputs(Buffer1,m_hTempFile);
					break;
				}
				strcpy(Buffer3,Buffer1);

				RemoveNewLine(Buffer1);


				if (!strlen(Buffer1) || Buffer1[0] == '[')
					break;

				char* PChar = strchr(Buffer1,'=');

				if (!PChar)
					return FALSE;

				*PChar = '\0';
				PChar++;

				if (!strcmp(Buffer1,Buffer2))
				{
					delete m_pszSectionValue;
					int n = strlen(szValue)+1;
					m_pszSectionValue = new char[n];
					strcpy(m_pszSectionValue,szValue);
					fprintf(m_hTempFile,"%s=\"%s\"\n",Buffer1,szValue);
					break;
				}
				else
				{
					fputs(Buffer3,m_hTempFile);
				}
			}

		}
	}

	fclose(m_hTempFile);
	m_hTempFile = NULL ;

	fclose(m_hFile);
	m_hFile = NULL ;

	if (!unlink(m_pRegFile))
	{
		rename(TEMPFILE,m_pRegFile);
		m_hFile = fopen(m_pRegFile,"a+");
		return TRUE;
	}

	return FALSE;
}

void ZRegistry::RemoveNewLine(LPSTR StrBuffer)
{
	int x = strlen(StrBuffer) - 1;
	while(StrBuffer[x] == '\r' || StrBuffer[x] == '\n')
	{
		StrBuffer[x] = '\0';
	}
}

// opens the RegFile as our virtual Registry,
// creates a [szSection] entry
// creates a "key" entry
void ZRegistry::Init(LPCSTR szSection, BOOL bCreate, HKEY hKey, LPCSTR szRoot, LPCSTR RegFile)
{
	m_pszRegPath = new char[strlen(IdentifyKeyValue(hKey))+strlen(szRoot)+4];

	// build string for filename to ProfileFile
	// store character array
	m_pRegFile = strdup(RegFile);
	m_pszSectionValue = NULL;

	m_hFile = fopen(m_pRegFile,"a+");

	if (m_hFile)
	{
		// following handling inside file,
		// [section]
		// key=value

		// remove trailing backslashes
		LPSTR _szRoot = strdup(szRoot);
		if (_szRoot[strlen(_szRoot)-1] == '\\')
			_szRoot[strlen(_szRoot)-1] = '\0';


		if (strlen(IdentifyKeyValue(hKey)))
		{
			sprintf(m_pszRegPath,"[%s\\%s]",IdentifyKeyValue(hKey),_szRoot);
		}
		else
		{
			sprintf(m_pszRegPath,"[%s]",_szRoot);
		}
		delete _szRoot;


		if (!RootExists( m_pszRegPath) && bCreate)

			CreateRoot( m_pszRegPath);

		if (szSection)
		{
			if (!SectionExists(szSection) && bCreate)
				CreateSection( m_pszRegPath, szSection);
		}
	}
	else
		assert( false ) ;

}


BOOL WritePrivateProfileString(
  LPCSTR lpAppName,  // section name
  LPCSTR lpKeyName,  // key name
  LPCSTR lpString,   // string to add
  LPCSTR lpFileName  // initialization file
)
{
	ZRegistry *Reg = new ZRegistry(lpKeyName,1, 0, lpAppName,lpFileName);
	BOOL res = Reg -> SetValue(lpKeyName, lpString, TRUE ) ;
	delete Reg;
	return res;
};


int GetPrivateProfileInt(
  LPCSTR lpAppName,        // section name
  LPCSTR lpKeyName,        // key name
  int nDefault,        // default string
  LPCSTR lpFileName        // initialization file name
)
{
	char Buffer[255];
	Buffer[0] = '\0';

	::GetPrivateProfileString( lpAppName, lpKeyName, "", Buffer, sizeof(Buffer), lpFileName ) ;

	int retVal = 0 ;

	// in case key not found, use default value
	if ( strlen( Buffer )	== 0 )
	{
		retVal = nDefault ;
	}
	else

	{
		retVal = atoi( Buffer );
	}

	return ( retVal ) ;
}



DWORD GetPrivateProfileString(
  LPCSTR lpAppName,        // section name
  LPCSTR lpKeyName,        // key name
  LPCSTR lpDefault,        // default string
  LPSTR lpReturnedString,  // destination buffer
  DWORD nSize,              // size of destination buffer
  LPCSTR lpFileName        // initialization file name
)
{
	ZRegistry *Reg = new ZRegistry(lpKeyName,0, 0,lpAppName,lpFileName);
	Reg -> QueryValue(lpKeyName,(unsigned char *)lpReturnedString,nSize);

	// Key not found
	if( strlen(lpReturnedString) == 0 )
	{
		strcpy(lpReturnedString, lpDefault);
		nSize = strlen( lpDefault ) ;
	}
	delete Reg;

	return nSize;
}

#endif // __UNIX

/*-------------------------------------------------------------
ZLargeInteger constructor
-------------------------------------------------------------*/
/*INLINE ZLargeInteger :: ZLargeInteger (DWORD nLow, DWORD nHigh, int nBits) :
	length (nBits / (sizeof (DWORD) * 8))
{
	if (length > 0)
		number.SetAt (0, nLow);
	if (length > 1)
		number.SetAt (1, nHigh);
	for (int i = 2; i < length; i++)
		number.SetAt (i, 0);
} */

/*INLINE ZLargeInteger :: ZLargeInteger (const ZLargeInteger &n)
{
	length = n.length;
	number = n.number;
} */

/*-------------------------------------------------------------
Assignment of an int to ZLargeInteger
-------------------------------------------------------------*/
/*INLINE void ZLargeInteger :: Assign (DWORD nLow)
{
	number.SetAt (0, nLow);
	for (int i = 1; i < length; i++)
		number.SetAt (i, 0);
} */

/*-------------------------------------------------------------
Compare ZLargeIntegers
-------------------------------------------------------------*/
/*INLINE bool ZLargeInteger :: operator == (const ZLargeInteger &n) const
{
	for (int l = max (number.GetLength (), n.number.GetLength ()); --l >= 0;)
	{	DWORD n1 = l >= number.GetLength () ? 0 : number [l];
		DWORD n2 = l >= n.number.GetLength () ? 0 : n.number [l];
		if (n1 != n2)
			return false;
	}
	return true;
} */

/*bool ZLargeInteger :: operator == (DWORD n) const
{
	ZLargeInteger l(n);
	return *this == l;
}*/

/*INLINE bool ZLargeInteger :: operator > (const ZLargeInteger &n) const
{
	for (int l = max (number.GetLength (), n.number.GetLength ()); --l >= 0;)
	{	DWORD n1 = l >= number.GetLength () ? 0 : number [l];
		DWORD n2 = l >= n.number.GetLength () ? 0 : n.number [l];
		if (n1 > n2)
			return true;
		else if (n1 < n2)
			return false;
	}
	return false;
} */

/*INLINE bool ZLargeInteger :: operator < (const ZLargeInteger &n) const
{
	for (int l = max (number.GetLength (), n.number.GetLength ()); --l >= 0;)
	{	DWORD n1 = l >= number.GetLength () ? 0 : number [l];
		DWORD n2 = l >= n.number.GetLength () ? 0 : n.number [l];
		if (n1 < n2)
			return true;
		else if (n1 > n2)
			return false;
	}
	return false;
} */

enum {BITS = sizeof (WORD) * 8};

/*-------------------------------------------------------------
Add large integers
-------------------------------------------------------------*/
void ZLargeInteger :: AddOnto (const ZLargeInteger n)
{
	DWORD dwCarry = 0;
	for (int i = 0; i < length; i++)
	{	DWORD dwLow1 = 0;
		DWORD dwHigh1 = 0;
		if (i < number.GetLength ())
		{	dwLow1 = LOWORD (number [i]);
			dwHigh1 = HIWORD (number [i]);
		}
		DWORD dwLow2 = 0, dwHigh2 = 0;
		if (i < n.number.GetLength ())
		{	dwLow2 = LOWORD (n.number [i]);
			dwHigh2 = HIWORD (n.number [i]);
		}
		dwCarry += dwLow1 + dwLow2;
		dwLow1 = LOWORD (dwCarry);
		dwCarry >>= BITS;
		dwCarry += dwHigh1 + dwHigh2;
		dwHigh1 = LOWORD (dwCarry);
		dwCarry >>= BITS;
		number.SetAt (i, MAKELONG (dwLow1, dwHigh1));
	}
}

/*-------------------------------------------------------------
Subtract large integers
-------------------------------------------------------------*/
void ZLargeInteger :: SubtractFrom (const ZLargeInteger n)
{
	DWORD dwCarry = 0;
	for (int i = 0; i < length; i++)
	{	DWORD dwLow1 = 0;
		DWORD dwHigh1 = 0;
		if (i < number.GetLength ())
		{	dwLow1 = LOWORD (number [i]);
			dwHigh1 = HIWORD (number [i]);
		}
		DWORD dwLow2 = 0, dwHigh2 = 0;
		if (i < n.number.GetLength ())
		{	dwLow2 = LOWORD (n.number [i]);
			dwHigh2 = HIWORD (n.number [i]);
		}
		dwCarry = (1 << BITS) + dwLow1 - dwLow2 - dwCarry;
		dwLow1 = LOWORD (dwCarry);
		dwCarry = 1 - (dwCarry >> BITS);
		dwCarry = (1 << BITS) + dwHigh1 - dwHigh2 - dwCarry;
		dwHigh1 = LOWORD (dwCarry);
		dwCarry = 1 - (dwCarry >> BITS);
		number.SetAt (i, MAKELONG (dwLow1, dwHigh1));
	}
}

/*-------------------------------------------------------------
Multiply large integers
-------------------------------------------------------------*/
void ZLargeInteger :: MultiplyBy (const ZLargeInteger n)
{
	ZLargeInteger temp (*this);
	Assign (0);

	for (int i = 0; i < length; i++)
	{	DWORD dwCarry = 0;
		for (int j = 0; i + j < length; j++)
		{	DWORD dwLow1 = 0;
			DWORD dwHigh1 = 0;
			if (j < temp.number.GetLength ())
			{	dwLow1 = LOWORD (temp.number [j]);
				dwHigh1 = HIWORD (temp.number [j]);
			}
			DWORD dwLow2 = 0, dwHigh2 = 0;
			if (i < n.number.GetLength ())
			{	dwLow2 = LOWORD (n.number [i]);
				dwHigh2 = HIWORD (n.number [i]);
			}
			DWORD dw1 = dwLow1 * dwLow2;
			DWORD dw2 = dwLow1 * dwHigh2;
			DWORD dw3 = dwHigh1 * dwLow2;
			DWORD dwa = LOWORD (dw1) + LOWORD (dwCarry);
			DWORD dwb =
				HIWORD (dw1) + LOWORD (dw2) + LOWORD (dw3) +
				HIWORD (dwCarry);
			dwLow1 = LOWORD (dwa);
			dwa = HIWORD (dwa) + LOWORD (dwb);
			dwCarry =
				dwHigh1 * dwHigh2 + HIWORD (dw2) + HIWORD (dw3) +
				HIWORD (dwa) + HIWORD (dwb);
			dwHigh1 = LOWORD (dwa);
			DWORD dwCarry2 = 0;
			for (int k = i + j; k < length; k++)
			{	DWORD dwLow2 = 0, dwHigh2 = 0;
				if (k < number.GetLength ())
				{	dwLow2 = LOWORD (number [k]);
					dwHigh2 = HIWORD (number [k]);
				}
				dwCarry2 += dwLow1 + dwLow2;
				dwLow1 = LOWORD (dwCarry2);
				dwCarry2 >>= BITS;
				dwCarry2 += dwHigh1 + dwHigh2;
				dwHigh1 = LOWORD (dwCarry2);
				dwCarry2 >>= BITS;
				number.SetAt (k, MAKELONG (dwLow1, dwHigh1));
				dwLow1 = dwHigh1 = 0;
			}
		}
	}
}

/*-------------------------------------------------------------
Divide large integers
-------------------------------------------------------------*/
void ZLargeInteger :: DivideBy (const ZLargeInteger n)
{
	ZLargeInteger dividend (*this);
	Assign (0);
	ZLargeInteger shift (*this);

	if (dividend.GetSize () == 0)
		dividend.number.SetAt (0, 0);
	for (int i = length * (sizeof (DWORD) * 8); --i >= 0;)
	{
		shift <<= 1;
		dividend <<= 1;
		shift.number [0] |= dividend.m_dwCarry;
		bool b = shift >= n;
		*this <<= 1;
		if (b)
		{	number [0] |= 1;
			shift.SubtractFrom (n);
		}
	}
}

/*-------------------------------------------------------------
Calculate remainder of Division of large integers
-------------------------------------------------------------*/
void ZLargeInteger :: RemainderOf (const ZLargeInteger n)
{
	ZLargeInteger dividend (*this);
	Assign (0);

	if (dividend.GetSize () == 0)
		dividend.number.SetAt (0, 0);
	for (int i = length * (sizeof (DWORD) * 8); --i >= 0;)
	{
		*this <<= 1;
		dividend <<= 1;
		number [0] |= dividend.m_dwCarry;
		bool b = *this >= n;
		if (b)
			SubtractFrom (n);
	}
}

/*-------------------------------------------------------------
Raise large number in power
-------------------------------------------------------------*/
ZLargeInteger ZLargeInteger :: pwr (const ZLargeInteger &exp)
{
	ZLargeInteger temp (*this);
	ZLargeInteger e (exp);
	temp.Assign (1);
	for (int i = 1; !e.IsZero (); i++)
	{	e >>= 1;
		if (e.m_dwCarry)
		{	ZLargeInteger t (*this);
			for (int j = i; --j > 0;)
				t.MultiplyBy (t);
			temp.MultiplyBy (t);
		}
	}
	return temp;
}


/*-------------------------------------------------------------
Return true if the number is zero
-------------------------------------------------------------*/
bool ZLargeInteger :: IsZero () const
{
	for (int i = length; --i >= 0;)
		if (number [i] != 0)
			return false;
	return true;
}

/*-------------------------------------------------------------
Shift a large number (by a nunber <= 32). Store the carry in m_dwCarry.
-------------------------------------------------------------*/
ZLargeInteger & ZLargeInteger :: operator <<= (unsigned n)
{
	int i = 0;
	m_dwCarry = 0;
	for (i = 0; i < number.GetLength (); i++)
	{	DWORD dw = m_dwCarry;
		m_dwCarry = number [i] >> (sizeof (DWORD) * 8 - n);
		if (n < 32)
			number [i] <<= n;
		else
			number [i] = 0;
		number [i] |= dw;
	}
	if (i < length)
	{	number.SetAt (i, m_dwCarry);
		m_dwCarry = 0;
	}
	return *this;
}

ZLargeInteger & ZLargeInteger :: operator >>= (unsigned n)
{
	m_dwCarry = 0;
	for (int i = number.GetLength (); --i >= 0;)
	{	DWORD dw = m_dwCarry;
		m_dwCarry = number [i] << (sizeof (DWORD) * 8 - n);
		number [i] >>= n;
		number [i] |= dw;
	}
	m_dwCarry >>= (sizeof (DWORD) * 8 - n);
	return *this;
}

/*-------------------------------------------------------------
Verifies that a certain file exists
-------------------------------------------------------------*/
BOOL ZFileExists (LPCSTR szFile)
{
	/*-------------------------------------------------------------
	Use _findfirst to find the file then make sure this name uniquely
	identifies a file (not a wildcard name)
	-------------------------------------------------------------*/
#ifdef WIN32
    struct _finddata_t fileDetails;
    long hFile;
	BOOL bResult = FALSE;

	if ((hFile = _findfirst (szFile, &fileDetails)) != -1L)
	{	if (_findnext (hFile, &fileDetails) != 0)
			bResult = TRUE;
		_findclose (hFile);
	}
	return bResult;
#endif //WIN32
#ifdef __UNIX

	FILE* stream = fopen(szFile,"r");

	if (stream)
	{
		fclose(stream);
		return TRUE;
	}

	return FALSE;

#endif
}

/*-------------------------------------------------------------
Replace the file type (suffix) of szFileName, by szFileType
-------------------------------------------------------------*/
ZString ComposeFileName (LPCSTR szFileName, LPCSTR szFileType)
{
	LPCSTR szSuffix = strrchr (szFileName, '.');
	int nLength =
		szSuffix == NULL ? strlen (szFileName) : szSuffix - szFileName;
	return (ZString ((const LPBYTE) szFileName, nLength) += '.') += szFileType;
}

/*-------------------------------------------------------------
Expand a relative to an absolute URL.
Returns 0 if successful (enough buffer size of the result) or required buffer size if not.
-------------------------------------------------------------*/
int ZExpandURL (LPCSTR szRootURL, LPCSTR szRelURL, LPSTR szAbsURL, int nSize)
{
	int nRootLength = 0;
	if (szRootURL != NULL &&
		strspn (szRelURL, " \t") < strcspn (szRelURL, "/\\") &&		// relURL doesn't start with \ or /
		strcspn (szRelURL, "/\\?") <= strcspn (szRelURL, ":|")		// relURL doesn't start with xxx: or xxx|
	)																// then this is a relative URL
	{	for (nRootLength = strlen (szRootURL); --nRootLength >= 0;)
		{	char c = szRootURL [nRootLength];
			if (c == '/' || c == '\\' || c == ':' || c == '|')
				break;
		}
		++nRootLength;
	}

	/*-------------------------------------------------------------
	Check if there is enough space for the combined URL
	-------------------------------------------------------------*/
	int n = nRootLength + strlen (szRelURL);
	if (n > nSize)
		return n;

	/*-------------------------------------------------------------
	Concatenate the name
	-------------------------------------------------------------*/
	memcpy (szAbsURL, szRootURL, nRootLength);
	strcpy (szAbsURL + nRootLength, szRelURL);
	return 0;
}

/*-------------------------------------------------------------
Translate a binary string to Base64
-------------------------------------------------------------*/
void ZBin2Base64 (const ZString &stFrom, ZString &stTo)
{
	int i, j = 0, k = 0;

	/*-------------------------------------------------------------
	Calcualte target length
	-------------------------------------------------------------*/
	stTo.InsertAt (((stFrom.GetLength () + 2) / 3) * 4, 0);

	/*-------------------------------------------------------------
	Encode
	-------------------------------------------------------------*/
	for (i = 0; i < stFrom.GetLength (); i += 3)
	{	UINT n = 0;
		BYTE c;

		for (j = i; j < i + 3; j++)
		{	c = j < stFrom.GetLength () ? stFrom [j] : 0;
			n <<= 8;
			n |= c;
		}

		for (int nShift = 18; nShift >= 0; nShift -= 6)
		{	c = (n >> nShift) & 0x3f;
			stTo [k++] =
				c <= 25 ? 'A' + c :
				c <= 51 ? 'a' + c - 26 :
				c <= 61 ? '0' + c - 52 :
				c == 62 ? '+' :
				'/';
		}
	}

	/*-------------------------------------------------------------
	Replace last characters by appropriate pads
	-------------------------------------------------------------*/
	stTo.Cut (k);
	for (j -= stFrom.GetLength (); --j >= 0;)
		stTo [--k] = '=';

}

#ifdef __UNIX
char * itoa(int nValue, char *pString, int radix)
{
	if ( pString == NULL )
		return NULL;

	int x = 0;
	pString[x] = '\0';
	int tValue = nValue;

	while(tValue)
	{
		tValue /= radix;
		if (tValue)
			x++;
	}

	pString[x+1] = '\0';
	while( x >= 0 )
	{
		const int v = nValue % radix;
		if (v > 10)
			pString[x] = 'a' + v - 10;
		else
			pString[x] = '0' + v;

		nValue /= radix;
		x--;
	}
	return pString;
}


#endif //__UNIX


// DEBUG, include methods here: disable inlining
#ifndef _ENABLE_INLINE_EXPANSION
#include "ztl.inl"
#endif
