/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Mikael Bourges-Sevenier at mikael.bourgessevenier@cnet.francetelecom.fr.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Presenter (simulation)
FILE:       MNodes.cpp
AUTHOR:     Mikael Bourges-Sevenier (France Telecom CNET/DIH/ATI)
OVERVIEW:   Implementation of my overloading nodes
-------------------------------------------------------------------------*/

#include "defs.h"
#include "MNodes.h"
#include "Executiv.h"
#include "Presenter.h"
#include "Stream.h"
#include "Clock.h"
#include "ODImp.h"
#include "Decoder.h"
#include "AnimStreamDec.h"
#include "Anim.h"
#include "..\IM1Decoders\AFX\MshGrd\DataDefs.h"
#include "..\IM1Decoders\AFX\MshGrd\MshGrd.h"
#include "..\IM1Decoders\AFX\WaveSurf\WaveSurfDec.h"

void MikeOverloads ()
{
	OVERLOAD_NODE (MSphere);
	OVERLOAD_NODE (MTouchSensor);
	OVERLOAD_NODE (MTimeSensor);
	OVERLOAD_NODE (MOrientationInterpolator);
	OVERLOAD_NODE (MScalarInterpolator);
	OVERLOAD_NODE (MGroup);
	OVERLOAD_NODE (MMaterial);
	OVERLOAD_NODE (MTransform);
	OVERLOAD_NODE (MTransform2D);
	OVERLOAD_NODE (MViewpoint);
	OVERLOAD_NODE (MBitWrapper);
	OVERLOAD_NODE (MMeshGrid);
	OVERLOAD_NODE (MWaveletSubdivisionSurface); //PG 021211
}

BOOL MSphere :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:Sphere - radius: %f\n",
			     context.m_nTabs*3,
			     ' ',
			     GetID (),
			     (float) radius
			     );
  return TRUE;
}

BOOL MOrientationInterpolator :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:OrientationInterpolator \n",
			     context.m_nTabs*3,
			     ' ',
			     GetID ()
			     );
  return TRUE;
}

BOOL MScalarInterpolator :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:ScalarInterpolator \n",
			     context.m_nTabs*3,
			     ' ',
			     GetID ()
			     );
  return TRUE;
}

BOOL MTouchSensor :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:TouchSensor \n",
			     context.m_nTabs*3,
			     ' ',
			     GetID ()
			     );
  return TRUE;
}

BOOL MTimeSensor :: Render (PresenterContext *pContext)
{
	/*-------------------------------------------------------------
	startTime / stopTime fields need to be offset by current time on first access
	-------------------------------------------------------------*/
	startTime.ApplyOffset (pContext);
	stopTime.ApplyOffset (pContext);

  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:TimeSensor \n",
			     context.m_nTabs*3,
			     ' ',
			     GetID ()
			     );
  return TRUE;
}

BOOL MGroup :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     //MBS 010603: "%*c%d:Group\n\tbboxCenter: %f %f %f\n\tbboxSize: %f %f %f\n",
			     "%*c%d:Group\n",
			     context.m_nTabs*3,
			     ' ',
			     GetID () //,
			     //MBS 110498 ((Vec3f)bboxCenter).x, ((Vec3f)bboxCenter).y, ((Vec3f)bboxCenter).z,
			     //MBS 110498 ((Vec3f)bboxSize).x, ((Vec3f)bboxSize).y, ((Vec3f)bboxSize).z
			     );
  ++context.m_nTabs;
  for (int i = 0; i < children.GetLength (); i++)
    children [i] -> Render (&context);
  return TRUE;
}

BOOL MMaterial :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:Material\n\tdiffuseColor: %f %f %f\n\ttransparency: %f\n",
			     context.m_nTabs*3,
			     ' ',
			     GetID (),
			     ((Color3f)diffuseColor).r,
			     ((Color3f)diffuseColor).g,
			     ((Color3f)diffuseColor).b,
			     ((float)transparency)
			     );
  return TRUE;
}

BOOL MTransform :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:Transform\n\torientation: (%f, %f, %f), %f\n\tposition: %f, %f, %f\n",
			     context.m_nTabs*3,
			     ' ',
			     GetID (),
			     ((Rotation)rotation).x,
			     ((Rotation)rotation).y,
			     ((Rotation)rotation).z,
			     ((Rotation)rotation).theta,
			     ((Vec3f)translation).x,
			     ((Vec3f)translation).y,
			     ((Vec3f)translation).z
			     );
  ++context.m_nTabs;
  for (int i = 0; i < children.GetLength (); i++)
    children [i] -> Render (&context);
  return TRUE;
}

BOOL MTransform2D :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:Transform2D\n\trotationAngle: %f\n\ttranslation: %f, %f\n",
			     context.m_nTabs*3,
			     ' ',
			     GetID (),
			     ((float)rotationAngle),
			     ((Vec2f)translation).x,
			     ((Vec2f)translation).y
			     );
  ++context.m_nTabs;
  for (int i = 0; i < children.GetLength (); i++)
    children [i] -> Render (&context);
  return TRUE;
}

BOOL MViewpoint :: Render (PresenterContext *pContext)
{
  ZPresenterContext context = *(ZPresenterContext*) pContext;
  if (GetUpdateTime () == GetPresenter () -> GetTime ())
    GetPresenter () -> Logf (context.GetTime (), 
			     "%*c%d:Viewpoint\n\torientation: (%f, %f, %f), %f\n\tposition: %f, %f, %f\n",
			     context.m_nTabs*3,
			     ' ',
			     GetID (),
			     ((Rotation)orientation).x,
			     ((Rotation)orientation).y,
			     ((Rotation)orientation).z,
			     ((Rotation)orientation).theta,
			     ((Vec3f)position).x,
			     ((Vec3f)position).y,
			     ((Vec3f)position).z
			     );
  return TRUE;
}

BOOL MBitWrapper :: Render (PresenterContext *pContext)
{
  ZPresenterContext *pCont = (ZPresenterContext *) pContext;
	++pCont -> m_nTabs;

	if (!IsStreamOpen())
  {
    OpenStream();
    return TRUE;
  }

  //double dwTime;
	pCont -> m_dwCTS = 0xffffffff;
	if (node != NULL_OBJECT)
	{	node -> Render (pContext);
		/* if (pCont ->m_dwCTS != 0xffffffff)
		{	if (pCont ->m_dwCTS == 1)
				dwTime = 0;
			else
				dwTime = pCont ->m_dwCTS / (double)1000;
			currentTime = dwTime;
			if ((double) startTime != -1)
			{	bool b =
					dwTime >= (double) startTime &&
					dwTime < (double) startTime + (double) duration -
					GetPresenter () ->GetIterationInterval () / 1000.;
				if (b != bActive)
				{	bActive = b;
					isActive = b;
				}
			}
    } */
	}
	--pCont -> m_nTabs;
	return TRUE;
}

MMeshGrid :: MMeshGrid ()
{
  m_bInit = false;
  m_lastTime = 0;
  m_pInfo = NULL;
}


void MMeshGrid :: SetDecoderSpecificInfo(MBitWrapper *pParent)
{
  AFXDecoder *afxDec = (AFXDecoder *) pParent->GetDecoder();
  MGDecoder  *mgDec = (MGDecoder *)afxDec->GetDecoder();
  //MeshGridDecoderSpecificInfo *m_pInfo = mgDec->m_pDecSpecificInfo;

  for (int k = 0; k < 3; k++)
  {
    nLevels.InsertAt(k, mgDec->nLevels[k], false);
    nSlices.InsertAt(k, mgDec->nSlices[k], false);
  }

  // set the isLoading flag
  isLoading = true;
}

BOOL MMeshGrid :: Render (PresenterContext *pContext)
{
	ZPresenterContext *pCont = (ZPresenterContext *) pContext;
	++pCont -> m_nTabs;

  MBitWrapper *pParent = (MBitWrapper *)GetParent();
  ASSERT(pParent != NULL);

  // first time get the decoder specific info from the stream
  if (!m_bInit)
  {
    m_bInit = true;
    SetDecoderSpecificInfo(pParent);
  }

  // get the stream
  MediaStream *pStream = pParent->GetStream();
  ASSERT(pStream != NULL);

  // get the decoded data buffer
  int rnSize, rnStatus;
  DWORD rdvTime;
  BOOL rbEOU;
	BYTE * pOutput = (BYTE *)pStream->Fetch(rnSize, rdvTime, rbEOU, rnStatus);
  if (pOutput != NULL && rdvTime > m_lastTime)
  {
    // update the time
    m_lastTime = rdvTime;

    // retrieve tha data from the stream
    MGDataExchange dataExchange(pOutput, true);

    BYTE tag;
    DWORD size;
    void *pData = NULL;
    while (pData = dataExchange.FetchNext(tag, size))
    {
       switch (tag)
       {
       case MGDataExchange::TAG_GRID_COORD:   GetGridCoord(pData, size);  break;
       case MGDataExchange::TAG_VERTEX_COORD: GetVertCoord(pData, size);  break;
       case MGDataExchange::TAG_TRIANGLES:    GetTriangles(pData, size);  break;
       case MGDataExchange::TAG_OFFSETS:      GetOffsets(pData, size);    break;
       case MGDataExchange::TAG_NO_VERTICES:  GetNoVertices(pData, size); break;
       default: ASSERT(0); return FALSE;
       }
    }
  }

	return TRUE;
}

bool MMeshGrid :: GetGridCoord (void *pData, int size)
{
  ASSERT(size % 3 == 0);
  int nElements = size / 3;

  Vec3f *pointArray = (Vec3f *)pData;
  Coordinate *coord = (Coordinate *) gridCoord.GetNode();
  MF<Vec3f> &points = coord->point;

  points.Remove();
  for (int k = 0; k < nElements; k++)
    points += pointArray[k];

  return true;
}

bool MMeshGrid :: GetVertCoord (void *pData, int size)
{
  ASSERT(size % 3 == 0);
  int nElements = size / 3;

  Vec3f *pointArray = (Vec3f *)pData;
  Coordinate *vertices = (Coordinate *) coord.GetNode();
  MF<Vec3f> &points = vertices->point;

  points.Remove();
  for (int k = 0; k < nElements; k++)
    points += pointArray[k];

  return true;
}

bool MMeshGrid :: GetTriangles (void *pData, int size)
{
  ASSERT(size % 4 == 0);

  int *indices = (int *)pData;
  coordIndex.Remove();
  for (int k = 0; k < size; k++)
    coordIndex += indices[k];

  return true;
}

bool MMeshGrid :: GetOffsets (void *pData, int size)
{
  ASSERT(size % 4 == 0);

  int *indices = (int *)pData;
  coordIndex.Remove();
  for (int k = 0; k < size; k++)
    coordIndex += indices[k];

  return true;
}

bool MMeshGrid :: GetNoVertices (void *pData, int size)
{
  ASSERT(size % 4 == 0);

  int *indices = (int *)pData;
  coordIndex.Remove();
  for (int k = 0; k < size; k++)
    coordIndex += indices[k];

  return true;
}

//PG 021211
MWaveletSubdivisionSurface :: MWaveletSubdivisionSurface ()
{

  m_bInit = false;
  m_pInfo = NULL;

}


void MWaveletSubdivisionSurface :: SetDecoderSpecificInfo(MBitWrapper *)
{

//  AFXDecoder *afxDec = (AFXDecoder *) pParent->GetDecoder();
//  WaveSurfDecoder  *wsDec = (WaveSurfDecoder *)afxDec->GetDecoder();
//  WMDecoderSpecificInfo *m_pInfo = wsDec->m_pDecSpecificInfo;

  
}

/*--------------------------------------------
Fill the base mesh and decoderSpecificInfo
--------------------------------------------*/
BOOL MWaveletSubdivisionSurface :: Render (PresenterContext *)
{
/*	int i;
	


	MBitWrapper *pParent = (MBitWrapper *)GetParent();
  ASSERT(pParent != NULL);

  AFXDecoder *afxDec = (AFXDecoder *) pParent->GetDecoder();
  WaveSurfDecoder  *wsDec = (WaveSurfDecoder *)afxDec->GetDecoder();

  // first time get the decoder specific info from the stream
  if (!m_bInit)
  {
		m_bInit = true;
		SetDecoderSpecificInfo(pParent);
  

	  //Fill the basemesh	
	  SFGeometryNode *WIFS=(SFGeometryNode *)baseMesh.GetNode();
	  IndexedFaceSet *WWIFS=(IndexedFaceSet*)WIFS;
	  Coordinate *crd=(Coordinate*)WWIFS->coord.GetNode();
	  MF<Vec3f> &pt=crd->point;
	  MF<int> &crdInd=WWIFS->coordIndex;
	  
	  for (i=0; i<pt.GetLength(); i++) {
		  wsDec->m_BM.Vertex[i][0]=pt[i].x;
		  wsDec->m_BM.Vertex[i][1]=pt[i].y;
		  wsDec->m_BM.Vertex[i][2]=pt[i].z;
	  }
	  for (i=0; i<crdInd.GetLength(); i++) {
		  wsDec->m_BM.Face[i][0]=crdInd[i*4];
		  wsDec->m_BM.Face[i][1]=crdInd[i*4+1];
		  wsDec->m_BM.Face[i][2]=crdInd[i*4+2];
	  }
		wsDec->m_BM.NbFace=pt.GetLength();
		wsDec->m_BM.NbVertex=crdInd.GetLength()/4;

		


  }

	*/ //PG : is replaced by the render() function in WaveletSubdivisionProxy.cpp
	
  return TRUE;
}


