/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Mikael Bourges-Sevenier at mikael.bourgessevenier@cnet.francetelecom.fr.
----------------------------------------------------------------------*/

#if !defined(_MNodes_h)	// Sentry, use file only if it's not already included.
#define _MNodes_h

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Compositor (simulation)
FILE:       MNodes.h
AUTHORS:    Mikael Bourges-Sevenier (France Telecom CNET/DIH/ATI)
OVERVIEW:   Definitions of my node overloading.
-------------------------------------------------------------------------*/

#include "Nodes.h"
#include "ODP.h"

class MGroup : public Group
{
	virtual BOOL Render (PresenterContext *context);
};

class MSphere : public Sphere
{
	virtual BOOL Render (PresenterContext *context);
};

class MOrientationInterpolator : public OrientationInterpolator
{
	virtual BOOL Render (PresenterContext *context);
};

class MScalarInterpolator : public ScalarInterpolator
{
	virtual BOOL Render (PresenterContext *context);
};

class MTouchSensor : public TouchSensor
{
	virtual BOOL Render (PresenterContext *context);
};

class MTimeSensor : public TimeSensor
{
	virtual BOOL Render (PresenterContext *context);
};

class MMaterial : public Material
{
	virtual BOOL Render (PresenterContext *context);
};

class MTransform : public Transform
{
	virtual BOOL Render (PresenterContext *context);
};

class MTransform2D : public Transform2D
{
	virtual BOOL Render (PresenterContext *context);
};

class MViewpoint : public Viewpoint
{
	virtual BOOL Render (PresenterContext *context);
};

class MBitWrapper : public BitWrapper
{
	virtual BOOL Render (PresenterContext *context);
};

class MMeshGrid : public MeshGrid
{
public:
  MMeshGrid();

private:
	virtual BOOL Render (PresenterContext *context);

  bool GetGridCoord(void *pData, int size);
  bool GetVertCoord(void *pData, int size);
  bool GetTriangles(void *pData, int size);
  bool GetOffsets(void *pData, int size);
  bool GetNoVertices(void *pData, int size);

  void SetDecoderSpecificInfo(MBitWrapper *pParent);

protected:
  bool  m_bInit;
  DWORD m_lastTime;
  MeshGridDecoderSpecificInfo *m_pInfo;
};

class MWaveletSubdivisionSurface : public WaveletSubdivisionSurface
{
public:
	MWaveletSubdivisionSurface(); 

private:
	virtual BOOL Render (PresenterContext *context);


  void SetDecoderSpecificInfo(MBitWrapper *pParent);

protected:
 
	bool  m_bInit;
  WMDecoderSpecificInfo *m_pInfo;
};

void MikeOverloads ();

#endif                                      // _ZNodes_h sentry.
