/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  Presenter (simulation)
FILE:       Nodes-Zvi.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Implementation of my overloading nodes
-------------------------------------------------------------------------*/

#include "defs.h"
#include "ZNodes.h"
#include "Executiv.h"
#include "Presenter.h"
#include "Stream.h"
#include "Clock.h"

BOOL ZGroup :: Render (PresenterContext *pContext)
{
	ZPresenterContext context = *(ZPresenterContext *) pContext;
	++context.m_nTabs;
	for (int i = 0; i < children.GetLength (); i++)
	{	MediaObject *pObject = children [i];
		if (pObject)
			pObject -> Render (&context);
	}
	return TRUE;
}

BOOL ZOrderedGroup :: Render (PresenterContext *pContext)
{
	ZPresenterContext context = *(ZPresenterContext *) pContext;
	++context.m_nTabs;
	for (int i = 0; i < children.GetLength (); i++)
	{	MediaObject *pObject = children [i];
		if (pObject)
			pObject -> Render (&context);
	}
	return TRUE;
}

BOOL ZTransform :: Render (PresenterContext *pContext)
{
	ZPresenterContext context = *(ZPresenterContext *) pContext;

	if (GetUpdateTime () == GetPresenter () -> GetTime ())
		GetPresenter () -> Logf (context.GetTime (), 
			"%*c%d:Transform - center: %f %f %f - scale: %f %f %f\n",
			context.m_nTabs*3,
			' ',
			GetID (),
			((Vec3f)center).x, ((Vec3f)center).y, ((Vec3f)center).z,
			((Vec3f)scale).x, ((Vec3f)scale).y, ((Vec3f)scale).z
		);
	++context.m_nTabs;
	for (int i = 0; i < children.GetLength (); i++)
		children [i] -> Render (&context);
	return TRUE;
}

BOOL ZTransform2D :: Render (PresenterContext *pContext)
{
	ZPresenterContext context = *(ZPresenterContext *) pContext;
	if (GetUpdateTime () == GetPresenter () -> GetTime ())
		GetPresenter () -> Logf (context.GetTime (), 
			"%*c%d:Transform2D - center: %f %f - scale: %f %f\n",
			context.m_nTabs*3,
			' ',
			GetID (),
			((Vec2f)center).x, ((Vec2f)center).y,
			((Vec2f)scale).x, ((Vec2f)scale).y
		);
	++context.m_nTabs;
	for (int i = 0; i < children.GetLength (); i++)
		children [i] -> Render (&context);
	return TRUE;
}

void ZTransform2D :: OnUpdate ()
{
	GetPresenter () -> Logf (GetClock (), 
		"Transform2D was updated. Now contains %d children\n",
		children.GetLength (),
		GetUpdateTime ()
	);
}

BOOL ZShape :: Render (PresenterContext *pContext)
{
	ZPresenterContext context = *(ZPresenterContext *) pContext;
	++context.m_nTabs;
	if (appearance != NULL_OBJECT)
		appearance -> Render (&context);
	if (geometry != NULL_OBJECT)
		geometry -> Render (&context);
	return TRUE;
}

BOOL ZAppearance :: Render (PresenterContext *pContext)
{
	ZPresenterContext context = *(ZPresenterContext *) pContext;
	++context.m_nTabs;
	if (material != NULL_OBJECT) //MBS 110498 added "!=NULL_OBJECT"
		material -> Render (&context);
	if (texture != NULL_OBJECT) //MBS 110498 added "!=NULL_OBJECT"
		texture -> Render (&context);
	return TRUE;
}

BOOL ZMaterial2D :: Render (PresenterContext *pContext)
{
    //MBS 110498: changed diffuseColor to emissiveColor
	ZPresenterContext context = *(ZPresenterContext *) pContext;
	if (GetUpdateTime () + 1 == GetPresenter () -> GetTime ())
		GetPresenter () -> Logf (context.GetTime (), 
			"%*c%d:Material2D - emissiveColor: %f %f %f\n",
			context.m_nTabs*3,
			' ',
			GetID (),
			((Color3f)emissiveColor).r,
			((Color3f)emissiveColor).g,
			((Color3f)emissiveColor).b
		);
	return TRUE;
}

BOOL ZMaterial :: Render (PresenterContext *pContext)
{
	ZPresenterContext context = *(ZPresenterContext *) pContext;
	if (GetUpdateTime () + 1 == GetPresenter () -> GetTime ())
		GetPresenter () -> Logf (context.GetTime (), 
			"%*c%d:Material - diffuseColor: %f %f %f\n",
			context.m_nTabs*3,
			' ',
			GetID (),
			((Color3f)diffuseColor).r,
			((Color3f)diffuseColor).g,
			((Color3f)diffuseColor).b
		);
	return TRUE;
}

BOOL ZSound2D :: Render (PresenterContext *pContext)
{
	ZPresenterContext context = *(ZPresenterContext *) pContext;
	++context.m_nTabs;
	if (source) //MBS 110498: source replaces sound now
		source -> Render (&context);
	return TRUE;
}

BOOL ZSound :: Render (PresenterContext *pContext)
{
	ZPresenterContext context = *(ZPresenterContext *) pContext;
	++context.m_nTabs;
	if (source)
		source -> Render (&context);
	return TRUE;
}

BOOL ZCircle :: Render (PresenterContext *pContext)
{
	ZPresenterContext context = *(ZPresenterContext *) pContext;
	if (GetUpdateTime () != m_dUpdated)
	{	m_dUpdated = GetUpdateTime ();
		GetPresenter () -> Logf (context.GetTime (), 
			"%*c%d:Circle - radius: %f\n",
			context.m_nTabs*3,
			' ',
			GetID (),
			(float) radius
		);
	}
	return TRUE;
}

BOOL ZText :: Render (PresenterContext *pContext)
{
	ZPresenterContext context = *(ZPresenterContext *) pContext;
	if (GetUpdateTime () == GetPresenter () -> GetTime ())
	{	int i;
		for (i = 0; i < string.GetLength (); i++)
			GetPresenter () -> Logf (context.GetTime (), 
				"%*c%d:Text2D - string [%d]: %s\n",
				context.m_nTabs*3,
				' ',
				GetID (),
				i,
				(LPCSTR) string [i]
		);
		if (fontStyle != NULL_OBJECT)
			fontStyle -> Render (&context);
	}
	return TRUE;
}

BOOL ZFontStyle :: Render (PresenterContext *pContext)
{
	ZPresenterContext context = *(ZPresenterContext *) pContext;
	if (GetUpdateTime () == GetPresenter () -> GetTime ())
	{	int i;
		for (i = 0; i < family.GetLength (); i++)
			GetPresenter () -> Logf (context.GetTime (), 
				"%*c%d:FontStyle - family [%d]: %s\n",
				context.m_nTabs*3,
				' ',
				GetID (),
				i,
				(LPCSTR) family [i]
		);
		for (i = 0; i < justify.GetLength (); i++)
			GetPresenter () -> Logf (context.GetTime (), 
				"%*c%d:FontStyle - justify [%d]: %s\n",
				context.m_nTabs*3,
				' ',
				GetID (),
				i,
				(LPCSTR) justify [i]
		);
	}
	return TRUE;
}

BOOL ZMovieTexture :: Render (PresenterContext *pContext)
{
	int nSize;
	BOOL bRecurrent, bEOU;
	LPVOID pFrame;
	DWORD dwTime;
	ZPresenterContext *pCont = (ZPresenterContext *) pContext;
	
	/*-------------------------------------------------------------
	startTime / stopTime fields need to be offset by current time on first access
	-------------------------------------------------------------*/
	startTime.ApplyOffset (pContext);
	stopTime.ApplyOffset (pContext);

	double currentTime = pContext -> GetTime () / 1000.;
	bool bActive =
		 (startTime >= stopTime || currentTime < stopTime) && currentTime >= startTime;
	if (!m_bStart && bActive) 
	{	m_bStart = TRUE;
		OpenStream ();
		GetPresenter () -> Logf (pCont ->GetTime (), "Movie started\n");
		printf ("Movie started\n");
		if (!GetPresenter () -> ClockActive ())
			GetPresenter () -> SetClock (TRUE);
	}

	if (m_bStart && !bActive)
	{	CloseStream ();
		GetPresenter () -> Logf (pCont ->GetTime (), "Movie stopped\n");
		printf ("Movie stopped\n");
		m_bStart = FALSE;
	}

	if (m_bStart)
	{	pFrame = Fetch (
			nSize,
			dwTime,
			bEOU,
			bRecurrent,
			TRUE,
			FALSE
		);
		if (pFrame == END_OF_STREAM)
		{	CloseStream ();
			m_bStart = FALSE;
		}
		else if (pFrame != NULL)
		{	if (dwTime != m_dwLastFrameTS)
			{	GetPresenter () ->
				    //MBS 110498: url is of type MFUrl so display the ODid
					//LogStream ("MovieTexture", url, nSize, dwTime);
					LogStream (pCont ->GetTime(), "MovieTexture", GetID (), GetObjectDescriptorID (), nSize, dwTime);
				pCont ->m_dwCTS = m_dwLastFrameTS = dwTime;
			}
			Release (FALSE);
		}
	}
	return TRUE;
}


BOOL ZImageTexture :: Render (PresenterContext *pContext)
{
	int nSize;
	BOOL bRecurrent, bEOU;
	LPVOID pFrame;
	DWORD dwTime;
	ZPresenterContext *pCont = (ZPresenterContext *) pContext;
	
	if (!m_bStart)
	{	m_bStart = TRUE;
		OpenStream ();
		GetPresenter () -> Logf (pCont ->GetTime (), "Image opened\n");
		printf ("Image opened\n");
		if (!GetPresenter () -> ClockActive ())
			GetPresenter () -> SetClock (TRUE);
	}

	if (m_bStart)
	{	pFrame = Fetch (
			nSize,
			dwTime,
			bEOU,
			bRecurrent,
			TRUE,
			FALSE,
			0,
			TRUE
		);
		if (pFrame == END_OF_STREAM)
		{	CloseStream ();
			m_bStart = FALSE;
		}
		else if (pFrame != NULL)
		{	if (!bRecurrent)
			{	GetPresenter () ->
					LogStream (pCont ->GetTime (), "ImageTexture", GetID (), url [0], nSize, dwTime);
				pCont ->m_dwCTS = dwTime;
			}
			Release (FALSE);
		}
	}
	return TRUE;
}

void ZImageTexture :: DeleteObject ()
{
	if (m_bStart)
		GetPresenter () -> Logf (GetClock (), "Image node deleted\n");
	ImageTexture :: DeleteObject ();
}

BOOL ZAudioSource :: Render (PresenterContext *pContext)
{
	int nSize;
	DWORD dwTime;
	LPVOID pFrame;
	BOOL bEOU;
	ZPresenterContext *pCont = (ZPresenterContext *) pContext;
	
	/*-------------------------------------------------------------
	startTime / stopTime fields need to be offset by current time on first access
	-------------------------------------------------------------*/
	startTime.ApplyOffset (pContext);
	stopTime.ApplyOffset (pContext);

	double currentTime = pContext -> GetTime () / 1000.;
	bool bActive =
		 (startTime >= stopTime || currentTime < stopTime) && currentTime >= startTime;

	if (!m_bStart && bActive)
	{	m_bStart = TRUE;
		OpenStream ();
		GetPresenter () -> Logf (pCont ->GetTime (), "Audio started\n");
		printf ("Audio started\n");
		m_nSamples = 0;
		GetPresenter () -> SetClock (TRUE);
	}

	if (m_bStart && !bActive)
	{	CloseStream ();
		GetPresenter () -> Logf (pCont ->GetTime (), "Audio stopped\n");
		printf ("Audio stopped\n");
		m_bStart = FALSE;
		GetPresenter () -> SetClock (FALSE);
	}

	while (
		m_bStart &&
		m_nSamples <
		(int) (GetPresenter () -> GetClockTime () / 30 + 2)
	)
	{	if (GetStream () != NULL)
		{	pFrame = GetStream () -> FetchNext (nSize, dwTime, bEOU, FALSE);
			if (pFrame == END_OF_STREAM)
			{	CloseStream ();
				m_bStart = FALSE;
				GetPresenter () -> SetClock (FALSE);
			}
			else if (pFrame != NULL)
			{
				/*-----------------------------------------------------------------
				First time you do it, reset the global clock reference
				-----------------------------------------------------------------*/
				if (m_nSamples == 0)
				{	GetPresenter () -> SetClock (TRUE);
					GetStream () -> AdjustClock (dwTime);
				}

				GetPresenter () ->
					LogStream (pCont ->GetTime (), "AudioSource", GetID (), GetObjectDescriptorID (), nSize, dwTime);
				pCont ->m_dwCTS = dwTime;
				Release (TRUE);
				++m_nSamples;
				continue;
			}
		}
		else if (m_bODdeleted == false)
		{	m_bODdeleted = true;
			printf ("Audio OD deleted\n");
		}
		break;
	}
	return TRUE;
}

BOOL ZAudioClip :: Render (PresenterContext *pContext)
{
	int nSize;
	DWORD dwTime;
	LPVOID pFrame;
	BOOL bEOU;
	ZPresenterContext *pCont = (ZPresenterContext *) pContext;
	
	/*-------------------------------------------------------------
	startTime / stopTime fields need to be offset by current time on first access
	-------------------------------------------------------------*/
	startTime.ApplyOffset (pContext);
	stopTime.ApplyOffset (pContext);

	double currentTime = pContext -> GetTime () / 1000.;
	bool bActive =
		 (startTime >= stopTime || currentTime < stopTime) && currentTime >= startTime;

	if (!m_bStart && bActive)
	{	m_bStart = TRUE;
		OpenStream ();
		GetPresenter () -> Logf (pCont ->GetTime (), "Audio clip started\n");
		printf ("Audio clip started\n");
		m_nSamples = 0;
		GetPresenter () -> SetClock (TRUE);
	}

	if (m_bStart && !bActive)
	{	CloseStream ();
		GetPresenter () -> Logf (pCont ->GetTime (), "Audio clip stopped\n");
		printf ("Audio clip stopped\n");
		m_bStart = FALSE;
		GetPresenter () -> SetClock (FALSE);
	}

	while (
		m_bStart &&
		m_nSamples <
		(int) (GetPresenter () -> GetClockTime () / 30 + 2)
	)
	{	if (GetStream () != NULL)
		{	pFrame = GetStream () -> FetchNext (nSize, dwTime, bEOU, FALSE);
			if (pFrame == END_OF_STREAM)
			{	CloseStream ();
				GetPresenter () -> Logf (pCont ->GetTime (), "Audio clip finished\n");
				printf ("Audio clip finished\n");
				m_bStart = FALSE;
				GetPresenter () -> SetClock (FALSE);
			}
			else if (pFrame != NULL)
			{
				/*-----------------------------------------------------------------
				First time you do it, reset the global clock reference
				-----------------------------------------------------------------*/
				if (m_nSamples == 0)
				{	GetPresenter () -> SetClock (TRUE);
					GetStream () -> AdjustClock (dwTime);
				}

				GetPresenter () ->
					//MBS 110498: url is now of type, dump only first url
					//MFUrlLogStream ("AudioSource", url, nSize, dwTime);
					LogStream (pCont ->GetTime (), "AudioSource", GetID (), GetObjectDescriptorID (), nSize, dwTime);
				pCont ->m_dwCTS = dwTime;
				Release (TRUE);
				++m_nSamples;
				continue;
			}
		}
		break;
	}
	return TRUE;
}

/*-------------------------------------------------------------
This sample implementation implements most of TimeSensor functionality, and displays a
line whenever 'cycleTime' or 'isActive'	is changed
-------------------------------------------------------------*/
BOOL ZTimeSensor :: Render (PresenterContext *pContext)
{
	/*-------------------------------------------------------------
	startTime / stopTime fields need to be offset by current time on first access
	-------------------------------------------------------------*/
	startTime.ApplyOffset (pContext);
	stopTime.ApplyOffset (pContext);

	double currentTime = pContext -> GetTime () / 1000.;
	bool bActive =
		(startTime >= stopTime || currentTime < stopTime) && currentTime >= startTime &&
		!m_bFinished && startTime > 0;

	if (!(Bool) enabled || !pContext ->IsActive ())
	{	m_bActive = false;
		return true;
	}
	
	/*-------------------------------------------------------------
	Check if toggling activity
	-------------------------------------------------------------*/
	if (bActive != m_bActive)
	{	isActive = m_bActive = bActive;
		GetPresenter () -> Logf (
			pContext -> GetTime (), 
			"%*c%d:TimeSensor - isActive: %d, startTime: %lf, stopTime: %lf\n",
			((ZPresenterContext *) pContext) -> m_nTabs*3,
			' ',
			GetID (),
			(int) bActive,
			(double) startTime,
			(double) stopTime
		);
		if (bActive)
			m_dCycleEndTime = currentTime + cycleInterval;
	}

	/*-------------------------------------------------------------
	Operate an active sensor
	-------------------------------------------------------------*/
	if (!bActive)
		return true;

	time = currentTime;
	if (currentTime >= m_dCycleEndTime)
	{	cycleTime = currentTime;
		GetPresenter () -> Logf (
			pContext -> GetTime (), 
			"%*c%d:TimeSensor - cycleTime: %lf\n",
			((ZPresenterContext *) pContext) -> m_nTabs*3,
			' ',
			GetID (),
			currentTime
		);
		if ((Bool) loop)
			m_dCycleEndTime += cycleInterval;
		else
			m_bFinished = true;
	}
		
	return true;
}

#ifdef AUDIOBIFSV3
/*-------------------------------------------------------------
This sample implementation implements a basic render function 
for testing
-------------------------------------------------------------*/

BOOL ZAdvancedAudioBuffer :: Render (PresenterContext *pContext)
{
 	ZPresenterContext *pCont = (ZPresenterContext *) pContext;

	ZPresenterContext context = *(ZPresenterContext *) pContext;
	++context.m_nTabs;
	GetPresenter () -> Logf (pCont ->GetTime (), "AdvancedAudioBuffer\n");
	for (int i = 0; i < children.GetLength (); i++)
	{	MediaObject *pObject = children [i];
		if (pObject)
			pObject -> Render (&context);
	}	 
	return TRUE;        
};

BOOL ZAudioChannelConfig :: Render (PresenterContext *pContext)
{
	ZPresenterContext *pCont = (ZPresenterContext *) pContext;

	ZPresenterContext context = *(ZPresenterContext *) pContext;
	++context.m_nTabs;
	GetPresenter () -> Logf (pCont ->GetTime (), "AudioChannelConfig\n");
	for (int i = 0; i < children.GetLength (); i++)
	{	MediaObject *pObject = children [i];
		if (pObject)
			pObject -> Render (&context);
	}	 
	return TRUE;          
};

BOOL ZSurroundingSound :: Render (PresenterContext *pContext)
{
	ZPresenterContext *pCont = (ZPresenterContext *) pContext;

	ZPresenterContext context = *(ZPresenterContext *) pContext;
	++context.m_nTabs;
	GetPresenter () -> Logf (pCont ->GetTime (), "SurroundingSound\n");
	if (source)
	{
		source -> Render (&context);
	}
	return TRUE;        
};

BOOL ZTransform3DAudio :: Render (PresenterContext *pContext)
{
	ZPresenterContext *pCont = (ZPresenterContext *) pContext;

	ZPresenterContext context = *(ZPresenterContext *) pContext;
	++context.m_nTabs;
	GetPresenter () -> Logf (pCont ->GetTime (), "Transform3DAudio\n");
	for (int i = 0; i < children.GetLength (); i++)
	{	MediaObject *pObject = children [i];
		if (pObject)
			pObject -> Render (&context);
	}
	return TRUE;         
};

BOOL ZWideSound :: Render (PresenterContext *pContext)
{
	ZPresenterContext *pCont = (ZPresenterContext *) pContext;

	ZPresenterContext context = *(ZPresenterContext *) pContext;
	++context.m_nTabs;
	GetPresenter () -> Logf (pCont ->GetTime (), "WideSound\n");
	if (source)
	{
		source -> Render (&context);
	}
	return TRUE;
}
#endif
