/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1997.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

#if !defined(_ZNodes_h)	// Sentry, use file only if it's not already included.
#define _ZNodes_h

#define AUDIOBIFSV3


/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Compositor (simulation)
FILE:       ZNodes.h
AUTHORS:    Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   Definitions of my node overloading.
-------------------------------------------------------------------------*/

#include "Nodes.h"
#include "RendObj.h"

class ZGroup : public Group
{
	virtual BOOL Render (PresenterContext *context);
};

class ZOrderedGroup : public OrderedGroup
{
	virtual BOOL Render (PresenterContext *context);
};

class ZTransform2D : public Transform2D
{
	virtual BOOL Render (PresenterContext *context);
	virtual void OnUpdate ();
};

class ZTransform : public Transform
{
	virtual BOOL Render (PresenterContext *context);
};

class ZShape : public Shape
{
	virtual BOOL Render (PresenterContext *context);
};

class ZAppearance : public Appearance
{
	virtual BOOL Render (PresenterContext *context);
};

class ZMaterial : public Material
{
	virtual BOOL Render (PresenterContext *context);
};

class ZMaterial2D : public Material2D
{
	virtual BOOL Render (PresenterContext *context);
};

class ZCircle : public Circle
{
	virtual BOOL Render (PresenterContext *context);
	double m_dUpdated;

public:
	ZCircle () : m_dUpdated (0) {}
};

class ZText : public Text 
{
	virtual BOOL Render (PresenterContext *context);
};

class ZFontStyle : public FontStyle 
{
	virtual BOOL Render (PresenterContext *context);
};

class ZSound2D : public Sound2D
{
	virtual BOOL Render (PresenterContext *context);
};

class ZSound : public Sound
{
	virtual BOOL Render (PresenterContext *context);
};

class ZTimeSensor : public TimeSensor
{
	bool m_bActive, m_bFinished;
	double m_dCycleEndTime;
	virtual BOOL Render (PresenterContext *context);

public:
	ZTimeSensor () : m_bActive (false), m_bFinished (false) {}
};

class ZMovieTexture : public MovieTexture
{ 
	BOOL m_bStart;
	DWORD m_dwIteration;
	int m_nInstance;
	DWORD m_dwLastFrameTS;
	BOOL m_bSkip;

	virtual BOOL Render (PresenterContext *);

public:
	ZMovieTexture () :
		m_bStart (FALSE),
		m_dwIteration (0),
		m_dwLastFrameTS (0xffffffff)
	{}
};

class ZImageTexture : public ImageTexture
{ 
	BOOL m_bStart;
	virtual BOOL Render (PresenterContext *);
	void DeleteObject ();

public:
	ZImageTexture () : m_bStart (FALSE) {}
};

class ZAudioSource : public AudioSource
{
	DWORD m_dwTime;
	int m_nSamples;
	BOOL m_bStart, m_bODdeleted;

	virtual BOOL Render (PresenterContext *);

public:
	ZAudioSource () :
		m_nSamples (0),
		m_bStart (FALSE),
		m_bODdeleted (false)
	{}
};

class ZAudioClip : public AudioClip
{
	DWORD m_dwTime;
	int m_nSamples;
	BOOL m_bStart;

	virtual BOOL Render (PresenterContext *);

public:
	ZAudioClip () :
		m_nSamples (0),
		m_bStart (FALSE)
	{}
};

#ifdef AUDIOBIFSV3

class ZAdvancedAudioBuffer : public AdvancedAudioBuffer
{
	virtual BOOL Render (PresenterContext *context);
};

class ZAudioChannelConfig : public AudioChannelConfig
{
	virtual BOOL Render (PresenterContext *context);
};

class ZSurroundingSound : public SurroundingSound
{
	virtual BOOL Render (PresenterContext *context);
};

class ZTransform3DAudio : public Transform3DAudio
{
	virtual BOOL Render (PresenterContext *context);
};

class ZWideSound : public WideSound
{
	virtual BOOL Render (PresenterContext *context);
};

#endif                                      // AUDIOBIFSV3 

#endif                                      // _ZNodes_h sentry.
