/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from_using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1998.
************************************************************************

For more information or to receive an up to date version of this module,
you may call Zvi Lifshitz at zvil@csi.com.
----------------------------------------------------------------------*/

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:  all
FILE:       main.cpp
AUTHOR:     Zvi Lifshitz (Triton R&D Ltd.)
OVERVIEW:   main() function for testing.
-------------------------------------------------------------------------*/
#include "defs.h"
#include <stdio.h>
#include "Executiv.h"
#include "ztl.h"
#include "App.h"

//#define SECOND_TRY

/*-----------------------------------------------------------------
The application body is derived from_Application
-----------------------------------------------------------------*/
class MyApp : public Application
{
	LPCSTR m_szFileName;
	ZSemaphore m_smpDemuxWorking;
	Executive m_myExecutive;

	virtual void OnConnect (
		RESPONSE eResponse = RESPONSE_OK
	);
	virtual void OnFinish (
		RESPONSE eResponse = RESPONSE_END_OF_DATA
	);

	//stubs for new callbacks
	virtual void OnFatalError (FATAL_ERROR, LPCSTR ) {}
	virtual int Navigate(LPCSTR ) {return 1;}
	virtual void OverSensor(unsigned int ) {}
	virtual void SetSize(unsigned int , unsigned int ) {}
	virtual void ViewportStackChanged(class BindableStack *) {}
	virtual const char *GetApplicationName() {return "IM1Player";}
	virtual const char *GetApplicationVersion() {return "Unknown";}
	virtual const char *GetWorldURL() {return m_szFileName;}

public:
	MyApp (FILE *fTraceFileName, int flags) :
		m_smpDemuxWorking (1, 0),
		m_myExecutive (this, NULL, fTraceFileName, flags)
	{}
	void Run (LPCSTR szFileName);
private:
	virtual void OnError (NON_FATAL_ERROR nError, va_list args);
};


/*-----------------------------------------------------------------
Instaniates a derivation from_Application, run it,
and waits till Application :: Finish() is called.
-----------------------------------------------------------------*/
int main (int argc, char **argv)
{
	if (argc < 2 || argv [1][0] == '-')
	{	printf ("Missing file name\n");
		return -1;
	}
	LPCSTR szFileName = *++argv;
	FILE *fTrace = NULL;
	int flags = TRACE_NONE;

	/*-------------------------------------------------------------
	Parse the trace argument
	-------------------------------------------------------------*/
	if (argc >= 3 && (*++argv)[0] == '-')
	{	int c;
		for (int i = 1; (c = (*argv) [i]) != 0; i++)
			switch (tolower (c)) {
			case 'p':
				flags |= TRACE_PARSE;
				break;
			case 'v':
				flags |= TRACE_VERBOSE;
				break;
			case 'c':   // BF Conformance
				flags |= TRACE_CONFORMANT;  
				break;
			case 'h':
				flags |= TRACE_HEADERS;
				break;
			case 's':
				flags = -1;
				break;
			}
		--argc;
	}
	if (flags == -1)
		flags = TRACE_NONE;
	else if (flags == TRACE_NONE)
		flags = TRACE_PARSE;
	if (flags != TRACE_NONE)
	{	/*-------------------------------------------------------------
		Open the trace file
		-------------------------------------------------------------*/
		LPCSTR szTraceFile = "Trace.txt";
		if (argc >= 3)
			szTraceFile = *++argv;
		fTrace = fopen (szTraceFile, "w");
		if (!fTrace)
			printf ("Cannot open %s\n", szTraceFile);
	}
			
	MyApp myApp (fTrace, flags);
	myApp.Run (szFileName);

#ifdef SECOND_TRY
//	Sleep (100);
	printf ("Starting second run\n");
	myApp.Run (szFileName);
#endif

	if (fTrace)
		fclose (fTrace);
	return 0;
}

/*-----------------------------------------------------------------
Run a session and wait on a semaphore till it finishes
-----------------------------------------------------------------*/
void MyApp :: Run (LPCSTR szFileName)
{
	m_szFileName = szFileName;
	m_myExecutive.Start (szFileName);
	printf ("Hit Enter to finish\n");
	getchar ();
	m_myExecutive.Stop ();
}

/*-----------------------------------------------------------------
If could not start reading the file display a message and release the
semaphore
-----------------------------------------------------------------*/
void MyApp :: OnConnect (RESPONSE result)
{
	if (result != RESPONSE_OK)
	{	printf ("Cannot open %s\n", m_szFileName);
		m_smpDemuxWorking.V ();
	}
}

/*-----------------------------------------------------------------
Release the semaphore when finished
-----------------------------------------------------------------*/
void MyApp :: OnFinish (RESPONSE)
{
	printf ("Finished\x7\x7\n");
	m_smpDemuxWorking.V ();
}

/*-----------------------------------------------------------------
Report an error
-----------------------------------------------------------------*/
void MyApp :: OnError (NON_FATAL_ERROR nError, va_list args)
{
	static LPCSTR szError [] =
	{
		"INVALID STREAM_TYPE",
		"STREAM_TYPE COLLISION",
		"NO BIFS STREAM",
		"SCENE PARSING ERROR",
		"OD PARSING ERROR",
		"OD UPDATE NOT SUPPORTED",
		"ODID COLLISION",
		"UNKNOWN ODID",
		"UPDATE ESD NOT SUPPORTED",
		"REMOVE ESD NOT SUPPORTED",
		"UNKNOWN DECODER TYPE",
		"INCOMPATIBLE DECODER TYPE",
		"NO STREAM",
		"NO STREAM_TYPE",
		"REMOTE OD REQUIRES URL",
		"UNKNOWN URL PROTOCOL",
		"BUFFER TOO SMALL",
		"DECODER SETUP FAILURE",
		"REMOTE OD REQUIRES NEW SESSION",
		"REMOTE OD FLAG REQUIRED",
		"CHANNEL ADD FAILED",
		"CHANNEL DELETE FAILED",
		"DATA TRANSMISSION ERROR",
		"INVALID URL: %s",
		"IPMP NOT AVAILABLE",
		"NO ACCESS TO PROTECTED STREAM",
		"MISSING DESCRIPTOR",
		"TOO MANY DESCRIPTORS",
		"INITIAL OBJECT DESCRIPTOR NOT FOUND",
		"NO SLCONFIG INFO",
		"ERROR IN PARSING INITIAL OBJECT DESCRIPTOR",
		"BIFS UPDATE REQUEST DOES NOT MATCH SCENE",
		"MULTIPLE ACCESS OF A SINGLE STREAM NOT PERMITTED",
		"IPMP STREAM ALREADY IN USE",
		"INVALID BIFS CONFIG",
		"CANNOT ALTER REMOTE OD",
		"TRYING TO UPDATE UNKNOWN OD",
		"TRYING TO REMOVE UNKNOWN ESD",
		"OCI PARSING ERROR",
		"SERVICE DETACH FAILED",
		"SEND COMMAND FAILED",
		"UNEXPECTED SERVICE DETACH",
		"UNEXPECTED CHANNEL DELETE",
		"DMIF FILTER NOT FOUND",
		"URL MUST REFER TO A SINGLE OD: %s",
		"",
		"",
		"ONLY ODid URL IS ALLOWED: %s",
		"IPMP SYSTEM COULD NOT PROCESS OBJECT DESCRIPTOR NO. %d",
		"IPMP SYSTEM COULD NOT REMOVE OBJECT DESCRIPTOR NO. %d",
		"IPMP SYSTEM COULD NOT PROCESS TOOL DESCRIPTOR NO. %d",
		"IPMP SYSTEM COULD NOT DELETE TOOL DESCRIPTOR NO. %d"
	};

	vprintf (szError [nError - 1], args);
	printf ("\n");
}
