 /****************************************************************
 This software module is an implementation of part of one or more 
 MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 
 Systems (ISO/IEC 14496-1). ISO/IEC gives users of the MPEG-4 
 Systems (ISO/IEC 14496-1) free license to this software module or
 modifications thereof for use in hardware or software products 
 claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
 Those intending to use this software module in hardware or software 
 products are advised that its use may infringe existing patents. 
 THE ORIGINAL DEVELOPER OF THIS SOFTWARE MODULE AND HIS/HER COMPANY,
 THE SUBSEQUENT EDITORS AND THEIR COMPANIES, AND ISO/IEC HAVE NO 
 LIABILITY FOR USE OF THIS SOFTWARE MODULE OR MODIFICATIONS THEREOF. 
*******************************************************************/
/*-------------------------------------------------------------------------
MPEGJ 
AUTHORS:    Viswanathan Swaminathan - vishy@eng.sun.com 
			Robert Schmidt(RLS)
			Praveen Alavilli(PA) 
			YUNTAI KYONG(YT) - yuntaikyong@ieee.org
-------------------------------------------------------------------------*/

// SOLARIS PORTING
#ifdef WIN32
//#include <windows.h>
#endif

#include "MPDecoderIM1Imp.h"
#include "MPDecoderInterface.h"
#include <jni.h>
#include "Decoder.h"
#include "Root.h"
#include "StreamCo.h"
#include "AL.h"
#include "ODImp.h"
#include "DecoderImp.h"
#include "Inline.h"
#include "Executiv.h"

Executive* MPDecoder::myExecutive = 0;
ZArray<decoderAssociation>* MPDecoder::s_decoderAssociation = new ZArray<decoderAssociation>;
ZSemaphore MPDecoder::zLock;

// native side will call this function

MPJErr MPDecoder::AttachDecoder(int odID, int esID) 
{
	//*** YT START (132)
	// AttachDecoder Implementation
	// 1. get esdescriptor from odID and esID
	// 2. check decoder is already detached
	// two cases:
	// a. decoder is detached from this esID
	// b. decoder is detached from another esID from same od

	// for case a:
	// the refrerence to the detached decoder is saved in the s_decoderAssociation
	// for case b:
	// search decoder detached from another esID
	// change input and output buffer of hte decoder
	// change decoder of ObjectDescriptorImp
	// set input buffer of Node to the output buffer of the decoder
	//  --> changing stream rendered

	// attach decoder to elementary stream descriptor
	// and delete from s_decoderAssociation
	
	ES_DescriptorImp* esi = GetESDescriptor(odID,esID);
	
	if(!esi)
		// no such a elemenatry stream
		return -1;
	
	DecoderImp* decoder = NULL;
	if((decoder = (DecoderImp*)esi->GetDecoder()))
		// you need to detach first
		return -1;
	zLock.P();
	int i = s_decoderAssociation->Find(esID);
	
	// find decoder
	if(i>=0) {
		decoder = (*s_decoderAssociation)[i].m_decoder;
	}
	else {
		for(int i=s_decoderAssociation->GetLength()-1;i >= 0 ;i--) {
			decoder = (*s_decoderAssociation)[i].m_decoder;
			if(decoder)
				break;
		}
		if(i == -1) {
			zLock.V();
			return -1;
		}
		else {
			decoder->SetInputStream(0,(MediaStream *)esi->m_pInputStream);
			decoder->SetOutputStream(0,(MediaStream *)esi->m_pOutputStream);
			ObjectDescriptorImp* od = GetObjectDescriptor(odID);
			assert(od);
			od->SetDecoder(decoder);
			StreamConsumer* pNode = GetStreamConsumer(odID);
			assert(pNode);
			myExecutive->LockScene();
			pNode->SetMediaStream(esi->m_pOutputStream);
			myExecutive->UnlockScene();
		}
	}
	// delete from association table
	s_decoderAssociation->DeleteFrom(i);
	// attach it!
	esi->AttachDecoder(decoder);
	zLock.V();
	//*** YT END (132)
	return 1;
}


MPJErr MPDecoder::DetachDecoder (int odID, int esID)
{
	//*** YT START (133)
	/*
	DetachDecoder Implementation
	1. Find ES_DescriptorImp from odID and esID
	2. detach decoder
	3. save the refrenece to the detached decoder in the decoderAssociation
	*/
	ES_DescriptorImp* esi = GetESDescriptor(odID,esID);
	if(!esi) 
		return -1;
	zLock.P();
	DecoderImp *decoder = esi->DetachDecoder();
	
	// remember decoder object
	if(decoder) {
		decoderAssociation da;
		da.m_decoder = decoder;
		da.m_odID = odID;
		da.m_esID = esID;
		(*s_decoderAssociation) += da;
		zLock.V();
		return 1;
	}
	else {
		zLock.V();
		return -1;
	}
	//*** YT END (133)
}

//*** YT START (134)
// utility function
// return StreamConsumer from OD_ID
StreamConsumer* 
MPDecoder::GetStreamConsumer(int odID) 
{
	InlineImp* pInline = myExecutive->GetInline();
	// support only streams associated with root inline!
	int len = pInline->m_streamAssociation.GetLength();
	for ( int i=0; i< len; i++ ) {
		ObjectDescriptorImp* odImp = pInline->m_streamAssociation[i].m_pOD;
		if( odImp->ObjectDescriptorID == odID ) {			
			return pInline->m_streamAssociation[i].m_pNode;
		}
	}
	return NULL;
}
//*** YT END (134)

//*** YT START (135)
// utility function
// return ObjectDescriptorImp from OD_ID
ObjectDescriptorImp* 
MPDecoder::GetObjectDescriptor(int odID) 
{
	InlineImp* pInline = myExecutive->GetInline();
	// support only streams associated with root inline!
	int len = pInline->m_streamAssociation.GetLength();
	for ( int i=0; i< len; i++ ) {
		ObjectDescriptorImp* odImp = pInline->m_streamAssociation[i].m_pOD;
		if( odImp->ObjectDescriptorID == odID ) {			
			return odImp;
		}
	}
	return NULL;
}
//*** YT END (135)

//*** YT START (136)
// utility function
// return ES_DescriptorImp from OD_ID and ES_ID
ES_DescriptorImp* 
MPDecoder::GetESDescriptor(int odID,int esID) 
{
	InlineImp* pInline = myExecutive->GetInline();
	// support only streams associated with root inline!
	int len = pInline->m_streamAssociation.GetLength();
	for ( int i=0; i< len; i++ ) {
		ObjectDescriptorImp* odImp = pInline->m_streamAssociation[i].m_pOD;
		if( odImp->ObjectDescriptorID == odID ) {			
			for ( int j = 0; j < odImp->esDescr.GetLength(); j++ ) {
				ES_DescriptorImp* esImp = (ES_DescriptorImp*)odImp->esDescr[j];
				if ( esImp->ES_ID == esID ) {
					return esImp;
				}
			}
		}
	}
	return NULL;
}
//*** YT END (136)
int MPDecoder::GetDecoderInstance ()
{
	return 0;
}

int MPDecoder::GetDecoderType (int strType)
{	
	return -1;
}

int MPDecoder::GetDecoderTypeNat (long od_id, long esid)
{
	// check whether the given decoder exists or not - PA	
	InlineImp* pInline;
	
//	#ifdef MPGJ
	pInline = myExecutive->GetInline();
	int len = pInline->m_streamAssociation.GetLength();
	for ( int i=0; i< len; i++ ) {
		ObjectDescriptorImp* odImp = pInline->m_streamAssociation[i].m_pOD;
		//int nodeId = pInline->m_streamAssociation[i].m_pNode->GetID();				
		if( odImp->ObjectDescriptorID == od_id ) {			
			//*** YT START (136)
			DecoderImp* od_dec = (DecoderImp*)odImp->GetDecoder();
			//*** YT END (136)
			for ( int j = 0; j < odImp->esDescr.GetLength(); j++ ) {
				ES_DescriptorImp* esImp = (ES_DescriptorImp*)odImp->esDescr[j];
				if ( esImp->ES_ID == esid ) {
					DecoderImp* dec = (DecoderImp *)esImp->GetDecoder();	
					//*** YT START (136)
					if(dec == NULL)
						dec = od_dec;
					//*** YT END (136)
					return dec->decoderType;
				}
			}
		}
	}
//	#endif

	return -1;
}

char * MPDecoder::GetDecoderVendor ()
{
	return NULL;
}

MPJErr MPDecoder::StartDecoding (long ODid)
{	
	InlineImp* pInline;
//	#ifdef MPGJ
	pInline = myExecutive->GetInline();
	int len = pInline->m_streamAssociation.GetLength();
	int isSet = 0;
	for ( int i=0; i< len; i++ ) {
		ObjectDescriptorImp* odImp = pInline->m_streamAssociation[i].m_pOD;
		if ( odImp->ObjectDescriptorID == ODid ) {
			DecoderImp * dec = (DecoderImp*) odImp->GetDecoder();				
			dec->stop = 0;			
			return 1;
			//dec->Stop();						
		}		
	} 
//	#endif
	return -1;
}

MPJErr MPDecoder::StopDecoderOnNode (long odID)
{
	InlineImp* pInline;
//	#ifdef MPGJ
	pInline = myExecutive->GetInline();
	int len = pInline->m_streamAssociation.GetLength();
	int isSet = 0;
	for ( int i=0; i< len; i++ ) {
		ObjectDescriptorImp* odImp = pInline->m_streamAssociation[i].m_pOD;
		if ( odImp->ObjectDescriptorID  == odID ) {
			DecoderImp * dec = (DecoderImp*) odImp->GetDecoder();				
			dec->stop = 1;			
			return 1;
			//dec->Stop();			
			isSet = 1;
		}
		if( isSet == 1) 
			break;
	} 

//	#endif
	return -1;		
			
}

MPJErr MPDecoder::PauseDecoding ()
{
	return 0;
}

MPJErr MPDecoder::ResumeDecoding ()
{
	return 0;
}

int MPDecoder::IsPauseable ()
{
	return 0;
}


bool MPDecoder::initialize( Executive *e )
{
	bool success = false;

// SOLARIS PORTING
#ifdef WIN32
	HINSTANCE theDLLInstance = LoadLibrary("MPDecoder");
#else
	void * theDLLInstance = dlopen("libmpdecoder.so", RTLD_LAZY);
#endif
	if ( theDLLInstance )
	{
		typedef void (*setTableFunc)(MPJMPDecoderDispatchTablePtr);

// SOLARIS PORTING
#ifdef WIN32
		setTableFunc setTable = (setTableFunc) GetProcAddress( 
theDLLInstance, "MPJSetMPDecoderDispatchTable");
#else
		setTableFunc setTable = (setTableFunc) dlsym( 
theDLLInstance, "MPJSetMPDecoderDispatchTable");
#endif

		myExecutive = e;

		if ( setTable )
		{
			MPJMPDecoderDispatchTablePtr dispatch = 
				new MPJMPDecoderDispatchTable;
			dispatch->startDecoding = StartDecoding;
			dispatch->stopDecoderOnNode = StopDecoderOnNode;
			dispatch->attachDecoder = AttachDecoder;
			dispatch->detachDecoder = DetachDecoder;
			dispatch->getDecoderTypeNat = GetDecoderTypeNat;
			setTable( dispatch );
			success = true;			
		}
	}
	return success;
}
