/*
This software module was originally developed by Five Bats Research
in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Five Bats Research retains full right
to use the code for its purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
*/

 #include "defs.h"

#include "MPJScene.h"
#include "MPJSceneInterface.h"
#include "MPJNode.h"
#pragma warning(disable: 4213)  // cast on l-value
#include "Executiv.h"
#include "Root.h"


static void freeFieldValueContent( FieldValuePtr f )
{
	if ( (f->fieldValueCount == 1) )
	{
		if ( (f->fieldValueType == MPJ_SFSTRING_TYPE) )
		{
			::free( f->val.stringVal );
		}
		return;
	}
	for( int i = 0; i < f->fieldValueCount; i++ )
	{
		FieldValue item = f->val.mfValue[i];
		if ( (f->fieldValueType == MPJ_SFSTRING_TYPE) )
		{
			::free( item.val.stringVal );
		}
	}
	::free( f->val.mfValue );
}

static MPJErr allocFieldValueContent( FieldValuePtr f, int itemCount, int isMF )
{
	if ( (f == 0) || (itemCount < 0 ) )
	{
		return MPJBadParamErr;
	}
	f->fieldValueCount = itemCount;
	f->isMFField = isMF;
	if ( isMF )
	{
		f->val.mfValue = (FieldValuePtr) ::calloc( itemCount, sizeof(FieldValue) );
		if ( f->val.mfValue == 0 )
		{
			return MPJMemoryErr;
		}
	}
	return MPJNoErr;
}

MPJNode* MPJNode::getNode( int nodeHandle )
{
	// !!! should verify that the node is still valid
	return (MPJNode*) nodeHandle;
}

MPJNode::MPJNode( int nodeID, MediaObject *node, MPJScene *scene  )
: m_nodeID(nodeID),m_mediaObject(node),m_scene(scene)
{
}

MPJNode::~MPJNode()
{
	// TODO: gracefully kill off the listeners
}

int MPJNode::sendEventIn( int eventInID, FieldValuePtr v )
{
	NodeField *f = m_mediaObject->GetEventIn( eventInID );
	if ( f == 0 )
	{
		return MPJBadParamErr;
	}

	return sendEventIn( f, v );
}

/*
	TODO: compress repeated code into macro or template function.
	See also the readEventIn which needs similar attention.
*/
int MPJNode::sendEventIn( NodeField *f, FieldValuePtr v )
{
	MPJErr err = MPJBadParamErr;
	int fieldType = f->GetType();
	switch ( fieldType )
	{
		case SFBOOL:
			if ( (v->fieldValueType == MPJ_SFBOOL_TYPE) && (v->fieldValueCount == 1) )
			{
				*(SF<bool> *) f = v->val.boolVal;
				err = MPJNoErr;
			}
			break;

		case MFBOOL:
			if ( (v->fieldValueType == MPJ_SFBOOL_TYPE) && (v->fieldValueCount >= 1) )
			{
				MF<bool> *fld = (MF<bool>*) f;
				fld->Remove();
				for ( int i = 0; i < v->fieldValueCount; i++ )
				{
					*fld += v->val.mfValue[i].val.boolVal;
				}
				err = MPJNoErr;
			}
			break;

		case SFTIME:
			if ( (v->fieldValueType == MPJ_SFTIME_TYPE) && (v->fieldValueCount == 1) )
			{
				*(SF<Time> *) f = v->val.timeVal;
				err = MPJNoErr;
			}
			break;

		case MFTIME:
			if ( (v->fieldValueType == MPJ_SFTIME_TYPE) && (v->fieldValueCount >= 1) )
			{
				MF<Time> *fld = (MF<Time>*) f;
				fld->Remove();
				for ( int i = 0; i < v->fieldValueCount; i++ )
				{
					*fld += v->val.mfValue[i].val.timeVal;
				}
				err = MPJNoErr;
			}
			break;

		case SFCOLOR:
			if ( (v->fieldValueType == MPJ_SFCOLOR_TYPE) && (v->fieldValueCount == 1) )
			{
				*(SF<Color3f> *) f = Color3f( v->val.colorVal[0], v->val.colorVal[1], v->val.colorVal[2] );
				err = MPJNoErr;
			}
			break;

		case MFCOLOR:
			if ( (v->fieldValueType == MPJ_SFCOLOR_TYPE) && (v->fieldValueCount >= 1) )
			{
				MF<Color3f> *fld = (MF<Color3f>*) f;
				fld->Remove();
				for ( int i = 0; i < v->fieldValueCount; i++ )
				{
					*fld += Color3f( v->val.mfValue[i].val.colorVal[0], 
									v->val.mfValue[i].val.colorVal[1],
									v->val.mfValue[i].val.colorVal[2] );
				}
				err = MPJNoErr;
			}
			break;

		case SFFLOAT:     
			if ( (v->fieldValueType == MPJ_SFFLOAT_TYPE) && (v->fieldValueCount == 1) )
			{
				*(SF<float> *) f = v->val.floatVal;
				err = MPJNoErr;
			}
			break;

		case MFFLOAT:
			if ( (v->fieldValueType == MPJ_SFFLOAT_TYPE) && (v->fieldValueCount >= 1) )
			{
				MF<float> *fld = (MF<float>*) f;
				fld->Remove();
				for ( int i = 0; i < v->fieldValueCount; i++ )
				{
					*fld += v->val.mfValue[i].val.floatVal;
				}
				err = MPJNoErr;
			}
			break;

		case SFINT32:
			if ( (v->fieldValueType == MPJ_SFINT32_TYPE) && (v->fieldValueCount == 1) )
			{
				*(SF<int> *) f = v->val.int32Val;
				err = MPJNoErr;
			}
			break;

		case MFINT32:
			if ( (v->fieldValueType == MPJ_SFINT32_TYPE) && (v->fieldValueCount >= 1) )
			{
				MF<int> *fld = (MF<int>*) f;
				fld->Remove();
				for ( int i = 0; i < v->fieldValueCount; i++ )
				{
					*fld += v->val.mfValue[i].val.int32Val;
				}
				err = MPJNoErr;
			}
			break;

		case SFROTATION:
			if ( (v->fieldValueType == MPJ_SFROTATION_TYPE) && (v->fieldValueCount == 1) )
			{
				*(SF<Rotation> *) f = Rotation( v->val.rotationVal[0],
												v->val.rotationVal[1],
												v->val.rotationVal[2],
//Son Tran-Artemis
												v->val.rotationVal[3] );
				err = MPJNoErr;
			}
			break;

		case MFROTATION:
			if ( (v->fieldValueType == MPJ_SFROTATION_TYPE) && (v->fieldValueCount >= 1) )
			{
				MF<Rotation> *fld = (MF<Rotation>*) f;
				fld->Remove();
				for ( int i = 0; i < v->fieldValueCount; i++ )
				{
					*fld += Rotation( v->val.mfValue[i].val.rotationVal[0],
									v->val.mfValue[i].val.mfValue[i].val.rotationVal[1],
									v->val.mfValue[i].val.rotationVal[2],
									v->val.mfValue[i].val.rotationVal[2] );
				}
				err = MPJNoErr;
			}
			break;

		case SFSTRING:
			if ( (v->fieldValueType == MPJ_SFSTRING_TYPE) && (v->fieldValueCount == 1) )
			{
				*(SF<ZString> *) f = ZString( v->val.stringVal );
				err = MPJNoErr;
			}
			break;

		case MFSTRING:
			if ( (v->fieldValueType == MPJ_SFSTRING_TYPE) && (v->fieldValueCount >= 1) )
			{
				MF<ZString> *fld = (MF<ZString>*) f;
				fld->Remove();
				for ( int i = 0; i < v->fieldValueCount; i++ )
				{
					*fld += ZString( v->val.mfValue[i].val.stringVal );
				}
				// Update the update time
				m_mediaObject->SetUpdateTime();
				err = MPJNoErr;
			}
			break;

		case SFVEC2F:
			if ( (v->fieldValueType == MPJ_SFVEC2F_TYPE) && (v->fieldValueCount == 1) )
			{
				*(SF<Vec2f> *) f = Vec2f( v->val.vec2fVal[0], v->val.vec2fVal[1] );
				err = MPJNoErr;
			}
			break;

		case MFVEC2F:
			if ( (v->fieldValueType == MPJ_SFVEC2F_TYPE) && (v->fieldValueCount >= 1) )
			{
				MF<Vec2f> *fld = (MF<Vec2f>*) f;
				fld->Remove();
				for ( int i = 0; i < v->fieldValueCount; i++ )
				{
					*fld += Vec2f( v->val.mfValue[i].val.vec2fVal[0],
								   v->val.mfValue[i].val.vec2fVal[1] );
				}
				err = MPJNoErr;
			}
			break;

		case SFVEC3F:
			if ( (v->fieldValueType == MPJ_SFVEC3F_TYPE) && (v->fieldValueCount == 1) )
			{
				*(SF<Vec3f> *) f = Vec3f(	v->val.vec3fVal[0],
											v->val.vec3fVal[1], 
											v->val.vec3fVal[2] );
				err = MPJNoErr;
			}
			break;

		case MFVEC3F:
			if ( (v->fieldValueType == MPJ_SFVEC3F_TYPE) && (v->fieldValueCount >= 1) )
			{
				MF<Vec3f> *fld = (MF<Vec3f>*) f;
				fld->Remove();
				for ( int i = 0; i < v->fieldValueCount; i++ )
				{
					*fld += Vec3f( v->val.mfValue[i].val.vec3fVal[0],
								   v->val.mfValue[i].val.vec3fVal[1],
								   v->val.mfValue[i].val.vec3fVal[2] );
				}
				err = MPJNoErr;
			}
			break;

		case SFURL:
			if ( (v->fieldValueType == MPJ_SFINT32_TYPE) && (v->fieldValueCount == 1) )
			{
				*(SF<Url> *) f = SF<Url>( Url(v->val.int32Val) );
				err = MPJNoErr;
			}
			break;

		case MFURL:
			if ( (v->fieldValueType == MPJ_SFINT32_TYPE) && (v->fieldValueCount >= 1) )
			{
				MF<Url> *fld = (MF<Url>*) f;
				fld->Remove();
				for ( int i = 0; i < v->fieldValueCount; i++ )
				{
					*fld += SF<Url>( Url(v->val.mfValue[i].val.int32Val) );
				}
				err = MPJNoErr;
			}
			break;

		case SFNODE:
			if ( (v->fieldValueType == MPJ_SFNODE_TYPE) && (v->fieldValueCount == 1) )
			{
				SFGenericNode *fld = (SFGenericNode *) f;

				int kind = v->val.nodeVal.kind;

				if ( kind == MPJ_NATIVE_NODE )
				{
					MPJNode *mpjnode = MPJNode::getNode( v->val.nodeVal.node.nativeNode.handle );
					if ( mpjnode == 0 ) // mpjnode is null, which means we must clear the field value
					{
						fld->SetObject( NULL, NULL, false );
					}
					else if ( fld->GetNode() != mpjnode->m_mediaObject ) // don't replace node with itself!
					{
						fld->SetObject( mpjnode->m_mediaObject, m_mediaObject, false );
					}
					err = MPJNoErr;
				}
				else if ( kind == MPJ_NEW_NODE )
				{
					MediaObject *newNode = SFWorldNode().CreateNode( v->val.nodeVal.node.newNode.type );
					if ( newNode == 0 )
					{
						return MPJBadParamErr; // oops
					}

					newNode->SetExecutive( m_mediaObject->GetExecutive() );
					newNode->SetID( v->val.nodeVal.node.newNode.id );
					newNode->SetRoot( m_mediaObject->GetRoot() );

					MPJNode newMpjNode( v->val.nodeVal.node.newNode.id, newNode, m_scene );

					err = MPJNoErr;
					for ( int i = 0; i < newNode->GetNumFields() && err == MPJNoErr; i++ )
					{
						NodeField *childFld = newNode->GetField(i);
						FieldValue childFldValue;
						allocFieldValueContent( &childFldValue, 1, false );
						err = m_scene->doGetNewFieldValue(
										v->val.nodeVal.node.newNode.param1,
										v->val.nodeVal.node.newNode.param2,
										i,
										&childFldValue );
						if ( err == MPJNoErr && childFldValue.fieldValueType != MPJ_UNINITIALIZED_FIELDVALUE )
						{
							newMpjNode.sendEventIn( childFld, &childFldValue );
						}
						freeFieldValueContent( &childFldValue );
					}

					if ( err == MPJNoErr )
					{
						fld->SetObject( newNode, m_mediaObject, true ); // bugcheck: really true here???
					}
					else
					{
						// bug: delete node on failure
					}
				}
				else
				{
					break; // oops
				}
			}
			break;

		case MFNODE:
			if ( v->fieldValueType == MPJ_SFNODE_TYPE )
			{
				// bug: implement MFNode
			}
			break;

		case SFIMAGE:
		case MFIMAGE:
		case UNKNOWN_TYPE:
		default:
			// err remains set to MPJBadParamErr
			break;
	}
	return err;
}

int MPJNode::startListenEventOut( int eventOutID )
{
	MPJErr err = MPJBadParamErr;
	NodeField *f = m_mediaObject->GetEventOut( eventOutID );
	if ( f == 0 )
	{
		return MPJBadParamErr;
	}
	if ( !m_listeners.Exists(eventOutID) )
	{
		MPJEventListener *l = new MPJEventListener( this, eventOutID );
		f->AddListener( l );
		m_listeners.InsertAt( eventOutID, l );
	}
	return err;
}

int MPJNode::stopListenEventOut( int eventOutID )
{
	MPJErr err = MPJBadParamErr;
	NodeField *f = m_mediaObject->GetField( eventOutID );
	if ( f == 0 )
	{
		return MPJBadParamErr;
	}
	return err;
}

int MPJNode::getNodeType( int *outNodeType )
{
	int nodeCode = m_mediaObject->GetNodeCode();

	// add one because nodeTypes start at one and the returned value from GetNodeType starts at zero
	*outNodeType = 1 + SFWorldNode().GetNodeType( nodeCode );

	return MPJNoErr;
}

void MPJNode::doEventNotification( NodeField* /* f */, int eventOutID )
{
	// get FieldValue
	FieldValue v;
	v.fieldValueCount = 1;
	v.fieldValueType = MPJ_UNINITIALIZED_FIELDVALUE;
	// TODO: refactor -- we don't need all of this since we already have f
	int err = readEventOut( eventOutID, &v );
	if ( err == MPJNoErr )
	{
		// send notification
		err = m_scene->doEventNotification( (int) this, eventOutID, &v );
	}
	freeFieldValueContent( &v );
}

int MPJNode::readEventOut( int eventOutID, FieldValuePtr outFieldValue )
{
	MPJErr err = MPJBadParamErr;
	NodeField *f = m_mediaObject->GetEventOut( eventOutID );
	if ( (f == 0) || (outFieldValue == 0) )
	{
		return MPJBadParamErr;
	}
	int fieldType = f->GetType();
	freeFieldValueContent( outFieldValue );
	int fieldCount;
	switch ( fieldType )
	{
		
		case SFBOOL:
			{
				allocFieldValueContent( outFieldValue, 1, 0 );
				outFieldValue->fieldValueType = MPJ_SFBOOL_TYPE;
				outFieldValue->val.boolVal = ((SF<Bool>*) f)->GetValue();
				err = MPJNoErr;
			}
			break;
		
		case MFBOOL:
			{
				MF<Bool> m = *((MF<Bool>*) f);
				fieldCount = m.GetLength();
				allocFieldValueContent( outFieldValue, fieldCount, 1 );
				outFieldValue->fieldValueType = MPJ_SFBOOL_TYPE;
				for ( int i = 0; i < fieldCount; i++ )
				{
					outFieldValue->val.mfValue[i].fieldValueCount = 1;
					outFieldValue->val.mfValue[i].fieldValueType = outFieldValue->fieldValueType;
					outFieldValue->val.mfValue[i].val.boolVal = m[i];
				}
				err = MPJNoErr;
			}
			break;
			

		case SFTIME:
			{
				allocFieldValueContent( outFieldValue, 1, 0 );
				outFieldValue->fieldValueType = MPJ_SFTIME_TYPE;
				outFieldValue->val.timeVal = ((SF<Time>*) f)->GetValue();
				err = MPJNoErr;
			}
			break;

		case MFTIME:
			{
				MF<Time> m = *((MF<Time>*) f);
				fieldCount = m.GetLength();
				allocFieldValueContent( outFieldValue, fieldCount, 1 );
				outFieldValue->fieldValueType = MPJ_SFTIME_TYPE;
				for ( int i = 0; i < fieldCount; i++ )
				{
					outFieldValue->val.mfValue[i].fieldValueCount = 1;
					outFieldValue->val.mfValue[i].fieldValueType = outFieldValue->fieldValueType;
					outFieldValue->val.mfValue[i].val.timeVal = m[i];
				}
				err = MPJNoErr;
			}
			break;

		case SFCOLOR:
			{
				allocFieldValueContent( outFieldValue, 1, 0 );
				outFieldValue->fieldValueType = MPJ_SFCOLOR_TYPE;
				Color3f v = ((SF<Color3f>*) f)->GetValue();
				outFieldValue->val.colorVal[0] = v.r;
				outFieldValue->val.colorVal[1] = v.g;
				outFieldValue->val.colorVal[2] = v.b;
				err = MPJNoErr;
			}
			break;


		case MFCOLOR:
			{
				MF<Color3f> m = *((MF<Color3f>*) f);
				fieldCount = m.GetLength();
				allocFieldValueContent( outFieldValue, fieldCount, 1 );
				outFieldValue->fieldValueType = MPJ_SFCOLOR_TYPE;
				for ( int i = 0; i < fieldCount; i++ )
				{
					Color3f v = m[i];
					outFieldValue->val.mfValue[i].fieldValueCount = 1;
					outFieldValue->val.mfValue[i].fieldValueType = outFieldValue->fieldValueType;
					outFieldValue->val.mfValue[i].val.colorVal[0] = v.r;
					outFieldValue->val.mfValue[i].val.colorVal[1] = v.g;
					outFieldValue->val.mfValue[i].val.colorVal[2] = v.b;
				}
				err = MPJNoErr;
			}
			break;

		case SFFLOAT:	
			{
				allocFieldValueContent( outFieldValue, 1, 0 );
				outFieldValue->fieldValueType = MPJ_SFFLOAT_TYPE;
				outFieldValue->val.floatVal = ((SF<float>*) f)->GetValue();
				err = MPJNoErr;
			}
			break;

		case MFFLOAT:
			{
				MF<float> m = *((MF<float>*) f);
				fieldCount = m.GetLength();
				allocFieldValueContent( outFieldValue, fieldCount, 1 );
				outFieldValue->fieldValueType = MPJ_SFFLOAT_TYPE;
				for ( int i = 0; i < fieldCount; i++ )
				{
					outFieldValue->val.mfValue[i].fieldValueCount = 1;
					outFieldValue->val.mfValue[i].fieldValueType = outFieldValue->fieldValueType;
					outFieldValue->val.mfValue[i].val.floatVal = m[i];
				}
				err = MPJNoErr;
			}
			break;

		case SFINT32:	
			{
				allocFieldValueContent( outFieldValue, 1, 0 );
				outFieldValue->fieldValueType = MPJ_SFINT32_TYPE;
				outFieldValue->val.int32Val = ((SF<int>*) f)->GetValue();
				err = MPJNoErr;
			}
			break;

		case MFINT32:
			{
				MF<int> m = *((MF<int>*) f);
				fieldCount = m.GetLength();
				allocFieldValueContent( outFieldValue, fieldCount, 1 );
				outFieldValue->fieldValueType = MPJ_SFINT32_TYPE;
				for ( int i = 0; i < fieldCount; i++ )
				{
					outFieldValue->val.mfValue[i].fieldValueCount = 1;
					outFieldValue->val.mfValue[i].fieldValueType = outFieldValue->fieldValueType;
					outFieldValue->val.mfValue[i].val.int32Val = m[i];
				}
				err = MPJNoErr;
			}
			break;

		case SFROTATION:
			{
				allocFieldValueContent( outFieldValue, 1, 0 );
				outFieldValue->fieldValueType = MPJ_SFROTATION_TYPE;
				Rotation v = ((SF<Rotation>*) f)->GetValue();
				outFieldValue->val.rotationVal[0] = v.x;
				outFieldValue->val.rotationVal[1] = v.y;
				outFieldValue->val.rotationVal[2] = v.z;
				outFieldValue->val.rotationVal[3] = v.theta;
				err = MPJNoErr;
			}
			break;


		case MFROTATION:
			{
				MF<Rotation> m = *((MF<Rotation>*) f);
				fieldCount = m.GetLength();
				allocFieldValueContent( outFieldValue, fieldCount, 1 );
				outFieldValue->fieldValueType = MPJ_SFROTATION_TYPE;
				for ( int i = 0; i < fieldCount; i++ )
				{
					Rotation v = m[i];
					outFieldValue->val.mfValue[i].fieldValueCount = 1;
					outFieldValue->val.mfValue[i].fieldValueType = outFieldValue->fieldValueType;
					outFieldValue->val.mfValue[i].val.rotationVal[0] = v.x;
					outFieldValue->val.mfValue[i].val.rotationVal[1] = v.y;
					outFieldValue->val.mfValue[i].val.rotationVal[2] = v.z;
					outFieldValue->val.mfValue[i].val.rotationVal[3] = v.theta;
				}
				err = MPJNoErr;
			}
			break;

		case SFSTRING:	
			{
				allocFieldValueContent( outFieldValue, 1, 0 );
				outFieldValue->fieldValueType = MPJ_SFSTRING_TYPE;
				ZString s = ((SF<ZString>*) f)->GetValue();
				int len = s.GetLength();
				if ( len > 0 )
				{
					outFieldValue->val.stringVal = (char*) ::calloc( 1, len + 1 );
					if ( outFieldValue->val.stringVal == 0 )
					{
						return MPJMemoryErr;
					}
					::strcpy( outFieldValue->val.stringVal, s );
				}
				err = MPJNoErr;
			}
			break;

		case MFSTRING:
			{
				MF<ZString> *m = ((MF<ZString>*) f);
				fieldCount = m->GetLength();
				allocFieldValueContent( outFieldValue, fieldCount, 1 );
				outFieldValue->fieldValueType = MPJ_SFSTRING_TYPE;
				for ( int i = 0; i < fieldCount; i++ )
				{
					const char *s = (*m)[i];
					int len = ::strlen( s );
					outFieldValue->val.mfValue[i].fieldValueCount = 1;
					outFieldValue->val.mfValue[i].fieldValueType = outFieldValue->fieldValueType;
					if ( len > 0 )
					{
						outFieldValue->val.mfValue[i].val.stringVal = (char*) ::calloc( 1, len + 1 );
						if ( outFieldValue->val.mfValue[i].val.stringVal == 0 )
						{
							return MPJMemoryErr;
						}
						::strcpy( outFieldValue->val.mfValue[i].val.stringVal, s );
					}
				}
				err = MPJNoErr;
			}
			break;

		case SFVEC2F:
			{
				allocFieldValueContent( outFieldValue, 1, 0 );
				outFieldValue->fieldValueType = MPJ_SFVEC2F_TYPE;
				Vec2f v = ((SF<Vec2f>*) f)->GetValue();
				outFieldValue->val.vec2fVal[0] = v.x;
				outFieldValue->val.vec2fVal[1] = v.y;
				err = MPJNoErr;
			}
			break;


		case MFVEC2F:
			{
				MF<Vec2f> m = *((MF<Vec2f>*) f);
				fieldCount = m.GetLength();
				allocFieldValueContent( outFieldValue, fieldCount, 1 );
				outFieldValue->fieldValueType = MPJ_SFVEC2F_TYPE;
				for ( int i = 0; i < fieldCount; i++ )
				{
					Vec2f v = m[i];
					outFieldValue->val.mfValue[i].fieldValueCount = 1;
					outFieldValue->val.mfValue[i].fieldValueType = outFieldValue->fieldValueType;
					outFieldValue->val.mfValue[i].val.vec2fVal[0] = v.x;
					outFieldValue->val.mfValue[i].val.vec2fVal[1] = v.y;
				}
				err = MPJNoErr;
			}
			break;

		case SFVEC3F:
			{
				allocFieldValueContent( outFieldValue, 1, 0 );
				outFieldValue->fieldValueType = MPJ_SFVEC3F_TYPE;
				Vec3f v = ((SF<Vec3f>*) f)->GetValue();
				outFieldValue->val.vec3fVal[0] = v.x;
				outFieldValue->val.vec3fVal[1] = v.y;
				outFieldValue->val.vec3fVal[2] = v.z;
				err = MPJNoErr;
			}
			break;


		case MFVEC3F:
			{
				MF<Vec3f> m = *((MF<Vec3f>*) f);
				fieldCount = m.GetLength();
				allocFieldValueContent( outFieldValue, fieldCount, 1 );
				outFieldValue->fieldValueType = MPJ_SFVEC3F_TYPE;
				for ( int i = 0; i < fieldCount; i++ )
				{
					Vec3f v = m[i];
					outFieldValue->val.mfValue[i].fieldValueCount = 1;
					outFieldValue->val.mfValue[i].fieldValueType = outFieldValue->fieldValueType;
					outFieldValue->val.mfValue[i].val.vec3fVal[0] = v.x;
					outFieldValue->val.mfValue[i].val.vec3fVal[1] = v.y;
					outFieldValue->val.mfValue[i].val.vec3fVal[2] = v.z;
				}
				err = MPJNoErr;
			}
			break;

		case SFURL:	
			{
				allocFieldValueContent( outFieldValue, 1, 0 );
				outFieldValue->fieldValueType = MPJ_SFINT32_TYPE;
				outFieldValue->val.int32Val = ((SF<Url>*) f)->GetValue();
				err = MPJNoErr;
			}
			break;

		case MFURL:
			{
				MF<Url> m = *((MF<Url>*) f);
				fieldCount = m.GetLength();
				allocFieldValueContent( outFieldValue, fieldCount, 1 );
				outFieldValue->fieldValueType = MPJ_SFINT32_TYPE;
				for ( int i = 0; i < fieldCount; i++ )
				{
					outFieldValue->val.mfValue[i].fieldValueCount = 1;
					outFieldValue->val.mfValue[i].fieldValueType = outFieldValue->fieldValueType;
					outFieldValue->val.mfValue[i].val.int32Val = m[i];
				}
				err = MPJNoErr;
			}
			break;

		case SFNODE:
			{
				allocFieldValueContent( outFieldValue, 1, 0 );
				outFieldValue->fieldValueType = MPJ_SFNODE_TYPE;
				MediaObject *node = ((SFGenericNode*) f)->GetNode();
				if ( node == 0 )
				{
					outFieldValue->val.nodeVal.kind = MPJ_NATIVE_NODE;
					outFieldValue->val.nodeVal.node.nativeNode.handle = 0;
					outFieldValue->val.nodeVal.node.nativeNode.id = 0;
				}
				else
				{
					int nodeID = node->GetID();
					outFieldValue->val.nodeVal.kind = MPJ_NATIVE_NODE;
					outFieldValue->val.nodeVal.node.nativeNode.handle = (int) new MPJNode( nodeID, node, m_scene );
					outFieldValue->val.nodeVal.node.nativeNode.id = nodeID;
				}
				err = MPJNoErr;
			}
			break;

		case MFNODE:
			{
				MFGenericNode *m = ((MFGenericNode*) f);
				fieldCount = m->GetLength();
				allocFieldValueContent( outFieldValue, fieldCount, 1 );
				outFieldValue->fieldValueType = MPJ_SFNODE_TYPE;
				for ( int i = 0; i < fieldCount; i++ )
				{
					outFieldValue->val.mfValue[i].fieldValueCount = 1;
					outFieldValue->val.mfValue[i].fieldValueType = outFieldValue->fieldValueType;
					MediaObject *node = (*m)[i];
					if ( node == 0 )
					{
						outFieldValue->val.mfValue[i].val.nodeVal.kind = MPJ_NATIVE_NODE;
						outFieldValue->val.mfValue[i].val.nodeVal.node.nativeNode.handle = 0;
						outFieldValue->val.mfValue[i].val.nodeVal.node.nativeNode.id = 0;
					}
					else
					{
						int nodeID = node->GetID();
						outFieldValue->val.mfValue[i].val.nodeVal.kind = MPJ_NATIVE_NODE;
						outFieldValue->val.mfValue[i].val.nodeVal.node.nativeNode.handle = (int) new MPJNode( nodeID, node, m_scene );
						outFieldValue->val.mfValue[i].val.nodeVal.node.nativeNode.id = nodeID;
					}
				}
				err = MPJNoErr;
			}
			break;

		case SFIMAGE:
		case MFIMAGE:
		case UNKNOWN_TYPE:
		default:
			// err remains set to MPJBadParamErr
			break;	

	}
	return err;
}

int MPJNode::modifyRouteTo( int outID, MPJNode *toNode, int toID, int insert )
{
	if ( insert )
	{
		m_mediaObject->InsertRouteTo( outID, toNode->m_mediaObject, toID );
	}
	else
	{
		m_mediaObject->DeleteRouteTo( outID, toNode->m_mediaObject, toID );
	}
	return 0;
}

// MPEGJ - Added a new method for getting the field type
// for a given eventInID
int MPJNode::getFieldType( int eventInID, int *outFieldType )
{
	NodeField *f = m_mediaObject->GetEventIn( eventInID );
	if ( f == 0 )
	{
		return MPJBadParamErr;
	}
	*outFieldType = f->GetType();
	int fieldType = *outFieldType;
	// This type is in Core terms.. We have to change it to 
	// MPEGJ terms
	switch ( fieldType )
	{
		case SFBOOL:
		case MFBOOL:
			*outFieldType = MPJ_SFBOOL_TYPE;
			break;

		case SFTIME:
		case MFTIME:
			*outFieldType = MPJ_SFTIME_TYPE;
			break;

		case SFCOLOR:
		case MFCOLOR:
			*outFieldType = MPJ_SFCOLOR_TYPE;
			break;

		case SFFLOAT:     
		case MFFLOAT:
			*outFieldType = MPJ_SFFLOAT_TYPE;
			break;

		case SFINT32:
		case MFINT32:
			*outFieldType = MPJ_SFINT32_TYPE;
			break;

		case SFROTATION:
		case MFROTATION:
			*outFieldType = MPJ_SFROTATION_TYPE;
			break;

		case SFSTRING:
		case MFSTRING:
			*outFieldType = MPJ_SFSTRING_TYPE;
			break;

		case SFVEC2F:
		case MFVEC2F:
			*outFieldType = MPJ_SFVEC2F_TYPE;
			break;

		case SFVEC3F:
		case MFVEC3F:
			*outFieldType = MPJ_SFVEC3F_TYPE;
			break;

		case SFURL:
		case MFURL:
			*outFieldType = MPJ_SFINT32_TYPE;
			break;

		case SFNODE:
		case MFNODE:
			*outFieldType = MPJ_SFNODE_TYPE ;
			break;

		case SFIMAGE:
		case MFIMAGE:
		case UNKNOWN_TYPE:
		default:
			// err remains set to MPJBadParamErr
			break;
	}
	return MPJNoErr;
}
