/*
This software module was originally developed by Five Bats Research
in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Five Bats Research retains full right
to use the code for its purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
*/
#include "defs.h"
#include "MPJScene.h"
#include "MPJSceneInterface.h"
#include "MPJNode.h"
#pragma warning(disable: 4213)  // cast on l-value
#include "Executiv.h"
#include "Root.h"
#include "ztl.h"



/*
	DISPATCH TABLE HANDLERS
*/
static MPJErr getSceneHandle( int *outSceneHandle )
{
	MPJErr err = MPJNoErr;
	if ( outSceneHandle == 0 )
	{
		return MPJBadParamErr;
	}
	*outSceneHandle = MPJScene::getSceneHandle();
	return err;
}

static MPJErr findNode( int sceneHandle, int nodeID, int *outNodeHandle )
{
	MPJScene *s = MPJScene::getScene( sceneHandle );
	int err = MPJUnspecifiedErr;
	if ( s == 0 )
	{
		return MPJInvalidSceneErr;
	}
	try
	{
		err = s->findNode( nodeID, outNodeHandle );
	}
	catch ( MPJException& e )
	{
		err = e.getErrorCode();
	}
	return err;
}

static MPJErr sendEventIn( int nodeHandle, int eventInID, FieldValuePtr fieldValue )
{
	MPJNode *n = MPJNode::getNode( nodeHandle );
	if ( n == 0 )
	{
		return MPJInvalidNodeErr;
	}
	return n->sendEventIn( eventInID, fieldValue );
}

// Add code to get the event field type
static MPJErr getEventType( int nodeHandle, int eventInID, int *eventType )
{
	MPJNode *n = MPJNode::getNode( nodeHandle );
	if ( n == 0 )
	{
		return MPJInvalidNodeErr;
	}
	return n->getFieldType( eventInID, eventType );
}

static MPJErr readEventOut( int nodeHandle, int eventOutID, FieldValuePtr outFieldValue )
{
	int err = MPJUnspecifiedErr;
	try
	{
		MPJNode *n = MPJNode::getNode( nodeHandle );
		if ( n == 0 )
		{
			return MPJInvalidNodeErr;
		}
		err = n->readEventOut( eventOutID, outFieldValue );
	}
	catch ( MPJException& e )
	{
		err = e.getErrorCode();
	}
	return err;
}

static MPJErr startListenEventOut( int nodeHandle, int eventOutID )
{
	int err = MPJUnspecifiedErr;
	try
	{
		MPJNode *n = MPJNode::getNode( nodeHandle );
		if ( n == 0 )
		{
			return MPJInvalidNodeErr;
		}
		err = n->startListenEventOut( eventOutID );
	}
	catch ( MPJException& e )
	{
		err = e.getErrorCode();
	}
	return err; 
}

static MPJErr stopListenEventOut( int nodeHandle, int eventOutID )
{
	int err = MPJUnspecifiedErr;
	try
	{
		MPJNode *n = MPJNode::getNode( nodeHandle );
		if ( n == 0 )
		{
			return MPJInvalidNodeErr;
		}
		err = n->stopListenEventOut( eventOutID );
	}
	catch ( MPJException& e )
	{
		err = e.getErrorCode();
	}
	return err;
}

static MPJErr getNodeType( int nodeHandle, int *outNodeType )
{
	int err = MPJUnspecifiedErr;
	try
	{
		MPJNode *n = MPJNode::getNode( nodeHandle );
		if ( n == 0 )
		{
			return MPJInvalidNodeErr;
		}
		err = n->getNodeType( outNodeType );
	}
	catch ( MPJException& e )
	{
		err = e.getErrorCode();
	}
	return err;
}

static MPJErr getNodeIDFromName( int sceneHandle, char *defName, int *outDefID )
{
	MPJScene *s = MPJScene::getScene( sceneHandle );
	int err = MPJUnspecifiedErr;
	if ( s == 0 )
	{
		return MPJInvalidSceneErr;
	}
	try
	{
		err = s->getNodeIDFromName( defName, outDefID );
	}
	catch ( MPJException& e )
	{
		err = e.getErrorCode();
	}
	return err;
}

static MPJErr modifyRouteTo( int fromHandle, int fromID, int toHandle, int toID, int insert )
{
	int err = MPJNoErr;
	try
	{
		MPJNode *fromNode = MPJNode::getNode( fromHandle );
		if ( fromNode == 0 )
		{
			return MPJInvalidNodeErr;
		}
		MPJNode *toNode = MPJNode::getNode( toHandle );
		if ( toNode == 0 )
		{
			return MPJInvalidNodeErr;
		}
		fromNode->modifyRouteTo( fromID, toNode, toID, insert );
	}
	catch ( MPJException& e )
	{
		err = e.getErrorCode();
	}
	return err;
}

/*
	MPJScene implementation
*/
MPJScene *MPJScene::s_instance = 0;
MPJSceneDispatchTable *MPJScene::s_dispatch = 0;

bool MPJScene::initialize( Executive *e )
{
	bool success = false;
#ifdef WIN32
	HINSTANCE theDLLInstance = LoadLibrary( "mpjscene" );
#else
	HINSTANCE theDLLInstance; // = LoadLibrary( "mpjscene" );
	static ZRegistry registry( "Software\\MPEG-4\\IM1\\MPEGJ" );
	registry.CreateInstance( "mpjscene", theDLLInstance );
#endif
	if ( theDLLInstance )
	{
		typedef void (*setTableFunc)(MPJSceneDispatchTablePtr, int);
// SOLARIS PORTING
#ifdef WIN32
		setTableFunc setTable = (setTableFunc) GetProcAddress( theDLLInstance, "MPJSetSceneDispatchTable" );
#else
		setTableFunc setTable = (setTableFunc) dlsym( theDLLInstance, "MPJSetSceneDispatchTable" );
#endif
		if ( setTable )
		{
			MPJSceneDispatchTable *dispatch = new MPJSceneDispatchTable;
			/*MPJScene *s = */new MPJScene( e, dispatch );
			setTable( dispatch, MPJ_SCENEAPI_VERSION );
			success = true;
		}
	}
	return success;
}

int MPJScene::getSceneHandle()
{
	return (int) s_instance;
}

MPJErr MPJScene::findNode( int nodeID, int *outNodeHandle )
{
	MPJErr ret = MPJNoErr;
	if ( outNodeHandle == 0 )
	{
		return MPJBadParamErr;
	}	Root *r = m_executive->GetScene();
	MediaObject *node = r->FindNode( nodeID );
	if ( node == 0 )
	{
		return MPJBadParamErr;
	}
	MPJNode *mpjnode = new MPJNode( nodeID, node, this );
	// TODO: !!! add node to vector so we can destroy it !!!
	*outNodeHandle = (int) mpjnode;
	return ret;
}

MPJErr MPJScene::getNodeIDFromName( char *defName, int *outDefID )
{
	MPJErr ret = MPJNoErr;
	if ( outDefID == 0 )
	{
		return MPJBadParamErr;
	}
	Root *r = m_executive->GetScene();
	MediaObject *node = r->FindNode( defName );
	if ( node == 0 )
	{
		return MPJBadParamErr;
	}
	*outDefID = (int) node->GetID();
	return ret;
}

MPJScene *MPJScene::getScene( int sceneHandle )
{
	if ( sceneHandle == getSceneHandle() )
	{
		return s_instance;
	}
	else
	{
		return 0;
	}
}

void MPJScene::sceneReplaced( Executive *e, bool initialScene )
{
	// TODO: kill any existing nodes
	if ( s_instance )
	{
		MPJSceneDispatchTable *t = s_instance->getDispatchTable();
		int whatHappened = initialScene ? MPJInitialSceneHappened : MPJSceneReplaceHappened;
		
		delete s_instance;
		new MPJScene( e, t );
		int sceneHandle = s_instance->getSceneHandle();
		if ( t->sceneEventNotification )
		{
			(*t->sceneEventNotification)( sceneHandle, whatHappened );
		}
	}
}

int MPJScene::doEventNotification( int nodeHandle, int eventOutID, FieldValue *val )
{
	if ( s_dispatch->eventOutNotification )
	{
		return (*s_dispatch->eventOutNotification)( nodeHandle, eventOutID, val );
	}
	return MPJUnspecifiedErr;
}

int MPJScene::doGetNewFieldValue( int param1, int param2, int fieldID, FieldValue *newFieldValue )
{
	if ( s_dispatch->getNewFieldValue )
	{
		return (*s_dispatch->getNewFieldValue)( param1, param2, fieldID, newFieldValue );
	}
	return MPJUnspecifiedErr;
}

MPJScene::MPJScene( Executive *e, MPJSceneDispatchTable *t )
: m_executive(e)
{
	s_instance = this;
	s_dispatch = t;
	s_dispatch->getSceneHandle      = ::getSceneHandle;
	s_dispatch->findNode            = ::findNode;
	s_dispatch->sendEventIn         = ::sendEventIn;
	s_dispatch->readEventOut        = ::readEventOut;
	s_dispatch->startListenEventOut = ::startListenEventOut;
	s_dispatch->stopListenEventOut  = ::stopListenEventOut;
	s_dispatch->getNodeType         = ::getNodeType;
	s_dispatch->getNodeIDFromName   = ::getNodeIDFromName;
	s_dispatch->modifyRouteTo       = ::modifyRouteTo;
	s_dispatch->getEventType        = ::getEventType;
}

MPJScene::~MPJScene()
{
	//free( s_dispatch );
}
