 /****************************************************************
 This software module is an implementation of part of one or more 
 MPEG-4 Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 
 Systems (ISO/IEC 14496-1). ISO/IEC gives users of the MPEG-4 
 Systems (ISO/IEC 14496-1) free license to this software module or
 modifications thereof for use in hardware or software products 
 claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
 Those intending to use this software module in hardware or software 
 products are advised that its use may infringe existing patents. 
 THE ORIGINAL DEVELOPER OF THIS SOFTWARE MODULE AND HIS/HER COMPANY,
 THE SUBSEQUENT EDITORS AND THEIR COMPANIES, AND ISO/IEC HAVE NO 
 LIABILITY FOR USE OF THIS SOFTWARE MODULE OR MODIFICATIONS THEREOF. 
*******************************************************************/
/*-------------------------------------------------------------------------
MPEGJ 
AUTHORS:    Viswanathan Swaminathan - vishy@eng.sun.com 
			Robert Schmidt(RLS)
			Praveen Alavilli(PA) 
			YUNTAI KYONG(YT) - yuntaikyong@ieee.org
-------------------------------------------------------------------------*/

// SOLARIS PORTING
#ifdef WIN32
//#include <windows.h>
#endif

#include "ResourceManagerIM1Imp.h"
#include "ResourceManagerInterface.h"

// header files
#include "ODImp.h"	// for OD and ESD
#include "Executiv.h"	// for Executive
#include "Inline.h"	// for InlineImp


//*** YT START (145)
// make compiler happy
InlineImp::StreamAssociation::~StreamAssociation()
{
	delete m_pOD;
}
//*** YT END (145)

//*** YT START (146)
// utility function 
// get ES_DescriptorImp from esID
ES_DescriptorImp* 
GetESDescriptor(int esID) 
{
	InlineImp* pInline = ResourceManager::s_pExecutive->GetInline();
	// support only streams associated with root inline!
	for ( int i=0; i< pInline->m_streamAssociation.GetLength(); i++ ) {
		ObjectDescriptorImp* odImp = pInline->m_streamAssociation[i].m_pOD;
		for ( int j = 0; j < odImp->esDescr.GetLength(); j++ ) {
			ES_DescriptorImp* esImp = (ES_DescriptorImp*)odImp->esDescr[j];
			if ( esImp->ES_ID == esID ) {
				return esImp;
			}
		}
	}
	return NULL;
}
// refrence to Executive
Executive* ResourceManager::s_pExecutive = NULL;
//*** YT END (146)

// constructor 
ResourceManager::ResourceManager() {
	
}

//*** YT START (147)
//change decoder from old_esid to esid
MPJErr changeDecoderImp(long odid, long esid, long old_esid)
{
	ES_DescriptorImp* to = GetESDescriptor(esid);
	ES_DescriptorImp* from = GetESDescriptor(old_esid);
	if(!to || !from) 
		return -1;
	DecoderImp* pDispose = to->DetachDecoder();
	DecoderImp* pDec = from->DetachDecoder();
	if(!pDec || !pDispose)
		return -1;
	to->AttachDecoder(pDec);
	return 1;
}
//*** YT END (147)

MPJErr checkODIDImp(long odid)
{
	InlineImp* pInline = ResourceManager::s_pExecutive->GetInline();
	if(pInline->m_streamAssociation.Find(odid) < 0) {
		// TODO:
		// should go through all sub InlineImp
		// type infomation has to be generated by compiler to find out typeinfo
		return -1;
	}
	else
		return 1;
}

MPJErr checkESIDImp(long esid)
{
	ObjectDescriptorImp* pOD;
	InlineImp* pInline = ResourceManager::s_pExecutive->GetInline();
	for(int j = pInline->m_streamAssociation.GetLength();j>0;j--) {
		pOD = pInline->m_streamAssociation[j].m_pOD;
		for(int k = pInline->m_streamAssociation.GetLength();k>0;k--) {
			if( pOD->esDescr[k]->ES_ID == esid)
				return 1;
		}
	}
	return -1;
}

MPJErr ResourceManagerCall( jint nodeID )
{
	return 0;
}

bool ResourceManager::initialize( Executive *e )
{
	bool success = false;
	if(!e)
		return false;

	s_pExecutive = e;
// SOLARIS PORTING
#ifdef WIN32
	HINSTANCE theDLLInstance = LoadLibrary("resourcemanager");
#else
	void * theDLLInstance = dlopen("libresourcemanager.so", RTLD_LAZY);
#endif
	if ( theDLLInstance )
	{
		typedef void (*setTableFunc)(MPJResourceManagerDispatchTablePtr);
		
// SOLARIS PORTING
#ifdef WIN32
		setTableFunc setTable = (setTableFunc) GetProcAddress( 
theDLLInstance, "MPJSetResourceManagerDispatchTable");
#else
		setTableFunc setTable = (setTableFunc) dlsym( 
theDLLInstance, "MPJSetResourceManagerDispatchTable");
#endif
		if ( setTable )
		{
			MPJResourceManagerDispatchTablePtr dispatch = 
				new MPJResourceManagerDispatchTable;
			// populate the function table 
			dispatch->getESID = ResourceManagerCall;
			dispatch->changeDecoder = changeDecoderImp;
			dispatch->checkODID = checkODIDImp;
			dispatch->checkESID = checkESIDImp;

			setTable( dispatch );
			success = true;
		}
	}
	return success;
}

