/*
This software module was originally developed by Five Bats Research
in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Five Bats Research retains full right
to use the code for its purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
*/
#ifndef INCLUDED_MPJNODE_H
#define INCLUDED_MPJNODE_H

#include "defs.h"
#include "ztl.h"
#include "NodeFld.h"
#include "MPJSceneInterface.h"

class MediaObject;
class MPJScene;
class MPJEventListener;

/*
	MPJNode helper class. This handles transfers of data
	between MPEG-J and IM1.
*/
class MPJNode
{
	public:
		static MPJNode* getNode( int nodeHandle );
		MPJNode( int nodeID, MediaObject *node, MPJScene *scene );
		int sendEventIn( int eventInID, FieldValuePtr fieldValue );
		int sendEventIn( NodeField *f, FieldValuePtr fieldValue );
		int readEventOut( int eventOutID, FieldValuePtr outFieldValue );
		int startListenEventOut( int eventOutID );
		int stopListenEventOut( int eventOutID );
		int getNodeType( int *outNodeType );
		void doEventNotification( NodeField *f, int eventOutID );
		int modifyRouteTo( int outID, MPJNode *toNode, int toID, int insert );
		virtual ~MPJNode();
	private:
		int m_nodeID;
		MediaObject *m_mediaObject;
		MPJScene *m_scene;
		ZMap<MPJEventListener*> m_listeners;
};

class MPJEventListener: public Listener
{
	public:
		MPJEventListener( MPJNode *node, int eventOutID ) : m_node(node),
		m_eventOutID(eventOutID) {}
		virtual void notify( NodeField *f )
		{
			m_node->doEventNotification( f, m_eventOutID );
		}

	private:
		MPJNode *m_node;
		int m_eventOutID;
};

#endif
