/*
This software module was originally developed by Five Bats Research
in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Five Bats Research retains full right
to use the code for its purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
*/
#ifndef INCLUDED_MPJSCENE_INTERFACE_H
#define INCLUDED_MPJSCENE_INTERFACE_H

/*
	This file defines the interface between JNI code (in a DLL)
	and IM1 code (in the IM1 application) that allows communication
	between IM1 and the MPEG-J scene manager.

	MPEGJDLLAPI should be defined in the JNI project, which is
	the creator of the DLL. IM1 is a client and does not define
	this.

	The interface primarily consists of a table of function pointers,
	some set by IM1 and some set by MPEG-J. The table should be created
	by IM1, and its address made known to MPEG-J by calling 
	MPJSetSceneDispatchTable after first setting the function pointers
	that are known by IM1.

	The interface is defined using C calling conventions. Neither IM1
	nor MPEG-J should assume that the other is using any language other
	than C for its implementation. Please remember this as you edit
	this file.
*/

// SOLARIS PORTING
#ifdef WIN32
#ifdef MPEGJDLLAPI
#define MPJ_EXTERN(v) extern __declspec( dllexport ) v __cdecl
#else
#define MPJ_EXTERN(v) extern __declspec( dllimport ) v __cdecl
#endif
#else

#define MPJ_EXTERN(var) extern "C" var
#include "../Include/defs.h"

#endif

#define MPJ_SCENEAPI_VERSION 12

#ifdef __cplusplus
extern "C" {
#endif

/*
	FieldValue type enumeration. Note that this is intended to
	decouple the interface from the same enumerations that
	are currently in use by IM1.
*/
enum
{
	MPJ_UNINITIALIZED_FIELDVALUE = 0,
	MPJ_SFBOOL_TYPE,
	MPJ_SFCOLOR_TYPE,
	MPJ_SFFLOAT_TYPE,
	MPJ_SFINT32_TYPE,
	MPJ_SFNODE_TYPE,
	MPJ_SFROTATION_TYPE,
	MPJ_SFSTRING_TYPE,
	MPJ_SFTIME_TYPE,
	MPJ_SFVEC2F_TYPE,
	MPJ_SFVEC3F_TYPE,
	MPJ_FIELDVALUE_TYPE_COUNT
};

enum
{
	MPJ_NATIVE_NODE,
	MPJ_NEW_NODE
};

/*
	This struct contains the values for a field of
	a BIFS node. MFFields are implemented by setting
	the type to the corresponding SF type, and setting
	the count to a number larger than one.
*/
typedef struct FieldValueStruct
{
	int fieldValueType;
	int fieldValueCount;
	int isMFField;
	union
	{
		int boolVal;
		int int32Val;
		float floatVal;
		float colorVal[3];
		float rotationVal[4];
		float vec2fVal[2];
		float vec3fVal[3];
		double timeVal;
		char *stringVal;
		struct FieldValueStruct *mfValue;
		struct {
			int kind; // whether it's an existing native node (MPJNativeNode) or a node creation (MPJNewNode)
			union {
				struct {
					int handle; // a handle to the node or 0 for a NULL node (typically and pointer to an MPJNode for native node)
					int id; // the node ID
				} nativeNode;
				struct {
					int type; // the node type
					int id; // the node ID
					int param1; // param given to getNewFieldValue callback
					int param2; // param given to getNewFieldValue callback
				} newNode;
			} node;
		} nodeVal;
	} val;
} FieldValue, *FieldValuePtr;

typedef int MPJErr;
enum
{
	MPJVersionMismatchErr = -1,
	MPJNoErr = 0,
	MPJBadParamErr,
	MPJInvalidSceneErr,
	MPJInvalidNodeErr,
	MPJInvalidFieldErr,
	MPJMemoryErr,
	MPJUnspecifiedErr
};

enum
{
	MPJInitialSceneHappened = 0,
	MPJSceneReplaceHappened
};

struct MPJSceneDispatchTable
{
	MPJErr (*getSceneHandle)( int *outSceneHandle );
	MPJErr (*findNode)( int sceneHandle, int nodeID, int *outNodeHandle );
	MPJErr (*sendEventIn)( int nodeHandle, int eventInID, FieldValuePtr fieldValue );
	MPJErr (*readEventOut)( int nodeHandle, int eventOutID, FieldValuePtr outFieldValue );
	MPJErr (*startListenEventOut)( int nodeHandle, int eventOutID );
	MPJErr (*stopListenEventOut)( int nodeHandle, int eventOutID );
	MPJErr (*getNodeType)( int nodeHandle, int *outNodeType );
	MPJErr (*getNodeIDFromName)( int sceneHandle, char *defName, int *outDefID );
	MPJErr (*modifyRouteTo)( int inHandle, int inID, int outHandle, int outID, int insert );

	// the following are implemented on Java side
	MPJErr (*sceneEventNotification)( int sceneHandle, int whatHappened );
	MPJErr (*eventOutNotification)( int nodeHandle, int eventOutID, FieldValuePtr changedFieldValue );
	MPJErr (*getNewFieldValue)( int param1, int param2, int fieldIF, FieldValuePtr newFieldValue );
};
typedef MPJSceneDispatchTable* MPJSceneDispatchTablePtr;


/*
	A do-nothing (for us) function that ZRegistry wants to call
*/
/*
MPJ_EXTERN( MPJErr ) CreateInstance();
*/
/*
	IM1 should call this after creating and setting the dispatch table
*/
MPJ_EXTERN( MPJErr ) MPJSetSceneDispatchTable( MPJSceneDispatchTablePtr table, int version );

/*
	Use this to get the address of the current dispatch table
*/
MPJ_EXTERN( MPJSceneDispatchTablePtr ) MPJGetSceneDispatchTable();

#ifdef __cplusplus
}
#endif

#endif
