/**********************************************************************
 * Copyright  2001 ISO/IEC All Rights Reserved                        
 *                                                                       
 * This software module was developed by 
 *                                                                       
 * Sun Microsystems, Inc.
 *                                                                       
 * in the course of development of the MPEG-4 standard. This software
 * module is an implementation of a part of one or more MPEG-4 Systems
 * (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems standard.
 *                                                                       
 * The copyright of this software belongs to ISO/IEC. ISO/IEC gives
 * users of the MPEG-4 Systems (ISO/IEC 14496-1) standard free license
 * to this software module or modifications thereof for hardware
 * or software products claiming conformance to the MPEG-4 Systems.
 *                                                                       
 * Those intending to use this software module in hardware or software   
 * products are advised that use may infringe existing  patents. The     
 * original developers of this software module and their companies, the  
 * subsequent editors and their companies, and ISO/IEC have no liability 
 * and ISO/IEC have no liability for use of this software module or      
 * modification thereof in an implementation.                            
 **********************************************************************/
/*
 * File: ResourceManager.java
 *
 * Author: Vishy Swaminathan (vishy@eng.sun.com)
 *
 * Description:
 *
 * Created: March, 1999
 *
 * Modified: 
 * 
 */

package org.iso.mpeg.mpegj;
import java.lang.Runnable;

/**
 * This the interface that has to be implemented by a remote MPEG-J Application.
 * This methods in this interface are very similar syntactically and semantically 
 * to the Applet Class, except that this is an interface and it extends the Runnable
 * interface, so that it can be executed as separate thread. Note that there is no
 * start method. This is because run() of the Runnable interface is used instead of 
 * start
 */
public interface MPEGlet extends Runnable {
  /** Called by the MPEG-J player to inform the MPEGlet that it has been loaded into
   * the system. This is always called before the first time the MPEGlet is run as a 
   * separate by calling the run() method.  Any initialization goes here.
   */
  public void init();

  /** Called by the MPEG-J player to inform the MPEGlet that it should stop its execution.
   * This always called before destroy() is called.
   *  This can also be called anytime the execution of the MPEGlet has to be stopped. 
   * Any operations before it is stopped should go in here.
   */
  public void stop();

  /** Called by the MPEG-J player to inform the MPEGlet that is beinf reclaimed and that is 
   * should destroy any resources it has allocated. The stop method will always be called
   * before destroy. Any operations before it is destroyed should go in here.
   */
  public void destroy();
}
