/**********************************************************************
 * Copyright  2001 ISO/IEC All Rights Reserved
 *
 * This software module was developed by
 *
 * Sun Microsystems, Inc.
 *
 * in the course of development of the MPEG-4 standard. This software
 * module is an implementation of a part of one or more MPEG-4 Systems
 * (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems standard.
 *
 * The copyright of this software belongs to ISO/IEC. ISO/IEC gives
 * users of the MPEG-4 Systems (ISO/IEC 14496-1) standard free license
 * to this software module or modifications thereof for hardware
 * or software products claiming conformance to the MPEG-4 Systems.
 *
 * Those intending to use this software module in hardware or software
 * products are advised that use may infringe existing  patents. The
 * original developers of this software module and their companies, the
 * subsequent editors and their companies, and ISO/IEC have no liability
 * and ISO/IEC have no liability for use of this software module or
 * modification thereof in an implementation.
 **********************************************************************/

/*
 * File: MpegjTerminal.java
 */

package org.iso.mpeg.mpegj;
import java.util.*;
import org.iso.mpeg.mpegj.scene.*;
import org.iso.mpeg.mpegj.net.*;
import org.iso.mpeg.mpegj.resource.*;

import org.iso.mpeg.mpegjRefImpl.net.*;
import org.iso.mpeg.mpegjRefImpl.scene.*;
import org.iso.mpeg.mpegjRefImpl.MPEGJDebugPrintStream;
/**
 * MPEG-J Terminal class. This class provides the information about the managers that are implemented in the terminal. Each applet or application instantiates a new environment once it is loaded.
 */
public class MpegjTerminal
{
	/** Attributes storing managers*/
	private ResourceManager rm;
	private SceneManager sm;
	private NetworkManagerImp nm;

	/** Attributes storing MPEGlet */
	private MPEGlet _mpeglet;

	static
	{
		System.loadLibrary( "mpjterminal" );
	}

	/**
	* Constructor for the MpegjTerminal. This constructor may only be used by local applications.
	*
	* @exception SecurityException Thrown if this method is called by a non-local application.
	*/

	//static TextArea ta = null;

	public MpegjTerminal()
	{
		rm = new ResourceManager();
		nm = new NetworkManagerImp();
		sm = (SceneManager)new NativeScene(666);
		// System.out.println("The MpejTerminal constructor");
	}

	/**
	* Constructor for the MpegjTerminal. This constructor may be called by an MPEGlet to indicate
	* the name scope in which this MPEG-J Terminal object should operate.
	*
	* @param mpeglet the MPEGlet that identifies the name scope to use.
	* @exception SecurityException Thrown if this method is called by an unrecognized caller.
	*/
	public MpegjTerminal(MPEGlet mpeglet)
	{
		this();
		_mpeglet = mpeglet;
	}

	/**
	* Obtain the ResourceManager.
	*
	* @return the ResourceManager if available.
	* @exception ResourceManagerNotFoundException Thrown when the ResourceManager is not available.
	*/
	public ResourceManager getResourceManager() throws ResourceManagerNotFoundException
	{
		if (rm==null)
			throw new ResourceManagerNotFoundException();
		return rm;
	}

	/**
	* Obtain the SceneManager.
	*
	* @return the SceneManager if available.
	* @exception SceneManagerNotFoundException Thrown when the SceneManager is not available.
	*/
	public SceneManager getSceneManager() throws SceneManagerNotFoundException
	{
		if (sm == null)
			throw new SceneManagerNotFoundException();
		return sm;
	}

	/**
	* Obtain the NetworkManager used to get all the information related to a DMIF session and to enable/disable DMIF channels.
	* @return the NetworkManager if available.
	* @exception NetworkManagerNotFoundException Thrown when the NetworkManager is not available.
	*/
	public NetworkManager getNetworkManager() throws NetworkManagerNotFoundException
	{
		if (nm == null)
			throw new NetworkManagerNotFoundException();
		return nm;
	}

	/**
	*
	* Returns initOD associated with the scene
	* @return ObjectDescriptor
	*/
	public ObjectDescriptor getinitOD()
	{
		return null;
	}

	/**
	*
	* Returns the vector of ODs associated to that scene that were received by the OD stream.
	* @param int initOD
	* @return vector of Object Descriptors
	*/

	public Vector getODs(int initOD)
	{
		return null;
	}
}

