/**********************************************************************
 * Copyright  1998 ISO/IEC All Rights Reserved                        
 *                                                                       
 * This software module was developed by 
 *                                                                       
 * Sun Microsystems, Inc.
 *                                                                       
 * in the course of development of the MPEG-4 standard. This software
 * module is an implementation of a part of one or more MPEG-4 Systems
 * (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems standard.
 *                                                                       
 * The copyright of this software belongs to ISO/IEC. ISO/IEC gives
 * users of the MPEG-4 Systems (ISO/IEC 14496-1) standard free license
 * to this software module or modifications thereof for hardware
 * or software products claiming conformance to the MPEG-4 Systems.
 *                                                                       
 * Those intending to use this software module in hardware or software   
 * products are advised that use may infringe existing  patents. The     
 * original developers of this software module and their companies, the  
 * subsequent editors and their companies, and ISO/IEC have no liability 
 * and ISO/IEC have no liability for use of this software module or      
 * modification thereof in an implementation.                            
 **********************************************************************/
/*
 * File: MPDecoder.java
 *
 * Author: Yihan Fang (yihan.fang@sun.com or fang@ee.columbia.edu)
 *
 * Description:
 *
 * Created: August, 1998
 *
 * Modified: Vishy Swaminathan (vishy@eng.sun.com)
 * Fixed the Package Name
 * 
 *
 */

package org.iso.mpeg.mpegj.decoder;

import org.iso.mpeg.mpegj.resource.*;
import org.iso.mpeg.mpegj.ES_Descriptor;
import org.iso.mpeg.mpegj.decoder.*;
/** 
 * This is the base interface for media decoders.
 */
public interface MPDecoder 
extends MPDecoderEventGenerator
{
  /**  Start decoding of data. This method is not supposed to be
   * called for normal decoding. Because normal decoding starts
   * automatically from the underlying BIFS construct */
  public void start();
	
  /** 
   * Stop decoding of data.
   */
  public void stop();

  /** 
   * Pause decoding of data.
   */
  public void pause();

  /** 
   * Resume decoding of data from paused instant.
   */
  public void resume();

  /**
   * Attach a decoder for decoding of data.
   */
  public void attach(int esId);

  /**
   * Detach a decoder already decoding data.
   */
  public void detach();

  /**
   * Get decoder type.
   */
  public DecoderType getType();

  /**
   * Get decoder vendor name.
   */
  public String getVendor();

  /**
   * Get instance number.
   */
  public int getInstance();
  /** 
   * Determines if the decoder is pauseable.
   */
  public boolean isPauseable();
  
  //*** YUNTAI START
  // get ES_Descriptor from MPEDecoder 
  // from new spec  
  /** 
   * Get the attached ESDescriptor 
   */
  public ES_Descriptor getESDescriptor();

  //*** YUNTAI END
}
