
package org.iso.mpeg.mpegj.resource;

/********************************************************************
 This software module was originally developed by Luca Deri (Finsiel
- I - SOMMIT Consortium) and edited by Roberto Becchini (SIA - I -
SOMMIT Consortium), Barbara Silano (SIA - I - SOMMIT Consortium),
Gianluca De Petris (CSELT - I - SOMMIT Consortium) in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1). This software
module is an implementation of part of one or more MPEG-4 Systems
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC
14496-1). ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1)
free license to this software module or modifications thereof for use
in hardware or software products claiming conformance to the MPEG-4
Systems (ISO/IEC 14496-1). Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents. THE ORIGINAL DEVELOPER OF THIS SOFTWARE
MODULE AND HIS/HER COMPANY, THE SUBSEQUENT EDITORS AND THEIR
COMPANIES, AND ISO/IEC HAVE NO LIABILITY FOR USE OF THIS SOFTWARE
MODULE OR MODIFICATIONS THEREOF. No license to this software module
is granted for non MPEG-4 Systems (ISO/IEC 14496-1) conforming
products. The SOMMIT Consortium retains full right to use the
software module for his/her own purpose, assign or donate the
software module to a third party and to inhibit third parties from
using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming
products. This copyright notice must be included in all copies or
derivative works of the software module. Copyright 1999
********************************************************************/

import java.lang.*;
import java.util.*;


/**
 *
 *
 * This class is used to store information related to dynamic observers.
 *
 * @author  Luca Deri (l.deri@tecsiel.it)
 * @version 1.0, 11/1998
 */

public class CapabilityObserver extends Observable implements Runnable {
  private CapabilityManager term;
  private short capabilityCode;
  private Thread runnableThread;
  Hashtable capabilityInfo;

  /**
   *
   * CapabilityObserver constructor
   *
   * @param   t   a reference to the CapabilityManager
   * @param   _capabilityCode objToDeregister   the object to be deregistered.
   *
   */
  public CapabilityObserver(CapabilityManager t, short _capabilityCode) {
    super();
    capabilityInfo = null;
    term = t;
    capabilityCode = _capabilityCode;
    runnableThread = null;
  }


  /**
   *
   * Add a new observer
   *
   * @param   o the Observer
   * @param   _threshold      the threshold value
   * @parm    _aboveThreshold true if the notificatio has to be issued when the
   *                          actual value is above the specified threshold,
   *			      false otherwise
   *
   *
   */
  public synchronized void addObserver(Observer o, long _threshold,
				       boolean _aboveThreshold) {
    addObserver(o);

    if(capabilityInfo == null)
      capabilityInfo = new Hashtable();

    capabilityInfo.put(o, new CapabilityInfo(o, _threshold, _aboveThreshold));

    if(runnableThread == null) {
      runnableThread = new Thread(this);
      runnableThread.start();
    } else if(runnableThread.isInterrupted())
      runnableThread.resume();
  }


  /**
   *
   * deleteObserver: overrided method
   *
   * @param   o the Observer
   *
   *
   */
  public synchronized void deleteObserver(Observer o) {
    super.deleteObserver(o);

    capabilityInfo.remove(o);

    if((runnableThread != null) && (capabilityInfo.size() == 0)) {
      runnableThread.suspend();
    }
  }


  /**
   *
   * deleteObservers: overrided method
   *
   * @param   o the Observer
   *
   *
   */
  public synchronized void deleteObservers() {
    super.deleteObservers();
    capabilityInfo.clear();

    if(runnableThread != null) {
      runnableThread.suspend();
    }
  }


  /**
   *
   * run: the run() method (called by Thread)
   *
   *
   */
  public void run() {
    long value;

    for(;;) {
      switch(capabilityCode) {
      case CapabilityManager.capabilityCode_getFreeTerminalMemory:
	value = term.getFreeTerminalMemory();
	break;
      case CapabilityManager.capabilityCode_getFreeApplicationMemory:
	value = term.getFreeApplicationMemory();
	break;
      case CapabilityManager.capabilityCode_getTerminalLoad:
	value = term.getTerminalLoad();
	break;
      case CapabilityManager.capabilityCode_getNetworkLoad:
	value = term.getNetworkLoad();
	break;
      default:
	value = -1;
      }

      // System.out.println("Read value: "+value+" ("+capabilityCode+")");

      if(value != -1) {
	for(Enumeration e = capabilityInfo.elements(); e.hasMoreElements();) {
	  CapabilityInfo c = (CapabilityInfo)e.nextElement();
	  if(c.verifyCapabilityValue(value)) {
	    // System.out.println("notifyObservers called ["+countObservers()+" observers]");
	    setChanged();
	    notifyObservers(new Long(value));
	  }
	}
      }

      try {
    	Thread.sleep(5000); // 5 seconds
      } catch(Exception ex) {
	; // The thread has been interruped
      }
    }
  }

}


