
package org.iso.mpeg.mpegj.resource;

/********************************************************************
 This software module was originally developed by Luca Deri (Finsiel 
- I - SOMMIT Consortium) and edited by Roberto Becchini (SIA - I - 
SOMMIT Consortium), Barbara Silano (SIA - I - SOMMIT Consortium), 
Gianluca De Petris (CSELT - I - SOMMIT Consortium) in the course of 
development of the MPEG-4 Systems (ISO/IEC 14496-1). This software 
module is an implementation of part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
14496-1). ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) 
free license to this software module or modifications thereof for use 
in hardware or software products claiming conformance to the MPEG-4 
Systems (ISO/IEC 14496-1). Those intending to use this software 
module in hardware or software products are advised that its use may 
infringe existing patents. THE ORIGINAL DEVELOPER OF THIS SOFTWARE 
MODULE AND HIS/HER COMPANY, THE SUBSEQUENT EDITORS AND THEIR 
COMPANIES, AND ISO/IEC HAVE NO LIABILITY FOR USE OF THIS SOFTWARE 
MODULE OR MODIFICATIONS THEREOF. No license to this software module 
is granted for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. The SOMMIT Consortium retains full right to use the 
software module for his/her own purpose, assign or donate the 
software module to a third party and to inhibit third parties from 
using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works of the software module. Copyright 1999
********************************************************************/
 /**
 *
 *
 * <p>[Adaptive Audio-Visual Session Format ISO/IEC JTC1/SC29/WG11 - MPEG97/2566]<p>
 *
  * On some platforms and scenarios, it may be impossible for the terminal to guarantee
 * the constant availability of all of its resources. For example:
 * <ul>
 * <li> A wireless multimedia unit may encounter widely varying communication capacities, including intermittent connections.
 * <li> A general-purpose computer may experience varying load factors as other processes run on the system.
 * <li> Audio-visual sessions with content generated at multiple sources may cause resource contention on the terminal.
 * </ul>
 * 
 * In order for the audio-visual session to respond gracefully to these situations, AAVS
 * provides mechanisms for an AVSession object to catch dynamic changes to terminal
 * information and modify its behavior.  Upon initialization, an AVSession object may
 * subscribe with the Terminal object to receive notification of changes to dynamic terminal
 * information that is important to that particular audio-visual session.  If such
 * information does change during an audio-visual session, the Terminal will notify the
 * AVSession of the change through standardized interfaces.  Then the AVSession may respond
 * in the manner prescribed by the content creator.
 *
 * For example, consider an AAVS scene representing an MPEG-4 video sequence of a newscaster
 * and a background news clip coded as separate video objects.  The content creator might
 * desire to freeze the news clip whenever the actual display frame rate is too slow, rather
 * than sacrificing the quality of the newscaster in the foreground.  The content creator
 * can specify this behavior programmatically within the media by subscribing to
 * notifications of frame-rate changes from the terminal.  Then, when frame rate drops
 * significantly due, for example, to limited CPU capacity, the media object can dynamically
 * adapt and not continually decode the news clip and further degrade the presentation
 * performance.
 *
 *
 * @author  Luca Deri (l.deri@tecsiel.it)
 * @version 1.0, 11/1998
 */
 
 public interface DynamicCapability  {

    /** 
    * 
    * Terminal free memory (RAM)
    *
    * @return  the amount of free memory (bytes) installed on the terminal.
    * 
    */
    public long getFreeTerminalMemory();


    /** 
    * 
    * Terminal total memory (RAM)
    *
    * @return  the total amount of memory (bytes) installed on the terminal.
    * 
    */
    public long getTotalTerminalMemory();


    /** 
    * 
    * Total application memory (RAM)
    *
    * @return  the total amount of memory (bytes).
    * 
    */
    public long getTotalApplicationMemory();


    /** 
    * 
    * Free application memory (RAM)
    *
    * @return  the amount of available (at application level e.g. JavaVM) 
    * free memory (bytes).
    * 
    */
    public long getFreeApplicationMemory();


    /** 
    * 
    * Terminal CPU Load
    *
    * @return  the load (percentage) of the terminal CPUs
    * 
    */
   public long getTerminalLoad();


    /** 
    * 
    * Terminal Network Load
    *
    * @return  the network load (percentage).
    * 
    */
    public long getNetworkLoad();
}
