
package org.iso.mpeg.mpegj.resource;

/********************************************************************
 This software module was originally developed by Luca Deri (Finsiel
- I - SOMMIT Consortium) and edited by Roberto Becchini (SIA - I -
SOMMIT Consortium), Barbara Silano (SIA - I - SOMMIT Consortium),
Gianluca De Petris (CSELT - I - SOMMIT Consortium) in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1). This software
module is an implementation of part of one or more MPEG-4 Systems
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC
14496-1). ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1)
free license to this software module or modifications thereof for use
in hardware or software products claiming conformance to the MPEG-4
Systems (ISO/IEC 14496-1). Those intending to use this software
module in hardware or software products are advised that its use may
infringe existing patents. THE ORIGINAL DEVELOPER OF THIS SOFTWARE
MODULE AND HIS/HER COMPANY, THE SUBSEQUENT EDITORS AND THEIR
COMPANIES, AND ISO/IEC HAVE NO LIABILITY FOR USE OF THIS SOFTWARE
MODULE OR MODIFICATIONS THEREOF. No license to this software module
is granted for non MPEG-4 Systems (ISO/IEC 14496-1) conforming
products. The SOMMIT Consortium retains full right to use the
software module for his/her own purpose, assign or donate the
software module to a third party and to inhibit third parties from
using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming
products. This copyright notice must be included in all copies or
derivative works of the software module. Copyright 1999
********************************************************************/

import java.lang.*;
import java.util.*;

 /**
 *
 *
 * <p>[Adaptive Audio-Visual Session Format ISO/IEC JTC1/SC29/WG11 - MPEG97/2566]<p>
 *
 * In order for the audio-visual session to respond gracefully to these situations,
 * AAVS provides mechanisms for an AVSession object to catch dynamic changes to terminal
 * information and modify its behavior.  Upon initialization, an AVSession object may
 * subscribe with the Terminal object to receive notification of changes to dynamic terminal
 * information that is important to that particular audio-visual session.  If such information
 * does change during an audio-visual session, the Terminal will notify the AVSession of the
 * change through standardized interfaces.  Then the AVSession may respond in the manner
 * prescribed by the content creator.
 *
 * For example, consider an AAVS scene representing an MPEG-4 video sequence of a newscaster
 * and a background news clip coded as separate video objects.  The content creator might
 * desire to freeze the news clip whenever the actual display frame rate is too slow, rather
 * than sacrificing the quality of the newscaster in the foreground.  The content creator can
 * specify this behavior programmatically within the media by subscribing to notifications of
 * frame-rate changes from the terminal.  Then, when frame rate drops significantly due, for
 * example, to limited CPU capacity, the media object can dynamically adapt and not
 * continually decode the news clip and further degrade the presentation performance.
 *
 *
 * @author  Luca Deri (l.deri@tecsiel.it)
 * @version 1.0, 11/1998
 */

public interface DynamicCapabilityObserver {

  /**
   *
   * This method is used by AVSession objects that want to be notified when the
   * application free memory goes below a certain threshold.
   *
   * @param   freeApplicationMemory  the free application memory threshold (bytes).
   * @param   objToNotify  the object to be notified.
   * @exception IllegalArgumentException  the threshold is out of range.
   *
   */
  public void notifyApplicationFreeMemory(long freeApplicationMemory, Observer objToNotify)
       throws IllegalArgumentException;


       /**
	*
	* This method is used by AVSession objects that want to be notified when the
	* total terminal free memory goes below a certain threshold.
	*
	* @param   freeTerminalMemory  the free application memory threshold (bytes).
	* @param   objToNotify  the object to be notified.
	* @exception IllegalArgumentException  the threshold is out of range.
	*
	*/
  public void notifyTerminalFreeMemory(long freeTerminalMemory, Observer objToNotify)
       throws IllegalArgumentException;



    /**
	*
	* This method is used by AVSession objects that want to be notified when the
	* terminal CPU load goes above a certain percentage.
	*
	* @param   cpuLoadPercentage   the cpu load (percentage) threshold.
	* @param   objToNotify   the object to be notified.
	* @exception IllegalArgumentException   the threshold out of range [0...100].
	*
	*/
  public void notifyTerminalLoad(long cpuLoadPercentage, Observer objToNotify)
       throws IllegalArgumentException;



       /**
	*
	* This method is used by AVSession objects that want to be notified when the
	* terminal network load goes above a certain percentage.
	*
	* @param   nwLoadPercentage   the network load (percentage) threshold.
	* @param   objToNotify   the object to be notified.
	* @exception IllegalArgumentException   the threshold out of range [0...100].
	*
	*/
  public void notifyTerminalNetworkLoad(long nwLoadPercentage, Observer objToNotify)
       throws IllegalArgumentException;

       /**
	*
	* This method is used to deregister an observer previously registered using
	* any of the notifyXXX methods.
	*
	* @param   objToDeregister   the object to be deregistered.
	*
	*/
  public void deleteObserver(Observer objToDeregister);

}
