/**********************************************************************
 * Copyright  1998 ISO/IEC All Rights Reserved                        
 *                                                                       
 * This software module was developed by 
 *                                                                       
 * Sun Microsystems, Inc.
 *                                                                       
 * in the course of development of the MPEG-4 standard. This software
 * module is an implementation of a part of one or more MPEG-4 Systems
 * (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems standard.
 *                                                                       
 * The copyright of this software belongs to ISO/IEC. ISO/IEC gives
 * users of the MPEG-4 Systems (ISO/IEC 14496-1) standard free license
 * to this software module or modifications thereof for hardware
 * or software products claiming conformance to the MPEG-4 Systems.
 *                                                                       
 * Those intending to use this software module in hardware or software   
 * products are advised that use may infringe existing  patents. The     
 * original developers of this software module and their companies, the  
 * subsequent editors and their companies, and ISO/IEC have no liability 
 * and ISO/IEC have no liability for use of this software module or      
 * modification thereof in an implementation.                            
 **********************************************************************/
/*
 * File: MPDecoderMediaEvents.java
 *
 * Author: Yihan Fang (yihan.fang@sun.com or fang@ee.columbia.edu)
 *
 * Description:
 *
 * Created: August, 1998
 *
 * Modified: March, 1999
 * Vishy Swaminathan (vishy@eng.sun.com)
 * Fixed Package Name
 *
 */


package org.iso.mpeg.mpegj.resource;
import java.util.*;

/**
 * An event object that is given to an MPDecoderMediaListener.
 */

public class MPDecoderMediaEvents extends java.util.EventObject
{
  public static final int DECODER_OVERFLOW = 0;
  public static final int DECODER_SYNC_ERROR = 1;
  public static final int STREAM_UNDERFLOW = 2;
  public static final int STREAM_OVERFLOW = 3;
  public static final int STREAM_START = 4;
  public static final int STREAM_END = 5;

  private int condition;

  /**
   * Construct an event object with condition.
   * condition can be decoder overflow, sync error, or end of decode,
   * otherwise throws exception
   * @exception IllegalDecoderMediaEventsException thrown an Illegal Decoder Event is thrown
   * @param source the source of the event
   * @param condition The condition variable can correspond the to decoder overflow, sync error, 
   * stream start or stop
   */
  public MPDecoderMediaEvents(Object source, int condition) 
    throws IllegalDecoderMediaEventsException { 
      super(source);
      if (condition!=DECODER_OVERFLOW &&
	  condition!=DECODER_SYNC_ERROR &&
	  condition!=STREAM_UNDERFLOW &&
	  condition!=STREAM_OVERFLOW &&
	  condition!=STREAM_START &&
	  condition!=STREAM_END)
	throw new IllegalDecoderMediaEventsException();
      else 
	this.condition = condition;
  }

  /**
   * The obtained condition can correspond to decoder overflow, sync error, 
   * stream underflow, stream overflow, stream start or stream end
   */
  public int getCondition(){ return condition; }
}


