/**********************************************************************
 * Copyright  2001 ISO/IEC All Rights Reserved                        
 *                                                                       
 * This software module was developed by 
 *                                                                       
 * Sun Microsystems, Inc.
 *                                                                       
 * in the course of development of the MPEG-4 standard. This software
 * module is an implementation of a part of one or more MPEG-4 Systems
 * (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems standard.
 *                                                                       
 * The copyright of this software belongs to ISO/IEC. ISO/IEC gives
 * users of the MPEG-4 Systems (ISO/IEC 14496-1) standard free license
 * to this software module or modifications thereof for hardware
 * or software products claiming conformance to the MPEG-4 Systems.
 *                                                                       
 * Those intending to use this software module in hardware or software   
 * products are advised that use may infringe existing  patents. The     
 * original developers of this software module and their companies, the  
 * subsequent editors and their companies, and ISO/IEC have no liability 
 * and ISO/IEC have no liability for use of this software module or      
 * modification thereof in an implementation.                            
 **********************************************************************/
/*
 * File: ResourceManager.java
 *
 * Author: Yihan Fang (yihan.fang@sun.com or fang@ee.columbia.edu)
 *
 * Description:
 *
 * Created: August, 1998
 *
 * Modified: March, 1999  Vishy Swaminathan (vishy@eng.sun.com)
 * Fixed the Package Name
 * It now throws the right Exceptions
 * get CapabilityManager API
 * Basic cleanup.

*/

package org.iso.mpeg.mpegj.resource;

import org.iso.mpeg.mpegj.decoder.*;
import org.iso.mpeg.mpegj.scene.*;
import org.iso.mpeg.mpegj.*;
import java.util.*;
import java.io.*;


import org.iso.mpeg.mpegjRefImpl.decoder.*;
import org.iso.mpeg.mpegjRefImpl.resource.*;
import org.iso.mpeg.mpegjRefImpl.*;
/**
 * The resource manager is used for regulation of performance. 
 * The Resource Manager API provides a centralized facility for managing 
 * resources. It is a collection of a number of classes and interfaces
 * summarized as follows. 
 */
public class ResourceManager
{
    
    /** Attribute for storing capability manager, decoder priority, decoder,
     * renderer and available decoder*/
    private CapabilityManager cm;
    private int decoderPriority;
    private MPDecoderImp decoder;
    private Renderer renderer;
    private Vector decoderVec;
    private int count = 1;

    // native methods
	
    public native int getESIDOnNode(int nodeId);
	
	//**** YUNTAI START
	private native int changeDecoder(long odID, long esID, long oldesID);
	private native int checkODID(long odID);
	private native int checkESID(long esID);
	//**** YUNTAI END
    static
    {
	System.loadLibrary( "resourcemanager" );
    } 

    /** Constructor */
    public ResourceManager()
    {
		cm = new CapabilityManager();	
    }
    
      /** 
    * 
    * Returns the CapabilityManager used to retrieve all the capabilities (static, dynamic, profile) of the terminal.
    *
    * @return the CapabilityManager if available
    * @exception CapabilityManagerNotFoundException Thrown when the CapabilityManager is not available (e.g. it is impossible to retrieve information about CPU load, memory etc.)
    */
	public CapabilityManager getCapabilityManager() throws 
                CapabilityManagerNotFoundException {	
		if (cm == null) throw new CapabilityManagerNotFoundException();
		return cm;
	}
	
	
	
	/** 
   	* Override/change priority of a media elementary stream.
   	* @exception DecoderNotFoundException Thrown when the Decoder is not available 
   	* @exception InvalidDecoderPriorityException thrown when the prirority is invalid
   	*/
	public void setDecPriority (MPDecoder dec, int priority) throws 
  					InvalidDecoderPriorityException, DecoderNotFoundException
	{
  		if (dec == null) throw new DecoderNotFoundException ();
		// invalid priority number
		if (priority < 0) throw new InvalidDecoderPriorityException ();
		decoderPriority = priority;
	}

  /** 
   * Get priority of a media elementary stream.
   * @exception DecoderNotFoundException Thrown when the Decoder is not available 
   */
	public int getDecPriority (MPDecoder dec) throws 
  				DecoderNotFoundException
	{
    	if (dec == null) throw new DecoderNotFoundException ();
		return decoderPriority;
	}  
  

  /**
   * Get Decoder.
   * @exception DecoderNotFoundException Thrown when the Decoder is not available 
   * @exception BadNodeException is thrown when the nodeId does not correspond to a valid node in the scenegraph
   */
  public MPDecoder getDecoder(Node node) throws 
      DecoderNotFoundException, BadNodeException{
        // to be implemented   
        return decoder;
    }
    
    /** this method is not supported any more - Praveen
   * - need to remove this after implementing the other 2 methods for getDecoder() - Praveen
   * Get Decoder.
   * @exception DecoderNotFoundException Thrown when the Decoder is not available 
   * @exception BadNodeException is thrown when the nodeId does not correspond to a valid node in the scenegraph
   */
    public MPDecoder getDecoder(int nodeId) throws 
	DecoderNotFoundException, BadNodeException{ 	
	    decoder = new MPDecoderImp (nodeId);
	if(decoderVec == null)
			decoderVec = new Vector();
	    decoderVec.addElement((Object)decoder);
	    return decoder;    	
    }

/**
   * Get Decoder for the given ObjectDescriptor and ES_Descriptor.
   * @param ObjectDescriptor 
   * @param ES_Descriptor
   * @return MPDecoder
   * @exception DecoderNotFoundException Thrown when the Decoder is not available 
   * @exception BadNodeException is thrown when the nodeId does not correspond to a valid node in the scenegraph
   */
	public MPDecoder getDecoder(ObjectDescriptor ODID, ES_Descriptor ESID ) throws 
  		DecoderNotFoundException, BadNodeException {
    	decoder = new MPDecoderImp ( ((ObjectDescriptorImp)ODID).getObjectDescriptorID() , ((ES_DescriptorImp)ESID).getES_ID() );
		if(decoderVec  == null)
			decoderVec = new Vector();
		decoderVec.addElement((Object)decoder);
		return decoder;    	
	}

  /**
   * Get renderer and be ready to add events to renderer.
   * @exception RendererNotFoundException is thrown if the Renderer is not available
   */
  public Renderer getRenderer() throws
	  RendererNotFoundException {
    if( renderer == null ) //praveen
		renderer = new RendererImp(); // praveen
//	if (renderer == null ) throw new RendererNotFoundException ();
	return renderer;
  }
  
  /** Change decoder on nodeId. The underlying implementation is
   * expected to instantiate, attach and start a decoder similar to
   * establishing a new decoder for a BIFS node. All the elementary
   * streams attached to the original decoder is attached to the new
   * decoder.  
   * @param Node
   * @param MPDecoder
   * @exception DecoderNotFoundException Thrown when the Decoder is not available and BadNodeException is thrown when the nodeId does not correspond to a valid node in the scenegraph
   * @exception BadNodeException is thrown when the nodeId does not correspond to a valid node in the scenegraph
   */
	public void changeDecoder(Node node, MPDecoder dec) throws
  					DecoderNotFoundException, BadNodeException
	{    
		// YUNTAI - 
		// ok you have a Node object and MPDecoder object
		// nothing can be done....	
		// should be methods to retrieve OD from node...
		// but this method is gone in the Jun 00 spec
	}

  /** Change decoder on the given Object Descriptor and ES_Descriptor
   * @param ObjectDescriptor 
   * @param ES_Descriptor
   * @param MPDecoder
   * @exception DecoderNotFoundException Thrown when the Decoder is not available 
   * @exception BadNodeException is thrown when the OD does not correspond to a valid node in the scenegraph
   */
   /*** YUNTAI ***/
   // implementation of changeDecoder method
	public void changeDecoder(ObjectDescriptor ODesc, ES_Descriptor esDesc, MPDecoder dec) throws
  		DecoderNotFoundException, BadNodeException {
		//*** YUNTAI START
		int iRet;
		
		// check ESID assoicated with the decoder 
		iRet = checkESID(dec.getESDescriptor().getES_ID());
		if(iRet < 0)
			throw new DecoderNotFoundException();
		
		// check ESID assoicated with the decoder 
		iRet = checkESID(esDesc.getES_ID());
		if(iRet < 0)
			throw new BadNodeException();		
		
		iRet = checkODID(ODesc.getObjectDescriptorID());
		if(iRet < 0) 
			throw new BadNodeException();
			
		changeDecoder(ODesc.getObjectDescriptorID(),esDesc.getES_ID(),dec.getESDescriptor().getES_ID());
		//*** YUNTAI END
  	}


  	/**
   	* Get available decoders given decoder type. 
   	* @return returns an array of available decoders that are not used currently.
   	* @exception InvalidDecoderTypeException Thrown when the DecoderType not valid
   	*/
	public /*MPDecoder[]*/ Vector getAvailableDecoder(DecoderType type) throws
		InvalidDecoderTypeException {
    	if (type.getDecoderType() < 0 || type.getDecoderType() > 16) {// only 17 types
			throw new InvalidDecoderTypeException ();
    	}
		return decoderVec;
  	}
}


