
package org.iso.mpeg.mpegj.resource;

/********************************************************************
 This software module was originally developed by Luca Deri (Finsiel 
- I - SOMMIT Consortium) and edited by Roberto Becchini (SIA - I - 
SOMMIT Consortium), Barbara Silano (SIA - I - SOMMIT Consortium), 
Gianluca De Petris (CSELT - I - SOMMIT Consortium) in the course of 
development of the MPEG-4 Systems (ISO/IEC 14496-1). This software 
module is an implementation of part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
14496-1). ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) 
free license to this software module or modifications thereof for use 
in hardware or software products claiming conformance to the MPEG-4 
Systems (ISO/IEC 14496-1). Those intending to use this software 
module in hardware or software products are advised that its use may 
infringe existing patents. THE ORIGINAL DEVELOPER OF THIS SOFTWARE 
MODULE AND HIS/HER COMPANY, THE SUBSEQUENT EDITORS AND THEIR 
COMPANIES, AND ISO/IEC HAVE NO LIABILITY FOR USE OF THIS SOFTWARE 
MODULE OR MODIFICATIONS THEREOF. No license to this software module 
is granted for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. The SOMMIT Consortium retains full right to use the 
software module for his/her own purpose, assign or donate the 
software module to a third party and to inhibit third parties from 
using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works of the software module. Copyright 1999
********************************************************************/

 import java.awt.*;
 import java.lang.*;

 /**
 *
 *
 * <p>Adaptive Audio-Visual Session Format ISO/IEC JTC1/SC29/WG11 - MPEG97/2566]<p>
 * 
 * When initialized at an adaptive terminal, an AVSession object may query a local
 * Terminal object through standardized interfaces to discover the terminal's configuration.
 * This information could include the available presentation devices and their
 * characteristics, the terminal's ability to handle certain types of media, and even cached
 * user preferences.  Then the AVSession may use this information when constructing its scene
 * representation.
 *
 * For example, suppose that an AVSession object were instantiated on a terminal with a
 * low-resolution display.  By querying the Terminal object to learn the limitations of the
 * display, the AVSession object may programmatically choose to enlarge the size of any text
 * used in the audio-visual session.
 *
 * @author  Luca Deri (l.deri@tecsiel.it)
 * @version 1.0, 11/1998
 */
 
 public interface StaticCapability {


    /** 
    * 
    * Number of Terminal CPUs
    *
    * @return  the number of CPUs that equip the terminal
    * 
    */
    public int getNumCPUs();


    /** 
    * 
    * Terminal CPU Speed 
    *
    * @param   idx   the index of the video card (idx >= 0)
    * @return  the speed (Mhz) of a terminal CPU.  
    * @exception IllegalArgumentException   the index is either negative or out of range.
    * 
    */
    public int getCPUSpeed(int idx);


    /** 
    * 
    * Terminal CPU type (Pentium for instance)
    *
    * @param   idx   the index of the CPU (idx >= 0)
    * @return  the type of CPU
    * @exception IllegalArgumentException   the index is either negative or out of range.
    * 
    */
    public String getCPUType(int idx) throws IllegalArgumentException;


    /** 
    * 
    * Terminal Mouse Type
    *
    * @return  the mouse type
    * 
    */
    public String getMouseType();


    /** 
    * 
    * Terminal Display Color Depth
    *
    * @return  the display color depth (bits per pixel)
    * 
    */
    public int getDisplayColorDepth();


    /** 
    * 
    * Terminal Display Type (e.g. VGA)
    *
    * @return  the display type
    * 
    */
   public String getDisplayType();


    /** 
    * 
    * Terminal Keyboard Type
    *
    * @return  the keyboard type, NULL if the keyboard is not present.
    * 
    */
    public String getKeyboardType();


    /** 
    * 
    * Terminal Network Type
    *
    * @return  the network type.
    * 
    */
    public String getNetworkType();


    /** 
    * 
    * Terminal parallel IEEE-1284 port(s)
    *
    * @return  the number of parallel ports present on the terminal.
    * 
    */
    public int getNumParallelPorts();


    /** 
    * 
    * Terminal serial RS-232 port(s)
    *
    * @return  the number of serial ports present on the terminal.
    * 
    */
    public int getNumSerialPorts();


    /** 
    * 
    * Terminal screen size
    *
    * @return  the screen size (pixel x pixel)
    * 
    */
    public Dimension getScreenSize();


    /** 
    * 
    * Terminal screen depth
    *
    * @return  the screen depth (bits/inch)
    * 
    */
    public short getScreenDepth();


    /** 
    * 
    * Terminal screen resolution
    *
    * @return  the screen resolution (dots/inch)
    * 
    */
    public short getScreenResolution();


    /** 
    * 
    * Terminal OS language
    * [lsLanguage]
    *
    * @return  the language of the OS version installed on the terminal.
    * 
    */
    public String getOSLanguage();


    /** 
    * 
    * Terminal OS type (for instance JavaPC, Windows, MacOS)
    * [lsOpSys/lsSysType]
    *
    * @return  the OS type installed on the terminal.
    * 
    */
    public String getOSType();	


    /** 
    * 
    * Terminal architecture (for instance Alpha, x86)
    *
    * @return  the terminal architecture type.
    * 
    */
    public String getTerminalArchitecture();	


    /** 
    * 
    * Terminal Audio Enc/Dec
    *
    * @return  the available audio drivers.
    * 
    */
    public String[] getAudioDrivers();


    /** 
    * 
    * Terminal MIDI Enc/Dec
    *
    * @return  the available MIDI drivers.
    * 
    */
    public String[] getMIDIDrivers();	


    /** 
    * 
    * Terminal Video Enc/Dec
    *
    * @return  the available video drivers.
    * 
    */
    public String[] getVideoDrivers();	


    /** 
    * 
    * Terminal modem type
    * [lsInternalModem]
    *
    * @return  the modem speed ("28K", "56K" etc.) or NULL is the modem is not installed.
    * 
    */
    public String getModemType();
}
