/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
package org.iso.mpeg.mpegj.scene;

/**
 * An interface used to specify the creation of a new node.
 */
public interface NewNode extends NodeValue {
    /**
     * Obtain the node type of the new node. For example, this
     * method may return NodeType.Transform.
     *
     * @return the integer node type.
     * @see org.iso.mpeg.mpegj.scene.NodeType
     */
    public int getNodeType();

    /**
     * Obtain the DEF identifier of the new node, or zero if the node does
     * not have a DEF identifier.
     *
     * @return the DEF identifier, or zero if none.
     */
    public int getNodeID();

    /**
     * Obtain the value of a field in the new node identified by the
     * given defID, or null if the field has the default value. For
     * example, the defID argument may be Field.Transform.translation,
     * in which case the object returned shall implement the
     * SFVec3fFieldValue interface.
     *
     * @param defID the field defID.
     * @return the field value, or null for the default value.
     * @exception BadParameterException if the defID is not a valid field defID.
     * @see org.iso.mpeg.mpegj.scene.Field
     */
    public FieldValue getField(int defID) throws BadParameterException;
}