/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
/*
  $Id: Scene.java,v 1.1.1.1 2003/12/24 00:52:57 mikeseven Exp $
*/

package org.iso.mpeg.mpegj.scene;

/**
	An interface that acts as a proxy for a BIFS scene.
*/
public interface Scene
{
	/**
		Returns an instance of a Node proxy for an instanced
		node in the scene.
		
		@param defID The id of the desired node. This node must have been
		instanced using DEF.
		@return a Node proxy for the desired node in the scene.
		@exception BadParameterException Thrown if no node exists with the
		specified id.
		@exception InvalidSceneException Thrown if the Scene is no longer
		valid (due to removal or replacement).
	*/
	Node getNode( int defID ) throws BadParameterException, InvalidSceneException;

	/**
		Returns an instance of a Node proxy for an instanced
		node in the scene.
		
		@param defName The name of the desired node. This node must have been
		instanced using DEF with an associated USENAME.
		@return a Node proxy for the desired node in the scene.
		@exception BadParameterException Thrown if no node exists with the
		specified name.
		@exception InvalidSceneException Thrown if the Scene is no longer
		valid (due to removal or replacement).
	*/
    Node getNode( String defName ) throws BadParameterException, InvalidSceneException;

	/**
		Inserts a route into the scene. Does nothing if there is already
		a route between the specified fields.
		
		@param fromNode the node containing the source field of the route.
		@param fromID the outID of the source field.
		@param toNode the node containing the destination field of the route.
		@param toID the inID of the destination field.
		@exception BadParameterException Thrown if the nodes or fields do
		not exist.
		@exception InvalidSceneException Thrown if the Scene is no longer
		valid (due to removal or replacement).
	*/
    void insertRoute( Node fromNode, int fromID, Node toNode, int toID ) throws BadParameterException, InvalidSceneException;

	/**
		Removes a route from the scene. Does nothing if there is no route
		between the specified fields.
		
		@param fromNode the node containing the source field of the route.
		@param fromID the outID of the source field.
		@param toNode the node containing the destination field of the route.
		@param toID the inID of the destination field.
		@exception BadParameterException Thrown if the nodes or fields do
		not exist.
		@exception InvalidSceneException Thrown if the Scene is no longer
		valid (due to removal or replacement).
	*/
    void deleteRoute( Node fromNode, int fromID, Node toNode, int toID ) throws BadParameterException, InvalidSceneException;
}
