/***********************************************************************
This software module was originally developed by Mikael Bourges-Sevenier
(Mindego Inc.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-11) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1 and 11) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1 and 11) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1 and 11) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1 and 11). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1 and 11) conforming products. Mindego Inc. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1 and 11) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  2003.
************************************************************************/

package org.iso.mpeg.mpegj.scene;

/**
 * @author Mikael Bourges-Sevenier, Mindego Inc. (mikael@mindego.com)
 * @version 1.0 October 23, 2003 (MPEG 66 meeting)
 */
public interface SceneControllerListener
{
  /**
   * This method is called when the listener is being attached to the renderer or when it is reinitialized.
   * This enables the scene controller listener to initialize its resources, possibly from the scene already
   * rendered by the renderer.
   * Note: a scene might not always be present.
   * 
	 * @param scene, the current scene being rendered by the renderer
	 */
	void init(Scene scene);

  /**
   * This method is called by the renderer before rendering the scene.
   * This is where modifications to the scene can be done before rendering.
   * 
	 * @param scene, the current scene being rendered by the renderer
	 */
	void preRender(Scene scene);

  /**
   * This method is called after the scene has been rendered.
   * 
	 * @param scene, the current scene being rendered by the renderer
	 */
	void postRender(Scene scene);

  /**
	 * This method is called when the scene controller listener is destroyed to enable the listener to deallocate its
	 * resources (typically allocated by init()).
	 */
	void dispose();

}
