/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
/*
  $Id: SceneManager.java,v 1.1.1.1 2003/12/24 00:52:57 mikeseven Exp $
*/

package org.iso.mpeg.mpegj.scene;

/**
	An interface that allows access to the MPEG-4 terminal's
	native scene. An instance is obtained using getSceneManager()
	in the MPEGJTerminal class.
	<p>
	Note that scene graph access requires first obtaining a Scene instance.
	The only normative way to get a Scene instance is through a
	notification on a SceneListener instance.
	
	@see org.iso.mpeg.terminal.MPEGJTerminal
	@see org.iso.mpeg.mpegj.scene.SceneListener
	@see org.iso.mpeg.mpegj.scene.Scene
*/
public interface SceneManager
{
	/**
		Add a SceneListener to this scene manager. The listener will
		be informed of important changes to the scene, and will be
		given the appropriate Scene instance that will allow modification
		to nodes and fields of the scene. The base specification does not limit
		the number of SceneListeners that may be registered.
		
		@param listener the SceneListener instance to add.
	*/
	void addSceneListener( SceneListener listener );
	
	/**
		Remove a SceneListener from the scene manager.
		
		@param listener the SceneListener instance to remove.
		@exception BadParameterException Thrown if the specified listener
		was not found on the scene manager's list of listeners.
		*/
	void removeSceneListener( SceneListener listener ) throws BadParameterException;
}
