/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
package org.iso.mpeg.mpegjRefImpl;

import java.lang.ClassLoader;
import java.util.Hashtable;
import java.io.*;
import java.net.*;
import java.util.zip.*;
import java.util.jar.*;
import java.lang.reflect.*;
import java.util.*;
import org.iso.mpeg.mpegj.*;


public class MPEGJClassLoader extends ClassLoader
{
	public static String FULLY_QUALIFIED_MPEGLET_CLASS_NAME = "org.iso.mpeg.mpegj.MPEGlet";
	Hashtable classCache = new Hashtable();
	Hashtable dataCache = new Hashtable();
	static MPEGJClassLoader m_loader = null;
	
	
	public synchronized void addClassData(byte[] bytesFromFile, String className, int len) {
		if(!dataCache.containsKey(className)) {
			
			byte [] newba = new byte[len];
			System.arraycopy(bytesFromFile,0,newba,0,len);
			dataCache.put(className,newba);
		}
	}
	
	public synchronized void addClassData(byte[] bytesFromFile, String className) {
		if(!dataCache.containsKey(className)) {
			dataCache.put(className,bytesFromFile);
		}
	}
	
	public synchronized Class loadClass(String name, boolean resolve) throws	   
			ClassNotFoundException, ClassFormatError
	{
		
		Class c = (java.lang.Class) classCache.get(name); 
		
		if (c == null) {
			try {
				try {
					c = findSystemClass(name);
				} 
				catch (ClassNotFoundException cnfe) {
					c = null;
				}
				
				
				if (c==null) {
					byte [] data = null;
					System.out.println(name);
					if((data = (byte[])dataCache.get(name)) != null) {
						c = defineClass(name, data, 0, data.length);
							if (c != null) {
							classCache.put(name,c);
							dataCache.remove(name);
						}
                        else
                        {
                            System.out.println("Could not defineClass");
                        }
		
					} 
                    else
                    {
                        System.out.println("No Data to read bytes from");
                    }
				}
				else {
					//System.out.println("System Class " + name + " Defined");
				}
			}
			catch (ClassFormatError cfe) {
				cfe.printStackTrace();
				//System.out.println("Data Corrupted");
				throw cfe;
				
			}
		}
		else 
			;//System.out.println("Class " + name + " already Defined");
		if (resolve && (c != null)) {
			resolveClass(c);
		}
		return c;
	}

	public Class loadClass(String name) throws ClassNotFoundException
	{
		return(loadClass(name,false));
	}

	public Enumeration readZipEntry(byte[] buffer)
	{
		//System.out.println("readZipEntry");

		Vector v = new Vector();
		try {
		JarInputStream zin = new JarInputStream(new ByteArrayInputStream(buffer));

		//ZipEntry zentry = null;
		JarEntry zentry = null;
		byte[] entryBuffer = new byte[buffer.length*10];



		System.out.println("readZipEntry" + buffer.length);
		//try {
			//while((zentry = zin.getNextEntry()) != null) {
			while((zentry = zin.getNextJarEntry()) != null) {
				System.out.println("Got nextEntry");
				int read = 0;
				int ret;
				do {
					ret = zin.read(entryBuffer,read,entryBuffer.length-read);
					if(ret > 0)
						read += ret;
				} while(ret > 0);

    			String classFullName = zentry.getName();
				System.out.println("readZipEntry: classFullName " + classFullName );
    			if( classFullName.lastIndexOf(".class") >= 0 )
    			{
					classFullName = classFullName.substring(0,classFullName.lastIndexOf(".class"));
					classFullName = classFullName.replace('.','/');

					addClassData(entryBuffer,classFullName,read);
					v.addElement(classFullName);
				}
			}
		}
		catch(Exception e) {
			return null;
		}

		return v.elements();
	}


	/** 
		This method is used by the application using the class Loader. 
		The application provides the byte array which is zip data
	*/
	public static void loadZipArchive(byte[] zippedData)
	{
		
		//System.out.println("Load Zip Archive...");
		if(m_loader == null)
			m_loader = new MPEGJClassLoader();

		
						
		Enumeration e = m_loader.readZipEntry(zippedData);
		Class c = null;
		try {
			
			for (; e.hasMoreElements() ;) {
				String s = (String)e.nextElement();
				//System.out.println("element:" + s);
				c = m_loader.loadClass(s);
				System.out.println("loaded:" + s);
				if(c == null)
					System.out.println("Fatal Error!: " + s);
				m_loader.startIfMPEGlet(c);
			}
       	} catch(Exception ex) {
			System.out.println("[loadZipArchive]" + ex.toString());
            ex.printStackTrace();
		}
	}
	
	public static byte[] readFile(String filename) {
    DataInputStream in=null;
    byte[] buffer;
    int readLength=0;


    try {
      in = new DataInputStream(new BufferedInputStream(new FileInputStream(filename)));
      
    } catch (FileNotFoundException e) {
      System.err.println("Could not open " + filename);
    }
    
    buffer = new byte[(int) (new File(filename)).length()];
    try {
      readLength = in.read(buffer);
    } catch(IOException ioe) {
      ioe.printStackTrace();
    }
    if (readLength != buffer.length) {
      System.out.println("Read Error ");
      System.exit(-1);
    }
    try {
      in.close();
    } catch (IOException ioe) {
      ioe.printStackTrace();
      System.err.println("Could not Close");    
    }
    return(buffer);
  }

	private void runMPEGlet(Class c) throws Exception {
        System.out.println("Before newInstance");
		MPEGlet mpeglet = (MPEGlet)c.newInstance();
        System.out.println("Before MPEG init");
	    mpeglet.init();
        System.out.println("After MPEG init");
		new Thread(mpeglet).start();
        System.out.println("After MPEG start");
		
	}
	
	private void startIfMPEGlet(Class c) throws Exception {
		//System.out.println("class name: " + c.getName());
		Class [] interfaces = c.getInterfaces();
		//System.out.println("interfaces length: " + interfaces.length);
		for(int i=0;i < interfaces.length;i++) {
			String s = interfaces[i].getName();
			//System.out.println("Interface name: " + s);
			//System.out.println(interfaces[i].getName());
			if(interfaces[i].getName().compareTo(FULLY_QUALIFIED_MPEGLET_CLASS_NAME) == 0) {
				System.out.println(c.getName());
				runMPEGlet(c);	
			}
		}
	}
	
	 public static void main(String[] args) {
     	if (args.length != 2) {
      		System.err.println("Usage: MPEGJClassLoader [classname|zip filename]  zip(0/1)");
      		System.exit(-1);
    	}
		byte [] data = MPEGJClassLoader.readFile(args[0]);
    	boolean zip = (Integer.parseInt(args[1]) == 1);
		if(zip)
			loadZipArchive(data);
	 }
}
  

