/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
/*
 * File: MPEGJEventMulticaster.java
 */

package org.iso.mpeg.mpegjRefImpl;

import java.util.*;
import org.iso.mpeg.mpegj.decoder.*;
import org.iso.mpeg.mpegj.resource.*;
import org.iso.mpeg.mpegjRefImpl.resource.*;
import org.iso.mpeg.mpegjRefImpl.decoder.*;

public class MPEGJEventMulticaster {
	
	/* queues for each decoder and renderer */
   	public static Vector decoderListeners;
  	public static RendererImp rendererListener; 
  
  	public MPEGJEventMulticaster( int num ) {
		// initialize event queues
		decoderListeners = new Vector(num);
		rendererListener = null;
  	}
  
  	public static void add(Renderer rend) {
		if ( rendererListener == null )
			rendererListener = (RendererImp)rend;
  	}

  	public static void remove(Renderer rend) {
		if ( rendererListener == rend )
			rendererListener = null;
  	}
  	
	public static void add(MPDecoder dec) {
		// need to find out what type of decoder is it and add it to that queue
		decoderListeners.addElement(dec);	
  	}

  	public static void remove(MPDecoder dec) {
		// need to find out what type of decoder is it and remove it from that queue
		decoderListeners.removeElement(dec);
  	}

  	public static void dispatchRendererEvents( MPRendererMediaEvents  myre) {
		rendererListener.localMPRendererMediaListener.mPCompositeMediaHandler(myre); 	
  	}
  
  	public static void dispatchFrameEvents( MPRendererFrameEvents  fe) {
		if (rendererListener != null && ++rendererListener.localMPRendererFrameCount >= rendererListener.localMPRendererFrameFrequency) {
	  		rendererListener.localMPRendererFrameListener.frameEvent(fe); 	
	  		rendererListener.localMPRendererFrameCount = 0;
		}
  	}

	// using ESID instead of OD
	// beucase there could be two different decoder attached to the same OD
	// but different ES
	public static MPDecoderImp getSource(long decoderESID) {
		int numList = decoderListeners.size();
		for ( int i=0; i< numList; i++ ) {	
			if ( ((MPDecoderImp)decoderListeners.elementAt(i)) != null )	
				if ( ((MPDecoderImp)decoderListeners.elementAt(i)).getESID() == decoderESID )
					return ((MPDecoderImp)decoderListeners.elementAt(i));
		}
		return null;
	}

	public static void dispatchDecoderEvents( MPDecoderMediaEvents  myde, long decoderESID) {
		int numList = decoderListeners.size();
		for (int i=0; i< numList; i++ ) {	
			if ( ((MPDecoderImp)decoderListeners.elementAt(i)) != null ) {
				if ( ((MPDecoderImp)decoderListeners.elementAt(i)).getESID() == decoderESID ) {
					((MPDecoderImp)decoderListeners.elementAt(i)).localMPDecoderMediaListener.mPDecoderMediaHandler(myde); 	

				}
			}
		}
	}
}
