/**********************************************************************
 * Copyright  2001 ISO/IEC All Rights Reserved                        
 *                                                                       
 * This software module was developed by 
 *                                                                       
 * Sun Microsystems, Inc.
 *                                                                       
 * in the course of development of the MPEG-4 standard. This software
 * module is an implementation of a part of one or more MPEG-4 Systems
 * (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems standard.
 *                                                                       
 * The copyright of this software belongs to ISO/IEC. ISO/IEC gives
 * users of the MPEG-4 Systems (ISO/IEC 14496-1) standard free license
 * to this software module or modifications thereof for hardware
 * or software products claiming conformance to the MPEG-4 Systems.
 *                                                                       
 * Those intending to use this software module in hardware or software   
 * products are advised that use may infringe existing  patents. The     
 * original developers of this software module and their companies, the  
 * subsequent editors and their companies, and ISO/IEC have no liability 
 * and ISO/IEC have no liability for use of this software module or      
 * modification thereof in an implementation.                            
 **********************************************************************/
/*
 * File: MPDecoderImp.java
 *
 * Author: Yihan Fang (yihan.fang@sun.com or fang@ee.columbia.edu)
 * 
 * Description: Implementation of MPDecoder interface
 *
 * Created: August, 1998
 *
 * Modified: March 2000, Jan 2001
 * By: 	Praveen Alavilli
 *    	Yuntai Kyong, Columbia University (yuntaikyong@ieee.org)		
 * Description: mpegjRefImpl package
 */
package org.iso.mpeg.mpegjRefImpl.decoder;
import org.iso.mpeg.mpegj.decoder.*;

import org.iso.mpeg.mpegj.resource.*;
import java.io.*;
import org.iso.mpeg.mpegj.*;
import org.iso.mpeg.mpegjRefImpl.*;

/** 
 * This is the implementation of MPDecoder interface
 */
public class MPDecoderImp implements MPDecoder
{
    private int nodeId;    
    
    public int type; 
    
    private int isAttached;
    
    private long odID; 
    private long esID; 
    private int decoderType; 
    
    public MPDecoderMediaListener localMPDecoderMediaListener  = null; 


    // native methods
    public native int getDecoderInstance ();
    public native int getDecoderType (int odid); 
    public native String getDecoderVendor ();
    public native void stopDecoderOnNode (long nodeId);
    public native void startDecoding ( long odID);
    public native void pauseDecoding();
	public native void resumeDecoding();
    public native int attachDecoder( int odID, int nodeID);
    public native int detachDecoder( int odID, int nodeID);
    public native int getDecoderTypeNat (long odid, long esid); 
	

    static {
		System.loadLibrary("mpdecoder");
    }

    /** 
   * in this constructor, set nodeId value and from nodeId, set decoder type
   */
    public MPDecoderImp(int nodeId  ) { 
	    this.nodeId = nodeId;	
	    switch(nodeId) {
	        case 69:
	        case 90:
	        case 17:
	            type = DecoderType.MPEG2V; // MPEG2 video decoder
	            break;
	        case 54:    // Face node
	        case 55:	// FaceDefMesh
	        case 56:	// FaceDefTables
	        case 57: 	// FaceDefTransform
	        case 58:	// FIT
	        case 59:    // FAP node
	            type = DecoderType.MPEG4F; // MPEG4 Face decoder
	            break;
	        case 4:      // AudioSource node
	        case 21:     // Sound node
	            type = DecoderType.MPEG2A; // MPEG2 audio decoder
	            break;
	        default: break;
	    }	
    }

    public MPDecoderImp( long  odid, long esid) {
        this.odID = odid;
        this.esID = esid;        
        decoderType = getDecoderTypeNat(odid, esid);       
    }    
   

	 /* added to get the OD of this decoder to throw events - praveen*/
	public long getOD() {
		return odID;
	} // praveen
		 /* added to get the OD of this decoder to throw events - praveen*/
	public long getESID() {
		return esID;
	} 
	public int getDecoderType() {
		return decoderType;
	} // praveen


    /**  Start decoding of data. This method is not supposed to be
   * called for normal decoding. Because normal decoding starts
   * automatically from the underlying BIFS construct */
    public void start(){
		startDecoding(odID);	
    }
	
    /** 
   * Stop decoding of data.
   */
    public void stop(){
	    System.out.println("MPDecoderImp.java: type is " +type);
	    System.out.println("MPDecoderImp.java: nodeId is " +nodeId);
	    stopDecoderOnNode(odID); //praveen
	    //    if (type==DecoderType.MPEG4F) {
	    // System.out.println("MPDecoderImp.java:found face decoder type in stop()");
	    //      stopFaceDecoder(nodeId);
	    // }
    }

    /** Determines if the decoder is pauseable 
     */
    public boolean isPauseable()
    {
	/* not supported yet */
	return false;
    }

    /** 
   * Pause decoding of data.
   */
    public void pause(){
	/* not yet supported */
	//pauseDecoding();
    }

    /** 
   * Resume decoding of data from paused instant.
   */
    public void resume(){
	/* not yet supported */
	resumeDecoding();
    }

    /**
   * Attach a decoder for decoding of data.
   */
    public void attach(int esId) 
    {
		isAttached = 1;
		attachDecoder( (int)odID,(int) esId);
    }

    /**
   * Detach a decoder already decoding data.
   */
    public void detach() {
		detachDecoder((int)odID,(int)esID);
    }

    /**
   * Get Node ID associated with this decoder.
   */
    public int getNodeId(){
		return nodeId;
    }

    /**
   * Get decoder type.
   */
    public DecoderType getType() //throws InvalidDecoderTypeException
    {
		return (new DecoderTypeImp(type));
    }

    /**
   * Get decoder vendor name.
   */
    public String getVendor(){
		//return getDecoderVendor();
		return (new String("Not Supported."));
    }

    /**
   * Get instance number.
   */
    public int getInstance(){
		/* not yet supported */
		//return getDecoderInstance();
		return -1;
    }
    
    public void addMPDecoderMediaListener(MPDecoderMediaListener l) {	
		MPEGJEventMulticaster.add((MPDecoder)this);    
		localMPDecoderMediaListener= l;    
    }

    public void removeMPDecoderMediaListener(MPDecoderMediaListener l) {
		if( localMPDecoderMediaListener== l ) {
			MPEGJEventMulticaster.remove(this);
			localMPDecoderMediaListener= null; 
		}
	}
  
  // *** YUNTAI START
  // implements getESDescriptor
  public ES_Descriptor getESDescriptor() {
  	return new ES_DescriptorImp(esID);
  }
   // *** YUNTAI END
  
}

