/********************************************************************
 This software module was originally developed by Roberto Becchini
 (SIA - I - SOMMIT Consortium), Barbara Silano (SIA - I - SOMMIT 
Consortium) and edited by Luca Deri (Finsiel - I - SOMMIT Consortium),
 Gianluca De Petris (CSELT - I - SOMMIT Consortium) in the course of 
development of the MPEG-4 Systems (ISO/IEC 14496-1). This software 
module is an implementation of part of one or more MPEG-4 Systems 
(ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 
14496-1). ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) 
free license to this software module or modifications thereof for use 
in hardware or software products claiming conformance to the MPEG-4 
Systems (ISO/IEC 14496-1). Those intending to use this software 
module in hardware or software products are advised that its use may 
infringe existing patents. THE ORIGINAL DEVELOPER OF THIS SOFTWARE 
MODULE AND HIS/HER COMPANY, THE SUBSEQUENT EDITORS AND THEIR 
COMPANIES, AND ISO/IEC HAVE NO LIABILITY FOR USE OF THIS SOFTWARE 
MODULE OR MODIFICATIONS THEREOF. No license to this software module 
is granted for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. The SOMMIT Consortium retains full right to use the 
software module for his/her own purpose, assign or donate the 
software module to a third party and to inhibit third parties from 
using the code for non MPEG-4 Systems (ISO/IEC 14496-1) conforming 
products. This copyright notice must be included in all copies or 
derivative works of the software module. Copyright 1999
********************************************************************/
package org.iso.mpeg.mpegjRefImpl.net;
import org.iso.mpeg.mpegj.net.*;
import java.util.Vector;
import java.io.*;

public class DMIFMonitorImp implements DMIFMonitor {

	public DMIFMonitorImp() {        
	}



	private Vector sessionInfo = new Vector();	
	private Vector channelsInfo = new Vector();

	private native int    	getChannelIDNat(int pos);
	private native long		getSessionIDNat();
	private native int		getnumOfChannelsNat();
	private native String	getURLNat();
	private native int      getAUrateNat(long channelID);
	private native int		getStatusNat(long channelID);
	private native int		getErrNat(long channelID);
	private native long		getODIDNat(long channelID); // get the Object descriptor ID associated with this stream


	public long getODID(long channelID) {
		return getODIDNat(channelID);
	}

	static {

		System.loadLibrary("DMIFMonitorLib");
	}

	private long	getSessionID() {
		return getSessionIDNat();
	}
	private int flag = 0; 
	private int		checkSessionID(long sessionID) {
		if(flag == 0) {
			flag = 1;
			return NOTFOUND;
		}
		return FOUND;
	}

	private int getAUrate(long sessionID, long channelID) {
		int temp;

		temp = getAUrateNat(channelID);

		return (temp);
	}

	private int	getnumOfChannels(long sessionID) {     
		return getnumOfChannelsNat();
	}
	private String  getURL(long sessionID) {    
		return getURLNat();	
	}

	private long   getChannelID(long sessionID, int nchan) {
		return getChannelIDNat(nchan);
	}

	private int		getStatus(long sessionID, long channelID) {
		return getStatusNat(channelID);
	}

	private int		getErr(long sessionID, long channelID) {
		return getErrNat(channelID);
	}

	private QoSData getQoSData(long sessionID, long channelID) {
		int i = 0;
		QoSData qos = new QoSData();
		Qualifier qualif;

		qos.qualifierCount = 8;

		QualifierTag qTag = new QualifierTag();

		qualif = new Qualifier();
		qualif.metric = qTag.PRIORITY; 
		qualif.tagValue = new Integer(2);
		qos.qualifiers.addElement(qualif);

		qualif = new Qualifier();
		qualif.metric = qTag.MAX_DELAY; 
		qualif.tagValue = new Integer(100);
		qos.qualifiers.addElement(qualif);

		qualif = new Qualifier();
		qualif.metric = qTag.AVG_DELAY;
		qualif.tagValue = new Integer(90);
		qos.qualifiers.addElement(qualif);

			qualif = new Qualifier();
		qualif.metric = qTag.LOSS_PROB;
		qualif.tagValue = new Double(5/100);
		qos.qualifiers.addElement(qualif);

		qualif = new Qualifier();
		qualif.metric = qTag.MAX_GAP_LOSS;
		qualif.tagValue = new Integer(6);
		qos.qualifiers.addElement(qualif);

		qualif = new Qualifier();
		qualif.metric = qTag.MAX_AU_SIZE;
		qualif.tagValue = new Integer(512);
		qos.qualifiers.addElement(qualif);

		qualif = new Qualifier();
		qualif.metric = qTag.MAX_AU_RATE;
		qualif.tagValue = new Integer(500);
		qos.qualifiers.addElement(qualif);

		qualif = new Qualifier();
		qualif.metric = qTag.AVG_AU_SIZE;
		qualif.tagValue = new Integer(512);
		qos.qualifiers.addElement(qualif);

		return qos;

	}

	private static final int   FOUND	= 0;
	private static final int   NOTFOUND	= 1;

	/**									
	* This method returns a Vector of object SessionDescriptors that contains information
	*
	* about the existing sessions. The returned value refers to entities that may change over the time.
	*/
	public Vector getServiceSessionDescriptors() { 
		// to be implemented
		return getServiceSessionsInfos();
	}

	/**									
	* This method returns a Vector of object SessionDescriptors that contains information
	*
	* about the existing sessions.
	*/
	public Vector getServiceSessionsInfos() {

		int							i		=0;
		int							nchan	=0;
		long						sID		=0;
		ServiceSessionDescriptorImp	sDescriptor;
		ChannelsDescriptor       cDescriptor;
		String						stringURL;
		int err=0;


		if((err = checkSessionID(sID)) == NOTFOUND) {
			if((sDescriptor = new ServiceSessionDescriptorImp(sID)) == null) {
				return null;
			}
			sID			= getSessionID();
			stringURL	= new String(getURL(sID));
			if(stringURL == null) {
				return null;
			}
			sDescriptor.url = stringURL;

			sessionInfo.addElement(sDescriptor);
		}

		for( i = 0; i < sessionInfo.size(); i++) {
			ServiceSessionDescriptorImp myobject	= (ServiceSessionDescriptorImp)sessionInfo.elementAt(i);
			myobject.numberOfChannels	= getnumOfChannels(sID);
			sessionInfo.setElementAt(myobject,i);			
		}

		return sessionInfo;
	}

	/**					
	* This method returns a Vector of object ChannelsDescriptors that contains informations about the
	*
	* channels existing in a specified session. The returned value refers to entities that may change over the time.
	*/
	public Vector getChannelDescriptors(long serviceSessionID) {
		// to be implemented  
		return getChannelsInfos(serviceSessionID);
	}


	/**					
	* This method returns a Vector of object ChannelsDescriptors that contains informations about the
	*
	* channels existing in a specified session.
	*/
	public Vector getChannelsInfos(long serviceSessionID) {
		int			i = 0;
		int			nchan = 0;
		long		cID;
		try {
			nchan = ((ServiceSessionDescriptorImp)sessionInfo.elementAt(0)).numberOfChannels;
		} catch(Exception ex) {
		}

		if(channelsInfo.size() < nchan) {	
			for(i = channelsInfo.size(); i < nchan; i++) {
				ChannelsDescriptor cdesc = new ChannelsDescriptor( serviceSessionID);
				cID	= getChannelID(serviceSessionID, i);
				cdesc.channelId = cID; 
				cdesc.status	= getStatus(serviceSessionID, cID);
				cdesc.qos		= getQoSData(serviceSessionID, cID);

				cdesc.auRate	= getAUrate(serviceSessionID, cID);

				cdesc.error		= getErr(serviceSessionID, cID);
				channelsInfo.addElement(cdesc);
			}					
		}
		else {	

			for(i = 0; i < nchan; i++) {
				cID	= getChannelID(serviceSessionID, i);
				ChannelsDescriptor mycobject = (ChannelsDescriptor)channelsInfo.elementAt(i); 
				mycobject.channelId = cID; 
				mycobject.status	= getStatus(serviceSessionID, cID);
				mycobject.auRate	= getAUrate(serviceSessionID, cID);
				mycobject.error		= getErr(serviceSessionID, cID);
				channelsInfo.setElementAt(mycobject, i);
			}
		}		
		return channelsInfo;
	}

}


