/***********************************************************************
This software module was originally developed by Zvi Lifshitz
(Triton R&D Ltd.) in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Triton R&D Ltd. retains full right
to use the code for his/her own purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
************************************************************************/
/*
  $Id: NativeScene.java,v 1.1.1.1 2003/12/24 00:52:57 mikeseven Exp $
*/

package org.iso.mpeg.mpegjRefImpl.scene;
import org.iso.mpeg.mpegj.scene.*;
import java.util.*;
import java.io.*;

import org.iso.mpeg.mpegjRefImpl.*;

public class NativeScene implements Scene, SceneManager
{
	static
	{
		System.loadLibrary( "mpjscene" );
	}

	static NativeScene s_instance = null;
	static int s_handle = 0;

	static Vector s_listeners = new Vector(); // SceneListener
	static Hashtable s_nodes = new Hashtable();         // NativeNode[Integer]

	public static NativeScene getSceneManager()
	{
	    System.out.println( "NativeScene getSceneManager(), s_instance = " + s_instance );
	    if ( s_instance == null )
	    {
	        System.out.println( "new NativeScene()" );
	        s_instance = new NativeScene( 666 );
	        System.out.println( "native scene = " + s_instance );
	    }
	    return s_instance;
	}

	public NativeScene( int val )
	{
	    setSceneManager( this );
		s_handle = getScene();
		// System.out.println("Value of s_handle:" + s_handle);

	}

    // from Scene interface
	public Node getNode( int defID ) throws BadParameterException
	{

		NativeNode n = new NativeNode( defID, this );
	    s_nodes.put( new Integer(n.getHandle()), n );
		return n;
	}

    public Node getNode( String defName ) throws BadParameterException, InvalidSceneException
	{
		System.out.println(defName);
		int defID = getNodeIDFromName( getHandle(), defName );
		return getNode( defID );
	}

    public void insertRoute( Node fromNode, int fromID, Node toNode, int toID ) throws BadParameterException, InvalidSceneException
	{
		int fromHandle = ((NativeNode) fromNode).getHandle();
		int toHandle = ((NativeNode) toNode).getHandle();
		modifyRouteTo( fromHandle, fromID, toHandle, toID, 1 );
	}

    public void deleteRoute( Node fromNode, int fromID, Node toNode, int toID ) throws BadParameterException, InvalidSceneException
	{
		int fromHandle = ((NativeNode) fromNode).getHandle();
		int toHandle = ((NativeNode) toNode).getHandle();
		modifyRouteTo( fromHandle, fromID, toHandle, toID, 0 );
	}

    // from SceneManager interface
	public void addSceneListener( SceneListener listener )
	{
	    s_listeners.addElement( listener );
	    if ( s_handle != 0 )
	    {
	        listener.notify( SceneListener.Message.SCENE_READY, this );
	    }
	}

	public void unregister()
	{
	    setSceneManager( null );
	}

	public void removeSceneListener( SceneListener listener ) throws BadParameterException
	{
	    s_listeners.removeElement( listener );
	}

    // notifications from native code
	public static int sceneEventNotification( int sceneHandle, int whatHappened )
	{
		if( s_handle == 0 )
		{
			s_handle = sceneHandle;
		}
		System.out.println("In sceneEventNotification sceneHandle:" + sceneHandle + " s_handle:" + s_handle );
	    if ( s_instance == null )
	    {
	        System.out.println( "new NativeScene()" );
	        s_instance = new NativeScene( 666 );
	        System.out.println( "native scene = " + s_instance );
	    }
	    if ( sceneHandle == s_handle )
	    {
	        for ( Enumeration i = s_listeners.elements(); i.hasMoreElements(); )
	        {
	            SceneListener listener = (SceneListener) i.nextElement();
	            //listener.notify( whatHappened, this );
	            listener.notify( SceneListener.Message.SCENE_READY, s_instance );
	        }
	    }
	    return 0;
	}

	public static int eventOutNotification( int nodeHandle, int eventOutID, FieldValue changedFieldValue )
	{
	    System.out.println( "#### eventOutNotification ####" );
	    NativeNode n = (NativeNode) s_nodes.get( new Integer(nodeHandle) );
	    if ( n != null )
	    {
	        n.eventOutNotification( eventOutID, changedFieldValue );
	    }
	    return 0;
	}

    // helpers
	public static void setHandle( int sceneNumber, int handle )
	{
		s_handle = handle;
	}

	int getHandle()
	{

		return s_handle;
	}

	native void setSceneManager( NativeScene mgr );
	native int getScene();

	native int getNodeIDFromName( int sceneHandle, String defName );
	native void modifyRouteTo( int fromHandle, int fromID, int toHandle, int toID, int insert );
}