/*
This software module was originally developed by Five Bats Research
in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Five Bats Research retains full right
to use the code for its purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
*/
#include "FieldValueMaker.h"
#include <string.h>
#include <stdlib.h>
#include <assert.h>

FieldValueMaker* FieldValueMaker::s_self = 0;
jclass FieldValueMaker::s_classTypes[ MPJ_FIELDVALUE_TYPE_COUNT ];
jclass FieldValueMaker::s_MFFields[ MPJ_FIELDVALUE_TYPE_COUNT ];
jclass FieldValueMaker::s_SFFields[ MPJ_FIELDVALUE_TYPE_COUNT ];
char* FieldValueMaker::s_MFFieldValueClassNames[ MPJ_FIELDVALUE_TYPE_COUNT ];

#define TESTEXCEPTION \
	if ( env->ExceptionOccurred() )	throw MPJException(MPJUnspecifiedErr)

int FieldValueMaker::setFieldValue( JNIEnv *env, jobject val, FieldValuePtr f )
{
	//if ( s_self == 0 )
		s_self = new FieldValueMaker( env );
	return s_self->doSetFieldValue( env, val, f );
}

void FieldValueMaker::setTableFor( JNIEnv *env, int idx, const char *fieldType )
{
	char buf[ 256 ];

	sprintf( buf, "org/iso/mpeg/mpegjRefImpl/scene/SF%sImp", fieldType );
	s_SFFields[ idx ] = env->FindClass( buf ); TESTEXCEPTION;
	sprintf( buf, "org/iso/mpeg/mpegjRefImpl/scene/MF%sImp", fieldType );
	s_MFFields[ idx ] = env->FindClass( buf ); TESTEXCEPTION;
	sprintf( buf, "[Lorg/iso/mpeg/mpegj/scene/SF%sFieldValue;", fieldType );
	s_MFFieldValueClassNames[ idx ] = (char *) malloc( strlen(buf) + 1 );
	strcpy( s_MFFieldValueClassNames[idx], buf ); 
	sprintf( buf, "org/iso/mpeg/mpegj/scene/SF%sFieldValue", fieldType );
	s_classTypes[ idx ] = env->FindClass( buf ); TESTEXCEPTION;
}

FieldValueMaker::FieldValueMaker( JNIEnv *env )
{
	setFields( env );
}
void FieldValueMaker::setFields( JNIEnv *env )
{
	int idx = MPJ_UNINITIALIZED_FIELDVALUE + 1;	

	/* number and order are from the struct in MPJSceneInterface.h */
	setTableFor( env, idx++, "Bool" );
	setTableFor( env, idx++, "Color" );
	setTableFor( env, idx++, "Float" );
	setTableFor( env, idx++, "Int32" );
	setTableFor( env, idx++, "Node" );
	setTableFor( env, idx++, "Rotation" );
	setTableFor( env, idx++, "String" );
	setTableFor( env, idx++, "Time" );
	setTableFor( env, idx++, "Vec2f" );
	setTableFor( env, idx++, "Vec3f" );
	assert( idx == MPJ_FIELDVALUE_TYPE_COUNT );
}

int FieldValueMaker::getFieldValueType( JNIEnv *env, jobject val )
{
	int idx = -1;
	setFields( env );
	for ( int i = MPJ_UNINITIALIZED_FIELDVALUE + 1; i < MPJ_FIELDVALUE_TYPE_COUNT; i++ )
	{
		if ( env->IsInstanceOf(val, s_classTypes[i]) == JNI_TRUE )
		{
			idx = i;
			break;
		}
		TESTEXCEPTION;
	}
	return idx;
}

void FieldValueMaker::freeFieldValueContent( FieldValuePtr f )
{
	if ( (f->isMFField == 0) )
	{
		if ( (f->fieldValueType == MPJ_SFSTRING_TYPE) )
		{
			::free( f->val.stringVal );
		}
		return;
	}
	for( int i = 0; i < f->fieldValueCount; i++ )
	{
		FieldValue item = f->val.mfValue[i];
		if ( (f->fieldValueType == MPJ_SFSTRING_TYPE) )
		{
			::free( item.val.stringVal );
		}
	}
	::free( f->val.mfValue );
}


int FieldValueMaker::doSetFieldValue( JNIEnv *env, jobject val, FieldValuePtr f )
{
	int ret = MPJNoErr;
	int myType = getFieldValueType( env, val );
	// Add code to get the event field type
	// We are changing the logic here. The fieldValueType
	// would be populated before this method is called.
	// If not, we will use the previous logic of interface
	// implementation
	if( f->fieldValueType != MPJ_UNINITIALIZED_FIELDVALUE )
	{
		myType = f->fieldValueType;
		f->fieldValueType = MPJ_UNINITIALIZED_FIELDVALUE;
	}
	freeFieldValueContent( f );
	// Reset the value of f->fieldValueType
	f->fieldValueType = myType;

	f->fieldValueCount = 1;
	jmethodID get;
	jfloatArray floatArray;
	jboolean isCopy;

	new FieldValueMaker( env ); //--awm hack

	switch( myType )
	{
		case MPJ_SFBOOL_TYPE:
			get = env->GetMethodID( s_classTypes[myType], "getSFBoolValue", "()Z" ); TESTEXCEPTION;
			f->val.boolVal = env->CallBooleanMethod(val, get) ? true : false; TESTEXCEPTION;
			break;

		case MPJ_SFFLOAT_TYPE:
			get = env->GetMethodID( s_classTypes[myType], "getSFFloatValue", "()F" ); TESTEXCEPTION;
			f->val.floatVal = env->CallFloatMethod( val, get ); TESTEXCEPTION;
			break;

		case MPJ_SFINT32_TYPE:
			get = env->GetMethodID( s_classTypes[myType], "getSFInt32Value", "()I" ); TESTEXCEPTION;
			f->val.int32Val = env->CallIntMethod( val, get ); TESTEXCEPTION;
			break;
			
		case MPJ_SFTIME_TYPE:
			get = env->GetMethodID( s_classTypes[myType], "getSFTimeValue", "()D" ); TESTEXCEPTION;
			f->val.timeVal = env->CallDoubleMethod( val, get ); TESTEXCEPTION;
			break;
			
		case MPJ_SFCOLOR_TYPE:
			get = env->GetMethodID( s_classTypes[myType], "getSFColorValue", "()[F" ); TESTEXCEPTION;
			floatArray = (jfloatArray) env->CallObjectMethod( val, get ); TESTEXCEPTION;
			env->GetFloatArrayRegion( floatArray, 0, 3, f->val.colorVal ); TESTEXCEPTION;
			break;

		case MPJ_SFVEC3F_TYPE:
			get = env->GetMethodID( s_classTypes[myType], "getSFVec3fValue", "()[F" ); TESTEXCEPTION;
			floatArray = (jfloatArray) env->CallObjectMethod( val, get ); TESTEXCEPTION;
			env->GetFloatArrayRegion( floatArray, 0, 3, f->val.vec3fVal ); TESTEXCEPTION;
			break;

		case MPJ_SFVEC2F_TYPE:
			get = env->GetMethodID( s_classTypes[myType], "getSFVec2fValue", "()[F" ); TESTEXCEPTION;
			floatArray = (jfloatArray) env->CallObjectMethod( val, get ); TESTEXCEPTION;
			env->GetFloatArrayRegion( floatArray, 0, 2, f->val.vec2fVal ); TESTEXCEPTION;
			break;

		case MPJ_SFROTATION_TYPE:
			get = env->GetMethodID( s_classTypes[myType], "getSFRotationValue", "()[F" ); TESTEXCEPTION;
			floatArray = (jfloatArray) env->CallObjectMethod( val, get ); TESTEXCEPTION;
			env->GetFloatArrayRegion( floatArray, 0, 4, f->val.rotationVal ); TESTEXCEPTION;
			break;

		case MPJ_SFSTRING_TYPE:
			{
				get = env->GetMethodID( s_classTypes[myType], "getSFStringValue", "()Ljava/lang/String;" ); TESTEXCEPTION;
				jstring js = (jstring) env->CallObjectMethod( val, get ); TESTEXCEPTION;
				const char *s = env->GetStringUTFChars( js, &isCopy ); TESTEXCEPTION;
				// Changing the logic here.. 
				// Making the field value to be a MF instead of SF
				//f->val.stringVal = (char *) calloc( 1, strlen(s)+1 ); // we use malloc, since API says OK to free this
				//strcpy( f->val.stringVal, s );
				f->isMFField = 1; // Indicate that this a MF field
				// Allocate memory for the mfValue
				f->val.mfValue = (FieldValuePtr) ::calloc( 1, sizeof(FieldValue) );
				f->val.mfValue[0].val.stringVal = (char *) calloc( 1, strlen(s)+1 ); // we use malloc, since API says OK to free this
				strcpy( f->val.mfValue[0].val.stringVal, s );
				if ( isCopy == JNI_TRUE )
				{
					env->ReleaseStringUTFChars( js, s ); TESTEXCEPTION;
				}
			}
			break;

		case MPJ_SFNODE_TYPE:
			{
				get = env->GetMethodID( s_classTypes[myType], "getSFNodeValue", "()Lorg/iso/mpeg/mpegj/scene/NodeValue;" ); TESTEXCEPTION;
				jobject jnode = (jstring) env->CallObjectMethod( val, get ); TESTEXCEPTION;

				if ( jnode == 0 )
				{
					// field value is NULL
					f->val.nodeVal.kind = MPJ_NATIVE_NODE;
					f->val.nodeVal.node.nativeNode.handle = 0;
					f->val.nodeVal.node.nativeNode.id = 0;
					break;
				}

				jclass clsNative = env->FindClass( "org/iso/mpeg/mpegjRefImpl/scene/PrivateNode" ); TESTEXCEPTION;
				jboolean isNative = env->IsInstanceOf( jnode, clsNative ); TESTEXCEPTION;
				if ( isNative )
				{
					// field value is private (i.e. it's from the native scene), possibly non-DEFed, or DEFed
					jfieldID fid;
					fid = env->GetFieldID( clsNative, "m_id", "I" ); TESTEXCEPTION;
					jint id = env->GetIntField( jnode, fid ); TESTEXCEPTION;
					fid = env->GetFieldID( clsNative, "m_handle", "I" ); TESTEXCEPTION;
					jint handle = env->GetIntField( jnode, fid ); TESTEXCEPTION;

					f->val.nodeVal.kind = MPJ_NATIVE_NODE;
					f->val.nodeVal.node.nativeNode.handle = handle;
					f->val.nodeVal.node.nativeNode.id = id;
					break;
				}

				jclass clsNew = env->FindClass( "org/iso/mpeg/mpegj/scene/NewNode" ); TESTEXCEPTION;
				jboolean isNew = env->IsInstanceOf( jnode, clsNew ); TESTEXCEPTION;
				if ( isNew )
				{
					// field value is a new node object
					jmethodID mid;
					mid = env->GetMethodID( clsNew, "getNodeType", "()I" ); TESTEXCEPTION;
					jint type = env->CallIntMethod( jnode, mid ); TESTEXCEPTION;
					mid = env->GetMethodID( clsNew, "getNodeID", "()I" ); TESTEXCEPTION;
					jint id = env->CallIntMethod( jnode, mid ); TESTEXCEPTION;

					f->val.nodeVal.kind = MPJ_NEW_NODE;
					f->val.nodeVal.node.newNode.type = type;
					f->val.nodeVal.node.newNode.id = id;
					f->val.nodeVal.node.newNode.param1 = (int) env;
					f->val.nodeVal.node.newNode.param2 = (int) jnode;
					break;
				}

				ret = MPJBadParamErr; // oops
			}
			break;

	}
	return ret;
}

jobject FieldValueMaker::asJavaFieldValue( JNIEnv *env, FieldValuePtr val )
{
	jobject ret = 0;
	jfieldID fid;
	jmethodID mid;
	jclass cls;
	//if ( s_self == 0 ) //--awm hack
		s_self = new FieldValueMaker( env );
	if ( val->isMFField )
	{
		cls = s_MFFields[ val->fieldValueType ];
		mid = env->GetMethodID( cls, "<init>", "()V" ); TESTEXCEPTION;
		ret = env->NewObject( cls, mid ); TESTEXCEPTION;
		fid  = env->GetFieldID( cls, "m_value", s_MFFieldValueClassNames[val->fieldValueType] ); TESTEXCEPTION;
		jobjectArray values = env->NewObjectArray( val->fieldValueCount, s_SFFields[val->fieldValueType], NULL ); TESTEXCEPTION;
		for ( int i = 0; i < val->fieldValueCount; i++ )
		{
			jobject item = asJavaFieldValue( env, &(val->val.mfValue[i]) );
			env->SetObjectArrayElement( values, i, item ); TESTEXCEPTION;
			env->DeleteLocalRef( item ); TESTEXCEPTION;
		}
		env->SetObjectField( ret, fid, values ); TESTEXCEPTION;
	}
	else
	{
		jclass cls = s_SFFields[ val->fieldValueType ];
		if ( cls )
		{
			jmethodID mid = env->GetMethodID( cls, "<init>", "()V" ); TESTEXCEPTION;
			ret = env->NewObject( cls, mid ); TESTEXCEPTION;
			switch ( val->fieldValueType )
			{
			case MPJ_SFBOOL_TYPE:
				{
					fid  = env->GetFieldID( cls, "m_value", "Z" ); TESTEXCEPTION;
					env->SetBooleanField( ret, fid, val->val.boolVal ); TESTEXCEPTION;
				}
				break;
				
			case MPJ_SFFLOAT_TYPE:
				{
					fid  = env->GetFieldID( cls, "m_value", "F" ); TESTEXCEPTION;
					env->SetFloatField( ret, fid, val->val.floatVal ); TESTEXCEPTION;
				}
				break;
				
			case MPJ_SFINT32_TYPE:
				{
					fid  = env->GetFieldID( cls, "m_value", "I" ); TESTEXCEPTION;
					env->SetIntField( ret, fid, val->val.int32Val ); TESTEXCEPTION;
				}
				break;
				
			case MPJ_SFTIME_TYPE:
				{
					fid  = env->GetFieldID( cls, "m_value", "D" ); TESTEXCEPTION;
					env->SetDoubleField( ret, fid, val->val.timeVal ); TESTEXCEPTION;
				}
				break;
				
			case MPJ_SFCOLOR_TYPE:
				{
					fid  = env->GetFieldID( cls, "m_value", "[F" ); TESTEXCEPTION;
					jfloatArray arr = env->NewFloatArray( 3 ); TESTEXCEPTION;
					env->SetFloatArrayRegion( arr, 0, 3, val->val.colorVal ); TESTEXCEPTION;
					env->SetObjectField( ret, fid, arr ); TESTEXCEPTION;
				}
				break;
				
			case MPJ_SFVEC3F_TYPE:
				{
					fid  = env->GetFieldID( cls, "m_value", "[F" ); TESTEXCEPTION;
					jfloatArray arr = env->NewFloatArray( 3 ); TESTEXCEPTION;
					env->SetFloatArrayRegion( arr, 0, 3, val->val.vec3fVal ); TESTEXCEPTION;
					env->SetObjectField( ret, fid, arr ); TESTEXCEPTION;
				}
				break;
				
			case MPJ_SFVEC2F_TYPE:
				{
					fid  = env->GetFieldID( cls, "m_value", "[F" ); TESTEXCEPTION;
					jfloatArray arr = env->NewFloatArray( 2 ); TESTEXCEPTION;
					env->SetFloatArrayRegion( arr, 0, 2, val->val.vec2fVal ); TESTEXCEPTION;
					env->SetObjectField( ret, fid, arr ); TESTEXCEPTION;
				}
				break;
				
			case MPJ_SFROTATION_TYPE:
				{
					fid  = env->GetFieldID( cls, "m_value", "[F" ); TESTEXCEPTION;
					jfloatArray arr = env->NewFloatArray( 4 ); TESTEXCEPTION;
					env->SetFloatArrayRegion( arr, 0, 4, val->val.rotationVal ); TESTEXCEPTION;
					env->SetObjectField( ret, fid, arr ); TESTEXCEPTION;
				}
				break;
				
			case MPJ_SFSTRING_TYPE:
				{
					fid  = env->GetFieldID( cls, "m_value", "Ljava/lang/String;" ); TESTEXCEPTION;
					jstring s = env->NewStringUTF( val->val.stringVal ); TESTEXCEPTION;
					env->SetObjectField( ret, fid, s ); TESTEXCEPTION;
				}
				break;
				
			case MPJ_SFNODE_TYPE:
				{
					jobject node;
					if ( val->val.nodeVal.kind != MPJ_NATIVE_NODE )
					{
						// oops!
						node = 0;
					}
					else if ( val->val.nodeVal.node.nativeNode.handle == 0 )
					{
						// null node
						node = 0;
					}
					else if ( val->val.nodeVal.node.nativeNode.id == 0 )
					{
						// private non-DEFed node
						jclass nodeCls = env->FindClass( "org/iso/mpeg/mpegjRefImpl/scene/PrivateNode" ); TESTEXCEPTION;
						jmethodID nodeCtor = env->GetMethodID( nodeCls, "<init>", "(II)V" ); TESTEXCEPTION;
						node = env->NewObject( nodeCls, nodeCtor, val->val.nodeVal.node.nativeNode.id, val->val.nodeVal.node.nativeNode.handle ); TESTEXCEPTION;
					}
					else
					{
						// private DEFed node
						jclass nodeCls = env->FindClass( "org/iso/mpeg/mpegjRefImpl/scene/NativeNode" ); TESTEXCEPTION;
						jmethodID nodeCtor = env->GetMethodID( nodeCls, "<init>", "(II)V" ); TESTEXCEPTION;
						node = env->NewObject( nodeCls, nodeCtor, val->val.nodeVal.node.nativeNode.id, val->val.nodeVal.node.nativeNode.handle ); TESTEXCEPTION;
					}

					fid  = env->GetFieldID( cls, "m_value", "Lorg/iso/mpeg/mpegj/scene/NodeValue;" ); TESTEXCEPTION;
					env->SetObjectField( ret, fid, node ); TESTEXCEPTION;
				}
				break;
			}
		}
	}
	return ret;
}
