/*
This software module was originally developed by Five Bats Research
in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Five Bats Research retains full right
to use the code for its purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
*/
#include "../mpegj-player/MPJSceneInterface.h"
#include "NativeNode.h"
#include "FieldValueMaker.h"

JNIEXPORT jint JNICALL
Java_org_iso_mpeg_mpegjRefImpl_scene_NativeNode_getNode(JNIEnv *env, 
										   jobject self, 
										   jint sceneHandle, 
										   jint nodeID )
{
	jint ret = 0;

	if ( sceneHandle )
	{
		MPJSceneDispatchTablePtr t = MPJGetSceneDispatchTable();
		if ( t )
		{
			int nodeHandle;
			int err = t->findNode( sceneHandle, nodeID, &nodeHandle );
			if ( err == 0 )
				ret = nodeHandle;
		}
	}
	return ret;
}

JNIEXPORT void JNICALL
Java_org_iso_mpeg_mpegjRefImpl_scene_NativeNode_doSendEventIn(JNIEnv *env, 
												 jobject self, 
												 jint nodeHandle, 
												 jint inID, 
												 jobject event )
{
	try
	{
		MPJSceneDispatchTablePtr t = MPJGetSceneDispatchTable();
		if ( t )
		{
			FieldValue val;
			val.fieldValueCount = 1;
			val.fieldValueType = MPJ_UNINITIALIZED_FIELDVALUE;
			val.isMFField = 0;
			
			// Add code to assign the field type
			int eventType = MPJ_UNINITIALIZED_FIELDVALUE;
			int err1 = t->getEventType( nodeHandle, inID, &eventType );
			if( err1 == MPJNoErr)
			{
				val.fieldValueType = eventType;
			}
			int err = FieldValueMaker::setFieldValue( env, event, &val );
			if ( err == MPJNoErr )
			{
				err = t->sendEventIn( nodeHandle, inID, &val );
				FieldValueMaker::freeFieldValueContent( &val );
			}
		}
	}
	catch ( MPJException& /*e*/ )
	{
	}	
}

JNIEXPORT jobject JNICALL 
Java_org_iso_mpeg_mpegjRefImpl_scene_NativeNode_doGetEventOut(JNIEnv *env, 
												 jobject self, 
												 jint nodeHandle, 
												 jint outID )
{
	jobject ret = 0;
	try
	{
		MPJSceneDispatchTablePtr t = MPJGetSceneDispatchTable();
		if ( t )
		{
			FieldValue val;
			val.fieldValueCount = 1;
			val.fieldValueType = MPJ_UNINITIALIZED_FIELDVALUE;
			val.isMFField = 0;
			int err = t->readEventOut( nodeHandle, outID, &val );
			if ( err == MPJNoErr )
			{
				ret = FieldValueMaker::asJavaFieldValue( env, &val );
			}
		}
	}
	catch ( MPJException& /*e*/ )
	{
	}
	return ret;
}

JNIEXPORT void JNICALL 
Java_org_iso_mpeg_mpegjRefImpl_scene_NativeNode_doAddEventOutListener(JNIEnv *env, 
														 jobject self,
														 jint nodeHandle, 
														 jint outID, 
														 jobject listener )
{
	try
	{
		MPJSceneDispatchTablePtr t = MPJGetSceneDispatchTable();
		if ( t && t->startListenEventOut )
		{
			t->startListenEventOut(nodeHandle, outID);
		}
	}
	catch ( MPJException& /*e*/ )
	{
	}
}

JNIEXPORT jint JNICALL 
Java_org_iso_mpeg_mpegjRefImpl_scene_NativeNode_doGetNodeType(JNIEnv *env, 
														 jobject self,
														 jint nodeHandle )
{
	jint ret = 0;
	try
	{
		MPJSceneDispatchTablePtr t = MPJGetSceneDispatchTable();
		if ( t && t->getNodeType )
		{
			int nodeType;
			int err = t->getNodeType(nodeHandle, &nodeType);
			if ( err == MPJNoErr )
			{
				ret = nodeType;
			}
		}
	}
	catch ( MPJException& /*e*/ )
	{
	}
	return (ret-1);// *** IMPORTANT - TEMP CHANGE subtracting 1
}
