/*
This software module was originally developed by Five Bats Research
in the course of development of the MPEG-4 Systems
(ISO/IEC 14496-1) standard. This software module is an implementation of
a part of one or more MPEG-4 Systems (ISO/IEC 14496-1) tools as
specified by the MPEG-4 Systems (ISO/IEC 14496-1) standard. ISO/IEC
gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license to
this software module or modifications thereof for use in hardware or
software products claiming conformance to the MPEG-4 Systems (ISO/IEC
14496-1). Those intending to use this software module in hardware or
software products are advised that its use may infringe existing patents.
The original developer of this software module and his/her company, the
subsequent editors and their companies, and ISO/IEC have no liability
for use of this software module or modifications thereof in an
implementation. Copyright is not released for non MPEG-4 Systems
(ISO/IEC 14496-1) conforming products. Five Bats Research retains full right
to use the code for its purpose, assign or donate the code to a
third party and to inhibit third parties from using the code for non
MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright
notice must be included in all copies or derivative works.
Copyright  1999.
*/
#include "../mpegj-player/MPJSceneInterface.h"
#include "NativeScene.h"
#include "FieldValueMaker.h"

static MPJSceneDispatchTablePtr MPEGJ_Scene_Dispatch = 0;
static int MPEGJ_sceneHandle;
static JNIEnv *MPEGJ_env;
static JavaVM *MPEGJ_vm; // Java 2 PORTING - Added these as well
static jobject MPEGJ_obj; // Java 2 PORTING - Added these as well

#define TESTEXCEPTION \
	if ( MPEGJ_env->ExceptionOccurred() )	throw MPJException(MPJUnspecifiedErr)
#define TESTEXCEPTIONENV(env) \
	if ( env->ExceptionOccurred() )	throw MPJException(MPJUnspecifiedErr)

static MPJErr sceneEventNotification( int sceneHandle, int whatHappened )
{
	/* The place where MPEGJ_sceneHandle is assigned a value */
	MPEGJ_sceneHandle = sceneHandle;
	MPJErr err = MPJUnspecifiedErr;
	try
	{
		if ( MPEGJ_env )
		{
			// PORTING to Java 2 - Attach thread to JVM before invoking Java methods
			jint res = MPEGJ_vm->AttachCurrentThread( (void **)&MPEGJ_env, NULL);
			jclass cls = MPEGJ_env->FindClass( "org/iso/mpeg/mpegjRefImpl/scene/NativeScene" ); TESTEXCEPTION;
			jmethodID mid = MPEGJ_env->GetStaticMethodID( cls, "sceneEventNotification", "(II)I" ); TESTEXCEPTION;
			err = MPEGJ_env->CallStaticIntMethod( cls, mid, sceneHandle, whatHappened ); TESTEXCEPTION;
			// PORTING to Java 2 - Detach thread from JVM
			res = MPEGJ_vm->DetachCurrentThread();
		}
	}
	catch ( MPJException& e )
	{
		err = e.getErrorCode();
	}
	return err;
}

static MPJErr eventOutNotification( int nodeHandle, int eventOutID, FieldValuePtr changedFieldValue )
{
	MPJErr err = MPJUnspecifiedErr;

	try
	{
		if ( MPEGJ_env )
		{
			// PORTING to Java 2 - Attach thread to JVM before invoking Java methods
			jint res = MPEGJ_vm->AttachCurrentThread( (void **)&MPEGJ_env, NULL);
			jclass cls = MPEGJ_env->FindClass( "org/iso/mpeg/mpegjRefImpl/scene/NativeScene" ); TESTEXCEPTION;
			jmethodID mid = MPEGJ_env->GetStaticMethodID( cls, "eventOutNotification", "(IILorg/iso/mpeg/mpegj/scene/FieldValue;)I" ); TESTEXCEPTION;
			jobject fieldValue = FieldValueMaker::asJavaFieldValue( MPEGJ_env, changedFieldValue ); TESTEXCEPTION;
			err = MPEGJ_env->CallStaticIntMethod( cls, mid, nodeHandle, eventOutID, fieldValue ); TESTEXCEPTION;
			// PORTING to Java 2 - Detach thread from JVM
			res = MPEGJ_vm->DetachCurrentThread();
		}
	}
	catch ( MPJException& e )
	{
		err = e.getErrorCode();
	}
	return err;
}

static MPJErr getNewFieldValue( int param1, int param2, int fieldID, FieldValuePtr newFieldValue )
{
	MPJErr err = MPJUnspecifiedErr;
	try
	{
		JNIEnv *env = (JNIEnv *) param1;
		jobject jnode = (jobject) param2;

		jclass cls = env->FindClass( "org/iso/mpeg/mpegj/scene/NewNode" ); TESTEXCEPTIONENV(env);
		jmethodID mid = env->GetMethodID( cls, "getField", "(I)Lorg/iso/mpeg/mpegj/scene/FieldValue;" ); TESTEXCEPTIONENV(env);
		jobject val = env->CallObjectMethod( jnode, mid, fieldID ); TESTEXCEPTIONENV(env);
		
		if ( val == 0 )
		{
			// the getField() method returned null, indicating that the
			// default value should be used for this new field
			newFieldValue->fieldValueType = MPJ_UNINITIALIZED_FIELDVALUE;
			err = MPJNoErr;
		}
		else
		{
			// we now have the object from Java, write it to the newFieldValue parameter
			err = FieldValueMaker::setFieldValue( env, val, newFieldValue );
		}
	}
	catch ( MPJException& e )
	{
		err = e.getErrorCode();
	}
	return err;
}

MPJ_EXTERN(MPJErr)
MPJSetSceneDispatchTable( MPJSceneDispatchTable *table, int version )
{
	if ( version != MPJ_SCENEAPI_VERSION )
	{
		return MPJVersionMismatchErr;
	}
	MPEGJ_Scene_Dispatch = table;

	MPEGJ_Scene_Dispatch->sceneEventNotification = sceneEventNotification;
	MPEGJ_Scene_Dispatch->eventOutNotification   = eventOutNotification;
	MPEGJ_Scene_Dispatch->getNewFieldValue       = getNewFieldValue;
	return MPJNoErr;
}

extern "C" MPJ_EXTERN(MPJErr) 
CreateInstance()
{
	return (MPJErr) CreateInstance;
}

MPJ_EXTERN(MPJSceneDispatchTablePtr)
MPJGetSceneDispatchTable()
{
	return MPEGJ_Scene_Dispatch;
}

JNIEXPORT void JNICALL
Java_org_iso_mpeg_mpegjRefImpl_scene_NativeScene_setSceneManager( JNIEnv *env, jobject self , jobject mgr )
{
	MPEGJ_env = env;
	MPEGJ_obj = self;
	jint res = env->GetJavaVM(&MPEGJ_vm);

	if ( mgr == NULL )
	{
		MPEGJ_env = NULL;
	}
}

JNIEXPORT jint JNICALL
Java_org_iso_mpeg_mpegjRefImpl_scene_NativeScene_getScene( JNIEnv *env, jobject self )
{
	return MPEGJ_sceneHandle;
}

JNIEXPORT jint JNICALL
Java_org_iso_mpeg_mpegjRefImpl_scene_NativeScene_getNodeIDFromName( JNIEnv *env, jobject self, jint sceneHandle, jstring defName )
{
	jint ret = 0;
	printf("getNodeIDFromNAme\n");

	try
	{
		MPJSceneDispatchTablePtr t = MPJGetSceneDispatchTable();
		jboolean isCopy;
		const char *s = env->GetStringUTFChars( defName, &isCopy ); //TESTEXCEPTION;
		if ( isCopy == JNI_TRUE ) {				 
			if ( t && t->getNodeIDFromName ) {
				
				int defID;
				int err = t->getNodeIDFromName( sceneHandle, (char *) s, &defID );
				if ( err == MPJNoErr )
				{
					ret = defID;
				}
			}
			env->ReleaseStringUTFChars( defName, s ); //TESTEXCEPTION;
		}
	}
	catch ( MPJException& /*e*/ )
	{
	}
	return ret;
}

JNIEXPORT void JNICALL
Java_org_iso_mpeg_mpegjRefImpl_scene_NativeScene_modifyRouteTo( JNIEnv *env, jobject self, jint fromHandle, jint fromID, jint toHandle, jint toID, jint insert )
{
	try
	{
		MPJSceneDispatchTablePtr t = MPJGetSceneDispatchTable();
		if ( t && t->modifyRouteTo )
		{
			t->modifyRouteTo( fromHandle, fromID, toHandle, toID, insert );
		}
	}
	catch ( MPJException& /*e*/ )
	{
	}
}
