/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  JSFieldWrapper.h
//
// Author: Jean Le Feuvre, ENST
// Description:
//   script fields and SF/MF type wrappers for spiderMonkey JS
//
// Created: December, 2002.
//
//	
///////////////////////////////////////////////////////////////////////////


#ifndef JSFIELD_H_
#define JSFIELD_H_

//to get JSAPI.h
#include "ScriptImp.h"

#include "NodeFld.h"




//the script field types
enum SCRIPT_EVENT_TYPE  
{
	SCRIPT_FIELD,         // 0b00
	SCRIPT_EVENTIN,       // 0b01
	SCRIPT_EVENTOUT,      // 0b10
    SCRIPT_UNDEFINED
};

//base interface between the script node and the scene graph. ScriptNodeField are created during
//parsing of the Script node for each field except the static ones (url, directOutput and mustEvaluate)
class ScriptNodeField
{
public:
    ScriptNodeField(NodeField *pField, LPCSTR szName)
	{
		m_Name = strdup(szName);
		m_pNodeField = pField;
		m_LastEventTime = new SF<Time>;
		*((SF<Time> *) m_LastEventTime) = -1.0;
	}
    ~ScriptNodeField()
	{
		free(m_Name);
		delete m_LastEventTime;
	}

	//field name
	char *m_Name;
    NodeField *m_pNodeField;
	//used to store the eventIn time parameters and discard calls if needed
    NodeField *m_LastEventTime;
};

class JSField
{
public:
	JSField()
	{
		m_pField = NULL;
		m_pOwner = NULL;
		m_index = -1;
	}
	virtual void SetParentField(NodeField *pField, MediaObject *pOwner, int pIndex = -1) = 0;

	NodeField *m_pField;
	MediaObject *m_pOwner;
	int m_index;
};

template <class T> class JS : public JSField
{
public:

	JS<T> (T val) : m_value(val), JSField() {}

	virtual void SetParentField(NodeField *pField, MediaObject *pOwner, int index = -1) 
	{
		m_pField = pField;
		m_pOwner = pOwner;
		m_index = index;
	}
	void Assign(T val)
	{
		if (!m_pField) return;
		//typecast SF field
		if (m_index<0) {
			* ((SF<T> *) m_pField) = val;
			m_pField->OnFieldUpdate();
			return;
		} 
		//typecast MF field
		((MF<T> *)m_pField)->SetAt(m_index, val);
		m_pField->OnFieldUpdate();

	}
	
	//assign value and modify scene graph if needed
	void operator = (const T value) 
	{
		if ( ! ( value == m_value ) && m_pOwner) {
			m_pOwner->SetUpdateTime();
			m_pOwner->GetExecutive()->InvalidateScene(m_pOwner);
		}
		Assign(value);
		m_value = value;
	}

	T m_value;
};

class JSNode : public JSField 
{
public:
	JSNode() : JSField() {}

	
	virtual void SetParentField(NodeField *pField, MediaObject *pOwner, int index = -1) 
	{
		m_pField = pField;
		m_pOwner = pOwner;
		m_index = index;
		if (index<0) {
			m_pNode = ((SFGenericNode *) pField)->GetNode();
		} else {
			m_pNode = ((MFGenericNode *) pField)->GetObject(index);
		}
	}
	
	MediaObject *m_pNode;
};


class MFWrapper
{
public:
	MFWrapper() 
	{
		m_pList = NULL;
		m_pField = NULL;
		m_pOwner = NULL;
	}

	JSObject *m_pList;
	NodeField *m_pField;
	MediaObject *m_pOwner;
};


#endif //JSFIELD_H_
