/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  ParseScript.cpp
//
// Author: Jean Le Feuvre, ENST
// Description:
//   SFScript to text parser
//
// Created: December, 2002.
//
//	
///////////////////////////////////////////////////////////////////////////

#include "defs.h"
#include "ParseScript.h"
#include "ScriptImp.h"

#define BINOP_MINVAL ET_EQ

char* ScriptParser::binaryOp[] =
{

	"==", "!=","<", "<=",
	">", ">=", "+", "-",
	"*", "/", "%",
	"&&", "||",
	"&", "|", "^",
	"<<", ">>", ">>>"
};

char* ScriptParser::assignmentOp[] =
{
	"=", "+=", "-=", "*=", "/=", "%=", "<<=", ">>=", ">>>=",
	"&=", "^=", "|="
};


ScriptParser::ScriptParser(ScriptImp *script)
{
	m_pScript = script;
	m_Indent = 0;
}

ScriptParser::~ScriptParser()
{
}

void ScriptParser::NewLine()
{
#if 0
	Append("\n");
#endif
}

void ScriptParser::Indent(int increment)
{
#if 0
	m_Indent += increment;
	if (m_Indent < 0) m_Indent = 0;
	for (int i=0; i<m_Indent; i++) {
		Append(" ");
	}
#else
	increment;
#endif
}

void ScriptParser::Append(LPCSTR string)
{
	m_ScriptText += string;
}
void ScriptParser::Append(ZString string)
{
	m_ScriptText += string;
}

void ScriptParser::Append(int val)
{
	char msg[100];
	sprintf(msg, "%d", val);
	Append(msg);
}

ZString ScriptParser::ParseIdentifier(BIFSDecoder *pCoder)
{
	ZString ret;

	if (pCoder -> ParseInt(1, "received")) {
		int nbits, index, length;
		nbits = 0;
		length = m_pScript->identifierLength() - 1;
		while (length > 0) {
			length >>= 1;
			nbits ++;
		}
		index = pCoder->ParseInt(nbits,"identifierCode");
		ret = m_pScript->identifierAt(index);
	} else{
		pCoder->ParseString (ret);
		m_pScript->addIdentifier(ret);
	}
	return ret;
}

ZString ScriptParser::ParseNumber(BIFSDecoder *pCoder)
{
	int val;
	ZString str;
	char tmp[100];

	// integer
	if (pCoder -> ParseInt(1,"isInteger")) { 
		int nbits;
		nbits = pCoder -> ParseInt(5,"numbits");
		val = pCoder -> ParseInt(nbits,"value");
		sprintf(tmp, "%d", val);
		str = tmp;
		return str;
	} 

	// real number
	val = pCoder -> ParseInt(4,"floatChar");
	while ( val!= 15) {
		if (val>=0 && val<=9) {
			str += (char) (val + '0');
		} else if (val==10) {
			str += '.';
		} else if (val==11) {
			str += 'E';
		} else if (val==12){
			str += '-';
		} else
		{
			fprintf(stderr, "Error: %d is not a valid real number character code\n", val);
		}
		val = pCoder -> ParseInt(4,"floatChar");
	}
	return str;
}

ZString ScriptParser::ParseString(BIFSDecoder *pCoder)
{
	ZString string;
	pCoder->ParseString(string);
	return string;
}

ZString ScriptParser::ParseBoolean(BIFSDecoder *pCoder)
{
	ZString string;
	if (pCoder->ParseInt(1,"value") == 1) {
		string = "true";
	} else {
		string = "false";
	}
	return string;
}


void ScriptParser::Decode(BIFSDecoder *pCoder)
{
	ZString str;

	//init
	m_ScriptText = "";

	Append("javascript:");
	NewLine();
	Indent(1);

	while (pCoder -> ParseInt(1,"hasFunction")) {
		Append("function ");
		str = ParseIdentifier(pCoder);
		Append(str);

		Arguments(pCoder);

		StatementBlock(pCoder, TRUE);
	}

	NewLine();
	Indent(-1);
	NewLine();
}

void ScriptParser::Arguments(BIFSDecoder *pCoder)
{
	int val = pCoder -> ParseInt(1,"hasArgument");

	Append("(");
	while(val) {
		ZString str = ParseIdentifier(pCoder);
		Append(str);
		val = pCoder -> ParseInt(1, "hasArgument");
		if (val) Append(", ");
	}
	Append(")");
	NewLine();
}

void ScriptParser::StatementBlock(BIFSDecoder *pCoder, BOOL isFuncBody)
{
    if (pCoder -> ParseInt(1,"isCompoundStatement")) {
		Indent(0);
		Append("{");
		NewLine();
		Indent(1);
		while (pCoder -> ParseInt(1, "hasStatement")) {
			Statement(pCoder);
		}
		Indent(-1);
		Append("}");
		NewLine();
	} else {
		if (isFuncBody) Append("{");
		Statement(pCoder);
		if (isFuncBody) Append("}");
	}
}

void ScriptParser::Statement(BIFSDecoder *pCoder)
{
	int val;

	val = pCoder -> ParseInt(NUMBITS_STATEMENT,"statementType");
	switch (val) {
	case ST_IF:
		ifStatement(pCoder);
		break;
	case ST_FOR:
		forStatement(pCoder);
		break;
	case ST_WHILE:
		whileStatement(pCoder);
		break;
	case ST_RETURN:
		returnStatement(pCoder);
		Append(";");
		NewLine();
		break;
	case ST_BREAK:
		breakStatement(pCoder);
		Append(";");
		NewLine();
		break;
	case ST_CONTINUE:
		continueStatement(pCoder);
		Append(";");
		NewLine();
		break;
	case ST_COMPOUND_EXPR:
		CompoundExpression(pCoder);
		Append(";");
		NewLine();
		break;
	case ST_SWITCH:
		switchStatement(pCoder);
		break;
	default:
		fprintf(stderr, "Error: Unrecognized statement type %d\n", val);
	}
}

void ScriptParser::ifStatement(BIFSDecoder *pCoder)
{
	Append("if (");
	CompoundExpression(pCoder);
	Append(")");
	NewLine();
	StatementBlock(pCoder);

	if (pCoder -> ParseInt(1, "hasELSEStatement")) {
		Append("else ");
		NewLine();
		StatementBlock(pCoder);
	}
}

void ScriptParser::switchStatement(BIFSDecoder *pCoder)
{
	int numBits, caseValue;

	Append("switch (");
	CompoundExpression(pCoder);
	Append(") {");
	NewLine();
	Indent(0);

	do {
		Append("case ");
		numBits = pCoder -> ParseInt(5, "numbits"); // mp conformance
		caseValue = pCoder -> ParseInt(numBits, "caseValue");
		Append(caseValue);
		Append(":");
		NewLine();
		StatementBlock(pCoder);

	}
	while (pCoder->ParseInt(1,"hasMoreCases")==1);

	if (pCoder->ParseInt(1,"hasDefault") == 1) {
		Indent(0);
		Append("default :");
		NewLine();
		StatementBlock(pCoder);

	}
	Indent(0);
	Append("}");
	NewLine();
}


void ScriptParser::forStatement(BIFSDecoder *pCoder)
{
	Append("for (");
	OptionalExpression(pCoder);
	Append("; ");
	OptionalExpression(pCoder);
	Append("; ");
	OptionalExpression(pCoder);
	Append(")");
	NewLine();
	StatementBlock(pCoder);
}

void ScriptParser::whileStatement(BIFSDecoder *pCoder)
{
	Append("while (");
	CompoundExpression(pCoder);
	Append(")");
	NewLine();
	StatementBlock(pCoder);
}

void ScriptParser::returnStatement(BIFSDecoder *pCoder)
{
	Indent(0);
	Append("return ");
	if (pCoder -> ParseInt(1,"returnValue")) {
		CompoundExpression(pCoder);
	}
}

void ScriptParser::breakStatement(BIFSDecoder *)
{
	Indent(0);
	Append("break");
}

void ScriptParser::continueStatement(BIFSDecoder *)
{	
	Append("continue");
}

void ScriptParser::CompoundExpression(BIFSDecoder *pCoder)
{
	Expression(pCoder);
	if(!pCoder->ParseInt(1,"hasExpression")) return;
	Append(", ");
	CompoundExpression(pCoder);
}

void ScriptParser::OptionalExpression(BIFSDecoder *pCoder)
{
	if (pCoder -> ParseInt(1,"hasCompoundExpression"))
		CompoundExpression(pCoder);
}

void ScriptParser::Expression(BIFSDecoder *pCoder)
{
	int val = pCoder -> ParseInt(NUMBITS_EXPR_TYPE,"expressionType");
	switch(val) {
	case ET_CURVED_EXPR:
		Append("(");
		CompoundExpression(pCoder);
		Append(")");
		break;
	case ET_NEGATIVE:
		Append("-");
		Expression(pCoder);
		break;
	case ET_NOT:
		Append("!");
		Expression(pCoder);
		break;
	case ET_ONESCOMP:
		Append("~");
		Expression(pCoder);
		break;
	case ET_INCREMENT:
		Append("++");
		Expression(pCoder);
		break;
	case ET_DECREMENT:
		Append("--");
		Expression(pCoder);
		break;
	case ET_POST_INCREMENT:
		Expression(pCoder);
		Append("++");
		break;
	case ET_POST_DECREMENT:
		Expression(pCoder);
		Append("--");
		break;
	case ET_CONDTEST:
		Expression(pCoder);
		Append(" ? ");
		Expression(pCoder);
		Append(" : ");
		Expression(pCoder);
		break;
	case ET_STRING:
		Append("'");
		Append(ParseString(pCoder));
		Append("'");
		break;
	case ET_NUMBER:
		Append(ParseNumber(pCoder));
		break;
	case ET_IDENTIFIER:
		Append(ParseIdentifier(pCoder));
		break;
	case ET_FUNCTION_CALL:
		FunctionCall(pCoder);
		break;
	case ET_NEW:
        NewObject(pCoder);
		break;
	case ET_OBJECT_MEMBER_ACCESS: 
   		ObjectMemberAccess(pCoder);
		break;
	case ET_OBJECT_METHOD_CALL:
        ObjectMethodCall(pCoder);
		break;
	case ET_ARRAY_DEREFERENCE:
        ArrayDeref(pCoder);
		break;

	case ET_MULTIPLY:
	case ET_DIVIDE:
	case ET_MOD:
	case ET_PLUS:
	case ET_MINUS:
	case ET_LSHIFT:
	case ET_RSHIFT:
	case ET_RSHIFTFILL:
	case ET_AND:
	case ET_XOR:
	case ET_OR:
		Expression(pCoder);
		Append((LPCSTR) binaryOp[val - BINOP_MINVAL] );
		Expression(pCoder);
		break;
	case ET_LT:
	case ET_LE:
	case ET_GT:
	case ET_GE:
	case ET_EQ:
	case ET_NE:
	case ET_LAND:
	case ET_LOR:
		Expression(pCoder);
		Append((LPCSTR) binaryOp[val - BINOP_MINVAL]);
		Expression(pCoder);
		break;
	case ET_ASSIGN:
		Expression(pCoder);
		Append((LPCSTR) assignmentOp[0]);
		Expression(pCoder);
		break;
	case ET_PLUSEQ:
		Expression(pCoder);
		Append((LPCSTR) assignmentOp[1]);
		Expression(pCoder);
		break;
	case ET_MINUSEQ:
		Expression(pCoder);
		Append((LPCSTR) assignmentOp[2]);
		Expression(pCoder);
		break;
	case ET_MULTIPLYEQ:
		Expression(pCoder);
		Append((LPCSTR)assignmentOp[3]);
		Expression(pCoder);
		break;
	case ET_DIVIDEEQ:
		Expression(pCoder);
		Append((LPCSTR) assignmentOp[4]);
		Expression(pCoder);
		break;
	case ET_MODEQ:
		Expression(pCoder);
		Append((LPCSTR)assignmentOp[5]);
		Expression(pCoder);
		break;
	case ET_LSHIFTEQ:
		Expression(pCoder);
		Append((LPCSTR) assignmentOp[6]);
		Expression(pCoder);
		break;
	case ET_RSHIFTEQ:
		Expression(pCoder);
		Append((LPCSTR) assignmentOp[7]);
		Expression(pCoder);
		break;
	case ET_RSHIFTFILLEQ:
		Expression(pCoder);
		Append( (LPCSTR)assignmentOp[8]);
		Expression(pCoder);
		break;
	case ET_ANDEQ:
		Expression(pCoder);
		Append( (LPCSTR)assignmentOp[9]);
		Expression(pCoder);
		break;
	case ET_XOREQ:
		Expression(pCoder);
		Append( (LPCSTR)assignmentOp[10]);
		Expression(pCoder);
		break;
	case ET_OREQ:
		Expression(pCoder);
		Append( (LPCSTR)assignmentOp[11]);
		Expression(pCoder);
		break;
    case ET_BOOLEAN:
		Append(ParseBoolean(pCoder));
        break;
	default:
		Error("Error: illegal expression type");
	}
}

void ScriptParser::NewObject(BIFSDecoder *pCoder)
{
	Append("new ");
    Append(ParseIdentifier(pCoder));
	Append("(");
	Params(pCoder);
	Append(") ");
}

void ScriptParser::ArrayDeref(BIFSDecoder *pCoder)
{
	Expression(pCoder);
    Append("[");
	CompoundExpression(pCoder);
	Append("] ");
}

void ScriptParser::FunctionCall(BIFSDecoder *pCoder)
{
	ZString  str;
	Append(ParseIdentifier(pCoder));
	Append(" (");
	Params(pCoder);
	Append(")");
}


void ScriptParser::ObjectMemberAccess(BIFSDecoder *pCoder)
{
	Expression(pCoder);
    Append(".");
    Append(ParseIdentifier(pCoder));
}


void ScriptParser::ObjectMethodCall(BIFSDecoder *pCoder)
{
	Expression(pCoder);
	Append(".");
	Append(ParseIdentifier(pCoder));
	Append("(");
	Params(pCoder);
	Append(")");
}

void ScriptParser::Params(BIFSDecoder *pCoder)
{
	int val = pCoder -> ParseInt(1,"hasParam");
	while (val) {
		Expression(pCoder);
		val = pCoder -> ParseInt(1,"hasParam");
		if(val) Append(", ");
	}
}


void ScriptParser::Error(LPCSTR errmsg)
{
	fprintf(stderr, "Script node: %s\n", errmsg);
}








