/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  ParseScript.cpp
//
// Author: Jean Le Feuvre, ENST
// Description:
//   SFScript to text parser
//
// Created: December, 2002.
//
//	
///////////////////////////////////////////////////////////////////////////

#ifndef PARSESCRIPT_H
#define PARSESCRIPT_H

#include "ztl.h"
#include "ScriptImp.h"
#include "BifsDec.h"


#define NUMBITS_STATEMENT	3
#define NUMBITS_EXPR_TYPE	6
#define MAX_NUM_EXPR		100

enum 
{
	ST_IF=0, 
	ST_FOR=1, 
	ST_WHILE=2, 
	ST_RETURN=3, 
	ST_BREAK=5, 
	ST_CONTINUE=6, 
	ST_COMPOUND_EXPR=4,
	ST_SWITCH=7
};

enum
{
	ET_CURVED_EXPR=0,
	ET_NEGATIVE=1,
	ET_NOT=2,
	ET_ONESCOMP=3,
	ET_INCREMENT=4,
	ET_DECREMENT=5,
	ET_POST_INCREMENT=6,
	ET_POST_DECREMENT=7,
	ET_CONDTEST=8,
	ET_STRING=9,
	ET_NUMBER=10,
	ET_IDENTIFIER=11,
	ET_FUNCTION_CALL=12,
	ET_NEW=13,ET_OBJECTCONSTRUCT=13,//for mapping purpose
	ET_OBJECT_MEMBER_ACCESS=14,
	ET_OBJECT_METHOD_CALL=15,
	ET_ARRAY_DEREFERENCE=16,
	ET_ASSIGN=17,
	ET_PLUSEQ=18,
	ET_MINUSEQ=19,
	ET_MULTIPLYEQ=20,
	ET_DIVIDEEQ=21,
	ET_MODEQ=22,
	ET_ANDEQ=23,
	ET_OREQ=24,
	ET_XOREQ=25,
	ET_LSHIFTEQ=26,
	ET_RSHIFTEQ=27,
	ET_RSHIFTFILLEQ=28,
	ET_EQ=29,
	ET_NE=30,
	ET_LT=31,
	ET_LE=32,
	ET_GT=33,
	ET_GE=34,
	ET_PLUS=35,
	ET_MINUS=36,
	ET_MULTIPLY=37,
	ET_DIVIDE=38,
	ET_MOD=39,
	ET_LAND=40,
	ET_LOR=41,
    ET_AND=42,
	ET_OR=43,
	ET_XOR=44,
	ET_LSHIFT=45,
	ET_RSHIFT=46,
	ET_RSHIFTFILL=47,
	ET_BOOLEAN=48,
	NUMBER_OF_EXPR_TYPE=49
};


class ScriptParser 
{
public:
	ScriptParser(ScriptImp *node);
	~ScriptParser();
	ZString GetScriptText() {return m_ScriptText; }

	void Decode(BIFSDecoder *pBifs);
	
private:
	void NewLine();
	void Indent(int increment);
	void Append(LPCSTR string);
	void Append(ZString string);
	void Append(int val);

	ZString m_ScriptText;
	int m_Indent;
	ScriptImp *m_pScript;

	void Error(LPCSTR errmsg);

	ZString ParseIdentifier(BIFSDecoder *pBifs);
	ZString ParseNumber(BIFSDecoder *pBifs);
	ZString ParseString(BIFSDecoder *pBifs);
	ZString ParseBoolean(BIFSDecoder *pCoder);

	
	void Arguments(BIFSDecoder *pBifs);
	void StatementBlock(BIFSDecoder *pBifs, BOOL isFuncBody = FALSE);
	void Statement(BIFSDecoder *pBifs);
	void ifStatement(BIFSDecoder *pBifs);
	void switchStatement(BIFSDecoder *pBifs);
	void forStatement(BIFSDecoder *pBifs);
	void whileStatement(BIFSDecoder *pBifs);
	void returnStatement(BIFSDecoder *pBifs);
	void breakStatement(BIFSDecoder *pBifs);
	void continueStatement(BIFSDecoder *pBifs);
	void CompoundExpression(BIFSDecoder *pBifs);
	void OptionalExpression(BIFSDecoder *pBifs);
	void Expression(BIFSDecoder *pBifs);
	void NewObject(BIFSDecoder *pBifs);
	void ArrayDeref(BIFSDecoder *pBifs);
	void FunctionCall(BIFSDecoder *pBifs);
	void ObjectMemberAccess(BIFSDecoder *pBifs);
	void ObjectMethodCall(BIFSDecoder *pBifs);
	void Params(BIFSDecoder *pBifs);

	static char* binaryOp[];
	static char* assignmentOp[];
};

#endif // PARSESCRIPT_H




