#include "defs.h"

#include "ScriptImp.h" 
#include "JSFieldWrapper.h"
#include "NodeCode.h"

#include "BifsDec.h"
#include "Route.h"
#include "Root.h"
#include "ParseScript.h"

//the JS runtime
JSRuntime *js_rt = 0;
//the number of script nodes
static unsigned int numInstances = 0;
const long MAX_HEAP_BYTES = 4L * 1024L * 1024L;
const long STACK_CHUNK_BYTES = 4024L;



ScriptImp::ScriptImp()
{
	BifsFieldTable table;
	// initialize url field
	memset(&table, 0, sizeof(BifsFieldTable));
	table.m_offset = (WORD) ((size_t) ((NodeField *) & ((ScriptImp *)0) -> url) - (size_t) (MediaObject *)((ScriptImp *)0));
	table.m_dynID = -1;
	table.m_quantParams = -1;
	table.m_fMin [0] = -FLT_MAX;
	table.m_fMax [0]= FLT_MAX;
	table.m_szName = "url";
	m_fieldTable += table;

	// initialize directOutput field
	memset(&table, 0, sizeof(BifsFieldTable));
	table.m_offset = (WORD) ((size_t) ((NodeField *) & ((ScriptImp *)0) -> directOutput) - (size_t) (MediaObject *)((ScriptImp *)0));
	table.m_defID = 1;
	table.m_inID = -1;
	table.m_outID = -1;
	table.m_dynID = -1;
	table.m_quantParams = -1;
	table.m_fMin [0] = -FLT_MAX;
	table.m_fMax [0]= FLT_MAX;
	table.m_szName = "directOutput";
	m_fieldTable += table;

	// initialize mustEvaluate field
	memset(&table, 0, sizeof(BifsFieldTable));
	table.m_offset = (WORD) ((size_t) ((NodeField *) & ((ScriptImp *)0) -> mustEvaluate) - (size_t) (MediaObject *)((ScriptImp *)0));
	table.m_defID = 2;
	table.m_inID = -1;
	table.m_outID = -1;
	table.m_dynID = -1;
	table.m_quantParams = -1;
	table.m_fMin [0] = -FLT_MAX;
	table.m_fMax [0]= FLT_MAX;
	table.m_szName = "mustEvaluate";
	m_fieldTable += table;

	// ajust bifsBounds
	m_bifsBounds.m_nTotalAll = 3;
	m_bifsBounds.m_nTotalDefs = 3;
	m_bifsBounds.m_nTotalIns = 1;
	m_bifsBounds.m_nTotalOuts = 1;
	m_bifsBounds.m_nTotalDyns = 0;

	m_bifsBounds.CalculateBounds ();

	// ajust offset
	m_nChildrenOffset = -1;
	m_fieldCount = 3;

	m_bInScriptParsing = FALSE;

	//Init JS
	if (!js_rt) {
		js_rt = JS_NewRuntime(MAX_HEAP_BYTES);
		assert(js_rt);
	}
	//create JS context for this script node
	m_context = JS_NewContext(js_rt, STACK_CHUNK_BYTES);
	assert(m_context);

	numInstances++;
	JS_SetContextPrivate(m_context, this);

	InitJSAPI();
}


ScriptImp::~ScriptImp()
{
	Reset();

	JS_DestroyContext(m_context);
	numInstances --;
	if (numInstances == 0) {
		JS_DestroyRuntime(js_rt);
		js_rt = 0;
	}
}

void ScriptImp::Reset()
{
	while (FieldList.GetLength()) {
		delete FieldList[0];
		FieldList.DeleteFrom(0);
	}
	while (EventInList.GetLength()) {
		delete EventInList[0];
		EventInList.DeleteFrom(0);
	}
	while (EventOutList.GetLength()) {
		delete EventOutList[0];
		EventOutList.DeleteFrom(0);
	}
	while (m_fields.GetLength()) {
		delete m_fields[0];
		m_fields.DeleteFrom(0);
	}
	//WARNING: the 3 first fields are static (cf constructor)
	while (m_fieldTable.GetLength()>3) {
		free( (void *) m_fieldTable[3].m_szName);
		m_fieldTable.DeleteFrom(3);
	}
	m_fieldNames.Clear();
}

/*-------------------------------------------------------------
Create a script field. This involves instantiation of the proper NodeField object,
and adding proper values to the field table for this proto node
-------------------------------------------------------------*/
NodeField *ScriptImp :: CreateInterfaceField(int eventType, FIELD_TYPE fieldType, LPCSTR szName)
{
   	/*-------------------------------------------------------------
	Instantiate the proper field object and add it to the list
	-------------------------------------------------------------*/
	NodeField *pField = NULL;
	switch (fieldType) {
		case SFBOOL:
			pField = new SFBool;
			break;
		case SFFLOAT:
			pField = new SF<float>;
			break;
		case SFTIME:
			pField = new SF<Time>;
			break;
		case SFINT32:
			pField = new SF<int>;
			break;
		case SFSTRING:
			pField = new SFString;
			break;
		case SFVEC3F:
			pField = new SF<Vec3f>;
			break;
		case SFVEC2F:
			pField = new SF<Vec2f>;
			break;
		case SFCOLOR:
			pField = new SF<Color3f>;
			break;
		case SFROTATION:
			pField = new SF<Rotation>;
			break;
		case SFIMAGE:
			pField = new SF<Image>;
			break;
		case SFNODE:
			pField = new SFWorldNode;
			break;
		case MFBOOL:
			pField = new MF<Bool>;
			break;
		case MFFLOAT:
			pField = new MF<float>;
			break;
		case MFTIME:
			pField = new MF<Time>;
			break;
		case MFINT32:
			pField = new MF<int>;
			break;
		case MFSTRING:
			pField = new MF<ZString>;
			break;
		case MFVEC3F:
			pField = new MF<Vec3f>;
			break;
		case MFVEC2F:
			pField = new MF<Vec2f>;
			break;
		case MFCOLOR:
			pField = new MF<Color3f>;
			break;
		case MFROTATION:
			pField = new MF<Rotation>;
			break;
		case MFIMAGE:
			pField = new MF<Image>;
			break;
		case MFNODE:
			pField = new MFWorldNode;
			break;
		default:
			return NULL;
	}

	/*-------------------------------------------------------------
	Add an entry to the BifsFieldTable
	-------------------------------------------------------------*/
	int nOffset = m_fieldTable.GetLength ();
	BifsFieldTable rEntry;
	rEntry.m_AnimationType = -1;
	rEntry.m_bChildren = FALSE;
	rEntry.m_dynID = -1;
	rEntry.m_defID = rEntry.m_inID = rEntry.m_outID = -1;
	rEntry.m_fMax [0] = FLT_MAX;
	rEntry.m_fMin [0] = -FLT_MAX;
	rEntry.m_function = NULL;
	rEntry.m_nDimension = 0;
	rEntry.m_nQuantizationType = -1;
	rEntry.m_offset = -nOffset;
	rEntry.m_quantParams = -1;
	rEntry.m_szName = strdup(szName);
	m_bifsBounds.m_nTotalAll++;


	ScriptNodeField *a_field = new ScriptNodeField(pField, szName);
	switch (eventType) {
    case SCRIPT_FIELD: 
		rEntry.m_defID = m_bifsBounds.m_nTotalDefs++;
        FieldList += a_field;
        break; 
    case SCRIPT_EVENTIN: 
		rEntry.m_inID = m_bifsBounds.m_nTotalIns++;
        EventInList += a_field;
        break; 
    case SCRIPT_EVENTOUT: 
		rEntry.m_outID = m_bifsBounds.m_nTotalOuts++;
        EventOutList += a_field;
        break; 
	}

	m_fieldTable += rEntry;
	m_bifsBounds.CalculateBounds();
	m_fieldCount ++;

    m_fields += pField;
	return pField;
}



int ScriptImp::ParseScriptField(BIFSDecoder *pCoder)
{
	NodeField *pField;

	SCRIPT_EVENT_TYPE eType = (SCRIPT_EVENT_TYPE) pCoder -> ParseInt(2, "eventType");
	FIELD_TYPE fieldType = (FIELD_TYPE) pCoder -> ParseInt(6,"fieldType");
	ZString fieldName;
	pCoder->ParseString (fieldName);
	pField = CreateInterfaceField(eType, fieldType, fieldName);

	// save the name in the list of identifiers
	addIdentifier(fieldName) ;  

	if (eType == SCRIPT_FIELD) {
		if (pCoder->ParseInt(1,"hasInitialValue")) {
			pField->Parse(pCoder,this);
		}
	}
	return TRUE;
}

BOOL ScriptImp::ParseSFScript(BIFSDecoder *pCoder)
{
	//first reset the script
	Reset();
	ScriptParser *pParser = new ScriptParser(this);
	
	//set parsing active
	m_bInScriptParsing = TRUE;

	//parse all fields definition
	if (pCoder -> ParseInt(1,"isListDescription")) {
		while (!pCoder -> ParseInt(1, "endFlag")){
			ParseScriptField(pCoder);
		}
	} else {
		int fieldBits = pCoder -> ParseInt(4,"fieldBits");

		int numFields = pCoder -> ParseInt(fieldBits,"numFields");
		for (int i=0;i<numFields;i++) {
			ParseScriptField(pCoder);
		}
	}

	//set parsing inactive now that all fileds are parsed
	m_bInScriptParsing = FALSE;

    TRACEF("url [", 0); 
    TRACENEWLINE; 
	TRACEINDENT (1); 

	if (pCoder -> ParseInt(1,"reserved") != 1){
		printf("Error: reserved should be 1\n");
	}

  	pParser->Decode(pCoder);

	m_scriptText = pParser->GetScriptText();
	delete pParser;

	//setup fields
    InitFields();

	//load script
	if (!strnicmp(m_scriptText, "javascript:", 11) || !strnicmp(m_scriptText, "vrmlscript:", 11)) {
		LoadScript( ((LPCTSTR)m_scriptText) + 11);
	}

   
	TRACEF("%s", (LPCSTR) m_scriptText); 
	TRACEINDENT (-1);
    TRACEF("]", 0);
	TRACENEWLINE;
	return TRUE;
}

void ScriptImp::LoadScript(const char *script)
{
	char *filename = 0;
	uintN lineno = 0;
    jsval rval;

	JSBool ret;
	ret = JS_EvaluateScript(m_context, m_pObj, script, strlen(script), filename, lineno, &rval);
}


void ScriptImp::RouteEvent (NodeField *pField, int nFieldIndex)
{
	int in_id = m_fieldTable[nFieldIndex].m_inID;
	if (in_id<0) return;

	//don't touch url, mustEvaluate and directOutput 
	if (nFieldIndex < 3) return;

	// -3 because url, mustEvaluate and directOutput are not physically in the list
    NodeField *myField = GetField (nFieldIndex - 3);
	myField -> Copy (*pField, this, false);
	myField -> OnFieldUpdate();
	
	// -1 because url is an eventIn (exposedField) but not added to the list
	OnEventIn(EventInList[in_id - 1]);
}



void ScriptImp::OnEventIn(ScriptNodeField *pEventIn)
{
	jsval fval, rval;

	//if we haven't changed since last call discard
	double now = GetRoot()->GetSceneTime();
	now /= 1000.;

//	if ( *((SF<Time> *) pEventIn->m_LastEventTime) == now) return; 
	*((SF<Time> *) pEventIn->m_LastEventTime) = now;

	//locate function
	if (! JS_LookupProperty(m_context, m_pObj, (LPCSTR) pEventIn->m_Name, &fval)) return;
	if (JSVAL_IS_VOID(fval)) return;

	jsval *argv = new jsval[2];
	argv[0] = NodeFieldToJSVal(pEventIn->m_pNodeField, true, this);
	argv[1] = NodeFieldToJSVal(pEventIn->m_LastEventTime, true, this);



	JS_CallFunctionValue(m_context, m_pObj, fval, 2, argv, &rval);

	//release our args
	if (JSVAL_IS_GCTHING(argv[0])) JS_RemoveRoot(m_context, JSVAL_TO_GCTHING(argv[0]));
	if (JSVAL_IS_GCTHING(argv[1])) JS_RemoveRoot(m_context, JSVAL_TO_GCTHING(argv[1]));

	//we let JS cleanup memory - we could use JS_GC at each step but that leads to a bug in eventOut fields
	//so ...

	delete [] argv;
}
