/*
	This software module was originally developed by ENST in the course of
development of the MPEG-4 Systems (ISO/IEC 14496-1) standard.
	This software module is an implementation of a part of one or more MPEG-4
Systems (ISO/IEC 14496-1) tools as specified by the MPEG-4 Systems (ISO/IEC 14496-1) 
standard. ISO/IEC gives users of the MPEG-4 Systems (ISO/IEC 14496-1) free license 
to this software module or modifications thereof for use in hardware or software 
products claiming conformance to the MPEG-4 Systems (ISO/IEC 14496-1). 
Those intending to use this software module in hardware or software products are 
advised that its use may infringe existing patents.
	The original developer of this software module and his/her company, the 
subsequent editors and their companies, and ISO/IEC have no liability for use of 
this software module or modifications thereof in an implementation. Copyright is 
not released for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. 
	ENST retains full right to use the code for his/her own purpose, assign or 
donate the code to a third party and to inhibit third parties from using the code 
for non MPEG-4 Systems (ISO/IEC 14496-1) conforming products. This copyright    
notice must be included in all copies or derivative works.              
						Copyright  2002.
*/

///////////////////////////////////////////////////////////////////////////
// File:  SFScript.h
//
// Author: Jean Le Feuvre
// Description:
//   SFScript for SpiderMonkey interpreter
//
// Created: December, 2002.
//
//	
///////////////////////////////////////////////////////////////////////////


#ifndef SCRIPTIMP_H_
#define SCRIPTIMP_H_

//#include "nodes.h"
#include "Nodes.h" // SOLARIS PORTING - typo
#include "BifsDec.h"

//define for spidermonkey 
// SOLARIS PORTING
#ifdef WIN32
#define XP_PC
#else
#define XP_UNIX
#endif
#include "jsapi.h" 


class ScriptParser;
class ScriptNodeField;

//script implementation
class ScriptImp : public Script
{
public:
	ScriptImp();
	virtual ~ScriptImp();
	BOOL operator == (const ScriptImp &node) const;

	BOOL InScriptParsing() { return m_bInScriptParsing; }
	BOOL ParseSFScript(BIFSDecoder *pCoder);

	void addIdentifier(ZString name){ m_fieldNames += name;}
	int findIdentifier(ZString name){return m_fieldNames.Find(name);}
	ZString identifierAt(int index){return m_fieldNames[index];}
	int	identifierLength() {return m_fieldNames.GetLength();}

	// overloaded methods
	virtual const BifsFieldTable *GetFieldTable () const {return m_fieldTable;}
	virtual const BifsBounds *GetBifsBounds () const 	 {return &m_bifsBounds;}
	virtual int GetChildrenOffset () const {return m_nChildrenOffset;}
	virtual int GetFieldCount () const {return m_fieldCount;}
	NodeField *GetField (int index) {return m_fields [index]; }
	
	ZString GetScriptText() {return m_scriptText;}

private:
	NodeField *CreateInterfaceField (int eventType, FIELD_TYPE fieldType, LPCSTR szName);

	ZArray<NodeField *> m_fields;
	ZArray<ZString> m_fieldNames;

	//reset all fields
	void Reset();
	void LoadScript(const char *script);

	//JS vars
	JSContext *m_context;
	struct JSObject *m_pObj;
	struct JSObject *m_pBrowser;
	BOOL InitJSAPI();

	//script activation
	void OnEventIn(ScriptNodeField *pEventIn);

	//node modification
	void CheckFieldChange(BOOL changed, MediaObject *pNode, NodeField *pField);
    void InitFields();

	//this is used to parse SFString in script interfaces, since IM1 uses ZString for both SFString and SFScript
	BOOL m_bInScriptParsing;
	ZString m_scriptText;


	// overloaded attributes
	int m_nChildrenOffset;
	int m_fieldCount;
	ZArray<BifsFieldTable> m_fieldTable;
	BifsBounds m_bifsBounds;

public:
	//translate a field to a JSVal. pNode is the parent object, and is used to signal updates
	//to the executive. If not set, this is a native field created from within the script
	jsval NodeFieldToJSVal(NodeField *pField, BOOL protect, MediaObject *pNode);
	//translate a JSVal to a field. 
	void JSValToNodeField(JSContext *cx, jsval val, NodeField *pField, MediaObject *pNode);
	
public: 
	//public because access is needed in some JS C bindings
    ZArray<ScriptNodeField *> FieldList;
	ZArray<ScriptNodeField *> EventInList;
	ZArray<ScriptNodeField *> EventOutList;

	virtual void RouteEvent (NodeField *pField, int nFieldIndex);
	virtual ParseScriptField(BIFSDecoder *pCoder);

};

#endif // SCRIPTIMP_H_

