// This software module was originally developed by Wei Wu(Rockwell Science
// Center) and Wenlong Dong(Rockwell Science Center) in the course of 
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wei Wu at weiwu@jupiter.risc.rockwell.com or Wenlong Dong 
// at wdong@usc.edu.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE:       Buffer.cpp
AUTHOR:		Wei Wu       (Rockwell Science Center)
			Wenlong Dong (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Interface for the class VSNodeField.
--------------------------------------------------------------------*/
#include "defs.h"
#include "Buffer.h"

CBuffer::CBuffer()
{
	m_size = DEF_BUF_SIZE;
	m_curPos = 0;
	m_buf = (CTOKEN_CODE*) malloc(m_size*sizeof(CTOKEN_CODE));
}

CBuffer::~CBuffer()
{
	free(m_buf);
}

CTOKEN_CODE& CBuffer::operator[](int i)
{
	if (i >= m_size) {
		fprintf(stderr, "Error: Buffer out of range (%d)\n", i);
		exit(-1);
	}
	return m_buf[i];
}

void CBuffer::insert(CTOKEN_CODE tc)
{
	if (m_curPos >= m_size) {
		m_size += BUF_SIZE_INC;
		m_buf = (CTOKEN_CODE*) realloc(m_buf, m_size*sizeof(CTOKEN_CODE));
	}
	m_buf[m_curPos++] = tc;
}

void CBuffer::print()
{
	int i;

	printf("{ ");
	for (i=0; i<m_curPos-1; i++) {
		printf("%d, ", m_buf[i]);
	}
	printf("%d }\n", m_buf[i]);
}
