// This software module was originally developed by Wei Wu(Rockwell Science
// Center) and Wenlong Dong(Rockwell Science Center) in the course of 
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wei Wu at weiwu@jupiter.risc.rockwell.com or Wenlong Dong 
// at wdong@usc.edu.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE:       CParser.cpp
AUTHOR:		Wei Wu       (Rockwell Science Center)
			Wenlong Dong (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Implementation of the CParser class
--------------------------------------------------------------------*/
#include "defs.h"
#include "BifsEnc.h"
#include "CParser.h"
#include "NodeFld.h"
#include "ScriptDefs.h" //AC 200005

CParser::CParser(LPCSTR str, int flag)
	: m_lex(str), m_flag(flag)
{
	m_cur = m_lex();
}

/*void CParser::InsertExID(LPCSTR str)
{
	m_savedStrBuffer.insert(str);
}
*/
inline void CParser::Consume(CTOKEN_CODE tc)
{
	if (m_cur != tc) {
		fprintf(stderr, "Error: token %d read, %d expected\n",
			m_cur, tc);
		exit(-1);
	}
	m_cur = m_lex();
}

void CParser::PutIdentifier(LPCSTR str, BIFSEncoder *pBifs) 
{
	int index = pSNode->findIdentifier(str);
	if (index!=-1) 
	{
		int nbits, length;
		nbits = 0;
		length = pSNode->identifierLength() - 1;
		while (length > 0) {
			length >>= 1;
			nbits ++;
		}
		pBifs -> EncodeInt (1, 1, "received");
		pBifs -> EncodeInt (index, nbits, "identifierCode");
	} 
	else 
	{
		pBifs -> EncodeInt (0, 1, "received");
		pSNode ->addIdentifier(str);
		//if (m_flag) {
			PutString(str, pBifs);
		//}
	}
}

void CParser::PutNumber(LPCSTR str, BIFSEncoder *pBifs) {

    //if (hasChar(str, '.') == 0) { 
    if (strpbrk(str,".eE-") == 0) {
		pBifs -> EncodeInt (1, 1, "isInteger", "integer number"); // mp conformance
		PutInteger(str, pBifs);
	} else {
		pBifs -> EncodeInt (0, 1, "isInteger", "real number"); // mp conformance
		PutReal(str, pBifs);
	}
}

void CParser::PutBoolean(LPCSTR str, BIFSEncoder *pBifs) {
    unsigned int val = 1;
	if (!strcmp("TRUE", str) || !strcmp("true", str)) {
		val = 1;
	} else if (!strcmp("FALSE", str) || !strcmp("false", str)) {
		val = 0;
    } // cannot be else !
    pBifs->EncodeInt(val, 1, "value", "boolean");
}

int CParser::hasChar(LPCSTR str, int c)
{
	int length = strlen(str);

	for (int i=0; i<=length; i++) {
		if (c == *str)
			return 1;
		else
			str++;
	}
	return 0;
}

void CParser::PutInteger(LPCSTR str, BIFSEncoder *pBifs)
{
	char c0, c1;
	unsigned int val = 0;
	int nbits;

	c0 = str[0];
	c1 = str[1];
	if (c0=='0' && (c1=='x' || c1=='X')) { // hex
		val = strtoul(str, (char **) NULL, 16);
	} else if (c0=='0' && isdigit(c1)) { // oct
		val = strtoul(str, (char **) NULL, 8);
	} else if (isdigit(c0)) { // base 10
		val = strtoul(str, (char **) NULL, 10);
	} else {
		fprintf(stderr, "Error: %s is not an integer\n", str);
		exit(-1);
	}
	nbits = calNbits(val);
	pBifs -> EncodeInt (nbits, 5, "numbits", "NBITS_For_Integer"); // mp conformance
	pBifs -> EncodeInt (val, nbits, "value", str); // mp conformance
}

void CParser::PutReal(LPCSTR str, BIFSEncoder *pBifs)
{
	int length = strlen(str);
	int charVal;

	for (int i=0; i<length; i++) {
		charVal = str[i];
		if (charVal >= '0' && charVal <= '9') {
			pBifs -> EncodeInt (charVal-'0', 4, "floatChar", "Digital"); // mp conformance
		} else if (charVal == '.') {
			pBifs -> EncodeInt (10, 4, "floatChar", "Decimal Point"); // mp conformance
		} else if (charVal == 'e' || charVal == 'E') {
			pBifs -> EncodeInt (11, 4, "floatChar", "Exponential"); // mp conformance
		} else if (charVal == '-') { //AC 200005 9.3.7.45
			pBifs -> EncodeInt (12, 4, "floatChar", "MinusSign"); // mp conformance
		} else {
			fprintf(stderr, "Error: %s is not an real number\n", 
				str);
			exit (-1);
		}
	}
	pBifs -> EncodeInt (15, 4, "floatChar", "End Symbol"); // end symbol // mp conformance
}

void CParser::PutString(LPCSTR str, BIFSEncoder *pBifs) 
{
	pBifs->EncodeString(str);
}

void CParser::Encode(BIFSEncoder *pBifs)
{ 
	while (m_cur != CTID_EOF) {
		pBifs -> EncodeInt (1, 1, "hasFunction");
		Function(pBifs);
	}

	pBifs -> EncodeInt (0, 1, "hasFunction");
}

void CParser::Function(BIFSEncoder *pBifs)
{
	Consume(CTID_FUNCTION);
	PutIdentifier(m_lex.GetText(), pBifs);
	Consume(CTID_IDENTIFIER);
	Consume(CTID_LEFT_CURVE);
	Argument(pBifs);
	Consume(CTID_RIGHT_CURVE);
	StatementBlock(pBifs);
}

void CParser::Argument(BIFSEncoder *pBifs)
{
	while (m_cur != CTID_RIGHT_CURVE) {
		pBifs -> EncodeInt (1, 1, "hasArgument");
		PutIdentifier(m_lex.GetText(), pBifs);
		Consume(CTID_IDENTIFIER);
		if (m_cur == CTID_COMMA) {
			Consume(CTID_COMMA);
		}
	}
	
	pBifs -> EncodeInt(0, 1, "hasArgument");

}

void CParser::StatementBlock(BIFSEncoder *pBifs)
{
	if (m_cur == CTID_LEFT_BRACE) {
		Consume(CTID_LEFT_BRACE);
		pBifs -> EncodeInt (1, 1, "isCompoundStatement");
		Statements(pBifs);
		Consume(CTID_RIGHT_BRACE);
	} else {
		pBifs -> EncodeInt (0, 1, "isCompoundStatement");
		Statement(pBifs);
	}
}

void CParser::Statements(BIFSEncoder *pBifs)
{
	while (m_cur != CTID_RIGHT_BRACE) {
		pBifs -> EncodeInt (1, 1, "hasStatement");
		Statement(pBifs);
	}
	pBifs -> EncodeInt (0, 1, "hasStatement");
}

void CParser::Statement(BIFSEncoder *pBifs)
{
	//AC 200005 : - changed integer IDs with ST_* enum defined in ScriptDefs
	//			  - changed values so as to be spec-compliant N2501/9.3.7.34.2 (p111)
	/*				_ScriptDefs_h
					ST_IF=0, 
					ST_FOR=1, 
					ST_WHILE=2, 
					ST_RETURN=3, 
					ST_BREAK=5, 
					ST_CONTINUE=6, 
					ST_COMPOUND_EXPR=4		
	*/
	switch (m_cur) {
	case CTID_IF:
		pBifs -> EncodeInt (ST_IF, NUMBITS_STATEMENT, "statementType");
		ifStatement(pBifs);
		break;
	case CTID_FOR:
		pBifs -> EncodeInt (ST_FOR, NUMBITS_STATEMENT, "statementType");
		forStatement(pBifs );
		break;
	case CTID_WHILE:
		pBifs -> EncodeInt (ST_WHILE, NUMBITS_STATEMENT, "statementType");
		whileStatement(pBifs );
		break;
	case CTID_RETURN:
		pBifs -> EncodeInt (ST_RETURN, NUMBITS_STATEMENT, "statementType");
		returnStatement(pBifs );
		Consume(CTID_SEMICOLON);
		break;
	case CTID_BREAK:
		pBifs -> EncodeInt (ST_BREAK, NUMBITS_STATEMENT, "statementType");
		breakStatement();
		Consume(CTID_SEMICOLON);
		break;
	case CTID_CONTINUE:
		pBifs -> EncodeInt (ST_CONTINUE, NUMBITS_STATEMENT, "statementType");
		continueStatement();
		Consume(CTID_SEMICOLON);
		break;
	case CTID_SWITCH:
		pBifs -> EncodeInt (ST_SWITCH, NUMBITS_STATEMENT, "statementType");
		switchStatement(pBifs);
		//Consume(CTID_SEMICOLON);
		break;
	default:
		pBifs -> EncodeInt (ST_COMPOUND_EXPR, NUMBITS_STATEMENT, "statementType");
		compoundExpression(pBifs );
		Consume(CTID_SEMICOLON);
	}
}

void CParser::ifStatement(BIFSEncoder *pBifs)
{
	Consume(CTID_IF);
	Consume(CTID_LEFT_CURVE);
	compoundExpression(pBifs);
	Consume(CTID_RIGHT_CURVE);
	StatementBlock(pBifs);
	if (m_cur == CTID_ELSE) {
		Consume(CTID_ELSE);
		pBifs -> EncodeInt (1, 1, "hasELSEStatement");
		StatementBlock(pBifs);
	} else {
		pBifs -> EncodeInt (0, 1, "hasELSEStatement");
	}
}



void CParser::forStatement(BIFSEncoder *pBifs)
{
	Consume(CTID_FOR);
	Consume(CTID_LEFT_CURVE);
	optionalExpression(pBifs );
	Consume(CTID_SEMICOLON);
	optionalExpression(pBifs );
	Consume(CTID_SEMICOLON);
	optionalExpression(pBifs );
	Consume(CTID_RIGHT_CURVE);
	StatementBlock(pBifs);
}

void CParser::whileStatement(BIFSEncoder *pBifs)
{
	Consume(CTID_WHILE);
	Consume(CTID_LEFT_CURVE);
	compoundExpression(pBifs );
	Consume(CTID_RIGHT_CURVE);
	StatementBlock(pBifs);
}

void CParser::returnStatement(BIFSEncoder *pBifs)
{
	Consume(CTID_RETURN);
	if (m_cur != CTID_SEMICOLON) {
		pBifs -> EncodeInt (1, 1, "returnValue");
		compoundExpression(pBifs );
	} else {
		pBifs -> EncodeInt (0, 1, "returnValue");
	}
}

void CParser::breakStatement()
{
	Consume(CTID_BREAK);
}

void CParser::continueStatement()
{
	Consume(CTID_CONTINUE);
}
//AC 200005 : swtch statement
// semantics as in ScriptChangesMemo - ECMA like.
void CParser::switchStatement(BIFSEncoder *pBifs)
{

	Consume(CTID_SWITCH);
	Consume(CTID_LEFT_CURVE);
	compoundExpression(pBifs);
	Consume(CTID_RIGHT_CURVE);
	Consume(CTID_LEFT_BRACE);
    while (m_cur == CTID_CASE) {
        	Consume(CTID_CASE);
            // begin mp change
			PutInteger(m_lex.GetText(), pBifs);
            Consume(CTID_NUMBER);
            // end mp change
			// compoundExpression(pBifs); 
			Consume(CTID_CONDSEP);
            CaseBlock(pBifs);
            pBifs->EncodeInt (m_cur == CTID_CASE, 1, "hasMoreCases");
    }
    if (m_cur == CTID_DEFAULT) {
		pBifs->EncodeInt (1, 1, "hasDefault");
		Consume(CTID_DEFAULT);
		Consume(CTID_CONDSEP);
        CaseBlock(pBifs);
    } else {
		pBifs->EncodeInt (0, 1, "hasDefault");
    }
    Consume (CTID_RIGHT_BRACE);

    //mp this code was looping in case of statements without {}
	/*do {
		switch(m_cur){
		case CTID_RIGHT_BRACE:
			if (first) 
			{
				fprintf(stderr,"[CParser::switchStatement] Error no case at all\n");
				return;
			}
			pBifs->EncodeInt (0, 1, "hasMoreCases");
			break;
		case CTID_CASE:
			if (!first) pBifs->EncodeInt (1, 1, "hasMoreCases");
			first=false;
			Consume(CTID_CASE);
			compoundExpression(pBifs);
			Consume(CTID_CONDSEP);
			StatementBlock(pBifs);
			break;
		case CTID_DEFAULT:
			pBifs->EncodeInt (0, 1, "hasMoreCases");
			pBifs->EncodeInt (1, 1, "hasDefault");
			Consume(CTID_DEFAULT);
			Consume(CTID_CONDSEP);
			StatementBlock(pBifs);
			if (m_cur!=CTID_RIGHT_BRACE) 
			{
				fprintf(stderr,"[CParser::switchStatement] Error : default has to be followed by }\n");
				return;
			}
			break;
		}
	} while (m_cur!=CTID_RIGHT_BRACE);*/
}

void CParser::CaseBlock(BIFSEncoder *pBifs) {
	pBifs -> EncodeInt (1, 1, "isCompoundStatement"); // mp : as we don't know how many statements are coming, we'll always consider we have a compundStatement
	if (m_cur == CTID_LEFT_BRACE) {
		Consume(CTID_LEFT_BRACE);
	    while (m_cur != CTID_RIGHT_BRACE) {
		    pBifs -> EncodeInt (1, 1, "hasStatement");
		    Statement(pBifs);
	    }
		Consume(CTID_RIGHT_BRACE);
	}
    while (m_cur != CTID_CASE 
        && m_cur != CTID_DEFAULT
        && m_cur != CTID_RIGHT_BRACE
        && m_cur != CTID_EOF) {
		    pBifs -> EncodeInt (1, 1, "hasStatement");
		    Statement(pBifs);
    }
    pBifs -> EncodeInt (0, 1, "hasStatement");
}

void CParser::optionalExpression(BIFSEncoder *pBifs)
{
	if (m_cur != CTID_SEMICOLON) {
		pBifs -> EncodeInt (1, 1, "hasCompoundExpression", "has optional expression"); // mp conformance
		compoundExpression(pBifs );
	} else {
		pBifs -> EncodeInt (0, 1, "hasCompoundExpression", "no optional expression"); // mp conformance
	}
}

void CParser::compoundExpression(BIFSEncoder *pBifs, int startPos, int endPos, bool logParam)
{
	int numExpr;
	int commaPos[MAX_NUM_EXPR];

	if(endPos==0) {
		numExpr = LoadCompExpr(commaPos);
	} else {
		numExpr = ScanCompExpr(startPos, endPos, commaPos);
	}

	Expression(pBifs, commaPos[0], commaPos[1]);
	for (int i=1; i<numExpr; i++) {
        pBifs -> EncodeInt (1, 1, logParam ? "hasParam" : "hasExpression");
		Expression(pBifs, commaPos[i]+1, commaPos[i+1]);
	}
	pBifs -> EncodeInt (0, 1, logParam ? "hasParam" : "hasExpression");
}
//AC200005 : changed to deal with "case compexpr :"
//if a ? has been seen go ahead else return.
int CParser::LoadCompExpr(int* commaPos)
{
	int cnt = 0;
	int numExpr = 1;
	int numInterrogation=0;
	commaPos[0] = 0;
	m_buffer.reset();

	while (
		m_cur != CTID_SEMICOLON && m_cur != CTID_RIGHT_CURVE
		&& m_cur != CTID_EOF
	) {
		if (m_cur == CTID_CONDTEST) numInterrogation++;
		if (m_cur == CTID_CONDSEP) 
		{
			if (numInterrogation > 0)
				numInterrogation--;
			else // should be a case.
				break;
		}
		if (
			m_cur == CTID_IDENTIFIER || m_cur == CTID_NUMBER
			|| m_cur == CTID_STRING || m_cur == CTID_BOOLEAN
		) {
			m_strBuffer.insert(m_lex.GetText());
		}
		m_buffer.insert(m_cur);
		if (m_cur == CTID_LEFT_CURVE) {
			cnt++;
			do {
				m_cur = m_lex();
				if (
					m_cur == CTID_IDENTIFIER
					|| m_cur == CTID_NUMBER
					|| m_cur == CTID_STRING
                    || m_cur == CTID_BOOLEAN
				) {
					m_strBuffer.insert(m_lex.GetText());
				}
				m_buffer.insert(m_cur);
				if (m_cur == CTID_LEFT_CURVE) {
					cnt++;
				} else if (m_cur == CTID_RIGHT_CURVE) {
					cnt--;
				}
			} while (
				(m_cur!=CTID_RIGHT_CURVE || cnt!=0)
				&& m_cur!=CTID_EOF
			);
		} else if (m_cur == CTID_LEFT_BRACKET) {
			cnt++;
			do {
				m_cur = m_lex();
				if (
					m_cur == CTID_IDENTIFIER
					|| m_cur == CTID_NUMBER
					|| m_cur == CTID_STRING
                    || m_cur == CTID_BOOLEAN
				) {
					m_strBuffer.insert(m_lex.GetText());
				}
				m_buffer.insert(m_cur);
				if (m_cur == CTID_LEFT_BRACKET) {
					cnt++;
				} else if (m_cur == CTID_RIGHT_BRACKET) {
					cnt--;
				}
			} while (
				(m_cur!=CTID_RIGHT_BRACKET || cnt!=0)
				&& m_cur!=CTID_EOF
			);
		} else if (m_cur == CTID_COMMA) {
			commaPos[numExpr++] = m_buffer.getPos()-1;
		}
		m_cur = m_lex();
	}
	if (m_cur == CTID_EOF) {
		fprintf(stderr, "Error: Non-end compoundExpression\n");
		exit(-1);
	}
	commaPos[numExpr] = m_buffer.getPos();
	if (
		m_cur == CTID_IDENTIFIER || m_cur == CTID_NUMBER
		|| m_cur == CTID_STRING || m_cur == CTID_BOOLEAN
	) {
		m_strBuffer.insert(m_lex.GetText());
	}
	if (m_cur != CTID_CONDSEP) m_buffer.insert(m_cur);//AC
	return numExpr;
}

int CParser::ScanCompExpr(int startPos, int endPos, int* commaPos)
{
	CTOKEN_CODE curTC;
	int n = startPos;
	int numExpr = 1;

	commaPos[0] = startPos;
	while (n<endPos) {
		curTC = m_buffer[n++];
		if (curTC == CTID_LEFT_CURVE) {
			n = MoveUntil(CTID_RIGHT_CURVE, n-1, endPos);
			if (n==-1) {
				fprintf(stderr, "Error: can't find closing )\n");
				exit(-1);
			}
			n++;
		} else if (curTC == CTID_LEFT_BRACKET) {
			n = MoveUntil(CTID_RIGHT_BRACKET, n-1, endPos);
			if (n==-1) {
				fprintf(stderr, "Error: can't find closing ]\n");
				exit(-1);
			}
			n++;
		} else if (curTC == CTID_COMMA) {
			commaPos[numExpr++] = n-1;
		}
	}
	commaPos[numExpr] = endPos;
	return numExpr;
}

int CParser::MoveUntil(CTOKEN_CODE endTC, int curPos, int endPos)
{
	int cnt = 0;
	CTOKEN_CODE startTC = CTID_EOF, curTC;
	
	if (endTC == CTID_RIGHT_CURVE) {
		startTC = CTID_LEFT_CURVE;
	} else if (endTC == CTID_RIGHT_BRACKET) {
		startTC = CTID_LEFT_BRACKET;
	} else if (endTC == CTID_CONDSEP) {
		startTC = CTID_CONDTEST;
	} else {
		fprintf(stderr, "Error: illegal MoveUntil %d\n", endTC);
		exit(-1);
	}

	do {
		curTC = m_buffer[curPos++];
		if (curTC == startTC) {
			cnt++;
		} else if (curTC == endTC) {
			cnt--;
		}
	} while ( (curTC!=endTC || cnt!=0) && curPos<endPos );
	if (curTC==endTC && cnt==0)
		return curPos-1;
	else 
		return -1;
}
