// This software module was originally developed by Wei Wu(Rockwell Science
// Center) and Wenlong Dong(Rockwell Science Center) in the course of
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wei Wu at weiwu@jupiter.risc.rockwell.com or Wenlong Dong
// at wdong@usc.edu.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	ScriptImp Node
FILE:       CParser.h
AUTHOR:		Wei Wu       (Rockwell Science Center)
			Wenlong Dong (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Interface of the CParser class
--------------------------------------------------------------------*/
#ifndef _CParser_h_
#define _CParser_h_

#include "ScriptDefs.h"
#include "CToken.h"
#include "Buffer.h"
#include "StrBuffer.h"
#include "SFScript.h"

class BIFSEncoder;
//class ScriptImp;	// NODEFIELD

typedef int EXPR_TYPE;

class CParser
{
public:
	CParser(LPCSTR str, int flag=0);
	~CParser(){}
	void Encode(BIFSEncoder *pBifs);
//	void InsertExID(LPCSTR str);
	void SetSNode(ScriptImp *node) { pSNode = node; }	// NODEFIELD

private:
	CTOKEN_CODE m_cur;
	CToken m_lex;

	CBuffer m_buffer; // for compoundExpression
	CStrBuffer m_strBuffer;
//	CStrBuffer m_savedStrBuffer;
	int m_flag;
	int nNodeID; // NODEFIELD
	ScriptImp *pSNode; // NODEFIELD

	void Consume(CTOKEN_CODE);
	void PutIdentifier(LPCSTR str, BIFSEncoder *pBifs);
	void PutNumber(LPCSTR str, BIFSEncoder *pBifs);
	void PutInteger(LPCSTR str, BIFSEncoder *pBifs);
    void PutBoolean(LPCSTR str, BIFSEncoder *pBifs); // mp
	void PutReal(LPCSTR str, BIFSEncoder *pBifs);
	void PutString(LPCSTR str, BIFSEncoder *pBifs);
    void CaseBlock(BIFSEncoder *pBifs); //mp created to handle the specificities of Case Statement Blocks

	void Function(BIFSEncoder *pBifs);
	void Argument(BIFSEncoder *pBifs);
	void StatementBlock(BIFSEncoder *pBifs);
	void Statements(BIFSEncoder *pBifs);
	void Statement(BIFSEncoder *pBifs);
	void ifStatement(BIFSEncoder *pBifs);
	void switchStatement(BIFSEncoder *pBifs);
	void forStatement(BIFSEncoder *pBifs);
	void whileStatement(BIFSEncoder *pBifs);
	void returnStatement(BIFSEncoder *pBifs);
	void breakStatement();
	void continueStatement();
	void compoundExpression(BIFSEncoder *pBifs, int startPos=0, int endPos=0, bool logParam = false); // mp for conformance purposes
	void optionalExpression(BIFSEncoder *pBifs);
	int LoadCompExpr(int*);
	int ScanCompExpr(int startPos, int endPos, int*);
	int MoveUntil(CTOKEN_CODE, int, int);

	EXPR_TYPE Expression(BIFSEncoder *pBifs, int startPos, int endPos, bool n_flag = false);
	void FunctionCall(BIFSEncoder *pBifs, int startPos, int opPosition, int endPos);
	void ObjectMemberAccess(BIFSEncoder *pBifs, int startPos, int opPosition, int endPos);
	void ObjectMethodCall(BIFSEncoder *pBifs, int startPos, int opPosition, int endPos);
	void ArrayDereference(BIFSEncoder *pBifs, int startPos, int opPosition, int endPos);
	void ObjectConstruct(BIFSEncoder *pBifs, int startPos, int opPosition, int endPos);
	void ConditionTest(BIFSEncoder *pBifs, int startPos, int opPosition, int endPos);
	void Params(BIFSEncoder *pBifs, int, int);
	int hasChar(LPCSTR str, int c);
	EXPR_TYPE CParser::convertCTIDtoET(CTOKEN_CODE ctid);//AC 200005
};

#endif // _CParser_h_
