// This software module was originally developed by Wei Wu(Rockwell Science
// Center) and Wenlong Dong(Rockwell Science Center) in the course of 
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wei Wu at weiwu@jupiter.risc.rockwell.com or Wenlong Dong 
// at wdong@usc.edu.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE:       CToken.cpp
AUTHOR:		Wei Wu       (Rockwell Science Center)
			Wenlong Dong (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Implementation of the CToken class
--------------------------------------------------------------------*/
#include "defs.h"
#include <ctype.h>
#include "CToken.h"

CToken::CToken(const char *str)
{
	int length = strlen(str) +1;
	m_buffer = new char[length];
	strcpy(m_buffer, str);

	m_nextPos = 0;
	m_next = GetChar();
}

CTOKEN_CODE CToken::operator() ()
{
	skipWhite();
	if (m_next == 0) {
		m_code = CTID_EOF;
		return m_code;
	}
	m_string = "";
	if (isalpha((char)m_next) || m_next == '_') {
		GetName();
	} else if (isdigit((char)m_next)) {
		GetNumber();
	} else if (m_next=='\'' || m_next=='\"') {
		char ctmp = m_next;
		m_next = GetChar();
		GetString(ctmp);
	} else {
		GetOthers();
	}
	return m_code;
}

void CToken::skipWhite() 
{
	while (
		m_next==' ' || m_next=='\t' || m_next=='\n' ||
		m_next=='\r' || m_next=='\f'||
		m_next == '/' && m_buffer[m_nextPos] == '/' ||
		m_next == '/' && m_buffer[m_nextPos] == '*'
	) { 
		if (m_next=='/' && m_buffer[m_nextPos]=='/') {
			while (m_next != '\n' && m_next != 0) {
				m_next = GetChar();
			}
		} else if (m_next=='/' && m_buffer[m_nextPos]=='*') { 
			GetChar();
			m_next = GetChar();
			for (;;) {
				while (m_next!='*' && m_next!= 0) {
					m_next = GetChar();
				}
				if (m_next == 0) {
					fprintf(stderr, "Closing comment missing\n");
					exit(-1);
				} else {
					m_next = GetChar();
					if (m_next == '/') break;
				}
			}
		} 
		m_next = GetChar();
	}
}

int CToken::GetChar()
{
	if(m_buffer[m_nextPos] == 0)
		return 0;

	return m_buffer[m_nextPos++];
}
	
void CToken::GetName()
{
	while (isalnum((char)m_next) || m_next=='_') {
		m_string += m_next;
		m_next = GetChar();
	}
	m_code = CTID_IDENTIFIER;

	for (int i=0; i<NUMBER_OF_KEYWORD; i++) {
		if (!strcmp(m_string, m_keyword[i])) {
			m_code = (CTOKEN_CODE) i;
			return;
		}
	}

	if (
		!strcmp(m_string, "TRUE") || !strcmp(m_string, "true")
		|| !strcmp(m_string, "FALSE") || !strcmp(m_string, "false")

	) {
		m_code = CTID_BOOLEAN;
	}
}

void CToken::GetNumber()
{
    bool exponent = false;
	while (
		isdigit((char)m_next) || toupper((char)m_next)=='X' ||
		toupper((char)m_next)>='A' && toupper((char)m_next)<='F' ||
		m_next=='.' || tolower(m_next)=='e' || (exponent && m_next == '-')
	) {
		m_string += m_next;
        if (tolower(m_next)=='e') {
            exponent = true;
        }
		m_next = GetChar();
	}
	m_code = CTID_NUMBER;
}

void CToken::GetString(char delimiter)
{
	while (m_next!=delimiter) {
		m_string += m_next;
		m_next = GetChar();
	}
	m_next = GetChar();
	m_code = CTID_STRING;
}

void CToken::GetOthers()
{
	switch (m_next) {
	case '.':
		m_next = GetChar();
		if(isdigit((char)m_next)) {
			m_string += '.';
			GetNumber();
		} else {
			m_code = CTID_PERIOD;
		}
		break;
	case '!':
		m_next = GetChar();
		if(m_next=='=') {
			m_code = CTID_NE;
			m_next = GetChar();
		} else {
			m_code = CTID_NOT;
		}
		break;
	case '=':
		m_next = GetChar();
		if(m_next=='=') {
			m_code = CTID_EQ;
			m_next = GetChar();
		} else {
			m_code = CTID_ASSIGN;
		}
		break; 
	case '+':
		m_next = GetChar();
		if(m_next=='=') {
			m_code = CTID_PLUSEQ;
			m_next = GetChar();
		} else if(m_next=='+') {
			m_code = CTID_INCREMENT;
			m_next = GetChar();
		} else {
			m_code = CTID_PLUS;
		}
		break;
	case '-':
		m_next = GetChar();
		if(m_next=='=') {
			m_code = CTID_MINUSEQ;
			m_next = GetChar();
		} else if(m_next == '-') {
			m_code = CTID_DECREMENT;
			m_next = GetChar();
		} else {
			m_code = CTID_MINUS;
		}
		break;
	case '*':
		m_next = GetChar();
		if(m_next=='=') {
			m_code = CTID_MULTIPLYEQ;
			m_next = GetChar();
		} else {
			m_code = CTID_MULTIPLY;
		}
		break;
	case '/':
		m_next = GetChar();
		if(m_next=='=') {
			m_code = CTID_DIVIDEEQ;
			m_next = GetChar();
		} else {
			m_code = CTID_DIVIDE;
		}
		break;
	case '%':
		m_next = GetChar();
		if(m_next=='=') {
			m_code = CTID_MODEQ;
			m_next = GetChar();
		} else {
			m_code = CTID_MOD;
		}
		break;
	case '&':
		m_next = GetChar();
		if(m_next=='=') {
			m_code = CTID_ANDEQ;
			m_next = GetChar();
		} else if(m_next=='&') {
			m_code = CTID_LAND;
			m_next = GetChar();
		} else {
			m_code = CTID_AND;
		}
		break;
	case '|':
		m_next = GetChar();
		if(m_next=='=') {
			m_code = CTID_OREQ;
			m_next = GetChar();
		} else if(m_next=='|') {
			m_code = CTID_LOR;
			m_next = GetChar();
		} else {
			m_code = CTID_OR;
		}
		break;
	case '^':
		m_next = GetChar();
		if(m_next=='=') {
			m_code = CTID_XOREQ;
			m_next = GetChar();
		} else {
			m_code = CTID_XOR;
		}
		break;
	case '<':
		m_next = GetChar();
		if(m_next=='<') {
			m_next = GetChar();
			if(m_next=='=') {
				m_code = CTID_LSHIFTEQ;
				m_next = GetChar();
			} else {
				m_code = CTID_LSHIFT;
			}
		} else if(m_next=='=') {
			m_code = CTID_LE;
			m_next = GetChar();
		} else {
			m_code = CTID_LT;
		}
		break;
	case '>':
		m_next = GetChar();
		if(m_next=='>') {
			m_next = GetChar();
			if(m_next=='=') {
				m_code = CTID_RSHIFTEQ;
				m_next = GetChar();
			} else if(m_next=='>') {
				m_next = GetChar();
				if(m_next=='=') {
					m_code = CTID_RSHIFTFILLEQ;
					m_next = GetChar();
				} else {
					m_code = CTID_RSHIFTFILL;
				}
			} else {
				m_code = CTID_RSHIFT;
			}
		} else if(m_next=='=') {
			m_code = CTID_GE;
			m_next = GetChar();
		} else {
			m_code = CTID_GT;
		}
		break;
	case '?':
		m_next = GetChar();
		m_code = CTID_CONDTEST;
		break;
	case ':':
		m_next = GetChar();
		m_code = CTID_CONDSEP;
		break;
	case '~':
		m_next = GetChar();
		m_code = CTID_ONESCOMP;
		break;
	case ',':
		m_next = GetChar();
		m_code = CTID_COMMA;
		break;
	case ';':
		m_next = GetChar();
		m_code = CTID_SEMICOLON;
		break;
	case '{':
		m_next = GetChar();
		m_code = CTID_LEFT_BRACE;
		break;
	case '}':
		m_next = GetChar();
		m_code = CTID_RIGHT_BRACE;
		break;
	case '(':
		m_next = GetChar();
		m_code = CTID_LEFT_CURVE;
		break;
	case ')':
		m_next = GetChar();
		m_code = CTID_RIGHT_CURVE;
		break;
	case '[':
		m_next = GetChar();
		m_code = CTID_LEFT_BRACKET;
		break;
	case ']':
		m_next = GetChar();
		m_code = CTID_RIGHT_BRACKET;
		break;
	default:
		fprintf(stderr, "Unrecognized symbol %c\n", m_next);
		exit(-1);
	}
}

const char* CToken::m_keyword [] =
{
	"function",
	"if",
	"else",
	"for",
	"while",
	"return",
	"break",
	"continue",
	"new",
	"switch","case","default" //AC200005
};
