// This software module was originally developed by Wei Wu(Rockwell Science
// Center) and Wenlong Dong(Rockwell Science Center) in the course of 
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wei Wu at weiwu@jupiter.risc.rockwell.com or Wenlong Dong 
// at wdong@usc.edu.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE:       CToken.cpp
AUTHOR:		Wei Wu       (Rockwell Science Center)
			Wenlong Dong (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Interface of the CToken class
--------------------------------------------------------------------*/
#ifndef _CToken_h_
#define _CToken_h_

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include "ztl.h"

typedef enum 
{
	CTID_FUNCTION,
	CTID_IF,
	CTID_ELSE,
	CTID_FOR,
	CTID_WHILE,
	CTID_RETURN,
	CTID_BREAK,
	CTID_CONTINUE,
	CTID_NEW,
	CTID_SWITCH,
	CTID_CASE,
	CTID_DEFAULT,
	NUMBER_OF_KEYWORD,
	CTID_LEFT_BRACE = NUMBER_OF_KEYWORD,
	CTID_RIGHT_BRACE,
	CTID_LEFT_CURVE,
	CTID_RIGHT_CURVE,
	CTID_LEFT_BRACKET,
	CTID_RIGHT_BRACKET,
	CTID_PERIOD,
	CTID_NOT,
	CTID_ONESCOMP,
	CTID_NEGATIVE,
	CTID_INCREMENT,
	CTID_DECREMENT,
	CTID_MULTIPLY,
	CTID_DIVIDE,
	CTID_MOD,
	CTID_PLUS,
	CTID_MINUS,
	CTID_LSHIFT,
	CTID_RSHIFT,
	CTID_RSHIFTFILL,
	CTID_LT,
	CTID_LE,
	CTID_GT,
	CTID_GE,
	CTID_EQ,
	CTID_NE,
	CTID_AND,
	CTID_XOR,
	CTID_OR,
	CTID_LAND,
	CTID_LOR,
	CTID_CONDTEST,
	CTID_ASSIGN,
	CTID_PLUSEQ,
	CTID_MINUSEQ,
	CTID_MULTIPLYEQ,
	CTID_DIVIDEEQ,
	CTID_MODEQ,
	CTID_LSHIFTEQ,
	CTID_RSHIFTEQ,
	CTID_RSHIFTFILLEQ,
	CTID_ANDEQ,
	CTID_XOREQ,
	CTID_OREQ,
	CTID_COMMA,
	CTID_SEMICOLON,
	CTID_CONDSEP,
	CTID_IDENTIFIER,
	CTID_STRING,
	CTID_NUMBER,
	CTID_EOF,
    CTID_BOOLEAN // mp new token
} CTOKEN_CODE;

class CToken
{
public:
	CToken(const char *str);
	LPCSTR GetText() {return m_string;}
	CTOKEN_CODE operator () (); // get next token

private:
	CTOKEN_CODE m_code;
	ZString m_string;
	int m_next; // next char or EOF
	int m_nextPos;
	static const char* m_keyword [];
	char *m_buffer;

	int GetChar();
	void skipWhite();
	void GetName();
	void GetNumber();
	void GetString(char);
	void GetOthers();
};

#endif // _CToken_h_
