// This software module was originally developed by Wei Wu(Rockwell Science
// Center) and Wenlong Dong(Rockwell Science Center) in the course of
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wei Wu at weiwu@jupiter.risc.rockwell.com or Wenlong Dong
// at wdong@usc.edu.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE:       Expr.cpp
AUTHOR:		Wei Wu       (Rockwell Science Center)
			Wenlong Dong (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Implementation of the CParser class
--------------------------------------------------------------------*/
#include "defs.h"
#include "BifsEnc.h"
#include "CParser.h"
#include "VSNodeField.h"
#include "Token.h"
#include "SFScript.h" // NODEFIELD
#include "MediaObj.h" // NODEFIELD
#include "Root.h" // NODEFIELD
#include "NodeFld.h" // NODEFIELD

static const char *exprDesc[] ={
"ET_CURVED_EXPR",
"ET_NEGATIVE",
"ET_NOT",
"ET_ONESCOMP",
"ET_INCREMENT",
"ET_DECREMENT",
"ET_POST_INCREMENT",
"ET_POST_DECREMENT",
"ET_CONDTEST",
"ET_STRING",
"ET_NUMBER",
"ET_IDENTIFIER",
"ET_FUNCTION_CALL",
"ET_NEW",
"ET_OBJECT_MEMBER_ACCESS",
"ET_OBJECT_METHOD_CALL",
"ET_ARRAY_DEREFERENCE",
"ET_ASSIGN",
"ET_PLUSEQ",
"ET_MINUSEQ",
"ET_MULTIPLYEQ",
"ET_DIVIDEEQ",
"ET_MODEQ",
"ET_ANDEQ",
"ET_OREQ",
"ET_XOREQ",
"ET_LSHIFTEQ",
"ET_RSHIFTEQ",
"ET_RSHIFTFILLEQ",
"ET_EQ",
"ET_NE",
"ET_LT",
"ET_LE",
"ET_GT",
"ET_GE",
"ET_PLUS",
"ET_MINUS",
"ET_MULTIPLY",
"ET_DIVIDE",
"ET_MOD",
"ET_LAND",
"ET_LOR",
"ET_AND",
"ET_OR",
"ET_XOR",
"ET_LSHIFT",
"ET_RSHIFT",
"ET_RSHIFTFILL",
"ET_BOOLEAN",
"NUMBER_OF_EXPR_TYPE"
};
#define NUMBER_OF_RANK	15

static int rank[NUMBER_OF_EXPR_TYPE] =
{
1,// ET_CURVED_EXPR
2,// ET_NEGATIVE
2,// ET_NOT
2,// ET_ONESCOMP
2,// ET_INCREMENT
2,// ET_DECREMENT
2,// ET_POST_INCREMENT
2,// ET_POST_DECREMENT
14,// ET_CONDTEST
0,// ET_STRING
0,// ET_NUMBER
0,// ET_IDENTIFIER
1,// ET_FUNCTION_CALL
2,// ET_NEW
1,// ET_OBJECT_MEMBER_ACCESS
1,// ET_OBJECT_METHOD_CALL
1,// ET_ARRAY_DEREFERENCE
14,// ET_ASSIGN
14,// ET_PLUSEQ
14,// ET_MINUSEQ
14,// ET_MULTIPLYEQ
14,// ET_DIVIDEEQ
14,// ET_MODEQ
14,// ET_ANDEQ
14,// ET_OREQ
14,// ET_XOREQ
14,// ET_LSHIFTEQ
14,// ET_RSHIFTEQ
14,// ET_RSHIFTFILLEQ
8,// ET_EQ
9,// ET_NE
6,// ET_LT
6,// ET_LE
7,// ET_GT
7,// ET_GE
5,// ET_PLUS
5,// ET_MINUS
3,// ET_MULTIPLY
3,// ET_DIVIDE
3,// ET_MOD
13,// ET_LAND
14,// ET_LOR
10,// ET_AND
12,// ET_OR
11,// ET_XOR
5,// ET_LSHIFT
6,// ET_RSHIFT
6,// ET_RSHIFTFILL
0 // ET_BOOLEAN

/*
	0, 0, 0,			// variable, number, string
	1, 1, 1, 1, 1,			// curved expr, call, member, array
	2, 2, 2, 2, 2, 2, 2, 2,		// unary operator
	3, 3, 3,			// multiply, divide, mod
	4, 4,				// add, subtract
	5, 5, 5,			// bitwise shift
	6, 6, 6, 6,			// relational
	7, 7,				// equality
	8,				// bitwise and
	9,				// bitwise xor
	10,				// bitwise or
	11,				// logical and
	12,				// logical or
	13,				// conditional
	14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14, 14	// assignment
	*/
};

static int leftAssoc[NUMBER_OF_RANK] = {1, 1, 0, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 0};

#define CheckToken(x)	if (curTC!=x) { \
				fprintf(stderr, "Error: Token %d read, %d expected\n", \
					curTC, x); \
				exit (-1); }
//AC 200005 : convert CTID -> ET
EXPR_TYPE CParser::convertCTIDtoET(CTOKEN_CODE ctid)

{
	switch(ctid)
	{
		case CTID_INCREMENT:
			return ET_INCREMENT;
			break;
		case CTID_DECREMENT:
			return ET_DECREMENT;
			break;
		case CTID_NOT:
			return ET_NOT;
			break;
		case CTID_ONESCOMP:
			return ET_ONESCOMP;
			break;
		case CTID_MULTIPLY : return ET_MULTIPLY;
		case CTID_DIVIDE : return ET_DIVIDE;
		case CTID_MOD : return ET_MOD;
		case CTID_PLUS : return ET_PLUS;
		case CTID_MINUS : return ET_MINUS;
		case CTID_LSHIFT : return ET_LSHIFT;
		case CTID_RSHIFT : return ET_RSHIFT;
		case CTID_RSHIFTFILL : return ET_RSHIFTFILL;
		case CTID_LT : return ET_LT;
		case CTID_LE : return ET_LE;
		case CTID_GT : return ET_GT;
		case CTID_GE : return ET_GE;
		case CTID_EQ : return ET_EQ;
		case CTID_NE : return ET_NE;
		case CTID_AND : return ET_AND;
		case CTID_XOR : return ET_XOR;
		case CTID_OR : return ET_OR;
		case CTID_LAND : return ET_LAND;
		case CTID_LOR : return ET_LOR;
		case CTID_CONDTEST : return ET_CONDTEST;
		case CTID_ASSIGN : return ET_ASSIGN;
		case CTID_PLUSEQ : return ET_PLUSEQ;
		case CTID_MINUSEQ : return ET_MINUSEQ;
		case CTID_MULTIPLYEQ : return ET_MULTIPLYEQ;
		case CTID_DIVIDEEQ : return ET_DIVIDEEQ;
		case CTID_MODEQ : return ET_MODEQ;
		case CTID_LSHIFTEQ : return ET_LSHIFTEQ;
		case CTID_RSHIFTEQ : return ET_RSHIFTEQ;
		case CTID_RSHIFTFILLEQ : return ET_RSHIFTFILLEQ;
		case CTID_ANDEQ : return ET_ANDEQ;
		case CTID_XOREQ : return ET_XOREQ;
		case CTID_OREQ : return ET_OREQ;
		default:
			fprintf(stderr,"Error: [CTID CParser::convertCTIDtoET] illegal ctid\n");
			return -1;
	}
}
EXPR_TYPE CParser::Expression(BIFSEncoder *pBifs, int startPos, int endPos, bool n_flag)
{
	int n = startPos;
	int curPos = 0, finalPos = 0;
	CTOKEN_CODE curTC, prevTC;
	EXPR_TYPE curExprType = 0, exprType = 0;
	int curRank, maxRank=0;

	curTC = prevTC = m_buffer[n];

	do {
		curPos = n;
		switch (curTC) {
		case CTID_IDENTIFIER:
			curExprType = ET_IDENTIFIER;
			break;
		case CTID_NUMBER:
			curExprType = ET_NUMBER;
			break;
        case CTID_BOOLEAN:
            curExprType = ET_BOOLEAN;
            break;
		case CTID_STRING:
			curExprType = ET_STRING;
			break;
		case CTID_LEFT_CURVE:
			if (prevTC==CTID_IDENTIFIER) {
				curExprType = ET_FUNCTION_CALL;
			} else {
				curExprType = ET_CURVED_EXPR;
			}
			n = MoveUntil(CTID_RIGHT_CURVE, n, endPos);
			if (n!=-1) {
				curTC = CTID_RIGHT_CURVE;
			} else {
				fprintf(stderr, "Error: can't find closing )\n");
				exit(-1);
			}
			break;
		case CTID_LEFT_BRACKET:
			curExprType = ET_ARRAY_DEREFERENCE;
			n = MoveUntil(CTID_RIGHT_BRACKET, n, endPos);
			if (n!=-1) {
				curTC = CTID_RIGHT_BRACKET;
			} else {
				fprintf(stderr, "Error: can't find closing )\n");
				exit(-1);
			}
			break;
		case CTID_PERIOD:
			curTC = m_buffer[++n];
			CheckToken(CTID_IDENTIFIER);
			if (m_buffer[n+1]==CTID_LEFT_CURVE) {
				curExprType = ET_OBJECT_METHOD_CALL;
				n = MoveUntil(CTID_RIGHT_CURVE, n+1, endPos);
				if (n!=-1){
					curTC = CTID_RIGHT_CURVE;
				} else {
					fprintf(stderr, "Error: can't find closing )\n");
					exit(-1);
				}
			} else {
				curExprType = ET_OBJECT_MEMBER_ACCESS;
			}
			break;
		case CTID_NEW:
			curExprType = ET_NEW;
			curTC = m_buffer[++n];
			CheckToken(CTID_IDENTIFIER);
			curTC = m_buffer[++n];
			CheckToken(CTID_LEFT_CURVE);
			n = MoveUntil(CTID_RIGHT_CURVE, n, endPos);
			if (n!=-1) {
				curTC = CTID_RIGHT_CURVE;
			} else {
				fprintf(stderr, "Error: can't find closing )\n");
				exit(-1);
			}
			break;
		case CTID_MINUS:
			if (
				prevTC==CTID_RIGHT_CURVE || prevTC==CTID_RIGHT_BRACKET ||
				prevTC==CTID_IDENTIFIER || prevTC==CTID_NUMBER ||
				prevTC==CTID_STRING || prevTC==CTID_INCREMENT ||
				prevTC==CTID_DECREMENT
			) {
				curExprType = ET_MINUS;
			} else {
				curExprType = ET_NEGATIVE;
				curTC = CTID_NEGATIVE;
			}
			break;
		case CTID_INCREMENT:
			curExprType = ET_INCREMENT;
			break;
		case CTID_DECREMENT:
			curExprType = ET_DECREMENT;
			break;
		case CTID_NOT:
			curExprType = ET_NOT;
			break;
		case CTID_ONESCOMP:
			curExprType = ET_ONESCOMP;
			break;
		case CTID_CONDTEST:
			curExprType = ET_CONDTEST;
			break;
		case CTID_CONDSEP:
			break;
		default: // hack to reduce typing
			if (curTC < CTID_MULTIPLY || curTC > CTID_OREQ) {
				fprintf(stderr, "Error: illegal token %d read\n",
					curTC);
				exit(-1);
			}
			//AC 200005
			//curExprType =(EXPR_TYPE)(ET_MULTIPLY + (curTC - CTID_MULTIPLY));
			curExprType=convertCTIDtoET(curTC);
			//AC - end
		}
		//??? (curExprType==ET_CONDTEST)
		if (curTC == CTID_CONDSEP) {
			prevTC = curTC;
			curTC = m_buffer[++n];
			continue;
		}
		curRank = rank[curExprType];
		if (curRank > maxRank) {
			maxRank = curRank;
			exprType = curExprType;
			finalPos = curPos;
		} else if (curRank == maxRank && leftAssoc[curRank]) {
			exprType = curExprType;
			finalPos = curPos;
		}
		prevTC = curTC;
		curTC = m_buffer[++n];
	} while (n<endPos);

	if (exprType == ET_INCREMENT) {
		if (finalPos==startPos);
		else if (finalPos==endPos-1) exprType = ET_POST_INCREMENT;
		else {
			fprintf(stderr, "Error: illegal Increment expression\n");
			exit(-1);
		}
	} else if (exprType == ET_DECREMENT) {
		if (finalPos==startPos);
		else if (finalPos==endPos-1) exprType = ET_POST_DECREMENT;
		else {
			fprintf(stderr, "Error: illegal Decrement expression\n");
			exit(-1);
		}
	}

	pBifs -> EncodeInt ((int) exprType, NUMBITS_EXPR_TYPE, "expressionType", exprDesc[exprType]);
	//AC 200005
	switch (exprType)
	{
			case ET_MULTIPLY :
			case ET_DIVIDE :
			case ET_MOD :
			case ET_PLUS :
			case ET_MINUS :
			case ET_LSHIFT :
			case ET_RSHIFT :
			case ET_RSHIFTFILL :
			case ET_LT :
			case ET_LE :
			case ET_GT :
			case ET_GE :
			case ET_EQ :
			case ET_NE :
			case ET_AND :
			case ET_XOR :
			case ET_OR :
			case ET_LAND :
			case ET_LOR :
				Expression(pBifs, startPos, finalPos);
				Expression(pBifs, finalPos+1, endPos);
				break;
			/******************************************/
			case ET_ASSIGN :
			case ET_PLUSEQ :
			case ET_MINUSEQ :
			case ET_MULTIPLYEQ :
			case ET_DIVIDEEQ :
			case ET_MODEQ :
			case ET_LSHIFTEQ :
			case ET_RSHIFTEQ :
			case ET_RSHIFTFILLEQ :
			case ET_ANDEQ :
			case ET_XOREQ :
			case ET_OREQ :
				{ //begin
				EXPR_TYPE ret = Expression(pBifs, startPos, finalPos);
				if (
					ret != ET_IDENTIFIER && ret != ET_OBJECT_MEMBER_ACCESS &&
					ret != ET_ARRAY_DEREFERENCE
				) {
					fprintf(stderr, "Error: LeftVariable expected, %s returned\n",
						exprDesc[ret]);
					exit(-1);
				}
				Expression(pBifs, finalPos+1, endPos);
				} //end
				break;
			/******************************************/
			case ET_IDENTIFIER:
				{ //begin
				const char *str = m_strBuffer.getStr();
				PutIdentifier(str, pBifs);
				if(n_flag) // n_flag is set when the call is from Object Method call or Member access
				{
					nNodeID = -1; // NODEFIELD: should be named nNodeCode
					for (int i=0; i<(pSNode->m_fieldTable.GetLength()); i++) { // NODEFIELD
						const BifsFieldTable *pEntry = & pSNode->m_fieldTable[i];
						if (!strcmp(pEntry->m_szName, str)) {
							nNodeID = pEntry->m_defID;
							break;
						}
					}
					if (nNodeID == -1) {
						fprintf(stderr, "Error: node ID of %s not found\n", str);
						exit(-1);
					}

				}//end
                }
				break;
			case ET_NUMBER:
				{//begin
				const char *str = m_strBuffer.getStr();
				PutNumber(str, pBifs);
				}//end
				break;
            case ET_BOOLEAN:
				{//begin
				const char *str = m_strBuffer.getStr();
				PutBoolean(str, pBifs);
				}//end
				break;
			case ET_STRING:
				{//begin
				const char *str = m_strBuffer.getStr();
				PutString(str, pBifs);
				}//end
				break;
			case ET_NEGATIVE :
			case ET_INCREMENT :
			case ET_DECREMENT :
			case ET_NOT :
			case ET_ONESCOMP:
				{//begin
				Expression(pBifs, finalPos+1, endPos);
				}//end
				break;
			case  ET_CURVED_EXPR:
				{//begin
				compoundExpression(pBifs, finalPos+1, endPos-1);
				}//end
				break;
			case  ET_POST_INCREMENT :
			case  ET_POST_DECREMENT :
				{//begin
				Expression(pBifs, startPos, finalPos);
				}//end
				break;
			case  ET_FUNCTION_CALL:
				{//begin
				FunctionCall(pBifs, startPos, finalPos, endPos);
				}//end
				break;
			case  ET_OBJECT_MEMBER_ACCESS :
				{//begin
				ObjectMemberAccess(pBifs, startPos, finalPos, endPos);
				}//end
				break;
			case  ET_OBJECT_METHOD_CALL:
				{//begin
				ObjectMethodCall(pBifs, startPos, finalPos, endPos);
				}//end
				break;
			case  ET_ARRAY_DEREFERENCE:
				{//begin
				ArrayDereference(pBifs, startPos, finalPos, endPos);
				}//end
				break;
			case  ET_NEW:
				{//begin
				ObjectConstruct(pBifs, startPos, finalPos, endPos);
				}//end
				break;
			case  ET_CONDTEST:
				{//begin
				ConditionTest(pBifs, startPos, finalPos, endPos);
				}//end
				break;
//			case  :
//				{//begin
//				}//end
//				break;
			default:
				fprintf(stderr, "Error: illegal expression type %d\n", exprType);
				exit(-1);
				break;
	}
	//end switch
	/*if (exprType >= ET_MULTIPLY && exprType <= ET_LOR) { // hack
		Expression(pBifs, startPos, finalPos);
		Expression(pBifs, finalPos+1, endPos);
	} else if(exprType >= ET_ASSIGN && exprType <=ET_OREQ) { // hack
		EXPR_TYPE ret = Expression(pBifs, startPos, finalPos);
		if (
			ret != ET_IDENTIFIER && ret != ET_OBJECT_MEMBER_ACCESS &&
			ret != ET_ARRAY_DEREFERENCE
		) {
			fprintf(stderr, "Error: LeftVariable expected, %s returned\n",
				exprDesc[ret]);
			exit(-1);
		}
		Expression(pBifs, finalPos+1, endPos);
	} else if(exprType == ET_IDENTIFIER) {
		const char *str = m_strBuffer.getStr();
		if(n_flag) // n_flag is set when the call is from Object Method call or Member access
		{
			nNodeID = -1; // NODEFIELD: should be named nNodeCode
			if(!strcmp(str, "Math"))
				pBifs->EncodeInt(1, 1, "Math identifier");
			else {
				pBifs->EncodeInt(0, 1, "Not Math identifier");
				PutIdentifier(str, pBifs);
				for (int i=0; i<pSNode->m_fields.GetLength(); i++) { // NODEFIELD
					if(!strcmp(pSNode->m_fields[i]->GetName(), str)) {
						nNodeID = pSNode->m_fields[i]->GetID();
						break;
					}
				}
				if (nNodeID == -1) {
					fprintf(stderr, "Error: node ID of %s not found\n", str);
					exit(-1);
				}
			}
		}
		else
			PutIdentifier(str, pBifs);

	} else if(exprType == ET_NUMBER) {
		const char *str = m_strBuffer.getStr();
		PutNumber(str, pBifs);
	} else if(exprType == ET_STRING) {
		const char *str = m_strBuffer.getStr();
		PutString(str, pBifs);
	} else if (
		exprType == ET_NEGATIVE || exprType == ET_INCREMENT ||
		exprType == ET_DECREMENT || exprType == ET_NOT ||
		exprType == ET_ONESCOMP
	) {
		Expression(pBifs, finalPos+1, endPos);
	} else if (exprType == ET_CURVED_EXPR) {
		compoundExpression(pBifs, finalPos+1, endPos-1);
	} else if (exprType == ET_POST_INCREMENT || exprType == ET_POST_DECREMENT) {
		Expression(pBifs, startPos, finalPos);
	} else if (exprType == ET_FUNCTION_CALL) {
		FunctionCall(pBifs, startPos, finalPos, endPos);
	} else if (exprType == ET_OBJECT_MEMBER_ACCESS) {
		ObjectMemberAccess(pBifs, startPos, finalPos, endPos);
	} else if (exprType == ET_OBJECT_METHOD_CALL) {
		ObjectMethodCall(pBifs, startPos, finalPos, endPos);
	} else if (exprType == ET_ARRAY_DEREFERENCE) {
		ArrayDereference(pBifs, startPos, finalPos, endPos);
	} else if (exprType == ET_NEW) {
		ObjectConstruct(pBifs, startPos, finalPos, endPos);
	} else if (exprType == ET_CONDTEST) {
		ConditionTest(pBifs, startPos, finalPos, endPos);
	} else {
		fprintf(stderr, "Error: illegal expression type %d\n", exprType);
		exit(-1);
	}
	*/
	return exprType;
}

void CParser::FunctionCall(BIFSEncoder *pBifs, int startPos, int, int endPos)
{
	CTOKEN_CODE curTC;

	curTC = m_buffer[startPos++];
	CheckToken(CTID_IDENTIFIER);
	const char *str = m_strBuffer.getStr();
	PutIdentifier(str, pBifs);
	curTC = m_buffer[startPos++];
	CheckToken(CTID_LEFT_CURVE);
	Params(pBifs, startPos, endPos-1);
	curTC = m_buffer[endPos-1];
	CheckToken(CTID_RIGHT_CURVE);
}

void CParser::ObjectMemberAccess(BIFSEncoder *pBifs, int startPos, int opPosition, int endPos)
{
	CTOKEN_CODE curTC;

	Expression(pBifs, startPos, opPosition, true);
	curTC = m_buffer[opPosition];
	CheckToken(CTID_PERIOD);
	curTC = m_buffer[endPos-1];
	CheckToken(CTID_IDENTIFIER);
	const char *str = m_strBuffer.getStr();
    PutIdentifier(str, pBifs); // mp
	/*if (nNodeID == -1) { // NODEFIELD: Math case
		PutString(str, pBifs); // Put the string directly
	} else {
		MediaObject *pNode = SFGenericNode().CreateNode(nNodeID);
		const BifsFieldTable *pEntry = pNode->GetBifsEntry(str);
		if (pEntry) {
			char buffer[10];
//			pBifs->EncodeInt(pEntry->m_defID, pNode->GetBifsBounds()->m_nBitsPerDef, "fieldReference");
			_itoa(pEntry->m_defID, buffer, 10);
			PutNumber(buffer, pBifs);
		} else {
			fprintf(stderr, "unrecognized field\n");
			exit(-1);
		}
	}*/
}

void CParser::ObjectMethodCall(BIFSEncoder *pBifs, int startPos, int opPosition, int endPos)
{
	CTOKEN_CODE curTC;

	Expression(pBifs, startPos, opPosition);
	curTC = m_buffer[opPosition];
	CheckToken(CTID_PERIOD);
	curTC = m_buffer[opPosition+1];
	CheckToken(CTID_IDENTIFIER);
	const char *str = m_strBuffer.getStr();
	//PutString(str, pBifs); // Put the string directly
    PutIdentifier(str, pBifs); // mp
	curTC = m_buffer[opPosition+2];
	CheckToken(CTID_LEFT_CURVE);
	Params(pBifs, opPosition+3, endPos-1);
	curTC = m_buffer[endPos-1];
	CheckToken(CTID_RIGHT_CURVE);
}

void CParser::ArrayDereference(BIFSEncoder *pBifs, int startPos, int opPosition, int endPos)
{
	CTOKEN_CODE curTC;

	Expression(pBifs, startPos, opPosition);
	curTC = m_buffer[opPosition];
	CheckToken(CTID_LEFT_BRACKET);
	compoundExpression(pBifs, opPosition+1, endPos-1);
	curTC = m_buffer[endPos-1];
	CheckToken(CTID_RIGHT_BRACKET);
}

void CParser::ObjectConstruct(BIFSEncoder *pBifs, int startPos, int, int endPos)
{
	CTOKEN_CODE curTC;

	curTC = m_buffer[startPos++];
	CheckToken(CTID_NEW);
	curTC = m_buffer[startPos++];
	CheckToken(CTID_IDENTIFIER);
	const char *str = m_strBuffer.getStr();
	/*mp this is wrong
    int i;
	FIELD_TYPE type = UNKNOWN_TYPE;
//	for(i = SFWORLDNODE; i <= MFVEC3F; i++)
	for(i = SFNODE; i<=MFVEC3F; i++)
	{
		if(!strcmp(str, VSNodeField::field_reserved[i]))
		{
			type = (FIELD_TYPE)i;
			break;
		}
	}

	pBifs -> EncodeInt((int)type, 6, "Field ID number for new method");
	if(type == UNKNOWN_TYPE)
		nextToken.Error("Unrecognized Object type"); */

	PutIdentifier(str, pBifs);
	curTC = m_buffer[startPos++];
	CheckToken(CTID_LEFT_CURVE);
	Params(pBifs, startPos, endPos-1);
	curTC = m_buffer[endPos-1];
	CheckToken(CTID_RIGHT_CURVE);
}

void CParser::ConditionTest(BIFSEncoder *pBifs, int startPos, int opPosition, int endPos)
{
	CTOKEN_CODE curTC;

	Expression(pBifs, startPos, opPosition);
	curTC = m_buffer[opPosition];
	CheckToken(CTID_CONDTEST);
	startPos = opPosition+1;
	opPosition = MoveUntil(CTID_CONDSEP, opPosition, endPos-1);
	if (opPosition != -1) {
		Expression(pBifs, startPos, opPosition);
		curTC = m_buffer[opPosition];
		CheckToken(CTID_CONDSEP);
		Expression(pBifs, opPosition+1, endPos);
	} else {
		fprintf(stderr, "Error: can't find : for condition test\n");
		exit(-1);
	}
}

void CParser::Params(BIFSEncoder *pBifs, int startPos, int endPos)
{
	CTOKEN_CODE curTC;

	curTC = m_buffer[startPos];
	if (curTC != CTID_RIGHT_CURVE) {
		pBifs -> EncodeInt (1, 1, "hasParam"); // mp conformance
		compoundExpression(pBifs, startPos, endPos, true);
	} else {
		pBifs -> EncodeInt (0, 1, "hasParam"); // mp conformance
	}
}
