// This software module was originally developed by Wenlong Dong(Rockwell
// Science Center) and Wei Wu(Rockwell Science Center) in the course of
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wenlong Dong at wdong@usc.edu or Wei Wu at
// weiwu@jupiter.risc.rockwell.com.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE:       ScriptImp.h
AUTHOR:     Boris Felts  (Envivio)

OVERVIEW:   Implementation of the Script Node
--------------------------------------------------------------------*/
#if !defined(_ScriptImp_h__)
#define _ScriptImp_h__

//#include "nodes.h"
#include "Nodes.h" // SOLARIS PORTING - typo
#include "VSNodeField.h"
#include "SysCoder.h"

class BIFSEncoder;
class CParser;

// mp : this class overload a script node.
// there is a trick concerning the interface fields.
// a regular field is declared, and added to the pEntry, 
// but a VSNodeField is declared at the same time, linjed to the reglar one.
// thus when a routeEvent arrive to the script, the overloaded RouteEvent
// method make sure the regular field take the associated value and teh VSNodeField
// process the associated function.

class ScriptImp : public Script
{
public:
	ScriptImp();
	virtual ~ScriptImp();
	BOOL operator == (const ScriptImp &node) const;


	BOOL InScriptParsing() { return FALSE; }
	
	void addIdentifier(ZString name){ m_fieldNames += name;}
	int findIdentifier(ZString name){return m_fieldNames.Find(name);}
	ZString identifierAt(int index){return m_fieldNames[index];}
	int	identifierLength() {return m_fieldNames.GetLength();}

	// overloaded methods
	virtual const BifsFieldTable *GetFieldTable () const {return m_fieldTable;}
	virtual const BifsBounds *GetBifsBounds () const 	 {return &m_bifsBounds;}
	virtual int GetChildrenOffset () const {return m_nChildrenOffset;}
	virtual int GetFieldCount () const {return m_fieldCount;}
	NodeField *GetField (int index) {return m_fields [index]; }
	
	//reset all fields
	void Reset();

private:
	NodeField * CreateInterfaceField (
		SysCoder *pCoder,
		int	 eventType,
		FIELD_TYPE fieldType,
		LPCSTR szName
		);
	ZArray<NodeField *> m_fields;
	ZArray<ZString> m_fieldNames;

public:
	// overloaded attributes
	int m_nChildrenOffset;
	int m_fieldCount;
	ZArray<BifsFieldTable> m_fieldTable;
	BifsBounds m_bifsBounds;


public:
	virtual BOOL Encode (BIFSEncoder *pBifs);
	void ParseField (BIFSEncoder *pBifs);
private:
	ZString GetHeader(ZString string, int &pos);
	enum SCRIPT_EVENT_TYPE Scan_eType();
	CParser *pCParser;
    enum { URL_BEFORE, URL_INSIDE, URL_AFTER } urlState;
    void parseScriptField(BIFSEncoder *pCoder, int eventType);
    void encodeUrlStart(BIFSEncoder * pCoder);
    void encodeRegularScriptFields(BIFSEncoder * pCoder);
    void encodeUrlField(BIFSEncoder * pCoder);
    struct {
        char * fieldName;
        bool isDefault;
        bool value;
        bool hasProto;
        bool ISedField;
        int  ISedfieldRef;
        int  protoNbBits;
    } fieldBuffer[2];

};

#endif // _ScriptImp_h__

