// This software module was originally developed by Wei Wu(Rockwell Science
// Center) and Wenlong Dong(Rockwell Science Center) in the course of 
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wei Wu at weiwu@jupiter.risc.rockwell.com or Wenlong Dong 
// at wdong@usc.edu.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE: 		StrBuffer.cpp
AUTHOR:		Wei Wu       (Rockwell Science Center)
			Wenlong Dong (Rockwell Science Center)
			Yuval Fisher (Rockwell Science Center)
OVERVIEW:	Implementation of CStrBuffer class
----------------------------------------------------------------------*/
#include "defs.h"
#include "StrBuffer.h"

CStrBuffer::CStrBuffer()
{
	m_size = DEF_BUF_SIZE;
	m_headPos = m_curPos = 0;
	m_buf = (char**) malloc(m_size*sizeof(char*));
}

CStrBuffer::~CStrBuffer()
{
	for (int i=0; i<m_curPos; i++) {
		free(m_buf[i]);
	}
	free(m_buf);
}

char* CStrBuffer::operator[](int i)
{
	if (i >= m_size) {
		fprintf(stderr, "Error: StrBuffer out of range (%d)\n", i);
		exit(-1);
	}
	return m_buf[i];
}

void CStrBuffer::insert(LPCSTR str)
{
//	char *strdup(char *);
	if (m_curPos >= m_size) {
		m_size += BUF_SIZE_INC;
		m_buf = (char**) realloc(m_buf, m_size*sizeof(char*));
	}
	m_buf[m_curPos++] = _strdup(str);
}

LPCSTR CStrBuffer::getStr()
{
	if (m_headPos >= m_curPos) {
		fprintf(stderr, "Error: read from empty queue\n");
		exit(-1);
	}
	return m_buf[m_headPos++];
}

void CStrBuffer::print()
{
	int i;

	printf("{ ");
	for (i=0; i<m_curPos-1; i++) {
		printf("%s, ", m_buf[i]);
	}
	printf("%s }\n", m_buf[i]);
}

int CStrBuffer::hasStr(LPCSTR str)
{
//	printf("hasStr :%s ?\n",str);
	for (int i=0; i<m_curPos; i++) {
//		printf("[%d]%s\n",i,m_buf[i]);
		if (!strcmp(str, m_buf[i]))
			return i;
	}
	return -1;
}

int calNbits(int val)
{
	int n=1;
	int thrld = 2;

	while (val >= thrld) {
		n++;
		thrld *= 2;
	}

	return n;
}
