// This software module was originally developed by Wenlong Dong(Rockwell
// Science Center) and Wei Wu(Rockwell Science Center) in the course of
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wenlong Dong at wdong@usc.edu or Wei Wu at
// weiwu@jupiter.risc.rockwell.com.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	Script Node
FILE:       VSNodeField.h
AUTHOR:     Wenlong Dong (Rockwell Science Center)
			Wei Wu       (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Implementations for the class VSNodeField.
--------------------------------------------------------------------*/
//$Log: VSNodeField.cpp,v $
//Revision 1.1.1.1  2003/12/24 00:56:30  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.3  2003/01/20 18:02:49  jean_lf
//no message
//
//Revision 1.1  2002/11/13 09:45:12  jean_lf
//no message
//
//Revision 1.1.1.1  2002/07/18 01:08:36  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.1.1.1.2.1  2000/03/10 15:59:34  Alex Cotarmanac'h
//deuxime envoi
//

#include "defs.h"
#include "VSNodeField.h"

#include "Token.h"
#include "BifsEnc.h"


VSNodeField::~VSNodeField()
{
}

NodeField *VSNodeField::CreateField(FIELD_TYPE type)
{
	switch(type)
	{
	case SFNODE:
//		return NULL; // Not implement right now ??
		return new SFGenericNode(); // NODEFIELD
	case SFBOOL:
        return new SF<Bool>;
	case SFTIME:
		return new SF<Time>;
    case SFCOLOR:
		return new SF<Color3f>;
	case MFCOLOR:
		return new MF<Color3f>;
	case SFFLOAT:
		return new SF<float>;
	case MFFLOAT:
		return new MF<float>;
    case SFINT32:
		return new SF<int>;
	case MFINT32:
		return new MF<int>;
    case SFROTATION:
		return new SF<Rotation>;
	case MFROTATION:
		return new MF<Rotation>;
	case SFSTRING:
		return new SFString;
	case MFSTRING:
		return new MF<ZString>;
    case SFVEC2F:
		return new SF<Vec2f>;
	case MFVEC2F:
		return new MF<Vec2f>;
	case SFVEC3F:
		return new SF<Vec3f>;
	case MFVEC3F:
		return new MF<Vec3f>;
	default:
		return NULL;
	}
}
