// This software module was originally developed by Wenlong Dong(Rockwell
// Science Center) and Wei Wu(Rockwell Science Center) in the course of
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wenlong Dong at wdong@usc.edu or Wei Wu at
// weiwu@jupiter.risc.rockwell.com.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	ScriptImp Node
FILE:       VSNodeField.h
AUTHOR:     Wenlong Dong (Rockwell Science Center)
			Wei Wu       (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Interface for the class VSNodeField.
--------------------------------------------------------------------*/
//$Log: VSNodeField.h,v $
//Revision 1.1.1.1  2003/12/24 00:56:30  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.4  2003/04/03 04:30:16  mikeseven
//Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
//For installation, follow instructions in docs/Player3D_setup.doc.
//
//Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
//
//-- Mikael Bourges-Sevenier (Mindego)
//
//Revision 1.3  2003/01/20 18:02:49  jean_lf
//no message
//
//Revision 1.1  2002/11/13 09:45:12  jean_lf
//no message
//
//Revision 1.1.1.1  2002/07/18 01:08:36  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.1.1.1.2.1  2000/03/10 15:59:34  Alex Cotarmanac'h
//deuxime envoi
//
//Modified 2001/05/08 Boris Felts boris.felts@polytechnique.org
//

#if !defined(_VSNodeField_H__)
#define _VSNodeField_H__

// SOLARIS PORTING - moved to top because "ScriptImp.h" uses this
enum SCRIPT_EVENT_TYPE  //BF : eventTypes for Scripts.
{
	SCRIPT_FIELD,         // 0b00
	SCRIPT_EVENTIN,       // 0b01
	SCRIPT_EVENTOUT,      // 0b10
    SCRIPT_UNDEFINED
};

#include "ScriptDefs.h"
#include "defs.h"
#include "NodeFld.h"
#include "ScriptImp.h"

class BIFSEncoder;


class ScriptImp;


class ScriptOutput;

class VSNodeField
{
	friend class ScriptOutput;

public:

    VSNodeField() {}

    NodeField * getNodeField() { return m_NodeField; }

	virtual ~VSNodeField();
	NodeField *CreateField(FIELD_TYPE type);

	SCRIPT_EVENT_TYPE GetEventType() { return m_scriptEventType;}
	void SetEventType(SCRIPT_EVENT_TYPE type) {m_scriptEventType = type;}

    NodeField * m_NodeField;
    FIELD_TYPE GetType() { return m_fieldType;}
    FIELD_TYPE m_fieldType;

protected:

	int nNodeID;
	BOOL beenSet;
	ScriptImp *pSNode;
	ZString m_name;
	SCRIPT_EVENT_TYPE m_scriptEventType;

public:

	LPCSTR GetName() { return m_name; }
	int GetID() { return nNodeID; }
	virtual BOOL Encode(BIFSEncoder *pBifs) = 0;
	static LPCSTR field_reserved [];

};


#endif // _VSNodeField_H__


