// This software module was originally developed by Wenlong Dong(Rockwell 
// Science Center) and Wei Wu(Rockwell Science Center) in the course of 
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wenlong Dong at wdong@usc.edu or Wei Wu at 
// weiwu@jupiter.risc.rockwell.com.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	ScriptImp Node
FILE:       VScript.cpp
AUTHOR:     Wenlong Dong (Rockwell Science Center)
			Wei Wu       (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Implementation of the ScriptImp class.
--------------------------------------------------------------------*/
//$Log: VScript.cpp,v $
//Revision 1.1.1.1  2003/12/24 00:56:30  mikeseven
//Initial release of the CVS repository for MPEG-4 Systems code
//after the code was corrupted due to reorganization and human 
//errors.
//
//Revision 1.3  2003/04/03 04:30:16  mikeseven
//Initial release of Player3D and MPEG-J for all MPEG-4 reference softwares players with samples.
//For installation, follow instructions in docs/Player3D_setup.doc.
//
//Many thanks to Prashanth Gujjeti and Viswanathan Swaminathan from Sun Microsystems.
//
//-- Mikael Bourges-Sevenier (Mindego)
//
//Revision 1.2  2002/11/13 09:46:38  jean_lf
//no message
//
//Revision 1.1.1.1  2002/07/18 01:08:36  mikeseven
//Initial check in of July 17, 2002 version
//
//Revision 1.1.1.1.2.1  2000/03/10 15:59:34  Alex Cotarmanac'h
//deuxime envoi
//

// SOLARIS PORTING - converted "\" to"/"
#include "defs.h"
#include "VScript.h"
#include "VSEvents.h"
#include "../../Core/NodeCode.h"
#ifdef BIFSENC
#include "../../BifsEnc/Token.h"
#include "../../BifsEnc/BifsEnc.h"
#include "CParser.h"
#else
#include "../../Core/BifsDec.h"
#include "../../Core/Route.h"
#include "../../Core/Root.h"
#include "../interpreter/DecScript.h"
#endif
#include "../utils/ScriptOutput.h"
#include "ScriptInterface.h"

ScriptImp::ScriptImp()
{
	directOutput = FALSE;
	mustEvaluate = FALSE;
	EncUrl = FALSE;
	NumIn = 0;
	NumOut = 0;
	NumField = 0;
}

ScriptImp::~ScriptImp()
{
}

#ifdef BIFSENC
EVENT_TYPE ScriptImp::Scan_eType()
{
//AC
	switch (nextToken)
	{
	case TID_FIELD:
		return eFIELD;
		break;
	case TID_EVENTIN:
		return eEVENTIN;
		break;
	case TID_EVENTOUT:
		return eEVENTOUT;
		break;
	default:
		nextToken.Error("Unrecognized event type");
		printf("[Script] Error bad fieldtype \n");
		return eUNKNOWN_TYPE;
	}
//end AC
/*
	const char *str = nextToken.GetName();
	if(!strcmp(str, "eventIn"))
		return eEVENTIN;
	else if(!strcmp(str, "eventOut"))
		return eEVENTOUT;
	else if(!strcmp(str, "field"))
		return eFIELD;
	else {
		nextToken.Error("Unrecognized event type");
		return eUNKNOWN_TYPE;
	}
*/
}

/*-----------------------------------------------------------------
Encode the ScriptImp Node. Left problems: 
(1) How to deal with initialize and shutdown ???
(2) A field of type SFNode and MFNode is not implemented ???
(3) In script, the "new" method is not implemented ???
-----------------------------------------------------------------*/
BOOL ScriptImp::Encode(BIFSEncoder *pCoder)
{
//AC 20000304
	// we define this array to contain references to events in the order they're encoded.
	// as a consequence, their id is going to be the "natural reading" id
	// this makes decoding and debugging easier
	// formerly, ids where given in the following way : eventIns, eventOuts, fields.
	// I checked in the spec and this does not seem to be excluded.
	ZArray<VSNodeField *> eventArray;
//end AC
	if(nextToken != TID_LEFT_BRACE) 
	{
		nextToken.Error("Left brace { is expected!");
		return FALSE;
	}
		
	nextToken.Scan();
	while(nextToken != TID_RIGHT_BRACE) 
	{
		if(!stricmp(nextToken.GetName(),"url")) 
		{
			nextToken.Scan();
			if(nextToken != TID_STRING)
				nextToken.Error("Not a url string");

			ZString string = nextToken.GetString();
			int pos;
			ZString str = GetHeader(string, pos);
			if(!stricmp(str,"javascript:"))
			{
				EncUrl = TRUE;
				string.Cut(0,pos);
			}
			url += string;
			nextToken.Scan();
		}
		else if(!stricmp(nextToken.GetName(), "directOutput"))
		{
			nextToken.Scan();
			if(nextToken == TID_TRUE)
				directOutput = TRUE;
			else if(nextToken == TID_FALSE)
				directOutput = FALSE;
			else
				nextToken.Error("Unrecognized boolean value");
		}
		else if(!stricmp(nextToken.GetName(), "mustEvaluate"))
		{
			nextToken.Scan();
			if(nextToken == TID_TRUE)
				mustEvaluate = TRUE;
			else if(nextToken == TID_FALSE)
				mustEvaluate = FALSE;
			else
				nextToken.Error("Unrecognized boolean value");			
		}
		else
		{
			pCoder -> EncodeInt(1, 1, "list description");

			EVENT_TYPE eType = Scan_eType();
			VSNodeField *event = NULL;
			switch(eType)
			{
			case eEVENTIN:
				event = new EventIn;
				EventInList += event;
				eventArray +=event;//AC
				NumIn ++;
				break;
			case eEVENTOUT:
				event = new EventOut;
				EventOutList += event;
				eventArray +=event;//AC
				NumOut ++;
				break;
			case eFIELD:
				event = new Field;
				FieldList += event;
				eventArray +=event;//AC
				NumField ++;
				break;
			default:
				return FALSE;
			}
			
			nextToken.Scan();
			event -> Encode(pCoder);
		}
	}

	pCoder -> EncodeInt(0, 1, "endFlag");
	pCoder -> EncodeInt((Bool) directOutput, 1, "directOutput");
	pCoder -> EncodeInt((Bool) mustEvaluate, 1, "mustEvaluate");
	pCoder -> EncodeInt (EncUrl, 1, "Encode URL string");

	//AC 200003
	// since url is also a NumIn and NumOut (exposedField)
	NumIn++;
	NumOut++;
	// there's going to be a bug if we change url !!!!!!!
	// create a ScriptInterface.
	printf("[Nom :: %s %s]\n",getDefName(),GetName());
	ScriptInterface * mon_interface= new ScriptInterface(getDefName());
	mon_interface->StoreScriptDefinition(eventArray);
	nextToken.AddPredefinedScript(mon_interface);
	//AC end

	if(!EncUrl)
		url.Encode(pCoder);
	else
	{
		int i;
		pCParser = new CParser((LPCSTR)url [0]);
		for(i = 0; i < eventArray.GetLength(); i++)
			pCParser -> InsertExID (eventArray[i] -> GetName());
/*		//AC 20000304 : modified just above.
		for(i = 0; i < NumIn; i++)
			pCParser -> InsertExID (EventInList[i] -> GetName());
		for(i = 0; i < NumOut; i++)
			pCParser -> InsertExID (EventOutList[i] -> GetName());
		for(i = 0; i < NumField; i++)
			pCParser -> InsertExID (FieldList[i] -> GetName());
*/
		pCParser -> SetSNode(this);	// NODEFIELD
		pCParser -> Encode(pCoder);
		delete pCParser;
	}

	// Save this ScriptImp object in BifsEncoder for Route encoding
	if(GetID() >= 0)
		pCoder -> ScriptList += this;

	nextToken.Scan();
	return TRUE;
}

void ScriptImp::EncodeRoute(BIFSEncoder *pCoder, BOOL bIn)
{
	pCoder -> EncodeInt(GetID(), 10, "NodeID");
	if(nextToken != TID_PERIOD)
		nextToken.Error("period . expected");
	else
		nextToken.Scan();
		
	if(nextToken == TID_NAME)
	{
		for(int i=0; i < pCoder->ScriptList.GetLength(); i++) 
		{	

			ScriptImp *pScript = pCoder -> ScriptList[i];
			if(GetID() == pScript -> GetID()) 
			{
				int j;
				if(bIn) 
				{
					for(j = 0; j < pScript -> NumIn; j++) 
					{
						if(!strcmp(nextToken.GetName(), 
							pScript ->EventInList[j]->GetName()))
						{
							pCoder -> EncodeInt(j, BitFieldWidth(pScript -> NumIn), 
								"inFieldReference");							
							break;
						}
					}

					if(j >= pScript -> NumIn)
						nextToken.Error("No eventIn with this name");
				}
				else 
				{
					for(j = 0; j < pScript -> NumOut; j++) 
					{
						if(!strcmp(nextToken.GetName(), 
							pScript ->EventOutList[j]->GetName())) 
						{
							pCoder -> EncodeInt(j, BitFieldWidth(pScript -> NumOut), 
								"outFieldReference");
							break;
						}
					}

					if(j >= pScript -> NumOut)
						nextToken.Error("No eventOut with this name");
				} // if...else...
			} // if
		} // for
	} 
	else
		nextToken.Error("Unexpected symbol");
	
	nextToken.Scan();
}

ZString ScriptImp::GetHeader(ZString string, int &pos)
{
	int pos1 = 0;
	ZString header;

	while((string[pos1]==' ')||(string[pos1]=='\n')||(string[pos1]=='\t'))
		pos1++;

	pos = pos1;
	for(int i=0; i<15; i++)
	{
		char c = string[pos];
		pos++;
		if(!isalpha(c))
			break;
	}
	
	header = string.Substr(pos1, pos - pos1);
	return header;
}

#else
/*--------------------------------------------------------------------
Parse the ScriptImp Node. Problems left:
(1) directOutput and mustEvaluate are not implemented
(2) If the URL field is not an inline javascript, it is not 
	implemented
--------------------------------------------------------------------*/
BOOL ScriptImp::Parse(BIFSDecoder *pCoder)
{
	TRACEF ("%s {", GetName());
	TRACENEWLINE;
	TRACEINDENT (1);
	int curId=0;//AC changing the way ids are handled to get some textual output.	
	while(pCoder -> ParseInt(1))
	{
		EVENT_TYPE eType = (EVENT_TYPE)pCoder -> ParseInt(2);
		switch(eType)
		{
		case eEVENTIN:
			EventInList += new EventIn;
			EventInList[NumIn]->SetSNode(this);	// ROK1999
			EventInList[NumIn] -> SetID(curId); //AC20000304
			EventInList[NumIn] -> Parse(pCoder);
			printf("[EventIn] NumIn =%d CurId=%d \n",NumIn,curId);
			m_inIdToId += curId;
			curId++;
			NumIn++;
			break;
		case eEVENTOUT:
			EventOutList += new EventOut;
			EventOutList[NumOut] -> SetID(curId); //AC20000304
			EventOutList[NumOut] -> Parse(pCoder);
			printf("[EventOut] NumOut =%d CurId=%d \n",NumOut,curId);
			m_outIdToId +=curId;
			curId++;
			NumOut++;
			break;
		case eFIELD:
			FieldList += new Field;
			FieldList[NumField]->SetSNode(this); // NODEFIELD
			FieldList[NumField] -> SetID(curId);
			FieldList[NumField] -> Parse(pCoder);
			printf("[Field] NumField =%d CurId=%d \n",NumField,curId);
			curId++;
			NumField++;
			break;
		default:
			break;
		}
		TRACENEWLINE;
	}

/*	int i;
	for(i = 0; i < NumIn; i++)
		EventInList[i] -> SetID(i);

	for(i = 0; i < NumOut; i++)
		EventOutList[i] -> SetID(i + NumIn);

	for(i = 0; i < NumField; i++)
		FieldList[i] -> SetID(i + NumIn + NumOut);
*/
	directOutput = pCoder -> ParseInt(1) != 0;
	mustEvaluate = pCoder -> ParseInt(1) != 0;

	EncUrl = pCoder -> ParseInt(1);
	if(!EncUrl)
		url.Parse(pCoder, NULL);
	else
	{
		pDecScript = new DecScript;
		
		pDecScript -> InsertExID(NumIn + NumOut + NumField);

		int i;
		for(i = 0; i < NumOut; i++)
			EventOutList[i] -> InitField(pDecScript);

		for(i = 0; i < NumField; i++)
			FieldList[i] -> InitField(pDecScript);

		pDecScript -> Decode(pCoder);

		for(i = 0; i < NumIn; i++)
			EventInList[i] -> InitField(pDecScript);
	}
	//AC 20000303
	// url is also an eventIn and eventOut
	// therefore we have to increment NumIn and NumOut
	// --> we need this for getting the rignt number of bits when parsing.
	NumIn++;
	NumOut++;
	//AC end
	TRACEINDENT (-1);
	TRACEF ("}", 0);
	TRACENEWLINE;
	return TRUE;
}

/*---------------------------------------------------------------------
The function AddRoute is called for eventOuts.
(1) A Route is created and attached to the eventOut. 
(2) The eventOut is also bound to the Route.
(3) The TO field(eventIn) of the Route has the ID number in the Route
---------------------------------------------------------------------*/
BOOL ScriptImp::AddRoute(int nRouteID, BIFSDecoder *pCoder, LPCSTR name)
{
	int nIndex = pCoder -> ParseInt(BitFieldWidth(NumOut));

	NodeField *pField = (*(EventOut *)EventOutList[nIndex]).GetField();
	if(pField == NULL)
		return FALSE;

	//AC 20000304
	TRACEF("var%d TO ", m_outIdToId[nIndex]);
	Route *pRoute = new Route (nRouteID, pField, m_outIdToId[nIndex], this, name);

	int nNodeID = pCoder -> ParseInt (10);
	MediaObject *pNode = GetRoot () -> FindNode (nNodeID);
	if(pNode == NULL)
		return FALSE;
	TRACEF ("%s.", pNode ->GetNameID ());

	return pNode -> BindRoute (pRoute, pCoder);
}

BOOL ScriptImp::BindRoute(Route *pRoute, BIFSDecoder *pCoder)
{
	int nIndex = pCoder -> ParseInt(BitFieldWidth(NumIn));
//AC 20000304
	printf("[ScriptImp::BindRoute] inId %d, ID %d\n",nIndex,m_inIdToId[nIndex]);
	TRACEF ("var%d", m_inIdToId[nIndex]);//AC parce qu'on liait  var%d et non pas au nIndex(ieme) lt des eventIn
	pRoute -> Bind (this, m_inIdToId[nIndex]);
	return TRUE;
}

/*---------------------------------------------------------------------
The function RouteEvent is called by the Update procedure of the eventOut 
of another Node in the same ROUTE. This node is bound to the Route in the
this -> BindRoute() function:
(1) The eventIn with the fixed index is evaluated by the input Field value 
(2) All eventOuts are set to HasBeenSet = FALSE
(3) The corresponding function of the eventIn is evaluated
(4) Update all eventOuts at last
---------------------------------------------------------------------*/
void ScriptImp::RouteEvent (NodeField *pField, int nFieldIndex)
{
	int i;

	((NodeField *)*EventInList[nFieldIndex]) -> Copy (*pField, this, false);

	for(i = 0; i < NumOut-1; i++)//AC 200003 NumOut -1 since url not taken into account
		EventOutList[i] -> OnFieldUpdate(FALSE);

	EventInList[nFieldIndex] -> OnFieldUpdate();

	for(i = 0; i < NumOut-1; i++)//AC 200003 NumOut -1 since url not taken into account
		EventOutList[i] -> OnFieldUpdate(TRUE);
}

#endif