// This software module was originally developed by Wenlong Dong(Rockwell 
// Science Center) and Wei Wu(Rockwell Science Center) in the course of 
// development of the MPEG-4 Systems standard
// (ISO/IEC 14496-1).  This software module is an implementation of a part
// of one or more MPEG-4 Systems tools as specified by the MPEG-4 Systems
// standard.  ISO/IEC gives users of the MPEG-4 Systems standard free
// license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// For more information or to receive an up to date version of this module,
// you may call Wenlong Dong at wdong@usc.edu or Wei Wu at 
// weiwu@jupiter.risc.rockwell.com.
//
// Copyright 1998.

/*-------------------------------------------------------------------------
Project Im1Player
INTERNATIONAL ORGANIZATION FOR STANDARDIZATION
ISO/IEC JTC1/SC29/WG11

SUBSYSTEM:	ScriptImp Node
FILE:       VScript.h
AUTHOR:     Wenlong Dong (Rockwell Science Center)
			Wei Wu       (Rockwell Science Center)
		    Yuval Fisher (Rockwell Science Center)
OVERVIEW:   Interface for the ScriptImp class.
--------------------------------------------------------------------*/
#if !defined(_VScript_h__)
#define _VScript_h__

//#include "nodes.h"
#include "Nodes.h" // SOLARIS PORTING - typo
#include "VSNodeField.h"

#ifdef BIFSENC
class BIFSEncoder;
class CParser;
#else
class BIFSDecoder;
class DecScript;
#endif

class ScriptImp : public Script 
{
public:
	ScriptImp();
	virtual ~ScriptImp();
	BOOL operator == (const ScriptImp &node) const;

#ifdef BIFSENC
public:
	virtual BOOL Encode (BIFSEncoder *pBifs);

private:
	
	ZString GetHeader(ZString string, int &pos);
	EVENT_TYPE Scan_eType();
	virtual void EncodeRoute(BIFSEncoder *pBifs, BOOL bIn);

	CParser *pCParser;
#endif
#ifndef BIFSENC
public:

	virtual BOOL Parse(BIFSDecoder *pBifs);
	virtual BOOL AddRoute(int nRouteID, BIFSDecoder *pBifs, LPCSTR name);
	virtual BOOL BindRoute(Route *pRoute, BIFSDecoder *pBifs);
	virtual void RouteEvent (NodeField *pField, int nFieldIndex);
	virtual void RegisterObject() {}
	
private:

	DecScript *pDecScript;

#endif
public: // NODEFIELD
	ZArray<VSNodeField *> FieldList;

private:

//	SFString Urlstring; // Actually URL field is an MF<ZString>, will modify this later
//	ZArray<VSNodeField *> FieldList;
	ZArray<VSNodeField *> EventInList;
	ZArray<VSNodeField *> EventOutList;
//AC 20000304
	ZArray<int>			  m_inIdToId;	 //AC 20000304 inID to Id array.
	ZArray<int>			  m_outIdToId;   //AC 20000304 outID to Id Array
//AC end
//	BOOL directOutput;
//	BOOL mustEvaluate;
	BOOL EncUrl;
	int NumIn, NumOut, NumField;

};

#endif // _Script_h__

