/********************************************************************************/
/* This software module was developped by										*/

/*	   Marius Preda (MP), marius.preda@int-evry.fr,								*/
/*                        Institut National des Telecommunications, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

// BBADecoder.cpp: implementation of the BBADecoder class.
// dependences ::FBATypeDefs.h, BBADecoder.h
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BBATypeDefs.h"
#include "BBADecoder.h"
#include <math.h>

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////


BBADecoder::BBADecoder()
{
	initialised = 0;
	
	low=0;
	high=top;
	length=0;
	zerocount = 0;
	arithmeticAlreadyInit=false;
	dctAlreadyInit=false;

}

BBADecoder::~BBADecoder()
{

}

void BBADecoder::init(void)
{

  bbaop.firstBone=NULL;
  bbaop.lastBone=NULL;
  bbaop.firstMuscle=NULL;
  bbaop.lastMuscle=NULL;
  bbaop.firstMorph=NULL;
  bbaop.lastMorph=NULL;
}

void BBADecoder::ArithmeticInit(){
	if (arithmeticAlreadyInit) return;
	arithmeticAlreadyInit=true;
	int i;
	int bba_pred_scale_tmp[32] = { 0,  1,   2,  3,   5,   7,   9,  11,  14,  17,  20,  23,  27,  31,  35,  39,
									43, 47, 52, 57,  62,  67,  72,  77,  82,  88,  94, 100, 106, 113, 120, 127};
	for (i=0;i<32;i++)
		bba_pred_scale[i]=bba_pred_scale_tmp[i];
}

void BBADecoder::DCTInit(){
	if (dctAlreadyInit) return;
	dctAlreadyInit=true;

	int i,j;
	float c;
	static int bba_scale_tmp[32] = { 0, 1, 2, 3, 5, 7, 8, 10, 12, 15, 18, 21, 25, 30, 35, 42, 
								50, 60, 72, 87, 105, 128, 156, 191, 234, 288, 355, 439, 543, 
								674, 836, 1023 };

	for (i=0;i<32;i++)
		bba_pred_scale[i] = bba_scale_tmp[i];

    bba_init_htables();

    bbaop.idctMatrix = (float**) malloc(SEG_LENGTH*sizeof(float*));
    for (i=0; i<SEG_LENGTH; i++) {
        bbaop.idctMatrix[i] = (float*) malloc(SEG_LENGTH*sizeof(float));
    }
	for(i=0; i<SEG_LENGTH; i++) {
		if(i==0) 
			c = (float)(1.0/sqrt(2));
		else 
			c = 1;
		for(j=0; j<SEG_LENGTH; j++) {
		bbaop.idctMatrix[j][i] = c * (float)(sqrt(2.0/SEG_LENGTH))
					* (float)(cos(PI*(float)(2*j+1)*i/2.0/SEG_LENGTH));
		}
	}

}


//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////////

int BBADecoder::decode_a_symbol(int cumul_freq[ ])
{

  length = high - low + 1;
  cum = (-1 + (code_value - low + 1) * cumul_freq[0]) / length;
  for (sacindex = 1; cumul_freq[sacindex] > cum; sacindex++);
  high = low - 1 + (length * cumul_freq[sacindex-1]) / cumul_freq[0];
  low += (length * cumul_freq[sacindex]) / cumul_freq[0];

  for ( ; ; ) {  
    if (high < q2) ;
    else if (low >= q2) {
      code_value -= q2; 
      low -= q2; 
      high -= q2;
    }
    else if (low >= q1 && high < q3) {
      code_value -= q1; 
      low -= q1; 
      high -= q1;
    }
    else {
      break;
    }
 
    low *= 2; 
    high = 2*high + 1;
    bit_out_psc_layer(); 
    code_value = 2*code_value + bit;
  }

  return (sacindex-1);
}
 
/*********************************************************************
 *
 *        Name:        decoder_reset
 *
 *	Description:	Fills Decoder FIFO after a fixed word length
 *        string has been detected.
 *
 *	Input:        None
 *
 *	Returns:	Nothing
 *
 *	Side Effects:	Fills Arithmetic Decoder FIFO
 *
 *	Author:        Wayne Ellis <ellis_w_wayne@bt-web.bt.co.uk>
 *
 *********************************************************************/

/* 
   Comment by Wei Wu: This function is called right before arithmetic 
   coding after fix length string has been outputed
*/

void BBADecoder::decoder_reset( void )
{
  int i;
  zerorun = 0;        /* clear consecutive zero's counter */
  code_value = 0;
  low = 0;
  high = top;
  for (i = 1;   i <= 16;   i++) {
    bit_out_psc_layer(); 
    code_value = 2 * code_value + bit;
  }
}

/*********************************************************************
 * 
 *        Name:        bit_out_psc_layer
 *
 *	Description:	Gets a bit from the Encoded Stream, Checks for
 *        and removes any PSC emulation prevention bits
 *        inserted at the decoder, provides 'zeros' to the
 *        Arithmetic Decoder FIFO to allow it to finish 
 *        data prior to the next PSC. (Garbage bits)
 *
 *	Input:        None
 *
 *	Returns:	Nothing
 *
 *	Side Effects:	Gets a bit from the Input Data Stream
 *
 *	Author:        Wayne Ellis <ellis_w_wayne@bt-web.bt.co.uk>
 *
 *********************************************************************/

void BBADecoder::bit_out_psc_layer(void) {

  if(zerorun==ZEROMAX) {
    bit = getbits(1);
	if(!bit) {
		AfxMessageBox("FBA DECODER ERROR ::Marker bit stuffing error!\n");
		return ;
	}
	zerorun = 0;
  }
  
  bit = getbits(1);
  if(!bit) zerorun++;
  else zerorun = 0;

}


void BBADecoder::bba_update(int symbol, int index) {
 
  int n;
  int *cumul_tmp, *freq_tmp;
 
  n = bbaop.range_max[index]-bbaop.range_min[index]+1;
  cumul_tmp = bbaop.cumul_freq[index];
  freq_tmp = bbaop.freq[index];

  if (cumul_tmp[0] == Max_frequency) {
    int i, cum;
    cum = 0;
    for(i=n-1; i>=0; i--) {
        freq_tmp[i] = (freq_tmp[i]+1)/2;
        cum += freq_tmp[i];
        cumul_tmp[i] = cum;
    }
    cumul_tmp[n] = 0;
  }
  freq_tmp[symbol] += 1;
  cumul_tmp[symbol] += 1;
  while(symbol>0) {
        symbol -= 1;
        cumul_tmp[symbol] += 1;
  }
 
}

void BBADecoder::bba_init_arith() {
 
  int i, j, n;

  bbaop.Ifreq = (int **)malloc(bbaop.NumberOfSBComponentsInPlane * sizeof(int*));
  bbaop.Icumul_freq = (int **)malloc(bbaop.NumberOfSBComponentsInPlane * sizeof(int*));
  bbaop.cumul_freq = (int **)malloc(bbaop.NumberOfSBComponentsInPlane * sizeof(int*));
   bbaop.freq = (int **)malloc(bbaop.NumberOfSBComponentsInPlane * sizeof(int*));
 
  for(i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
	if(bbaop.sbc_mask[i]) {
		// intra data
        n = bbaop.Irange_max[i]-bbaop.Irange_min[i]+1;
        bbaop.Ifreq[i] = (int *) malloc(n*sizeof(int));
        bbaop.Icumul_freq[i] = (int *) malloc((n+1)*sizeof(int));
        for(j=0; j<n; j++) {
			bbaop.Ifreq[i][j] = 1;
            bbaop.Icumul_freq[i][j] = n-j;
		}
        bbaop.Icumul_freq[i][n] = 0;
		// predictive data

        n = bbaop.range_max[i]-bbaop.range_min[i]+1;
        bbaop.freq[i] = (int *) malloc(n*sizeof(int));
        bbaop.cumul_freq[i] = (int *) malloc((n+1)*sizeof(int));
        for(j=0; j<n; j++) {
          bbaop.freq[i][j] = 1;
          bbaop.cumul_freq[i][j] = n-j;
		}
        bbaop.cumul_freq[i][n] = 0;
	}
  }
}

void BBADecoder::decode_bba_i_segment() 
{
  int i,k,j;
  float dctCoef[SEG_LENGTH];
  int dctQCoef[SEG_LENGTH];

    int qstep = (int)pow(2,bbaop.pow2quant) * bba_pred_scale[bbaop.bba_quant]; // TO 

	
  for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
	  if(bbaop.sbc_mask[i]) {
		dctQCoef[0] = getIDC();	
		bba_getAC(dctQCoef);
		bba_dctdequant(dctQCoef, dctCoef ,qstep);
		bbaop.sbc_values_prev[i] = dctCoef[0];
		bba_idctTrans(dctCoef, bbaop.sbc_DCTvalues[i]);
	  }
  }
}

void BBADecoder::decode_bba_p_segment() 
{

  int i,j,k;
  float dctCoef[SEG_LENGTH];
  int dctQCoef[SEG_LENGTH];

    int qstep = (int)pow(2,bbaop.pow2quant) * bba_pred_scale[bbaop.bba_quant]; // TO 

  for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
	  if(bbaop.sbc_mask[i]) {
  		dctQCoef[0] = bba_getPDC();	
		bba_getAC(dctQCoef);
		bba_dctdequant(dctQCoef, dctCoef, qstep);
		dctCoef[0] += bbaop.sbc_values_prev[i];
		bbaop.sbc_values_prev[i] = dctCoef[0];
		bba_idctTrans(dctCoef, bbaop.sbc_DCTvalues[i]);
	  }
  }
}

void BBADecoder::bba_idctTrans(float *src, int *dest) {
  int i, j;
  for (i=0; i<SEG_LENGTH; i++) {
    dest[i] = 0;
    for (j=0; j<SEG_LENGTH; j++) {
		dest[i] += (int)(bbaop.idctMatrix[i][j]*src[j]);
    }
  }
}

void BBADecoder::bba_dctdequant(int *src, float *dest, int q) {
  int i;

  dest[0] = (float) (src[0] * (q/3.0));
  for (i=1; i<SEG_LENGTH; i++)
	dest[i] = (float)(src[i] * q);
}


int BBADecoder::getIDC() {

  int symbol;
  symbol = shortgetbits(16);

  if (symbol == SHORT_MIN-1) {
	  if (bbaop.stream.incnt!=0) getbits(bbaop.stream.incnt);
	  symbol = getbits(32);
  }

  return symbol;
}
int BBADecoder::bba_getPDC() {

  int symbol;
  symbol = huffman_decode(bbaop.dc_table, bbaop.dc_table_size);

  if (symbol==0) {
	symbol = getIDC();
  } else {
	symbol -= 256;
  }
  return symbol;
}
void BBADecoder::bba_getAC(int *coef) {
  int i, n;
  int symbol;

  for (i=1; i<SEG_LENGTH; ) {
	symbol = huffman_decode(bbaop.runlength_table, bbaop.runlength_table_size);
	if (symbol==0) {
		for(n=i; n<SEG_LENGTH; n++) {
			coef[n] = 0;
		}
		return;
	}
	for (n=0; n<symbol-1; n++) {
		coef[i++] = 0;
	}
	symbol = huffman_decode(bbaop.ac_table, bbaop.ac_table_size);

	if (symbol==256) {
		coef[i++] = getIDC();
	} else {
		coef[i++] = symbol-256;
	}
  }
}

int BBADecoder::huffman_decode(int **table, int size) {
  
  int i, n;
  int bit;
  char *valid;

  valid = (char *) malloc(size);
  for (i=0; i<size; i++) {
	valid[i] = 1;
  }
  
  n = 1;
  while(n<=30) {
    bit = getbits(1);
    for (i=0; i<size; i++) {
      if(valid[i]) {
  		if (bit == table[i][n]) {
			if (table[i][0] == n) {
				return i;
			}
		}
		else {
			valid[i] = 0;
		}
      }
    }
    n++;
  }
  AfxMessageBox("Error: 30 bits has been read and huffman decoder \
	still cannot find a symbol\n");
  exit(-1);
}

void BBADecoder::initbits() {
  bbaop.stream.incnt = 0;
}

void BBADecoder::resetbits() {
  bbaop.stream.bitcnt = 0;
}

int BBADecoder::cntbits() {
  return bbaop.stream.p * 8 - bbaop.stream.incnt;
}


int BBADecoder::getbit() {
  int t;
 
  if(bbaop.stream.incnt==0) {
      if (bbaop.stream.p >= bbaop.stream.bufsize) {
	  AfxMessageBox("FBA DECODER ERROR :: Warning: buffer underflowing");
	  return 0;
	}

	bbaop.stream.inbfr = bbaop.stream.buf[bbaop.stream.p++];
	bbaop.stream.incnt = 8;
  }
  t = (bbaop.stream.inbfr&0x80)>>7;
  bbaop.stream.inbfr <<= 1;
  bbaop.stream.incnt -= 1;
  return t;
}


int BBADecoder::getbits(int n) {
  int i;
  int ibuf = 0;

  if(n>32) {
        fprintf(stderr, "Cannot fetch more than 32 bits from getbits()\n");
        exit(-1);
  }

  for(i=0; i<n; i++) { /* MSB first */
        ibuf <<= 1;
        ibuf += getbit();
  }

  bbaop.stream.bitcnt += n;

  return ibuf;
}
 
int BBADecoder::shortgetbits(int n) 
{
	int counter;
	short value = 0;	

	if(n>16) {
        AfxMessageBox("Cannot fetch more than 16 bits from shortgetbits()");
        exit(-1);
  }
	
   bbaop.stream.bitcnt += n;

	for(counter=0; counter<16; counter++) {
        value <<= 1;
        value += getbit();
	}
	
	return (int)value;
}

 
void BBADecoder::next_start_code(void) {

  if(bbaop.stream.incnt==0)
	getbits(8);
  else 
	getbits(bbaop.stream.incnt);
  
}
 
int BBADecoder::nextbits_bytealigned(void) {
  int incnt_save, inbfr_save;
  int sc;
  int flag=0;

  incnt_save = bbaop.stream.incnt;
  inbfr_save = bbaop.stream.inbfr;
  if(bbaop.stream.incnt==0) {
	getbits(8);
	flag = 1;
  }
  else 
	  getbits(bbaop.stream.incnt);

  sc = getbits(32);
  bbaop.stream.incnt = incnt_save;
  bbaop.stream.inbfr = inbfr_save;
  if(flag==1) {
    bbaop.stream.p -= 5;
  } else {
    bbaop.stream.p -= 4;
  }
  return sc;
}

/*
   This function is called right before fix length string is about to
   be outputed after arithmetic coding
*/
void BBADecoder::adjust_bits() {
  int incnt_save;

  bbaop.stream.p -= 3;
  incnt_save = bbaop.stream.incnt;
  bbaop.stream.incnt = 0;
  getbits(8-incnt_save+2);
  bbaop.stream.bitcnt -= (8-incnt_save+2);
  bbaop.stream.bitcnt -= 14; /* 16-2 */

}

int BBADecoder::decode_bba_object_plane_header() {



  int sc;
  //nextbits_bytealigned();
  next_start_code();
  sc = getbits(32);
  if (sc!=BBA_OBJECT_PLANE_START_CODE) {
		AfxMessageBox("BBA DECODER ERROR ::BBA_OBJECT_PLANE_START_CODE");
		exit(1);
		return 1;
	}	

  bbaop.is_intra = getbits(1);
  if (bbaop.is_intra) {
	bbaop.is_frame_rate = getbits(1);
	if(bbaop.is_frame_rate) {
		bba_decode_frame_rate();
	}
	bbaop.is_time_code = getbits(1);
	if(bbaop.is_time_code) 
		bba_decode_time_code();      
  }
  bbaop.skip_frames = getbits(1);
  if(bbaop.skip_frames)
	bba_decode_skip_frames();
  else 
	bbaop.number_of_frames_to_skip = 0;

  return 0;
}

int BBADecoder::UnLoadThePlane(){

	int count=0;
	CBone *lb= new CBone();
	lb = bbaop.firstBone;
	while (lb){
		if (!lb->isInCurrentFrame)
			lb=lb->suiv;
		else{
			//lb->isInCurrentFrame=false;
			lb->boneData.TonX=bbaop.sbc_values[count];
			lb->boneDataPreviousFrame.TonX=bbaop.sbc_values_prev[count];
			count++;

			lb->boneDataPreviousFrame.TonY=bbaop.sbc_values_prev[count];
			lb->boneData.TonY=bbaop.sbc_values[count];
			count++;

			lb->boneDataPreviousFrame.TonZ=bbaop.sbc_values_prev[count];
			lb->boneData.TonZ=bbaop.sbc_values[count];
			count++;

			if (!lb->boneMask.isQuaternion){
				lb->boneDataPreviousFrame.RonAx1=bbaop.sbc_values_prev[count];
				lb->boneData.RonAx1=bbaop.sbc_values[count];
				count++;
			
				lb->boneDataPreviousFrame.RonAx2=bbaop.sbc_values_prev[count];
				lb->boneData.RonAx2=bbaop.sbc_values[count];
				count++;

				lb->boneDataPreviousFrame.RonAx3=bbaop.sbc_values_prev[count];
				lb->boneData.RonAx3=bbaop.sbc_values[count];
				count++;
			}else{
				lb->boneDataPreviousFrame.RonQx=bbaop.sbc_values_prev[count];
				lb->boneData.RonQx=bbaop.sbc_values[count];
				count++;
				lb->boneDataPreviousFrame.RonQy=bbaop.sbc_values_prev[count];
				lb->boneData.RonQy=bbaop.sbc_values[count];
				count++;
				lb->boneDataPreviousFrame.RonQz=bbaop.sbc_values_prev[count];
				lb->boneData.RonQz=bbaop.sbc_values[count];
				count++;
				lb->boneDataPreviousFrame.RonQw=bbaop.sbc_values_prev[count];
				lb->boneData.RonQw=bbaop.sbc_values[count];
				count++;
			}
			lb->boneDataPreviousFrame.SonX=bbaop.sbc_values_prev[count];
			lb->boneData.SonX=bbaop.sbc_values[count];
			count++;

			lb->boneDataPreviousFrame.SonY=bbaop.sbc_values_prev[count];
			lb->boneData.SonY=bbaop.sbc_values[count];
			count++;

			lb->boneDataPreviousFrame.SonZ=bbaop.sbc_values_prev[count];
			lb->boneData.SonZ=bbaop.sbc_values[count];
			count++;

			lb->boneDataPreviousFrame.SOonAxX=bbaop.sbc_values_prev[count];
			lb->boneData.SOonAxX=bbaop.sbc_values[count];
			count++;

			lb->boneDataPreviousFrame.SOonAxY=bbaop.sbc_values_prev[count];
			lb->boneData.SOonAxY=bbaop.sbc_values[count];
			count++;

			lb->boneDataPreviousFrame.SOonAxZ=bbaop.sbc_values_prev[count];
			lb->boneData.SOonAxZ=bbaop.sbc_values[count];
			count++;

			lb->boneDataPreviousFrame.SOonVal=bbaop.sbc_values_prev[count];
			lb->boneData.SOonVal=bbaop.sbc_values[count];
			count++;
			
			lb->boneDataPreviousFrame.ConX=bbaop.sbc_values_prev[count];
			lb->boneData.ConX=bbaop.sbc_values[count];
			count++;

			lb->boneDataPreviousFrame.ConY=bbaop.sbc_values_prev[count];
			lb->boneData.ConY=bbaop.sbc_values[count];
			count++;

			lb->boneDataPreviousFrame.ConZ=bbaop.sbc_values_prev[count];
			lb->boneData.ConZ=bbaop.sbc_values[count];
			count++;
			
			lb=lb->suiv;
		}
	}
/////////////////////////////muscles///////////////////////////
	CMuscle *lm= new CMuscle();
	int i;
	lm = bbaop.firstMuscle;
	while (lm){
		if (!lm->isInCurrentFrame)
			lm=lm->suiv;
		else{
			//lm->isInCurrentFrame=false;
			for (i=0;i<lm->NumberControlPoints;i++){
				lm->muscleDataPreviousFrame.ControlPointOnX[i]=bbaop.sbc_values_prev[count];
				lm->muscleData.ControlPointOnX[i]=bbaop.sbc_values[count];
				count++;

				lm->muscleDataPreviousFrame.ControlPointOnY[i]=bbaop.sbc_values_prev[count];
				lm->muscleData.ControlPointOnY[i]=bbaop.sbc_values[count];
				count++;

				lm->muscleDataPreviousFrame.ControlPointOnZ[i]=bbaop.sbc_values_prev[count];
				lm->muscleData.ControlPointOnZ[i]=bbaop.sbc_values[count];
				count++;
			}
			for (i=0;i<lm->NumberControlPoints;i++){
				lm->muscleDataPreviousFrame.WheighOfControlPoint[i]=bbaop.sbc_values_prev[count];
				lm->muscleData.WheighOfControlPoint[i]=bbaop.sbc_values[count];
				count++;
			}
			for (i=0;i<lm->NumberKnots;i++){
				lm->muscleDataPreviousFrame.Knot[i]=bbaop.sbc_values_prev[count];
				lm->muscleData.Knot[i]=bbaop.sbc_values[count];
				count++;
			}			
			lm=lm->suiv;
		}
	}

/////////////////////////////morph///////////////////////////
	CMorph *lmf= new CMorph();
	lmf = bbaop.firstMorph;
	while (lmf){
		if (!lmf->isInCurrentFrame)
			lmf=lmf->suiv;
		else{
			for (i=0;i<lmf->NumberWeights;i++){
				lmf->morphDataPreviousFrame.Weights[i]=bbaop.sbc_values_prev[count];
				lmf->morphData.Weights[i]=bbaop.sbc_values[count];
				count++;
			}
			lmf=lmf->suiv;
		}
	}
	return true;
}
int BBADecoder::decode_bba_object_plane_data() {

	decode_bba_object_plane_mask();
	if (bbaop.is_intra)
		bbaop.bba_object_coding_type=getbits(1);
	if (bbaop.bba_object_coding_type == PREDICTIVE_CODING_TYPE){
		ArithmeticInit();
		SetMinMax();
		AllocThePlane();
		LoadThePlane();
	}else{
		DCTInit();
		AllocThePlaneSegment();
		LoadThePlaneSegment();
	}
	decode_bba_object_plane_values();

	if (bbaop.bba_object_coding_type == PREDICTIVE_CODING_TYPE){
		UnLoadThePlane();
		FreeThePlane();
	}else{
		UnLoadThePlaneSegment();
		FreeThePlaneSegment();
	}


	return 0;
}

bool BBADecoder::decode_bone_mask(CBone *bn) {
	int marker;
	bool isrot;
	CBoneMask *bml=&(bn->boneMask);
	if (!getbits(1)) {
		bml->isTonXChanged=false;
		bml->isTonYChanged=false;
		bml->isTonZChanged=false;
		marker = getbits(1);
	}else{
		marker = getbits(1);
		bml->isTonXChanged=getbits(1);
		bml->isTonYChanged=getbits(1);
		bml->isTonZChanged=getbits(1);
	}
	isrot=getbits(1);
	marker = getbits(1);
	bml->isQuaternion=false;
	if (isrot){
		bml->isQuaternion=getbits(1);
		if (!bml->isQuaternion) {
			bml->isRonAx1Changed=getbits(1);
			bml->isRonAx2Changed=getbits(1);
			bml->isRonAx3Changed=getbits(1);
		}else{
			bml->isRonQxChanged=getbits(1);
			bml->isRonQyChanged=getbits(1);
			bml->isRonQzChanged=getbits(1);
			bml->isRonQwChanged=getbits(1);
		}
	}else{
		bml->isRonAx1Changed=false;
		bml->isRonAx2Changed=false;
		bml->isRonAx3Changed=false;
		bml->isRonQxChanged=false;
		bml->isRonQyChanged=false;
		bml->isRonQzChanged=false;
		bml->isRonQwChanged=false;
	}
	if (!getbits(1)) {
		bml->isSonXChanged=false;
		bml->isSonYChanged=false;
		bml->isSonZChanged=false;
		marker = getbits(1);
	}else{
		marker = getbits(1);
		bml->isSonXChanged=getbits(1);
		bml->isSonYChanged=getbits(1);
		bml->isSonZChanged=getbits(1);
	}
	if (!getbits(1)) {
		bml->isSOonAxXChanged=false;
		bml->isSOonAxYChanged=false;
		bml->isSOonAxZChanged=false;
		bml->isSOonValChanged=false;
		marker = getbits(1);
	}else{
		marker = getbits(1);
		bml->isSOonAxXChanged=getbits(1);
		bml->isSOonAxYChanged=getbits(1);
		bml->isSOonAxZChanged=getbits(1);
		bml->isSOonValChanged=getbits(1);
	}
	if (!getbits(1)) {
		bml->isConXChanged=false;
		bml->isConYChanged=false;
		bml->isConZChanged=false;
		marker = getbits(1);
	}else{
		marker = getbits(1);
		bml->isConXChanged=getbits(1);
		bml->isConYChanged=getbits(1);
		bml->isConZChanged=getbits(1);
	}
	return true;
}

bool BBADecoder::decode_muscle_mask(CMuscle *ms) {
	int marker;
	int i;
	CMuscleMask *mml=&(ms->muscleMask);
	if (!getbits(1)) {
		for (i=0;i<ms->NumberControlPoints;i++){
			mml->isControlPointOnXChanged[i]=false;
			mml->isControlPointOnYChanged[i]=false;
			mml->isControlPointOnZChanged[i]=false;
		}
		marker = getbits(1);
	}else{
		marker = getbits(1);
		for (i=0;i<ms->NumberControlPoints;i++){
			marker = getbits(1);
			mml->isControlPointOnXChanged[i]=getbits(1);
			mml->isControlPointOnYChanged[i]=getbits(1);
			mml->isControlPointOnZChanged[i]=getbits(1);
		}
	}
	
	if (!getbits(1)) {
		for (i=0;i<ms->NumberControlPoints;i++){
			mml->isWheighOfControlPointChanged[i]=false;
		}
		marker = getbits(1);
	}else{
		marker = getbits(1);
		for (i=0;i<ms->NumberControlPoints;i++){
			marker = getbits(1);
			mml->isWheighOfControlPointChanged[i]=getbits(1);
		}
	}

	if (!getbits(1)) {
		for (i=0;i<ms->NumberKnots;i++){
			mml->isKnotChanged[i]=false;
		}
		marker = getbits(1);
	}else{
		marker = getbits(1);
		for (i=0;i<ms->NumberKnots;i++){
			marker = getbits(1);
			mml->isKnotChanged[i]=getbits(1);
		}
	}

	return true;
}

bool BBADecoder::decode_bba_object_plane_mask() {
	int i;
	CBone *lb;
	CMuscle *lm;
	CMorph *lmf;
	bool hasBones=false, hasMuscle=false, hasMorph=false;
	bbaop.KeyFrameIndex = getbits(5);
	if (bbaop.is_intra){
		bbaop.bba_quant = getbits(5);
		bbaop.pow2quant = getbits(3);
		bbaop.noOfControllerTypes=getbits(3);
		switch (bbaop.noOfControllerTypes){
			case 0: return false;
			case 1: hasBones=true; break;
			case 2: hasBones=true; hasMuscle=true; break;
			case 3: hasBones=true; hasMuscle=true; hasMorph=true; break;
			default: return false;
		}
		if (hasBones) bbaop.NumberOfBones=getbits(10);
		else bbaop.NumberOfBones=0;
		if (hasMuscle) bbaop.NumberOfMuscles=getbits(10);
		else bbaop.NumberOfMuscles=0;
		if (hasMorph) bbaop.NumberOfMorphs=getbits(10);
		else bbaop.NumberOfMorphs=0;

		if (hasBones){
			lb=bbaop.firstBone;
			while (lb){
				lb->canBePredicted=false;
				lb->isInCurrentFrame=false;
				lb=lb->suiv;
			}
			for (i=0;i<bbaop.NumberOfBones;i++){
				lb=new CBone();
				lb->boneIdentifier=getbits(10);
				lb->canBePredicted=true;
				lb->boneMask.init();
				lb->isInCurrentFrame=true;
				decode_bone_mask(lb);
				innod(lb,bbaop.firstBone,bbaop.lastBone);
				lb=NULL;
			}
		}
		if (hasMuscle){
			lm=bbaop.firstMuscle;
			while (lm){
				lm->canBePredicted=false;
				lm->isInCurrentFrame=false;
				lm=lm->suiv;
			}
			for (i=0;i<bbaop.NumberOfMuscles;i++){
				lm=new CMuscle();
				lm->muscleIdentifier=getbits(10);
				lm->canBePredicted=true;
				lm->isInCurrentFrame=true;
				lm->NumberControlPoints=getbits(6);
				lm->NumberKnots=getbits(6);
				lm->AllocMuscle();
				lm->AllocPrevMuscle();
				decode_muscle_mask(lm);
				innod(lm,bbaop.firstMuscle,bbaop.lastMuscle);
				lb=NULL;
			}
		}
		if (hasMorph){
			lmf=bbaop.firstMorph;
			while (lmf){
				lmf->canBePredicted=false;
				lmf->isInCurrentFrame=false;
				lmf=lmf->suiv;
			}
			for (i=0;i<bbaop.NumberOfMorphs;i++){
				lmf=new CMorph();
				lmf->morphIdentifier=getbits(10);
				lmf->canBePredicted=true;
				lmf->isInCurrentFrame=true;
				lmf->NumberWeights=getbits(6);
				lmf->AllocMorph();
				lmf->AllocPrevMorph();
				decode_morph_mask(lmf);
				innod(lmf,bbaop.firstMorph,bbaop.lastMorph);
				lmf=NULL;
			}
		}
	}
	return true;
}

int BBADecoder::decode_bba_object_plane_values() {


	if (bbaop.is_intra){

		if (bbaop.bba_object_coding_type == 0) {
			bbaop.is_i_new_max = getbits(1);
			bbaop.is_i_new_min = getbits(1);
			bbaop.is_p_new_max = getbits(1);
			bbaop.is_p_new_min = getbits(1);
			if (bbaop.is_i_new_max || bbaop.is_i_new_min || bbaop.is_p_new_max || bbaop.is_p_new_min)
				decode_bba_new_minmax();
			bba_init_arith();// before I frame the arithmetic statistics are reset
			decode_bba_i_frame();
		}
		else {
			decode_bba_i_segment();
		}
	}
	else {
		if (bbaop.bba_object_coding_type == 0) 
			decode_bba_p_frame();
		else
			decode_bba_p_segment();
	}
	
	if (bbaop.bba_object_coding_type == 0) { 
		adjust_bits(); 
  }
  return cntbits();


}

///////////////////////////////////////////////////////////////////

void BBADecoder::bba_decode_frame_rate() {

  bbaop.frame_rate = getbits(8);
  bbaop.seconds = getbits(4);
  bbaop.frequency_offset = getbits(1);
}

void BBADecoder::bba_decode_time_code() {

  bbaop.time_code_hours = getbits(5);
  bbaop.time_code_minutes = getbits(6);
  bbaop.marker_bit = getbits(1);
  bbaop.time_code_seconds = getbits(6);
}

void BBADecoder::bba_decode_skip_frames(void) {

  int ivalue;

  bbaop.number_of_frames_to_skip = 0;
  do {
        ivalue = getbits(4);
        bbaop.number_of_frames_to_skip += ivalue;
  } while (ivalue==15);
}

void BBADecoder::decode_bba_new_minmax() {
  int i, j;
  int minmax;
  int marker;
  int scaler;
  int temp_index;


  if (bbaop.bba_quant) scaler = bbaop.bba_quant;
  else scaler = 1;
  
  if(bbaop.is_i_new_max) {
    for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
			marker = getbits(1);
			assert(marker);
            if(bbaop.sbc_mask[i]) {
               minmax = getbits(5);
               bbaop.Irange_max[i] = minmax*RANGE_DELTA/scaler;
			}
	}
  }

  if(bbaop.is_i_new_min) {
    for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
			marker = getbits(1);
			assert(marker);
            if(bbaop.sbc_mask[i]) {
               minmax = getbits(5);
               bbaop.Irange_min[i] = -minmax*RANGE_DELTA/scaler;
			}
	}
  }
   if(bbaop.is_p_new_max) {
    for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
			marker = getbits(1);
			assert(marker);
            if(bbaop.sbc_mask[i]) {
               minmax = getbits(5);
               bbaop.range_max[i] = minmax*RANGE_DELTA/scaler;
			}
	}
  }
   if(bbaop.is_p_new_min) {
    for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
			marker = getbits(1);
			assert(marker);
            if(bbaop.sbc_mask[i]) {
               minmax = getbits(5);
               bbaop.range_min[i] = -minmax*RANGE_DELTA/scaler;
			}
	}
  }
}

 
void BBADecoder::decode_bba_i_frame() {

	int i, j, k;
	int symbol, qstep;
	int Q;
			
  qstep = (int)pow(2, bbaop.pow2quant) * bba_pred_scale[bbaop.bba_quant];
  decoder_reset(); 

  for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
	if(bbaop.sbc_mask[i]) {
		symbol = decode_a_symbol(bbaop.Icumul_freq[i]);
		Q = symbol + bbaop.Irange_min[i];
		bbaop.sbc_values_prev[i] = dequant(Q, qstep);
		bbaop.sbc_values[i] = bbaop.sbc_values_prev[i];
	}
  }
}


void BBADecoder::decode_bba_p_frame() {
  
	int i;
	int symbol, qstep;
	int DIF,Q;


  decoder_reset();
  qstep = (int)pow(2, bbaop.pow2quant) * bba_pred_scale[bbaop.bba_quant];

  for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
	if(bbaop.sbc_mask[i]) {
		symbol = decode_a_symbol(bbaop.cumul_freq[i]);
	    bba_update(symbol, i);
	    Q = symbol + bbaop.range_min[i];
	    DIF = dequant(Q, qstep);
		//bbaop.sbc_values_prev[i] = recon(DIF, bbaop.sbc_values_prev[i]);
	    //bbaop.sbc_values[i]=bbaop.sbc_values_prev[i];
		bbaop.sbc_values[i] = recon(DIF, bbaop.sbc_values_prev[i]);
	}
  }
}
 
int BBADecoder::recon(int fap_diff, int fap_prev) {
  int fap_rec;

  fap_rec = fap_diff + fap_prev;
  return fap_rec;
}

int BBADecoder::dequant(int qfap, int qstep) {


  if(qstep) {
        return qfap*qstep;
  } else {
        return qfap;
  }      
}

int BBADecoder::WriteDecodedData(FILE *bbafp){


	if(bbaop.bba_object_coding_type == PREDICTIVE_CODING_TYPE) {
		WriteOneFrame(bbafp);
	}
	else{
		WriteOneSegment(bbafp);
	}
	return 0;
}

void BBADecoder::WriteOneFrame(FILE *bbafp){

	int i,j;
	CBone *q;
	CMuscle *m;
	CMorph *mf;

	fprintf(bbafp, "%d ", bbaop.KeyFrameIndex);
	if (bbaop.NumberOfBones+bbaop.NumberOfMuscles+bbaop.NumberOfMorphs==0) return;
	bbaop.noOfControllerTypes=1;
	if (bbaop.NumberOfMuscles) bbaop.noOfControllerTypes=2;
	if (bbaop.NumberOfMorphs) bbaop.noOfControllerTypes=3;
	fprintf(bbafp, "%d ", bbaop.noOfControllerTypes);
	fprintf(bbafp, "%d ", bbaop.NumberOfBones);
	if (bbaop.noOfControllerTypes>1) fprintf(bbafp, "%d ", bbaop.NumberOfMuscles);
	if (bbaop.noOfControllerTypes>2) fprintf(bbafp, "%d ", bbaop.NumberOfMorphs);
	q=bbaop.firstBone;
	while (q){
		fprintf(bbafp, "%d ", q->boneIdentifier);
		if (q->isTranslation()){
			fprintf(bbafp, "%d ", 1);
			fprintf(bbafp, "%d ", q->boneMask.isTonXChanged);
			fprintf(bbafp, "%d ", q->boneMask.isTonYChanged);
			fprintf(bbafp, "%d ", q->boneMask.isTonZChanged);
		}else
			fprintf(bbafp, "%d ", 0);
		if (q->isRotation()){
			fprintf(bbafp, "%d ", 1);
			if (!q->boneMask.isQuaternion){
				fprintf(bbafp, "%d ", 0);// EA
				fprintf(bbafp, "%d ", q->boneMask.isRonAx1Changed);
				fprintf(bbafp, "%d ", q->boneMask.isRonAx2Changed);
				fprintf(bbafp, "%d ", q->boneMask.isRonAx3Changed);
			}else{
				fprintf(bbafp, "%d ", 1);// Q
				fprintf(bbafp, "%d ", q->boneMask.isRonQxChanged);
				fprintf(bbafp, "%d ", q->boneMask.isRonQyChanged);
				fprintf(bbafp, "%d ", q->boneMask.isRonQzChanged);
				fprintf(bbafp, "%d ", q->boneMask.isRonQwChanged);
			}
		}else
			fprintf(bbafp, "%d ", 0);
		if (q->isScale()){
			fprintf(bbafp, "%d ", 1);
			fprintf(bbafp, "%d ", q->boneMask.isSonXChanged);
			fprintf(bbafp, "%d ", q->boneMask.isSonYChanged);
			fprintf(bbafp, "%d ", q->boneMask.isSonZChanged);
		}else
			fprintf(bbafp, "%d ", 0);
		if (q->isScaleOrientation()){
			fprintf(bbafp, "%d ", 1);
			fprintf(bbafp, "%d ", q->boneMask.isSOonAxXChanged);
			fprintf(bbafp, "%d ", q->boneMask.isSOonAxYChanged);
			fprintf(bbafp, "%d ", q->boneMask.isSOonAxZChanged);
			fprintf(bbafp, "%d ", q->boneMask.isSOonValChanged);
		}else
			fprintf(bbafp, "%d ", 0);
		if (q->isCenter()){
			fprintf(bbafp, "%d ", 1);
			fprintf(bbafp, "%d ", q->boneMask.isConXChanged);
			fprintf(bbafp, "%d ", q->boneMask.isConYChanged);
			fprintf(bbafp, "%d ", q->boneMask.isConZChanged);
		}else
			fprintf(bbafp, "%d ", 0);
		q=q->suiv;
	}
	m=bbaop.firstMuscle;
	while (m){
			fprintf(bbafp, "%d ", m->muscleIdentifier);
			fprintf(bbafp, "%d ", m->NumberControlPoints);
			fprintf(bbafp, "%d ", m->NumberKnots);
			if (m->isControlPointsChanged()){
				fprintf(bbafp, "%d ", 1);
				for (j=0;j<m->NumberControlPoints;j++){
					fprintf(bbafp, "%d ", m->muscleMask.isControlPointOnXChanged[j]);
					fprintf(bbafp, "%d ", m->muscleMask.isControlPointOnYChanged[j]);
					fprintf(bbafp, "%d ", m->muscleMask.isControlPointOnZChanged[j]);
				}
			}else
				fprintf(bbafp, "%d ", 0);
			if (m->isWheightOfControlPointsChanged()){
				fprintf(bbafp, "%d ", 1);
				for (j=0;j<m->NumberControlPoints;j++){
					fprintf(bbafp, "%d ", m->muscleMask.isWheighOfControlPointChanged[j]);
				}
			}else
				fprintf(bbafp, "%d ", 0);
			if (m->isKnotChanged()){
				fprintf(bbafp, "%d ", 1);
				for (j=0;j<m->NumberKnots;j++){
					fprintf(bbafp, "%d ", m->muscleMask.isKnotChanged[j]);
				}
			}else
				fprintf(bbafp, "%d ", 0);
			m=m->suiv;
	}
	mf=bbaop.firstMorph;
	while (mf){
			fprintf(bbafp, "%d ", mf->morphIdentifier);
			fprintf(bbafp, "%d ", mf->NumberWeights);
			if (mf->isWeightsChanged()){
				fprintf(bbafp, "%d ", 1);
				for (j=0;j<mf->NumberWeights;j++){
					fprintf(bbafp, "%d ", mf->morphMask.isWeightChanged[j]);
				}
			}else
				fprintf(bbafp, "%d ", 0);
			mf=mf->suiv;
	}
	fprintf(bbafp, "\n");
	q=bbaop.firstBone;
	while(q){
		if (q->boneMask.isTonXChanged) 
			fprintf(bbafp, "%d ", q->boneData.TonX);
		if (q->boneMask.isTonYChanged) 
			fprintf(bbafp, "%d ", q->boneData.TonY);
		if (q->boneMask.isTonZChanged) 
			fprintf(bbafp, "%d ", q->boneData.TonZ);
		if (!q->boneMask.isQuaternion){
			if (q->boneMask.isRonAx1Changed) 
				fprintf(bbafp, "%d ", q->boneData.RonAx1);
			if (q->boneMask.isRonAx2Changed) 
				fprintf(bbafp, "%d ", q->boneData.RonAx2);
			if (q->boneMask.isRonAx3Changed) 
				fprintf(bbafp, "%d ", q->boneData.RonAx3);
		}else{
			if (q->boneMask.isRonQxChanged) 
				fprintf(bbafp, "%d ", q->boneData.RonQx);
			if (q->boneMask.isRonQyChanged) 
				fprintf(bbafp, "%d ", q->boneData.RonQy);
			if (q->boneMask.isRonQzChanged) 
				fprintf(bbafp, "%d ", q->boneData.RonQz);
			if (q->boneMask.isRonQwChanged) 
				fprintf(bbafp, "%d ", q->boneData.RonQw);
		}
		if (q->boneMask.isSonXChanged) 
			fprintf(bbafp, "%d ", q->boneData.SonX);
		if (q->boneMask.isSonYChanged) 
			fprintf(bbafp, "%d ", q->boneData.SonY);
		if (q->boneMask.isSonZChanged) 
			fprintf(bbafp, "%d ", q->boneData.SonZ);
		if (q->boneMask.isSOonAxXChanged) 
			fprintf(bbafp, "%d ", q->boneData.SOonAxX);
		if (q->boneMask.isSOonAxYChanged) 
			fprintf(bbafp, "%d ", q->boneData.SOonAxY);
		if (q->boneMask.isSOonAxZChanged) 
			fprintf(bbafp, "%d ", q->boneData.SOonAxZ);
		if (q->boneMask.isSOonValChanged) 
			fprintf(bbafp, "%d ", q->boneData.SOonVal);
		if (q->boneMask.isConXChanged) 
			fprintf(bbafp, "%d ", q->boneData.ConX);
		if (q->boneMask.isConYChanged) 
			fprintf(bbafp, "%d ", q->boneData.ConY);
		if (q->boneMask.isConZChanged) 
			fprintf(bbafp, "%d ", q->boneData.ConZ);
		q=q->suiv;
	}
	m=bbaop.firstMuscle;
	while(m){
		if (m->isControlPointsChanged()){
			for (j=0;j<m->NumberControlPoints;j++){
				if (m->muscleMask.isControlPointOnXChanged[j])
					fprintf(bbafp, "%d ", m->muscleData.ControlPointOnX[j]);
				if (m->muscleMask.isControlPointOnYChanged[j])
					fprintf(bbafp, "%d ", m->muscleData.ControlPointOnY[j]);
				if (m->muscleMask.isControlPointOnZChanged[j])
					fprintf(bbafp, "%d ", m->muscleData.ControlPointOnZ[j]);
			}
		}
		if (m->isWheightOfControlPointsChanged()){
			for (j=0;j<m->NumberControlPoints;j++){
				if (m->muscleMask.isWheighOfControlPointChanged[j])
					fprintf(bbafp, "%d ", m->muscleData.WheighOfControlPoint[j]);
			}
		}
		if (m->isKnotChanged()){
			for (j=0;j<m->NumberKnots;j++){
				if (m->muscleMask.isKnotChanged[j])
					fprintf(bbafp, "%d ", m->muscleData.Knot[j]);
			}
		}
		m=m->suiv;
	}
	mf=bbaop.firstMorph;
	while(mf){
		if (mf->isWeightsChanged()){
			for (j=0;j<mf->NumberWeights;j++){
				if (mf->morphMask.isWeightChanged[j])
					fprintf(bbafp, "%d ", mf->morphData.Weights[j]);
			}
		}
		mf=mf->suiv;
	}
	fprintf(bbafp, "\n");
	fflush(bbafp);
}

void BBADecoder::WriteOneSegment(FILE *bbafp){

	int i,j;
	CBone *q;
	CMuscle *m;
for (int seg=0;seg<SEG_LENGTH;seg++){
	fprintf(bbafp, "%d ", bbaop.KeyFrameIndex);
	fprintf(bbafp, "%d ", bbaop.NumberOfBones);
	fprintf(bbafp, "%d ", bbaop.NumberOfMuscles);
	q=bbaop.firstBone;
	while (q){
		fprintf(bbafp, "%d ", q->boneIdentifier);
		if (q->isTranslation()){
			fprintf(bbafp, "%d ", 1);
			fprintf(bbafp, "%d ", q->boneMask.isTonXChanged);
			fprintf(bbafp, "%d ", q->boneMask.isTonYChanged);
			fprintf(bbafp, "%d ", q->boneMask.isTonZChanged);
		}else
			fprintf(bbafp, "%d ", 0);
		if (q->isRotation()){
			if (!q->boneMask.isQuaternion){
				fprintf(bbafp, "%d ", 1);
				fprintf(bbafp, "%d ", 0);//EA
				fprintf(bbafp, "%d ", q->boneMask.isRonAx1Changed);
				fprintf(bbafp, "%d ", q->boneMask.isRonAx2Changed);
				fprintf(bbafp, "%d ", q->boneMask.isRonAx3Changed);
			}else{
				fprintf(bbafp, "%d ", 1);
				fprintf(bbafp, "%d ", 1);//Q
				fprintf(bbafp, "%d ", q->boneMask.isRonQxChanged);
				fprintf(bbafp, "%d ", q->boneMask.isRonQyChanged);
				fprintf(bbafp, "%d ", q->boneMask.isRonQzChanged);
				fprintf(bbafp, "%d ", q->boneMask.isRonQwChanged);
			}
		}else
			fprintf(bbafp, "%d ", 0);
		if (q->isScale()){
			fprintf(bbafp, "%d ", 1);
			fprintf(bbafp, "%d ", q->boneMask.isSonXChanged);
			fprintf(bbafp, "%d ", q->boneMask.isSonYChanged);
			fprintf(bbafp, "%d ", q->boneMask.isSonZChanged);
		}else
			fprintf(bbafp, "%d ", 0);
		if (q->isScaleOrientation()){
			fprintf(bbafp, "%d ", 1);
			fprintf(bbafp, "%d ", q->boneMask.isSOonAxXChanged);
			fprintf(bbafp, "%d ", q->boneMask.isSOonAxYChanged);
			fprintf(bbafp, "%d ", q->boneMask.isSOonAxZChanged);
			fprintf(bbafp, "%d ", q->boneMask.isSOonValChanged);
		}else
			fprintf(bbafp, "%d ", 0);
		if (q->isCenter()){
			fprintf(bbafp, "%d ", 1);
			fprintf(bbafp, "%d ", q->boneMask.isConXChanged);
			fprintf(bbafp, "%d ", q->boneMask.isConYChanged);
			fprintf(bbafp, "%d ", q->boneMask.isConZChanged);
		}else
			fprintf(bbafp, "%d ", 0);
		q=q->suiv;
	}
	m=bbaop.firstMuscle;
	while (m){
			fprintf(bbafp, "%d ", m->muscleIdentifier);
			fprintf(bbafp, "%d ", m->NumberControlPoints);
			fprintf(bbafp, "%d ", m->NumberKnots);
			if (m->isControlPointsChanged()){
				fprintf(bbafp, "%d ", 1);
				for (j=0;j<m->NumberControlPoints;j++){
					fprintf(bbafp, "%d ", m->muscleMask.isControlPointOnXChanged[j]);
					fprintf(bbafp, "%d ", m->muscleMask.isControlPointOnYChanged[j]);
					fprintf(bbafp, "%d ", m->muscleMask.isControlPointOnZChanged[j]);
				}
			}else
				fprintf(bbafp, "%d ", 0);
			if (m->isWheightOfControlPointsChanged()){
				fprintf(bbafp, "%d ", 1);
				for (j=0;j<m->NumberControlPoints;j++){
					fprintf(bbafp, "%d ", m->muscleMask.isWheighOfControlPointChanged[j]);
				}
			}else
				fprintf(bbafp, "%d ", 0);
			if (m->isKnotChanged()){
				fprintf(bbafp, "%d ", 1);
				for (j=0;j<m->NumberKnots;j++){
					fprintf(bbafp, "%d ", m->muscleMask.isKnotChanged[j]);
				}
			}else
				fprintf(bbafp, "%d ", 0);
			m=m->suiv;
	}
	fprintf(bbafp, "\n");
	q=bbaop.firstBone;
	while(q){
		if (q->boneMask.isTonXChanged) 
			fprintf(bbafp, "%d ", q->boneDCTData[seg].TonX);
		if (q->boneMask.isTonYChanged) 
			fprintf(bbafp, "%d ", q->boneDCTData[seg].TonY);
		if (q->boneMask.isTonZChanged) 
			fprintf(bbafp, "%d ", q->boneDCTData[seg].TonZ);
		if (!q->boneMask.isQuaternion){
			if (q->boneMask.isRonAx1Changed) 
				fprintf(bbafp, "%d ", q->boneDCTData[seg].RonAx1);
			if (q->boneMask.isRonAx2Changed) 
				fprintf(bbafp, "%d ", q->boneDCTData[seg].RonAx2);
			if (q->boneMask.isRonAx3Changed) 
				fprintf(bbafp, "%d ", q->boneDCTData[seg].RonAx3);
		}else{
			if (q->boneMask.isRonQxChanged) 
				fprintf(bbafp, "%d ", q->boneDCTData[seg].RonQx);
			if (q->boneMask.isRonQyChanged) 
				fprintf(bbafp, "%d ", q->boneDCTData[seg].RonQy);
			if (q->boneMask.isRonQzChanged) 
				fprintf(bbafp, "%d ", q->boneDCTData[seg].RonQz);
			if (q->boneMask.isRonQwChanged) 
				fprintf(bbafp, "%d ", q->boneDCTData[seg].RonQw);
		}
		if (q->boneMask.isSonXChanged) 
			fprintf(bbafp, "%d ", q->boneDCTData[seg].SonX);
		if (q->boneMask.isSonYChanged) 
			fprintf(bbafp, "%d ", q->boneDCTData[seg].SonY);
		if (q->boneMask.isSonZChanged) 
			fprintf(bbafp, "%d ", q->boneDCTData[seg].SonZ);
		if (q->boneMask.isSOonAxXChanged) 
			fprintf(bbafp, "%d ", q->boneDCTData[seg].SOonAxX);
		if (q->boneMask.isSOonAxYChanged) 
			fprintf(bbafp, "%d ", q->boneDCTData[seg].SOonAxY);
		if (q->boneMask.isSOonAxZChanged) 
			fprintf(bbafp, "%d ", q->boneDCTData[seg].SOonAxZ);
		if (q->boneMask.isSOonValChanged) 
			fprintf(bbafp, "%d ", q->boneDCTData[seg].SOonVal);
		if (q->boneMask.isConXChanged) 
			fprintf(bbafp, "%d ", q->boneDCTData[seg].ConX);
		if (q->boneMask.isConYChanged) 
			fprintf(bbafp, "%d ", q->boneDCTData[seg].ConY);
		if (q->boneMask.isConZChanged) 
			fprintf(bbafp, "%d ", q->boneDCTData[seg].ConZ);
		q=q->suiv;
	}
	m=bbaop.firstMuscle;
	while(m){
		if (m->isControlPointsChanged()){
			for (j=0;j<m->NumberControlPoints;j++){
				if (m->muscleMask.isControlPointOnXChanged[j])
					fprintf(bbafp, "%d ", m->muscleDCTData[seg].ControlPointOnX[j]);
				if (m->muscleMask.isControlPointOnYChanged[j])
					fprintf(bbafp, "%d ", m->muscleDCTData[seg].ControlPointOnY[j]);
				if (m->muscleMask.isControlPointOnZChanged[j])
					fprintf(bbafp, "%d ", m->muscleDCTData[seg].ControlPointOnZ[j]);
			}
		}
		if (m->isWheightOfControlPointsChanged()){
			for (j=0;j<m->NumberControlPoints;j++){
				if (m->muscleMask.isWheighOfControlPointChanged[j])
					fprintf(bbafp, "%d ", m->muscleDCTData[seg].WheighOfControlPoint[j]);
			}
		}
		if (m->isKnotChanged()){
			for (j=0;j<m->NumberKnots;j++){
				if (m->muscleMask.isKnotChanged[j])
					fprintf(bbafp, "%d ", m->muscleDCTData[seg].Knot[j]);
			}
		}
		m=m->suiv;
	}
	fprintf(bbafp, "\n");
	fflush(bbafp);
	}
}


int BBADecoder::AllocThePlane(){
	
	int no=0;
	CBone *lb=bbaop.firstBone;
	while (lb){
		if (lb->boneMask.isQuaternion) no+=17;
		else no+=16;
		lb=lb->suiv;
	}
	//no+=bbaop.NumberOfBones*16;
	if (bbaop.noOfControllerTypes>1){
		CMuscle *lm=bbaop.firstMuscle;
		while(lm){
			no+=lm->NumberControlPoints*4;
			no+=lm->NumberKnots;
			lm=lm->suiv;
		}
	}
	if (bbaop.noOfControllerTypes>2){
		CMorph *lmf=bbaop.firstMorph;
		while(lmf){
			no+=lmf->NumberWeights;
			lmf=lmf->suiv;
		}
	}

	bbaop.NumberOfSBComponentsInPlane=no;
	bbaop.sbc_mask  = (bool*) malloc (no*sizeof(bool));
	bbaop.sbc_values= (int*) malloc (no*sizeof(int));
	bbaop.sbc_values_prev= (int*) malloc (no*sizeof(int));
	bbaop.Irange_max= (int*) malloc (no*sizeof(int));
	bbaop.Irange_min= (int*) malloc (no*sizeof(int));
	bbaop.range_max = (int*) malloc (no*sizeof(int));
	bbaop.range_min = (int*) malloc (no*sizeof(int));
	return 0;
}

int BBADecoder::FreeThePlane(){

	free (bbaop.sbc_mask);
	free (bbaop.sbc_values);
	free (bbaop.sbc_values_prev);
	free (bbaop.Irange_max);
	free (bbaop.Irange_min);
	free (bbaop.range_max); 
	free (bbaop.range_min); 
	return 0;
}



int BBADecoder::LoadThePlane(){
	
	// compute the plane specific data 
	int i, no=0;
	int count=0;
	CBone *lb= new CBone();
	lb = bbaop.firstBone;
	while (lb){
		if ((!lb->isInCurrentFrame)&&(bbaop.is_intra))
			lb=lb->suiv;
		else if ((!lb->canBePredicted)&&(!bbaop.is_intra))
			lb=lb->suiv;
		else{
			bbaop.sbc_mask[count]=lb->boneMask.isTonXChanged;
			bbaop.sbc_values[count]=lb->boneData.TonX;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.TonX;
			bbaop.Irange_max[count]=lb->boneDataIMax.TonX;
			bbaop.Irange_min[count]=lb->boneDataIMin.TonX;
			bbaop.range_max[count]=lb->boneDataPMax.TonX;
			bbaop.range_min[count]=lb->boneDataPMin.TonX;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isTonYChanged;
			bbaop.sbc_values[count]=lb->boneData.TonY;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.TonY;
			bbaop.Irange_max[count]=lb->boneDataIMax.TonY;
			bbaop.Irange_min[count]=lb->boneDataIMin.TonY;
			bbaop.range_max[count]=lb->boneDataPMax.TonY;
			bbaop.range_min[count]=lb->boneDataPMin.TonY;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isTonZChanged;
			bbaop.sbc_values[count]=lb->boneData.TonZ;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.TonZ;
			bbaop.Irange_max[count]=lb->boneDataIMax.TonZ;
			bbaop.Irange_min[count]=lb->boneDataIMin.TonZ;
			bbaop.range_max[count]=lb->boneDataPMax.TonZ;
			bbaop.range_min[count]=lb->boneDataPMin.TonZ;
			count++;

			if (!lb->boneMask.isQuaternion){
				bbaop.sbc_mask[count]=lb->boneMask.isRonAx1Changed;
				bbaop.sbc_values[count]=lb->boneData.RonAx1;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonAx1;
				bbaop.Irange_max[count]=lb->boneDataIMax.RonAx1;
				bbaop.Irange_min[count]=lb->boneDataIMin.RonAx1;
				bbaop.range_max[count]=lb->boneDataPMax.RonAx1;
				bbaop.range_min[count]=lb->boneDataPMin.RonAx1;
				count++;
			
				bbaop.sbc_mask[count]=lb->boneMask.isRonAx2Changed;
				bbaop.sbc_values[count]=lb->boneData.RonAx2;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonAx2;
				bbaop.Irange_max[count]=lb->boneDataIMax.RonAx2;
				bbaop.Irange_min[count]=lb->boneDataIMin.RonAx2;
				bbaop.range_max[count]=lb->boneDataPMax.RonAx2;
				bbaop.range_min[count]=lb->boneDataPMin.RonAx2;
				count++;

				bbaop.sbc_mask[count]=lb->boneMask.isRonAx3Changed;
				bbaop.sbc_values[count]=lb->boneData.RonAx3;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonAx3;
				bbaop.Irange_max[count]=lb->boneDataIMax.RonAx3;
				bbaop.Irange_min[count]=lb->boneDataIMin.RonAx3;
				bbaop.range_max[count]=lb->boneDataPMax.RonAx3;
				bbaop.range_min[count]=lb->boneDataPMin.RonAx3;
				count++;
			}else{
				bbaop.sbc_mask[count]=lb->boneMask.isRonQxChanged;
				bbaop.sbc_values[count]=lb->boneData.RonQx;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQx;
				bbaop.Irange_max[count]=lb->boneDataIMax.RonQx;
				bbaop.Irange_min[count]=lb->boneDataIMin.RonQx;
				bbaop.range_max[count]=lb->boneDataPMax.RonQx;
				bbaop.range_min[count]=lb->boneDataPMin.RonQx;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isRonQyChanged;
				bbaop.sbc_values[count]=lb->boneData.RonQy;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQy;
				bbaop.Irange_max[count]=lb->boneDataIMax.RonQy;
				bbaop.Irange_min[count]=lb->boneDataIMin.RonQy;
				bbaop.range_max[count]=lb->boneDataPMax.RonQy;
				bbaop.range_min[count]=lb->boneDataPMin.RonQy;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isRonQzChanged;
				bbaop.sbc_values[count]=lb->boneData.RonQz;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQz;
				bbaop.Irange_max[count]=lb->boneDataIMax.RonQz;
				bbaop.Irange_min[count]=lb->boneDataIMin.RonQz;
				bbaop.range_max[count]=lb->boneDataPMax.RonQz;
				bbaop.range_min[count]=lb->boneDataPMin.RonQz;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isRonQwChanged;
				bbaop.sbc_values[count]=lb->boneData.RonQw;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQw;
				bbaop.Irange_max[count]=lb->boneDataIMax.RonQw;
				bbaop.Irange_min[count]=lb->boneDataIMin.RonQw;
				bbaop.range_max[count]=lb->boneDataPMax.RonQw;
				bbaop.range_min[count]=lb->boneDataPMin.RonQw;
				count++;
			}
			bbaop.sbc_mask[count]=lb->boneMask.isSonXChanged;
			bbaop.sbc_values[count]=lb->boneData.SonX;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SonX;
			bbaop.Irange_max[count]=lb->boneDataIMax.SonX;
			bbaop.Irange_min[count]=lb->boneDataIMin.SonX;
			bbaop.range_max[count]=lb->boneDataPMax.SonX;
			bbaop.range_min[count]=lb->boneDataPMin.SonX;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isSonYChanged;
			bbaop.sbc_values[count]=lb->boneData.SonY;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SonY;
			bbaop.Irange_max[count]=lb->boneDataIMax.SonY;
			bbaop.Irange_min[count]=lb->boneDataIMin.SonY;
			bbaop.range_max[count]=lb->boneDataPMax.SonY;
			bbaop.range_min[count]=lb->boneDataPMin.SonY;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isSonZChanged;
			bbaop.sbc_values[count]=lb->boneData.SonZ;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SonZ;
			bbaop.Irange_max[count]=lb->boneDataIMax.SonZ;
			bbaop.Irange_min[count]=lb->boneDataIMin.SonZ;
			bbaop.range_max[count]=lb->boneDataPMax.SonZ;
			bbaop.range_min[count]=lb->boneDataPMin.SonZ;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isSOonAxXChanged;
			bbaop.sbc_values[count]=lb->boneData.SOonAxX;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonAxX;
			bbaop.Irange_max[count]=lb->boneDataIMax.SOonAxX;
			bbaop.Irange_min[count]=lb->boneDataIMin.SOonAxX;
			bbaop.range_max[count]=lb->boneDataPMax.SOonAxX;
			bbaop.range_min[count]=lb->boneDataPMin.SOonAxX;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isSOonAxYChanged;
			bbaop.sbc_values[count]=lb->boneData.SOonAxY;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonAxY;
			bbaop.Irange_max[count]=lb->boneDataIMax.SOonAxY;
			bbaop.Irange_min[count]=lb->boneDataIMin.SOonAxY;
			bbaop.range_max[count]=lb->boneDataPMax.SOonAxY;
			bbaop.range_min[count]=lb->boneDataPMin.SOonAxY;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isSOonAxZChanged;
			bbaop.sbc_values[count]=lb->boneData.SOonAxZ;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonAxZ;
			bbaop.Irange_max[count]=lb->boneDataIMax.SOonAxZ;
			bbaop.Irange_min[count]=lb->boneDataIMin.SOonAxZ;
			bbaop.range_max[count]=lb->boneDataPMax.SOonAxZ;
			bbaop.range_min[count]=lb->boneDataPMin.SOonAxZ;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isSOonValChanged;
			bbaop.sbc_values[count]=lb->boneData.SOonVal;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonVal;
			bbaop.Irange_max[count]=lb->boneDataIMax.SOonVal;
			bbaop.Irange_min[count]=lb->boneDataIMin.SOonVal;
			bbaop.range_max[count]=lb->boneDataPMax.SOonVal;
			bbaop.range_min[count]=lb->boneDataPMin.SOonVal;
			count++;
			
			bbaop.sbc_mask[count]=lb->boneMask.isConXChanged;
			bbaop.sbc_values[count]=lb->boneData.ConX;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.ConX;
			bbaop.Irange_max[count]=lb->boneDataIMax.ConX;
			bbaop.Irange_min[count]=lb->boneDataIMin.ConX;
			bbaop.range_max[count]=lb->boneDataPMax.ConX;
			bbaop.range_min[count]=lb->boneDataPMin.ConX;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isConYChanged;
			bbaop.sbc_values[count]=lb->boneData.ConY;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.ConY;
			bbaop.Irange_max[count]=lb->boneDataIMax.ConY;
			bbaop.Irange_min[count]=lb->boneDataIMin.ConY;
			bbaop.range_max[count]=lb->boneDataPMax.ConY;
			bbaop.range_min[count]=lb->boneDataPMin.ConY;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isConZChanged;
			bbaop.sbc_values[count]=lb->boneData.ConZ;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.ConZ;
			bbaop.Irange_max[count]=lb->boneDataIMax.ConZ;
			bbaop.Irange_min[count]=lb->boneDataIMin.ConZ;
			bbaop.range_max[count]=lb->boneDataPMax.ConZ;
			bbaop.range_min[count]=lb->boneDataPMin.ConZ;
			count++;
			
			lb=lb->suiv;
		}
	}
	/////////////////////muscles////////////////////////////
	if (bbaop.noOfControllerTypes>1){
		CMuscle *lm= new CMuscle();
		lm = bbaop.firstMuscle;
		while (lm){
			if ((!lm->isInCurrentFrame)&&(bbaop.is_intra))
				lm=lm->suiv;
			else if ((!lm->canBePredicted)&&(!bbaop.is_intra))
				lm=lm->suiv;
			else{
				if (lm->isControlPointsChanged()){
					for (i=0;i<lm->NumberControlPoints;i++){
						bbaop.sbc_mask[count]=lm->muscleMask.isControlPointOnXChanged[i];
						bbaop.sbc_values[count]=lm->muscleData.ControlPointOnX[i];
						bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.ControlPointOnX[i];
						bbaop.Irange_max[count]=lm->muscleDataIMax.ControlPointOnX[i];
						bbaop.Irange_min[count]=lm->muscleDataIMin.ControlPointOnX[i];
						bbaop.range_max[count]=lm->muscleDataPMax.ControlPointOnX[i];
						bbaop.range_min[count]=lm->muscleDataPMin.ControlPointOnX[i];
						count++;

						bbaop.sbc_mask[count]=lm->muscleMask.isControlPointOnYChanged[i];
						bbaop.sbc_values[count]=lm->muscleData.ControlPointOnY[i];
						bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.ControlPointOnY[i];
						bbaop.Irange_max[count]=lm->muscleDataIMax.ControlPointOnY[i];
						bbaop.Irange_min[count]=lm->muscleDataIMin.ControlPointOnY[i];
						bbaop.range_max[count]=lm->muscleDataPMax.ControlPointOnY[i];
						bbaop.range_min[count]=lm->muscleDataPMin.ControlPointOnY[i];
						count++;

						bbaop.sbc_mask[count]=lm->muscleMask.isControlPointOnZChanged[i];
						bbaop.sbc_values[count]=lm->muscleData.ControlPointOnZ[i];
						bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.ControlPointOnZ[i];
						bbaop.Irange_max[count]=lm->muscleDataIMax.ControlPointOnZ[i];
						bbaop.Irange_min[count]=lm->muscleDataIMin.ControlPointOnZ[i];
						bbaop.range_max[count]=lm->muscleDataPMax.ControlPointOnZ[i];
						bbaop.range_min[count]=lm->muscleDataPMin.ControlPointOnZ[i];
						count++;
					}
				}
				if (lm->isWheightOfControlPointsChanged()){
						bbaop.sbc_mask[count]=lm->muscleMask.isWheighOfControlPointChanged[i];
						bbaop.sbc_values[count]=lm->muscleData.WheighOfControlPoint[i];
						bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.WheighOfControlPoint[i];
						bbaop.Irange_max[count]=lm->muscleDataIMax.WheighOfControlPoint[i];
						bbaop.Irange_min[count]=lm->muscleDataIMin.WheighOfControlPoint[i];
						bbaop.range_max[count]=lm->muscleDataPMax.WheighOfControlPoint[i];
						bbaop.range_min[count]=lm->muscleDataPMin.WheighOfControlPoint[i];
						count++;
				}
				if (lm->isKnotChanged()){
						bbaop.sbc_mask[count]=lm->muscleMask.isKnotChanged[i];
						bbaop.sbc_values[count]=lm->muscleData.Knot[i];
						bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.Knot[i];
						bbaop.Irange_max[count]=lm->muscleDataIMax.Knot[i];
						bbaop.Irange_min[count]=lm->muscleDataIMin.Knot[i];
						bbaop.range_max[count]=lm->muscleDataPMax.Knot[i];
						bbaop.range_min[count]=lm->muscleDataPMin.Knot[i];
						count++;
				}
				lm=lm->suiv;
			}
		}
	}
		///////////////// morph
	if (bbaop.noOfControllerTypes>2){
		CMorph *lmf= new CMorph();
		lmf = bbaop.firstMorph;
		while (lmf){
			if (!lmf->isInCurrentFrame)
				lmf=lmf->suiv;
			else{
				if (lmf->isWeightsChanged()){
					for (i=0;i<lmf->NumberWeights;i++){
						bbaop.sbc_mask[count]=lmf->morphMask.isWeightChanged[i];
						bbaop.sbc_values[count]=lmf->morphData.Weights[i];
						bbaop.sbc_values_prev[count]=lmf->morphDataPreviousFrame.Weights[i];
						bbaop.Irange_max[count]=lmf->morphDataIMax.Weights[i];
						bbaop.Irange_min[count]=lmf->morphDataIMin.Weights[i];
						bbaop.range_max[count]=lmf->morphDataPMax.Weights[i];
						bbaop.range_min[count]=lmf->morphDataPMin.Weights[i];
						count++;
					}
				}
				lmf=lmf->suiv;
			}
		}
	}
	return true;
}

void BBADecoder::bba_init_htables() {
  int i, j, n, m;

  static unsigned char dc[512][18]={
	{17,1,1,0,1,0,1,1,1,0,0,1,1,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,1,1,1},
	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,1,1,1},
	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,1,1,1},
	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,1,0,1},
	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,1,0,1},
	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,1,0,1},
	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,1,0,1},
	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1},	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,1,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,1,1,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,1,0,1,0},
	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,1,1,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,1,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,1,0,1},
	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,1,1,1},	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,1,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,1,0,1},
	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,1,0,1},
	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,1,1,1},	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,1,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,1,1,1},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,1,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,1,1,1},
	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,1,1,1},
	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,1,1,1},
	{16,1,1,0,1,0,1,1,1,0,0,1,1,0,0,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,1,1,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,0,1,1},
	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,0,1,1},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,0,1,1},	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,0,0,1},
	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,0,1,1},	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,0,1,1},
	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,0,1,1},	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,0,1,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,0,1,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,0,1,1},
	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,0,1,1},
	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,0,1,1},
	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,0,1,1},
	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,0,0,1},
	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,0,0,1},
	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,0,0,1},
	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,0,0,1},
	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,0,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,0,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,0,0,1},	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,0,0,1},
	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,0,0,1},	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,0,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,0,0,1},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,0,0,1},
	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,0,0,1},	{11,1,1,1,0,1,1,0,0,1,0,1,0,0,0,0,0,0},	{11,1,1,0,1,1,1,1,1,0,1,1,0,0,0,0,0,0},	{11,1,1,0,1,1,1,1,0,0,0,1,0,0,0,0,0,0},	{10,1,1,0,1,1,1,0,0,1,1,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,0,0,0},
	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,1,0,1},
	{11,1,1,1,0,1,1,0,0,1,1,1,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,1,0,1},	{11,1,1,1,0,1,1,0,0,1,1,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,1,0,0},	{8,1,1,0,1,0,1,1,0,0,0,0,0,0,0,0,0,0},	{11,1,1,0,1,1,1,1,0,0,1,0,0,0,0,0,0,0},	{9,1,1,0,0,1,0,1,0,0,0,0,0,0,0,0,0,0},
	{10,1,1,0,1,1,1,0,0,0,1,0,0,0,0,0,0,0},	{9,1,1,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0},	{10,1,1,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0},	{9,1,1,0,0,1,0,1,0,1,0,0,0,0,0,0,0,0},	{9,1,1,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0},	{10,1,1,0,1,1,1,0,1,0,0,0,0,0,0,0,0,0},	{7,1,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0},	{8,1,1,1,0,1,1,0,1,0,0,0,0,0,0,0,0,0},
	{8,1,1,0,0,1,0,1,1,0,0,0,0,0,0,0,0,0},	{7,1,1,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0},	{7,1,1,0,1,1,0,1,0,0,0,0,0,0,0,0,0,0},	{7,1,1,1,0,1,1,1,0,0,0,0,0,0,0,0,0,0},	{6,1,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0},	{6,1,1,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0},	{5,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0},	{3,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
	{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},	{3,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0},	{5,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0},	{6,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0},	{6,1,1,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0},	{6,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},	{7,1,1,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0},	{7,1,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0},
	{7,1,1,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0},	{8,1,1,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0},	{8,1,1,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0},	{8,1,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,1,0,1},	{9,1,1,1,0,1,1,0,0,0,0,0,0,0,0,0,0,0},	{11,1,1,0,1,1,1,1,1,0,1,0,0,0,0,0,0,0},	{11,1,1,0,1,1,1,1,0,1,0,1,0,0,0,0,0,0},
	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,1,0,1},	{10,1,1,0,1,1,1,1,0,1,1,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,1,0,1},	{11,1,1,0,1,1,1,1,0,0,1,1,0,0,0,0,0,0},	{9,1,1,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0},	{11,1,1,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0},	{10,1,1,0,1,1,1,0,1,1,1,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,1,0,1},
	{16,1,1,0,1,0,1,1,1,0,0,1,1,0,1,0,0,0},	{10,1,1,0,1,1,1,0,0,1,0,0,0,0,0,0,0,0},	{10,1,1,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0},	{11,1,1,0,1,1,1,0,1,0,1,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,1,0,0},	{11,1,1,0,1,1,1,0,1,1,0,0,0,0,0,0,0,0},
	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,1,0,0},	{11,1,1,1,0,1,1,0,0,1,0,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,0,0,0},	{11,1,1,0,1,1,1,1,0,1,0,0,0,0,0,0,0,0},	{11,1,1,0,1,1,1,0,1,0,1,1,0,0,0,0,0,0},
	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,0,0,0},	{11,1,1,0,1,1,1,0,1,1,0,1,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,0,0,0},
	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,0,0,0,0},
	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,0,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,0,0,0},
	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,0,0,0},
	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,0,0,0},
	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,0,0,0},
	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,0,1,0},
	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,0,1,0},
	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,0,1,0},
	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,0,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,0,1,0,0},
	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,0,1,0},
	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,0,1,0},
	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,0,0,0},
	{16,1,1,0,1,0,1,1,1,0,0,1,1,0,0,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,1,1,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,1,1,0},
	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,1,1,0},
	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,1,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,1,1,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,1,1,0,0},
	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,1,1,0},
	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,1,1,0,0},
	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,1,1,0,0},
	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,1,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,1,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,1,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,1,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,1,1,0},
	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,1,1,0},
	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,1,1,0},
	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,1,1,0},
	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,1,0,0},
	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,1,1,0},
	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,1,0,0},
	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,0,1,1,0,0},
	};

	static unsigned char ac[512][18]={
	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,1,0,1,0},
	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,1,1,1,0},
	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,1,1,1,1,0,0},
	{15,1,0,0,0,0,1,0,1,1,1,1,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,1,1,1,0},
	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,1,1,1,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,1,1,1,0,0},
	{15,1,0,0,0,0,1,0,1,1,0,0,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,1,1,1,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,1,1,1,0,0},
	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,1,1,1,0},	{15,1,0,0,0,0,1,0,0,0,1,0,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,1,1,1,0,0},
	{15,1,0,0,0,0,1,0,1,0,1,0,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,1,1,1,0},
	{15,1,0,0,0,0,1,0,0,0,0,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,1,1,1,0,0},
	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,1,1,1,0},	{15,1,0,0,0,0,1,1,1,0,0,0,1,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,0,1,1,0},
	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,0,1,1,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,0,1,1,0,0},
	{15,1,0,0,0,0,1,0,1,0,0,1,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,0,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,0,1,1,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,0,1,1,0,0},
	{15,1,0,0,0,0,1,0,0,1,0,0,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,0,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,0,1,1,0},
	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,0,1,1,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,0,1,1,0},
	{15,1,0,0,0,0,1,0,0,0,1,0,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,0,1,1,0,0},
	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,0,1,1,0},	{15,1,0,0,0,0,1,0,0,0,0,1,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,1,0,1,1,0,0},
	{15,1,0,0,0,0,1,0,1,1,1,1,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,0,1,1,0},
	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,0,0,1,0},
	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,0,0,1,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,0,0,1,0,0},
	{15,1,0,0,0,0,1,0,0,1,0,1,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,1,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,0,0,1,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0},
	{15,1,0,0,0,0,1,0,1,0,0,0,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,1,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,0,0,1,0},
	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,0,0,1,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,0,0,1,0},	{15,1,0,0,0,0,1,0,0,0,1,0,0,0,0,1,0,0},
	{15,1,0,0,0,0,1,0,0,1,1,0,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,0,0,1,0},
	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,0,0,1,0},	{15,1,0,0,0,0,1,0,0,0,0,1,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,0,0,0,1,0,0},
	{15,1,0,0,0,0,1,0,1,1,0,1,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,0,0,1,0},
	{15,1,0,0,0,0,1,1,1,0,0,0,1,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,1,0,1,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,0,1,0,1,0,0},
	{15,1,0,0,0,0,1,0,1,1,0,1,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,1,0,1,0},
	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,1,0,1,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,0,1,0,1,0,0},
	{15,1,0,0,0,0,1,0,0,1,1,0,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,1,0,1,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,1,0,1,0},
	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,1,0,1,0},	{15,1,0,0,0,0,1,0,0,0,1,0,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,1,1,0,1,0,0},
	{15,1,0,0,0,0,1,0,1,0,0,0,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,1,0,1,0},	{15,1,0,0,0,0,1,0,0,0,0,1,1,1,0,1,0,0},
	{6,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,1,1,0,1,0,0},	{7,1,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0},	{6,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0},	{5,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0},	{4,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0},	{2,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,1,0,0,0},	{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},	{4,1,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0},	{6,1,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0},	{6,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0},	{7,1,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,1,0,1,0},	{8,1,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0},
	{15,1,0,0,0,0,1,0,1,0,1,1,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,1,0,0,0,0},
	{15,1,0,0,0,0,1,0,0,1,0,0,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,1,0,0,0},
	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,1,0,0,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,1,0,0,0},
	{15,1,0,0,0,0,1,0,0,0,1,0,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,1,0,0,0,0},
	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,1,0,0,0,0},
	{15,1,0,0,0,0,1,0,1,1,1,1,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,1,0,0,0},
	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,1,0,0,0},	{15,1,0,0,0,0,1,1,1,0,0,0,1,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,0,0,0,0,0},
	{15,1,0,0,0,0,1,0,0,1,1,1,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,0,0,0,0},
	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,0,0,0,0,0},
	{15,1,0,0,0,0,1,0,1,1,0,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,0,0,0,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0},
	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,0,0,0,0,0},
	{15,1,0,0,0,0,1,0,1,0,1,0,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,0,0,0,0},
	{15,1,0,0,0,0,1,0,0,0,0,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,0,0,0,0,0},
	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,0,1,0,0},
	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,0,1,0,0,0},
	{15,1,0,0,0,0,1,0,1,1,0,1,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,1,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,0,1,0,0},
	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,1,0,1,0,0,0}, {15,1,0,0,0,0,1,0,1,0,1,0,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,1,0,1,0,0,0},
	{15,1,0,0,0,0,1,0,0,1,1,0,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,1,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,0,1,0,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,0,1,0,0},
	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,0,0,1,0,0,0},
	{15,1,0,0,0,0,1,0,1,0,0,0,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,0,0,1,0,0,0},
	{15,1,0,0,0,0,1,0,0,1,0,1,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,0,1,0,0},
	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,0,1,0,0},	{15,1,0,0,0,0,1,1,1,0,0,0,1,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,1,1,0,0},
	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,0,1,1,0,0,0},
	{15,1,0,0,0,0,1,0,0,1,0,1,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,0,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,0,1,1,0,0,0},
	{15,1,0,0,0,0,1,0,1,0,0,0,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,0,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,1,1,0,0},
	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,1,1,0,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,1,1,1,0,0,0},
	{15,1,0,0,0,0,1,0,0,1,1,0,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,1,1,0,0},
	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,1,1,1,0,0,0},
	{15,1,0,0,0,0,1,0,1,1,1,1,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,1,0,0,0,0},
	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,1,1,0,0},
	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,0,1,0,0}
	};
	
	static unsigned char run[16][10]={
		{1,1,0,0,0,0,0,0,0,0},		{2,0,1,0,0,0,0,0,0,0},
		{3,0,0,1,0,0,0,0,0,0},		{4,0,0,0,1,0,0,0,0,0},
		{5,0,0,0,0,1,0,0,0,0},		{9,0,0,0,0,0,1,0,1,0},
		{9,0,0,0,0,0,1,0,1,1},		{9,0,0,0,0,0,1,1,0,1},
		{9,0,0,0,0,0,1,1,1,1},		{8,0,0,0,0,0,0,1,1,0},
		{8,0,0,0,0,0,0,0,1,0},		{8,0,0,0,0,0,1,0,0,0},
		{8,0,0,0,0,0,0,0,0,0},		{8,0,0,0,0,0,0,1,0,0},
		{9,0,0,0,0,0,1,1,1,0},		{9,0,0,0,0,0,1,1,0,0},
	};


  n=512;
  bbaop.dc_table_size = n;
  bbaop.dc_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	m=dc[i][0];
	bbaop.dc_table[i] = (int*) malloc((m+1)*sizeof(int));
	bbaop.dc_table[i][0] = m;
	for (j=1; j<=m; j++) {
		bbaop.dc_table[i][j]=dc[i][j];
	}
  }

  n=16;
  bbaop.runlength_table_size = n;
  bbaop.runlength_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	m=run[i][0];
	bbaop.runlength_table[i] = (int*) malloc((m+1)*sizeof(int));
	bbaop.runlength_table[i][0] = m;
	for (j=1; j<=m; j++) {
		bbaop.runlength_table[i][j]=run[i][j];
	}
  }
  

  
  n=512;
  bbaop.ac_table_size = n;
  bbaop.ac_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	m=ac[i][0];
	bbaop.ac_table[i] = (int*) malloc((m+1)*sizeof(int));
	bbaop.ac_table[i][0] = m;
	for (j=1; j<=m; j++) {
		bbaop.ac_table[i][j]=ac[i][j];
	}
  }

}

void BBADecoder::SetMinMax(){
	
	CBone *lb=bbaop.firstBone;
	while(lb){
		if (!lb->boneMask.isQuaternion){
			lb->boneDataIMax.init(1860, 1860, 1860, 1860, 1860, 1860, 
				1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860);
			lb->boneDataIMin.init(-1860, -1860, -1860, -1860, -1860, -1860, 
				0, 0, 0, 0, 0, 0, -1860, -1860, -1860, -1860);
			lb->boneDataPMax.init(600, 600, 600, 600, 600, 600, 
				600, 600, 600, 600, 600, 600, 600, 600, 600, 600);
			lb->boneDataPMin.init(-600, -600, -600, -600, -600, -600, 
				0, 0, 0, 0, 0, 0, -600, -600, -600, -600);
		}else{
			lb->boneDataIMax.initQ(1860, 1860, 1860, 1860, 1860, 1860, 1860, 
				1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860);
			lb->boneDataIMin.initQ(-1860, -1860, -1860, -1860, -1860, -1860, -1860, 
				0, 0, 0, 0, 0, 0, -1860, -1860, -1860, -1860);
			lb->boneDataPMax.initQ(600, 600, 600, 600, 600, 600, 600, 
				600, 600, 600, 600, 600, 600, 600, 600, 600, 600);
			lb->boneDataPMin.initQ(-600, -600, -600, -600, -600, -600, -600, 
				0, 0, 0, 0, 0, 0, -600, -600, -600, -600);
		}
		lb=lb->suiv;
	}
	if (bbaop.noOfControllerTypes>1){
		CMuscle *lm=bbaop.firstMuscle;
		while (lm){
			lm->AllocMinMaxMuscle();
			lm=lm->suiv;
		}
	}
	if (bbaop.noOfControllerTypes>2){
		CMorph *lmf=bbaop.firstMorph;
		while (lmf){
			lmf->AllocMinMaxMorph();
			lmf=lmf->suiv;
		}
	}
}
void BBADecoder::AllocThePlaneSegment(){

	int no=0,i;
	CBone *lb=bbaop.firstBone;
	while (lb){
		if (lb->boneMask.isQuaternion) no+=17;
		else no+=16;
		lb=lb->suiv;
	}
	//no+=bbaop.NumberOfBones*16;
	if (bbaop.noOfControllerTypes>1){
		CMuscle *lm=bbaop.firstMuscle;
		while(lm){
			no+=lm->NumberControlPoints*4;
			no+=lm->NumberKnots;
			lm=lm->suiv;
		}
	}
	if (bbaop.noOfControllerTypes>2){
		CMorph *lmf=bbaop.firstMorph;
		while(lmf){
			no+=lmf->NumberWeights;
			lmf=lmf->suiv;
		}
	}

	bbaop.NumberOfSBComponentsInPlane=no;

	bbaop.sbc_mask  = (bool*) malloc (no*sizeof(bool));
	bbaop.sbc_DCTvalues= (int**) malloc (no*sizeof(int*));
	for (i=0;i<no;i++)
		bbaop.sbc_DCTvalues[i]=(int*) malloc (SEG_LENGTH*sizeof(int*));
	bbaop.sbc_values_prev= (int*) malloc (no*sizeof(int));

}

void BBADecoder::LoadThePlaneSegment(){
	int i, no=0,seg;
	int count=0;
	CBone *lb= new CBone();
	CMuscle *lm= new CMuscle();
	CMorph *lmf= new CMorph();

	/////////// load mask and previous values ///////////
	lb=bbaop.firstBone;
	while (lb){
		if (!lb->isInCurrentFrame)
			lb=lb->suiv;
		else{
				bbaop.sbc_mask[count]=lb->boneMask.isTonXChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.TonX;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isTonYChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.TonY;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isTonZChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.TonZ;
				count++;
				if (!lb->boneMask.isQuaternion){
					bbaop.sbc_mask[count]=lb->boneMask.isRonAx1Changed;
					bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonAx1;
					count++;				
					bbaop.sbc_mask[count]=lb->boneMask.isRonAx2Changed;
					bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonAx2;
					count++;
					bbaop.sbc_mask[count]=lb->boneMask.isRonAx3Changed;
					bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonAx3;
					count++;
				}else{
					bbaop.sbc_mask[count]=lb->boneMask.isRonQxChanged;
					bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQx;
					count++;				
					bbaop.sbc_mask[count]=lb->boneMask.isRonQyChanged;
					bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQy;
					count++;				
					bbaop.sbc_mask[count]=lb->boneMask.isRonQzChanged;
					bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQz;
					count++;				
					bbaop.sbc_mask[count]=lb->boneMask.isRonQwChanged;
					bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQw;
					count++;				
				}
				bbaop.sbc_mask[count]=lb->boneMask.isSonXChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SonX;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isSonYChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SonY;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isSonZChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SonZ;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isSOonAxXChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonAxX;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isSOonAxYChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonAxY;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isSOonAxZChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonAxZ;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isSOonValChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonVal;
				count++;				
				bbaop.sbc_mask[count]=lb->boneMask.isConXChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.ConX;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isConYChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.ConY;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isConZChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.ConZ;
				count++;				
				lb=lb->suiv;
		}
	}
	if (bbaop.noOfControllerTypes>1){
		lm = bbaop.firstMuscle;
		while (lm){
			if (!lm->isInCurrentFrame)
				lm=lm->suiv;
			else{
					for (i=0;i<lm->NumberControlPoints;i++){
						bbaop.sbc_mask[count]=lm->muscleMask.isControlPointOnXChanged[i];
						bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.ControlPointOnX[i];
						count++;
						bbaop.sbc_mask[count]=lm->muscleMask.isControlPointOnYChanged[i];
						bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.ControlPointOnY[i];
						count++;
						bbaop.sbc_mask[count]=lm->muscleMask.isControlPointOnZChanged[i];
						bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.ControlPointOnZ[i];
						count++;
					}
					for (i=0;i<lm->NumberControlPoints;i++){
						bbaop.sbc_mask[count]=lm->muscleMask.isWheighOfControlPointChanged[i];
						bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.WheighOfControlPoint[i];
						count++;
					}
					for (i=0;i<lm->NumberKnots;i++){
						bbaop.sbc_mask[count]=lm->muscleMask.isKnotChanged[i];
						bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.Knot[i];
						count++;
					}
					lm=lm->suiv;
			}
		}
	}
	if (bbaop.noOfControllerTypes>2){
		lmf = bbaop.firstMorph;
		while (lmf){
			if (!lmf->isInCurrentFrame)
				lmf=lmf->suiv;
			else{
					for (i=0;i<lmf->NumberWeights;i++){
						bbaop.sbc_mask[count]=lmf->morphMask.isWeightChanged[i];
						bbaop.sbc_values_prev[count]=lmf->morphDataPreviousFrame.Weights[i];
						count++;
					}	
				lmf=lmf->suiv;
			}
		}
	}
}

void BBADecoder::UnLoadThePlaneSegment(){
	int i, no=0,seg;
	int count=0;
	CBone *lb= new CBone();
	CMuscle *lm= new CMuscle();
	CMorph *lmf= new CMorph();

	/////////// load mask and previous values ///////////
	lb=bbaop.firstBone;
	while (lb){
		if (!lb->isInCurrentFrame)
			lb=lb->suiv;
		else{
				lb->boneMask.isTonXChanged=bbaop.sbc_mask[count];
				lb->boneDataPreviousFrame.TonX=bbaop.sbc_values_prev[count];
				count++;
				lb->boneMask.isTonYChanged=bbaop.sbc_mask[count];
				lb->boneDataPreviousFrame.TonY=bbaop.sbc_values_prev[count];
				count++;
				lb->boneMask.isTonZChanged=bbaop.sbc_mask[count];
				lb->boneDataPreviousFrame.TonZ=bbaop.sbc_values_prev[count];
				count++;
				if (!lb->boneMask.isQuaternion){
					lb->boneMask.isRonAx1Changed=bbaop.sbc_mask[count];
					lb->boneDataPreviousFrame.RonAx1=bbaop.sbc_values_prev[count];
					count++;				
					lb->boneMask.isRonAx2Changed=bbaop.sbc_mask[count];
					lb->boneDataPreviousFrame.RonAx2=bbaop.sbc_values_prev[count];
					count++;
					lb->boneMask.isRonAx3Changed=bbaop.sbc_mask[count];
					lb->boneDataPreviousFrame.RonAx3=bbaop.sbc_values_prev[count];
					count++;
				}else{
					lb->boneMask.isRonQxChanged=bbaop.sbc_mask[count];
					lb->boneDataPreviousFrame.RonQx=bbaop.sbc_values_prev[count];
					count++;				
					lb->boneMask.isRonQyChanged=bbaop.sbc_mask[count];
					lb->boneDataPreviousFrame.RonQy=bbaop.sbc_values_prev[count];
					count++;				
					lb->boneMask.isRonQzChanged=bbaop.sbc_mask[count];
					lb->boneDataPreviousFrame.RonQz=bbaop.sbc_values_prev[count];
					count++;				
					lb->boneMask.isRonQwChanged=bbaop.sbc_mask[count];
					lb->boneDataPreviousFrame.RonQw=bbaop.sbc_values_prev[count];
					count++;				
				}
				lb->boneMask.isSonXChanged=bbaop.sbc_mask[count];
				lb->boneDataPreviousFrame.SonX=bbaop.sbc_values_prev[count];
				count++;
				lb->boneMask.isSonYChanged=bbaop.sbc_mask[count];
				lb->boneDataPreviousFrame.SonY=bbaop.sbc_values_prev[count];
				count++;
				lb->boneMask.isSonZChanged=bbaop.sbc_mask[count];
				lb->boneDataPreviousFrame.SonZ=bbaop.sbc_values_prev[count];
				count++;
				lb->boneMask.isSOonAxXChanged=bbaop.sbc_mask[count];
				lb->boneDataPreviousFrame.SOonAxX=bbaop.sbc_values_prev[count];
				count++;
				lb->boneMask.isSOonAxYChanged=bbaop.sbc_mask[count];
				lb->boneDataPreviousFrame.SOonAxY=bbaop.sbc_values_prev[count];
				count++;
				lb->boneMask.isSOonAxZChanged=bbaop.sbc_mask[count];
				lb->boneDataPreviousFrame.SOonAxZ=bbaop.sbc_values_prev[count];
				count++;
				lb->boneMask.isSOonValChanged=bbaop.sbc_mask[count];
				lb->boneDataPreviousFrame.SOonVal=bbaop.sbc_values_prev[count];
				count++;				
				lb->boneMask.isConXChanged=bbaop.sbc_mask[count];
				lb->boneDataPreviousFrame.ConX=bbaop.sbc_values_prev[count];
				count++;
				lb->boneMask.isConYChanged=bbaop.sbc_mask[count];
				lb->boneDataPreviousFrame.ConY=bbaop.sbc_values_prev[count];
				count++;
				lb->boneMask.isConZChanged=bbaop.sbc_mask[count];
				lb->boneDataPreviousFrame.ConZ=bbaop.sbc_values_prev[count];
				count++;				
				lb=lb->suiv;
		}
	}
	if (bbaop.noOfControllerTypes>1){
		lm = bbaop.firstMuscle;
		while (lm){
			if (!lm->isInCurrentFrame)
				lm=lm->suiv;
			else{
				if (lm->isControlPointsChanged()){
					for (i=0;i<lm->NumberControlPoints;i++){
						lm->muscleMask.isControlPointOnXChanged[i]=bbaop.sbc_mask[count];
						lm->muscleDataPreviousFrame.ControlPointOnX[i]=bbaop.sbc_values_prev[count];
						count++;
						lm->muscleMask.isControlPointOnYChanged[i]=bbaop.sbc_mask[count];
						lm->muscleDataPreviousFrame.ControlPointOnY[i]=bbaop.sbc_values_prev[count];
						count++;
						lm->muscleMask.isControlPointOnZChanged[i]=bbaop.sbc_mask[count];
						lm->muscleDataPreviousFrame.ControlPointOnZ[i]=bbaop.sbc_values_prev[count];
						count++;
					}
				}
				if (lm->isWheightOfControlPointsChanged()){
						lm->muscleMask.isWheighOfControlPointChanged[i]=bbaop.sbc_mask[count];
						lm->muscleDataPreviousFrame.WheighOfControlPoint[i]=bbaop.sbc_values_prev[count];
						count++;
				}
				if (lm->isKnotChanged()){
						lm->muscleMask.isKnotChanged[i]=bbaop.sbc_mask[count];
						lm->muscleDataPreviousFrame.Knot[i]=bbaop.sbc_values_prev[count];
						count++;
				}
				lm=lm->suiv;
			}
		}
	}
	if (bbaop.noOfControllerTypes>2){
		lmf = bbaop.firstMorph;
		while (lmf){
			if (!lmf->isInCurrentFrame)
				lmf=lmf->suiv;
			else{
				if (lmf->isWeightsChanged()){
					for (i=0;i<lmf->NumberWeights;i++){
						lmf->morphMask.isWeightChanged[i]=bbaop.sbc_mask[count];
						lmf->morphDataPreviousFrame.Weights[i]=bbaop.sbc_values_prev[count];
						count++;
					}
				}
				lmf=lmf->suiv;
			}
		}
	}
	//////////// load the data for the entire segment

	for (seg=0;seg<SEG_LENGTH;seg++){
		count=0;
		lb = bbaop.firstBone;
		while (lb){
			if (!lb->isInCurrentFrame)
				lb=lb->suiv;
			else{
				if (lb->boneDCTData == NULL){
					lb->boneDCTData = (CBoneData*)malloc(SEG_LENGTH*sizeof(CBoneData));
				}
				lb->boneDCTData[seg].TonX=bbaop.sbc_DCTvalues[count][seg];
				count++;
				lb->boneDCTData[seg].TonY=bbaop.sbc_DCTvalues[count][seg];
				count++;
				lb->boneDCTData[seg].TonZ=bbaop.sbc_DCTvalues[count][seg];
				count++;
				if (!lb->boneMask.isQuaternion){
					lb->boneDCTData[seg].RonAx1=bbaop.sbc_DCTvalues[count][seg];
					count++;				
					lb->boneDCTData[seg].RonAx2=bbaop.sbc_DCTvalues[count][seg];
					count++;
					lb->boneDCTData[seg].RonAx3=bbaop.sbc_DCTvalues[count][seg];
					count++;
				}else{
					lb->boneDCTData[seg].RonQx=bbaop.sbc_DCTvalues[count][seg];
					count++;				
					lb->boneDCTData[seg].RonQy=bbaop.sbc_DCTvalues[count][seg];
					count++;				
					lb->boneDCTData[seg].RonQz=bbaop.sbc_DCTvalues[count][seg];
					count++;				
					lb->boneDCTData[seg].RonQw=bbaop.sbc_DCTvalues[count][seg];
					count++;				
				}
				lb->boneDCTData[seg].SonX=bbaop.sbc_DCTvalues[count][seg];
				count++;
				lb->boneDCTData[seg].SonY=bbaop.sbc_DCTvalues[count][seg];
				count++;
				lb->boneDCTData[seg].SonZ=bbaop.sbc_DCTvalues[count][seg];
				count++;
				lb->boneDCTData[seg].SOonAxX=bbaop.sbc_DCTvalues[count][seg];
				count++;
				lb->boneDCTData[seg].SOonAxY=bbaop.sbc_DCTvalues[count][seg];
				count++;
				lb->boneDCTData[seg].SOonAxZ=bbaop.sbc_DCTvalues[count][seg];
				count++;
				lb->boneDCTData[seg].SOonVal=bbaop.sbc_DCTvalues[count][seg];
				count++;				
				lb->boneDCTData[seg].ConX=bbaop.sbc_DCTvalues[count][seg];
				count++;
				lb->boneDCTData[seg].ConY=bbaop.sbc_DCTvalues[count][seg];
				count++;
				lb->boneDCTData[seg].ConZ=bbaop.sbc_DCTvalues[count][seg];
				count++;				
				lb=lb->suiv;
			}
		}
		/////////////////////muscles////////////////////////////
		if (bbaop.noOfControllerTypes>1){
			lm = bbaop.firstMuscle;
			while (lm){
				if (!lm->isInCurrentFrame)
					lm=lm->suiv;
				else{
					if (lm->muscleDCTData == NULL){
						lm->muscleDCTData = (CMuscleData*)malloc(SEG_LENGTH*sizeof(CMuscleData));
					}
					lm->AllocDCTMuscle();
					if (lm->isControlPointsChanged()){
						for (i=0;i<lm->NumberControlPoints;i++){
							lm->muscleDCTData[seg].ControlPointOnX[i]=bbaop.sbc_DCTvalues[count][seg];
							count++;
							lm->muscleDCTData[seg].ControlPointOnY[i]=bbaop.sbc_DCTvalues[count][seg];
							count++;
							lm->muscleDCTData[seg].ControlPointOnZ[i]=bbaop.sbc_DCTvalues[count][seg];
							count++;
						}
					}
					if (lm->isWheightOfControlPointsChanged()){
							lm->muscleDCTData[seg].WheighOfControlPoint[i]=bbaop.sbc_DCTvalues[count][seg];
							count++;
					}
					if (lm->isKnotChanged()){
							lm->muscleDCTData[seg].Knot[i]=bbaop.sbc_DCTvalues[count][seg];
							count++;
					}
					lm=lm->suiv;
				}
			}
		}
		if (bbaop.noOfControllerTypes>2){
		lmf = bbaop.firstMorph;
			while (lmf){
				if (!lmf->isInCurrentFrame)
					lmf=lmf->suiv;
				else{
					if (lmf->morphDCTData == NULL){
						lmf->morphDCTData = (CMorphData*)malloc(SEG_LENGTH*sizeof(CMorphData));
					}
					lmf->AllocDCTMorph();
					if (lmf->isWeightsChanged()){
						for (i=0;i<lmf->NumberWeights;i++){
							lmf->morphDCTData[seg].Weights[i]=bbaop.sbc_DCTvalues[count][seg];
							count++;
						}	
					}
					lmf=lmf->suiv;
				}
			}
		}
	}
}

void BBADecoder::FreeThePlaneSegment(){
	
	free (bbaop.sbc_mask);
	free (bbaop.sbc_values_prev);
	
	for (int i=0;i<bbaop.NumberOfSBComponentsInPlane;i++)
		free (bbaop.sbc_DCTvalues[i]);
	free (bbaop.sbc_DCTvalues);

}

void BBADecoder::initStream(FILE *infp)
{
	long i;
	int inb;
	fseek(infp,0, SEEK_END);
	i = ftell(infp);
	
	bbaop.stream.buf = (unsigned char*)malloc(i);
    bbaop.stream.bufsize = i;
	bbaop.stream.p = 0;
	bbaop.stream.incnt = 0;
	bbaop.stream.bitcnt = 0;
	i = 0;
	rewind(infp);
	while((inb = getc(infp)) != EOF) bbaop.stream.buf[i++] = inb;

}

bool BBADecoder::decode_morph_mask(CMorph *ms) {
	int marker;
	int i;
	CMorphMask *mml=&(ms->morphMask);
	
	for (i=0;i<ms->NumberWeights;i++){
			marker = getbits(1);
			mml->isWeightChanged[i]=getbits(1);
	}


	return true;
}