// -*- c++ -*-

// This software module was originally developed by Karl Anders ygard
// (Telenor R&D) in the course of development of the MPEG-4 Systems
// standard (ISO/IEC 14496-1).  This software module is an implementation
// of a part of one or more MPEG-4 Systems tools as specified by the MPEG-4
// Systems standard.  ISO/IEC gives users of the MPEG-4 Systems standard
// free license to this software module or modifications thereof for use in
// hardware or software products claiming conformance to the MPEG-4 Systems
// standard.  Those intending to use this software module in hardware or
// software products are advised that its use may infringe existing
// patents.  The original developer of this software module and his/her
// company, the subsequent editors and their companies, and ISO/IEC have no
// liability for use of this software module or modifications thereof in an
// implementation.  Copyright is not released for non MPEG-4 Systems
// conforming products.  The original developer retains full right to use
// the code for his/her own purpose, assign or donate the code to a third
// party and to inhibit third parties from using the code for non MPEG-4
// Systems conforming products.  This copyright notice must be included in
// all copies or derivative works.
//
// Copyright 1998.

#include "OpenGl/OpenGl.h"
#include "TextureNode2DProxy.h"
#include "Stream.h"
#include "opengl/base/OpenGLVisualRenderer.h"
#include "opengl/base/OpenGLPresenter.h"

// SOLARIS PORTING - removed "::"
// static members

/* static */ BOOL TextureNode2DProxy::initialised = FALSE;
/* static */ struct TextureNode2DProxy::yuvconvtable TextureNode2DProxy::convtab[256];
/* static */ int TextureNode2DProxy::cliptab[768];

// destructor

/* virtual */
TextureNode2DProxy::~TextureNode2DProxy()
{
  if (m_nTextureObject)
#if !defined(GL_VERSION_1_1)
    ::glDeleteTexturesEXT(1, &m_nTextureObject);
#else
  ::glDeleteTextures(1, &m_nTextureObject);
#endif
}

// Render object description

/* virtual */ void
TextureNode2DProxy::RenderDescription(OpenGLVisualRenderer* pRenderer, DWORD /* dwTime */)
{
  RenderDescriptionBox(pRenderer, "TextureNode2D");
}

// Make an RGB Bitmap out of the YUV  

void
TextureNode2DProxy::Yuv2Rgb(unsigned char* rgbdata,
                            unsigned char* yptr,
                            unsigned char* uptr,
                            unsigned char* vptr,
                            int lumcols, int lumrows,
                            int chrcols, int chrrows)
{
  // subsampling factors for chrominance register 
  
  int hsub = lumcols / chrcols;
  int vsub = lumrows / chrrows;
  
  // row loop
  
  for (int j = 1; j <= lumrows; ++j) {
    // loop chrominance columns 
    
    for (int i = chrcols; i > 0; --i) {
      // calculate intermediate values that can be recycled
      
      int u = *uptr++ - 128;
      int v = *vptr++ - 128;
      
      int u1 = (137 * u) /  100;
      int u2 = (698 * u) / 1000;
      int v2 = (336 * v) / 1000;
      int v3 = (173 * v) /  100;
      
      // exploit horizontal downsampling by recycling values   
      
      for (int k = hsub; k > 0; --k) { 
        int y = *yptr++;
        int tmp;
        
        tmp = y      + v3; *rgbdata++ = tmp < 255 ? (tmp > 0 ? tmp : 0) : 255; // r
        tmp = y - u2 - v2; *rgbdata++ = tmp < 255 ? (tmp > 0 ? tmp : 0) : 255; // g
        tmp = y + u1     ; *rgbdata++ = tmp < 255 ? (tmp > 0 ? tmp : 0) : 255; // b
      }
    }
    
    if (j % vsub) {
      vptr -= chrcols;
      uptr -= chrcols;
    }
  }
}

void
TextureNode2DProxy::Yuv4222Rgb(unsigned char* rgbdata,
                               unsigned char* yptr,
                               unsigned char* uptr,
                               unsigned char* vptr,
                               int lumcols, int lumrows)
{
  if (!initialised) {
    int i;
    
    for (i = 0; i < 256; ++i) {
      convtab[i].u1 = int(float(i - 128) * 1.37f);
      convtab[i].u2 = int(float(i - 128) * 0.698f);
      convtab[i].v1 = int(float(i - 128) * 0.336f);
      convtab[i].v2 = int(float(i - 128) * 1.73f);
    }
    
    for (i = 0; i < 256; ++i)
      cliptab[i] = 0;
    for (; i < 512; ++i)
      cliptab[i] = i - 256;
    for (; i < 768; ++i)
      cliptab[i] = 255;
    
    initialised = TRUE;
  }
  
  // row loop
  
  for (int j = lumrows >> 1; j > 0; --j) {
    unsigned char* tuptr = uptr;
    unsigned char* tvptr = vptr;
    int i;
    
    // loop chrominance columns 
    
    for (i = lumcols >> 1; i > 0; --i) {
      int u = *tuptr++;
      int v = *tvptr++;
      
      // horizontal downsampling
      
      for (int k = 2; k > 0; --k) { 
        int y = *yptr++;
        
        *rgbdata++ = (cliptab + 256)[y                 + convtab[v].v2]; // r
        *rgbdata++ = (cliptab + 256)[y - convtab[u].u2 - convtab[v].v1]; // g
        *rgbdata++ = (cliptab + 256)[y + convtab[u].u1                ]; // b
      }
    }
    
    for (i = lumcols >> 1; i > 0; --i) {
      // calculate intermediate values that can be recycled
      
      int u = *uptr++;
      int v = *vptr++;
      
      // horizontal downsampling   
      
      for (int k = 2; k > 0; --k) { 
        int y = *yptr++;
        
        *rgbdata++ = (cliptab + 256)[y                 + convtab[v].v2]; // r
        *rgbdata++ = (cliptab + 256)[y - convtab[u].u2 - convtab[v].v1]; // g
        *rgbdata++ = (cliptab + 256)[y + convtab[u].u1                ]; // b
      }
    }
  }
}

// render object

/* virtual */ BOOL
TextureNode2DProxy::Render(const Matrix& rTransformation, const CullFlag& rCullFlag,
                           OpenGLVisualRenderer* pRenderer, DWORD dwTime)
{
  MediaStream* pStream = ((StreamConsumer *) GetMediaObject())->GetStream();
  /* TextureNode2D */ StreamConsumer * pTextureNode2D = 
      /* (TextureNode2D*) */ (StreamConsumer *) GetMediaObject();
  
  if (pStream) {
    int nSize;
    DWORD dwTime;
    BOOL bRecurrent;
    
    /***
    unsigned char* pData = (unsigned char*) pStream->Fetch(
    nSize,
				dwTime,
        bRecurrent,
        TRUE,
        FALSE,
        0,
        TRUE);
    ***/
    
    // SB @@@@ I am calling Fetch without using nRecurrent!!!
    // what is th edefault? guess: FALSE
    BOOL bSkip = TRUE;
    BOOL bWait = FALSE;
    BOOL bEOU = FALSE;
    int nStatus = 0;
    unsigned char * pData = (unsigned char *) pStream->Fetch(
      nSize, dwTime, bEOU, nStatus, bSkip, bWait);
    bRecurrent = FALSE;
    
    /* if (rTransformation.HasOnlyTranslation()) {
    if (pData != END_OF_STREAM) {
    if (pData) {
    int width;
    int height;
    
      width = m_nWidth = (((unsigned short) pData[0]) << 8) | pData[1];
      height = m_nHeight = (((unsigned short) pData[2]) << 8) | pData[3];
      
        pData += 4;
        
          ::glPushAttrib(GL_ENABLE_BIT);
          
            // Disable stuff that's likely to slow down glDrawPixels
            
              ::glDisable(GL_DEPTH_TEST);
              ::glDisable(GL_FOG);
              ::glDisable(GL_LIGHTING);
              
                #ifdef GL_SGIS_multisample
                ::glDisable(GL_MULTISAMPLE_SGIS);
                #endif
                
                  ::glRasterPos2f(0, 0);
                  
                    if (pTextureNode2D->GetStreamFormat() == FORMAT_RGBA)
                    ::glDrawPixels(width, height,
                    GL_RGBA, GL_UNSIGNED_BYTE, pData);
                    else {	  
                    ::glDisable(GL_ALPHA_TEST);
                    ::glDisable(GL_BLEND);
                    ::glDrawPixels(width, height,
                    GL_RGB, GL_UNSIGNED_BYTE, pData);
                    }
                    
                      ::glPopAttrib();
                      
                        pStream->Release(FALSE);
                        }
                        } else
                        pStream->Terminate();
  } else */ {
    // bind texture object
    
#if !defined(GL_VERSION_1_1)
    if (m_nTextureObject == 0)
      ::glGenTexturesEXT(1, &m_nTextureObject);
    
    ::glBindTextureEXT(GL_TEXTURE_2D, m_nTextureObject);
#else
    if (m_nTextureObject == 0)
      ::glGenTextures(1, &m_nTextureObject);
    
    ::glBindTexture(GL_TEXTURE_2D, m_nTextureObject);
#endif
    
    ::glEnable(GL_TEXTURE_2D);
    
    if (pData != END_OF_STREAM)
    {
      if (pData)
      {
        if (!bRecurrent)
        {
          int width;
          int height;
          
          width = m_nWidth = (((unsigned short) pData[0]) << 8) | pData[1];
          height = m_nHeight = (((unsigned short) pData[2]) << 8) | pData[3];
          
          pData += 4;
          
          {
            Vec3f size;
            Vec3f center;

// SOLARIS PORTING
// Created new objects to avoid memory dereferencing
            SF<Vec3f> sf_size, sf_center;
            sf_size = (SF<Vec3f>) size;
            sf_center = (SF<Vec3f>) center;

            GetBbox( sf_size, sf_center);

            size = (Vec3f)sf_size;
            center = (Vec3f)sf_center;

// SOLARIS PORTING - END
            
            
            if (size != Vec3f(m_nWidth, m_nHeight) ||
              center != Vec3f())
            {
// SOLARIS PORTING
// To avoid memory dereferencing
              sf_size = (SF<Vec3f>) Vec3f(m_nWidth, m_nHeight, 0.0f);
              sf_center = (SF<Vec3f>) Vec3f(0.0f, 0.0f, 0.0f);
              /* pTextureNode2D-> */
              SetBbox( sf_size, sf_center );
// SOLARIS PORTING - END
            }
          }
          
          {
            // textures must have size and width == 2^n
            
            int numshifts;
            
            // align width
            
            numshifts = -1;
            
            for (int i = 0; i < 16; ++i) {
              if (width & 1)
                numshifts = i;
              
              width >>= 1;
            }
            
            width = 1 << numshifts;
            if (width < m_nWidth)
              width <<= 1;
            
            // align height
            
            numshifts = -1;
            
            for (int j = 0; j < 15; ++j) {
              if (height & 1)
                numshifts = j;
              
              height >>= 1;
            }
            
            height = 1 << numshifts;
            if (height < m_nHeight)
              height <<= 1;
          }	    
          
          // has the texture environment been set up properly?
          
          if (width != m_nTextureWidth ||
            height != m_nTextureHeight) {
#if !defined(GL_VERSION_1_1)
            ::glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);
#else
            ::glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_REPLACE);
#endif
            
            //        int UVwidth = m_nWidth/2;
            //        int UVheight = m_nHeight/2;
            //
            //        Yuv2Rgb(buffer,
            //                ((unsigned char*) pData),
            //                ((unsigned char*) pData) + (m_nWidth * m_nHeight),
            //                ((unsigned char*) pData) + (m_nWidth * m_nHeight) + (UVwidth * UVheight),
            //                m_nWidth, m_nHeight,
            //                UVwidth, UVheight);
            
            if (height != m_nHeight ||
              width != m_nWidth)
              if (pTextureNode2D->GetStreamFormat() == MediaFormat::RGBA /* FORMAT_RGBA */ ) {
#if defined(_IRIX) || defined(MESA)
                // SGI and Mesa conveniently allows us to send a NULL pointer
                
                ::glTexImage2D(GL_TEXTURE_2D, 0,
                  4, width, height,
                  0, GL_RGBA, GL_UNSIGNED_BYTE, NULL);
#else
                {
                  // this is incredibly stupid, but currently has to be this
                  // way to set up texturing correctly
                  
                  unsigned char* buffer = new unsigned char[width * height * 4];
                  
                  ::glTexImage2D(GL_TEXTURE_2D, 0,
                    4, width, height,
                    0, GL_RGBA, GL_UNSIGNED_BYTE, buffer);
                  delete buffer;
                }
#endif
              } else {
#if defined(_IRIX) || defined(MESA)
                // SGI and Mesa conveniently allows us to send a NULL pointer
                
                ::glTexImage2D(GL_TEXTURE_2D, 0,
                  3, width, height,
                  0, GL_RGB, GL_UNSIGNED_BYTE, NULL);
#else
                {
                  // this is incredibly stupid, but currently has to be this
                  // way to set up texturing correctly
                  
                  unsigned char* buffer = new unsigned char[width * height * 3];
                  
                  ::glTexImage2D(GL_TEXTURE_2D, 0,
                    3, width, height,
                    0, GL_RGB, GL_UNSIGNED_BYTE, buffer);
                  delete buffer;
                }
#endif
              }
              
              m_nTextureWidth = width;
              m_nTextureHeight = height;
          }
          
          if (pTextureNode2D->GetStreamFormat() == MediaFormat::RGBA /* FORMAT_RGBA */ )
            if (m_nTextureHeight != m_nHeight ||
              m_nTextureWidth != m_nWidth) {
#if !defined(GL_VERSION_1_1)
              ::glTexSubImage2DEXT(GL_TEXTURE_2D, 0,
                m_nTextureWidth - m_nWidth, 0, m_nWidth, m_nHeight,
                GL_RGBA, GL_UNSIGNED_BYTE, pData);
#else
              ::glTexSubImage2D(GL_TEXTURE_2D, 0,
                m_nTextureWidth - m_nWidth, 0, m_nWidth, m_nHeight,
                GL_RGBA, GL_UNSIGNED_BYTE, pData);
#endif
            } else
              ::glTexImage2D(GL_TEXTURE_2D, 0,
              4, m_nTextureWidth, m_nTextureHeight,
              0, GL_RGBA, GL_UNSIGNED_BYTE, pData);
            else
              if (m_nTextureHeight != m_nHeight ||
                m_nTextureWidth != m_nWidth) {
#if !defined(GL_VERSION_1_1)
                ::glTexSubImage2DEXT(GL_TEXTURE_2D, 0,
                  m_nTextureWidth - m_nWidth, 0, m_nWidth, m_nHeight,
                  GL_RGB, GL_UNSIGNED_BYTE, pData);
#else
                ::glTexSubImage2D(GL_TEXTURE_2D, 0,
                  m_nTextureWidth - m_nWidth, 0, m_nWidth, m_nHeight,
                  GL_RGB, GL_UNSIGNED_BYTE, pData);
#endif
              } else
                ::glTexImage2D(GL_TEXTURE_2D, 0,
                3, m_nTextureWidth, m_nTextureHeight,
                0, GL_RGB, GL_UNSIGNED_BYTE, pData);
    }
    
    pStream->Release(FALSE);
  }
  
  ::glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, pRenderer->GetTextureMinFunc());
  ::glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, pRenderer->GetTextureMinFunc());
      } else
        pStream->Terminate();
      
      if (m_nTextureHeight != m_nHeight ||
        m_nTextureWidth != m_nWidth) {
        ::glMatrixMode(GL_TEXTURE);
        ::glPushMatrix();
        ::glTranslatef(1.0f - float(m_nWidth) / float(m_nTextureWidth), 0.0f, 0.0f);
        ::glScalef(float(m_nWidth) / float(m_nTextureWidth),
          float(m_nHeight) / float(m_nTextureHeight), 1.0f);
      }
      
      {
        static GLfloat texCoord[][2] = {{ 1.0f, 1.0f},
        { 0.0f, 1.0f},
        { 0.0f, 0.0f},
        { 1.0f, 0.0f}};
        
        ::glBegin(GL_QUADS);
        glNormal3f(0.0f, 0.0f, -1.0f);
#if 0
        glTexCoord2fv(texCoord[3]); glVertex2i( m_nWidth / 2,  m_nHeight / 2);
        glTexCoord2fv(texCoord[2]); glVertex2i(-m_nWidth / 2,  m_nHeight / 2);
        glTexCoord2fv(texCoord[1]); glVertex2i(-m_nWidth / 2, -m_nHeight / 2);
        glTexCoord2fv(texCoord[0]); glVertex2i( m_nWidth / 2, -m_nHeight / 2);
#endif
        glTexCoord2fv(texCoord[3]); glVertex2i( m_nWidth,          0);
        glTexCoord2fv(texCoord[2]); glVertex2i(        0,          0);
        glTexCoord2fv(texCoord[1]); glVertex2i(        0, -m_nHeight);
        glTexCoord2fv(texCoord[0]); glVertex2i( m_nWidth, -m_nHeight);
        ::glEnd();
      }
      
      if (m_nTextureHeight != m_nHeight ||
        m_nTextureWidth != m_nWidth) {
        ::glPopMatrix();
        ::glMatrixMode(GL_MODELVIEW);
      }
      
#if !defined(GL_VERSION_1_1)
      ::glBindTextureEXT(GL_TEXTURE_2D, 0);
#else
      ::glBindTexture(GL_TEXTURE_2D, 0);
#endif
      
      ::glDisable(GL_TEXTURE_2D);
    }
  }
  
#ifdef _DEBUG
  if (pRenderer->CheckGLError("TextureNode2DProxy:")) {
    TRACE ("Texture width: %d, %d\n", m_nTextureWidth, m_nTextureHeight);
    TRACE (" width: %d, %d\n", m_nWidth, m_nHeight);
    TRACE ("TextureNode2DProxy::Render() [%08x]: ", this);
  }
#endif
  
  return Node2DProxy::Render(rTransformation, rCullFlag,
    pRenderer, dwTime);
}

// traverse scene graph, checking for intersection

/* virtual */ void
TextureNode2DProxy::Select(
    OpenGLVisualRenderer* /* pRenderer */, 
    DWORD /* dwTime */,
    MediaObjectProxy *& intersectingObject, 
    unsigned int& depth)
{
  GLuint selectBuffer[4];
  
  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);
  
  ::glBegin(GL_QUADS);
#if 0
  glVertex2i( m_nWidth / 2,  m_nHeight / 2);
  glVertex2i(-m_nWidth / 2,  m_nHeight / 2);
  glVertex2i(-m_nWidth / 2, -m_nHeight / 2);
  glVertex2i( m_nWidth / 2, -m_nHeight / 2);
#endif
  glVertex2i( m_nWidth,          0);
  glVertex2i(        0,          0);
  glVertex2i(        0, -m_nHeight);
  glVertex2i( m_nWidth, -m_nHeight);
  ::glEnd();
  
  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];
      
      intersectingObject = this;
    }
}

// Traverse scene graph, checking for activation of sensors

/* virtual */ void
TextureNode2DProxy::Select(
    const Matrix& /* rTransformation */, 
    const CullFlag& /* rCullFlag */,
    OpenGLVisualRenderer* /* pRenderer */, 
    DWORD /* dwTime */,
    MediaObjectProxy *& intersectingObject,
    ZArray<MediaObjectProxy *>& activatedSensors, 
    unsigned int& depth,
    const ZArray<MediaObjectProxy *>& localSensors)
{
  GLuint selectBuffer[4];
  
  ::glSelectBuffer(4, selectBuffer);
  ::glRenderMode(GL_SELECT);
  
  ::glBegin(GL_QUADS);
#if 0
  glVertex2i( m_nWidth / 2,  m_nHeight / 2);
  glVertex2i(-m_nWidth / 2,  m_nHeight / 2);
  glVertex2i(-m_nWidth / 2, -m_nHeight / 2);
  glVertex2i( m_nWidth / 2, -m_nHeight / 2);
#endif
  glVertex2i( m_nWidth,          0);
  glVertex2i(        0,          0);
  glVertex2i(        0, -m_nHeight);
  glVertex2i( m_nWidth, -m_nHeight);
  ::glEnd();
  
  if (::glRenderMode(GL_RENDER) > 0)
    if (depth >= selectBuffer[1]) {
      depth = selectBuffer[1];
      
      intersectingObject = this;
      activatedSensors = localSensors;
    }
}

