/********************************************************************************/
/* This software module was developped by										*/

/*	   Marius Preda (MP), marius.preda@int-evry.fr,								*/
/*                        Institut National des Telecommunications, France		*/

/* in the course of development of the MPEG-4 standard (Part 16 AFX).			*/
/* This software module is an implementation of a part of one or				*/
/* more MPEG-4 tools as specified by the										*/
/* MPEG-4 standard. ISO/IEC gives users of the MPEG-4 standard					*/
/* free license to this software module or modifications thereof				*/
/* for use in hardware or software products claiming conformance				*/
/* to the MPEG-4 standard. Those intending to use this software					*/
/* module in hardware or software products are advised that its					*/
/* use may infringe existing patents. The original developer of					*/
/* this software module and his company, the subsequent							*/
/* editors and their companies, and ISO/IEC have no liability					*/
/* for use of this software module or modifications thereof in					*/
/* an implementation. Copyright is not released for non							*/
/* MPEG-4 standard conforming products. INT ARTEMIS								*/
/* retain full right to use the code for their own purposes,					*/
/* assign or donate the code to a third party and to inhibit					*/
/* third parties from using the code for non MPEG-4 standard					*/
/* conforming products. This copyright notice must be included					*/
/* in all copies or derivative works.    										*/
/* Copyright (c)2002 - 2005														*/
/********************************************************************************/

  /// BBAEncoder.cpp: implementation of the BBAEncoder class.
//
//////////////////////////////////////////////////////////////////////

#include "stdafx.h"
#include "BBATypeDefs.h"
#include "BBAEncoder.h"
#include "BBAFileUtil.h"
#include <math.h>


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[]=__FILE__;
#define new DEBUG_NEW
#endif

//////////////////////////////////////////////////////////////////////
// Construction/Destruction
//////////////////////////////////////////////////////////////////////

BBAEncoder::BBAEncoder()
{
	bbaop.bba_quant=1;
	low=0;
	high=top;
	zerocount=0;
	opposite_bits=0;
	length=0;
	// init stream data
	bbaop.stream.p = 0;
	bbaop.stream.incnt = 0;
	bbaop.stream.bitcnt = 0;
	bbaop.stream.outbfr=0;
	bbaop.stream.outcnt=8;
	
	bbaop.stream.buf = (unsigned char*)malloc(1000*MAX_BITS_PER_FRAME); // 1000 frames is maybe enough 
    
	bbaop.marker_bit=MARKER_BIT;
	//fop.stream.bufsize = i;
}

BBAEncoder::~BBAEncoder()
{
// TO ADD
}

bool BBAEncoder::encode_muscle_mask(CMuscle *ms){
	
	CMuscleMask *mm=&ms->muscleMask;
	int i;
	bool tochange=false;

	tochange=ms->isControlPointsChanged();
	PutBits(1,tochange);
	PutBits(1,bbaop.marker_bit);
	if (tochange){
		for (i=0;i<ms->NumberControlPoints;i++){
			PutBits(1,bbaop.marker_bit);
			PutBits(1,mm->isControlPointOnXChanged[i]);
			PutBits(1,mm->isControlPointOnYChanged[i]);
			PutBits(1,mm->isControlPointOnZChanged[i]);
		}
	}
	tochange=ms->isWheightOfControlPointsChanged();
	PutBits(1,tochange);
	PutBits(1,bbaop.marker_bit);
	if (tochange){
		for (i=0;i<ms->NumberControlPoints;i++){
			PutBits(1,bbaop.marker_bit);
			PutBits(1,mm->isWheighOfControlPointChanged[i]);
		}
	}
	tochange=ms->isKnotChanged();
	PutBits(1,tochange);
	PutBits(1,bbaop.marker_bit);
	if (tochange){
		for (i=0;i<ms->NumberKnots;i++){
			PutBits(1,bbaop.marker_bit);
			PutBits(1,mm->isKnotChanged[i]);
		}
	}
	return true;
}

bool BBAEncoder::encode_bone_mask(CBone *bone){

	CBoneMask *bm = &bone->boneMask;

	//////////// translation ///////////
	if (bm->isTonXChanged || bm->isTonYChanged|| bm->isTonZChanged){
		PutBits(1,1);
		PutBits(1,bbaop.marker_bit);
		if (bm->isTonXChanged) PutBits(1,1);
		else PutBits(1,0);
		if (bm->isTonYChanged) PutBits(1,1);
		else PutBits(1,0);
		if (bm->isTonZChanged) PutBits(1,1);
		else PutBits(1,0);
	}
	else {
		PutBits(1,0);
		PutBits(1,bbaop.marker_bit);
	}

	//////////// rotation ///////////
	if (!bm->isQuaternion){
		if ((bm->isRonAx1Changed || bm->isRonAx2Changed || bm->isRonAx3Changed)){
			PutBits(1,1);
			PutBits(1,bbaop.marker_bit);
			PutBits(1,0);//EA
			if (bm->isRonAx1Changed) PutBits(1,1);
			else PutBits(1,0);
			if (bm->isRonAx2Changed) PutBits(1,1);
			else PutBits(1,0);
			if (bm->isRonAx3Changed) PutBits(1,1);
			else PutBits(1,0);
		}
		else {
			PutBits(1,0);
			PutBits(1,bbaop.marker_bit);
		}
	}else{
		if ((bm->isRonQxChanged)||(bm->isRonQyChanged)||(bm->isRonQzChanged)||(bm->isRonQwChanged)){
			PutBits(1,1);
			PutBits(1,bbaop.marker_bit);
			PutBits(1,1);//Q
			if (bm->isRonQxChanged) PutBits(1,1);
			else PutBits(1,0);
			if (bm->isRonQyChanged) PutBits(1,1);
			else PutBits(1,0);
			if (bm->isRonQzChanged) PutBits(1,1);
			else PutBits(1,0);
			if (bm->isRonQwChanged) PutBits(1,1);
			else PutBits(1,0);
		}
		else {
			PutBits(1,0);
			PutBits(1,bbaop.marker_bit);
		}
	}
	
	//////////// scale ///////////
	if (bm->isSonXChanged || bm->isSonYChanged || bm->isSonZChanged){
		PutBits(1,1);
		PutBits(1,bbaop.marker_bit);
		if (bm->isSonXChanged) PutBits(1,1);
		else PutBits(1,0);
		if (bm->isSonYChanged) PutBits(1,1);
		else PutBits(1,0);
		if (bm->isSonZChanged) PutBits(1,1);
		else PutBits(1,0);
	}
	else {
		PutBits(1,0);
		PutBits(1,bbaop.marker_bit);
	}

	//////////// scale Orientation///////////
	if (bm->isSOonAxXChanged || bm->isSOonAxYChanged || bm->isSOonAxZChanged || bm->isSOonValChanged ){
		PutBits(1,1);
		PutBits(1,bbaop.marker_bit);
		if (bm->isSOonAxXChanged) PutBits(1,1);
		else PutBits(1,0);
		if (bm->isSOonAxYChanged) PutBits(1,1);
		else PutBits(1,0);
		if (bm->isSOonAxZChanged) PutBits(1,1);
		else PutBits(1,0);
		if (bm->isSOonValChanged) PutBits(1,1);
		else PutBits(1,0);
	}
	else {
		PutBits(1,0);
		PutBits(1,bbaop.marker_bit);
	}
	//////////// center ///////////
	if (bm->isConXChanged || bm->isConYChanged || bm->isConZChanged){
		PutBits(1,1);
		PutBits(1,bbaop.marker_bit);
		if (bm->isConXChanged) PutBits(1,1);
		else PutBits(1,0);
		if (bm->isConYChanged) PutBits(1,1);
		else PutBits(1,0);
		if (bm->isConZChanged) PutBits(1,1);
		else PutBits(1,0);
	}
	else {
		PutBits(1,0);
		PutBits(1,bbaop.marker_bit);
	}
	return true;
}

bool BBAEncoder::encode_bba_object_plane_mask(){

	CBone *cb=new CBone();
	cb=bbaop.firstBone;
	CMuscle *cm=new CMuscle();
	cm=bbaop.firstMuscle;
	CMorph *cmf=new CMorph();
	cmf=bbaop.firstMorph;

	int i;		
	PutBits(5,bbaop.KeyFrameIndex);
	if(bbaop.is_intra) {
		PutBits(5,bbaop.bba_quant);
		PutBits(3,bbaop.pow2quant);
		PutBits(3,bbaop.noOfControllerTypes);
		PutBits(10,bbaop.NumberOfBones);
		if (bbaop.noOfControllerTypes>1) PutBits(10,bbaop.NumberOfMuscles);
		if (bbaop.noOfControllerTypes>2) PutBits(10,bbaop.NumberOfMorphs);

		while (cb){
			if (cb->isInCurrentFrame){
				PutBits(10,cb->boneIdentifier);
				encode_bone_mask(cb);
			}
			cb=cb->suiv;
		}
		if (bbaop.noOfControllerTypes>1){
			while (cm){
				if (cm->isInCurrentFrame){
					PutBits(10,cm->muscleIdentifier);
					PutBits(6,cm->NumberControlPoints);
					PutBits(6,cm->NumberKnots);
					encode_muscle_mask(cm);
				}
				cm=cm->suiv;
			}
		}
		if (bbaop.noOfControllerTypes>2){
			while (cmf){
				if (cmf->isInCurrentFrame){
					PutBits(10,cmf->morphIdentifier);
					PutBits(6,cmf->NumberWeights);
					encode_morph_mask(cmf);
				}
				cmf=cmf->suiv;
			}
		}
	}
	return true;
}

bool BBAEncoder::encode_bba_object_plane_data(){
	encode_bba_object_plane_mask();
	encode_bba_object_plane_values();
	return true;
}

bool BBAEncoder::encode_bba_object_plane_values(){
	if(bbaop.is_intra) {
		PutBits(1,bbaop.bba_object_coding_type);
		if (bbaop.bba_object_coding_type==0){// frame based encoding
		    PutBits(1,bbaop.is_i_new_max);				
			PutBits(1,bbaop.is_i_new_min);
			PutBits(1,bbaop.is_p_new_max);	
			PutBits(1,bbaop.is_p_new_min);
			encode_bba_new_minmax();
			bba_init_arith(); // before I frame the arithmetic statistics are reset
			encode_bba_i_frame();
			encoder_flush();
		}
		else
			encode_bba_i_segment();
	}
	else{
		if (bbaop.bba_object_coding_type==0) {// frame based encoding
			encode_bba_p_frame();
			encoder_flush();
		}
		else
			encode_bba_p_segment();
	}
	return true;
}



bool  BBAEncoder::encode_bba_object_plane_header(){

	NextStartCode();
	PutBBAPlaneStartCode();

  PutBits(1,bbaop.is_intra);
  if(bbaop.is_intra) {
	if (bbaop.frame_rate) bbaop.is_frame_rate=1;
	else bbaop.is_frame_rate=0;
	PutBits(1,bbaop.is_frame_rate);
	if (bbaop.is_frame_rate) 
		bba_encode_frame_rate();
	PutBits(1,bbaop.is_time_code);
	if (bbaop.is_time_code) 
		bba_encode_time_code();
  }
  PutBits(1,bbaop.skip_frames);
  if (bbaop.skip_frames) 
	bba_encode_skip_frames();
  return 0;
}

void BBAEncoder::bba_encode_frame_rate(){
  
  if(bbaop.frame_rate>=0 && bbaop.frame_rate<256)
      PutBits(8,bbaop.frame_rate);
  else {
    AfxMessageBox(" Error: Frame_rate range from 0 to 255 \n");
    exit(-1);
  }

  if(bbaop.seconds>=0 && bbaop.seconds<16)
      PutBits(4,bbaop.seconds);
  else {
    AfxMessageBox("Error: Seconds(framerate) range from 0 to 15 \n");
    exit(-1);
  }
  PutBits(1,bbaop.frequency_offset);
}

void BBAEncoder::bba_encode_time_code(){

  if(bbaop.time_code_hours>=0 && bbaop.time_code_hours<24)
      PutBits(5,bbaop.time_code_hours);
  else {
    AfxMessageBox("Error: Hours(timecode) range from 0 to 23 \n");
    exit(-1);
  }

  
  if(bbaop.time_code_minutes>=0 && bbaop.time_code_minutes<60)
      PutBits(6,bbaop.time_code_minutes);
  else {
    AfxMessageBox("\n Error: Minutes(timecode) range from 0 to 59 \n");
    exit(-1);
  }
  
  PutBits(1,bbaop.marker_bit);
  
  if(bbaop.time_code_seconds>=0 && bbaop.time_code_seconds<60)
      PutBits(6,bbaop.time_code_seconds);
  else {
    AfxMessageBox("Error: Seconds(timecode) range from 0 to 59 \n");
    exit(-1);
  }
}



void BBAEncoder::bba_encode_skip_frames(){

  int nfs;
  
  if(bbaop.number_of_frames_to_skip>254) {
    AfxMessageBox("Maximum allowable frames skip is 254.\n");
    exit(-1);
  }
  if(bbaop.number_of_frames_to_skip > 14) {
    PutBits(4,15);
    nfs = bbaop.number_of_frames_to_skip - 15;
  }
  else {
    nfs = bbaop.number_of_frames_to_skip;
    PutBits(4,nfs);
    return;
  }

  if(nfs<15) {
      PutBits(4,nfs);
      return;
  }
  
  do {
    PutBits(4,15);
    nfs-=15;
  }
  while (nfs > 14);
  
  nfs+=16;
  PutBits(4,nfs);

}


//////////////// bits operations /////////////////////////////

// unsigned char byte;// outbfr
// bitcount -- bitcnt
// framebits -- outframebits
// bitsout -- outcnt

void BBAEncoder::PutBits (int nobits, int value)
{
	
	int counter,i;
	unsigned int mask;
	int bit;
	
	bbaop.stream.bitcnt+=nobits;
	bbaop.stream.outframebits+=nobits;
	mask=1;
	for (i=0;i<(nobits-1);i++)
	  mask=mask*2;

	
	for (counter=0; counter<nobits; counter++) {
		
	  bbaop.stream.outcnt--;
	  bbaop.stream.outbfr <<= 1;
	  
	  bit = mask&value;
	  bit>>=(nobits-1-counter);
	  
	  bbaop.stream.outbfr|=bit;
	  
	  mask >>= 1; 
	  
	  if (bbaop.stream.outcnt==0)  {
		  bbaop.stream.buf[bbaop.stream.p++]=bbaop.stream.outbfr;
	    bbaop.stream.outcnt = 8;
	  }
	  
	}
}


void BBAEncoder::ShortPutBits (short symbol)
{
	int counter;
	unsigned int mask,value;
	int bit;
	
	bbaop.stream.bitcnt+=16;
	bbaop.stream.outframebits+=16;	
	
	value = (unsigned int)symbol;
	
	mask = (int)pow(2,15); 
	
	for (counter=0; counter<16; counter++) {		
		bbaop.stream.outcnt--; 
		bbaop.stream.outbfr <<= 1;		
		bit = mask&value;
		bit>>=(15-counter);		
		bbaop.stream.outbfr |=bit;
		mask >>= 1; 		
		if (bbaop.stream.outcnt==0)  {
			bbaop.stream.buf[bbaop.stream.p++]=bbaop.stream.outbfr;
			bbaop.stream.outcnt=8;
		}
		
	}
}

void  BBAEncoder::NextStartCode()
{

  if(bbaop.stream.outcnt!=8){
    PutBits(1,0);
    while(bbaop.stream.outcnt!=8)
		PutBits(1,1);
  }
  else 
      PutBits(8,127);
}



void BBAEncoder::bba_init_arith() {
 
  int i, j, n;
	// TO ADD a procedure to free the memory
  bbaop.Ifreq = (int **)malloc(bbaop.NumberOfSBComponentsInPlane * sizeof(int*));
  bbaop.Icumul_freq = (int **)malloc(bbaop.NumberOfSBComponentsInPlane * sizeof(int*));
  bbaop.freq = (int **)malloc(bbaop.NumberOfSBComponentsInPlane * sizeof(int*));
  bbaop.cumul_freq = (int **)malloc(bbaop.NumberOfSBComponentsInPlane * sizeof(int*));
  
  for(i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
	if(bbaop.sbc_mask[i]) {
		// intra data
        n = bbaop.Irange_max[i]-bbaop.Irange_min[i]+1;
        bbaop.Ifreq[i] = (int *) malloc(n*sizeof(int));
        bbaop.Icumul_freq[i] = (int *) malloc((n+1)*sizeof(int));
        for(j=0; j<n; j++) {
			bbaop.Ifreq[i][j] = 1;
            bbaop.Icumul_freq[i][j] = n-j;
		}
        bbaop.Icumul_freq[i][n] = 0;
		// predictive data

        n = bbaop.range_max[i]-bbaop.range_min[i]+1;
        bbaop.freq[i] = (int *) malloc(n*sizeof(int));
        bbaop.cumul_freq[i] = (int *) malloc((n+1)*sizeof(int));
        for(j=0; j<n; j++) {
          bbaop.freq[i][j] = 1;
          bbaop.cumul_freq[i][j] = n-j;
		}
        bbaop.cumul_freq[i][n] = 0;
	}
  }
}
		

void BBAEncoder::encode_bba_new_minmax() {
  int i, j;
  int minmax;
  int scaler;


  if (bbaop.bba_quant) scaler = bbaop.bba_quant;
  else scaler = 1;

  if(bbaop.is_i_new_max) {
    for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
		PutBits(1,bbaop.marker_bit);
		if(bbaop.sbc_mask[i]) {
			minmax=bbaop.Irange_max[i]*scaler/RANGE_DELTA;
			PutBits(5,minmax);
		}
	}
  }
  
  if(bbaop.is_i_new_min) {
    for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
		PutBits(1,bbaop.marker_bit);
		if(bbaop.sbc_mask[i]) {
			minmax=-bbaop.Irange_min[i]*scaler/RANGE_DELTA;
			PutBits(5,minmax);
		}
	}
  }
  
  if(bbaop.is_p_new_max) {
    for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
		PutBits(1,bbaop.marker_bit);
		if(bbaop.sbc_mask[i]) {
			minmax=bbaop.range_max[i]*scaler/RANGE_DELTA;
			PutBits(5,minmax);
		}
	}
  }
  if(bbaop.is_p_new_min) {
    for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
		PutBits(1,bbaop.marker_bit);
		if(bbaop.sbc_mask[i]) {
			minmax=-bbaop.range_min[i]*scaler/RANGE_DELTA;
			PutBits(5,minmax);
		}
	}
  }
}



void BBAEncoder::encode_bba_i_frame() {

	int i, j, k;
	int symbol, qstep;
	int Q;


	qstep = (int)pow(2,bbaop.pow2quant) * bba_pred_scale[bbaop.bba_quant]; // TO 

  for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
	if(bbaop.sbc_mask[i]) {
		Q=quantize(bbaop.sbc_values[i], qstep);
		symbol = Q - bbaop.Irange_min[i];
		encode_a_symbol(symbol,bbaop.Icumul_freq[i]);
		update_arith_encoder(symbol,i);
		bbaop.sbc_values_prev[i] = dequant(Q, qstep);
	}
  }
}

void BBAEncoder::encode_bba_p_frame() {

	int i, j, k;
	int symbol, qstep;
	int diff;
	int Q;


	qstep = (int)pow(2,bbaop.pow2quant) * bba_pred_scale[bbaop.bba_quant]; // do we need different quantization between the components ???
  for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
		if(bbaop.sbc_mask[i]) {
			Q=quantize(bbaop.sbc_values[i]-bbaop.sbc_values_prev[i], qstep);
			symbol = Q - bbaop.range_min[i];
			encode_a_symbol(symbol,bbaop.cumul_freq[i]);
			update_arith_encoder(symbol,i);			
			diff = dequant(Q, qstep);
			//bbaop.sbc_values_prev[i]+=diff;
		}
  }
}

void BBAEncoder::update_arith_encoder(int symbol, int index){

  int n;
  int *cumul_tmp, *freq_tmp;

  if(bbaop.is_intra) {
    n = bbaop.Irange_max[index]-bbaop.Irange_min[index]+1;
    cumul_tmp = bbaop.Icumul_freq[index];
    freq_tmp = bbaop.Ifreq[index];
  } 
  else {
    n = bbaop.range_max[index]-bbaop.range_min[index]+1;
    cumul_tmp = bbaop.cumul_freq[index];
    freq_tmp = bbaop.freq[index];
  }

  if (cumul_tmp[0] == Max_frequency) {
    int i, cum;
    cum = 0;
    for(i=n-1; i>=0; i--) {
      freq_tmp[i] = (freq_tmp[i]+1)/2;
      cum += freq_tmp[i];
      cumul_tmp[i] = cum;
    }
    cumul_tmp[n] = 0;
  }

  freq_tmp[symbol] += 1;
  cumul_tmp[symbol] += 1;
  while(symbol>0) {
    symbol -= 1;
    cumul_tmp[symbol] += 1;
  }
}


void BBAEncoder::encode_a_symbol(int index, int cumul_freq[ ]){
  

  if (index<0) {
    AfxMessageBox("Error AR coding, symbol is negative");
    exit(-1);
  }
  length = high - low + 1;
  high = low - 1 + (length * cumul_freq[index]) / cumul_freq[0];
  low += (length * cumul_freq[index+1]) / cumul_freq[0];
  
  for ( ; ; ) {
    if (high < q2) {
      bit_opp_bits(0);
    }
    else if (low >= q2) {
      bit_opp_bits(1);	
      low -= q2; 
      high -= q2;
    }
    else if (low >= q1 && high < q3) {
      opposite_bits += 1; 
      low -= q1; 
      high -= q1;
    }
    else break;
  
    low *= 2; 
    high = 2*high+1;
  }
  
}

int BBAEncoder::quantize(int bap, int quantStep) 
{	
	if(!quantStep)
		return bap;
	else  if( abs(bap)<=quantStep/2 )
		return 0;
	else 
		return (bap + sign(bap)*quantStep/2 ) / quantStep;

}

int BBAEncoder::dequant(int qfap, int qstep) {
  if(qstep) {
        return qfap*qstep;
  } else {
        return qfap;
  }      
}


void BBAEncoder::bit_opp_bits(int bit) 
{                                   
  bit_in_psc_layer(bit);

  while(opposite_bits > 0){
    bit_in_psc_layer(!bit);
    opposite_bits--;
  }

}

void BBAEncoder::bit_in_psc_layer(int bit)
{
  if(zerocount==ZEROMAX) {
    PutBits(1,bbaop.marker_bit);
    zerocount=0;
  }
  
  PutBits(1, bit);
  if(!bit) zerocount++;
  else zerocount = 0;
}


void BBAEncoder::encoder_flush()
{
  opposite_bits++;

  if (low < q1) {
    bit_opp_bits(0);
  }
  else {
    bit_opp_bits(1);
  }
  low = 0; 
  high = top;
  zerocount = 0;
}



bool BBAEncoder::ReadAMuscleMask(unsigned char *buff, CMuscle *ms, int *ct){
	
	int count=*ct;
	CMuscleMask *mml=&(ms->muscleMask);
	int i;

	////////// NR Control points /////////////
	ms->NumberControlPoints=mygetint(buff, &count);

	////////// NR Knots /////////////
	ms->NumberKnots=mygetint(buff, &count);

	ms->AllocMuscle();
//	ms->AllocMinMaxMuscle();
	ms->AllocPrevMuscle();

	if (mygetint(buff, &count)){
		for (i=0;i<ms->NumberControlPoints;i++) {
			mml->isControlPointOnXChanged[i]=(bool)mygetint(buff, &count);
			mml->isControlPointOnYChanged[i]=(bool)mygetint(buff, &count);
			mml->isControlPointOnZChanged[i]=(bool)mygetint(buff, &count);
		}
	}else{
		for (i=0;i<ms->NumberControlPoints;i++) {
			mml->isControlPointOnXChanged[i]=false;
			mml->isControlPointOnYChanged[i]=false;
			mml->isControlPointOnZChanged[i]=false;
		}
	}

	if (mygetint(buff, &count)){
		for (i=0;i<ms->NumberControlPoints;i++) 
			mml->isWheighOfControlPointChanged[i]=(bool)mygetint(buff, &count);
	}else{
		for (i=0;i<ms->NumberControlPoints;i++) 
			mml->isWheighOfControlPointChanged[i]=false;
	}
			
	if (mygetint(buff, &count)){
		for (i=0;i<ms->NumberKnots;i++) 
			mml->isKnotChanged[i]=(bool)mygetint(buff, &count);
	}else{
		for (i=0;i<ms->NumberControlPoints;i++) 
			mml->isKnotChanged[i]=false;
	}
	*ct=count;
	return true;
}
bool BBAEncoder::ReadAMuscleData(unsigned char *buff, CMuscle *ms, int *ct){
	
	int count=*ct;
	int i;
	CMuscleMask *mml=&(ms->muscleMask);
	CMuscleData *mdl=&(ms->muscleData);

	////////// translation /////////////

	if (ms->isControlPointsChanged()){
		for (i=0;i<ms->NumberControlPoints;i++){
			if (mml->isControlPointOnXChanged[i])
				mdl->ControlPointOnX[i]=mygetint(buff, &count);
			if (mml->isControlPointOnYChanged[i])
				mdl->ControlPointOnY[i]=mygetint(buff, &count);
			if (mml->isControlPointOnZChanged[i])
				mdl->ControlPointOnZ[i]=mygetint(buff, &count);
		}
	}
	if (ms->isWheightOfControlPointsChanged()){
		for (i=0;i<ms->NumberControlPoints;i++){
			if (mml->isWheighOfControlPointChanged[i])
				mdl->WheighOfControlPoint[i]=mygetint(buff, &count);
		}
	}
	if (ms->isKnotChanged()){
		for (i=0;i<ms->NumberKnots;i++){
			if (mml->isKnotChanged[i])
				mdl->Knot[i]=mygetint(buff, &count);
		}
	}

	*ct=count;
	return true;

}

bool BBAEncoder::FreeTheFrame(){

	return true;
}

void BBAEncoder::SetMinMax(){
	
	CBone *lb=bbaop.firstBone;
	while(lb){
		if (!lb->boneMask.isQuaternion){
			lb->boneDataIMax.init(1860, 1860, 1860, 1860, 1860, 1860, 
				1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860);
			lb->boneDataIMin.init(-1860, -1860, -1860, -1860, -1860, -1860, 
				0, 0, 0, 0, 0, 0, -1860, -1860, -1860, -1860);
			lb->boneDataPMax.init(600, 600, 600, 600, 600, 600, 
				600, 600, 600, 600, 600, 600, 600, 600, 600, 600);
			lb->boneDataPMin.init(-600, -600, -600, -600, -600, -600, 
				0, 0, 0, 0, 0, 0, -600, -600, -600, -600);
		}else{
			lb->boneDataIMax.initQ(1860, 1860, 1860, 1860, 1860, 1860, 1860, 
				1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860, 1860);
			lb->boneDataIMin.initQ(-1860, -1860, -1860, -1860, -1860, -1860, -1860, 
				0, 0, 0, 0, 0, 0, -1860, -1860, -1860, -1860);
			lb->boneDataPMax.initQ(600, 600, 600, 600, 600, 600, 600, 
				600, 600, 600, 600, 600, 600, 600, 600, 600, 600);
			lb->boneDataPMin.initQ(-600, -600, -600, -600, -600, -600, -600, 
				0, 0, 0, 0, 0, 0, -600, -600, -600, -600);
		}
		lb=lb->suiv;
	}
	if (bbaop.noOfControllerTypes>1){
		CMuscle *lm=bbaop.firstMuscle;
		while (lm){
			lm->AllocMinMaxMuscle();
			lm=lm->suiv;
		}
	}
	if (bbaop.noOfControllerTypes>2){
		CMorph *lmf=bbaop.firstMorph;
		while (lmf){
			lmf->AllocMinMaxMorph();
			lmf=lmf->suiv;
		}
	}
}


bool BBAEncoder::ReadAFrame(unsigned char *buff)
{
	// read a frame from a non-encoded buffer
	// the buffer has the same syntax as in the non-compressed file

	int i, count=0;
	bool ismask;
	
	bbaop.KeyFrameIndex  =mygetint(buff, &count);
	bbaop.noOfControllerTypes=mygetint(buff, &count);
	bbaop.NumberOfBones  =mygetint(buff, &count);
	if (bbaop.noOfControllerTypes>1) bbaop.NumberOfMuscles=mygetint(buff, &count);
	else bbaop.NumberOfMuscles=0;
	if (bbaop.noOfControllerTypes>2) bbaop.NumberOfMorphs=mygetint(buff, &count);
	else bbaop.NumberOfMorphs=0;

	int *id = new int [bbaop.NumberOfBones];
	int *idm = new int [bbaop.NumberOfMuscles];
	int *idmf = new int [bbaop.NumberOfMorphs];

	CBone *lb;
	CMuscle *lm;
	CMorph *lmf;
	
	for(i=0;i<bbaop.NumberOfBones;i++){
		lb=new CBone();
		lb->boneIdentifier=mygetint(buff, &count);
		id[i]=lb->boneIdentifier;
		lb->boneMask.init();
		ReadABoneMask(buff,lb, &count);
		lb->isInCurrentFrame=true;
		innod(lb,bbaop.firstBone,bbaop.lastBone);
		lb=NULL;
	}
	if(bbaop.noOfControllerTypes>1){
		for(i=0;i<bbaop.NumberOfMuscles;i++){
			lm=new CMuscle();
			lm->muscleIdentifier=mygetint(buff, &count);
			idm[i]=lm->muscleIdentifier;
			ReadAMuscleMask(buff,lm, &count);
			lm->isInCurrentFrame=true;
			innod(lm,bbaop.firstMuscle,bbaop.lastMuscle);
			lm=NULL;
		}
	}
	if(bbaop.noOfControllerTypes>2){
		for(i=0;i<bbaop.NumberOfMorphs;i++){
			lmf=new CMorph();
			lmf->morphIdentifier=mygetint(buff, &count);
			idmf[i]=lmf->morphIdentifier;
			ReadAMorphMask(buff,lmf, &count);
			lmf->isInCurrentFrame=true;
			innod(lmf,bbaop.firstMorph,bbaop.lastMorph);
			lmf=NULL;
		}
	}

	for(i=0;i<bbaop.NumberOfBones;i++){
		lb = getnodIDi(id[i],bbaop.firstBone,bbaop.lastBone);
		lb->boneData.init();
		ReadABoneData(buff,lb, &count);
	}
	delete id; 
	if (bbaop.noOfControllerTypes>1){
		for(i=0;i<bbaop.NumberOfMuscles;i++){
			lm = getnodIDi(idm[i],bbaop.firstMuscle,bbaop.lastMuscle);
			ReadAMuscleData(buff,lm, &count);
		}
		delete idm; 
	}
	if (bbaop.noOfControllerTypes>2){
		for(i=0;i<bbaop.NumberOfMorphs;i++){
			lmf = getnodIDi(idmf[i],bbaop.firstMorph,bbaop.lastMorph);
			ReadAMorphData(buff,lmf, &count);
		}
	if (bbaop.NumberOfMorphs) delete idmf; 
	}

	return count;

}

int BBAEncoder::AllocThePlane(){
	
	int no=0;
	CBone *lb=bbaop.firstBone;
	while (lb){
		if (lb->boneMask.isQuaternion) no+=17;
		else no+=16;
		lb=lb->suiv;
	}
	//no+=bbaop.NumberOfBones*16;
	if (bbaop.noOfControllerTypes>1){
		CMuscle *lm=bbaop.firstMuscle;
		while(lm){
			no+=lm->NumberControlPoints*4;
			no+=lm->NumberKnots;
			lm=lm->suiv;
		}
	}
	if (bbaop.noOfControllerTypes>2){
		CMorph *lmf=bbaop.firstMorph;
		while(lmf){
			no+=lmf->NumberWeights;
			lmf=lmf->suiv;
		}
	}

	bbaop.NumberOfSBComponentsInPlane=no;
	bbaop.sbc_mask  = (bool*) malloc (no*sizeof(bool));
//	bbaop.sbc_mask_prev  = (bool*) malloc (no*sizeof(bool));
	bbaop.sbc_values= (int*) malloc (no*sizeof(int));
	bbaop.sbc_values_prev= (int*) malloc (no*sizeof(int));
	bbaop.Irange_max= (int*) malloc (no*sizeof(int));
	bbaop.Irange_min= (int*) malloc (no*sizeof(int));
	bbaop.range_max = (int*) malloc (no*sizeof(int));
	bbaop.range_min = (int*) malloc (no*sizeof(int));
	return 0;
}

int BBAEncoder::FreeThePlane(){

	free (bbaop.sbc_mask);
	free (bbaop.sbc_values);
	free (bbaop.sbc_values_prev);
	free (bbaop.Irange_max);
	free (bbaop.Irange_min);
	free (bbaop.range_max); 
	free (bbaop.range_min); 
	return 0;
}

int BBAEncoder::UnLoadThePlane(){

	int count=0,i;
	CBone *lb= new CBone();
	lb = bbaop.firstBone;
	while (lb){
		if (!lb->isInCurrentFrame)
			lb=lb->suiv;
		else{
			lb->isInCurrentFrame=false;
			lb->boneDataPreviousFrame.TonX=bbaop.sbc_values_prev[count];
			count++;

			lb->boneDataPreviousFrame.TonY=bbaop.sbc_values_prev[count];
			count++;

			lb->boneDataPreviousFrame.TonZ=bbaop.sbc_values_prev[count];
			count++;

			if (!lb->boneMask.isQuaternion){
				lb->boneDataPreviousFrame.RonAx1=bbaop.sbc_values_prev[count];
				count++;
			
				lb->boneDataPreviousFrame.RonAx2=bbaop.sbc_values_prev[count];
				count++;

				lb->boneDataPreviousFrame.RonAx3=bbaop.sbc_values_prev[count];
				count++;
			}else{
				lb->boneDataPreviousFrame.RonQx=bbaop.sbc_values_prev[count];
				count++;
				lb->boneDataPreviousFrame.RonQy=bbaop.sbc_values_prev[count];
				count++;
				lb->boneDataPreviousFrame.RonQz=bbaop.sbc_values_prev[count];
				count++;
				lb->boneDataPreviousFrame.RonQw=bbaop.sbc_values_prev[count];
				count++;	
			}

			lb->boneDataPreviousFrame.SonX=bbaop.sbc_values_prev[count];
			count++;

			lb->boneDataPreviousFrame.SonY=bbaop.sbc_values_prev[count];
			count++;

			lb->boneDataPreviousFrame.SonZ=bbaop.sbc_values_prev[count];
			count++;

			lb->boneDataPreviousFrame.SOonAxX=bbaop.sbc_values_prev[count];
			count++;

			lb->boneDataPreviousFrame.SOonAxY=bbaop.sbc_values_prev[count];
			count++;

			lb->boneDataPreviousFrame.SOonAxZ=bbaop.sbc_values_prev[count];
			count++;

			lb->boneDataPreviousFrame.SOonVal=bbaop.sbc_values_prev[count];
			count++;
			
			lb->boneDataPreviousFrame.ConX=bbaop.sbc_values_prev[count];
			count++;

			lb->boneDataPreviousFrame.ConY=bbaop.sbc_values_prev[count];
			count++;

			lb->boneDataPreviousFrame.ConZ=bbaop.sbc_values_prev[count];
			count++;
			
			lb=lb->suiv;
		}
	}
/////////////////////////////muscles///////////////////////////
	if (bbaop.noOfControllerTypes>1){
		CMuscle *lm= new CMuscle();
		lm = bbaop.firstMuscle;
		while (lm){
			if (!lm->isInCurrentFrame)
				lm=lm->suiv;
			else{
				lm->isInCurrentFrame=false;
				for (i=0;i<lm->NumberControlPoints;i++){
					lm->muscleDataPreviousFrame.ControlPointOnX[i]=bbaop.sbc_values_prev[count];
					count++;

					lm->muscleDataPreviousFrame.ControlPointOnY[i]=bbaop.sbc_values_prev[count];
					count++;
	
					lm->muscleDataPreviousFrame.ControlPointOnZ[i]=bbaop.sbc_values_prev[count];
					count++;
				}
				for (i=0;i<lm->NumberControlPoints;i++){
					lm->muscleDataPreviousFrame.WheighOfControlPoint[i]=bbaop.sbc_values_prev[count];
					count++;
				}
				for (i=0;i<lm->NumberKnots;i++){
					lm->muscleDataPreviousFrame.Knot[i]=bbaop.sbc_values_prev[count];
					count++;
				}			
				lm=lm->suiv;
			}
		}
	}
	if (bbaop.noOfControllerTypes>2){
		CMorph *lmf= new CMorph();
		lmf = bbaop.firstMorph;
		while (lmf){
			if (!lmf->isInCurrentFrame)
				lmf=lmf->suiv;
			else{
				lmf->isInCurrentFrame=false;
				for (i=0;i<lmf->NumberWeights;i++){
					lmf->morphDataPreviousFrame.Weights[i]=bbaop.sbc_values_prev[count];
					count++;
				}
				lmf=lmf->suiv;
			}
		}
	}
	return true;
}

int BBAEncoder::LoadThePlane(){
	
	// compute the plane specific data 
	int i, no=0;
	int count=0;
	CBone *lb= new CBone();
	lb = bbaop.firstBone;
	while (lb){
		if (!lb->isInCurrentFrame)
			lb=lb->suiv;
		else{
			bbaop.sbc_mask[count]=lb->boneMask.isTonXChanged;
			bbaop.sbc_values[count]=lb->boneData.TonX;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.TonX;
			bbaop.Irange_max[count]=lb->boneDataIMax.TonX;
			bbaop.Irange_min[count]=lb->boneDataIMin.TonX;
			bbaop.range_max[count]=lb->boneDataPMax.TonX;
			bbaop.range_min[count]=lb->boneDataPMin.TonX;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isTonYChanged;
			bbaop.sbc_values[count]=lb->boneData.TonY;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.TonY;
			bbaop.Irange_max[count]=lb->boneDataIMax.TonY;
			bbaop.Irange_min[count]=lb->boneDataIMin.TonY;
			bbaop.range_max[count]=lb->boneDataPMax.TonY;
			bbaop.range_min[count]=lb->boneDataPMin.TonY;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isTonZChanged;
			bbaop.sbc_values[count]=lb->boneData.TonZ;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.TonZ;
			bbaop.Irange_max[count]=lb->boneDataIMax.TonZ;
			bbaop.Irange_min[count]=lb->boneDataIMin.TonZ;
			bbaop.range_max[count]=lb->boneDataPMax.TonZ;
			bbaop.range_min[count]=lb->boneDataPMin.TonZ;
			count++;

			if (!lb->boneMask.isQuaternion){
				bbaop.sbc_mask[count]=lb->boneMask.isRonAx1Changed;
				bbaop.sbc_values[count]=lb->boneData.RonAx1;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonAx1;
				bbaop.Irange_max[count]=lb->boneDataIMax.RonAx1;
				bbaop.Irange_min[count]=lb->boneDataIMin.RonAx1;
				bbaop.range_max[count]=lb->boneDataPMax.RonAx1;
				bbaop.range_min[count]=lb->boneDataPMin.RonAx1;
				count++;
				
				bbaop.sbc_mask[count]=lb->boneMask.isRonAx2Changed;
				bbaop.sbc_values[count]=lb->boneData.RonAx2;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonAx2;
				bbaop.Irange_max[count]=lb->boneDataIMax.RonAx2;
				bbaop.Irange_min[count]=lb->boneDataIMin.RonAx2;
				bbaop.range_max[count]=lb->boneDataPMax.RonAx2;
				bbaop.range_min[count]=lb->boneDataPMin.RonAx2;
				count++;

				bbaop.sbc_mask[count]=lb->boneMask.isRonAx3Changed;
				bbaop.sbc_values[count]=lb->boneData.RonAx3;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonAx3;
				bbaop.Irange_max[count]=lb->boneDataIMax.RonAx3;
				bbaop.Irange_min[count]=lb->boneDataIMin.RonAx3;
				bbaop.range_max[count]=lb->boneDataPMax.RonAx3;
				bbaop.range_min[count]=lb->boneDataPMin.RonAx3;
				count++;
			}else{
				bbaop.sbc_mask[count]=lb->boneMask.isRonQxChanged;
				bbaop.sbc_values[count]=lb->boneData.RonQx;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQx;
				bbaop.Irange_max[count]=lb->boneDataIMax.RonQx;
				bbaop.Irange_min[count]=lb->boneDataIMin.RonQx;
				bbaop.range_max[count]=lb->boneDataPMax.RonQx;
				bbaop.range_min[count]=lb->boneDataPMin.RonQx;
				count++;
				
				bbaop.sbc_mask[count]=lb->boneMask.isRonQyChanged;
				bbaop.sbc_values[count]=lb->boneData.RonQy;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQy;
				bbaop.Irange_max[count]=lb->boneDataIMax.RonQy;
				bbaop.Irange_min[count]=lb->boneDataIMin.RonQy;
				bbaop.range_max[count]=lb->boneDataPMax.RonQy;
				bbaop.range_min[count]=lb->boneDataPMin.RonQy;
				count++;
				
				bbaop.sbc_mask[count]=lb->boneMask.isRonQzChanged;
				bbaop.sbc_values[count]=lb->boneData.RonQz;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQz;
				bbaop.Irange_max[count]=lb->boneDataIMax.RonQz;
				bbaop.Irange_min[count]=lb->boneDataIMin.RonQz;
				bbaop.range_max[count]=lb->boneDataPMax.RonQz;
				bbaop.range_min[count]=lb->boneDataPMin.RonQz;
				count++;
				
				bbaop.sbc_mask[count]=lb->boneMask.isRonQwChanged;
				bbaop.sbc_values[count]=lb->boneData.RonQw;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQw;
				bbaop.Irange_max[count]=lb->boneDataIMax.RonQw;
				bbaop.Irange_min[count]=lb->boneDataIMin.RonQw;
				bbaop.range_max[count]=lb->boneDataPMax.RonQw;
				bbaop.range_min[count]=lb->boneDataPMin.RonQw;
				count++;
			}


			bbaop.sbc_mask[count]=lb->boneMask.isSonXChanged;
			bbaop.sbc_values[count]=lb->boneData.SonX;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SonX;
			bbaop.Irange_max[count]=lb->boneDataIMax.SonX;
			bbaop.Irange_min[count]=lb->boneDataIMin.SonX;
			bbaop.range_max[count]=lb->boneDataPMax.SonX;
			bbaop.range_min[count]=lb->boneDataPMin.SonX;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isSonYChanged;
			bbaop.sbc_values[count]=lb->boneData.SonY;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SonY;
			bbaop.Irange_max[count]=lb->boneDataIMax.SonY;
			bbaop.Irange_min[count]=lb->boneDataIMin.SonY;
			bbaop.range_max[count]=lb->boneDataPMax.SonY;
			bbaop.range_min[count]=lb->boneDataPMin.SonY;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isSonZChanged;
			bbaop.sbc_values[count]=lb->boneData.SonZ;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SonZ;
			bbaop.Irange_max[count]=lb->boneDataIMax.SonZ;
			bbaop.Irange_min[count]=lb->boneDataIMin.SonZ;
			bbaop.range_max[count]=lb->boneDataPMax.SonZ;
			bbaop.range_min[count]=lb->boneDataPMin.SonZ;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isSOonAxXChanged;
			bbaop.sbc_values[count]=lb->boneData.SOonAxX;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonAxX;
			bbaop.Irange_max[count]=lb->boneDataIMax.SOonAxX;
			bbaop.Irange_min[count]=lb->boneDataIMin.SOonAxX;
			bbaop.range_max[count]=lb->boneDataPMax.SOonAxX;
			bbaop.range_min[count]=lb->boneDataPMin.SOonAxX;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isSOonAxYChanged;
			bbaop.sbc_values[count]=lb->boneData.SOonAxY;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonAxY;
			bbaop.Irange_max[count]=lb->boneDataIMax.SOonAxY;
			bbaop.Irange_min[count]=lb->boneDataIMin.SOonAxY;
			bbaop.range_max[count]=lb->boneDataPMax.SOonAxY;
			bbaop.range_min[count]=lb->boneDataPMin.SOonAxY;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isSOonAxZChanged;
			bbaop.sbc_values[count]=lb->boneData.SOonAxZ;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonAxZ;
			bbaop.Irange_max[count]=lb->boneDataIMax.SOonAxZ;
			bbaop.Irange_min[count]=lb->boneDataIMin.SOonAxZ;
			bbaop.range_max[count]=lb->boneDataPMax.SOonAxZ;
			bbaop.range_min[count]=lb->boneDataPMin.SOonAxZ;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isSOonValChanged;
			bbaop.sbc_values[count]=lb->boneData.SOonVal;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonVal;
			bbaop.Irange_max[count]=lb->boneDataIMax.SOonVal;
			bbaop.Irange_min[count]=lb->boneDataIMin.SOonVal;
			bbaop.range_max[count]=lb->boneDataPMax.SOonVal;
			bbaop.range_min[count]=lb->boneDataPMin.SOonVal;
			count++;
			
			bbaop.sbc_mask[count]=lb->boneMask.isConXChanged;
			bbaop.sbc_values[count]=lb->boneData.ConX;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.ConX;
			bbaop.Irange_max[count]=lb->boneDataIMax.ConX;
			bbaop.Irange_min[count]=lb->boneDataIMin.ConX;
			bbaop.range_max[count]=lb->boneDataPMax.ConX;
			bbaop.range_min[count]=lb->boneDataPMin.ConX;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isConYChanged;
			bbaop.sbc_values[count]=lb->boneData.ConY;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.ConY;
			bbaop.Irange_max[count]=lb->boneDataIMax.ConY;
			bbaop.Irange_min[count]=lb->boneDataIMin.ConY;
			bbaop.range_max[count]=lb->boneDataPMax.ConY;
			bbaop.range_min[count]=lb->boneDataPMin.ConY;
			count++;

			bbaop.sbc_mask[count]=lb->boneMask.isConZChanged;
			bbaop.sbc_values[count]=lb->boneData.ConZ;
			bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.ConZ;
			bbaop.Irange_max[count]=lb->boneDataIMax.ConZ;
			bbaop.Irange_min[count]=lb->boneDataIMin.ConZ;
			bbaop.range_max[count]=lb->boneDataPMax.ConZ;
			bbaop.range_min[count]=lb->boneDataPMin.ConZ;
			count++;
			
			lb=lb->suiv;
		}
	}

	/////////////////////muscles////////////////////////////
	if (bbaop.noOfControllerTypes>1){
		CMuscle *lm= new CMuscle();
		lm = bbaop.firstMuscle;
		while (lm){
			if (!lm->isInCurrentFrame)
				lm=lm->suiv;
			else{
				if (lm->isControlPointsChanged()){
					for (i=0;i<lm->NumberControlPoints;i++){
						bbaop.sbc_mask[count]=lm->muscleMask.isControlPointOnXChanged[i];
						bbaop.sbc_values[count]=lm->muscleData.ControlPointOnX[i];
						bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.ControlPointOnX[i];
						bbaop.Irange_max[count]=lm->muscleDataIMax.ControlPointOnX[i];
						bbaop.Irange_min[count]=lm->muscleDataIMin.ControlPointOnX[i];
						bbaop.range_max[count]=lm->muscleDataPMax.ControlPointOnX[i];
						bbaop.range_min[count]=lm->muscleDataPMin.ControlPointOnX[i];
						count++;

						bbaop.sbc_mask[count]=lm->muscleMask.isControlPointOnYChanged[i];
						bbaop.sbc_values[count]=lm->muscleData.ControlPointOnY[i];
						bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.ControlPointOnY[i];
						bbaop.Irange_max[count]=lm->muscleDataIMax.ControlPointOnY[i];
						bbaop.Irange_min[count]=lm->muscleDataIMin.ControlPointOnY[i];
						bbaop.range_max[count]=lm->muscleDataPMax.ControlPointOnY[i];
						bbaop.range_min[count]=lm->muscleDataPMin.ControlPointOnY[i];
						count++;

						bbaop.sbc_mask[count]=lm->muscleMask.isControlPointOnZChanged[i];
						bbaop.sbc_values[count]=lm->muscleData.ControlPointOnZ[i];
						bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.ControlPointOnZ[i];
						bbaop.Irange_max[count]=lm->muscleDataIMax.ControlPointOnZ[i];
						bbaop.Irange_min[count]=lm->muscleDataIMin.ControlPointOnZ[i];
						bbaop.range_max[count]=lm->muscleDataPMax.ControlPointOnZ[i];
						bbaop.range_min[count]=lm->muscleDataPMin.ControlPointOnZ[i];
						count++;
					}
				}
				if (lm->isWheightOfControlPointsChanged()){
						bbaop.sbc_mask[count]=lm->muscleMask.isWheighOfControlPointChanged[i];
						bbaop.sbc_values[count]=lm->muscleData.WheighOfControlPoint[i];
						bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.WheighOfControlPoint[i];
						bbaop.Irange_max[count]=lm->muscleDataIMax.WheighOfControlPoint[i];
						bbaop.Irange_min[count]=lm->muscleDataIMin.WheighOfControlPoint[i];
						bbaop.range_max[count]=lm->muscleDataPMax.WheighOfControlPoint[i];
						bbaop.range_min[count]=lm->muscleDataPMin.WheighOfControlPoint[i];
						count++;
				}
				if (lm->isKnotChanged()){
						bbaop.sbc_mask[count]=lm->muscleMask.isKnotChanged[i];
						bbaop.sbc_values[count]=lm->muscleData.Knot[i];
						bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.Knot[i];
						bbaop.Irange_max[count]=lm->muscleDataIMax.Knot[i];
						bbaop.Irange_min[count]=lm->muscleDataIMin.Knot[i];
						bbaop.range_max[count]=lm->muscleDataPMax.Knot[i];
						bbaop.range_min[count]=lm->muscleDataPMin.Knot[i];
						count++;
				}
				lm=lm->suiv;
			}
		}
	}
	if (bbaop.noOfControllerTypes>2){
		CMorph *lmf= new CMorph();
		lmf = bbaop.firstMorph;
		while (lmf){
			if (!lmf->isInCurrentFrame)
				lmf=lmf->suiv;
			else{
				if (lmf->isWeightsChanged()){
					for (i=0;i<lmf->NumberWeights;i++){
						bbaop.sbc_mask[count]=lmf->morphMask.isWeightChanged[i];
						bbaop.sbc_values[count]=lmf->morphData.Weights[i];
						bbaop.sbc_values_prev[count]=lmf->morphDataPreviousFrame.Weights[i];
						bbaop.Irange_max[count]=lmf->morphDataIMax.Weights[i];
						bbaop.Irange_min[count]=lmf->morphDataIMin.Weights[i];
						bbaop.range_max[count]=lmf->morphDataPMax.Weights[i];
						bbaop.range_min[count]=lmf->morphDataPMin.Weights[i];
						count++;
					}
				}
				lmf=lmf->suiv;
			}
		}	
	}
	bbaop.NumberOfSBComponentsInPlane=count;
	return true;
}


int BBAEncoder::mygetint(unsigned char *buff, int *count){
	
	int ctr=*count;
	int myint=0;
	bool isminus=false;

	int a=(int)(char)(buff[ctr]);
	
	while(!((a==45) || ((a>=48) && (a<=57)))){ // 45 -, 48 0, 57 9;
		ctr++;
		a=(int)(char)buff[ctr];
	}
	//ctr--;
	if (a==45) {
		isminus=true;
		ctr++;
		a=(int)(char)buff[ctr];
	}

	
	int inctr=0;
	unsigned char inbfr[30];

	while((a>=48) && (a<=57)){
		inbfr[inctr]=buff[ctr];
		ctr++;inctr++;
		a=(int)(char)buff[ctr];		
	}
	//ctr--;
	for (int i=0;i<inctr;i++)
		myint+=(inbfr[i]-48)*pow(10,(inctr-1-i));

	if (isminus) myint*=-1;
	*count=ctr++;
	return myint;
}


bool BBAEncoder::CheckIsIntraFrame(){
	bbaop.is_intra=true;// for the moment encode just intraframe
	return true;
	// to add discution about bone and muscle ID
	int j;
	bbaop.is_intra=0;
	for (j=0; j<bbaop.NumberOfSBComponentsInPlane; j++) {
	   if (bbaop.sbc_mask[j]!=bbaop.sbc_mask_prev[j]){
		   bbaop.is_intra=1;
		   j=bbaop.NumberOfSBComponentsInPlane;
	   }
	}
	for (j=0; j<bbaop.NumberOfSBComponentsInPlane; j++) 
		bbaop.sbc_mask_prev[j]=bbaop.sbc_mask[j];

	return bbaop.is_intra;
}

void BBAEncoder::ConfigureEncoder(){
	// configure with defaults values
	// TODO add a configuration dialog

		bbaop.is_frame_rate=0;
		bbaop.frame_rate=10;
		
		bbaop.skip_frames=0;
		bbaop.number_of_frames_to_skip=0;

		bbaop.is_i_new_max=0;
		bbaop.is_i_new_min=0;
		bbaop.is_p_new_max=0;
		bbaop.is_p_new_min=0;
		
		bbaop.is_time_code=0;
		bbaop.time_code_hours=0;
		bbaop.time_code_minutes = 0;
		bbaop.time_code_seconds = 0;
		bbaop.seconds=0;
		bbaop.frequency_offset=0;
		bbaop.marker_bit=MARKER_BIT;
		bbaop.bba_object_coding_type=coding_type;

}

void BBAEncoder::saveInFile(char *fname){
	
	FILE *fp=fopen(fname,"wb");
	for (int i=0;i<bbaop.stream.p;i++)
		putc(bbaop.stream.buf[i],fp);
	fclose(fp);
}

void BBAEncoder::PutBBAStartCode(){
	PutBits(32,BBA_OBJECT_START_CODE);
}
void BBAEncoder::PutBBAPlaneStartCode(){
	PutBits(32,BBA_OBJECT_PLANE_START_CODE);

}

void BBAEncoder::PutBBAEndCode(){
	PutBits(32,BBA_OBJECT_END_CODE);
}


void BBAEncoder::init(void)
{
  int i,j,counter;

  bbaop.firstBone=NULL;
  bbaop.lastBone=NULL;
  bbaop.firstMuscle=NULL;
  bbaop.lastMuscle=NULL;
  bbaop.firstMorph=NULL;
  bbaop.lastMorph=NULL;

}

void BBAEncoder::ArithmeticInit(){
	int i;
	int bba_pred_scale_tmp[32] = { 0,  1,   2,  3,   5,   7,   9,  11,  14,  17,  20,  23,  27,  31,  35,  39,
									43, 47, 52, 57,  62,  67,  72,  77,  82,  88,  94, 100, 106, 113, 120, 127};
	for (i=0;i<32;i++)
		bba_pred_scale[i]=bba_pred_scale_tmp[i];
}

void BBAEncoder::DCTInit(){
	int i,j;
	float c;
	static int bba_scale_tmp[32] = { 0, 1, 2, 3, 5, 7, 8, 10, 12, 15, 18, 21, 25, 30, 35, 42, 
								50, 60, 72, 87, 105, 128, 156, 191, 234, 288, 355, 439, 543, 
								674, 836, 1023 };

	for (i=0;i<32;i++)
		bba_pred_scale[i] = bba_scale_tmp[i];

    bba_init_htables();
    bbaop.dctMatrix = (float**) malloc(SEG_LENGTH*sizeof(float*));
    for (i=0; i<SEG_LENGTH; i++) {
        bbaop.dctMatrix[i] = (float*) malloc(SEG_LENGTH*sizeof(float));
    }
 	for(i=0; i<SEG_LENGTH; i++) {
		if(i==0) c = (float)(1.0/sqrt(2));
		else c = 1;
		for(j=0; j<SEG_LENGTH; j++) {
		bbaop.dctMatrix[i][j] = c * (float)(sqrt(2.0/SEG_LENGTH))
					* (float)(cos(PI*(float)(2*j+1)*i/2.0/SEG_LENGTH));
    }
  }
}



void BBAEncoder::bba_setPDC(int dc_Q){

	if( (dc_Q>255) || (dc_Q<-255) ){
		HuffmanCoding(bbaop.dc_table,0);
		bba_setDC(dc_Q);
	}
	else 
		HuffmanCoding(bbaop.dc_table,dc_Q+256);
}



void BBAEncoder::bba_dctquant(float *src, int *dest, int q) {
  int i;

  if (src[0]>0.0)
	  dest[0] = (int) (src[0] / (q/3.0)+0.5);
  else 
	  dest[0] = (int) (src[0] / (q/3.0)-0.5);

  for (i=1; i<SEG_LENGTH; i++){
	  if (src[i]>0.0)
		  dest[i] = (int)(src[i] / q + 0.5);
	  else 
		  dest[i] = (int)(src[i] / q - 0.5);
  }
}

void BBAEncoder::bba_setDC(int DCCoef){
	  
	if( (DCCoef > SHORT_MAX) || (DCCoef < SHORT_MIN) ) {    
		ShortPutBits((short)(SHORT_MIN-1));    
		if (bbaop.stream.outcnt!=8) PutBits(bbaop.stream.outcnt,0);
		PutBits(32,DCCoef);
	}
	else 
      ShortPutBits((short)DCCoef);
}

void BBAEncoder::bba_setAC(int *ACCoef){

	int i;
	int run_length;
  
	run_length = 1;
    for (i=1; i<SEG_LENGTH; i++) {
		if(ACCoef[i] != 0) {
			HuffmanCoding(bbaop.runlength_table,run_length);
			if( (ACCoef[i]>255) || (ACCoef[i]<-256) ){
				HuffmanCoding(bbaop.ac_table,256);
				bba_setDC(ACCoef[i]);			
			}
		else
			HuffmanCoding(bbaop.ac_table,ACCoef[i]+256);	    
		run_length = 1;
		}
		else 
			run_length++;
	}
	if(run_length != 1)
		HuffmanCoding(bbaop.runlength_table,0);
 
}

void BBAEncoder::HuffmanCoding(int **table,int symbol)
{
	int counter,bitcounter;	
	bitcounter = table[symbol][0];
	for(counter=1;counter<=bitcounter;counter++)
		PutBits(1,table[symbol][counter]);
}


bool BBAEncoder::ReadABoneMask(unsigned char *buff,CBone *bn, int *ct){
	
	int count=*ct;
	CBoneMask *bml=&(bn->boneMask);

	////////// translation /////////////
	
	if (!(mygetint(buff, &count))) {
		bml->isTonXChanged=false;
		bml->isTonYChanged=false;
		bml->isTonZChanged=false;
	}else{
		if (!(mygetint(buff, &count))) bml->isTonXChanged=false;
		else bml->isTonXChanged=true;
		if (!(mygetint(buff, &count))) bml->isTonYChanged=false;
		else bml->isTonYChanged=true;
		if (!(mygetint(buff, &count))) bml->isTonZChanged=false;
		else bml->isTonZChanged=true;
	}

	////////// rotation /////////////
	if (!(mygetint(buff, &count))) {
		bml->isRonAx1Changed=false;
		bml->isRonAx2Changed=false;
		bml->isRonAx3Changed=false;
		bml->isRonQxChanged=false;
		bml->isRonQyChanged=false;
		bml->isRonQzChanged=false;
		bml->isRonQwChanged=false;
	}else{
		bml->isQuaternion=mygetint(buff, &count);
		if (!bml->isQuaternion){ // is EA
			if (!(mygetint(buff, &count))) bml->isRonAx1Changed=false;
			else bml->isRonAx1Changed=true;
			if (!(mygetint(buff, &count))) bml->isRonAx2Changed=false;
			else bml->isRonAx2Changed=true;
			if (!(mygetint(buff, &count))) bml->isRonAx3Changed=false;
			else bml->isRonAx3Changed=true;
		}else{
			if (!(mygetint(buff, &count))) bml->isRonQxChanged=false;
			else bml->isRonQxChanged=true;
			if (!(mygetint(buff, &count))) bml->isRonQyChanged=false;
			else bml->isRonQyChanged=true;
			if (!(mygetint(buff, &count))) bml->isRonQzChanged=false;
			else bml->isRonQzChanged=true;
			if (!(mygetint(buff, &count))) bml->isRonQwChanged=false;
			else bml->isRonQwChanged=true;
		}
	}

	////////// scale /////////////
	if (!(mygetint(buff, &count))) {
		bml->isSonXChanged=false;
		bml->isSonYChanged=false;
		bml->isSonZChanged=false;
	}else{
		if (!(mygetint(buff, &count))) bml->isSonXChanged=false;
		else bml->isSonXChanged=true;
		if (!(mygetint(buff, &count))) bml->isSonYChanged=false;
		else bml->isSonYChanged=true;
		if (!(mygetint(buff, &count))) bml->isSonZChanged=false;
		else bml->isSonZChanged=true;
	}

	////////// scaleOrientation /////////////
	if (!(mygetint(buff, &count))) {
		bml->isSOonAxXChanged=false;
		bml->isSOonAxYChanged=false;
		bml->isSOonAxZChanged=false;
		bml->isSOonValChanged=false;
	}else{
		if (!(mygetint(buff, &count))) bml->isSOonAxXChanged=false;
		else bml->isSOonAxXChanged=true;
		if (!(mygetint(buff, &count))) bml->isSOonAxYChanged=false;
		else bml->isSOonAxYChanged=true;
		if (!(mygetint(buff, &count))) bml->isSOonAxZChanged=false;
		else bml->isSOonAxZChanged=true;
		if (!(mygetint(buff, &count))) bml->isSOonValChanged=false;
		else bml->isSOonValChanged=true;
	}

	////////// center /////////////
	if (!(mygetint(buff, &count))) {
		bml->isConXChanged=false;
		bml->isConYChanged=false;
		bml->isConZChanged=false;
	}else{
		if (!(mygetint(buff, &count))) bml->isConXChanged=false;
		else bml->isConXChanged=true;
		if (!(mygetint(buff, &count))) bml->isConYChanged=false;
		else bml->isConYChanged=true;
		if (!(mygetint(buff, &count))) bml->isConZChanged=false;
		else bml->isConZChanged=true;
	}
	*ct=count;
//	bo->PrintTheMask();

	return true;
}

bool BBAEncoder::ReadABoneData(unsigned char *buff,CBone *bn, int *ct){
	
	int count=*ct;
	CBoneMask *bml=&(bn->boneMask);
	CBoneData *bdl=&(bn->boneData);

	////////// translation /////////////
	if (bml->isTonXChanged) bdl->TonX=mygetint(buff, &count);
	if (bml->isTonYChanged) bdl->TonY=mygetint(buff, &count);
	if (bml->isTonZChanged) bdl->TonZ=mygetint(buff, &count);

	////////// rotation /////////////
	if (!bml->isQuaternion){
		if (bml->isRonAx1Changed) bdl->RonAx1=mygetint(buff, &count);
		if (bml->isRonAx2Changed) bdl->RonAx2=mygetint(buff, &count);
		if (bml->isRonAx3Changed) bdl->RonAx3=mygetint(buff, &count);
	}else{
		if (bml->isRonQxChanged) bdl->RonQx=mygetint(buff, &count);
		if (bml->isRonQyChanged) bdl->RonQy=mygetint(buff, &count);
		if (bml->isRonQzChanged) bdl->RonQz=mygetint(buff, &count);
		if (bml->isRonQwChanged) bdl->RonQw=mygetint(buff, &count);
	}

	////////// scale /////////////
	if (bml->isSonXChanged) bdl->SonX=mygetint(buff, &count);
	if (bml->isSonYChanged) bdl->SonY=mygetint(buff, &count);
	if (bml->isSonZChanged) bdl->SonZ=mygetint(buff, &count);

	////////// scaleOrientation /////////////
	if (bml->isSOonAxXChanged) bdl->SOonAxX=mygetint(buff, &count);
	if (bml->isSOonAxYChanged) bdl->SOonAxY=mygetint(buff, &count);
	if (bml->isSOonAxZChanged) bdl->SOonAxZ=mygetint(buff, &count);
	if (bml->isSOonValChanged) bdl->SOonVal=mygetint(buff, &count);
	
	////////// center /////////////
	if (bml->isConXChanged) bdl->ConX=mygetint(buff, &count);
	if (bml->isConYChanged) bdl->ConY=mygetint(buff, &count);
	if (bml->isConZChanged) bdl->ConZ=mygetint(buff, &count);
	
	*ct=count;
	return true;
}

///////////////////     DCT Stuff ///////////////

void BBAEncoder::ReadASegment(unsigned char *buff){
	
	int i,seg, count=0;
	bool ismask;
	CBone *lb;
	CMuscle *lm;
	for (seg=0;seg<SEG_LENGTH;seg++){
		bbaop.KeyFrameIndex  =mygetint(buff, &count);
		bbaop.noOfControllerTypes  =mygetint(buff, &count);
		bbaop.NumberOfBones  =mygetint(buff, &count);
		bbaop.NumberOfMuscles=mygetint(buff, &count);

		int *id = new int [bbaop.NumberOfBones];
		int *idm = new int [bbaop.NumberOfMuscles];
		
		for(i=0;i<bbaop.NumberOfBones;i++){
			lb=new CBone();
			lb->boneIdentifier=mygetint(buff, &count);
			id[i]=lb->boneIdentifier;
			lb->boneMask.init();
			ReadABoneMask(buff,lb, &count);
			lb->isInCurrentFrame=true;
			innod(lb,bbaop.firstBone,bbaop.lastBone);
			lb=NULL;
		}
		for(i=0;i<bbaop.NumberOfMuscles;i++){
			lm=new CMuscle();
			lm->muscleIdentifier=mygetint(buff, &count);
			idm[i]=lm->muscleIdentifier;
			ReadAMuscleMask(buff,lm, &count);
			lm->isInCurrentFrame=true;
			innod(lm,bbaop.firstMuscle,bbaop.lastMuscle);
			lm=NULL;
		}
		for(i=0;i<bbaop.NumberOfBones;i++){
			lb = getnodIDi(id[i],bbaop.firstBone,bbaop.lastBone);
			lb->boneData.init();
			ReadABoneData(buff,lb, &count);
		}
		delete id; 
		for(i=0;i<bbaop.NumberOfMuscles;i++){
			lm = getnodIDi(idm[i],bbaop.firstMuscle,bbaop.lastMuscle);
			ReadAMuscleData(buff,lm, &count);
		}
		delete idm; 
		////////// put the current frame in DCT part of elements
		lb=bbaop.firstBone;
		while(lb){
			if (lb->boneDCTData == NULL){
				lb->boneDCTData = (CBoneData*)malloc(SEG_LENGTH*sizeof(CBoneData));
			}
			lb->boneDCTData[seg]=lb->boneData;
			lb=lb->suiv;
		}
		lm=bbaop.firstMuscle;
		while(lm){
			if (lm->muscleDCTData == NULL){
				lm->muscleDCTData = (CMuscleData*)malloc(SEG_LENGTH*sizeof(CMuscleData));
			}
			lm->AllocDCTMuscle();
			lm->muscleDCTData[seg]=lm->muscleData;
			lm=lm->suiv;
		}
	}
}
void BBAEncoder::AllocThePlaneSegment(){

	int no=0,i;
	CBone *lb=bbaop.firstBone;
	while (lb){
		if (lb->boneMask.isQuaternion) no+=17;
		else no+=16;
		lb=lb->suiv;
	}
//	no+=bbaop.NumberOfBones*16;
	CMuscle *lm=bbaop.firstMuscle;
	while(lm){
		no+=lm->NumberControlPoints*4;
		no+=lm->NumberKnots;
		lm=lm->suiv;
	}

	bbaop.NumberOfSBComponentsInPlane=no;

	bbaop.sbc_mask  = (bool*) malloc (no*sizeof(bool));
	bbaop.sbc_DCTvalues= (int**) malloc (no*sizeof(int*));
	for (i=0;i<no;i++)
		bbaop.sbc_DCTvalues[i]=(int*) malloc (SEG_LENGTH*sizeof(int*));
	bbaop.sbc_values_prev= (int*) malloc (no*sizeof(int));

}
void BBAEncoder::LoadThePlaneSegment(){
	int i, no=0,seg;
	int count=0;
	CBone *lb= new CBone();
	CMuscle *lm= new CMuscle();

	/////////// load mask and previous values ///////////
	lb=bbaop.firstBone;
	while (lb){
		if (!lb->isInCurrentFrame)
			lb=lb->suiv;
		else{
				bbaop.sbc_mask[count]=lb->boneMask.isTonXChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.TonX;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isTonYChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.TonY;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isTonZChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.TonZ;
				count++;
				if (!lb->boneMask.isQuaternion){
					bbaop.sbc_mask[count]=lb->boneMask.isRonAx1Changed;
					bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonAx1;
					count++;				
					bbaop.sbc_mask[count]=lb->boneMask.isRonAx2Changed;
					bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonAx2;
					count++;	
					bbaop.sbc_mask[count]=lb->boneMask.isRonAx3Changed;
					bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonAx3;
					count++;
				}else{
					bbaop.sbc_mask[count]=lb->boneMask.isRonQxChanged;
					bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQx;
					count++;				
					bbaop.sbc_mask[count]=lb->boneMask.isRonQyChanged;
					bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQy;
					count++;				
					bbaop.sbc_mask[count]=lb->boneMask.isRonQzChanged;
					bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQz;
					count++;				
					bbaop.sbc_mask[count]=lb->boneMask.isRonQwChanged;
					bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.RonQw;
					count++;				
				}
				bbaop.sbc_mask[count]=lb->boneMask.isSonXChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SonX;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isSonYChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SonY;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isSonZChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SonZ;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isSOonAxXChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonAxX;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isSOonAxYChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonAxY;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isSOonAxZChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonAxZ;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isSOonValChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.SOonVal;
				count++;				
				bbaop.sbc_mask[count]=lb->boneMask.isConXChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.ConX;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isConYChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.ConY;
				count++;
				bbaop.sbc_mask[count]=lb->boneMask.isConZChanged;
				bbaop.sbc_values_prev[count]=lb->boneDataPreviousFrame.ConZ;
				count++;				
				lb=lb->suiv;
		}
	}
	lm = bbaop.firstMuscle;
	while (lm){
		if (!lm->isInCurrentFrame)
			lm=lm->suiv;
		else{
			//if (lm->isControlPointsChanged()){
				for (i=0;i<lm->NumberControlPoints;i++){
					bbaop.sbc_mask[count]=lm->muscleMask.isControlPointOnXChanged[i];
					bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.ControlPointOnX[i];
					count++;
					bbaop.sbc_mask[count]=lm->muscleMask.isControlPointOnYChanged[i];
					bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.ControlPointOnY[i];
					count++;
					bbaop.sbc_mask[count]=lm->muscleMask.isControlPointOnZChanged[i];
					bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.ControlPointOnZ[i];
					count++;
				}
			//}
			//if (lm->isWheightOfControlPointsChanged()){
				for (i=0;i<lm->NumberControlPoints;i++){
					bbaop.sbc_mask[count]=lm->muscleMask.isWheighOfControlPointChanged[i];
					bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.WheighOfControlPoint[i];
					count++;
				}
			//}
			//if (lm->isKnotChanged()){
				for (i=0;i<lm->NumberKnots;i++){
					bbaop.sbc_mask[count]=lm->muscleMask.isKnotChanged[i];
					bbaop.sbc_values_prev[count]=lm->muscleDataPreviousFrame.Knot[i];
					count++;
				}
			//}
			lm=lm->suiv;
		}
	}

	//////////// load the data for the entire segment
	
	for (seg=0;seg<SEG_LENGTH;seg++){
		count=0;
		lb = bbaop.firstBone;
		while (lb){
			if (!lb->isInCurrentFrame)
				lb=lb->suiv;
			else{
				bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].TonX;
				count++;
				bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].TonY;
				count++;
				bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].TonZ;
				count++;
				if (!lb->boneMask.isQuaternion){
					bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].RonAx1;
					count++;				
					bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].RonAx2;
					count++;
					bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].RonAx3;
					count++;
				}else{
					bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].RonQx;
					count++;				
					bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].RonQy;
					count++;
					bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].RonQz;
					count++;
					bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].RonQw;
					count++;
				}
				bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].SonX;
				count++;
				bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].SonY;
				count++;
				bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].SonZ;
				count++;
				bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].SOonAxX;
				count++;
				bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].SOonAxY;
				count++;
				bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].SOonAxZ;
				count++;
				bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].SOonVal;
				count++;				
				bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].ConX;
				count++;
				bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].ConY;
				count++;
				bbaop.sbc_DCTvalues[count][seg]=lb->boneDCTData[seg].ConZ;
				count++;				
				lb=lb->suiv;
			}
		}
		/////////////////////muscles////////////////////////////
		lm = bbaop.firstMuscle;
		while (lm){
			if (!lm->isInCurrentFrame)
				lm=lm->suiv;
			else{
				//if (lm->isControlPointsChanged()){
					for (i=0;i<lm->NumberControlPoints;i++){
						bbaop.sbc_DCTvalues[count][seg]=lm->muscleDCTData[seg].ControlPointOnX[i];
						count++;
						bbaop.sbc_DCTvalues[count][seg]=lm->muscleDCTData[seg].ControlPointOnY[i];
						count++;
						bbaop.sbc_DCTvalues[count][seg]=lm->muscleDCTData[seg].ControlPointOnZ[i];
						count++;
					}
				//}
				//if (lm->isWheightOfControlPointsChanged()){
						bbaop.sbc_DCTvalues[count][seg]=lm->muscleDCTData[seg].WheighOfControlPoint[i];
						count++;
				//}
				//if (lm->isKnotChanged()){
						bbaop.sbc_DCTvalues[count][seg]=lm->muscleDCTData[seg].Knot[i];
						count++;
				//}
				lm=lm->suiv;
			}
		}
	}
//	bbaop.NumberOfSBComponentsInPlane=count;
}

void BBAEncoder::UnLoadThePlaneSegment(){
	UnLoadThePlane();
}
void BBAEncoder::FreeThePlaneSegment(){

	free (bbaop.sbc_mask);
	free (bbaop.sbc_values_prev);
	
	for (int i=0;i<bbaop.NumberOfSBComponentsInPlane;i++)
		free (bbaop.sbc_DCTvalues[i]);
	free (bbaop.sbc_DCTvalues);

}

void  BBAEncoder::encode_bba_i_segment(){

  int i,k,j;
  float dctCoef[SEG_LENGTH];
  int dctQCoef[SEG_LENGTH];

  int qstep = (int)pow(2,bbaop.pow2quant) * bba_pred_scale[bbaop.bba_quant]; // TO 


  for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
	  if(bbaop.sbc_mask[i]) {
		  bba_dctTrans(bbaop.sbc_DCTvalues[i],dctCoef);
	  	  bba_dctquant(dctCoef,dctQCoef, qstep);
		  bbaop.sbc_values_prev[i]=dctQCoef[0]*qstep/3.0;
		  bba_setDC(dctQCoef[0]);
		  bba_setAC(dctQCoef);
	  }
  }
}


void  BBAEncoder::encode_bba_p_segment(){

  int i,j,k;
  float dctCoef[SEG_LENGTH];
  int dctQCoef[SEG_LENGTH];
  float DCDiff;
  
  int   dc_Q;
  float diff;
  int qstep = (int)pow(2,bbaop.pow2quant) * bba_pred_scale[bbaop.bba_quant]; // TO 

  for (i=0; i<bbaop.NumberOfSBComponentsInPlane; i++) {
	  if(bbaop.sbc_mask[i]) {
		  bba_dctTrans(bbaop.sbc_DCTvalues[i],dctCoef);
		  DCDiff=dctCoef[0]-bbaop.sbc_values_prev[i];
		  dctCoef[0]=DCDiff;
	  	  bba_dctquant(dctCoef,dctQCoef, qstep);
		  bba_setPDC(dctQCoef[0]);		  
		  diff=dctQCoef[0]*qstep/3.0;
		  bbaop.sbc_values_prev[i]+=diff;
		  bba_setAC(dctQCoef);
	  }
  }
}

void BBAEncoder::bba_dctTrans(int *src, float *dest) {
  int i, j;
  for (i=0; i<SEG_LENGTH; i++) {
    dest[i] = 0;
    for (j=0; j<SEG_LENGTH; j++) {
		dest[i] += bbaop.dctMatrix[i][j]*(float)src[j];
    }
  }
}


void BBAEncoder::bba_init_htables() {
  int i, j, n, m;

  static unsigned char dc[512][18]={
	{17,1,1,0,1,0,1,1,1,0,0,1,1,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,1,1,1},
	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,1,1,1},
	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,1,1,1},
	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,1,0,1},
	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,1,0,1},
	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,1,0,1},
	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,1,0,1},
	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,1,1,1},	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,1,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,1,1,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,1,0,1,0},
	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,1,1,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,1,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,1,0,1},
	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,1,1,1},	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,1,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,1,0,1},
	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,1,0,1},
	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,1,1,1},	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,1,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,1,1,1},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,1,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,1,1,1},
	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,1,1,1},
	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,1,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,1,1,1},
	{16,1,1,0,1,0,1,1,1,0,0,1,1,0,0,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,1,1,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,0,1,1},
	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,0,1,1},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,0,1,1},	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,0,0,1},
	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,0,1,1},	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,0,1,1},
	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,0,1,1},	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,0,1,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,0,1,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,0,1,1},
	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,0,1,1},
	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,0,1,1},
	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,0,1,1},
	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,0,1,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,0,1,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,0,0,1},
	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,0,0,1},
	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,0,0,1},
	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,0,0,1},
	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,0,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,0,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,0,0,1},	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,0,0,1},
	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,0,0,1},	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,0,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,0,0,1},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,0,0,1},
	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,0,0,1},	{11,1,1,1,0,1,1,0,0,1,0,1,0,0,0,0,0,0},	{11,1,1,0,1,1,1,1,1,0,1,1,0,0,0,0,0,0},	{11,1,1,0,1,1,1,1,0,0,0,1,0,0,0,0,0,0},	{10,1,1,0,1,1,1,0,0,1,1,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,0,0,0},
	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,0,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,0,0,1},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,1,0,1},
	{11,1,1,1,0,1,1,0,0,1,1,1,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,1,0,1},	{11,1,1,1,0,1,1,0,0,1,1,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,1,0,1},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,1,0,0},	{8,1,1,0,1,0,1,1,0,0,0,0,0,0,0,0,0,0},	{11,1,1,0,1,1,1,1,0,0,1,0,0,0,0,0,0,0},	{9,1,1,0,0,1,0,1,0,0,0,0,0,0,0,0,0,0},
	{10,1,1,0,1,1,1,0,0,0,1,0,0,0,0,0,0,0},	{9,1,1,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0},	{10,1,1,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0},	{9,1,1,0,0,1,0,1,0,1,0,0,0,0,0,0,0,0},	{9,1,1,0,1,1,1,1,1,1,0,0,0,0,0,0,0,0},	{10,1,1,0,1,1,1,0,1,0,0,0,0,0,0,0,0,0},	{7,1,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0},	{8,1,1,1,0,1,1,0,1,0,0,0,0,0,0,0,0,0},
	{8,1,1,0,0,1,0,1,1,0,0,0,0,0,0,0,0,0},	{7,1,1,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0},	{7,1,1,0,1,1,0,1,0,0,0,0,0,0,0,0,0,0},	{7,1,1,1,0,1,1,1,0,0,0,0,0,0,0,0,0,0},	{6,1,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0},	{6,1,1,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0},	{5,1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0},	{3,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
	{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},	{3,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0},	{5,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0},	{6,1,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0},	{6,1,1,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0},	{6,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},	{7,1,1,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0},	{7,1,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0},
	{7,1,1,0,1,0,1,0,0,0,0,0,0,0,0,0,0,0},	{8,1,1,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0},	{8,1,1,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0},	{8,1,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,1,0,1},	{9,1,1,1,0,1,1,0,0,0,0,0,0,0,0,0,0,0},	{11,1,1,0,1,1,1,1,1,0,1,0,0,0,0,0,0,0},	{11,1,1,0,1,1,1,1,0,1,0,1,0,0,0,0,0,0},
	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,1,0,1},	{10,1,1,0,1,1,1,1,0,1,1,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,1,0,1},	{11,1,1,0,1,1,1,1,0,0,1,1,0,0,0,0,0,0},	{9,1,1,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0},	{11,1,1,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0},	{10,1,1,0,1,1,1,0,1,1,1,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,1,0,1},
	{16,1,1,0,1,0,1,1,1,0,0,1,1,0,1,0,0,0},	{10,1,1,0,1,1,1,0,0,1,0,0,0,0,0,0,0,0},	{10,1,1,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0},	{11,1,1,0,1,1,1,0,1,0,1,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,1,0,0},	{11,1,1,0,1,1,1,0,1,1,0,0,0,0,0,0,0,0},
	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,1,0,0},	{11,1,1,1,0,1,1,0,0,1,0,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,0,0,0},	{11,1,1,0,1,1,1,1,0,1,0,0,0,0,0,0,0,0},	{11,1,1,0,1,1,1,0,1,0,1,1,0,0,0,0,0,0},
	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,0,0,0},	{11,1,1,0,1,1,1,0,1,1,0,1,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,0,0,0},
	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,0,0,0,0},
	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,0,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,0,0,0},
	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,0,0,0},
	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,0,0,0},
	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,0,0,0},
	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,0,1,0},
	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,0,1,0},
	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,0,1,0},
	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,0,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,0,1,0,0},
	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,0,1,0},
	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,0,1,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,0,1,0},
	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,0,0,0},
	{16,1,1,0,1,0,1,1,1,0,0,1,1,0,0,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,1,1,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,1,1,0},
	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,1,1,0},
	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,0,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,1,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,0,1,1,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,1,1,0,0},
	{16,1,1,0,1,0,1,1,1,0,0,1,0,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,0,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,0,0,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,1,1,0},
	{17,1,1,0,1,0,1,1,1,0,1,1,1,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,0,0,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,1,1,0,0},
	{16,1,1,0,1,0,1,1,1,0,0,1,0,1,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,0,0,1,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,1,1,0,0},
	{16,1,1,0,1,0,1,1,1,0,0,0,1,1,1,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,1,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,1,1,0,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,1,1,0,0},	{16,1,1,0,1,0,1,1,1,0,0,0,0,0,1,0,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,1,1,0},
	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,1,1,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,1,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,1,1,0},
	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,1,0,0,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,1,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,1,1,0},
	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,1,0,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,1,1,0},
	{17,1,1,0,1,0,1,1,1,1,0,0,1,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,1,0,0,0,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,1,1,1,1,0,0},
	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,1,0,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,1,0,1,1,0},
	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,1,0,1,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,1,1,0,0},
	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,1,0,0,0,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,1,1,0,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,1,1,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,1,0,1,0,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,0,1,1,1,0},	{17,1,1,0,1,0,1,1,1,0,0,1,1,0,1,1,0,0},
	};

	static unsigned char ac[512][18]={
	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,1,0,1,0},
	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,1,1,1,0},
	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,1,1,1,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,1,1,1,1,0,0},
	{15,1,0,0,0,0,1,0,1,1,1,1,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,1,1,1,0},
	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,1,1,1,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,1,1,1,0,0},
	{15,1,0,0,0,0,1,0,1,1,0,0,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,1,1,1,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,1,1,1,0,0},
	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,1,1,1,0},	{15,1,0,0,0,0,1,0,0,0,1,0,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,1,1,1,0,0},
	{15,1,0,0,0,0,1,0,1,0,1,0,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,1,1,1,0},
	{15,1,0,0,0,0,1,0,0,0,0,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,1,1,1,0,0},
	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,1,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,1,1,1,0},	{15,1,0,0,0,0,1,1,1,0,0,0,1,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,0,1,1,0},
	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,0,1,1,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,0,1,1,0,0},
	{15,1,0,0,0,0,1,0,1,0,0,1,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,0,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,0,1,1,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,0,1,1,0,0},
	{15,1,0,0,0,0,1,0,0,1,0,0,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,0,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,0,1,1,0},
	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,0,1,1,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,0,1,1,0},
	{15,1,0,0,0,0,1,0,0,0,1,0,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,0,1,1,0,0},
	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,0,1,1,0},	{15,1,0,0,0,0,1,0,0,0,0,1,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,1,0,1,1,0,0},
	{15,1,0,0,0,0,1,0,1,1,1,1,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,0,1,1,0},
	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,0,1,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,0,0,1,0},
	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,0,0,1,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,0,0,1,0,0},
	{15,1,0,0,0,0,1,0,0,1,0,1,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,1,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,0,0,1,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,1,0,0,1,0,0},
	{15,1,0,0,0,0,1,0,1,0,0,0,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,1,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,0,0,1,0},
	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,0,0,1,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,0,0,1,0},	{15,1,0,0,0,0,1,0,0,0,1,0,0,0,0,1,0,0},
	{15,1,0,0,0,0,1,0,0,1,1,0,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,0,0,1,0},
	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,0,0,1,0},	{15,1,0,0,0,0,1,0,0,0,0,1,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,0,0,0,1,0,0},
	{15,1,0,0,0,0,1,0,1,1,0,1,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,0,0,1,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,0,0,1,0},
	{15,1,0,0,0,0,1,1,1,0,0,0,1,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,1,0,1,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,0,1,0,1,0,0},
	{15,1,0,0,0,0,1,0,1,1,0,1,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,1,0,1,0},
	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,1,0,1,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,0,1,0,1,0,0},
	{15,1,0,0,0,0,1,0,0,1,1,0,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,1,0,1,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,1,0,1,0},
	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,1,0,1,0},	{15,1,0,0,0,0,1,0,0,0,1,0,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,1,1,0,1,0,0},
	{15,1,0,0,0,0,1,0,1,0,0,0,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,1,0,1,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,1,0,1,0},	{15,1,0,0,0,0,1,0,0,0,0,1,1,1,0,1,0,0},
	{6,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,1,1,0,1,0,0},	{7,1,0,0,1,1,1,0,0,0,0,0,0,0,0,0,0,0},	{6,1,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0},	{5,1,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0},	{4,1,0,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0},	{2,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},
	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,1,0,0,0},	{1,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0},	{4,1,0,1,1,0,0,0,0,0,0,0,0,0,0,0,0,0},	{6,1,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0,0},	{6,1,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0,0},	{7,1,0,0,1,1,1,1,0,0,0,0,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,1,0,1,0},	{8,1,0,0,0,0,1,1,0,0,0,0,0,0,0,0,0,0},
	{15,1,0,0,0,0,1,0,1,0,1,1,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,1,0,0,0,0},
	{15,1,0,0,0,0,1,0,0,1,0,0,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,1,0,0,0},
	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,1,0,0,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,1,0,0,0},
	{15,1,0,0,0,0,1,0,0,0,1,0,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,1,0,0,0,0},
	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,1,0,0,0,0},
	{15,1,0,0,0,0,1,0,1,1,1,1,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,1,0,0,0},
	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,1,0,0,0},	{15,1,0,0,0,0,1,1,1,0,0,0,1,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,0,0,0,0,0},
	{15,1,0,0,0,0,1,0,0,1,1,1,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,0,0,0,0},
	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,0,0,0,0,0},
	{15,1,0,0,0,0,1,0,1,1,0,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,0,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,0,0,0,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,0,0,0,0,0},
	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,0,0,0,0,0},
	{15,1,0,0,0,0,1,0,1,0,1,0,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,0,0,0,0},
	{15,1,0,0,0,0,1,0,0,0,0,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,0,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,0,0,0,0,0},
	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,0,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,0,1,0,0},
	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,0,1,0,0,0},
	{15,1,0,0,0,0,1,0,1,1,0,1,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,1,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,0,1,0,0},
	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,1,0,1,0,0,0}, {15,1,0,0,0,0,1,0,1,0,1,0,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,1,0,1,0,0,0},
	{15,1,0,0,0,0,1,0,0,1,1,0,1,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,1,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,0,1,0,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,0,1,0,0},
	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,0,0,1,0,0,0},
	{15,1,0,0,0,0,1,0,1,0,0,0,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,0,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,0,0,1,0,0,0},
	{15,1,0,0,0,0,1,0,0,1,0,1,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,0,0,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,0,1,0,0},
	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,0,1,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,0,1,0,0},	{15,1,0,0,0,0,1,1,1,0,0,0,1,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,0,1,1,0,0},
	{16,1,0,0,0,0,1,1,1,1,1,0,1,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,0,1,1,0,0,0},
	{15,1,0,0,0,0,1,0,0,1,0,1,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,0,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,0,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,0,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,0,1,1,0,0,0},
	{15,1,0,0,0,0,1,0,1,0,0,0,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,0,0,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,0,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,0,1,1,0,0},
	{16,1,0,0,0,0,1,1,1,1,0,1,0,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,0,1,1,0,0},	{15,1,0,0,0,0,1,1,1,0,0,0,0,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,0,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,0,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,0,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,0,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,1,0,1,1,1,0,0,0},
	{15,1,0,0,0,0,1,0,0,1,1,0,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,0,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,0,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,0,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,0,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,0,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,0,0,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,0,1,1,1,0,0},
	{16,1,0,0,0,0,1,1,1,0,1,0,0,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,0,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,0,0,1,1,1,1,0,0},	{15,1,0,0,0,0,1,0,0,0,0,1,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,0,1,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,0,0,1,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,0,1,1,1,1,1,0,0,0},
	{15,1,0,0,0,0,1,0,1,1,1,1,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,1,1,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,1,1,0,1,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,1,1,1,1,1,0,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,1,1,0,0,0},	{15,1,0,0,0,0,1,0,0,0,1,1,1,1,0,0,0,0},
	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,1,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,1,1,0,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,1,0,1,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,1,1,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,1,1,0,0},
	{16,1,0,0,0,0,1,1,1,0,0,1,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,0,1,0,0,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,1,1,0,0},	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,1,0,0,0},	{16,1,0,0,0,0,1,1,1,0,0,0,1,1,0,1,0,0}
	};
	
	static unsigned char run[16][10]={
		{1,1,0,0,0,0,0,0,0,0},		{2,0,1,0,0,0,0,0,0,0},
		{3,0,0,1,0,0,0,0,0,0},		{4,0,0,0,1,0,0,0,0,0},
		{5,0,0,0,0,1,0,0,0,0},		{9,0,0,0,0,0,1,0,1,0},
		{9,0,0,0,0,0,1,0,1,1},		{9,0,0,0,0,0,1,1,0,1},
		{9,0,0,0,0,0,1,1,1,1},		{8,0,0,0,0,0,0,1,1,0},
		{8,0,0,0,0,0,0,0,1,0},		{8,0,0,0,0,0,1,0,0,0},
		{8,0,0,0,0,0,0,0,0,0},		{8,0,0,0,0,0,0,1,0,0},
		{9,0,0,0,0,0,1,1,1,0},		{9,0,0,0,0,0,1,1,0,0},
	};


  n=512;
  bbaop.dc_table_size = n;
  bbaop.dc_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	m=dc[i][0];
	bbaop.dc_table[i] = (int*) malloc((m+1)*sizeof(int));
	bbaop.dc_table[i][0] = m;
	for (j=1; j<=m; j++) {
		bbaop.dc_table[i][j]=dc[i][j];
	}
  }

  n=16;
  bbaop.runlength_table_size = n;
  bbaop.runlength_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	m=run[i][0];
	bbaop.runlength_table[i] = (int*) malloc((m+1)*sizeof(int));
	bbaop.runlength_table[i][0] = m;
	for (j=1; j<=m; j++) {
		bbaop.runlength_table[i][j]=run[i][j];
	}
  }
  

  
  n=512;
  bbaop.ac_table_size = n;
  bbaop.ac_table = (int**) malloc(n*sizeof(int*));
  for (i=0; i<n; i++) {
	m=ac[i][0];
	bbaop.ac_table[i] = (int*) malloc((m+1)*sizeof(int));
	bbaop.ac_table[i][0] = m;
	for (j=1; j<=m; j++) {
		bbaop.ac_table[i][j]=ac[i][j];
	}
  }

}


/////////////////: morphing 

bool BBAEncoder::ReadAMorphMask(unsigned char *buff, CMorph *ms, int *ct){
	
	int count=*ct;
	CMorphMask *mml=&(ms->morphMask);
	int i;

	////////// NR Weights /////////////
	ms->NumberWeights=mygetint(buff, &count);


	ms->AllocMorph();
	ms->AllocPrevMorph();

	for (i=0;i<ms->NumberWeights;i++) {
			mml->isWeightChanged[i]=(bool)mygetint(buff, &count);
		}

	*ct=count;
	return true;
}


bool BBAEncoder::ReadAMorphData(unsigned char *buff, CMorph *ms, int *ct){
	
	int count=*ct;
	int i;
	CMorphMask *mml=&(ms->morphMask);
	CMorphData *mdl=&(ms->morphData);

	////////// translation /////////////

	if (ms->isWeightsChanged()){
		for (i=0;i<ms->NumberWeights;i++){
			if (mml->isWeightChanged[i])
				mdl->Weights[i]=mygetint(buff, &count);
		}
	}

	*ct=count;
	return true;

}

bool BBAEncoder::encode_morph_mask(CMorph *ms){
	
	CMorphMask *mm=&ms->morphMask;
	int i;
	bool tochange=false;

	tochange=ms->isWeightsChanged();
	if (tochange){
		for (i=0;i<ms->NumberWeights;i++){
			PutBits(1,bbaop.marker_bit);
			PutBits(1,mm->isWeightChanged[i]);
		}
	}
	return true;
}
