/*======================================================================
PROJECT: "Strategien zur Optimierung des Depth Shadow Mapping Verfahrens
				 und deren Implementierung in die MPEG-4 AFX Referenzsoftware" 
Version: 1.0.0 (12/12/2005)
========================================================================
Author:  Matthias Frank, Technische Universitt Ilmenau
Contact: matthias.frank@stud.tu-ilmenau.de
				 matthiasfrank@gmx.net
=======================================================================
Parts of ShadowRendering were developed using Nvidia's
"simple shadow example" by Cass Everitt. It also uses several 
glut-helper libraries by NVidia.
========================================================================*/

/*--------------------------------------------------------------------------
ShadowRendering.cpp
- based on first version of ShadowRendering.cpp by M.Albrecht
- using glut and other Helper libraries by NVidia SDK.
- using SGManager by M.Albrecht.

Helge Drumm, University Of Technology Ilmenau, Germany, 11-2005
---------------------------------------------------------------------------*/



/*--------------------------------------------------------------------------
Extensions - added by M.Frank 11-2005 
---------------------------------------------------------------------------*/
#define GLH_EXT_SINGLE_FILE

#if defined(WIN32)
#define REQUIRED_EXTENSIONS "GL_ARB_multitexture " \
                            "GL_ARB_depth_texture " \
                            "GL_ARB_shadow " \
														"GL_NV_register_combiners " \
                            "WGL_ARB_pbuffer " \
                            "WGL_ARB_pixel_format " \
														//"GL_ARB_texture_rectangle "
						
#elif defined(UNIX)
#define REQUIRED_EXTENSIONS "GL_ARB_multitexture " \
                            "GL_ARB_depth_texture " \
                            "GL_ARB_shadow " \
														"GL_NV_register_combiners " \
                            "GLX_SGIX_pbuffer " \
                            "GLX_SGIX_fbconfig "

// hier fehlt noch die Untersttzung fr render to Texture
// als ersatz fr die wgl win variant

#elif defined(MACOS)
#define REQUIRED_EXTENSIONS "GL_ARB_multitexture " \
                            "GL_ARB_depth_texture " \
                            "GL_ARB_shadow "
														"GL_NV_register_combiners " \
														"GL_ARB_texture_cube_map "
#endif
//--------------------------------------------------------------------------

//old includes by Marco Albrecht
#include "OpenGl/shadow/ShadowRendering.h"
#include "opengl/base/OpenGLVisualRenderer.h"
//#include "glstuff.h"

/*--------------------------------------------------------------------------
Includes - added by M.Frank 11-2005 
--------------------------------------------------------------------------*/

#include <iostream>
#include <stdlib.h>
#include <vector>
#include <map>

#include <OpenGL/shadow/inc/glh/glh_extensions.h>
#include <OpenGL/shadow/inc/glh/glh_obs.h>
#include <gl/glu.h>
#include <OpenGL/shadow/inc/glh/glh_glut.h>
#include <OpenGL/shadow/inc/glh/glh_array.h>
#include <OpenGL/shadow/inc/glh/glh_convenience.h>

//zur Initialisierung der GLSL Shader
#include <OpenGL/shadow/inc/shared/data_path.h>
//#include <OpenGL/shadow/inc/shared/objload.h>
#include <OpenGL/shadow/inc/shared/read_text_file.h>
#include <OpenGL/shadow/inc/shared/pbuffer.h>
#include <OpenGL/shadow/inc/shared/array_texture.h>
//#include <OpenGL/shadow/inc/shared/load_cubemap.h>
//#include <OpenGL/shadow/inc/shared/nv_png.h>
#include <OpenGL/shadow/inc/shared/quitapp.h>

using namespace std;
using namespace glh;

//--------------------------------------------------------------------------

//old defines by Marco Albrecht
/*
#ifdef _WIN32
# define GET_PROC_ADDRESS(x) wglGetProcAddress(x)
#else
# ifdef macintosh
#  ifndef GL_MAC_GET_PROC_ADDRESS_NV
#   define GL_MAC_GET_PROC_ADDRESS_NV 0x84FC
#  endif
#  define GET_PROC_ADDRESS(x) macGetProcAddress(x)
void *(*macGetProcAddress)(char *string);
# else
#  define GET_PROC_ADDRESS(x) glXGetProcAddress(x)
# endif
#endif
*/
/*--------------------------------------------------------------------------
Shader Handles - added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
GLhandleARB diffSpotLight = 0;
GLhandleARB specSpotLight = 0;
GLhandleARB diffPointLight = 0;
GLhandleARB specPointLight = 0;
GLhandleARB diffDirectionalLight = 0;
GLhandleARB specDirectionalLight = 0;
GLhandleARB ambientLight = 0;
GLhandleARB softshadows = 0;
GLhandleARB composeSceneMap = 0;
GLhandleARB composeDiffLights = 0;
GLhandleARB composeSpecLights = 0;
GLhandleARB composeScene = 0;
GLhandleARB curProgram = 0;
// data for obj-format load
unsigned int nverts = 0;
unsigned int nindices = 0;
unsigned int *indices = NULL;
float *vertexdata = NULL;
float *normaldata = NULL;
float *tangendata = NULL;
float *binormdata = NULL;
float *texcoords = NULL;

/*--------------------------------------------------------------------------
Objects - added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
//needed by initpBuffers()
PBuffer pbuffer_depthmap("rgb depth=24");
PBuffer pbuffer_fog("rgb depth=24");
PBuffer pbuffer_rgb("rgb depth=24");
GLenum depth_format;



//--------------------------------------------------------------------------

ShadowRendering::ShadowRendering(int Height, int Width, double AspectRatio)
{
  //old variables by Marco Albrecht
	missingRequiredExtension = 0;	
  vsync = 1;
  requestedDepthMapSize = 1024;
  depthMapSize = 1024;
  requestedDepthMapRectWidth = 350;
  requestedDepthMapRectHeight = 300;
  depthMapRectWidth = 350;
  depthMapRectHeight = 300;
  depthMapPrecision = GL_UNSIGNED_BYTE;
  depthMapFormat = GL_LUMINANCE;
  depthMapInternalFormat = GL_INTENSITY8;
  hwDepthMapPrecision = GL_UNSIGNED_SHORT;
  hwDepthMapInternalFormat = GL_DEPTH_COMPONENT16_SGIX;
  hwDepthMapFiltering = GL_LINEAR;
  useCopyTexImage = 1;
  useTextureRectangle = 0;
  mipmapShadowMap = 0;
  depthBias8 = 2;
  depthBias16 = 6;
  depthBias24 = 8;
  
  slopeScale = 4.0;
  textureLodBias = 0.0;
  
  movingLight = 0;			// nur zum herunterschalten der Depthmap Auflsung -> wird nicht mehr gebraucht!
  quickLightMove = 1;		// nur zum herunterschalten der Depthmap Auflsung -> wird nicht mehr gebraucht!
  wireFrame = 0;
  
  depthMapBackFaceCulling = 0;
  winWidth = Width;
  winHeight = Height;
  winAspectRatio = AspectRatio;
  
  needMatrixUpdate = 1;
  needTitleUpdate = 1;
  needDepthMapUpdate = 1;
  rangeMap = 0;
  fullscreen = 0;
  forceEnvCombine = 0;
  useDisplayLists = 1;

  showLightViewFrustum = 0;  
  showDepthMapMSBs = 1;
  
  useStencil = 0;
  useDepth24 = 0;
  
  drawFront = 0;
  
  hasRegisterCombiners = 0;
  hasTextureEnvCombine = 0;
  hasShadow = 0;
  hasSwapControl = 0;
  hasDepthTexture = 0;
  hasShadowMapSupport = 0;
  useShadowMapSupport = 0;
  hasSeparateSpecularColor = 0;
  hasTextureRectangle = 0;
  hasTextureBorderClamp = 0;
  hasTextureEdgeClamp = 0;
  hasOpenGL12 = 0;
  
  maxRectangleTextureSize = 0;
  
  lightNear = 1.0;
  lightFar = 400.0;			// das ist wohl eventuell etwas zu viel des guten?! oder?!
  
  globalAmbientIntensity[0] = 0.01; 
  globalAmbientIntensity[1] = 0.01;
  globalAmbientIntensity[2] = 0.01;
  globalAmbientIntensity[3] = 1.0;
  globalAmbient[0] = 0.01;
  globalAmbient[1] = 0.01;
  globalAmbient[2] = 0.01;
  globalAmbient[3] = 1.0;

  lightDimColor[0] = LIGHT_DIMMING;  
  lightDimColor[1] = LIGHT_DIMMING;
  lightDimColor[2] = LIGHT_DIMMING;
  lightDimColor[3] = 1.0;

  zero[0] = 0.0; 
  zero[1] = 0.0;
  zero[2] = 0.0;
  zero[3] = 1.0;

	maximal[0] = 1.0; 
  maximal[1] = 1.0;
  maximal[2] = 1.0;
  maximal[3] = 1.0;

  lightColor[0] = 0.0; 
  lightColor[1] = 0.0;
  lightColor[2] = 0.0;
  lightColor[3] = 1.0;

  ambLightColor[0] = 0.0;
  ambLightColor[1] = 0.0;
  ambLightColor[2] = 0.0;
  ambLightColor[3] = 1.0;

  bgLightColor[0] = 0.2; 
  bgLightColor[1] = 0.2;
  bgLightColor[2] = 0.2;
  bgLightColor[3] = 1.0;

  Smatrix[0] = 0.5;
  Smatrix[1] = 0.0;
  Smatrix[2] = 0.0;
  Smatrix[3] = 0.0;
  
  Smatrix[4] = 0.0;
  Smatrix[5] = 0.5;
  Smatrix[6] = 0.0;
  Smatrix[7] = 0.0;

  Smatrix[8] = 0.0;
  Smatrix[9] = 0.0;
  Smatrix[10] = 0.5;
  Smatrix[11] = 0.0;

  Smatrix[12] = 0.5;
  Smatrix[13] = 0.5;
  Smatrix[14] = 0.5;
  Smatrix[15] = 1.0;

  RSmatrix[0] = 0.0;
  RSmatrix[1] = 0.0;
  RSmatrix[2] = 0.0;
  RSmatrix[3] = 0.0;
  
  RSmatrix[4] = 0.0;
  RSmatrix[5] = 0.0;
  RSmatrix[6] = 0.0;
  RSmatrix[7] = 0.0;

  RSmatrix[8] = 0.5;
  RSmatrix[9] = 128.0;
  RSmatrix[10] = 0.0;
  RSmatrix[11] = 0.0;

  RSmatrix[12] = 0.5;
  RSmatrix[13] = 128.0;
  RSmatrix[14] = 0.0;
  RSmatrix[15] = 1.0;

  light_num_max = 0;		//Summe aller Directional, Spot und Pointlights
  
  softpass_num = 5;			

  itv[0] = 0.2;
  itv[1] = 0.2;
  itv[2] = 0.2;
  itv[3] = 0.2;
  itv[4] = 0.2;

  penumbra = 0.0;

  jitter[0] = -penumbra;
  jitter[1] = penumbra;
  jitter[2] = 0;
  jitter[3] = penumbra/3;
  jitter[4] = penumbra/2;
	
	// NEUE VARIABLEN - added by M.Frank 11-2005-----------------------------------------
	lightConstantAttenuation = 0.0;
	lightLinearAttenuation = 0.0;
	lightQuadraticAttenuation = 0.0;
	lightSpotCutoff = 0.0;
	lightSpotExponent = 0.0;
	lightRadius = 100.0;

	amb_light_num_max = 0;

	ambLightColorSum[0] = 0.0;
  ambLightColorSum[1] = 0.0;
  ambLightColorSum[2] = 0.0;
  ambLightColorSum[3] = 1.0;
	
	fogColor[0] = 1.0;
	fogColor[1] = 1.0;
	fogColor[2] = 1.0;
	fogColor[3] = 1.0;

	fogVisRange = 0;
	fogType = false; // false = linear
	fogStart = 1.0;
	fogEnd = 300.0;
	fogDensity = 1.0;

	hasOpenGL20 = 0;

	white[0] = 1.0; 
  white[1] = 1.0;
  white[2] = 1.0;
  white[3] = 1.0;
	
	counterEnabled = true;
	frames = 0;

	softshadowsEnabled = true;
	perpixelLightingEnabled = true;

	static const GLint hwBorderColor[4] = { 256, 0, 0, 0 };

	TEX_SIZE = 1024;
	// -----------------------------------------------------------------------------------
}


ShadowRendering::~ShadowRendering()
{

}


GLboolean ShadowRendering::invertMatrix(GLdouble *out, const GLdouble *m)
{

#define SWAP_ROWS(a, b) { GLdouble *_tmp = a; (a)=(b); (b)=_tmp; }
#define MAT(m,r,c) (m)[(c)*4+(r)]

 GLdouble wtmp[4][8];
 GLdouble m0, m1, m2, m3, s;
 GLdouble *r0, *r1, *r2, *r3;

 r0 = wtmp[0], r1 = wtmp[1], r2 = wtmp[2], r3 = wtmp[3];

 r0[0] = MAT(m,0,0), r0[1] = MAT(m,0,1),
 r0[2] = MAT(m,0,2), r0[3] = MAT(m,0,3),
 r0[4] = 1.0, r0[5] = r0[6] = r0[7] = 0.0,

 r1[0] = MAT(m,1,0), r1[1] = MAT(m,1,1),
 r1[2] = MAT(m,1,2), r1[3] = MAT(m,1,3),
 r1[5] = 1.0, r1[4] = r1[6] = r1[7] = 0.0,

 r2[0] = MAT(m,2,0), r2[1] = MAT(m,2,1),
 r2[2] = MAT(m,2,2), r2[3] = MAT(m,2,3),
 r2[6] = 1.0, r2[4] = r2[5] = r2[7] = 0.0,

 r3[0] = MAT(m,3,0), r3[1] = MAT(m,3,1),
 r3[2] = MAT(m,3,2), r3[3] = MAT(m,3,3),
 r3[7] = 1.0, r3[4] = r3[5] = r3[6] = 0.0;

 /* choose pivot - or die */
 if (fabs(r3[0])>fabs(r2[0])) SWAP_ROWS(r3, r2);
 if (fabs(r2[0])>fabs(r1[0])) SWAP_ROWS(r2, r1);
 if (fabs(r1[0])>fabs(r0[0])) SWAP_ROWS(r1, r0);
 if (0.0 == r0[0]) {
  return GL_FALSE;
 }

 /* eliminate first variable     */
 m1 = r1[0]/r0[0]; m2 = r2[0]/r0[0]; m3 = r3[0]/r0[0];
 s = r0[1]; r1[1] -= m1 * s; r2[1] -= m2 * s; r3[1] -= m3 * s;
 s = r0[2]; r1[2] -= m1 * s; r2[2] -= m2 * s; r3[2] -= m3 * s;
 s = r0[3]; r1[3] -= m1 * s; r2[3] -= m2 * s; r3[3] -= m3 * s;
 s = r0[4];
 if (s != 0.0) { r1[4] -= m1 * s; r2[4] -= m2 * s; r3[4] -= m3 * s; }
 s = r0[5];
 if (s != 0.0) { r1[5] -= m1 * s; r2[5] -= m2 * s; r3[5] -= m3 * s; }
 s = r0[6];
 if (s != 0.0) { r1[6] -= m1 * s; r2[6] -= m2 * s; r3[6] -= m3 * s; }
 s = r0[7];
 if (s != 0.0) { r1[7] -= m1 * s; r2[7] -= m2 * s; r3[7] -= m3 * s; }

 /* choose pivot - or die */
 if (fabs(r3[1])>fabs(r2[1])) SWAP_ROWS(r3, r2);
 if (fabs(r2[1])>fabs(r1[1])) SWAP_ROWS(r2, r1);
 if (0.0 == r1[1]) {
  return GL_FALSE;
 }

 /* eliminate second variable */
 m2 = r2[1]/r1[1]; m3 = r3[1]/r1[1];
 r2[2] -= m2 * r1[2]; r3[2] -= m3 * r1[2];
 r2[3] -= m2 * r1[3]; r3[3] -= m3 * r1[3];
 s = r1[4]; if (0.0 != s) { r2[4] -= m2 * s; r3[4] -= m3 * s; }
 s = r1[5]; if (0.0 != s) { r2[5] -= m2 * s; r3[5] -= m3 * s; }
 s = r1[6]; if (0.0 != s) { r2[6] -= m2 * s; r3[6] -= m3 * s; }
 s = r1[7]; if (0.0 != s) { r2[7] -= m2 * s; r3[7] -= m3 * s; }

 /* choose pivot - or die */
 if (fabs(r3[2])>fabs(r2[2])) SWAP_ROWS(r3, r2);
 if (0.0 == r2[2]) {
  return GL_FALSE;
 }

 /* eliminate third variable */
 m3 = r3[2]/r2[2];
 r3[3] -= m3 * r2[3], r3[4] -= m3 * r2[4],
 r3[5] -= m3 * r2[5], r3[6] -= m3 * r2[6],
 r3[7] -= m3 * r2[7];

 /* last check */
 if (0.0 == r3[3]) {
  return GL_FALSE;
 }

 s = 1.0/r3[3];              /* now back substitute row 3 */
 r3[4] *= s; r3[5] *= s; r3[6] *= s; r3[7] *= s;

 m2 = r2[3];                 /* now back substitute row 2 */
 s  = 1.0/r2[2];
 r2[4] = s * (r2[4] - r3[4] * m2), r2[5] = s * (r2[5] - r3[5] * m2),
 r2[6] = s * (r2[6] - r3[6] * m2), r2[7] = s * (r2[7] - r3[7] * m2);
 m1 = r1[3];
 r1[4] -= r3[4] * m1, r1[5] -= r3[5] * m1,
 r1[6] -= r3[6] * m1, r1[7] -= r3[7] * m1;
 m0 = r0[3];
 r0[4] -= r3[4] * m0, r0[5] -= r3[5] * m0,
 r0[6] -= r3[6] * m0, r0[7] -= r3[7] * m0;

 m1 = r1[2];                 /* now back substitute row 1 */
 s  = 1.0/r1[1];
 r1[4] = s * (r1[4] - r2[4] * m1), r1[5] = s * (r1[5] - r2[5] * m1),
 r1[6] = s * (r1[6] - r2[6] * m1), r1[7] = s * (r1[7] - r2[7] * m1);
 m0 = r0[2];
 r0[4] -= r2[4] * m0, r0[5] -= r2[5] * m0,
 r0[6] -= r2[6] * m0, r0[7] -= r2[7] * m0;

 m0 = r0[1];                 /* now back substitute row 0 */
 s  = 1.0/r0[0];
 r0[4] = s * (r0[4] - r1[4] * m0), r0[5] = s * (r0[5] - r1[5] * m0),
 r0[6] = s * (r0[6] - r1[6] * m0), r0[7] = s * (r0[7] - r1[7] * m0);

 MAT(out,0,0) = r0[4]; MAT(out,0,1) = r0[5],
 MAT(out,0,2) = r0[6]; MAT(out,0,3) = r0[7],
 MAT(out,1,0) = r1[4]; MAT(out,1,1) = r1[5],
 MAT(out,1,2) = r1[6]; MAT(out,1,3) = r1[7],
 MAT(out,2,0) = r2[4]; MAT(out,2,1) = r2[5],
 MAT(out,2,2) = r2[6]; MAT(out,2,3) = r2[7],
 MAT(out,3,0) = r3[4]; MAT(out,3,1) = r3[5],
 MAT(out,3,2) = r3[6]; MAT(out,3,3) = r3[7]; 

 return GL_TRUE;

#undef MAT
#undef SWAP_ROWS

}

void ShadowRendering::transposeMatrix(GLdouble dst[16], GLdouble src[16])
{
  dst[0] = src[0];
  dst[1] = src[4];
  dst[2] = src[8];
  dst[3] = src[12];

  dst[4] = src[1];
  dst[5] = src[5];
  dst[6] = src[9];
  dst[7] = src[13];

  dst[8] = src[2];
  dst[9] = src[6];
  dst[10] = src[10];
  dst[11] = src[14];

  dst[12] = src[3];
  dst[13] = src[7];
  dst[14] = src[11];
  dst[15] = src[15];

}

void ShadowRendering::addMatrices(GLdouble dst[16], GLdouble a[16], GLdouble b[16])
{
  dst[0] = a[0] + b[0];
  dst[1] = a[1] + b[1];
  dst[2] = a[2] + b[2];
  dst[3] = a[3] + b[3];

  dst[4] = a[4] + b[4];
  dst[5] = a[5] + b[5];
  dst[6] = a[6] + b[6];
  dst[7] = a[7] + b[7];

  dst[8] = a[8] + b[8];
  dst[9] = a[9] + b[9];
  dst[10] = a[10] + b[10];
  dst[11] = a[11] + b[11];

  dst[12] = a[12] + b[12];
  dst[13] = a[13] + b[13];
  dst[14] = a[14] + b[14];
  dst[15] = a[15] + b[15];
}

void ShadowRendering::multMatrices(GLdouble dst[16], const GLdouble a[16], const GLdouble b[16])
{
  int i, j;

  for (i = 0; i < 4; i++) {
    for (j = 0; j < 4; j++) {
      dst[i * 4 + j] =
        b[i * 4 + 0] * a[0 * 4 + j] +
        b[i * 4 + 1] * a[1 * 4 + j] +
        b[i * 4 + 2] * a[2 * 4 + j] +
        b[i * 4 + 3] * a[3 * 4 + j];
    }
  }
}


void ShadowRendering::multMatricesdff(GLdouble dst[16], const GLfloat a[16], const GLfloat b[16])
{
  int i, j;

  for (i = 0; i < 4; i++) {
    for (j = 0; j < 4; j++) {
      dst[i * 4 + j] =
        b[i * 4 + 0] * a[0 * 4 + j] +
        b[i * 4 + 1] * a[1 * 4 + j] +
        b[i * 4 + 2] * a[2 * 4 + j] +
        b[i * 4 + 3] * a[3 * 4 + j];
    }
  }
}


void ShadowRendering::copyMatrix(GLdouble dst[16], GLdouble src[16])
{
  int i;

  for (i=0; i<16; i++) {
    dst[i] = src[i];
  }
}


void ShadowRendering::copyMatrixf(GLdouble dst[16], GLfloat src[16])
{
  int i;

  for (i=0; i<16; i++) {
    dst[i] = src[i];
  }
}

void ShadowRendering::copyMatrixff(GLfloat dst[16], GLfloat src[16])
{
  int i;

  for (i=0; i<16; i++) {
    dst[i] = src[i];
  }
}

void ShadowRendering::copyMatrixfd(GLfloat dst[16], GLdouble src[16])
{
  int i;

  for (i=0; i<16; i++) {
    dst[i] = src[i];
  }
}


void ShadowRendering::framebuffers_alloc(void)
{
  framebuffer = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  accumbuffer = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  sbuffer[0]  = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  sbuffer[1]  = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  sbuffer[2]  = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  sbuffer[3]  = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  sbuffer[4]  = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  sbuffer[5]  = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  fbuffer[0]  = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  fbuffer[1]  = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  fbuffer[2]  = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  fbuffer[3]  = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  fbuffer[4]  = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  fbuffer[5]  = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  fbuffer[6]  = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  fbuffer[7]  = (GLubyte *)malloc(winWidth*winHeight*3*sizeof(GLubyte));
  dbuffer     = (GLfloat *)malloc(winWidth*winHeight*sizeof(GLfloat));
}


void ShadowRendering::framebuffers_realloc(int Height, int Width)
{
   framebuffer = (GLubyte*)realloc(framebuffer,Width*Height*3*sizeof(GLubyte));
   accumbuffer = (GLubyte*)realloc(accumbuffer,Width*Height*3*sizeof(GLubyte));
   sbuffer[0]  = (GLubyte *)realloc(sbuffer[0],Width*Height*3*sizeof(GLubyte));
   sbuffer[1]  = (GLubyte *)realloc(sbuffer[1],Width*Height*3*sizeof(GLubyte));
   sbuffer[2]  = (GLubyte *)realloc(sbuffer[2],Width*Height*3*sizeof(GLubyte));
   sbuffer[3]  = (GLubyte *)realloc(sbuffer[3],Width*Height*3*sizeof(GLubyte));
   sbuffer[4]  = (GLubyte *)realloc(sbuffer[4],Width*Height*3*sizeof(GLubyte));
   sbuffer[5]  = (GLubyte *)realloc(sbuffer[5],Width*Height*3*sizeof(GLubyte));
   fbuffer[0]  = (GLubyte *)realloc(fbuffer[0],Width*Height*3*sizeof(GLubyte));
   fbuffer[1]  = (GLubyte *)realloc(fbuffer[1],Width*Height*3*sizeof(GLubyte));
   fbuffer[2]  = (GLubyte *)realloc(fbuffer[2],Width*Height*3*sizeof(GLubyte));
   fbuffer[3]  = (GLubyte *)realloc(fbuffer[3],Width*Height*3*sizeof(GLubyte));
   fbuffer[4]  = (GLubyte *)realloc(fbuffer[4],Width*Height*3*sizeof(GLubyte));
   fbuffer[5]  = (GLubyte *)realloc(fbuffer[5],Width*Height*3*sizeof(GLubyte));
   fbuffer[6]  = (GLubyte *)realloc(fbuffer[6],Width*Height*3*sizeof(GLubyte));
   fbuffer[7]  = (GLubyte *)realloc(fbuffer[7],Width*Height*3*sizeof(GLubyte)); 
   dbuffer     = (GLfloat *)realloc(dbuffer,Width*Height*sizeof(GLfloat));
   
   updateDepthMapSize();


}


void ShadowRendering::framebuffers_clear(int mode)
{  if(mode==0 || mode ==1)
	{
			memset(framebuffer,0,winWidth*winHeight*3-1);
	  memset(accumbuffer,0,winWidth*winHeight*3-1);
	  memset(dbuffer,0,winWidth*winHeight*3-1);
	}
	if(mode==0 || mode ==2) {
   	  memset(sbuffer[0],0,winWidth*winHeight*3-1);
	  memset(sbuffer[1],0,winWidth*winHeight*3-1);
	  memset(sbuffer[2],0,winWidth*winHeight*3-1);
	  memset(sbuffer[3],0,winWidth*winHeight*3-1);
	  memset(sbuffer[4],0,winWidth*winHeight*3-1);
      memset(sbuffer[5],0,winWidth*winHeight*3-1);
	}
	if(mode==0 || mode ==3) {
	  memset(fbuffer[0],0,winWidth*winHeight*3-1);
	  memset(fbuffer[1],0,winWidth*winHeight*3-1);
	  memset(fbuffer[2],0,winWidth*winHeight*3-1);
	  memset(fbuffer[3],0,winWidth*winHeight*3-1);
	  memset(fbuffer[4],0,winWidth*winHeight*3-1);
	  memset(fbuffer[5],0,winWidth*winHeight*3-1);
	  memset(fbuffer[6],0,winWidth*winHeight*3-1);
	  memset(fbuffer[7],0,winWidth*winHeight*3-1);
	}
	  memset(dbuffer,0,winWidth*winHeight*3-1);
}


void ShadowRendering::framebuffers_free(void)
{
     free(framebuffer);
	 free(accumbuffer);
	 free(sbuffer[0]);
	 free(sbuffer[1]);
	 free(sbuffer[2]);
	 free(sbuffer[3]);
	 free(sbuffer[4]);
	 free(sbuffer[5]);
	 free(fbuffer[0]);
	 free(fbuffer[1]);
	 free(fbuffer[2]);
	 free(fbuffer[3]);
	 free(fbuffer[4]);
	 free(fbuffer[5]);
	 free(fbuffer[6]);
	 free(fbuffer[7]);
	 free(dbuffer);
}


void ShadowRendering::fbuffer_accum(void)
{
 unsigned short accum;
 unsigned long count_def;
 unsigned long count_max;

 count_max = winWidth*winHeight*3-1;

 switch(light_num_max){
 case 8: { for(count_def=0;count_def<count_max;count_def++){
			accum = fbuffer[0][count_def]+fbuffer[1][count_def]+fbuffer[2][count_def]
		            +fbuffer[3][count_def]+fbuffer[4][count_def]+fbuffer[5][count_def]
					+fbuffer[6][count_def]+fbuffer[7][count_def];
			if(accum<255){
			 accumbuffer[count_def] = accum;
			}else
			{
              accumbuffer[count_def] = 255;
			}
         

		 } break;}
 case 7: { for(count_def=0;count_def<count_max;count_def++){
			accum = fbuffer[0][count_def]+fbuffer[1][count_def]+fbuffer[2][count_def]
		            +fbuffer[3][count_def]+fbuffer[4][count_def]+fbuffer[5][count_def]
					+fbuffer[6][count_def];
			if(accum<255){
			 accumbuffer[count_def] = accum;
			}else
			{
              accumbuffer[count_def] = 255;
			}
         

		 } break;}
 case 6: { for(count_def=0;count_def<count_max;count_def++){
			accum = fbuffer[0][count_def]+fbuffer[1][count_def]+fbuffer[2][count_def]
		            +fbuffer[3][count_def]+fbuffer[4][count_def]+fbuffer[5][count_def];
			if(accum<255){
			 accumbuffer[count_def] = accum;
			}else
			{
              accumbuffer[count_def] = 255;
			}
         

		 } break;}
 case 5: { for(count_def=0;count_def<count_max;count_def++){
			accum = fbuffer[0][count_def]+fbuffer[1][count_def]+fbuffer[2][count_def]
		            +fbuffer[3][count_def]+fbuffer[4][count_def];
			if(accum<255){
			 accumbuffer[count_def] = accum;
			}else
			{
              accumbuffer[count_def] = 255;
			}
         

		 } break;}
 case 4: { for(count_def=0;count_def<count_max;count_def++){
			accum = fbuffer[0][count_def]+fbuffer[1][count_def]+fbuffer[2][count_def]
		            +fbuffer[3][count_def];
			if(accum<255){
			 accumbuffer[count_def] = accum;
			}else
			{
              accumbuffer[count_def] = 255;
			}
         

		 } break;}
 case 3: { for(count_def=0;count_def<count_max;count_def++){
			accum = fbuffer[0][count_def]+fbuffer[1][count_def]+fbuffer[2][count_def];
		            
			if(accum<255){
			 accumbuffer[count_def] = accum;
			}else
			{
              accumbuffer[count_def] = 255;
			}
         

		 } break;}
 case 2: { for(count_def=0;count_def<count_max;count_def++){
			accum = fbuffer[0][count_def]+fbuffer[1][count_def];
			if(accum<255){
			 accumbuffer[count_def] = accum;
			}else
			{
              accumbuffer[count_def] = 255;
			}
         

		 } break;}
 case 1: { memcpy(accumbuffer,fbuffer[0],sizeof(accumbuffer));
         

		  break;}
 
 }
 


}


void ShadowRendering::sbuffer_accum(void)
{
 unsigned short accum;
 unsigned long  count_def;
 unsigned long  count_max;

 count_max = winWidth*winHeight*3-1;
 
 switch(softpass_num) {
 case  0 :  {     break; } 
 case  1 :  { //memcpy(framebuffer,sbuffer[0],sizeof(framebuffer)); 
	          for(count_def=0;count_def<count_max;count_def++) {
				  framebuffer[count_def]=sbuffer[0][count_def];
			}break; }
 case  2 :  { for(count_def=0;count_def<count_max;count_def++) {
			     
				accum = sbuffer[0][count_def]*itv[0]+sbuffer[1][count_def]*itv[1];
                 
				 if(accum<255)
				 {
                   framebuffer[count_def]=accum;
				 }else
				 {
                   framebuffer[count_def]=255;
				 }

				}break;}
 case  3 :  { for(count_def=0;count_def<count_max;count_def++) {
			     
				accum = sbuffer[0][count_def]*itv[0]+sbuffer[1][count_def]*itv[1]+sbuffer[2][count_def]*itv[2];
                 
				 if(accum<255)
				 {
                   framebuffer[count_def]=accum;
				 }else
				 {
                   framebuffer[count_def]=255;
				 }

				}break;}
 case  4 :  { for(count_def=0;count_def<count_max;count_def++) {
			     
				accum = sbuffer[0][count_def]*itv[0]+sbuffer[1][count_def]*itv[1]+sbuffer[2][count_def]*itv[2]+sbuffer[3][count_def]*itv[3];
                 
				 if(accum<255)
				 {
                   framebuffer[count_def]=accum;
				 }else
				 {
                   framebuffer[count_def]=255;
				 }

				}break;}
 case  5 :  { for(count_def=0;count_def<count_max;count_def++) {
			     
				accum = sbuffer[0][count_def]*itv[0]+sbuffer[1][count_def]*itv[1]+sbuffer[2][count_def]*itv[2]+sbuffer[3][count_def]*itv[3]+sbuffer[4][count_def]*itv[4];
                 
				 if(accum<255)
				 {
                   framebuffer[count_def]=accum;
				 }else
				 {
                   framebuffer[count_def]=255;
				 }

				}break;}
 
 }

}



void ShadowRendering::buildFrustumMatrix(GLdouble m[16],
				                         GLdouble l, GLdouble r, GLdouble b, GLdouble t,
                                         GLdouble n, GLdouble f)
{
  m[0] = (2.0*n) / (r-l);
  m[1] = 0.0;
  m[2] = 0.0;
  m[3] = 0.0;

  m[4] = 0.0;
  m[5] = (2.0*n) / (t-b);
  m[6] = 0.0;
  m[7] = 0.0;

  m[8] = (r+l) / (r-l);
  m[9] = (t+b) / (t-b);
  m[10] = -(f+n) / (f-n);
  m[11] = -1.0;

  m[12] = 0.0;
  m[13] = 0.0;
  m[14] = -(2.0*f*n) / (f-n);
  m[15] = 0.0;
}

void ShadowRendering::buildPerspectiveMatrix(GLdouble m[16],
                                             GLdouble fovy, GLdouble aspect,
                                             GLdouble zNear, GLdouble zFar)
{
  GLdouble xmin, xmax, ymin, ymax;

  ymax = zNear * tan(fovy * M_PI / 360.0);
  ymin = -ymax;

  xmin = ymin * aspect;
  xmax = ymax * aspect;

  buildFrustumMatrix(m, xmin, xmax, ymin, ymax, zNear, zFar);
}

void ShadowRendering::buildLookAtMatrix(GLdouble m[16],
                                        GLdouble eyex, GLdouble eyey, GLdouble eyez,
                                        GLdouble centerx, GLdouble centery, GLdouble centerz,
                                        GLdouble upx, GLdouble upy, GLdouble upz)
{
   GLdouble x[3], y[3], z[3];
   GLdouble mag;

   /* Make rotation matrix */

   /* Z vector */
   z[0] = eyex - centerx;
   z[1] = eyey - centery;
   z[2] = eyez - centerz;
   mag = sqrt( z[0]*z[0] + z[1]*z[1] + z[2]*z[2] );
   if (mag) {  /* mpichler, 19950515 */
      z[0] /= mag;
      z[1] /= mag;
      z[2] /= mag;
   }

   /* Y vector */
   y[0] = upx;
   y[1] = upy;
   y[2] = upz;

   /* X vector = Y cross Z */
   x[0] =  y[1]*z[2] - y[2]*z[1];
   x[1] = -y[0]*z[2] + y[2]*z[0];
   x[2] =  y[0]*z[1] - y[1]*z[0];

   /* Recompute Y = Z cross X */
   y[0] =  z[1]*x[2] - z[2]*x[1];
   y[1] = -z[0]*x[2] + z[2]*x[0];
   y[2] =  z[0]*x[1] - z[1]*x[0];

   /* mpichler, 19950515 */
   /* cross product gives area of parallelogram, which is < 1.0 for
    * non-perpendicular unit-length vectors; so normalize x, y here
    */

   mag = sqrt( x[0]*x[0] + x[1]*x[1] + x[2]*x[2] );
   if (mag) {
      x[0] /= mag;
      x[1] /= mag;
      x[2] /= mag;
   }

   mag = sqrt( y[0]*y[0] + y[1]*y[1] + y[2]*y[2] );
   if (mag) {
      y[0] /= mag;
      y[1] /= mag;
      y[2] /= mag;
   }

#define M(row,col)  m[col*4+row]
   M(0,0) = x[0];  M(0,1) = x[1];  M(0,2) = x[2];  M(0,3) = -x[0]*eyex + -x[1]*eyey + -x[2]*eyez;
   M(1,0) = y[0];  M(1,1) = y[1];  M(1,2) = y[2];  M(1,3) = -y[0]*eyex + -y[1]*eyey + -y[2]*eyez;
   M(2,0) = z[0];  M(2,1) = z[1];  M(2,2) = z[2];  M(2,3) = -z[0]*eyex + -z[1]*eyey + -z[2]*eyez;
   M(3,0) = 0.0;   M(3,1) = 0.0;   M(3,2) = 0.0;   M(3,3) = 1.0;
#undef M
}

void ShadowRendering::swapShorts(unsigned int size, GLubyte *depthBuffer)
{
  unsigned int i;
  GLubyte tmp;

  for (i=0; i<size; i++) {
    tmp = depthBuffer[2*i + 0];
    depthBuffer[2*i + 0] = depthBuffer[2*i + 1];
    depthBuffer[2*i + 1] = tmp;
  }
}

void ShadowRendering::setLittleEndian(void)
{
  unsigned short shortValue = 258;
  unsigned char *byteVersion = (unsigned char*) &shortValue;

  if (byteVersion[1] == 1) {
    assert(byteVersion[0] == 2);
    hwLittleEndian = 1;
  } else {
    assert(byteVersion[0] == 1);
    assert(byteVersion[1] == 2);
    hwLittleEndian = 0;
  }
  littleEndian = hwLittleEndian;
}

int APIENTRY ShadowRendering::glExtensionSupported(const char *extension)
{ 
  static const GLubyte *extensions = NULL;
  const GLubyte *start;
  GLubyte *where, *terminator;

  /* Extension names should not have spaces. */
  where = (GLubyte *) strchr(extension, ' ');
  if (where || *extension == '\0')
    return 0;

  if (!extensions) {
    extensions = glGetString(GL_EXTENSIONS);
  }
  /* It takes a bit of care to be fool-proof about parsing the
     OpenGL extensions string.  Don't be fooled by sub-strings,
     etc. */
  start = extensions;
  for (;;) {
    /* If your application crashes in the strstr routine below,
       you are probably calling glutExtensionSupported without
       having a current window.  Calling glGetString without
       a current OpenGL context has unpredictable results.
       Please fix your program. */
    where = (GLubyte *) strstr((const char *) start, extension);
    if (!where)
      break;
    terminator = where + strlen(extension);
    if (where == start || *(where - 1) == ' ') {
      if (*terminator == ' ' || *terminator == '\0') {
        return 1;
      }
    }
    start = terminator;
  }
  return 0; 
  }

int ShadowRendering::supportsOneDotTwo(void)
{
  const char *version;
  int major, minor;

  version = (char *) glGetString(GL_VERSION);
  if (sscanf(version, "%d.%d", &major, &minor) == 2) {
    return major > 1 || minor >= 2;
  }
  return 0;            /* OpenGL version string malformed! */
}





/*--------------------------------------------------------------------------
****************************************************************************
--------------------------------------------------------------------------*/








/*--------------------------------------------------------------------------
Test OpenGL 2.0 Support - added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
int ShadowRendering::supportsTwoDotZero(void)
{
  const char *version;
  int major, minor;

  version = (char *) glGetString(GL_VERSION);
  if (sscanf(version, "%d.%d", &major, &minor) == 2) {
    return major >= 2 || minor >= 0;
  }
  return 0;            /* OpenGL version string malformed! */
}

/*--------------------------------------------------------------------------
Init Required OpenGL Extensions - added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::initOpenGLExtensions()
{	
	hasOpenGL12 = supportsOneDotTwo();
	hasOpenGL20 = supportsTwoDotZero();
	
	cout << "Renderer: " << glGetString(GL_RENDERER) << endl << endl;
	cout << "OpenGL Version: " << glGetString(GL_VERSION) << endl << endl;
	
	if(! glh_init_extensions(REQUIRED_EXTENSIONS))
    {
        cerr << "Necessary extensions were not supported:" << endl
             << glh_get_unsupported_extensions() << endl << endl
             << "Press <enter> to quit." << endl << endl;
        cerr << "Extensions: " << glGetString(GL_EXTENSIONS) << endl << endl;
        cerr << "Renderer: " << glGetString(GL_RENDERER) << endl << endl;
        char buff[10];
        cin.getline(buff, 10);
				missingRequiredExtension = 1;
        //exit(0);
    }		
}

/*--------------------------------------------------------------------------
Exit GLSL Shaders (reqired by initOpenGLExtensions) 
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::cleanExit(int exitval)
{
    if (diffSpotLight) 
        glDeleteObjectARB(diffSpotLight);
    if (diffPointLight) 
        glDeleteObjectARB(diffPointLight);
		if (diffDirectionalLight) 
        glDeleteObjectARB(diffDirectionalLight);
		if (specSpotLight) 
        glDeleteObjectARB(specSpotLight);
    if (specPointLight) 
        glDeleteObjectARB(specPointLight);
		if (specDirectionalLight) 
        glDeleteObjectARB(specDirectionalLight);
		if (ambientLight) 
        glDeleteObjectARB(ambientLight);
		if (composeDiffLights) 
        glDeleteObjectARB(composeDiffLights);
		if (composeSpecLights) 
        glDeleteObjectARB(composeSpecLights);
		if (composeScene) 
        glDeleteObjectARB(composeScene);

    if(exitval == 0) { exit(0); }
    else { quitapp(exitval); }
}

/*--------------------------------------------------------------------------
InfoLog for GLSL Shaders (reqired by initOpenGLExtensions) 
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::printInfoLog(GLhandleARB object)
{
    int maxLength = 0;
    glGetObjectParameterivARB(object, GL_OBJECT_INFO_LOG_LENGTH_ARB, &maxLength);

    char *infoLog = new char[maxLength];
    glGetInfoLogARB(object, maxLength, &maxLength, infoLog);

    printf("%s\n", infoLog);
}

/*--------------------------------------------------------------------------
Add GLSL Shader Procedure (reqired by initOpenGLExtensions) 
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::addShader(GLhandleARB programObject, const GLcharARB *shaderSource, GLenum shaderType, bool linkProgram)
{
    assert(programObject != 0);
    assert(shaderSource != 0);
    assert(shaderType != 0);

    GLhandleARB object = glCreateShaderObjectARB(shaderType);
    assert(object != 0);

    GLint length = (GLint)strlen(shaderSource);
    glShaderSourceARB(object, 1, &shaderSource, &length);

    // compile vertex shader object
    glCompileShaderARB(object);

    // check if shader compiled
    GLint compiled = 0;
    glGetObjectParameterivARB(object, GL_OBJECT_COMPILE_STATUS_ARB, &compiled);

    if (compiled)
    {

    // attach vertex shader to program object
    glAttachObjectARB(programObject, object);

    // delete vertex object, no longer needed
    glDeleteObjectARB(object);

		if (linkProgram)
		{
			glLinkProgramARB(programObject);
		}

    }

    GLenum err = glGetError();
    if (err != GL_NO_ERROR)
        cout << "OpenGL error: " << gluErrorString(err) << endl;
}

/*--------------------------------------------------------------------------
Init GLSL Shaders - added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::initGLSLShaders()
{
  // reqired Extensions for GLSL Shaders 
	if(!glh_init_extensions("GL_ARB_shader_objects GL_ARB_vertex_shader GL_ARB_fragment_shader"))
	{
        cout << "Necessary extensions unsupported: " << glh_get_unsupported_extensions() << endl;
        quitapp(-1);
	}
	
	//define search path for shader files
    data_path media;
    media.path.push_back(".");
    media.path.push_back("../IM1");
    media.path.push_back("../../IM1");


	// Diff Pointlight Shader ********************************************************
  diffPointLight = glCreateProgramObjectARB();

	// Vertex Shader laden:
	string filename = media.get_file("OpenGL/shadow/shader/diff_pointlight.vert");
    if (filename == "")
    {
        printf("Unable to load diff_pointlight.vert, exiting...\n");
        quitapp(-1);
    }
    GLcharARB *shaderData = read_text_file(filename.c_str());
    addShader(diffPointLight, shaderData, GL_VERTEX_SHADER_ARB, true);
 
	// Fragment Shader laden:
    filename = media.get_file("OpenGL/shadow/shader/diff_pointlight.frag");
    if (filename == "")
    {
        printf("Unable to load diff_pointlight.frag, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(diffPointLight, shaderData, GL_FRAGMENT_SHADER_ARB, true);

	// Specular Pointlight Shader ************************************************
  specPointLight = glCreateProgramObjectARB();

	// Vertex Shader laden:
		filename = media.get_file("OpenGL/shadow/shader/spec_pointlight.vert");
    if (filename == "")
    {
        printf("Unable to load spec_pointlight.vert, exiting...\n");
        quitapp(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(specPointLight, shaderData, GL_VERTEX_SHADER_ARB, true);
 
	// Fragment Shader laden:
    filename = media.get_file("OpenGL/shadow/shader/spec_pointlight.frag");
    if (filename == "")
    {
        printf("Unable to load spec_pointlight.frag, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(specPointLight, shaderData, GL_FRAGMENT_SHADER_ARB, true);


	// Diff Spotlight Shader ******************************************************
	diffSpotLight = glCreateProgramObjectARB();
	
	// load and create vertex lighting program
	filename = media.get_file("OpenGL/shadow/shader/diff_spotlight.vert");
    if (filename == "")
    {
        printf("Unable to locate diff_spotlight.vert, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(diffSpotLight, shaderData, GL_VERTEX_SHADER_ARB, true);

	// load and create fragment lighting program
    filename = media.get_file("OpenGL/shadow/shader/diff_spotlight.frag");
    if (filename == "")
    {
        printf("Unable to locate diff_spotlight.frag, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(diffSpotLight, shaderData, GL_FRAGMENT_SHADER_ARB, true);
	
	//*****************************************************************************
	
	// Specular Spotlight Shader **************************************************
	specSpotLight = glCreateProgramObjectARB();
	
	// load and create vertex lighting program
	filename = media.get_file("OpenGL/shadow/shader/spec_spotlight.vert");
    if (filename == "")
    {
        printf("Unable to locate spec_spotlight.vert, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(specSpotLight, shaderData, GL_VERTEX_SHADER_ARB, true);

	// load and create fragment lighting program
    filename = media.get_file("OpenGL/shadow/shader/spec_spotlight.frag");
    if (filename == "")
    {
        printf("Unable to locate spec_spotlight.frag, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(specSpotLight, shaderData, GL_FRAGMENT_SHADER_ARB, true);
	
	//***************************************************************************
	

	//Diff Directionallight Shader-----------------------------------------------
	diffDirectionalLight = glCreateProgramObjectARB();
	
	//load and create vertex program
	filename = media.get_file("OpenGL/shadow/shader/diff_directionallight.vert");
    if (filename == "")
    {
        printf("Unable to locate diff_directionallight.vert, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(diffDirectionalLight, shaderData, GL_VERTEX_SHADER_ARB, true);

	//load and create fragment program
    filename = media.get_file("OpenGL/shadow/shader/diff_directionallight.frag");
    if (filename == "")
    {
        printf("Unable to locate diff_directionallight.frag, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(diffDirectionalLight, shaderData, GL_FRAGMENT_SHADER_ARB, true);

		
	//Spec Directionallight Shader-----------------------------------------------
	specDirectionalLight = glCreateProgramObjectARB();
	
	//load and create vertex program
	filename = media.get_file("OpenGL/shadow/shader/spec_directionallight.vert");
    if (filename == "")
    {
        printf("Unable to locate spec_directionallight.vert, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(specDirectionalLight, shaderData, GL_VERTEX_SHADER_ARB, true);

	//load and create fragment program
    filename = media.get_file("OpenGL/shadow/shader/spec_directionallight.frag");
    if (filename == "")
    {
        printf("Unable to locate spec_directionallight.frag, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(specDirectionalLight, shaderData, GL_FRAGMENT_SHADER_ARB, true);


	//White Shader---------------------------------------------------------
	ambientLight = glCreateProgramObjectARB();
	
	//load and create vertex program
	filename = media.get_file("OpenGL/shadow/shader/glslAddAmbient.vert");
    if (filename == "")
    {
        printf("Unable to locate glslAddAmbient.vert, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(ambientLight, shaderData, GL_VERTEX_SHADER_ARB, true);

	//load and create fragment program
    filename = media.get_file("OpenGL/shadow/shader/glslAddAmbient.frag");
    if (filename == "")
    {
        printf("Unable to locate glslAddAmbient.frag, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(ambientLight, shaderData, GL_FRAGMENT_SHADER_ARB, true);

		
	//Soft Shadow Compositing Shader------------------------------------------
	softshadows = glCreateProgramObjectARB();

	//load and create vertex program
	filename = media.get_file("OpenGL/shadow/shader/softshadows.vert");
    if (filename == "")
    {
        printf("Unable to locate softshadows.vert, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(softshadows, shaderData, GL_VERTEX_SHADER_ARB, true);

	//load and create fragment program
    filename = media.get_file("OpenGL/shadow/shader/softshadows.frag");
    if (filename == "")
    {
        printf("Unable to locate softshadows.frag, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(softshadows, shaderData, GL_FRAGMENT_SHADER_ARB, true);


	//Diff Light Compositing Shader-----------------------------------------------
	composeSceneMap = glCreateProgramObjectARB();
	
	//load and create vertex program
	filename = media.get_file("OpenGL/shadow/shader/mult_two_textures.vert");
    if (filename == "")
    {
        printf("Unable to locate mult_two_textures.vert, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(composeSceneMap, shaderData, GL_VERTEX_SHADER_ARB, true);

	//load and create fragment program
    filename = media.get_file("OpenGL/shadow/shader/mult_two_textures.frag");
    if (filename == "")
    {
        printf("Unable to locate mult_two_textures.frag, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(composeSceneMap, shaderData, GL_FRAGMENT_SHADER_ARB, true);


	//Diff Light Compositing Shader-----------------------------------------------
	composeDiffLights = glCreateProgramObjectARB();
	
	//load and create vertex program
	filename = media.get_file("OpenGL/shadow/shader/compose_diff_lights.vert");
    if (filename == "")
    {
        printf("Unable to locate compose_diff_lights.vert, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(composeDiffLights, shaderData, GL_VERTEX_SHADER_ARB, true);

	//load and create fragment program
    filename = media.get_file("OpenGL/shadow/shader/compose_diff_lights.frag");
    if (filename == "")
    {
        printf("Unable to locate compose_diff_lights.frag, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(composeDiffLights, shaderData, GL_FRAGMENT_SHADER_ARB, true);

		
	//Spec Light Compositing Shader-----------------------------------------------
	composeSpecLights = glCreateProgramObjectARB();
	
	//load and create vertex program
	filename = media.get_file("OpenGL/shadow/shader/compose_spec_lights.vert");
    if (filename == "")
    {
        printf("Unable to locate compose_spec_lights.vert, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(composeSpecLights, shaderData, GL_VERTEX_SHADER_ARB, true);

	//load and create fragment program
    filename = media.get_file("OpenGL/shadow/shader/compose_spec_lights.frag");
    if (filename == "")
    {
        printf("Unable to locate compose_spec_lights.frag, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(composeSpecLights, shaderData, GL_FRAGMENT_SHADER_ARB, true);


	//Texture Compositing Shader (fog, basetexture, etc)-----------------------
	composeScene = glCreateProgramObjectARB();
	
	//load and create vertex program
	filename = media.get_file("OpenGL/shadow/shader/compose_scenes.vert");
    if (filename == "")
    {
        printf("Unable to locate compose_scenes.vert, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(composeScene, shaderData, GL_VERTEX_SHADER_ARB, true);

	//load and create fragment program
    filename = media.get_file("OpenGL/shadow/shader/compose_scenes.frag");
    if (filename == "")
    {
        printf("Unable to locate compose_scenes.frag, exiting...\n");
        cleanExit(-1);
    }
    shaderData = read_text_file(filename.c_str());
    addShader(composeScene, shaderData, GL_FRAGMENT_SHADER_ARB, true);

    //set current lighting shader program
    curProgram = 0;
}

/*--------------------------------------------------------------------------
Init pBuffers - added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::initpBuffers()
{
	// pBuffer for RGB
	pbuffer_rgb.Initialize(winWidth, winHeight, true, true);

	
	// pBuffer for Fog Calculation
	pbuffer_fog.Initialize(winWidth, winHeight, false, true);
    pbuffer_fog.Activate();
    glEnable(GL_DEPTH_TEST);
  pbuffer_fog.Deactivate();

	// pBuffer for Depthmaps
	//pbuffer_depthmap.Initialize(depthMapSize, depthMapSize, false, false);
	pbuffer_depthmap.Initialize(DEPTHMAP_SIZE, DEPTHMAP_SIZE, true, false);
  pbuffer_depthmap.Activate();
  glEnable(GL_DEPTH_TEST);
		{
      GLint depth_bits;
      glGetIntegerv(GL_DEPTH_BITS, & depth_bits);
     
			if(depth_bits == 32)         
				{
				depth_format = GL_DEPTH_COMPONENT32_ARB;
				cerr << "...using 32bit depth_format" << endl;
				} 
			if(depth_bits == 24) 
				{
				depth_format = GL_DEPTH_COMPONENT24_ARB;
				cerr << "...using 24it depth_format" << endl;
				}
      else if (depth_bits == 16)  
				{ 
				depth_format = GL_DEPTH_COMPONENT16_ARB;
				cerr << "...using 16bit depth_format" << endl;
				}
			}
  pbuffer_depthmap.Deactivate();
}

void ShadowRendering::updatergbpBufferSize()
{
	// pBuffer for RGB rendering
	pbuffer_rgb.Destroy();
	pbuffer_rgb.Initialize(winWidth, winHeight, true, true);
}

void ShadowRendering::updatefogpBufferSize()
{
	// pBuffer for fog rendering
	pbuffer_fog.Destroy();
	pbuffer_fog.Initialize(winWidth, winHeight, false, true);
}

/*--------------------------------------------------------------------------
eye_linear TexGen Mode - added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::eye_linear_texgen()
{
  //matrix4f m;
	
	GLfloat s[4];
	GLfloat t[4];
	GLfloat r[4];
	GLfloat q[4];
	


  //GLfloat wrapScale;
  GLdouble m1[16], m2[16];
	
  copyMatrix(m1, Smatrix);
  
	glEnable(GL_TEXTURE_2D);

  //copyMatrix(m1, Smatrix);
    
	depthmap_light0.bind();
	depthmap_light0.enable();

  multMatrices(m2, m1, lightFrustumMatrix);
  multMatrices(m1, m2, lightViewMatrix);

		s[0] = m1[0];
		s[1] = m1[4];
		s[2] = m1[8];
		s[3] = m1[12];
		::glTexGenfv(GL_S, GL_EYE_PLANE, s);

		t[0] = m1[1];
		t[1] = m1[5];
		t[2] = m1[9];
		t[3] = m1[13];
		::glTexGenfv(GL_T, GL_EYE_PLANE, t);

		r[0] = m1[2];
		r[1] = m1[6];
		r[2] = m1[10];
		r[3] = m1[14];
		::glTexGenfv(GL_R, GL_EYE_PLANE, r);

		q[0] = m1[3];
		q[1] = m1[7];
		q[2] = m1[11];
		q[3] = m1[15];
		::glTexGenfv(GL_Q, GL_EYE_PLANE, q);

		::glEnable(GL_TEXTURE_GEN_S);
		::glEnable(GL_TEXTURE_GEN_T);
		::glEnable(GL_TEXTURE_GEN_R);
		::glEnable(GL_TEXTURE_GEN_Q);

    //set_texgen_planes(GL_EYE_PLANE, m);
    glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
    glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
}

/*--------------------------------------------------------------------------
object_linear TexGen Mode - added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::obj_linear_texgen()
{
  matrix4f m;

	
	GLfloat p[4];
		
	p[0] = 0.5;
	p[1] = 0;
  p[2] = 0;
  p[3] = 0;
  ::glTexGenfv(GL_S, GL_OBJECT_PLANE, p);
  p[0] = 0;
  p[1] = 0.5;
  p[2] = 0;
  p[3] = 0;
  ::glTexGenfv(GL_T, GL_OBJECT_PLANE, p);
	p[0] = 0;
  p[1] = 0;
  p[2] = 0.5;
  p[3] = 0;
  ::glTexGenfv(GL_R, GL_OBJECT_PLANE, p);
	p[0] = 0.5;
  p[1] = 0.5;
  p[2] = 0.5;
  p[3] = 1;
  ::glTexGenfv(GL_Q, GL_OBJECT_PLANE, p);
	
		::glEnable(GL_TEXTURE_GEN_S);
		::glEnable(GL_TEXTURE_GEN_T);
		::glEnable(GL_TEXTURE_GEN_R);
		::glEnable(GL_TEXTURE_GEN_Q);

		::glDisable(GL_TEXTURE_GEN_S);
		::glDisable(GL_TEXTURE_GEN_T);
		::glDisable(GL_TEXTURE_GEN_R);
		::glDisable(GL_TEXTURE_GEN_Q);



    //set_texgen_planes(GL_OBJECT_PLANE, m);
    glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
    glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
    glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
    glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_OBJECT_LINEAR);
}

/*--------------------------------------------------------------------------
TexGen Mode - added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::texgen(bool enable)
{
    if(enable = true)
    {
        glEnable(GL_TEXTURE_GEN_S);
        glEnable(GL_TEXTURE_GEN_T);
        glEnable(GL_TEXTURE_GEN_R);
        glEnable(GL_TEXTURE_GEN_Q);
    }
    if(enable = false)
    {
        glDisable(GL_TEXTURE_GEN_S);
        glDisable(GL_TEXTURE_GEN_T);
        glDisable(GL_TEXTURE_GEN_R);
        glDisable(GL_TEXTURE_GEN_Q);
    }
}

/*--------------------------------------------------------------------------
Init Multipass Textures - added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::initMultipassTextures()
{
	
	basetexture.bind();				// Textur fr Basetexture und Ambient anlegen
	glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_SIZE, TEX_SIZE, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
		basetexture.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    basetexture.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    basetexture.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    basetexture.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	basetexture.disable();
	
	ambientlight.bind();			// Textur fr Basetexture und Ambient anlegen
		glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_SIZE, TEX_SIZE, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
		ambientlight.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    ambientlight.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		ambientlight.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    ambientlight.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	ambientlight.disable();
	
	depthmap_camera.bind();		// Textur fr Depthmap0 aus Sicht der Camera anlegen
		glTexImage2D(GL_TEXTURE_2D, 0, depth_format, TEX_SIZE, TEX_SIZE, 0, GL_DEPTH_COMPONENT, GL_FLOAT, 0);
		depthmap_camera.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
		depthmap_camera.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		depthmap_camera.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
		depthmap_camera.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
		depthmap_camera.disable();
	
	if (light_num_max >= 1)
		{ diff_lightmap_light0.bind();		// Textur fr Lightmap0 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_SIZE, TEX_SIZE, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
			diff_lightmap_light0.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			diff_lightmap_light0.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			diff_lightmap_light0.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			diff_lightmap_light0.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			diff_lightmap_light0.disable();

			spec_lightmap_light0.bind();		// Textur fr Lightmap0 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_SIZE, TEX_SIZE, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
			spec_lightmap_light0.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			spec_lightmap_light0.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			spec_lightmap_light0.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			spec_lightmap_light0.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			spec_lightmap_light0.disable();

			depthmap_light0.bind();		// Textur fr Depthmap0 aus Sicht der Lichtquelle anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, depth_format, DEPTHMAP_SIZE, DEPTHMAP_SIZE, 0, GL_DEPTH_COMPONENT, GL_FLOAT, 0);	
			depthmap_light0.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			depthmap_light0.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			depthmap_light0.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
			depthmap_light0.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
			depthmap_light0.parameter(GL_TEXTURE_COMPARE_MODE_ARB, GL_COMPARE_R_TO_TEXTURE_ARB);
			depthmap_light0.parameter(GL_TEXTURE_COMPARE_FUNC_ARB, GL_LEQUAL);
			depthmap_light0.disable();

			shadowmap_light0.bind();	// Textur fr fertige Shadowmap0 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 1, TEX_SIZE, TEX_SIZE, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, 0);
			//glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_SIZE, TEX_SIZE, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
			shadowmap_light0.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			shadowmap_light0.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			shadowmap_light0.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			shadowmap_light0.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			shadowmap_light0.disable();}

	if (light_num_max >= 2)
		{	diff_lightmap_light1.bind();		// Textur fr Lightmap1 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_SIZE, TEX_SIZE, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
			diff_lightmap_light1.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			diff_lightmap_light1.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			diff_lightmap_light1.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			diff_lightmap_light1.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			diff_lightmap_light1.disable();

			spec_lightmap_light1.bind();		// Textur fr Lightmap1 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_SIZE, TEX_SIZE, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
			spec_lightmap_light1.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			spec_lightmap_light1.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			spec_lightmap_light1.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			spec_lightmap_light1.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			spec_lightmap_light1.disable();
			
			depthmap_light1.bind();		// Textur fr Depthmap1 aus Sicht der Lichtquelle anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, depth_format, DEPTHMAP_SIZE, DEPTHMAP_SIZE, 0, GL_DEPTH_COMPONENT, GL_FLOAT, 0);
			depthmap_light1.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			depthmap_light1.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			depthmap_light1.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
			depthmap_light1.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
			depthmap_light1.parameter(GL_TEXTURE_COMPARE_MODE_ARB, GL_COMPARE_R_TO_TEXTURE_ARB);
			depthmap_light1.parameter(GL_TEXTURE_COMPARE_FUNC_ARB, GL_LEQUAL);
			depthmap_light1.disable();

			shadowmap_light1.bind();	// Textur fr fertige Shadowmap1 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 1, TEX_SIZE, TEX_SIZE, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, 0);
			shadowmap_light1.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			shadowmap_light1.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			shadowmap_light1.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			shadowmap_light1.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			shadowmap_light1.disable();}

	if (light_num_max >= 3)
		{ diff_lightmap_light2.bind();		// Textur fr Lightmap2 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_SIZE, TEX_SIZE, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
			diff_lightmap_light2.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			diff_lightmap_light2.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			diff_lightmap_light2.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			diff_lightmap_light2.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			diff_lightmap_light2.disable();

			spec_lightmap_light2.bind();		// Textur fr Lightmap2 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_SIZE, TEX_SIZE, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
			spec_lightmap_light2.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			spec_lightmap_light2.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			spec_lightmap_light2.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			spec_lightmap_light2.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			spec_lightmap_light2.disable();
		
			depthmap_light2.bind();		// Textur fr Depthmap2 aus Sicht der Lichtquelle anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, depth_format, DEPTHMAP_SIZE, DEPTHMAP_SIZE, 0, GL_DEPTH_COMPONENT, GL_FLOAT, 0);
			depthmap_light2.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			depthmap_light2.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			depthmap_light2.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
			depthmap_light2.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
			depthmap_light2.parameter(GL_TEXTURE_COMPARE_MODE_ARB, GL_COMPARE_R_TO_TEXTURE_ARB);
			depthmap_light2.parameter(GL_TEXTURE_COMPARE_FUNC_ARB, GL_LEQUAL);
			depthmap_light2.disable();

			shadowmap_light2.bind();	// Textur fr fertige Shadowmap2 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 1, TEX_SIZE, TEX_SIZE, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, 0);
			shadowmap_light2.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			shadowmap_light2.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			shadowmap_light2.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			shadowmap_light2.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			shadowmap_light2.disable();}

	if (light_num_max >= 4)
		{ diff_lightmap_light3.bind();		// Textur fr Lightmap3 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_SIZE, TEX_SIZE, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
			diff_lightmap_light3.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			diff_lightmap_light3.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			diff_lightmap_light3.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			diff_lightmap_light3.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			diff_lightmap_light3.disable();

			spec_lightmap_light3.bind();		// Textur fr Lightmap3 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_SIZE, TEX_SIZE, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
			spec_lightmap_light3.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			spec_lightmap_light3.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			spec_lightmap_light3.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			spec_lightmap_light3.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			spec_lightmap_light3.disable();

			depthmap_light3.bind();		// Textur fr Depthmap2 aus Sicht der Lichtquelle anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, depth_format, DEPTHMAP_SIZE, DEPTHMAP_SIZE, 0, GL_DEPTH_COMPONENT, GL_FLOAT, 0);
			depthmap_light3.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			depthmap_light3.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			depthmap_light3.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER);
			depthmap_light3.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER);
			depthmap_light3.parameter(GL_TEXTURE_COMPARE_MODE_ARB, GL_COMPARE_R_TO_TEXTURE_ARB);
			depthmap_light3.parameter(GL_TEXTURE_COMPARE_FUNC_ARB, GL_LEQUAL);
			depthmap_light3.disable();

			shadowmap_light3.bind();	// Textur fr fertige Shadowmap3 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 1, TEX_SIZE, TEX_SIZE, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, 0);
			shadowmap_light3.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			shadowmap_light3.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			shadowmap_light3.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			shadowmap_light3.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			shadowmap_light3.disable();}
	
	if (light_num_max >= 5)
	{}
	if (light_num_max >= 6)
	{}
	if (light_num_max >= 7)
	{}
	if (light_num_max >= 8)
	{}

	if (softshadowsEnabled)
			{
			shadowmap_softpass0.bind();	// Textur fr Softpass0 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 1, TEX_SIZE, TEX_SIZE, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, 0);
			shadowmap_softpass0.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			shadowmap_softpass0.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			shadowmap_softpass0.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			shadowmap_softpass0.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			shadowmap_softpass0.disable();

			shadowmap_softpass1.bind();	// Textur fr Softpass1 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 1, TEX_SIZE, TEX_SIZE, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, 0);
			shadowmap_softpass1.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			shadowmap_softpass1.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			shadowmap_softpass1.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			shadowmap_softpass1.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			shadowmap_softpass1.disable();

			shadowmap_softpass2.bind();	// Textur fr Softpass2 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 1, TEX_SIZE, TEX_SIZE, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, 0);
			shadowmap_softpass2.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			shadowmap_softpass2.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			shadowmap_softpass2.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			shadowmap_softpass2.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			shadowmap_softpass2.disable();

			shadowmap_softpass3.bind();	// Textur fr Softpass3 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 1, TEX_SIZE, TEX_SIZE, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, 0);
			shadowmap_softpass3.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			shadowmap_softpass3.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			shadowmap_softpass3.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			shadowmap_softpass3.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			shadowmap_softpass3.disable();

			shadowmap_softpass4.bind();	// Textur fr Softpass4 anlegen
			glTexImage2D(GL_TEXTURE_2D, 0, 1, TEX_SIZE, TEX_SIZE, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, 0);
			shadowmap_softpass4.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
			shadowmap_softpass4.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
			shadowmap_softpass4.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
			shadowmap_softpass4.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
			shadowmap_softpass4.disable();
			}

	diff_scenemap.bind();							// Textur fr Lightmap*Shadowmap Lichtquelle 0-7
		glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_SIZE, TEX_SIZE, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
		diff_scenemap.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    diff_scenemap.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    diff_scenemap.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    diff_scenemap.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	diff_scenemap.disable();

	spec_scenemap.bind();							// Textur fr Lightmap*Shadowmap Lichtquelle 0-7
		glTexImage2D(GL_TEXTURE_2D, 0, 3, TEX_SIZE, TEX_SIZE, 0, GL_RGB, GL_UNSIGNED_BYTE, 0);
		spec_scenemap.parameter(GL_TEXTURE_MIN_FILTER, GL_LINEAR);
    spec_scenemap.parameter(GL_TEXTURE_MAG_FILTER, GL_LINEAR);
    spec_scenemap.parameter(GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
    spec_scenemap.parameter(GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
	spec_scenemap.disable();

}

/*--------------------------------------------------------------------------
Init OpenGL Stats
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::initOpenGLStats()
{
	//image plane for multipass texturing
 
	/*
	screen.new_list(GL_COMPILE);		
		glPushMatrix();
		glBegin(GL_QUADS);
			glNormal3f(0, 0, 1);
			glTexCoord2f(0.0f, 0.0f);
			glVertex3f(-1.0f, -1.0f,  0.0f);
			glTexCoord2f(1.0f, 0.0f);
			glVertex3f( 1.0f, -1.0f,  0.0f);
			glTexCoord2f(1.0f, 1.0f);
			glVertex3f( 1.0f,  1.0f,  0.0f);
			glTexCoord2f(0.0f, 1.0f);
			glVertex3f(-1.0f,  1.0f,  0.0f);
			glEnd();
		glPopMatrix();
  screen.end_list();
  */

	//OpenGL States aus alter GL-init vom Marco

	/*
		::glClearStencil(0);
		::glClearColor(0,0,0,0);

		::glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
		::glPixelStorei(GL_PACK_ALIGNMENT, 1);

		::glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
		::glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
		::glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
		::glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);


	::glDepthFunc(GL_LEQUAL);
	::glEnable(GL_DEPTH_TEST);
	::glEnable(GL_LIGHTING);
	::glLineStipple(1, 0xf0f0);
	::glMaterialf(GL_FRONT, GL_SHININESS, 30.0);
	::glEnable(GL_CULL_FACE);
	::glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, 1);
	::glEnable(GL_NORMALIZE);
	//--------------
*/
	glHint(GL_PERSPECTIVE_CORRECTION_HINT, GL_NICEST);
	glEnable(GL_DEPTH_TEST);
	glEnable(GL_TEXTURE_2D);
	glShadeModel(GL_SMOOTH);
}

/*--------------------------------------------------------------------------
diffuse lightmap for each lightsource
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::renderDiffLightMap(unsigned int light_number,
																		 unsigned int type_of_light,
																		 MediaObjectProxy * pSceneRenderer,
																		 const Matrix& Transformation, 
																		 const CullFlag& CullFlag,
																		 OpenGLVisualRenderer* pRenderer, 
																		 DWORD dwTime) 
{
	pbuffer_rgb.Activate();
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT0, GL_AMBIENT, zero);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightColor);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightColor);
	glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
	glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, light_dir);

	glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, lightConstantAttenuation);
	glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, lightLinearAttenuation);
	glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, lightQuadraticAttenuation);
	glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, lightSpotCutoff);
	glLightf(GL_LIGHT0, GL_SPOT_EXPONENT, lightSpotExponent);

	// Auswahl des Lightmodells
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, zero);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE); //??
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_TRUE);

	if (light_number == 1)
		curLightmap = diff_lightmap_light0;
	if (light_number == 2)
		curLightmap = diff_lightmap_light1;
	if (light_number == 3)
		curLightmap = diff_lightmap_light2;
	if (light_number == 4)
		curLightmap = diff_lightmap_light3;
	if (light_number == 5)
		curLightmap = diff_lightmap_light4;
	if (light_number == 6)
		curLightmap = diff_lightmap_light5;
	if (light_number == 7)
		curLightmap = diff_lightmap_light6;
	if (light_number == 8)
		curLightmap = diff_lightmap_light7;
	
	glActiveTextureARB(GL_TEXTURE0_ARB);
	
	if (perpixelLightingEnabled==true)
		{
		if (type_of_light == 1)
			curProgram = diffPointLight;
		if (type_of_light == 2)
			curProgram = diffSpotLight;
		if (type_of_light == 3)
			curProgram = diffDirectionalLight;

		glUseProgramObjectARB(curProgram);
		
		if (type_of_light == 1)
		glUniform1fARB(glGetUniformLocationARB(curProgram, "radius"), lightRadius);

		if (type_of_light == 2)
		glUniform1fARB(glGetUniformLocationARB(curProgram, "radius"), lightRadius);

		glVertexPointer(3, GL_FLOAT, 0, vertexdata);
		glNormalPointer(GL_FLOAT, 0, normaldata);
		glEnableClientState( GL_VERTEX_ARRAY );
		glEnableClientState( GL_NORMAL_ARRAY );
	
		// render lightmap from camera point of view
		::glDisable(GL_LIGHT0);
		::glDisable(GL_LIGHTING);
		}
	else
		{
		::glEnable(GL_LIGHT0);
		::glEnable(GL_LIGHTING);
		// -->  damit per-Vertex Lighting Sinn macht muss noch ein Weg gefunden werden die
		//			Szene ohne Objektfarben zu zeichnen!!
		}

	pSceneRenderer->CullOrRender(Transformation, CullFlag, pRenderer, dwTime); 
	
	curLightmap.bind();
	glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, winWidth, winHeight);
  glUseProgramObjectARB(0);

	// empty texture pipeline
	glActiveTextureARB(GL_TEXTURE0_ARB);
	curLightmap.disable();
	pbuffer_rgb.Deactivate();

//	--> hier "renderDiffSceneLight()" einfgen!
}

/*--------------------------------------------------------------------------
specular lightmap for each lightsource
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::renderSpecLightMap(unsigned int light_number,
																		 unsigned int type_of_light,
																		 MediaObjectProxy * pSceneRenderer,
																		 const Matrix& Transformation, 
																		 const CullFlag& CullFlag,
																		 OpenGLVisualRenderer* pRenderer, 
																		 DWORD dwTime) 
{
	pbuffer_rgb.Activate();
	glClearColor(0.0f, 0.0f, 0.0f, 0.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	glEnable(GL_LIGHT0);
	glLightfv(GL_LIGHT0, GL_AMBIENT, zero);
	glLightfv(GL_LIGHT0, GL_DIFFUSE, lightColor);
	glLightfv(GL_LIGHT0, GL_SPECULAR, lightColor);
	glLightfv(GL_LIGHT0, GL_POSITION, light_pos);
	glLightfv(GL_LIGHT0, GL_SPOT_DIRECTION, light_dir);

	glLightf(GL_LIGHT0, GL_CONSTANT_ATTENUATION, lightConstantAttenuation);
	glLightf(GL_LIGHT0, GL_LINEAR_ATTENUATION, lightLinearAttenuation);
	glLightf(GL_LIGHT0, GL_QUADRATIC_ATTENUATION, lightQuadraticAttenuation);
	glLightf(GL_LIGHT0, GL_SPOT_CUTOFF, lightSpotCutoff);
	glLightf(GL_LIGHT0, GL_SPOT_EXPONENT, lightSpotExponent);

	// Auswahl des Lightmodells
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, zero);
	glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE); //??
	glLightModeli(GL_LIGHT_MODEL_TWO_SIDE, GL_FALSE);

	if (light_number == 1)
		curLightmap = spec_lightmap_light0;
	if (light_number == 2)
		curLightmap = spec_lightmap_light1;
	if (light_number == 3)
		curLightmap = spec_lightmap_light2;
	if (light_number == 4)
		curLightmap = spec_lightmap_light3;
	if (light_number == 5)
		curLightmap = spec_lightmap_light4;
	if (light_number == 6)
		curLightmap = spec_lightmap_light5;
	if (light_number == 7)
		curLightmap = spec_lightmap_light6;
	if (light_number == 8)
		curLightmap = spec_lightmap_light7;
	
	glActiveTextureARB(GL_TEXTURE0_ARB);
	
	if (perpixelLightingEnabled==true)
		{
		if (type_of_light == 1)
			curProgram = specPointLight;
		if (type_of_light == 2)
			curProgram = specSpotLight;
		if (type_of_light == 3)
			curProgram = specDirectionalLight;

		glUseProgramObjectARB(curProgram);

		glVertexPointer(3, GL_FLOAT, 0, vertexdata);
		glNormalPointer(GL_FLOAT, 0, normaldata);
		glEnableClientState( GL_VERTEX_ARRAY );
		glEnableClientState( GL_NORMAL_ARRAY );
		
			if (type_of_light == 1)
		glUniform1fARB(glGetUniformLocationARB(curProgram, "radius"), lightRadius);

		if (type_of_light == 2)
		glUniform1fARB(glGetUniformLocationARB(curProgram, "radius"), lightRadius);
	
		// render lightmap from camera point of view
		::glDisable(GL_LIGHT0);
		::glDisable(GL_LIGHTING);
		}
	else
		{
		::glEnable(GL_LIGHT0);
		::glEnable(GL_LIGHTING);
		// -->  damit per-Vertex Lighting Sinn macht muss noch ein Weg gefunden werden die
		//			Szene ohne Objektfarben zu zeichnen!!
		}

	pSceneRenderer->CullOrRender(Transformation, CullFlag, pRenderer, dwTime); 
	
	curLightmap.bind();
	glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, winWidth, winHeight);
  glUseProgramObjectARB(0);

	// empty texture pipeline
	glActiveTextureARB(GL_TEXTURE0_ARB);
	curLightmap.disable();
	pbuffer_rgb.Deactivate();

//	--> hier "renderDiffSceneLight()" einfgen!
}

/*--------------------------------------------------------------------------
shadowmap 1st pass for each soft shadow Pass
- added by M.Frank 12-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::renderSoftDepthMap(unsigned int light_number,
																		 MediaObjectProxy * pSceneRenderer,
																		 const Matrix& TransformationX, 
																		 const CullFlag& CullFlag,
																		 OpenGLVisualRenderer* pRenderer, 
																		 DWORD dwTime)
{
	pbuffer_depthmap.Activate();
	
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 
	
	// select active depthmap and shadowmap texture
	if (light_number == 1) curDepthmap = depthmap_light0;
	if (light_number == 2) curDepthmap = depthmap_light1;
	if (light_number == 3) curDepthmap = depthmap_light2;
	if (light_number == 4) curDepthmap = depthmap_light3;
	if (light_number == 5) curDepthmap = depthmap_light4;
	if (light_number == 6) curDepthmap = depthmap_light5;
	if (light_number == 7) curDepthmap = depthmap_light6;
	if (light_number == 8) curDepthmap = depthmap_light7;
	// render depthmap from light point of view
	glActiveTextureARB(GL_TEXTURE0_ARB);
	
	//glPolygonOffset(tweaks["polygon offset scale"].val, tweaks["polygon offset bias"].val);
	glPolygonOffset(8.0, 4.0);
	//updateDepthBias(1);
		
	glColorMask(0,0,0,0);
      glDisable(GL_LIGHTING);
        glEnable(GL_POLYGON_OFFSET_FILL);

				::glViewport(0, 0, DEPTHMAP_SIZE, DEPTHMAP_SIZE);
				// render all cast objects from light point od view
				pSceneRenderer->CullOrRender(TransformationX, CullFlag, pRenderer, dwTime);

			glDisable(GL_POLYGON_OFFSET_FILL);
  glColorMask(1,1,1,1);

	// Depthmap in Textur speichern
	curDepthmap.bind();
	glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, DEPTHMAP_SIZE, DEPTHMAP_SIZE);
	glActiveTextureARB(GL_TEXTURE2_ARB);
	curDepthmap.disable();
	::glViewport(0, 0, winWidth, winHeight);
	pbuffer_depthmap.Deactivate();
}

/*--------------------------------------------------------------------------
shadowmap 2nd Pass for each soft shadow Pass
- added by M.Frank 12-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::renderSoftShadowMap(unsigned int light_number,
																		 unsigned int softpass_number,
																		 MediaObjectProxy * pSceneRenderer,
																		 const Matrix& Transformation,
																		 const CullFlag& CullFlag,
																		 OpenGLVisualRenderer* pRenderer, 
																		 DWORD dwTime)
{
	// select active depthmap and shadowmap texture
	if (light_number == 1){
		curDepthmap = depthmap_light0;}
	if (light_number == 2){
		curDepthmap = depthmap_light1;}
	if (light_number == 3){
		curDepthmap = depthmap_light2;}
	if (light_number == 4){
		curDepthmap = depthmap_light3;}
	if (light_number == 5){
		curDepthmap = depthmap_light4;}
	if (light_number == 6){
		curDepthmap = depthmap_light5;}
	if (light_number == 7){
		curDepthmap = depthmap_light6;}
	if (light_number == 8){
		curDepthmap = depthmap_light7;}

	if (softpass_number == 0){
		curSoftShadowmap = shadowmap_softpass0;}
	if (softpass_number == 1){
		curSoftShadowmap = shadowmap_softpass1;}
	if (softpass_number == 2){
		curSoftShadowmap = shadowmap_softpass2;}
	if (softpass_number == 3){
		curSoftShadowmap = shadowmap_softpass3;}
	if (softpass_number == 4){
		curSoftShadowmap = shadowmap_softpass4;}
		
	// Depthmap auf Szene aus Kamerasicht projizieren und Vergleichen (->shadowmap)

	pbuffer_rgb.Activate();

	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 

		glActiveTextureARB(GL_TEXTURE0_ARB);

		//glPushMatrix();
		//glLoadIdentity();
		setupEyeView();	
		//::glLoadMatrixd(lightViewMatrix);
		//camera.apply_inverse_transform();

		glMatrixMode(GL_TEXTURE);
		glLoadIdentity();

		//glTranslatef(.5f, .5f, .5f);
		//glScalef(.5f, .5f, .5f);
		eye_linear_texgen(); 
		//texgen(true);
		//glPopMatrix();
		
		//gluPerspective(lightshaper.fovy, 1, lightshaper.zNear, lightshaper.zFar);
		//::glLoadMatrixd(lightFrustumMatrix);
		//light0.apply_inverse_transform();

		glMatrixMode(GL_MODELVIEW);
		
		curDepthmap.bind();
		curDepthmap.enable();

		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

		::glDisable(GL_LIGHT0);
		::glDisable(GL_LIGHTING);
		::glViewport(0, 0, winWidth, winHeight);
		//::glDisable(GL_TEXTURE);
		glActiveTextureARB(GL_TEXTURE2_ARB);
		//glActiveTextureARB(GL_TEXTURE2_ARB);
		pSceneRenderer->CullOrRender(Transformation, CullFlag, pRenderer, dwTime);
		//glUseProgramObjectARB(0);
		curDepthmap.disable();
}

/*--------------------------------------------------------------------------
shadowmap 3rd Pass for each soft shadow Pass
- added by M.Frank 12-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::renderSoftShadowMap_pass3(MediaObjectProxy * pSceneRenderer,
																		 const Matrix& Transformation,
																		 const CullFlag& CullFlag,
																		 OpenGLVisualRenderer* pRenderer, 
																		 DWORD dwTime)
{
		::glDisable(GL_LIGHT0);
		::glDisable(GL_LIGHTING);
		::glDisable(GL_TEXTURE);
		pSceneRenderer->CullOrRender(Transformation, CullFlag, pRenderer, dwTime);

		// Shadowmap in Textur speichern
		glActiveTextureARB(GL_TEXTURE1_ARB);
		curSoftShadowmap.bind();
		glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, winWidth, winHeight);
		
		// Texturpipelines wieder freigeben
		glActiveTextureARB(GL_TEXTURE1_ARB);
		curSoftShadowmap.disable();
		glActiveTextureARB(GL_TEXTURE0_ARB);
		curDepthmap.disable();
		
	//glDisable(GL_DEPTH_TEST);
	pbuffer_rgb.Deactivate();
}


/*--------------------------------------------------------------------------
add all single Shadow Maps for one lightsource to create "jittered" soft shadows
- added by M.Frank 12-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::addSoftShadowMaps(unsigned int light_number)
{
	pbuffer_rgb.Activate();
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	if (light_number == 1){
		curShadowmap = shadowmap_light0;}
	if (light_number == 2){
		curShadowmap = shadowmap_light1;}
	if (light_number == 3){
		curShadowmap = shadowmap_light2;}
	if (light_number == 4){
		curShadowmap = shadowmap_light3;}
	if (light_number == 5){
		curShadowmap = shadowmap_light4;}
	if (light_number == 6){
		curShadowmap = shadowmap_light5;}
	if (light_number == 7){
		curShadowmap = shadowmap_light6;}
	if (light_number == 8){
		curShadowmap = shadowmap_light7;}

	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();	
		
	glMatrixMode(GL_PROJECTION);
	glPushMatrix();
	glLoadIdentity();

	glActiveTextureARB(GL_TEXTURE0_ARB);
	shadowmap_softpass0.bind();

	glActiveTextureARB(GL_TEXTURE1_ARB);
	shadowmap_softpass1.bind();
	
	glActiveTextureARB(GL_TEXTURE2_ARB);
	shadowmap_softpass2.bind();

	glActiveTextureARB(GL_TEXTURE3_ARB);
	shadowmap_softpass3.bind();
		
	glActiveTextureARB(GL_TEXTURE4_ARB);
	shadowmap_softpass4.bind();
		
	glUseProgramObjectARB(softshadows);

	glUniform1iARB(glGetUniformLocationARB(softshadows, "shadomap_softpass0"), 0);
	glUniform1iARB(glGetUniformLocationARB(softshadows, "shadomap_softpass1"), 1);
	glUniform1iARB(glGetUniformLocationARB(softshadows, "shadomap_softpass2"), 2);
	glUniform1iARB(glGetUniformLocationARB(softshadows, "shadomap_softpass3"), 3);
	glUniform1iARB(glGetUniformLocationARB(softshadows, "shadomap_softpass4"), 4);
	
	glPushMatrix();
	glBegin(GL_QUADS);
		glNormal3f(0, 0, 1);
		glTexCoord2f(0.0f, 0.0f);
		glVertex3f(-1.0f, -1.0f,  0.0f);
		glTexCoord2f(1.0, 0.0f);
		glVertex3f( 1.0f, -1.0f,  0.0f);
		glTexCoord2f(1.0, 1.0);
		glVertex3f( 1.0f,  1.0f,  0.0f);
		glTexCoord2f(0.0f, 1.0);
		glVertex3f(-1.0f,  1.0f,  0.0f);
		glEnd();
	glPopMatrix();

	glUseProgramObjectARB(0);
		
	glActiveTextureARB(GL_TEXTURE4_ARB);
	shadowmap_softpass4.disable();
	glActiveTextureARB(GL_TEXTURE3_ARB);
	shadowmap_softpass3.disable();
	glActiveTextureARB(GL_TEXTURE2_ARB);
	shadowmap_softpass2.disable();
	glActiveTextureARB(GL_TEXTURE1_ARB);
	shadowmap_softpass1.disable();
	glActiveTextureARB(GL_TEXTURE0_ARB);
	shadowmap_softpass0.disable();

	glActiveTextureARB(GL_TEXTURE0_ARB);
	curShadowmap.bind();
	glCopyTexSubImage2D(GL_TEXTURE_2D, 0,   0, 0,   0, 0,  winWidth, winHeight);
	curShadowmap.disable();

	pbuffer_rgb.Deactivate();
		
	glMatrixMode(GL_PROJECTION);
	glPopMatrix();
	glMatrixMode(GL_MODELVIEW);
	glPopMatrix();
}


/*--------------------------------------------------------------------------
shadowmap 1st pass for each shadowing lightsource
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::renderDepthMap(unsigned int light_number,
																		 MediaObjectProxy * pSceneRenderer,
																		 const Matrix& TransformationX, 
																		 const CullFlag& CullFlag,
																		 OpenGLVisualRenderer* pRenderer, 
																		 DWORD dwTime)
{
	pbuffer_depthmap.Activate();
	
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 
	
	// select active depthmap and shadowmap texture
	if (light_number == 1) curDepthmap = depthmap_light0;
	if (light_number == 2) curDepthmap = depthmap_light1;
	if (light_number == 3) curDepthmap = depthmap_light2;
	if (light_number == 4) curDepthmap = depthmap_light3;
	if (light_number == 5) curDepthmap = depthmap_light4;
	if (light_number == 6) curDepthmap = depthmap_light5;
	if (light_number == 7) curDepthmap = depthmap_light6;
	if (light_number == 8) curDepthmap = depthmap_light7;
	// render depthmap from light point of view
	glActiveTextureARB(GL_TEXTURE0_ARB);
	
	//glPolygonOffset(tweaks["polygon offset scale"].val, tweaks["polygon offset bias"].val);
	glPolygonOffset(8.0, 4.0);
	//updateDepthBias(1);
		
	glColorMask(0,0,0,0);
      glDisable(GL_LIGHTING);
        glEnable(GL_POLYGON_OFFSET_FILL);

				::glViewport(0, 0, DEPTHMAP_SIZE, DEPTHMAP_SIZE);
				// render all cast objects from light point od view
				pSceneRenderer->CullOrRender(TransformationX, CullFlag, pRenderer, dwTime);

			glDisable(GL_POLYGON_OFFSET_FILL);
  glColorMask(1,1,1,1);

	// Depthmap in Textur speichern
	curDepthmap.bind();
	glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, DEPTHMAP_SIZE, DEPTHMAP_SIZE);
	glActiveTextureARB(GL_TEXTURE2_ARB);
	curDepthmap.disable();
	::glViewport(0, 0, winWidth, winHeight);
	pbuffer_depthmap.Deactivate();
}

/*--------------------------------------------------------------------------
shadowmap 2nd Pass for each shadowing lightsource
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::renderShadowMap(unsigned int light_number,
																		 MediaObjectProxy * pSceneRenderer,
																		 const Matrix& Transformation,
																		 const CullFlag& CullFlag,
																		 OpenGLVisualRenderer* pRenderer, 
																		 DWORD dwTime)
{
	// select active depthmap and shadowmap texture
	if (light_number == 1){
		curDepthmap = depthmap_light0;
		curShadowmap = shadowmap_light0;}
	if (light_number == 2){
		curDepthmap = depthmap_light1;
		curShadowmap = shadowmap_light1;}
	if (light_number == 3){
		curDepthmap = depthmap_light2;
		curShadowmap = shadowmap_light2;}
	if (light_number == 4){
		curDepthmap = depthmap_light3;
		curShadowmap = shadowmap_light3;}
	if (light_number == 5){
		curDepthmap = depthmap_light4;
		curShadowmap = shadowmap_light4;}
	if (light_number == 6){
		curDepthmap = depthmap_light5;
		curShadowmap = shadowmap_light5;}
	if (light_number == 7){
		curDepthmap = depthmap_light6;
		curShadowmap = shadowmap_light6;}
	if (light_number == 8){
		curDepthmap = depthmap_light7;
		curShadowmap = shadowmap_light7;}
		
	// Depthmap auf Szene aus Kamerasicht projizieren und Vergleichen (->shadowmap)

	pbuffer_rgb.Activate();

	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT); 

		glActiveTextureARB(GL_TEXTURE0_ARB);

		//glPushMatrix();
		//glLoadIdentity();
		setupEyeView();	
		//::glLoadMatrixd(lightViewMatrix);
		//camera.apply_inverse_transform();

		glMatrixMode(GL_TEXTURE);
		glLoadIdentity();

		//glTranslatef(.5f, .5f, .5f);
		//glScalef(.5f, .5f, .5f);
		eye_linear_texgen(); 
		//texgen(true);
		//glPopMatrix();
		
		//gluPerspective(lightshaper.fovy, 1, lightshaper.zNear, lightshaper.zFar);
		//::glLoadMatrixd(lightFrustumMatrix);
		//light0.apply_inverse_transform();

		glMatrixMode(GL_MODELVIEW);
		
		curDepthmap.bind();
		curDepthmap.enable();

		glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);

		::glDisable(GL_LIGHT0);
		::glDisable(GL_LIGHTING);
		::glViewport(0, 0, winWidth, winHeight);
	//::glDisable(GL_TEXTURE);
		glActiveTextureARB(GL_TEXTURE2_ARB);
		//glActiveTextureARB(GL_TEXTURE2_ARB);
		pSceneRenderer->CullOrRender(Transformation, CullFlag, pRenderer, dwTime);
		//glUseProgramObjectARB(0);
		curDepthmap.disable();
}

/*--------------------------------------------------------------------------
shadowmap 3rd Pass for each shadowing lightsource
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::renderShadowMap_pass3(unsigned int light_number,
																		 MediaObjectProxy * pSceneRenderer,
																		 const Matrix& Transformation,
																		 const CullFlag& CullFlag,
																		 OpenGLVisualRenderer* pRenderer, 
																		 DWORD dwTime)
{
		::glDisable(GL_LIGHT0);
		::glDisable(GL_LIGHTING);
		::glDisable(GL_TEXTURE);
		pSceneRenderer->CullOrRender(Transformation, CullFlag, pRenderer, dwTime);
//	glEnable(GL_TEXTURE_2D);	//??
		// Shadowmap in Textur speichern
		glActiveTextureARB(GL_TEXTURE1_ARB);
		curShadowmap.bind();
		glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, winWidth, winHeight);
		//glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, pbuffer_rgb.GetWidth(), pbuffer_rgb.GetHeight());
		
		// Texturpipelines wieder freigeben
		glActiveTextureARB(GL_TEXTURE1_ARB);
		curShadowmap.disable();
		glActiveTextureARB(GL_TEXTURE0_ARB);
		curDepthmap.disable();
		
	//glDisable(GL_DEPTH_TEST);
	pbuffer_rgb.Deactivate();
}


/*--------------------------------------------------------------------------
multiplication of shadowmap and corresponding DIFFUSE lightmap
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/ 
void ShadowRendering::renderDiffSceneLight(unsigned int light_number)
{
	pbuffer_rgb.Activate();
	glClearColor(1.0f, 1.0f, 1.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	if (light_number == 1){
		curLightmap = diff_lightmap_light0;
		curShadowmap = shadowmap_light0;}
	if (light_number == 2){
		curLightmap = diff_lightmap_light1;
		curShadowmap = shadowmap_light1;}
	if (light_number == 3){
		curLightmap = diff_lightmap_light2;
		curShadowmap = shadowmap_light2;}
	if (light_number == 4){
		curLightmap = diff_lightmap_light3;
		curShadowmap = shadowmap_light3;}
	if (light_number == 5){
		curLightmap = diff_lightmap_light4;
		curShadowmap = shadowmap_light4;}
	if (light_number == 6){
		curLightmap = diff_lightmap_light5;
		curShadowmap = shadowmap_light5;}
	if (light_number == 7){
		curLightmap = diff_lightmap_light6;
		curShadowmap = shadowmap_light6;}
	if (light_number == 8){
		curLightmap = diff_lightmap_light7;
		curShadowmap = shadowmap_light7;}

		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glLoadIdentity();	
		
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();

		glActiveTextureARB(GL_TEXTURE0_ARB);
		curLightmap.bind();

		glActiveTextureARB(GL_TEXTURE1_ARB);
		curShadowmap.bind();
		
		glUseProgramObjectARB(composeSceneMap);

		glUniform1iARB(glGetUniformLocationARB(composeSceneMap, "curLightmap"), 0);
		glUniform1iARB(glGetUniformLocationARB(composeSceneMap, "curShadowmap"), 1);
		
		glPushMatrix();
		glBegin(GL_QUADS);
			glNormal3f(0, 0, 1);
			glTexCoord2f(0.0f, 0.0f);
			glVertex3f(-1.0f, -1.0f,  0.0f);
			glTexCoord2f(1.0, 0.0f);
			glVertex3f( 1.0f, -1.0f,  0.0f);
			glTexCoord2f(1.0, 1.0);
			glVertex3f( 1.0f,  1.0f,  0.0f);
			glTexCoord2f(0.0f, 1.0);
			glVertex3f(-1.0f,  1.0f,  0.0f);
			glEnd();
		glPopMatrix();

		glUseProgramObjectARB(0);

		glActiveTextureARB(GL_TEXTURE1_ARB);
		curShadowmap.disable();
		glActiveTextureARB(GL_TEXTURE0_ARB);
		curLightmap.disable();
		
		glActiveTextureARB(GL_TEXTURE0_ARB);
		curLightmap.bind();
		glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, winWidth, winHeight);
		curLightmap.disable();
	
		pbuffer_rgb.Deactivate();

		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();
}

/*--------------------------------------------------------------------------
multiplication of shadowmap and corresponding SPECULAR lightmap
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/ 
void ShadowRendering::renderSpecSceneLight(unsigned int light_number)
{
	pbuffer_rgb.Activate();
	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	if (light_number == 1){
		curLightmap = spec_lightmap_light0;
		curShadowmap = shadowmap_light0;}
	if (light_number == 2){
		curLightmap = spec_lightmap_light1;
		curShadowmap = shadowmap_light1;}
	if (light_number == 3){
		curLightmap = spec_lightmap_light2;
		curShadowmap = shadowmap_light2;}
	if (light_number == 4){
		curLightmap = spec_lightmap_light3;
		curShadowmap = shadowmap_light3;}
	if (light_number == 5){
		curLightmap = spec_lightmap_light4;
		curShadowmap = shadowmap_light4;}
	if (light_number == 6){
		curLightmap = spec_lightmap_light5;
		curShadowmap = shadowmap_light5;}
	if (light_number == 7){
		curLightmap = spec_lightmap_light6;
		curShadowmap = shadowmap_light6;}
	if (light_number == 8){
		curLightmap = spec_lightmap_light7;
		curShadowmap = shadowmap_light7;}

		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glLoadIdentity();	
		
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();

		glActiveTextureARB(GL_TEXTURE0_ARB);
		curLightmap.bind();

		glActiveTextureARB(GL_TEXTURE1_ARB);
		curShadowmap.bind();
		
		glUseProgramObjectARB(composeSceneMap);

		glUniform1iARB(glGetUniformLocationARB(composeSceneMap, "curLightmap"), 0);
		glUniform1iARB(glGetUniformLocationARB(composeSceneMap, "curShadowmap"), 1);
		
		glPushMatrix();
		glBegin(GL_QUADS);
			glNormal3f(0, 0, 1);
			glTexCoord2f(0.0f, 0.0f);
			glVertex3f(-1.0f, -1.0f,  0.0f);
			glTexCoord2f(1.0, 0.0f);
			glVertex3f( 1.0f, -1.0f,  0.0f);
			glTexCoord2f(1.0, 1.0);
			glVertex3f( 1.0f,  1.0f,  0.0f);
			glTexCoord2f(0.0f, 1.0);
			glVertex3f(-1.0f,  1.0f,  0.0f);
			glEnd();
		glPopMatrix();

		glUseProgramObjectARB(0);

		glActiveTextureARB(GL_TEXTURE1_ARB);
		curShadowmap.disable();
		glActiveTextureARB(GL_TEXTURE0_ARB);
		curLightmap.disable();
		
		glActiveTextureARB(GL_TEXTURE0_ARB);
		curLightmap.bind();
		glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, winWidth, winHeight);
		curLightmap.disable();
	
		pbuffer_rgb.Deactivate();

		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();
}

/*--------------------------------------------------------------------------
for pBuffer control out of OpenGLVisualRenderer
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/ 
void ShadowRendering::activatergbpBuffer()
{
	pbuffer_rgb.Activate();
}

/*--------------------------------------------------------------------------
object colors, textures, transparency and backgound
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/ 
void ShadowRendering::renderBaseTexture(MediaObjectProxy * pSceneRenderer,
																				const Matrix& Transformation, 
																				const CullFlag& CullFlag,
																				OpenGLVisualRenderer* pRenderer, 
																				DWORD dwTime)
{
	// kein ClearColorBuffer wegen Hintergrund!!!
	// pbuffer_rgb.Activate();
	obj_linear_texgen();
/*
	glPushMatrix();
  obj_linear_texgen();    
  texgen(true);
  glPopMatrix();
*/
	//::glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_DECAL);

	::glDisable(GL_LIGHT0);
	::glDisable(GL_LIGHTING);
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, maximal);

	pSceneRenderer->CullOrRender(Transformation, CullFlag, pRenderer, dwTime);
	glActiveTextureARB(GL_TEXTURE0_ARB);
	basetexture.bind();
	glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0,  winWidth, winHeight);
	
	glActiveTextureARB(GL_TEXTURE0_ARB);
	basetexture.disable();
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, zero);
	
  pbuffer_rgb.Deactivate();
}

/*--------------------------------------------------------------------------
summ of all ambient lights
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/ 
void ShadowRendering::renderAmbientLight(MediaObjectProxy * pSceneRenderer,
																				const Matrix& Transformation, 
																				const CullFlag& CullFlag,
																				OpenGLVisualRenderer* pRenderer, 
																				DWORD dwTime)		
{
	pbuffer_rgb.Activate();
	glActiveTextureARB(GL_TEXTURE0_ARB);

	glClearColor(0.0f, 0.0f, 0.0f, 1.0f);
	glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);
	
	ambLightColorSum[0] = ambLightColor[0] / amb_light_num_max;
	ambLightColorSum[1] = ambLightColor[1] / amb_light_num_max;
	ambLightColorSum[2] = ambLightColor[2] / amb_light_num_max;
	ambLightColorSum[3] = ambLightColor[3] / amb_light_num_max;

	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, ambLightColorSum);

	glUseProgramObjectARB(ambientLight);

	pSceneRenderer->CullOrRender(Transformation, CullFlag, pRenderer, dwTime);

	ambientlight.bind();
	glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, winWidth, winHeight);
	
	glUseProgramObjectARB(0);

	glActiveTextureARB(GL_TEXTURE0_ARB);
	ambientlight.disable();
	glLightModelfv(GL_LIGHT_MODEL_AMBIENT, zero);
	
	pbuffer_rgb.Deactivate();
}
 
/*--------------------------------------------------------------------------
empty ambient lightmap
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::renderAmbientBlack()	//falls ambient disabled	
{
	pbuffer_rgb.Activate();

	glActiveTextureARB(GL_TEXTURE0_ARB);
	
	texgen(true);
	obj_linear_texgen();

	// for non ambient lighting, ambient lightmap needs to be completely black (+)
	glClearColor(0.0f,0.0f,0.0f,1.0f);
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	ambientlight.bind();
	glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, winWidth, winHeight);
	ambientlight.disable();

	pbuffer_rgb.Deactivate();
}

/*--------------------------------------------------------------------------
depthMap from camera point of view for fog calculation
 - added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::renderCameraDepth(MediaObjectProxy * pSceneRenderer,
																	 const Matrix& Transformation, 
																	 const CullFlag& CullFlag,
																	 OpenGLVisualRenderer* pRenderer, 
																	 DWORD dwTime)	
{
	pbuffer_depthmap.Activate();
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	glActiveTextureARB(GL_TEXTURE0_ARB);
	
	//
	//--> OpenGL fog Parameter aud fogProxy setzen fr Shader
	//

	glColorMask(0,0,0,0);
	::glDisable(GL_LIGHTING);
	::glDisable(GL_LIGHT0);
	
	pSceneRenderer->CullOrRender(Transformation, CullFlag, pRenderer, dwTime);

	glColorMask(1,1,1,1);
	
	depthmap_camera.bind();
	glCopyTexSubImage2D(GL_TEXTURE_2D, 0,   0, 0,   0, 0,  winWidth, winHeight);

	glActiveTextureARB(GL_TEXTURE0_ARB);
	depthmap_camera.disable();
	pbuffer_depthmap.Deactivate();
}

/*--------------------------------------------------------------------------
compositing of all diffuse sceneMaps (0-7)
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/ 
void ShadowRendering::addDiffLights()				//alle SceneMaps
{
		pbuffer_rgb.Activate();
		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
/*
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glLoadIdentity();	
		
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();
*/
		glActiveTextureARB(GL_TEXTURE0_ARB);
		diff_lightmap_light0.bind();

		glActiveTextureARB(GL_TEXTURE1_ARB);
		diff_lightmap_light1.bind();
	
		glActiveTextureARB(GL_TEXTURE2_ARB);
		diff_lightmap_light2.bind();

		glActiveTextureARB(GL_TEXTURE3_ARB);
		diff_lightmap_light3.bind();
		
		glUseProgramObjectARB(composeDiffLights);

		glUniform1iARB(glGetUniformLocationARB(composeDiffLights, "diff_lightmap_light0"), 0);
		glUniform1iARB(glGetUniformLocationARB(composeDiffLights, "diff_lightmap_light1"), 1);
		glUniform1iARB(glGetUniformLocationARB(composeDiffLights, "diff_lightmap_light2"), 2);
		glUniform1iARB(glGetUniformLocationARB(composeDiffLights, "diff_lightmap_light3"), 3);
		
		glPushMatrix();
		glBegin(GL_QUADS);
			glNormal3f(0, 0, 1);
			glTexCoord2f(0.0f, 0.0f);
			glVertex3f(-1.0f, -1.0f,  0.0f);
			glTexCoord2f(1.0, 0.0f);
			glVertex3f( 1.0f, -1.0f,  0.0f);
			glTexCoord2f(1.0, 1.0);
			glVertex3f( 1.0f,  1.0f,  0.0f);
			glTexCoord2f(0.0f, 1.0);
			glVertex3f(-1.0f,  1.0f,  0.0f);
			glEnd();
		glPopMatrix();

		glUseProgramObjectARB(0);

		glActiveTextureARB(GL_TEXTURE3_ARB);
		diff_lightmap_light3.disable();
	
		glActiveTextureARB(GL_TEXTURE2_ARB);
		diff_lightmap_light2.disable();

		glActiveTextureARB(GL_TEXTURE1_ARB);
		diff_lightmap_light1.disable();

		glActiveTextureARB(GL_TEXTURE0_ARB);
		diff_lightmap_light0.disable();
		
		glActiveTextureARB(GL_TEXTURE0_ARB);
		diff_scenemap.bind();

		glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, winWidth, winHeight);
		diff_scenemap.disable();
	
		pbuffer_rgb.Deactivate();

		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();
}
 

/*--------------------------------------------------------------------------
compositing of all specular sceneMaps (0-7)
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/ 
void ShadowRendering::addSpecLights()				//alle SceneMaps
{
		pbuffer_rgb.Activate();
		glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
/*
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glLoadIdentity();	
		
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();
*/
		glActiveTextureARB(GL_TEXTURE0_ARB);
		spec_lightmap_light0.bind();

		glActiveTextureARB(GL_TEXTURE1_ARB);
		spec_lightmap_light1.bind();
	
		glActiveTextureARB(GL_TEXTURE2_ARB);
		spec_lightmap_light2.bind();

		glActiveTextureARB(GL_TEXTURE3_ARB);
		spec_lightmap_light3.bind();
		
		glUseProgramObjectARB(composeSpecLights);

		glUniform1iARB(glGetUniformLocationARB(composeSpecLights, "spec_lightmap_light0"), 0);
		glUniform1iARB(glGetUniformLocationARB(composeSpecLights, "spec_lightmap_light1"), 1);
		glUniform1iARB(glGetUniformLocationARB(composeSpecLights, "spec_lightmap_light2"), 2);
		glUniform1iARB(glGetUniformLocationARB(composeSpecLights, "spec_lightmap_light3"), 3);
		
		glPushMatrix();
		glBegin(GL_QUADS);
			glNormal3f(0, 0, 1);
			glTexCoord2f(0.0f, 0.0f);
			glVertex3f(-1.0f, -1.0f,  0.0f);
			glTexCoord2f(1.0, 0.0f);
			glVertex3f( 1.0f, -1.0f,  0.0f);
			glTexCoord2f(1.0, 1.0);
			glVertex3f( 1.0f,  1.0f,  0.0f);
			glTexCoord2f(0.0f, 1.0);
			glVertex3f(-1.0f,  1.0f,  0.0f);
			glEnd();
		glPopMatrix();

		glUseProgramObjectARB(0);

		glActiveTextureARB(GL_TEXTURE3_ARB);
		spec_lightmap_light3.disable();

		glActiveTextureARB(GL_TEXTURE2_ARB);
		spec_lightmap_light2.disable();

		glActiveTextureARB(GL_TEXTURE1_ARB);
		spec_lightmap_light1.disable();

		glActiveTextureARB(GL_TEXTURE0_ARB);
		spec_lightmap_light0.disable();
		
		glActiveTextureARB(GL_TEXTURE0_ARB);
		spec_scenemap.bind();

		glCopyTexSubImage2D(GL_TEXTURE_2D, 0, 0, 0, 0, 0, winWidth, winHeight);
		spec_scenemap.disable();
	
		pbuffer_rgb.Deactivate();

		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();
}

/*--------------------------------------------------------------------------
final compositing, fog calculation
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::composeEverything()			//SceneMap+Fog+Base+Amb...
{
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);
	
/*
		glMatrixMode(GL_MODELVIEW);
		glPushMatrix();
		glLoadIdentity();	
		
		glMatrixMode(GL_PROJECTION);
		glPushMatrix();
		glLoadIdentity();
*/

		//glViewport(0, 0, winWidth, winHeight);

		if (fogType == true)
			{
			glFogi(GL_FOG_MODE, GL_LINEAR);
			}
		else
			{
			glFogi(GL_FOG_MODE, GL_EXP);
			}
		
		glFogfv(GL_FOG_COLOR, fogColor);
		glFogf(GL_FOG_DENSITY, 1.0);
		glFogf(GL_FOG_START, 0.0);
		glFogf(GL_FOG_END, 	fogVisRange);
	
		glActiveTextureARB(GL_TEXTURE0_ARB);
		diff_scenemap.bind();

		glActiveTextureARB(GL_TEXTURE1_ARB);
		basetexture.bind();
	
		glActiveTextureARB(GL_TEXTURE2_ARB);
		spec_scenemap.bind();

		glActiveTextureARB(GL_TEXTURE3_ARB);
		ambientlight.bind();

		glActiveTextureARB(GL_TEXTURE4_ARB);
		depthmap_camera.bind();
		
		glUseProgramObjectARB(composeScene);

		glUniform1iARB(glGetUniformLocationARB(composeScene, "diff_scenemap")		, 0);
		glUniform1iARB(glGetUniformLocationARB(composeScene, "basetexture")			, 1);
		glUniform1iARB(glGetUniformLocationARB(composeScene, "spec_scenemap")		, 2);
		glUniform1iARB(glGetUniformLocationARB(composeScene, "ambientlight")		, 3);
		glUniform1iARB(glGetUniformLocationARB(composeScene, "depthmap_camera")	, 4);
	
		glPushMatrix();
		glBegin(GL_QUADS);
			glNormal3f(0, 0, 1);
			glTexCoord2f(0.0f, 0.0f);
			glVertex3f(-1.0f, -1.0f,  0.0f);
			glTexCoord2f(1.0, 0.0f);
			glVertex3f( 1.0f, -1.0f,  0.0f);
			glTexCoord2f(1.0, 1.0);
			glVertex3f( 1.0f,  1.0f,  0.0f);
			glTexCoord2f(0.0f, 1.0);
			glVertex3f(-1.0f,  1.0f,  0.0f);
			glEnd();
		glPopMatrix();
		
		glUseProgramObjectARB(0);
	
		glActiveTextureARB(GL_TEXTURE4_ARB);
		depthmap_camera.disable();
		glActiveTextureARB(GL_TEXTURE3_ARB);
		ambientlight.disable();
		glActiveTextureARB(GL_TEXTURE2_ARB);
		depthmap_camera.disable();
		glActiveTextureARB(GL_TEXTURE1_ARB);
		basetexture.disable();
		glActiveTextureARB(GL_TEXTURE0_ARB);
		diff_scenemap.disable();

		glMatrixMode(GL_PROJECTION);
		glPopMatrix();
		glMatrixMode(GL_MODELVIEW);
		glPopMatrix();
}

/*--------------------------------------------------------------------------
paintTexture,  paint a multipasstexture of your choice
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::paintTexture()
{
	glClear(GL_COLOR_BUFFER_BIT|GL_DEPTH_BUFFER_BIT);

	glDisable(GL_DEPTH_TEST);

	glMatrixMode( GL_PROJECTION );
  glPushMatrix();
  glLoadIdentity();
  //gluOrtho2D( 0, winWidth-1, 0, winHeight-1 );
  glMatrixMode( GL_MODELVIEW );
  glPushMatrix();
  glLoadIdentity();
	
	//glMatrixMode( GL_PROJECTION );
	//gluPerspective(60, 1, .1, 10);
	//gluOrtho2D(-1, 1, -1, 1);
	
/*	
	glPushMatrix();
	obj_linear_texgen();  
  texgen(true);
  glPopMatrix();
*/	
	
	glMatrixMode(GL_TEXTURE);
	glPushMatrix();
	glLoadIdentity();
	texgen(false);
	
	glTranslatef(.5f, .5f, .5f);
  glScalef(.5f, .5f, .5f);
  /*
		glTexCoord3s(0, 0, 0);
	*/

	glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_MODULATE);
	glColor3f(1.0,1.0,1.0);
  glDisable(GL_LIGHTING);
	
	glActiveTextureARB(GL_TEXTURE0_ARB);
//	ambientlight.bind();
//  ambientlight.enable();
//	diff_scenemap.bind();
//	diff_scenemap.enable();
//	spec_scenemap.bind();
//  spec_scenemap.enable();
//shadowmap_light0.bind();
//shadowmap_light0.enable();
	//basetexture.bind();
  //basetexture.enable();

	/*
	float aspect = winWidth / winHeight;
	// note texture coordinates use image dimensions, not [0,1]
	//glPushMatrix();
	if(aspect > 1)
		glScalef(1, 1/aspect, 1);
	else
		glScalef(aspect, 1, 1);
	*/
	
glMatrixMode(GL_MODELVIEW);
	glEnable(GL_TEXTURE_2D);
    glBegin( GL_QUADS );
				glTexCoord2f(0, 0);
				glVertex2f(-1, -1);
				glTexCoord2f(winWidth, 0);
				glVertex2f(1, -1);
				glTexCoord2f(winWidth, winHeight);
				glVertex2f( 1,  1);
				glTexCoord2f(0, winHeight);
				glVertex2f( -1, 1);
    glEnd();
  glDisable(GL_TEXTURE_2D);
	
	//glMatrixMode(GL_PROJECTION);
	//glPopMatrix();
	//glMatrixMode(GL_MODELVIEW);
	//glPopMatrix();
	
	glActiveTextureARB(GL_TEXTURE0_ARB);
//	ambientlight.disable();
// diff_scenemap.disable();
//	shadowmap_light0.disable();
	spec_scenemap.disable();
//	basetexture.disable();
}


/*--------------------------------------------------------------------------
draw Framerate
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/

void ShadowRendering::drawFramerate()
{
	int ww = winWidth;
	int wh = winHeight;

	glDisable(GL_DEPTH_TEST);

	glMatrixMode( GL_PROJECTION );
	glPushMatrix();
	glLoadIdentity();
	gluOrtho2D(0, ww - 1, 0, wh - 1);
	glMatrixMode(GL_MODELVIEW);
	glPushMatrix();
	glLoadIdentity();

	glColor3f(1, 0, 0);

	int x = 20;
	int y = 22;
	glRasterPos2i(x, y);

	char string[] = "fps = %.1f ";
	char tmp[1024];

	sprintf(tmp, string, fps);

	char *p;

	for (p = tmp; *p; p++) {
		if (*p == '\n') {
			y = y - 14;
			glRasterPos2i( x, y );
			continue;
		}
		glutBitmapCharacter(GLUT_BITMAP_9_BY_15, *p);
	}

	glMatrixMode( GL_PROJECTION );
	glPopMatrix();
	glMatrixMode( GL_MODELVIEW );
	glPopMatrix();

	glEnable(GL_DEPTH_TEST);
}

/*--------------------------------------------------------------------------
calculate Frame-Rate
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/

void ShadowRendering::calculateFramerate()
{
	if (counterEnabled) 
		{
		++frames;

		static int last_time = 0;
		int this_time = glutGet(GLUT_ELAPSED_TIME);
		if (this_time - last_time > 500) {
			fps = (float)frames * 1000 / (this_time - last_time);
			frames = 0;
			last_time = this_time;
			}
		drawFramerate();
		}
}


/*--------------------------------------------------------------------------
****************************************************************************
--------------------------------------------------------------------------*/

/*--------------------------------------------------------------------------
texgen test
- added by M.Frank 11-2005 
--------------------------------------------------------------------------*/
void ShadowRendering::configTexgen_v2(int rangeMap, int bits)
{
  GLfloat p[4];
  GLfloat wrapScale;
  GLdouble m1[16], m2[16];

  copyMatrix(m1, RSmatrix);
  wrapScale = (1 << bits);
  m1[0] *= wrapScale;
  m1[4] *= wrapScale;
  m1[8] *= wrapScale;
  m1[12] *= wrapScale;

  copyMatrix(m1, Smatrix);
    
	curDepthmap.bind();
	curDepthmap.enable();

  multMatrices(m2, m1, lightFrustumMatrix);
  multMatrices(m1, m2, lightViewMatrix);

  p[0] = m1[0];
  p[1] = m1[4];
  p[2] = m1[8];
  p[3] = m1[12];
  ::glTexGenfv(GL_S, GL_EYE_PLANE, p);
  p[0] = m1[3];
  p[1] = m1[7];
  p[2] = m1[11];
  p[3] = m1[15];
  ::glTexGenfv(GL_Q, GL_EYE_PLANE, p);

  p[0] = m1[1];
  p[1] = m1[5];
  p[2] = m1[9];
  p[3] = m1[13];
  ::glTexGenfv(GL_T, GL_EYE_PLANE, p);

  p[0] = m1[2];
  p[1] = m1[6];
  p[2] = m1[10];
  p[3] = m1[14];
  ::glTexGenfv(GL_R, GL_EYE_PLANE, p);

	 ::glEnable(GL_TEXTURE_GEN_S);
   ::glEnable(GL_TEXTURE_GEN_S);
   ::glEnable(GL_TEXTURE_GEN_T);
   ::glEnable(GL_TEXTURE_GEN_R);
}

void ShadowRendering::disableTexgen_v2(void)
{
  ::glDisable(GL_TEXTURE_GEN_S);
  ::glDisable(GL_TEXTURE_GEN_T);
  ::glDisable(GL_TEXTURE_GEN_R);
  ::glDisable(GL_TEXTURE_GEN_Q);
}


/*--------------------------------------------------------------------------
****************************************************************************
--------------------------------------------------------------------------*/









void ShadowRendering::initExtensions(void)
{
  //int missingRequiredExtension = 0;  /* Start optimistic. */
  int cx;
  hasOpenGL12 = supportsOneDotTwo();
  
   
  
  cx = glExtensionSupported("GL_ARB_multitexture");
  if (!glExtensionSupported("GL_ARB_multitexture")) {
    //printf("I require the GL_ARB_multitexture OpenGL extension to work.\n");
    missingRequiredExtension = 1;
  }
  if (!glExtensionSupported("GL_EXT_bgra")) {
    //printf("I require the GL_EXT_bgra OpenGL extension to work.\n");
    missingRequiredExtension = 1;
  }
  if (glExtensionSupported("GL_NV_register_combiners")) {
    hasRegisterCombiners = 1;
  }
  if (glExtensionSupported("GL_EXT_texture_env_combine")) {
    hasTextureEnvCombine = 1;
  }
  if (glExtensionSupported("GL_SGIX_shadow")) {
    hasShadow = 1;
  }
#ifdef _WIN32
  if (glExtensionSupported("WGL_EXT_swap_control")) {
    hasSwapControl = 1;
  }
#endif
  if (glExtensionSupported("GL_SGIX_depth_texture")) {
    hasDepthTexture = 1;
  }
  if (glExtensionSupported("GL_ARB_texture_border_clamp")) {
    hasTextureBorderClamp = 1;
  }
  if (hasOpenGL12 || glExtensionSupported("GL_EXT_texture_edge_clamp")) {
    hasTextureEdgeClamp = 1;
  }
  if (hasOpenGL12 || glExtensionSupported("GL_EXT_separate_specular_color")) {
    hasSeparateSpecularColor = 1;
  }
//  if (glutExtensionSupported("GL_NV_texture_rectangle")) {
//    hasTextureRectangle = 1;
//    glGetIntegerv(GL_MAX_RECTANGLE_TEXTURE_SIZE_NV, &maxRectangleTextureSize);
//  }

  if (forceEnvCombine) {
    //printf("Forcing use of GL_EXT_texture_env_combine.\n");
    hasRegisterCombiners = 0;
    if (!hasTextureEnvCombine) {
      //printf("I require the following OpenGL extension to work:\n"
      //       "  GL_EXT_texture_env_combine\n");
      missingRequiredExtension = 1;
    }
  } else {
    if (!hasRegisterCombiners && !hasTextureEnvCombine) {
      //printf("I require one of the following OpenGL extensions to work:\n"
      //       "  GL_EXT_texture_env_combine\n"
      //       "  GL_NV_register_combiners\n");
      missingRequiredExtension = 1;
    }
  }
  if (missingRequiredExtension) {
    //printf("\nshadowcast: "
    //       "Exiting because required OpenGL extensions are not supported.\n");
    exit(1);
  }
  if (hasShadow && hasDepthTexture &&
      hasRegisterCombiners && hasSeparateSpecularColor) {
    //printf("shadowcast: has hardware shadow mapping support.\n");
    hasShadowMapSupport = 1;
    useShadowMapSupport = 1;
  }
#if defined(NEEDS_GET_PROC_ADDRESS)

# if defined(macintosh)
  glGetPointerv(GL_MAC_GET_PROC_ADDRESS_NV, (GLvoid*) &macGetProcAddress);
# endif

# if !defined(macintosh) 
  /* Retrieve some ARB_multitexture routines. */
  ::glMultiTexCoord2iARB =
    (PFNGLMULTITEXCOORD2IARBPROC)
    GET_PROC_ADDRESS("glMultiTexCoord2iARB");
  ::glMultiTexCoord2fvARB =
    (PFNGLMULTITEXCOORD2FVARBPROC)
    GET_PROC_ADDRESS("glMultiTexCoord2fvARB");
  ::glMultiTexCoord3fARB =
    (PFNGLMULTITEXCOORD3FARBPROC)
    GET_PROC_ADDRESS("glMultiTexCoord3fARB");
  ::glMultiTexCoord3fvARB =
    (PFNGLMULTITEXCOORD3FVARBPROC)
    GET_PROC_ADDRESS("glMultiTexCoord3fvARB");
  ::glActiveTextureARB =
    (PFNGLACTIVETEXTUREARBPROC)
    GET_PROC_ADDRESS("glActiveTextureARB");
# endif

  if (hasRegisterCombiners) {
    /* Retrieve all NV_register_combiners routines. */
	  ::glCombinerParameterfvNV =
      (PFNGLCOMBINERPARAMETERFVNVPROC)
      GET_PROC_ADDRESS("glCombinerParameterfvNV");
	  ::glCombinerParameterivNV =
      (PFNGLCOMBINERPARAMETERIVNVPROC)
      GET_PROC_ADDRESS("glCombinerParameterivNV");
	  ::glCombinerParameterfNV =
      (PFNGLCOMBINERPARAMETERFNVPROC)
      GET_PROC_ADDRESS("glCombinerParameterfNV");
	  ::glCombinerParameteriNV =
      (PFNGLCOMBINERPARAMETERINVPROC)
      GET_PROC_ADDRESS("glCombinerParameteriNV");
	  ::glCombinerInputNV =
      (PFNGLCOMBINERINPUTNVPROC)
      GET_PROC_ADDRESS("glCombinerInputNV");
	  ::glCombinerOutputNV =
      (PFNGLCOMBINEROUTPUTNVPROC)
      GET_PROC_ADDRESS("glCombinerOutputNV");
	  ::glFinalCombinerInputNV =
      (PFNGLFINALCOMBINERINPUTNVPROC)
      GET_PROC_ADDRESS("glFinalCombinerInputNV");
	  ::glGetCombinerInputParameterfvNV =
      (PFNGLGETCOMBINERINPUTPARAMETERFVNVPROC)
      GET_PROC_ADDRESS("glGetCombinerInputParameterfvNV");
	  ::glGetCombinerInputParameterivNV =
      (PFNGLGETCOMBINERINPUTPARAMETERIVNVPROC)
      GET_PROC_ADDRESS("glGetCombinerInputParameterivNV");
	  ::glGetCombinerOutputParameterfvNV =
      (PFNGLGETCOMBINEROUTPUTPARAMETERFVNVPROC)
      GET_PROC_ADDRESS("glGetCombinerOutputParameterfvNV");
	  ::glGetCombinerOutputParameterivNV =
      (PFNGLGETCOMBINEROUTPUTPARAMETERIVNVPROC)
      GET_PROC_ADDRESS("glGetCombinerOutputParameterivNV");
	  ::glGetFinalCombinerInputParameterfvNV =
      (PFNGLGETFINALCOMBINERINPUTPARAMETERFVNVPROC)
      GET_PROC_ADDRESS("glGetFinalCombinerInputParameterfvNV");
	  ::glGetFinalCombinerInputParameterivNV =
      (PFNGLGETFINALCOMBINERINPUTPARAMETERIVNVPROC)
      GET_PROC_ADDRESS("glGetFinalCombinerInputParameterivNV");
  }

# ifdef _WIN32
  if (hasSwapControl) {
    /* Retrieve all WGL_EXT_swap_control routines. */
    wglSwapIntervalEXT = 
      (PFNWGLSWAPINTERVALEXTPROC)
      GET_PROC_ADDRESS("wglSwapIntervalEXT");
    wglGetSwapIntervalEXT = 
      (PFNWGLGETSWAPINTERVALEXTPROC)
      GET_PROC_ADDRESS("wglGetSwapIntervalEXT");
  }
# endif  /* _WIN32 */

#endif  /* NEEDS_GET_PROC_ADDRESS */

}



//Light-Functions

/* setAmbientAndDiffuseMaterial - update the ambient and diffuse
   material setting for per-vertex lighting, and if single-pass hardware
   shadow mapping is also supported, update a constant color in the
   NV_register_combiners with the global ambient. */
void ShadowRendering::setAmbientAndDiffuseMaterial(GLfloat *material)
{
  int i;

  glMaterialfv(GL_FRONT, GL_AMBIENT_AND_DIFFUSE, material);

  if (hasShadow && hasRegisterCombiners) {
    for (i=0; i<4; i++) {
      Ka[i] = material[i] * globalAmbientIntensity[i];
    }
    /* The single-pass hardware shadow mapping mode programs
       the register combiners to treat CONSTANT_COLOR1 as the
       global ambient. */
    glCombinerParameterfvNV(GL_CONSTANT_COLOR1_NV, Ka);
  }
}

//Setup Functions

void ShadowRendering::setupEyeView(void)
{
	::glMatrixMode(GL_PROJECTION);
	::glLoadMatrixd(eyeFrustumMatrix);

	::glMatrixMode(GL_MODELVIEW);
	::glLoadMatrixd(eyeViewMatrix);
}

void ShadowRendering::setupLightView(int square)
{
	::glMatrixMode(GL_PROJECTION);
  if (square) {
	  ::glLoadMatrixd(lightFrustumMatrix);
  } else {
    ::glLoadIdentity();
    ::glScalef(winAspectRatio, 1, 1);
    ::glMultMatrixd(lightFrustumMatrix);
  }

  ::glMatrixMode(GL_MODELVIEW);
  ::glLoadMatrixd(lightViewMatrix);
}

// Texture Functions

void ShadowRendering::configTexgen(int rangeMap, int bits)
{
  GLfloat p[4];
  GLfloat wrapScale;
  GLdouble m1[16], m2[16];

  if (rangeMap) {
    /* Generate the depth map space planar distance as the S/Q texture
       coordinate. */

    if ((depthMapPrecision == GL_UNSIGNED_SHORT) && !useShadowMapSupport) {
      ::glBindTexture(GL_TEXTURE_2D, TO_MAP_16BIT);
      ::glEnable(GL_TEXTURE_2D);
      
    } else {
      ::glBindTexture(GL_TEXTURE_1D, TO_MAP_8BIT);
      ::glEnable(GL_TEXTURE_1D);
      ::glDisable(GL_TEXTURE_2D);
      
    }

    copyMatrix(m1, RSmatrix);
    wrapScale = (1 << bits);
    m1[0] *= wrapScale;
    m1[4] *= wrapScale;
    m1[8] *= wrapScale;
    m1[12] *= wrapScale;
  } else {
    GLenum target;
    GLuint texobj;

    copyMatrix(m1, Smatrix);
    
     /* Pick the 2D target and texture object. */
      target = GL_TEXTURE_2D;
      if (useShadowMapSupport) {
        if (movingLight && quickLightMove) {
          texobj = TO_LOWRES_HW_DEPTH_MAP;
        } else {
          texobj = TO_HW_DEPTH_MAP;
        }
      } else {
        if (movingLight && quickLightMove) {
          texobj = TO_LOWRES_DEPTH_MAP;
        } else {
          texobj = TO_DEPTH_MAP;
        }
      }
   // }
    ::glBindTexture(target, texobj);
    ::glEnable(target);
  }

  multMatrices(m2, m1, lightFrustumMatrix);
  multMatrices(m1, m2, lightViewMatrix);

  /* S and Q are always needed */
  p[0] = m1[0];
  p[1] = m1[4];
  p[2] = m1[8];
  p[3] = m1[12];
  ::glTexGenfv(GL_S, GL_EYE_PLANE, p);
  p[0] = m1[3];
  p[1] = m1[7];
  p[2] = m1[11];
  p[3] = m1[15];
  ::glTexGenfv(GL_Q, GL_EYE_PLANE, p);

  ::glEnable(GL_TEXTURE_GEN_S);
  ::glEnable(GL_TEXTURE_GEN_Q);

  if (rangeMap && (bits == 0)) {
    /* rangeMap is a 1D mode so it does not need to generate T or R. */
    ::glDisable(GL_TEXTURE_GEN_T);
    ::glDisable(GL_TEXTURE_GEN_R);
  } else {
    if (useShadowMapSupport) {
      /* Hardware shadow mapping is a 2D texturing mode (uses S and T) that
         also compares with R so S, T, and R must be generated. */
      p[0] = m1[1];
      p[1] = m1[5];
      p[2] = m1[9];
      p[3] = m1[13];
      ::glTexGenfv(GL_T, GL_EYE_PLANE, p);

      p[0] = m1[2];
      p[1] = m1[6];
      p[2] = m1[10];
      p[3] = m1[14];
      ::glTexGenfv(GL_R, GL_EYE_PLANE, p);

      ::glEnable(GL_TEXTURE_GEN_S);
      ::glEnable(GL_TEXTURE_GEN_T);
      ::glEnable(GL_TEXTURE_GEN_R);
    } else {
      /* Standard 2D texture mapping uses S, T, and Q, but does not use R. */
      p[0] = m1[1];
      p[1] = m1[5];
      p[2] = m1[9];
      p[3] = m1[13];
      glTexGenfv(GL_T, GL_EYE_PLANE, p);

      ::glEnable(GL_TEXTURE_GEN_S);
      ::glEnable(GL_TEXTURE_GEN_T);
      ::glDisable(GL_TEXTURE_GEN_R);
    }
  }
}

void ShadowRendering::disableTexgen(void)
{
  ::glDisable(GL_TEXTURE_GEN_S);
  ::glDisable(GL_TEXTURE_GEN_T);
  ::glDisable(GL_TEXTURE_GEN_R);
  ::glDisable(GL_TEXTURE_GEN_Q);
  if (hasTextureRectangle) {
    ::glDisable(GL_TEXTURE_RECTANGLE_NV);
  }
  ::glDisable(GL_TEXTURE_2D);
  ::glDisable(GL_TEXTURE_1D);
}


// Combiners Functions

void ShadowRendering::configCombiners(int pass)
{
  if (hasRegisterCombiners) {
    if (pass == 1) {
      assert(depthMapPrecision == GL_UNSIGNED_SHORT ||
             depthMapPrecision == GL_UNSIGNED_BYTE);
      
      ::glCombinerParameteriNV(GL_NUM_GENERAL_COMBINERS_NV, 1);

      /* Discard entire RGB combiner stage 0 */
      ::glCombinerOutputNV(GL_COMBINER0_NV, GL_RGB,
        GL_DISCARD_NV, GL_DISCARD_NV, GL_DISCARD_NV,
        GL_NONE, GL_NONE, GL_FALSE, GL_FALSE, GL_FALSE);

      /* Aa = one */
      ::glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_A_NV,
        GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_ALPHA);
      /* Ba = texture0a = depth map MSB */
      ::glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_B_NV,
        GL_TEXTURE0_ARB, GL_SIGNED_IDENTITY_NV,
        littleEndian ? GL_ALPHA : GL_BLUE);
      /* Ca = one */
      ::glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_C_NV,
        GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_ALPHA);
      /* Da = texture1a = range MSB */
      ::glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_D_NV,
        GL_TEXTURE1_ARB, GL_SIGNED_NEGATE_NV, GL_ALPHA);

      /* spare0a = 1 * tex0a + 1 * -tex1a = tex0a - tex1a */
      ::glCombinerOutputNV(GL_COMBINER0_NV, GL_RGB,
        GL_DISCARD_NV, GL_DISCARD_NV, GL_SPARE0_NV,
        GL_NONE, GL_NONE, GL_FALSE, GL_FALSE, GL_FALSE);

    } else {
      assert(pass == 2);
      assert(depthMapPrecision == GL_UNSIGNED_SHORT);

      ::glCombinerParameteriNV(GL_NUM_GENERAL_COMBINERS_NV, 2);

      /* Argb = one */
      ::glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_A_NV,
        GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_RGB);
      /* Brgb = texture0rgb = depth map LSB */
      ::glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_B_NV,
        GL_TEXTURE0_ARB, GL_SIGNED_IDENTITY_NV,
        littleEndian ? GL_RGB : GL_ALPHA);
      /* Crgb = one */
      ::glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_C_NV,
        GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_RGB);
      /* Drgb = texture1rgb = range LSB */
      ::glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_D_NV,
        GL_TEXTURE1_ARB, GL_SIGNED_NEGATE_NV, GL_RGB);

      /* spare0rgb = 1 * tex0rgb + 1 * -tex1rgb = tex0rgb - tex1rgb */
      ::glCombinerOutputNV(GL_COMBINER0_NV, GL_RGB,
        GL_DISCARD_NV, GL_DISCARD_NV, GL_SPARE0_NV,
        GL_NONE, GL_NONE, GL_FALSE, GL_FALSE, GL_FALSE);

      /* Aa = one */
      ::glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_A_NV,
        GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_ALPHA);
      /* Ba = texture1a = range MSB */
      ::glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_B_NV,
        GL_TEXTURE1_ARB, GL_UNSIGNED_IDENTITY_NV, GL_ALPHA);
      /* Ca = one */
      ::glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_C_NV,
        GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_ALPHA);
      /* Da = texture0a = depth map MSB */
      ::glCombinerInputNV(GL_COMBINER0_NV, GL_ALPHA, GL_VARIABLE_D_NV,
        GL_TEXTURE0_ARB, GL_UNSIGNED_INVERT_NV,
        littleEndian ? GL_ALPHA : GL_BLUE);

      /* spare0a = 1 * tex1a + 1 * (1-tex0a) - 0.5 = tex1a - tex0a + 0.5 */
      ::glCombinerOutputNV(GL_COMBINER0_NV, GL_ALPHA,
        GL_DISCARD_NV, GL_DISCARD_NV, GL_SPARE0_NV,
        GL_NONE, GL_BIAS_BY_NEGATIVE_ONE_HALF_NV,
        GL_FALSE, GL_FALSE, GL_FALSE);

      /* Discard entire RGB combiner stage 1 */
      ::glCombinerOutputNV(GL_COMBINER1_NV, GL_RGB,
        GL_DISCARD_NV, GL_DISCARD_NV, GL_DISCARD_NV,
        GL_NONE, GL_NONE, GL_FALSE, GL_FALSE, GL_FALSE);

      /* Aa = one */
      ::glCombinerInputNV(GL_COMBINER1_NV, GL_ALPHA, GL_VARIABLE_A_NV,
        GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_ALPHA);
      /* Ba = spare0b = tex0b - tex1b */
      ::glCombinerInputNV(GL_COMBINER1_NV, GL_ALPHA, GL_VARIABLE_B_NV,
        GL_SPARE0_NV, GL_SIGNED_IDENTITY_NV, GL_BLUE);
      /* Ca = zero */
      ::glCombinerInputNV(GL_COMBINER1_NV, GL_ALPHA, GL_VARIABLE_C_NV,
        GL_ZERO, GL_UNSIGNED_IDENTITY_NV, GL_ALPHA);
      /* Da = zero */
      ::glCombinerInputNV(GL_COMBINER1_NV, GL_ALPHA, GL_VARIABLE_D_NV,
        GL_ZERO, GL_UNSIGNED_IDENTITY_NV, GL_ALPHA);

      /* spare0a = (tex1 - tex0 + 0.5) > 0.5 ? 0 : tex0b - tex1b */
      ::glCombinerOutputNV(GL_COMBINER1_NV, GL_ALPHA,
        GL_DISCARD_NV, GL_DISCARD_NV, GL_SPARE0_NV,
        GL_NONE, GL_NONE, GL_FALSE, GL_FALSE,
        GL_TRUE);  /* Use the MUX operation. */
    }

    /* A = one */
    ::glFinalCombinerInputNV(GL_VARIABLE_A_NV,
      GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_RGB);
    /* B = primary color */
    ::glFinalCombinerInputNV(GL_VARIABLE_B_NV,
      GL_PRIMARY_COLOR_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
    /* C = zero */
    ::glFinalCombinerInputNV(GL_VARIABLE_C_NV,
      GL_ZERO, GL_UNSIGNED_IDENTITY_NV, GL_RGB); //Test GL_FOG
    /* D = zero */
    ::glFinalCombinerInputNV(GL_VARIABLE_D_NV,
      GL_ZERO, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
    /* RGB = A*B + (1-A)*C + D = primary color */

    /* G = spare0a = max(0, tex0 - tex1) */
    ::glFinalCombinerInputNV(GL_VARIABLE_G_NV,
      GL_SPARE0_NV, GL_UNSIGNED_IDENTITY_NV, GL_ALPHA);
    /* Alpha = max(0, tex0 - tex1) */

    ::glEnable(GL_REGISTER_COMBINERS_NV);
  } else {
	  ::glActiveTextureARB(GL_TEXTURE0_ARB);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);

    ::glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_REPLACE);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_PRIMARY_COLOR_EXT);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_COLOR);

    ::glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA_EXT, GL_REPLACE);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA_EXT, GL_TEXTURE);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA_EXT, GL_SRC_ALPHA);

    ::glActiveTextureARB(GL_TEXTURE1_ARB);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);

    ::glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_REPLACE);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_PREVIOUS_EXT);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_COLOR);

    ::glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_ALPHA_EXT, GL_ADD_SIGNED_EXT);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_ALPHA_EXT, GL_PREVIOUS_EXT);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_ALPHA_EXT, GL_SRC_ALPHA);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE1_ALPHA_EXT, GL_TEXTURE);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND1_ALPHA_EXT, GL_ONE_MINUS_SRC_ALPHA);
  }
 
}

void ShadowRendering::configCombinersForHardwareShadowPass(int withTexture)
{
	::glCombinerParameteriNV(GL_NUM_GENERAL_COMBINERS_NV, 2);
	::glCombinerParameterfvNV(GL_CONSTANT_COLOR0_NV, lightDimColor);
	::glCombinerParameterfvNV(GL_CONSTANT_COLOR1_NV, Ka);

  /* Argb = one */
	::glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_A_NV,
    GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_RGB);
  /* Brgb = constant0rgb = light dimming */
	::glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_B_NV,
    GL_CONSTANT_COLOR0_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* Crgb = 1.0 - constant0rgb = 1.0 - light dimming */
	::glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_C_NV,
    GL_CONSTANT_COLOR0_NV, GL_UNSIGNED_INVERT_NV, GL_RGB);
  /* Drgb = texture0rgb = unshadowed percentage */
	::glCombinerInputNV(GL_COMBINER0_NV, GL_RGB, GL_VARIABLE_D_NV,
    GL_TEXTURE0_ARB, GL_UNSIGNED_IDENTITY_NV, GL_RGB);

  /* spare0rgb = light dimming + (1.0 - light dimming) * unshadowed percentage */
	::glCombinerOutputNV(GL_COMBINER0_NV, GL_RGB,
    GL_DISCARD_NV, GL_DISCARD_NV, GL_SPARE0_NV,
    GL_NONE, GL_NONE, GL_FALSE, GL_FALSE, GL_FALSE);

  /* Discard entire Alpha combiner stage 0 */
	::glCombinerOutputNV(GL_COMBINER0_NV, GL_ALPHA,
    GL_DISCARD_NV, GL_DISCARD_NV, GL_DISCARD_NV,
    GL_NONE, GL_NONE,
    GL_FALSE, GL_FALSE, GL_FALSE);

  /* Argb = spare0rgb = light dimming + (1.0 - light dimming) * unshadowed percentage */
  ::glCombinerInputNV(GL_COMBINER1_NV, GL_RGB, GL_VARIABLE_A_NV,
    GL_SPARE0_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* Brgb = primary color = light's diffuse contribution */
  ::glCombinerInputNV(GL_COMBINER1_NV, GL_RGB, GL_VARIABLE_B_NV,
    GL_PRIMARY_COLOR_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* Crgb = constant 1 = Ka = ambient contribution */
  ::glCombinerInputNV(GL_COMBINER1_NV, GL_RGB, GL_VARIABLE_C_NV,
    GL_CONSTANT_COLOR1_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* Drgb = 1 */
  ::glCombinerInputNV(GL_COMBINER1_NV, GL_RGB, GL_VARIABLE_D_NV,
    GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_RGB);

  /* spare0rgb =
     (light dimming + (1.0 - light dimming) * unshadowed percentage) * diffuse +
     ambient */
  ::glCombinerOutputNV(GL_COMBINER1_NV, GL_RGB,
    GL_DISCARD_NV, GL_DISCARD_NV, GL_SPARE0_NV,
    GL_NONE, GL_NONE, GL_FALSE, GL_FALSE, GL_FALSE);

  /* Discard entire Alpha combiner stage 1 */
  ::glCombinerOutputNV(GL_COMBINER1_NV, GL_ALPHA,
    GL_DISCARD_NV, GL_DISCARD_NV, GL_DISCARD_NV,
    GL_NONE, GL_NONE, GL_FALSE, GL_FALSE,
    GL_TRUE);  /* Use the MUX operation. */
  
  if (withTexture) {
    /* A = texture1 = decal */
	  ::glFinalCombinerInputNV(GL_VARIABLE_A_NV,
      GL_TEXTURE1_ARB, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  } else {
    /* A = one */
	  ::glFinalCombinerInputNV(GL_VARIABLE_A_NV,
      GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_RGB);
  }
  /* B = spare0 */
  ::glFinalCombinerInputNV(GL_VARIABLE_B_NV,
    GL_SPARE0_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* C = zero */
  ::glFinalCombinerInputNV(GL_VARIABLE_C_NV,
    GL_ZERO, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* E = unshadowed percentage */
  ::glFinalCombinerInputNV(GL_VARIABLE_E_NV,
    GL_TEXTURE0_ARB, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* F = specular */
  ::glFinalCombinerInputNV(GL_VARIABLE_F_NV,
    GL_SECONDARY_COLOR_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* D = EF product = unshadowed percentage * specular contribution */
  ::glFinalCombinerInputNV(GL_VARIABLE_D_NV,
    GL_E_TIMES_F_NV, GL_UNSIGNED_IDENTITY_NV, GL_RGB);
  /* RGB = A*B + (1-A)*C + D = (texture1 OR 1) * space0 + specular * unshadowed */

  /* G = 1 */
  ::glFinalCombinerInputNV(GL_VARIABLE_G_NV,
    GL_ZERO, GL_UNSIGNED_INVERT_NV, GL_ALPHA);
  /* Alpha = 1 */

  ::glEnable(GL_REGISTER_COMBINERS_NV);

}

void ShadowRendering::disableCombiners(void)
{
  if (hasRegisterCombiners) {
    ::glDisable(GL_REGISTER_COMBINERS_NV);
  }
}

// Update Function

void ShadowRendering::updateMatrices(void)
{
  buildPerspectiveMatrix(lightFrustumMatrix, 
    lightFieldOfView, 1.0, lightNear, lightFar);

  if (showLightViewFrustum) {
    invertMatrix(lightInverseViewMatrix, lightViewMatrix);
    invertMatrix(lightInverseFrustumMatrix, lightFrustumMatrix);
  }
}

void ShadowRendering::updateDepthBias(int delta)
{
  GLfloat scale, bias;

  if (useShadowMapSupport) {
    if (hwDepthMapPrecision == GL_UNSIGNED_SHORT) {
      depthBias16 += delta;
      scale = slopeScale;
      bias = depthBias16 * depthScale16;
    } else {
      depthBias24 += delta;
      scale = slopeScale;
      bias = depthBias24 * depthScale24;
    }
  } else {
    if (depthMapPrecision == GL_UNSIGNED_SHORT) {
      depthBias16 += delta;
      scale = slopeScale;
      bias = depthBias16 * depthScale16;
    } else {
      depthBias8 += delta;
      scale = slopeScale;
      bias = depthBias8 * depthScale8;
    }
  }
  ::glPolygonOffset(scale, bias);
  needTitleUpdate = 1;
  needDepthMapUpdate = 1;
}

void ShadowRendering::updateDepthMapSize(void)
{
  int oldDepthMapSize = depthMapSize;

  depthMapSize = requestedDepthMapSize;
  while ((winWidth < depthMapSize) || (winHeight < depthMapSize)) {
    depthMapSize >>= 1;  // Half the depth map size
  }
  if (depthMapSize < 1) {
    /* Just in case. */
    depthMapSize = 1;
  }
  if (depthMapSize != requestedDepthMapSize) {
    //printf("shadowcast: reducing depth map from %d to %d based on window size %dx%d\n",
    //  requestedDepthMapSize, depthMapSize, winWidth, winHeight);
  }
  if (!useTextureRectangle) {
    if (oldDepthMapSize != depthMapSize) {
      needDepthMapUpdate = 1;
      needTitleUpdate = 1;
      //glutPostRedisplay();
    }
  }
}

void ShadowRendering::updateDepthScale(void)
{
  GLint depthBits;

  glGetIntegerv(GL_DEPTH_BITS, &depthBits);
  if (depthBits < 24) {
    depthScale24 = 1;
  } else {
    depthScale24 = 1 << (depthBits - 24);
  }
  if (depthBits < 16) {
    depthScale16 = 1;
  } else {
    depthScale16 = 1 << (depthBits - 16);
  }
  if (depthBits < 8) {
    depthScale8 = 1;
  } else {
    depthScale8 = 1 << (depthBits - 8);
  }
  needDepthMapUpdate = 1;
}

void ShadowRendering::useBestShadowMapClamping(GLenum target)
{
  if (hasTextureBorderClamp) {
    ::glTexParameteri(target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_BORDER_ARB);
    ::glTexParameteri(target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_BORDER_ARB);
  } else {
    /* We really want "clamp to border", but this may be good enough. */
    if (hasTextureEdgeClamp) {
      ::glTexParameteri(target, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
      ::glTexParameteri(target, GL_TEXTURE_WRAP_T, GL_CLAMP_TO_EDGE);
    } else {
      /* A bad option, but still better than "repeat". */
      ::glTexParameteri(target, GL_TEXTURE_WRAP_S, GL_CLAMP);
      ::glTexParameteri(target, GL_TEXTURE_WRAP_T, GL_CLAMP);
    }
  }
}

void ShadowRendering::initShadowGL(void)
{
    GLubyte texmap[256*256*2];
        unsigned int i, j;
        GLint depthBits;

        #if defined(WIN32)
         if (hasSwapControl) {
          if (vsync) {
           wglSwapIntervalEXT(1);
		  } else {
         wglSwapIntervalEXT(0);
		  }
		 }
        #endif
   
  
		 ::glGetIntegerv(GL_DEPTH_BITS, &depthBits);
		
		if (depthBits >= 24) {
		hwDepthMapPrecision = GL_UNSIGNED_INT;
		hwDepthMapInternalFormat = GL_DEPTH_COMPONENT24_SGIX;
		}

		::glClearStencil(0);
		::glClearColor(0,0,0,0);

		::glPixelStorei(GL_UNPACK_ALIGNMENT, 1);
		::glPixelStorei(GL_PACK_ALIGNMENT, 1);

		::glTexGeni(GL_S, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
		::glTexGeni(GL_T, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
		::glTexGeni(GL_R, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);
		::glTexGeni(GL_Q, GL_TEXTURE_GEN_MODE, GL_EYE_LINEAR);

		::glLightModelfv(GL_LIGHT_MODEL_AMBIENT, globalAmbient);

		  // Make 8-bit identity texture that maps (s)=(z) to [0,255]/255. 
		for (i=0; i<256; i++) {
			texmap[i] = i;
			}
		::glBindTexture(GL_TEXTURE_1D, TO_MAP_8BIT);
		::glTexImage1D(GL_TEXTURE_1D, 0, GL_INTENSITY8,
			256, 0, GL_LUMINANCE, GL_UNSIGNED_BYTE, texmap);
		::glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		::glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		::glTexParameteri(GL_TEXTURE_1D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

		// Make 16-bit identity texture that maps (s,t)=(z,z*256) to [0,65535]/65535. //
		for (j=0; j<256; j++) {
			for (i=0; i<256; i++) {
			texmap[j*512+i*2+0] = j;  // Luminance has least sigificant bits. 
			texmap[j*512+i*2+1] = i;  // Alpha has most sigificant bits. 
			}
		}
		::glBindTexture(GL_TEXTURE_2D, TO_MAP_16BIT);
		::glTexImage2D(GL_TEXTURE_2D, 0, GL_LUMINANCE8_ALPHA8,
			256, 256, 0, GL_LUMINANCE_ALPHA, GL_UNSIGNED_BYTE, texmap);
		::glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_CLAMP_TO_EDGE);
		::glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		::glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
		::glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_NEAREST);

		// GL_LEQUAL ensures that when fragments with equal depth are
		//generated within a single rendering pass, the last fragment
		//results. 
		::glDepthFunc(GL_LEQUAL);
		::glEnable(GL_DEPTH_TEST);

		::glEnable(GL_LIGHTING);
		
		::glLineStipple(1, 0xf0f0);

		::glMaterialf(GL_FRONT, GL_SHININESS, 30.0);

		::glEnable(GL_CULL_FACE);
		::glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, 1);
		#if 000
		// This would work too. 
		::glEnable(GL_RESCALE_NORMAL_EXT);
		#else
		::glEnable(GL_NORMALIZE);
		#endif

		// Make the NVIDIA logo display list. 
		//  nvidiaLogoDList = makeNVidiaLogo(300);

  
		if (hasRegisterCombiners) {
			depthMapFormat = GL_LUMINANCE_ALPHA;
			depthMapInternalFormat = GL_LUMINANCE8_ALPHA8;
			depthMapPrecision = GL_UNSIGNED_SHORT;
		}
		updateDepthScale();
		updateDepthBias(0);  // Update with no offset change. 

		if (drawFront) {
		::glDrawBuffer(GL_FRONT);
		::glReadBuffer(GL_FRONT);
		}

		
}


void ShadowRendering::updateDepthMap(MediaObjectProxy * pSceneRenderer, const Matrix& rTransformation, 
                      const CullFlag& rCullFlag,
		              OpenGLVisualRenderer* pRenderer, 
                      DWORD dwTime)
{

/* Lazily update the depth map only if there is some indication (ie,
     needDepthMapUpdate is set) that the scene has changed.  This
     allows the scene to be viewed from different orientations without
     regenerating the depth map.  This optimization is possible because
     shadows are viewer independent. */
  if (needDepthMapUpdate) {
    
    /* Buffer for reading the depth buffer into. */
    static int depthBufferSize = 0;
    static GLubyte *depthBuffer;// = NULL;

    static int lastLowResWidth = 0, lastHiResWidth = 0;
    static int lastLowResHwWidth = 0, lastHiResHwWidth = 0;

    static int lastLowResHeight = 0, lastHiResHeight = 0;
    static int lastLowResHwHeight = 0, lastHiResHwHeight = 0;

    static GLenum lastLowResPrecision = 0, lastHiResPrecision = 0;
    static GLenum lastLowResHwPrecision = 0, lastHiResHwPrecision = 0;
    static GLenum lastLowResHwFiltering = 0, lastHiResHwFiltering = 0;

    int bytesPerDepthValue;
    int requiredSize;

    int width, height, lowRes;
    GLenum target;  /* Either 2D or rectangular. */
    GLuint texobj;

    if (movingLight && quickLightMove) {
      lowRes = 1;
      
        width = depthMapSize >> 1;
        height = depthMapSize >> 1;
      
      if (width < 1) {
        /* Just to be safe. */
        width = 1;
      }
      if (height < 1) {
        /* Just to be safe. */
        height = 1;
      }
    } else {
      lowRes = 0;
      
        width = depthMapSize;
        height = depthMapSize;
      
    }

    ::glViewport(0, 0, width, height);

    /* Setup light view with a square aspect ratio
       since the texture is square. */
    setupLightView(1);

    if (!depthMapBackFaceCulling) {
      /* Often it works to enable back face culling during the depth map
         construction stage.  Back face culling has the standard
         performance advantage of not rasterizing back facing polygons.
         However, there are two reasons to avoid back face culling
         when constructing the depth map.  The first issue involves
         what happens when a closed object is intersected by the near
         clip plane.  In this case, back facing polygons are discarded
         leaving a "hole" in the shadow map.  The second issue is what
         happens when front facing polygons have extreme slopes yet back
         facing polygons do not.  When combined with polygon offset,
         this can cause situations where holes open up in depth map
         due to extreme polygon offsets due to the extreme front facing
         polygon slopes.  If the back facing polygons did not have such
         extreme slopes, the shadowing artifacts could be avoided.  */
      ::glDisable(GL_CULL_FACE);
    }
 
    /* Only need depth values so avoid writing colors for speed. */
    ::glColorMask(0,0,0,0);
    ::glDisable(GL_LIGHTING);
    ::glEnable(GL_POLYGON_OFFSET_FILL);


	
	pSceneRenderer->CullOrRender(rTransformation, rCullFlag,
          pRenderer, dwTime);
	//m_pPresenter->sgm->shapecount=0;



    if (useShadowMapSupport) {
      int alreadyMatches = 0;
 
      /* The first component is the border depth (at the far clip
         plane).  The other three values are ignored. */
      static const GLfloat hwBorderColor[4] = { 1.0, 0.0, 0.0, 0.0 };

      /* Hardware shadow mapping using SGIX_shadow and SGIX_depth_texture
         extensions. */

      if (!useCopyTexImage) {

        /* Allocate/reallocate the depth buffer memory as necessary. */
        if (hwDepthMapPrecision == GL_UNSIGNED_SHORT) {
          bytesPerDepthValue = 2;
        } else {
          assert(hwDepthMapPrecision == GL_UNSIGNED_INT);
          bytesPerDepthValue = 4;
        }
        requiredSize = width * height * bytesPerDepthValue;
        
		if (requiredSize > depthBufferSize) {
          depthBuffer = (unsigned char*)realloc(depthBuffer, requiredSize);
          if (depthBuffer == NULL) {
            fprintf(stderr, "shadowcast: depth readback buffer realloc failed\n");
            exit(1);
          }
          depthBufferSize = requiredSize;
        }

        /* Read back the depth buffer to memory. */
        
		       
		 ::glReadPixels(0, 0, width, height,
          GL_DEPTH_COMPONENT, hwDepthMapPrecision, depthBuffer);
		 
      }

      if (lowRes) {
        
          target = GL_TEXTURE_2D;
          texobj = TO_LOWRES_HW_DEPTH_MAP;
        
        ::glBindTexture(target, texobj);
        if ((width != lastLowResHwWidth) ||
            (height != lastLowResHwHeight) ||
            (hwDepthMapPrecision != lastLowResHwPrecision) ||
            (hwDepthMapFiltering != lastLowResHwFiltering)) {
          ::glTexParameterfv(target, GL_TEXTURE_BORDER_COLOR, hwBorderColor);
          ::glTexParameteri(target, GL_TEXTURE_MIN_FILTER,
             hwDepthMapFiltering);

          ::glTexParameteri(target, GL_TEXTURE_MAG_FILTER,
            hwDepthMapFiltering);
          useBestShadowMapClamping(target);
          ::glTexParameteri(target,
            GL_TEXTURE_COMPARE_SGIX, GL_TRUE);
          ::glTexParameteri(target,
            GL_TEXTURE_COMPARE_OPERATOR_SGIX, GL_TEXTURE_LEQUAL_R_SGIX);
          if (useCopyTexImage) {
            ::glCopyTexImage2D(target, 0, hwDepthMapInternalFormat,
              0, 0, width, height, 0);
          } else {
            ::glTexImage2D(target, 0, hwDepthMapInternalFormat,
              width, height, 0, GL_DEPTH_COMPONENT,
              hwDepthMapPrecision, depthBuffer);
          }
          lastLowResHwWidth = width;
          lastLowResHwHeight = height;
          lastLowResHwPrecision = hwDepthMapPrecision;
          lastLowResHwFiltering = hwDepthMapFiltering;
        } else {
          /* Texture object dimensions already initialized. */
          alreadyMatches = 1;
          if (useCopyTexImage) {
            ::glCopyTexSubImage2D(target, 0, 0, 0, 
              0, 0, width, height);
          } else {
            ::glTexSubImage2D(target, 0,
              0, 0, width, height, GL_DEPTH_COMPONENT,
              hwDepthMapPrecision, depthBuffer);
          }
        }
      } else {
        /* Hi-res mode */
       
          target = GL_TEXTURE_2D;
          texobj = TO_HW_DEPTH_MAP;
        
        ::glBindTexture(target, texobj);
        if ((width != lastHiResHwWidth) ||
            (height != lastHiResHwHeight) ||
            (hwDepthMapPrecision != lastHiResHwPrecision) ||
            (hwDepthMapFiltering != lastHiResHwFiltering)) {
          ::glTexParameterfv(target, GL_TEXTURE_BORDER_COLOR, hwBorderColor);
            ::glTexParameteri(target, GL_TEXTURE_MIN_FILTER, hwDepthMapFiltering);

          ::glTexParameteri(target, GL_TEXTURE_MAG_FILTER, hwDepthMapFiltering);
          useBestShadowMapClamping(target);
          ::glTexParameteri(target, GL_TEXTURE_COMPARE_SGIX, GL_TRUE);
          ::glTexParameteri(target, GL_TEXTURE_COMPARE_OPERATOR_SGIX, GL_TEXTURE_LEQUAL_R_SGIX);
          if (useCopyTexImage) {
            ::glCopyTexImage2D(target, 0, hwDepthMapInternalFormat,
              0, 0, width, height, 0);
          } else {
            ::glTexImage2D(target, 0, hwDepthMapInternalFormat,
              width, height, 0, GL_DEPTH_COMPONENT, hwDepthMapPrecision, depthBuffer);
          }
          lastHiResHwWidth = width;
          lastHiResHwHeight = height;
          lastHiResHwPrecision = hwDepthMapPrecision;
          lastHiResHwFiltering = hwDepthMapFiltering;
        } else {
          /* Texture object dimensions already initialized. */
          alreadyMatches = 1;
          if (useCopyTexImage) {
            ::glCopyTexSubImage2D(target, 0, 0, 0, 0, 0, width, height);
          } else {
            ::glTexSubImage2D(target, 0,
              0, 0, width, height, GL_DEPTH_COMPONENT, hwDepthMapPrecision, depthBuffer);
          }
        }
      }
   
    } else 
	  {

      /* The border depth components (at the far clip
         plane) for intensity and luminance/alpha textures.
         The second and third components are ignored. */
      static const GLfloat swBorderColor[4] = { 1.0, 0.0, 0.0, 1.0 };

      /* Dual-texture shadow mapping (works on TNT and GeForce/Quadro GPUs). */

      /* Allocate/reallocate the depth buffer memory as necessary. */
      if (depthMapPrecision == GL_UNSIGNED_SHORT) {
        bytesPerDepthValue = 2;
      } else {
        assert(depthMapPrecision == GL_UNSIGNED_BYTE);
        bytesPerDepthValue = 1;
      }
      requiredSize = width * height * bytesPerDepthValue;
      if (requiredSize > depthBufferSize) {
        depthBuffer =(unsigned char*)realloc(depthBuffer, requiredSize);
        if (depthBuffer == NULL) {
          fprintf(stderr, "shadowcast: depth readback buffer realloc failed\n");
          exit(1);
        }
        depthBufferSize = requiredSize;
      }

      /* Read back the depth buffer to memory. */
              
		 ::glReadPixels(0, 0, width, height,
          GL_DEPTH_COMPONENT, depthMapPrecision, depthBuffer);
		
  
  
	  //glReadPixels(0, 0, width, height,
      //  GL_DEPTH_COMPONENT, depthMapPrecision, depthBuffer);
      
	  
	  if (depthMapPrecision == GL_UNSIGNED_SHORT &&
          littleEndian != hwLittleEndian) {
        /* Help debug bi-endian support. */
        swapShorts(width * height, depthBuffer);
      }

      if (lowRes) {
        
          target = GL_TEXTURE_2D;
          texobj = TO_LOWRES_DEPTH_MAP;
        
        ::glBindTexture(target, texobj);
        if ((width != lastLowResWidth) ||
            (height != lastLowResHeight) ||
            (depthMapPrecision != lastLowResPrecision)) {
          ::glTexImage2D(target, 0, depthMapInternalFormat,
            width, height, 0, depthMapFormat, GL_UNSIGNED_BYTE, depthBuffer);
          ::glTexParameterfv(target, GL_TEXTURE_BORDER_COLOR, swBorderColor);
          ::glTexParameteri(target, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
          ::glTexParameteri(target, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
          useBestShadowMapClamping(target);
          lastLowResWidth = width;
          lastLowResHeight = height;
          lastLowResPrecision = depthMapPrecision;
        } else {
          /* Texture object dimensions already initialized. */
          ::glTexSubImage2D(target, 0,
            0, 0, width, height, depthMapFormat, GL_UNSIGNED_BYTE, depthBuffer);
        }
      } else {
        /* Hi-res mode. */
        
          target = GL_TEXTURE_2D;
          texobj = TO_DEPTH_MAP;
        
        ::glBindTexture(target, texobj);
        if ((width != lastHiResWidth) ||
            (height != lastHiResHeight) ||
            (depthMapPrecision != lastHiResPrecision)) {
          ::glTexImage2D(target, 0, depthMapInternalFormat,
            width, height, 0, depthMapFormat, GL_UNSIGNED_BYTE, depthBuffer);
          ::glTexParameterfv(target, GL_TEXTURE_BORDER_COLOR, swBorderColor);
          ::glTexParameteri(target, GL_TEXTURE_MIN_FILTER, GL_NEAREST);
          ::glTexParameteri(target, GL_TEXTURE_MAG_FILTER, GL_NEAREST);
          useBestShadowMapClamping(target);
          lastHiResWidth = width;
          lastHiResHeight = height;
          lastHiResPrecision = depthMapPrecision;
        } else {
          /* Texture object dimensions already initialized. */
          ::glTexSubImage2D(target, 0,
            0, 0, width, height, depthMapFormat, GL_UNSIGNED_BYTE, depthBuffer);
        }
      }
    }

    /* Reset state back to normal. */
    ::glColorMask(1,1,1,1);
	//ShowDepthMap();
	//::glReadPixels(0,0,winWidth,winHeight,GL_RGB,GL_UNSIGNED_BYTE,framebuffer);
	//WriteBMP("new.bmp",winWidth,winHeight,framebuffer);
	::glClear(GL_COLOR_BUFFER_BIT);
	
	::glDisable(GL_POLYGON_OFFSET_FILL);
    ::glEnable(GL_LIGHTING);
    ::glColorMask(1,1,1,1);

    if (!depthMapBackFaceCulling) {
      ::glEnable(GL_CULL_FACE);
    }

    ::glViewport(0, 0, winWidth, winHeight);

    needDepthMapUpdate = 0;

  }

       //   ShowDepthMap();
	  //	  glReadPixels(0,0,winWidth,winHeight,GL_RGB,GL_UNSIGNED_BYTE,framebuffer);
	  //	  WriteBMP("test.bmp",winWidth,winHeight,framebuffer);
	 //	  glClear(GL_COLOR_BUFFER_BIT);
	
}


void ShadowRendering::set_win(int Width, int Height, double AspectRatio, int texsize)
{
 winWidth = Width;
 winHeight = Height;
 winAspectRatio = AspectRatio;
 TEX_SIZE = texsize;
}


void ShadowRendering::setup_renderMode(bool hwenable)
{

     if (hasShadowMapSupport && hwenable) {
       hwDepthMapInternalFormat = GL_DEPTH_COMPONENT16_SGIX;
       hwDepthMapPrecision = GL_UNSIGNED_SHORT;
       updateDepthBias(0);
	   useShadowMapSupport = true;
	   } else {
	   depthMapFormat = GL_LUMINANCE;
       depthMapInternalFormat = GL_INTENSITY8;
       depthMapPrecision = GL_UNSIGNED_BYTE;
       updateDepthBias(0);
	   useShadowMapSupport = false;
	   useStencil=0;
	   }
	   if (hasRegisterCombiners) {
       depthMapFormat = GL_LUMINANCE_ALPHA;
       depthMapInternalFormat = GL_LUMINANCE8_ALPHA8;
       depthMapPrecision = GL_UNSIGNED_SHORT;
       updateDepthBias(0);
	   }

}

void ShadowRendering::set_spot(float cutoff, float exponent)
{
 lightSpotCutoff = cutoff;
 lightSpotExponent = exponent;
}

void ShadowRendering::set_lightRadius(float radius)
{
 lightRadius = radius;
}

void ShadowRendering::set_attenuation(float x,float y,float z)
{
 lightConstantAttenuation = x;
 lightLinearAttenuation = y;
 lightQuadraticAttenuation = z;
}

void ShadowRendering::reset_ambLightColorSum()
{
 amb_light_num_max = 0;
 
 ambLightColor[0] = 0;
 ambLightColor[1] = 0;
 ambLightColor[2] = 0;
 ambLightColor[3] = 0;
}

void ShadowRendering::set_ambLightColor(float r,float g,float b,float i)
{
 ambLightColor[0] += r*i;
 ambLightColor[1] += g*i;
 ambLightColor[2] += b*i;
 ambLightColor[3] += i;
}

void ShadowRendering::set_ambLightColorSum()
{
 amb_light_num_max = amb_light_num_max +1;

 ambLightColorSum[0] =+ ambLightColor[0];// /(amb_light_num_max);
 ambLightColorSum[1] =+ ambLightColor[1];// /(amb_light_num_max);
 ambLightColorSum[2] =+ ambLightColor[2];// /(amb_light_num_max);
 ambLightColorSum[3] =+ ambLightColor[3];
}

void ShadowRendering::set_lightColor(float r,float g,float b,float i)
{
 lightColor[0] = r*i;
 lightColor[1] = g*i;
 lightColor[2] = b*i;
}

void ShadowRendering::set_lightDir(float x,float y,float z,float w)
{
 light_dir[0] = x;
 light_dir[1] = y;
 light_dir[2] = z;
 light_dir[3] = w;

}

void ShadowRendering::set_lightPos(float x,float y,float z,float w)
{
 light_pos[0] = x;
 light_pos[1] = y;
 light_pos[2] = z;
 light_pos[3] = w;

}

void ShadowRendering::set_fogColor(float r,float g,float b,float i, float visRange, bool type)
{
 fogColor[0] = r;
 fogColor[1] = g;
 fogColor[2] = b;
 fogColor[3] = i;

 fogVisRange = visRange;
 fogType = type;
 
 //fogStart= 0;
 //fogEnd= visRange;
 //fogDensity=
}

void ShadowRendering::setup_hwRendering(int mode)
{
	switch(mode){
	case 0:      { glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);
                   glLightModelfv(GL_LIGHT_MODEL_AMBIENT,zero);
                   glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
     
                   glLightfv(GL_LIGHT0, GL_AMBIENT, zero);
                   glLightfv(GL_LIGHT0, GL_DIFFUSE, lightColor);
                   glLightfv(GL_LIGHT0, GL_SPECULAR,lightColor);

                    /* Texture 0 is depth map. */
                   glActiveTextureARB(GL_TEXTURE0_ARB);
 
                   configTexgen(0, 0);
                   configCombinersForHardwareShadowPass(0);
				   break;}
	case 1:      { disableCombiners();
                   glActiveTextureARB(GL_TEXTURE0_ARB);
                   disableTexgen(); 
				   break;}
	
	
	}


}


void ShadowRendering::setup_dualTexRendering(int mode)
{
	switch(mode){
	case 0: { ::glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SINGLE_COLOR);
		      ::glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
	          //glLightModelfv(GL_LIGHT_MODEL_AMBIENT, globalAmbientIntensity);
              ::glLightfv(GL_LIGHT0, GL_AMBIENT, ambLightColor);
              ::glLightfv(GL_LIGHT0, GL_DIFFUSE, zero);
              ::glLightfv(GL_LIGHT0, GL_SPECULAR,zero);
		      break;}
	case 1: { /* Subsequent passes must match depth values of first pass. */
		      ::glDepthFunc(GL_EQUAL);
 
              /* Texture 0 is depth map. */
              ::glActiveTextureARB(GL_TEXTURE0_ARB);
              configTexgen(0, 0);
 
              /* Texture 1 is light Z range. */
              ::glActiveTextureARB(GL_TEXTURE1_ARB);
              configTexgen(1, 0);
   
              ::glLightfv(GL_LIGHT0, GL_AMBIENT,zero);
              ::glLightfv(GL_LIGHT0, GL_DIFFUSE,lightColor);
              ::glLightfv(GL_LIGHT0, GL_SPECULAR,lightColor);

              configCombiners(1);
              if (hasRegisterCombiners) {
				  ::glAlphaFunc(GL_GREATER, 0.0);
				 } else {
				  ::glAlphaFunc(GL_GREATER, 0.5);
				 }
                 ::glEnable(GL_ALPHA_TEST);
               if (depthMapPrecision == GL_UNSIGNED_SHORT) {

               if (useStencil) {
				   ::glStencilOp(GL_KEEP, GL_KEEP, GL_REPLACE);
				   ::glStencilFunc(GL_ALWAYS, 0x1, 0x1);
				   ::glEnable(GL_STENCIL_TEST);
				 }
				 }
		      break;}
	case 2: { ::glDisable(GL_ALPHA_TEST);
		      ::glDepthFunc(GL_LEQUAL);

              disableCombiners();

              ::glActiveTextureARB(GL_TEXTURE0_ARB);
              disableTexgen();
              ::glActiveTextureARB(GL_TEXTURE1_ARB);
              disableTexgen(); 
		      break;}
		
	
	}
   



}

int ShadowRendering::WriteBMP(char *file, int width, int height, unsigned char *image)
{
   // Remember that bitmaps are made up of 3 parts, the info header,
   // the file header, and the image.  We will have to write each one.

   FILE *pFile;                  // Our file pointer.
   BITMAPINFOHEADER infoHeader;  // info header.
   BITMAPFILEHEADER fileHeader;  // file header.
   int index;                    // index for the for loop.
   unsigned char tempColors;     // Use to change image from RGB to BGR.

   pFile = fopen(file, "wb");    // Open the file for write.

   if(!pFile) { return 0; }

   // Define whats going in the infoHeader.
   infoHeader.biSize = sizeof(BITMAPINFOHEADER);
   infoHeader.biPlanes        = 1;
   infoHeader.biBitCount      = 24;
   infoHeader.biCompression   = BI_RGB;
   infoHeader.biSizeImage     = width * abs(height) * 3;
   infoHeader.biXPelsPerMeter = 0;
   infoHeader.biYPelsPerMeter = 0;
   infoHeader.biClrUsed       = 0;
   infoHeader.biClrImportant  = 0;
   infoHeader.biWidth         = width;
   infoHeader.biHeight        = height;

   // Define whats going in the file header.
   fileHeader.bfSize       = sizeof(BITMAPFILEHEADER);
   fileHeader.bfType       = 0x4D42;
   fileHeader.bfReserved1  = 0;
   fileHeader.bfReserved2  = 0;
   fileHeader.bfOffBits    = sizeof(BITMAPFILEHEADER) + sizeof(BITMAPINFOHEADER);


   // Change image from RGB to BGR.  Thats how .bmp is saved.
   for(index = 0; index < infoHeader.biSizeImage; index += 3)
      {
         tempColors = image[index];
         image[index] = image[index + 2];
         image[index + 2] = tempColors;
      }

   // Now lets write to the file.
   // Write file header.
   fwrite(&fileHeader, 1, sizeof(BITMAPFILEHEADER), pFile);

   // Write info header.
   fwrite(&infoHeader, 1, sizeof(BITMAPINFOHEADER), pFile);

   // Write image.
   fwrite(image, 1, infoHeader.biSizeImage, pFile);

   // Last but not least close the file.
   fclose(pFile);
   return 0;
}


void ShadowRendering::ShowDepthMap(void)
{
	GLenum target;
  GLuint texobj;

  //updateDepthMap(pSceneRenderer,mtransform, cullFlag,
   //       this, m_pPresenter->GetTime()););

  ::glViewport(0, 0, winWidth, winHeight);
  ::glMatrixMode(GL_PROJECTION);
  ::glLoadIdentity();
  ::glScalef(winAspectRatio, 1, 1);
  ::glMatrixMode(GL_MODELVIEW);
  ::glLoadIdentity();

  ::glColor3f(1,1,1);
  ::glDisable(GL_LIGHTING);
  ::glDisable(GL_DEPTH_TEST);

  if (useShadowMapSupport) {
    if (movingLight && quickLightMove) {
      
      
        target = GL_TEXTURE_2D;
        texobj = TO_LOWRES_HW_DEPTH_MAP;
      
    } else {
      
        target = GL_TEXTURE_2D;
        texobj = TO_HW_DEPTH_MAP;
      
    }

    ::glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_REPLACE);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_TEXTURE);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_COLOR);
    // Turn off shadow mapping comparisons so we can "show" the depth
    //   values directly.  With GL_TEXTURE_COMPARE_SGIX set to GL_FALSE,
    //   a GL_DEPTH_COMPONENT texture behaves as a GL_LUMINANCE texture.
    //   However, while the depth components may be 16-bit or 24-bit values,
    //   be warned that viewing the depth components as luminance values
    //   will only show you the most significant bits. 
    ::glTexParameteri(target, GL_TEXTURE_COMPARE_SGIX, GL_FALSE);
  } else {
    if (movingLight && quickLightMove) {
      
        target = GL_TEXTURE_2D;
        texobj = TO_LOWRES_DEPTH_MAP;
      
    } else {
      
      
        target = GL_TEXTURE_2D;
        texobj = TO_DEPTH_MAP;
       }

    ::glTexEnvi(GL_TEXTURE_ENV, GL_TEXTURE_ENV_MODE, GL_COMBINE_EXT);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_COMBINE_RGB_EXT, GL_REPLACE);
    ::glTexEnvi(GL_TEXTURE_ENV, GL_SOURCE0_RGB_EXT, GL_TEXTURE);
    if (showDepthMapMSBs) {
      if (littleEndian) {
        ::glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_ALPHA);
      } else {
        ::glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_COLOR);
      }
    } else {
      if (littleEndian) {
        ::glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_COLOR);
      } else {
        ::glTexEnvi(GL_TEXTURE_ENV, GL_OPERAND0_RGB_EXT, GL_SRC_ALPHA);
      }
    }
  }

  ::glEnable(target);
  ::glBindTexture(target, texobj);
  ::glTexEnvf(GL_TEXTURE_FILTER_CONTROL_EXT, GL_TEXTURE_LOD_BIAS_EXT, textureLodBias);

  
    // Standard 2D textures use normalized texture coordinates in the
    //   [0..1]x[0..1] rnage. 
    ::glBegin(GL_QUADS);
      ::glTexCoord2f(0,0);
      ::glVertex2f(-1,-1);
      ::glTexCoord2f(1,0);
      ::glVertex2f(1,-1);
      ::glTexCoord2f(1,1);
      ::glVertex2f(1,1);
      ::glTexCoord2f(0,1);
      ::glVertex2f(-1,1);
    ::glEnd();
  

  if (useShadowMapSupport) {
    // Re-enable shadow map comparsions. 
    ::glTexParameteri(target, GL_TEXTURE_COMPARE_SGIX, GL_TRUE);
  }

  ::glTexEnvf(GL_TEXTURE_FILTER_CONTROL_EXT, GL_TEXTURE_LOD_BIAS_EXT, 0.0);
  ::glDisable(target);
  ::glEnable(GL_DEPTH_TEST);
  ::glEnable(GL_LIGHTING);
}





void ShadowRendering::DebugDepthMap(char *file,int width,int height,GLfloat *image)
{
  unsigned short accum;
 unsigned long  count_def;
 unsigned long  count_max;
 unsigned int depthm;
 int i;
 i=0;
 depthm=0;
 
 count_max = winWidth*winHeight*3-1;

 for(count_def=0;count_def<count_max;count_def++) {  
 
 framebuffer[count_def] = (unsigned char)(255*image[depthm]);
 i++;
 if(i==3)
 {
   i=0;
   depthm++;
 }
 
 
 }

 WriteBMP(file,width,height,framebuffer);

}

void ShadowRendering::drawHardwareShadowPass(MediaObjectProxy * pSceneRenderer, const Matrix& rTransformation, 
                      const CullFlag& rCullFlag,
		              OpenGLVisualRenderer* pRenderer, 
                      DWORD dwTime)

{
  ::glLightModeli(GL_LIGHT_MODEL_COLOR_CONTROL, GL_SEPARATE_SPECULAR_COLOR);
  //2::glLightModelfv(GL_LIGHT_MODEL_AMBIENT, zero);
  ::glLightModeli(GL_LIGHT_MODEL_LOCAL_VIEWER, GL_TRUE);
   
  
  ::glLightfv(GL_LIGHT0, GL_AMBIENT, zero);
  ::glLightfv(GL_LIGHT0, GL_DIFFUSE, lightColor);
  ::glLightfv(GL_LIGHT0, GL_SPECULAR, lightColor);

  /* Texture 0 is depth map. */
  ::glActiveTextureARB(GL_TEXTURE0_ARB);
 
  configTexgen(0, 0);
  
  
  configCombinersForHardwareShadowPass(0);
  
  pSceneRenderer->CullOrRender(rTransformation, rCullFlag,
          pRenderer, dwTime);
 
  
  
  
  
  disableCombiners();
  
  ::glActiveTextureARB(GL_TEXTURE0_ARB);
  disableTexgen();

  
}